@@ -138,8 +138,9 @@ void CSPELL_csr_header
 			p_operation->parameters = p_operation->parameters->next;
 		}
 	} else {
-		fprintf (fid, "\n");
-		emitted_name = p_operation->name;
+            sprintf(op_internal_name, "%s%s", cstub_pref, BE_get_name(p_operation->name));
+            emitted_name = NAMETABLE_add_id(op_internal_name);
+            fprintf (fid, "\n");
 	}
 
 	CSPELL_function_def_header (fid, p_operation, emitted_name);
@@ -213,7 +214,7 @@ static void CSPELL_client_stub_routine
     fprintf(fid, "rpc_transfer_syntax_t\t\tIDL_transfer_syntax;\n");
     fprintf(fid, "rpc_iovector_elt_t\t\tIDL_outs;\n");
     fprintf(fid, "volatile ndr_ulong_int\t\tIDL_fault_code=error_status_ok;\n");
-    fprintf(fid, "volatile ndr_ulong_int\t\tIDL_user_fault_id;\n");
+    fprintf(fid, "volatile ndr_ulong_int\t\tIDL_user_fault_id=0;\n");
     fprintf(fid,
            "volatile RPC_SS_THREADS_CANCEL_STATE_T IDL_async_cancel_state;\n");
     fprintf(fid, "IDL_ms_t\t\tIDL_ms;\n");
@@ -484,7 +485,7 @@ void DDBE_spell_pickling_stub
      * Standard local variables
      */
     fprintf(fid, "volatile ndr_ulong_int IDL_fault_code=error_status_ok;\n");
-    fprintf(fid, "volatile ndr_ulong_int IDL_user_fault_id;\n");
+    fprintf(fid, "volatile ndr_ulong_int IDL_user_fault_id=0;\n");
     fprintf(fid,
            "volatile RPC_SS_THREADS_CANCEL_STATE_T IDL_async_cancel_state;\n");
     fprintf(fid, "IDL_es_state_t *IDL_es_state_p;\n");
