@@ -93,7 +93,7 @@ typedef struct
     unsigned short              ifspec_vers;
     unsigned short              opcnt;
     unsigned long               vers;
-    uuid_t                      id;
+    dce_uuid_t                  id;
     unsigned short              stub_rtl_if_vers;
     rpc_endpoint_vector_t       endpoint_vector;
     rpc_syntax_vector_t         syntax_vector;
@@ -937,7 +937,7 @@ void rpc_ss_enable_reflect_deletes  _DCE_PROTOTYPE_ ((
  rpc_ss_node_table_t 
 ));
 
-long rpc_ss_register_node   _DCE_PROTOTYPE_ ((
+idl_ulong_int rpc_ss_register_node   _DCE_PROTOTYPE_ ((
     rpc_ss_node_table_t  /*tab*/,
     byte_p_t  /*ptr*/,
     long  /*marshalling*/,
@@ -947,19 +947,19 @@ long rpc_ss_register_node   _DCE_PROTOTYPE_ ((
 
 byte_p_t rpc_ss_lookup_node_by_num   _DCE_PROTOTYPE_ ((
     rpc_ss_node_table_t  /*tab*/,
-    unsigned long /*num*/
+    idl_ulong_int /*num*/
 ));
 
 byte_p_t rpc_ss_lookup_pointer_to_node   _DCE_PROTOTYPE_ ((
     rpc_ss_node_table_t  /*tab*/,
-    unsigned long  /*num*/,
+    idl_ulong_int  /*num*/,
     long * /*has_been_unmarshalled*/
 ));
 
 byte_p_t rpc_ss_return_pointer_to_node   _DCE_PROTOTYPE_ ((
     rpc_ss_node_table_t  /*tab*/,
-    unsigned long  /*num*/,
-    long  /*size*/,
+    idl_ulong_int  /*num*/,
+    idl_ulong_int  /*size*/,
     rpc_void_p_t (*p_allocate) _DCE_PROTOTYPE_(( idl_size_t)),
     long * /*has_been_unmarshalled*/,
     long * /*new_node*/
@@ -1246,12 +1246,12 @@ void rpc_ss_init_callee_ctx_tables   _DCE_PROTOTYPE_ ((void));
 #define uuid_tOue rpc_ss_u_uuid
 
 void rpc_ss_m_uuid  _DCE_PROTOTYPE_ ((
-   uuid_t *, 
+   dce_uuid_t *,
    rpc_ss_marsh_state_t *
 ));
 
 void rpc_ss_u_uuid   _DCE_PROTOTYPE_ ((
-   uuid_t *, 
+   dce_uuid_t *,
    rpc_ss_marsh_state_t *
 ));
 
