@@ -474,7 +474,7 @@ unsigned32              *st;
      * The start of stub data will depend upon whether the object
      * UUID field is present.
      */
-    if (memcmp (&binding_r->obj, &uuid_g_nil_uuid, sizeof (uuid_t)) != 0)
+    if (memcmp (&binding_r->obj, &uuid_g_nil_uuid, sizeof (dce_uuid_t)) != 0)
     {
         /*
          * A non-nil object UUID is present.
@@ -996,6 +996,10 @@ unsigned32              *st;
                            in_call_args,
                            call_rep,
                            temp_status);
+    if (temp_status != rpc_s_ok)
+    {
+	call_rep->cn_call_status = temp_status;
+    }
 
     if (call_rep->cn_call_status == rpc_s_ok)
     {
