@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC) 
+**      Remote Procedure Call (RPC)
 **
 **  ABSTRACT:
 **
@@ -57,7 +57,7 @@
 **  SCOPE:              PRIVATE - declared in cncthd.h
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      arg             The call rep for this call.
@@ -77,10 +77,10 @@
 **--
 **/
 
-PRIVATE void rpc__cn_call_executor 
+PRIVATE void rpc__cn_call_executor
 #ifdef _DCE_PROTO_
-(pointer_t arg, boolean32 call_was_queued ATTRIBUTE_UNUSED) 
-#else 
+(pointer_t arg, boolean32 call_was_queued ATTRIBUTE_UNUSED)
+#else
 (arg, call_was_queued)
 pointer_t               arg;
 boolean32               call_was_queued;
@@ -89,7 +89,7 @@ boolean32               call_was_queued;
     rpc_binding_rep_t           *binding_r;
     rpc_cn_call_rep_t           *call_r;
     rpc_iovector_t              iovector;
-    uuid_t                      type_uuid;
+    dce_uuid_t                      type_uuid;
     rpc_mgr_epv_t               manager_epv;
     rpc_v2_server_stub_epv_t    server_stub_epv;
     rpc_if_rep_p_t              if_spec_rep;
@@ -130,21 +130,21 @@ boolean32               call_was_queued;
         RPC_CN_UNLOCK ();
         goto CLEANUP;
     }
-    
+
     /*
      * Get the if rep and the server stub and manager EPV.
      */
     rpc__if_lookup2 (call_r->u.server.if_id,
                      call_r->u.server.if_vers,
-                     &type_uuid, 
-                     &call_r->u.server.ihint, 
+                     &type_uuid,
+                     &call_r->u.server.ihint,
                      &if_spec_rep,
-                     &server_stub_epv, 
-                     &manager_epv, 
-                     &flags, 
-                     &max_calls, 
-                     &max_rpc_size, 
-                     &if_callback, 
+                     &server_stub_epv,
+                     &manager_epv,
+                     &flags,
+                     &max_calls,
+                     &max_rpc_size,
+                     &if_callback,
                      &status);
     if (status != rpc_s_ok)
     {
@@ -156,7 +156,7 @@ boolean32               call_was_queued;
         RPC_CN_UNLOCK ();
         goto CLEANUP;
     }
-    
+
     /*
      * If the operation number is out of range, indicate a fault to
      * the protocol service, otherwise process the incoming packet(s).
@@ -176,7 +176,7 @@ boolean32               call_was_queued;
      * Receive the first packet.
      */
     rpc__cn_call_receive ((rpc_call_rep_t *) call_r,
-                          &iovector.elt[0], 
+                          &iovector.elt[0],
                           &status);
     if (status != rpc_s_ok)
     {
@@ -188,12 +188,12 @@ boolean32               call_was_queued;
         RPC_CN_UNLOCK ();
         goto CLEANUP;
     }
-    
+
     /*
      * Mark the call as having executed.
      */
     call_r->call_executed = true;
-    
+
     /*
      * Enable posting of cancels to this call executor thread.
      * This will also post any queued cancels.
@@ -203,7 +203,7 @@ boolean32               call_was_queued;
     RPC_CALL_LOCK ((rpc_call_rep_t *) call_r);
     rpc__cthread_cancel_enable_post ((rpc_call_rep_p_t) call_r);
     RPC_CALL_UNLOCK ((rpc_call_rep_t *) call_r);
-   
+
     /*
      * Dispatch appropriately depending on the stub version.
      */
@@ -231,19 +231,19 @@ boolean32               call_was_queued;
          * This is the v2 (new) stub runtime interface.
          */
         case 2:
-        
+
         RPC_LOG_SERVER_STUB_PRE;
-        ((*server_stub_epv[call_r->opnum])) 
+        ((*server_stub_epv[call_r->opnum]))
         ((handle_t) call_r->binding_rep,
          (rpc_call_handle_t) call_r,
          &iovector.elt[0],
          &(RPC_CN_ASSOC_NDR_FORMAT (call_r->assoc)),
          &call_r->transfer_syntax,
-         manager_epv, 
+         manager_epv,
          &status);
         RPC_LOG_SERVER_STUB_POST;
         break;
-        
+
         /*
          * Unknown version
          */
