@@ -48,6 +48,7 @@
 #include <cnfbuf.h>     /* NCA Connection fragment buffer service */
 #include <cncall.h>     /* NCA Connection call service */
 #include <cnnet.h>
+#include <lw/ntstatus.h>
 
 /***********************************************************************/
 /*
@@ -2083,6 +2084,20 @@ unsigned32              *st;
         *st = rpc_s_auth_skew;
         break;
 
+        // Using this to pass through an RODC error that
+        // requires reauthentication.
+        case LW_STATUS_KDC_CERT_REVOKED:
+        *st = rpc_s_auth_tkt_expired;
+        break;
+
+        case -1:
+        *st = rpc_s_unknown_status_code;
+        break;
+
+	case ENOENT:
+        *st = rpc_s_no_name_mapping;
+	break;
+
         default:
         *st = rpc_s_cannot_connect;
         break;
