@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -53,7 +53,6 @@
 #include <cncall.h>     /* NCA connection call service */
 #include <comcthd.h>    /* Externals for call thread services component */
 #include <cncthd.h>     /* NCA Connection call executor service */
-#include <cnnp.h>       /* Named Pipes */
 
 
 /******************************************************************************/
@@ -131,7 +130,7 @@ INTERNAL void receive_packet _DCE_PROTOTYPE_ ((
     ));
 
 /*
- * R P C _ C N _ S E N D _ F A U L T 
+ * R P C _ C N _ S E N D _ F A U L T
  *
  * This macro will cause a fault PDU to be sent back to the client
  * and will terminate the RPC.
@@ -156,11 +155,11 @@ INTERNAL void receive_packet _DCE_PROTOTYPE_ ((
 **
 **  SCOPE:              PRIVATE - declared in cnrcvr.h
 **
-**  DESCRIPTION:        
+**  DESCRIPTION:
 **
 **  This routine constitutes the top-level receiver thread (both client and
 **  server) and is invoked by "thread create" in "association
-**  lookaside alloc" routine to process incoming packets. 
+**  lookaside alloc" routine to process incoming packets.
 **
 **  It receives packets on an association until terminated by a
 **  cancel, the connection breaks or a resource exhaustion problem
@@ -185,7 +184,7 @@ INTERNAL void receive_packet _DCE_PROTOTYPE_ ((
 **--
 */
 
-PRIVATE void rpc__cn_network_receiver 
+PRIVATE void rpc__cn_network_receiver
 #ifdef _DCE_PROTO_
 (
   rpc_cn_assoc_p_t        assoc
@@ -199,7 +198,7 @@ rpc_cn_assoc_p_t        assoc;
     volatile boolean    done = false;
 
     //DO_NOT_CLOBBER(done);
- 
+
     RPC_CN_DBG_RTN_PRINTF (rpc__cn_network_receiver);
 
     RPC_DBG_PRINTF (rpc_e_dbg_threads, RPC_C_CN_DBG_THREADS,
@@ -230,7 +229,7 @@ rpc_cn_assoc_p_t        assoc;
         RPC_CN_LOCK ();
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                         ("CN: Global mutex locked\n"));
-        
+
         /*
          * Wait for a session/transport connection to be established.
          */
@@ -256,7 +255,7 @@ rpc_cn_assoc_p_t        assoc;
                 assoc->cn_ctlblk.cn_rcvr_waiters++;
                 RPC_DBG_PRINTF (rpc_e_dbg_threads, RPC_C_CN_DBG_THREADS,
                     ("####### assoc->%x Waiting for new connection \n", assoc));
-                DCETHREAD_TRY 
+                DCETHREAD_TRY
                 {
                     RPC_COND_WAIT (assoc->cn_ctlblk.cn_rcvr_cond,
                                    rpc_g_global_mutex);
@@ -302,11 +301,11 @@ rpc_cn_assoc_p_t        assoc;
                 RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
                                 ("CN: assoc->%x call_rep->none Receiver awake ... Connection established\n",
                                  assoc));
-    
+
                 /*
                  * Increment the association control block's reference count since we
                  * are now using it and receive packets as long as the
-                 * connection is open. 
+                 * connection is open.
                  */
                 RPC_CN_ASSOC_ACB_INC_REF (assoc);
 
@@ -352,19 +351,19 @@ rpc_cn_assoc_p_t        assoc;
                  * to closed.
                  */
                 RPC_CN_STATS_INCR (closed_connections);
-                serr = rpc__socket_close (assoc->cn_ctlblk.cn_sock); /* must not be a cancellation point */
+                serr = RPC_SOCKET_CLOSE (assoc->cn_ctlblk.cn_sock); /* must not be a cancellation point */
                 if (RPC_SOCKET_IS_ERR(serr))
                 {
                     /*
                      * The socket close failed.
                      */
                     RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_ERRORS,
-("(rpc__cn_network_receiver) assoc->%x desc->%x rpc__socket_close failed, error = %d\n", 
+("(rpc__cn_network_receiver) assoc->%x desc->%x RPC_SOCKET_CLOSE failed, error = %d\n",
                                      assoc,
-                                     assoc->cn_ctlblk.cn_sock,                                  
+                                     assoc->cn_ctlblk.cn_sock,
                                      RPC_SOCKET_ETOI(serr)));
                 }
-                
+
                 assoc->cn_ctlblk.cn_state = RPC_C_CN_CLOSED;
 
                 /*
@@ -405,32 +404,32 @@ rpc_cn_assoc_p_t        assoc;
                 }
                 DCETHREAD_CATCH_ALL(THIS_CATCH)
                 {
-         	    /*
-         	     * rpc_m_unexpected_exc
-            	     * "(%s) Unexpected exception was raised"
-            	     */
-        	    RPC_DCE_SVC_PRINTF ((
-        	        DCE_SVC(RPC__SVC_HANDLE, "%s"),
-        	        rpc_svc_recv,
-        	        svc_c_sev_fatal | svc_c_action_abort,
-        	        rpc_m_unexpected_exc,
-        	        "rpc__cn_network_receiver" ));
+		    /*
+		     * rpc_m_unexpected_exc
+		     * "(%s) Unexpected exception was raised"
+		     */
+		    RPC_DCE_SVC_PRINTF ((
+		        DCE_SVC(RPC__SVC_HANDLE, "%s"),
+		        rpc_svc_recv,
+		        svc_c_sev_fatal | svc_c_action_abort,
+		        rpc_m_unexpected_exc,
+		        "rpc__cn_network_receiver" ));
                 }
                 DCETHREAD_ENDTRY
             }
         }
         DCETHREAD_CATCH(dcethread_interrupt_e)
         {
- 	    /*
- 	     * rpc_m_unexpected_exc
-    	     * "(%s) Unexpected exception was raised"
-    	     */
+	    /*
+	     * rpc_m_unexpected_exc
+	     * "(%s) Unexpected exception was raised"
+	     */
 	    RPC_DCE_SVC_PRINTF ((
-        	DCE_SVC(RPC__SVC_HANDLE, "%s"),
-        	rpc_svc_recv,
-        	svc_c_sev_fatal | svc_c_action_abort,
-        	rpc_m_unexpected_exc,
-        	"rpc__cn_network_receiver" ));
+		DCE_SVC(RPC__SVC_HANDLE, "%s"),
+		rpc_svc_recv,
+		svc_c_sev_fatal | svc_c_action_abort,
+		rpc_m_unexpected_exc,
+		"rpc__cn_network_receiver" ));
         }
         DCETHREAD_CATCH_ALL(THIS_CATCH)
         {
@@ -446,16 +445,16 @@ rpc_cn_assoc_p_t        assoc;
         }
         DCETHREAD_CATCH_ALL(THIS_CATCH)
         {
- 	    /*
- 	     * rpc_m_unexpected_exc
-    	     * "(%s) Unexpected exception was raised"
-    	     */
+	    /*
+	     * rpc_m_unexpected_exc
+	     * "(%s) Unexpected exception was raised"
+	     */
 	    RPC_DCE_SVC_PRINTF ((
-        	DCE_SVC(RPC__SVC_HANDLE, "%s"),
-        	rpc_svc_recv,
-        	svc_c_sev_fatal | svc_c_action_abort,
-        	rpc_m_unexpected_exc,
-        	"rpc__cn_network_receiver" ));
+		DCE_SVC(RPC__SVC_HANDLE, "%s"),
+		rpc_svc_recv,
+		svc_c_sev_fatal | svc_c_action_abort,
+		rpc_m_unexpected_exc,
+		"rpc__cn_network_receiver" ));
         }
         DCETHREAD_ENDTRY
     } /* end while (!done && !assoc->cn_ctlblk.exit_rcvr) */
@@ -500,7 +499,7 @@ rpc_cn_assoc_p_t        assoc;
 **--
 **/
 
-INTERNAL void receive_dispatch 
+INTERNAL void receive_dispatch
 #ifdef _DCE_PROTO_
 (
   rpc_cn_assoc_p_t        assoc
@@ -525,7 +524,7 @@ rpc_cn_assoc_p_t        assoc;
 
     //DO_NOT_CLOBBER(unpack_ints);
     //DO_NOT_CLOBBER(i);
-	 
+
     /*
      * Onetime (auto) initialization.
      */
@@ -543,8 +542,6 @@ rpc_cn_assoc_p_t        assoc;
      */
     for (i = 0;; i++)
     {
-	rpc_protseq_id_t protseq_id;
-
         RPC_LOG_CN_PROCESS_PKT_NTR;
 
         /*
@@ -558,37 +555,13 @@ rpc_cn_assoc_p_t        assoc;
          */
         DCETHREAD_TRY
         {
-            if (i == 0 && (assoc->assoc_flags & RPC_C_CN_ASSOC_SERVER))
-            {
-                /*
-                 * For Named Pipes, we need to read a short preamble which
-                 * contains the security context. Before doing this, we
-                 * need first check whether the client is a named pipe or
-                 * not. 
-                 */
-                rpc__naf_desc_inq_protseq_id(assoc->cn_ctlblk.cn_sock,
-                    RPC_C_PROTOCOL_ID_NCACN,
-                    &protseq_id,
-                    &st);
-                RPC_DBG_PRINTF(rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                    ("CN_NP: call_rep->%x assoc->%x desc->%d protseq_id->%x%s\n",
-                        assoc->call_rep, assoc, assoc->cn_ctlblk.cn_sock, protseq_id,
-                        (protseq_id == RPC_C_PROTSEQ_ID_NCACN_NP) ? " (received on named pipe)" : ""));
-                if (st == rpc_s_ok && protseq_id == RPC_C_PROTSEQ_ID_NCACN_NP)
-                {
-                    rpc__np_get_auth_info(assoc, &st);
-                }
-            }
-            if (st == rpc_s_ok)
-            {
-                receive_packet (assoc, &fragbuf_p, &ovf_fragbuf_p, &st);
-            }
+            receive_packet (assoc, &fragbuf_p, &ovf_fragbuf_p, &st);
         }
         DCETHREAD_CATCH(dcethread_interrupt_e)
         {
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
 ("CN: call_rep->%x assoc->%x desc->%x receiver canceled, caught in receive_dispatch()\n",
-                            assoc->call_rep, 
+                            assoc->call_rep,
                             assoc,
                             assoc->cn_ctlblk.cn_sock));
            st = rpc_s_connection_closed;
@@ -641,7 +614,7 @@ rpc_cn_assoc_p_t        assoc;
 	 * Make sure that we have a valid packet type.
 	 * If not, we return an error, and the caller will close
 	 * the connection.
-	 */        
+	 */
         if (/* (ptype < 0) ||*/ (ptype > RPC_C_CN_PKT_MAX_TYPE) ||
             (packet_info_table[ptype].class == DGRAM_CLASS_PKT))
         {
@@ -661,7 +634,7 @@ rpc_cn_assoc_p_t        assoc;
              */
             NDR_UNPACK_DREP (&(RPC_CN_ASSOC_NDR_FORMAT (assoc)),
                              RPC_CN_PKT_DREP (pktp));
-            if ((NDR_DREP_INT_REP (RPC_CN_PKT_DREP (pktp)) != 
+            if ((NDR_DREP_INT_REP (RPC_CN_PKT_DREP (pktp)) !=
                  NDR_LOCAL_INT_REP))
             {
                 unpack_ints = true;
@@ -677,7 +650,7 @@ rpc_cn_assoc_p_t        assoc;
              * Sanity check the major and minor version numbers.
              * We let the association state machine do this check
              * in the case BIND packets because the
-             * protocol calls for a call reject if the versions 
+             * protocol calls for a call reject if the versions
              * do not match at that point.
              * For subsequent packets, we do the check here.
              */
@@ -709,7 +682,7 @@ rpc_cn_assoc_p_t        assoc;
             /*
              * If the pdu is a bind or alter-context pdu and we need to
              * unpack the packet, save the raw form of the pdu so that
-             * the checksum can be computed correctly later.  Do this by 
+             * the checksum can be computed correctly later.  Do this by
              * allocating a fragbuf and chaining it to the association
              * control block, then copying the packet to it.  The fragbuf
              * will be deallocated in the state machine after invoking
@@ -738,9 +711,9 @@ rpc_cn_assoc_p_t        assoc;
             {
                 SWAB_INPLACE_16 (auth_len);
             }
-            
+
 	    auth_tlr = (rpc_cn_auth_tlr_t *) ((unsigned8 *)(pktp) +
-                fragbuf_p->data_size - 
+                fragbuf_p->data_size -
                 (auth_len + RPC_CN_PKT_SIZEOF_COM_AUTH_TLR));
 
             /*
@@ -754,9 +727,9 @@ rpc_cn_assoc_p_t        assoc;
                 SWAB_INPLACE_32 (key_id);
             }
 
-            if ((ptype != RPC_C_CN_PKT_BIND) && 
+            if ((ptype != RPC_C_CN_PKT_BIND) &&
                 (ptype != RPC_C_CN_PKT_ALTER_CONTEXT) &&
-                (ptype != RPC_C_CN_PKT_BIND_ACK) && 
+                (ptype != RPC_C_CN_PKT_BIND_ACK) &&
                 (ptype != RPC_C_CN_PKT_ALTER_CONTEXT_RESP))
             {
                 rpc_authn_protocol_id_t authn_protocol = rpc_c_authn_none;
@@ -765,7 +738,7 @@ rpc_cn_assoc_p_t        assoc;
                                               key_id,
                                               &sec_context,
                                               &auth_st);
-                
+
                 if (auth_st == rpc_s_ok)
                 {
                     authn_protocol = RPC_CN_AUTH_CVT_ID_WIRE_TO_API (auth_tlr->auth_type, &auth_st);
@@ -812,9 +785,9 @@ rpc_cn_assoc_p_t        assoc;
                         }
                         auth_len = RPC_CN_PKT_AUTH_LEN (pktp);
                         auth_tlr = (rpc_cn_auth_tlr_t *) ((unsigned8 *)(pktp) +
-                                                          fragbuf_p->data_size - 
+                                                          fragbuf_p->data_size -
                                                           (auth_len
-                                                           + 
+                                                           +
                                                            RPC_CN_PKT_SIZEOF_COM_AUTH_TLR));
                         frag_len = RPC_CN_PKT_FRAG_LEN (pktp);
                         frag_len -= auth_tlr->stub_pad_length;
@@ -832,7 +805,7 @@ rpc_cn_assoc_p_t        assoc;
                          * On the client side this error should just be
                          * handed back to the client thread waiting, if
                          * any.
-                         * 
+                         *
                          * On the server side the error should be reflected
                          * back to the client on either a FAULT
                          * PDU if the recv_check failed on a call class
@@ -845,6 +818,7 @@ rpc_cn_assoc_p_t        assoc;
                         {
                             (*fragbuf_p->fragbuf_dealloc)(fragbuf_p);
                             sec_context->sec_status = auth_st;
+                            assoc->assoc_status = auth_st;
                             RPC_CN_ASSOC_WAKEUP (assoc);
                             continue;
                         }
@@ -878,17 +852,26 @@ rpc_cn_assoc_p_t        assoc;
         else
         {
             if ((assoc->assoc_flags & RPC_C_CN_ASSOC_CLIENT) &&
-               (ptype == RPC_C_CN_PKT_RESPONSE) &&
-               (RPC_CN_PKT_AUTH_REQUIRED(assoc->call_rep->binding_rep->auth_info)))
+                (ptype == RPC_C_CN_PKT_RESPONSE))
             {
-                RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                        ("CN: auth_info %x\n", assoc->call_rep->binding_rep->auth_info));
-                RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                        ("CN: should not continue further with this PDU\n"));
-                (*fragbuf_p->fragbuf_dealloc)(fragbuf_p);
-                st = rpc_s_authn_level_mismatch;
-                RPC_CN_ASSOC_WAKEUP (assoc);
-                break;
+                if (assoc->call_rep == NULL)
+                {
+                    /* The call was abandoned by the sending thread at the same time that
+                       a bind ack came back from the server, so bail out */
+                    st = rpc_s_connection_closed;
+                    break;
+                }
+
+                if ((RPC_CN_PKT_AUTH_REQUIRED(assoc->call_rep->binding_rep->auth_info)))
+                {
+                    RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
+                                    ("CN: auth_info %x\n", assoc->call_rep->binding_rep->auth_info));
+                    RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
+                                    ("CN: should not continue further with this PDU\n"));
+                    st = rpc_s_authn_level_mismatch;
+                    RPC_CN_ASSOC_WAKEUP (assoc);
+                    break;
+                }
             }
         }
 
@@ -906,8 +889,8 @@ rpc_cn_assoc_p_t        assoc;
          */
         if (packet_info_table[ptype].class == CALL_CLASS_PKT)
         {
-            if ((ptype == RPC_C_CN_PKT_REQUEST) 
-                && 
+            if ((ptype == RPC_C_CN_PKT_REQUEST)
+                &&
                 (RPC_CN_PKT_FLAGS (pktp) & RPC_C_CN_FLAGS_FIRST_FRAG))
             {
                 /*
@@ -915,21 +898,21 @@ rpc_cn_assoc_p_t        assoc;
                  * Allocate a call rep and mark it as being a server
                  * call rep.
                  */
-                call_r = (rpc_cn_call_rep_t *) 
-                    rpc__list_element_alloc (&rpc_g_cn_call_lookaside_list, 
-                                             true); 
+                call_r = (rpc_cn_call_rep_t *)
+                    rpc__list_element_alloc (&rpc_g_cn_call_lookaside_list,
+                                             true);
                 if (call_r == NULL)
                 {
                     st = rpc_s_no_memory;
                     break;
                 }
                 call_r->common.is_server = true;
-                
+
                 /*
                  * Place the new call rep in the association for
                  * use in cancel processing.
                  */
-                rpc__cn_assoc_push_call (assoc, call_r, &st); 
+                rpc__cn_assoc_push_call (assoc, call_r, &st);
                 if (st != rpc_s_ok)
                 {
                     rpc__list_element_free (&rpc_g_cn_call_lookaside_list,
@@ -962,7 +945,7 @@ rpc_cn_assoc_p_t        assoc;
                 }
 
                 /*
-                 * Initialize some cancel state information. 
+                 * Initialize some cancel state information.
                  */
                 call_r->common.u.server.cancel.accepting = true;
                 call_r->common.u.server.cancel.queuing = true;
@@ -975,24 +958,24 @@ rpc_cn_assoc_p_t        assoc;
                  * or the object UUID in the request packet header in
                  * it, if present.
                  */
-                if (RPC_CN_PKT_FLAGS (pktp) & RPC_C_CN_FLAGS_OBJECT_UUID) 
-                { 
-                    call_r->binding_rep = 
-                    rpc__binding_alloc (true, 
-                                        &RPC_CN_PKT_OBJECT (pktp), 
-                                        RPC_C_PROTOCOL_ID_NCACN, 
-                                        NULL, 
-                                        &st); 
-                } 
-                else 
-                { 
-                    call_r->binding_rep = 
-                    rpc__binding_alloc (true, 
-                                        &uuid_g_nil_uuid, 
-                                        RPC_C_PROTOCOL_ID_NCACN, 
-                                        NULL, 
-                                        &st); 
-                } 
+                if (RPC_CN_PKT_FLAGS (pktp) & RPC_C_CN_FLAGS_OBJECT_UUID)
+                {
+                    call_r->binding_rep =
+                    rpc__binding_alloc (true,
+                                        &RPC_CN_PKT_OBJECT (pktp),
+                                        RPC_C_PROTOCOL_ID_NCACN,
+                                        NULL,
+                                        &st);
+                }
+                else
+                {
+                    call_r->binding_rep =
+                    rpc__binding_alloc (true,
+                                        &uuid_g_nil_uuid,
+                                        RPC_C_PROTOCOL_ID_NCACN,
+                                        NULL,
+                                        &st);
+                }
 
                 /*
                  * If the binding_alloc failed, simply break out of
@@ -1006,9 +989,9 @@ rpc_cn_assoc_p_t        assoc;
                 /*
                  * Put the association group id in the binding rep.
                  */
-                ((rpc_cn_binding_rep_t *)call_r->binding_rep)->grp_id 
-                    = assoc->assoc_grp_id; 
-                call_r->assoc = assoc; 
+                ((rpc_cn_binding_rep_t *)call_r->binding_rep)->grp_id
+                    = assoc->assoc_grp_id;
+                call_r->assoc = assoc;
 
 		/*
 		 * If auth protection level is rpc_c_protect_level_connect
@@ -1033,22 +1016,18 @@ rpc_cn_assoc_p_t        assoc;
                 }
 
                 /*
-                 * Attach the named pipe auth info, if any, to the new 
-                 * binding rep. Make sure to add a reference to it.
+                 * Attach transport info to the bindng rep
                  */
-                if (assoc->security.assoc_named_pipe_info != NULL)
-                {
-                    call_r->binding_rep->np_auth_info = assoc->security.assoc_named_pipe_info;
-		    RPC_NP_AUTH_ADD_REFERENCE (assoc->security.assoc_named_pipe_info);
-                } 
+                call_r->binding_rep->transport_info = assoc->transport_info;
+                rpc__transport_info_retain(assoc->transport_info);
 
                 /*
                  * Post the event to the call state machine.
                  */
                 RPC_CN_POST_FIRST_CALL_SM_EVENT (call_r,
-                                                 assoc, 
-                                                 packet_info_table[ptype].event, 
-                                                 fragbuf_p, 
+                                                 assoc,
+                                                 packet_info_table[ptype].event,
+                                                 fragbuf_p,
                                                  st);
 
                 /*
@@ -1056,7 +1035,7 @@ rpc_cn_assoc_p_t        assoc;
                  * call_request state see whether the security PDU
                  * receive check or key ID lookup performed earlier in this routine,
                  * if required, passed. If it failed a fault PDU
-                 * will be sent back to the client. 
+                 * will be sent back to the client.
                  */
                 if (st != rpc_s_ok)
                 {
@@ -1085,7 +1064,7 @@ rpc_cn_assoc_p_t        assoc;
                     fragbuf_p = NULL;
                     continue;
                 }
-                
+
                 call_r->u.server.if_id = &pres_context->syntax_abstract_id.id;
                 call_r->u.server.if_vers = pres_context->syntax_abstract_id.version;
                 call_r->transfer_syntax.index = pres_context->syntax_vector_index;
@@ -1104,7 +1083,7 @@ rpc_cn_assoc_p_t        assoc;
                                           rpc__cn_call_executor,
                                           (pointer_t) call_r,
                                           &st);
-                
+
                 /*
                  * Check whether the call was queued. If it was, just
                  * set the status code to "ok".
@@ -1139,16 +1118,16 @@ rpc_cn_assoc_p_t        assoc;
             }
             else
             {
-                RPC_CN_POST_CALL_SM_EVENT (assoc, 
-                                           packet_info_table[ptype].event, 
+                RPC_CN_POST_CALL_SM_EVENT (assoc,
+                                           packet_info_table[ptype].event,
                                            fragbuf_p,
                                            st);
             }
         }
-        else 
+        else
         {
-            RPC_CN_ASSOC_EVAL_NETWORK_EVENT (assoc, 
-                                             packet_info_table[ptype].event, 
+            RPC_CN_ASSOC_EVAL_NETWORK_EVENT (assoc,
+                                             packet_info_table[ptype].event,
                                              fragbuf_p,
                                              st);
         }
@@ -1216,7 +1195,7 @@ rpc_cn_assoc_p_t        assoc;
 **      fragbuf_p       pointer to a fragbuf pointer
 **      ovf_fragbuf_p   pointer to a overflow fragbuf pointer
 **
-**  OUTPUTS:            
+**  OUTPUTS:
 **
 **      st              the return status of this routine
 **
@@ -1231,7 +1210,7 @@ rpc_cn_assoc_p_t        assoc;
 **--
 **/
 
-INTERNAL void receive_packet 
+INTERNAL void receive_packet
 #ifdef _DCE_PROTO_
 (
   rpc_cn_assoc_p_t        assoc,
@@ -1257,10 +1236,10 @@ unsigned32              *st;
 
     //DO_NOT_CLOBBER(fbp);
     //DO_NOT_CLOBBER(frag_length);
-	 
+
     RPC_LOG_CN_RCV_PKT_NTR;
     CODING_ERROR (st);
-    
+
     /*
      * One time (auto) initialization.
      */
@@ -1324,7 +1303,7 @@ unsigned32              *st;
         /*
          * Swap bytes if our integer formats are different.
          */
-        if (NDR_DREP_INT_REP(RPC_CN_PKT_DREP((rpc_cn_packet_p_t)fbp->data_p)) 
+        if (NDR_DREP_INT_REP(RPC_CN_PKT_DREP((rpc_cn_packet_p_t)fbp->data_p))
             != NDR_LOCAL_INT_REP)
         {
             SWAB_INPLACE_16 (frag_length);
@@ -1379,12 +1358,12 @@ unsigned32              *st;
 	    dcethread_enableasync_throw(1);
 	    dcethread_checkinterrupt();
 #endif /* NON_CANCELLABLE_IO */
-            RPC_SOCKET_RECVMSG (assoc->cn_ctlblk.cn_sock,
-                                &iov,
-                                1,
-                                addr,
-                                &bytes_rcvd,
-                                &serr);
+            serr = rpc__socket_recvmsg(
+                assoc->cn_ctlblk.cn_sock,
+                &iov,
+                1,
+                addr,
+                (int*) &bytes_rcvd);
 #ifdef NON_CANCELLABLE_IO
 	    dcethread_enableasync_throw(0);
 #endif /* NON_CANCELLABLE_IO */
@@ -1396,7 +1375,7 @@ unsigned32              *st;
 #endif /* NON_CANCELLABLE_IO */
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
 ("CN: call_rep->%x assoc->%x desc->%x receiver canceled, caught in receive_packet()\n",
-                             assoc->call_rep, 
+                             assoc->call_rep,
                              assoc,
                              assoc->cn_ctlblk.cn_sock));
 
@@ -1438,8 +1417,8 @@ unsigned32              *st;
         while (assoc->cn_ctlblk.in_sendmsg)
         {
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                            ("CN: call_rep->%x assoc->%x desc->%x waiting for sendmsg to complete...\n", 
-                             assoc->call_rep, 
+                            ("CN: call_rep->%x assoc->%x desc->%x waiting for sendmsg to complete...\n",
+                             assoc->call_rep,
                              assoc,
                              assoc->cn_ctlblk.cn_sock,
                              bytes_rcvd));
@@ -1447,16 +1426,16 @@ unsigned32              *st;
             RPC_COND_WAIT (assoc->cn_ctlblk.cn_rcvr_cond,
                            rpc_g_global_mutex);
             RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                            ("CN: call_rep->%x assoc->%x desc->%x sendmsg complete\n", 
-                             assoc->call_rep, 
+                            ("CN: call_rep->%x assoc->%x desc->%x sendmsg complete\n",
+                             assoc->call_rep,
                              assoc,
                              assoc->cn_ctlblk.cn_sock,
                              bytes_rcvd));
             assoc->cn_ctlblk.waiting_for_sendmsg_complete = false;
         }
-        
+
         /*
-         * Process any errors reading the socket or any errors detected by 
+         * Process any errors reading the socket or any errors detected by
          * other threads using this association. These other threads will have
          * cleaned up the assoc state and the receiver thread just has to close
          * the connection now.
@@ -1479,23 +1458,23 @@ unsigned32              *st;
                                 ("CN: call_rep->%x assoc->%x desc->%x connection closed recvmsg failed serr = %x, bytes_rcvd = %d\n",
                                  assoc->call_rep,
                                  assoc,
-                                 assoc->cn_ctlblk.cn_sock, 
+                                 assoc->cn_ctlblk.cn_sock,
                                  serr,
                                  bytes_rcvd));
-                
+
                 /*
                  * Deallocate the fragbuf which we were using.
                  */
                 (*fbp->fragbuf_dealloc)(fbp);
-                
+
                 *st = rpc_s_connection_closed;
                 return;
             }
         }
 
         RPC_DBG_PRINTF (rpc_e_dbg_general, RPC_C_CN_DBG_GENERAL,
-                        ("CN: call_rep->%x assoc->%x desc->%x received %d bytes\n", 
-                         assoc->call_rep, 
+                        ("CN: call_rep->%x assoc->%x desc->%x received %d bytes\n",
+                         assoc->call_rep,
                          assoc,
                          assoc->cn_ctlblk.cn_sock,
                          bytes_rcvd));
@@ -1559,7 +1538,7 @@ unsigned32              *st;
 			RPC_CN_PKT_VERS_MINOR ((rpc_cn_packet_p_t)fbp->data_p) ));
                 }
             }
-            
+
             /*
              * Sanity check the fragment size.
              * Signal an error if this fragment is bigger than a fragbuf.
@@ -1579,7 +1558,7 @@ unsigned32              *st;
 		    svc_c_sev_warning,
 		    rpc_m_frag_toobig,
                     assoc,
-                    frag_length, 
+                    frag_length,
                     rpc_g_cn_large_frag_size ));
 
                 /*
