@@ -67,7 +67,7 @@ typedef struct {
     ndr_boolean             done;
     rpc_binding_handle_t    ep_binding;
     unsigned32              inquiry_type;
-    uuid_t                  object;
+    dce_uuid_t                  object;
     rpc_if_id_t             if_id;
     unsigned32              vers_option;
     ept_lookup_handle_t     entry_handle;
@@ -315,7 +315,7 @@ unsigned32                  *status;
 {
     ept_entry_t                 *ept_entry;
     rpc_if_rep_p_t              if_rep = (rpc_if_rep_p_t) ifspec;
-    rpc_binding_handle_t        ep_binding;
+    rpc_binding_handle_t        ep_binding = NULL;
     unsigned32                  i, j;
     unsigned32                  curr_hand = 0, curr_tower = 0, curr_obj = 0; 
     unsigned32                  st;
@@ -336,71 +336,94 @@ unsigned32                  *status;
         return;
     }          
 
+#ifndef __hpux__
     /*
-     * Spin through the binding vector array looking for a non-NULL pointer
-     * to a binding handle.  If one is found set curr_hand to its offset in 
-     * the binding vector.
-     * Also check for a binding handle with no endpoint; if one is found
-     * return an error.
-     */                                      
+     * First attempt to contact the local endpoint mapper with ncalrpc.
+     * This should work even if the network has not yet come up
+     */
+    rpc_binding_from_string_binding(
+        (unsigned_char_t*) "ncalrpc:[epmapper]",
+        &ep_binding,
+        status);
 
-    curr_hand = binding_vec->count;
-    for (i = 0; i < binding_vec->count; i++) 
+    if (*status == rpc_s_ok)
     {
-        if (binding_vec->binding_h[i] != NULL) 
-        {   
-            rpc_binding_rep_p_t binding_rep = (rpc_binding_rep_p_t) binding_vec->binding_h[i];
-            if (binding_rep->addr_has_endpoint == false)
-            {
-                *status = rpc_s_invalid_binding;
-                return;
-            }
-
-#ifdef __hpux__
-            if (binding_rep->rpc_addr && binding_rep->rpc_addr->sa.family == RPC_C_NAF_ID_IP)
-            {
-                struct sockaddr_in* addr = (struct sockaddr_in*) &binding_rep->rpc_addr->sa;
-                unsigned char* addr_bytes = (unsigned char*) &addr->sin_addr.s_addr;
-             
-                /* Don't attempt to connect on the loopback interface on HP-UX
-                   because the ep mapper incorrectly rejects attempts to register
-                   mappings through it */
-                if (addr_bytes[0] == 127)
-                {
-                    continue;
-                }
-            }
-
-            if (binding_rep->rpc_addr && binding_rep->rpc_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCALRPC)
-            {
-                /* Don't attempt to connect over ncalrpc on HP-UX because its native DCE
-                   implementation does not understand it */
-                continue;
-            }
-#endif
-
-            curr_hand = i;
-            break;
+        if (!rpc_mgmt_is_server_listening(ep_binding, status))
+        {
+            rpc_binding_free(&ep_binding, status);
+            ep_binding = NULL;
         }
     }
+#endif
 
-    /*
-     * If all the handles were NULL'd out, return an error.
-     */
-
-    if (curr_hand >= binding_vec->count)
+    if (!ep_binding)
     {
-        *status = rpc_s_no_bindings;
-        return;
-    }          
-                                             
-    /*
-     * Otherwise, use the a valid handle to converse with the EP mapper.
-     */
+        /*
+         * Spin through the binding vector array looking for a non-NULL pointer
+         * to a binding handle.  If one is found set curr_hand to its offset in
+         * the binding vector.
+         * Also check for a binding handle with no endpoint; if one is found
+         * return an error.
+         */
 
-    get_ep_binding(binding_vec->binding_h[curr_hand], &ep_binding, status);
-    if (*status != rpc_s_ok)
-        return;
+        curr_hand = binding_vec->count;
+        for (i = 0; i < binding_vec->count; i++)
+        {
+            if (binding_vec->binding_h[i] != NULL)
+            {
+                rpc_binding_rep_p_t binding_rep = (rpc_binding_rep_p_t) binding_vec->binding_h[i];
+                if (binding_rep->addr_has_endpoint == false)
+                {
+                    *status = rpc_s_invalid_binding;
+                    return;
+                }
+
+#ifdef __hpux__
+                if (binding_rep->rpc_addr && binding_rep->rpc_addr->sa.family == RPC_C_NAF_ID_IP)
+                {
+                    struct sockaddr_in* addr = (struct sockaddr_in*) &binding_rep->rpc_addr->sa;
+                    unsigned char* addr_bytes = (unsigned char*) &addr->sin_addr.s_addr;
+
+                    /* Don't attempt to connect on the loopback interface on HP-UX
+                       because the ep mapper incorrectly rejects attempts to register
+                       mappings through it */
+                    if (addr_bytes[0] == 127)
+                    {
+                        continue;
+                    }
+                }
+
+                if (binding_rep->rpc_addr && binding_rep->rpc_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCALRPC)
+                {
+                    /* Don't attempt to connect over ncalrpc on HP-UX because its native DCE
+                       implementation does not understand it */
+                    continue;
+                }
+#endif
+
+                curr_hand = i;
+                break;
+            }
+        }
+
+        /*
+         * If all the handles were NULL'd out, return an error.
+         */
+
+        if (curr_hand >= binding_vec->count)
+        {
+            *status = rpc_s_no_bindings;
+            return;
+        }
+
+        /*
+         * Otherwise, use the a valid handle to converse with the EP mapper.
+         */
+
+        get_ep_binding(binding_vec->binding_h[curr_hand], &ep_binding, status);
+        if (*status != rpc_s_ok)
+            return;
+    }
 
     /*
      * Allocate the EP entry structure to avoid taking up stack space.
@@ -423,13 +446,11 @@ unsigned32                  *status;
             continue;
         }
 
-#ifdef __hpux__
-        /* Skip over ncalrpc endpoints on HP-UX because its DCE implementation does not understand them */
-        if (binding_rep->rpc_addr && binding_rep->rpc_addr->rpc_protseq_id == RPC_C_PROTSEQ_ID_NCALRPC)
+        if (binding_rep->rpc_addr && rpc_g_protseq_id[binding_rep->rpc_addr->rpc_protseq_id].uses_ep_mapper == 0)
         {
+            /* Skip over protocol sequences that do not require the endpoint mapper */
             continue;
         }
-#endif
 
         /*
          * Convert the binding handle to tower_ref vector.
@@ -553,7 +574,7 @@ unsigned32                  *status;
              * First create the partial object vector.
              */
 
-            ov_size = sizeof(uuid_vector_t) + (sizeof(uuid_t) * (curr_obj - 1));
+            ov_size = sizeof(uuid_vector_t) + (sizeof(dce_uuid_t) * (curr_obj - 1));
             RPC_MEM_ALLOC(ovp, uuid_vector_p_t, ov_size,
                 RPC_C_MEM_UUID_VECTOR, RPC_C_MEM_WAITOK);
             if (ovp == NULL)
@@ -645,7 +666,7 @@ unsigned32                  *status;
 { 
     ept_entry_t                 *ept_entry;
     rpc_if_rep_p_t              if_rep = (rpc_if_rep_p_t) ifspec;
-    rpc_binding_handle_t        ep_binding;
+    rpc_binding_handle_t        ep_binding = NULL;
     unsigned32                  i, j, k, st;
     rpc_tower_ref_vector_p_t    tower_vec;
     unsigned32                  curr_hand;
@@ -666,47 +687,70 @@ unsigned32                  *status;
         return;
     }          
 
+#ifndef __hpux__
     /*
-     * Spin through the binding vector array looking for a non-NULL pointer
-     * to a binding handle.  If one is found set curr_hand to its offset in 
-     * the binding vector.
-     * Also check for a binding handle with no endpoint; if one is found
-     * return an error.
-     */                                      
+     * First attempt to contact the local endpoint mapper with ncalrpc.
+     * This should work even if the network has not yet come up
+     */
+    rpc_binding_from_string_binding(
+        (unsigned_char_t*) "ncalrpc:[epmapper]",
+        &ep_binding,
+        status);
 
-    curr_hand = binding_vec->count;
-    for (i = 0; i < binding_vec->count; i++)
+    if (*status == rpc_s_ok)
     {
-        if (binding_vec->binding_h[i] != NULL) 
-        {   
-            if (((rpc_binding_rep_p_t) binding_vec->binding_h[i])
-                ->addr_has_endpoint == false)
-            {
-                *status = rpc_s_invalid_binding;
-                return;
-    }
-
-            curr_hand = i;
+        if (!rpc_mgmt_is_server_listening(ep_binding, status))
+        {
+            rpc_binding_free(&ep_binding, status);
+            ep_binding = NULL;
         }
     }
+#endif
 
-    /*
-     * If all the handles were NULL'd out, return an error.
-     */
-
-    if (curr_hand >= binding_vec->count)
+    if (!ep_binding)
     {
-        *status = rpc_s_no_bindings;
-        return;
-    }          
-                                             
-    /*
-     * Otherwise, use the a valid handle to converse with the EP mapper.
-     */
+        /*
+         * Spin through the binding vector array looking for a non-NULL pointer
+         * to a binding handle.  If one is found set curr_hand to its offset in
+         * the binding vector.
+         * Also check for a binding handle with no endpoint; if one is found
+         * return an error.
+         */
 
-    get_ep_binding(binding_vec->binding_h[curr_hand], &ep_binding, status);
-    if (*status != rpc_s_ok)
-        return;
+        curr_hand = binding_vec->count;
+        for (i = 0; i < binding_vec->count; i++)
+        {
+            if (binding_vec->binding_h[i] != NULL)
+            {
+                if (((rpc_binding_rep_p_t) binding_vec->binding_h[i])
+                    ->addr_has_endpoint == false)
+                {
+                    *status = rpc_s_invalid_binding;
+                    return;
+                }
+
+                curr_hand = i;
+            }
+        }
+
+        /*
+         * If all the handles were NULL'd out, return an error.
+         */
+
+        if (curr_hand >= binding_vec->count)
+        {
+            *status = rpc_s_no_bindings;
+            return;
+        }
+
+        /*
+         * Otherwise, use the a valid handle to converse with the EP mapper.
+         */
+
+        get_ep_binding(binding_vec->binding_h[curr_hand], &ep_binding, status);
+        if (*status != rpc_s_ok)
+            return;
+    }
 
     /*
      * Allocate the EP entry structure to avoid taking up stack space.
@@ -740,6 +784,13 @@ unsigned32                  *status;
         if (binding_vec->binding_h[i] == NULL)
             continue;
 
+        if (binding_vec->binding_h[i]->rpc_addr &&
+            rpc_g_protseq_id[binding_vec->binding_h[i]->rpc_addr->rpc_protseq_id].uses_ep_mapper == 0)
+        {
+            /* Skip over protocol sequences that do not require the endpoint mapper */
+            continue;
+        }
+
         /*
          * Convert the binding handle to tower_ref vector
          */
@@ -858,7 +909,7 @@ PUBLIC void rpc_mgmt_ep_elt_inq_begin
     unsigned32              inquiry_type,
     rpc_if_id_p_t           if_id,
     unsigned32              vers_option,
-    uuid_p_t                object_uuid,
+    dce_uuid_p_t                object_uuid,
     rpc_ep_inq_handle_t     *inquiry_context,
     unsigned32              *status
 )
@@ -875,7 +926,7 @@ unsigned32              *status;
 {
     mgmt_ep_inq_rep_t       *chp;
     rpc_binding_handle_t    ep_binding;
-    uuid_t                  UUID_NIL;
+    dce_uuid_t                  UUID_NIL;
     unsigned32              tmp_st;
 
     CODING_ERROR (status);
@@ -915,7 +966,7 @@ unsigned32              *status;
      * nil_uuid and versions = 0.
      */ 
 
-    uuid_create_nil(&UUID_NIL, &tmp_st);
+    dce_uuid_create_nil(&UUID_NIL, &tmp_st);
 
     switch ((int)inquiry_type)
     {
@@ -1021,7 +1072,7 @@ PUBLIC void rpc_mgmt_ep_elt_inq_next
     rpc_ep_inq_handle_t       inquiry_context,
     rpc_if_id_t               *if_id,
     rpc_binding_handle_t      *binding,
-    uuid_t                    *object_uuid,
+    dce_uuid_t                    *object_uuid,
     unsigned_char_p_t         *annotation,
     unsigned32                *status
 )
@@ -1030,7 +1081,7 @@ PUBLIC void rpc_mgmt_ep_elt_inq_next
 rpc_ep_inq_handle_t       inquiry_context;
 rpc_if_id_t               *if_id;
 rpc_binding_handle_t      *binding;
-uuid_t                    *object_uuid;
+dce_uuid_t                    *object_uuid;
 unsigned_char_p_t         *annotation;
 unsigned32                *status;
 #endif
@@ -1336,7 +1387,7 @@ PUBLIC void rpc_mgmt_ep_unregister
     rpc_binding_handle_t    input_binding,
     rpc_if_id_p_t           if_id,
     rpc_binding_handle_t    binding,
-    uuid_p_t                object_uuid,
+    dce_uuid_p_t                object_uuid,
     unsigned32              *status
 )
 #else
@@ -1352,8 +1403,8 @@ unsigned32              *status;
     rpc_tower_ref_vector_p_t    tower_vec;
     twr_p_t                     tower;
     boolean32                   object_speced;
-    uuid_t                      UUID_NIL;
-    uuid_t                      *objp;
+    dce_uuid_t                      UUID_NIL;
+    dce_uuid_t                      *objp;
     error_status_t              tmp_st;
     rpc_if_rep_t                ifspec;
 
@@ -1421,7 +1472,7 @@ unsigned32              *status;
     if (object_uuid == NULL)
     {
         object_speced = false;
-        uuid_create_nil(&UUID_NIL, &tmp_st);
+        dce_uuid_create_nil(&UUID_NIL, &tmp_st);
         objp = &UUID_NIL;
     }    
     else
@@ -1900,7 +1951,7 @@ unsigned32              *st;
     ept_lookup_handle_t         map_lookup_handle;
     rpc_addr_p_t                *rpc_addr;
     unsigned32                  saved_timeout;
-    uuid_t                      saved_object_uuid;
+    dce_uuid_t                      saved_object_uuid;
     rpc_tower_ref_vector_p_t    tower_vector = NULL;
     twr_p_t                     map_tower = NULL;
     twr_p_t                     * volatile towers = NULL;
