@@ -15,7 +15,6 @@
 
 #include <lwio/lwio.h>
 #include <lw/base.h>
-#include <lwmapsecurity/lwmapsecurity.h>
 
 #define SMB_SOCKET_LOCK(sock) (rpc__smb_socket_lock(sock))
 #define SMB_SOCKET_UNLOCK(sock) (rpc__smb_socket_unlock(sock))
@@ -58,6 +57,7 @@ typedef struct rpc_smb_socket_s
     IO_FILE_HANDLE np;
     rpc_smb_buffer_t sendbuffer;
     rpc_smb_buffer_t recvbuffer;
+    boolean received_last;
     struct
     {
         IO_FILE_HANDLE* queue;
@@ -168,10 +168,15 @@ rpc__smb_transport_info_equal(
     rpc_smb_transport_info_p_t smb_info1 = (rpc_smb_transport_info_p_t) info1;
     rpc_smb_transport_info_p_t smb_info2 = (rpc_smb_transport_info_p_t) info2;
 
-    return (smb_info1->schannel == smb_info2->schannel &&
-            ((smb_info1->creds == NULL && smb_info2->creds == NULL) ||
-             (smb_info1->creds != NULL && smb_info2->creds != NULL &&
-              LwIoCompareCredss(smb_info1->creds, smb_info2->creds))));
+    return
+        (smb_info2 == NULL
+         && smb_info1->creds == NULL
+         && smb_info1->schannel == FALSE) ||
+        (smb_info2 != NULL &&
+         smb_info1->schannel == smb_info2->schannel &&
+         ((smb_info1->creds == NULL && smb_info2->creds == NULL) ||
+          (smb_info1->creds != NULL && smb_info2->creds != NULL &&
+           LwIoCompareCredss(smb_info1->creds, smb_info2->creds))));
 }
 
 INTERNAL
@@ -338,38 +343,31 @@ rpc__smb_buffer_settle(
     buffer->end_cursor = buffer->base + filled;
 }
 
-/* Advance buffer start_cursor to the end of the last packet
-   or the last packet that is the final fragment in a series,
-   whichever comes first.  If the final fragment is found,
-   return true, otherwise false.
-*/
+/* Advance buffer start_cursor to the end of the next valid fragment */
 INTERNAL
 inline
 boolean
-rpc__smb_buffer_advance_cursor(rpc_smb_buffer_p_t buffer, size_t* amount)
+rpc__smb_buffer_advance_cursor(rpc_smb_buffer_p_t buffer, boolean* last)
 {
-    boolean last;
     size_t packet_size;
 
-    while (rpc__smb_buffer_packet_size(buffer) <= rpc__smb_buffer_pending(buffer))
+    if (rpc__smb_buffer_packet_size(buffer) <= rpc__smb_buffer_pending(buffer))
     {
-        last = rpc__smb_buffer_packet_is_last(buffer);
         packet_size = rpc__smb_buffer_packet_size(buffer);
 
-        buffer->start_cursor += packet_size;
-
         if (last)
         {
-            if (amount)
-            {
-                *amount = buffer->start_cursor - buffer->base;
-            }
-
-            return true;
+            *last = rpc__smb_buffer_packet_is_last(buffer);
         }
-    }
 
-    return false;
+        buffer->start_cursor += packet_size;
+
+        return true;
+    }
+    else
+    {
+        return false;
+    }
 }
 
 INTERNAL
@@ -567,6 +565,8 @@ rpc__smb_socket_construct(
                 goto error;
             }
         }
+
+        smb_sock->info.schannel = smb_info->schannel;
     }
 
     sock->data.pointer = (void*) smb_sock;
@@ -633,6 +633,10 @@ rpc__smb_socket_connect(
     USHORT sesskeylen = 0;
     IO_FILE_NAME filename = { 0 };
     IO_STATUS_BLOCK io_status = { 0 };
+    static const ACCESS_MASK pipeAccess =
+        READ_CONTROL | FILE_WRITE_ATTRIBUTES | FILE_READ_ATTRIBUTES |
+        FILE_WRITE_EA | FILE_READ_EA | FILE_READ_DATA | FILE_APPEND_DATA |
+        FILE_WRITE_DATA;
 
     SMB_SOCKET_LOCK(smb);
 
@@ -685,7 +689,7 @@ rpc__smb_socket_connect(
             &filename,                               /* Filename */
             NULL,                                    /* Security descriptor */
             NULL,                                    /* Security QOS */
-            GENERIC_READ | GENERIC_WRITE,            /* Access mode */
+            pipeAccess,                              /* Access mode */
             0,                                       /* Allocation size */
             0,                                       /* File attributes */
             FILE_SHARE_READ | FILE_SHARE_WRITE,      /* Sharing mode */
@@ -771,7 +775,7 @@ rpc__smb_socket_accept(
     IO_FILE_HANDLE np = NULL;
     size_t i;
     char c = 0;
-    BYTE clientaddr[4] = {0, 0, 0, 0};
+    BYTE clientaddr[16] = {0};
     USHORT clientaddrlen = sizeof(clientaddr);
 
     *newsock = NULL;
@@ -836,10 +840,31 @@ rpc__smb_socket_accept(
         goto error;
     }
 
-    if (clientaddrlen == sizeof(clientaddr))
+    switch (clientaddrlen)
     {
-        snprintf(npsmb->peeraddr.remote_host, sizeof(npsmb->peeraddr.remote_host) - 1,
-                 "%u.%u.%u.%u", clientaddr[0], clientaddr[1], clientaddr[2], clientaddr[3]);
+    case 0:
+        /* No client address */
+        break;
+    case 4:
+        if (inet_ntop(AF_INET, clientaddr, npsmb->peeraddr.remote_host, sizeof(npsmb->peeraddr.remote_host)) == NULL)
+        {
+            serr = errno;
+            goto error;
+        }
+        break;
+#ifdef AF_INET6
+    case 16:
+        if (inet_ntop(AF_INET6, clientaddr, npsmb->peeraddr.remote_host, sizeof(npsmb->peeraddr.remote_host)) == NULL)
+        {
+            serr = errno;
+            goto error;
+        }
+        break;
+#endif
+    default:
+        serr = ENOTSUP;
+        goto error;
+        break;
     }
 
     if (addr)
@@ -1202,7 +1227,7 @@ rpc__smb_socket_sendmsg(
     rpc_socket_error_t serr = RPC_C_SOCKET_OK;
     rpc_smb_socket_p_t smb = (rpc_smb_socket_p_t) sock->data.pointer;
     int i;
-    size_t pending = 0;
+    boolean last = false;
 
     SMB_SOCKET_LOCK(smb);
 
@@ -1219,7 +1244,7 @@ rpc__smb_socket_sendmsg(
 
     *cc = 0;
 
-    /* Append all fragments into a single buffer */
+    /* Gather vector into a single buffer */
     for (i = 0; i < iov_len; i++)
     {
         serr = rpc__smb_buffer_append(&smb->sendbuffer, iov[i].iov_base, iov[i].iov_len);
@@ -1232,8 +1257,8 @@ rpc__smb_socket_sendmsg(
         *cc += iov[i].iov_len;
     }
 
-    /* Look for the last fragment and do send if we find it */
-    if (rpc__smb_buffer_advance_cursor(&smb->sendbuffer, &pending))
+    /* Send all fragments */
+    while (rpc__smb_buffer_advance_cursor(&smb->sendbuffer, &last))
     {
         serr = rpc__smb_socket_do_send(sock);
         if (serr)
@@ -1241,8 +1266,12 @@ rpc__smb_socket_sendmsg(
             goto error;
         }
 
-        /* Switch into recv mode */
-        rpc__smb_socket_change_state(smb, SMB_STATE_RECV);
+        if (last)
+        {
+            /* Switch into recv mode */
+            rpc__smb_socket_change_state(smb, SMB_STATE_RECV);
+            break;
+        }
     }
 
 cleanup:
@@ -1307,7 +1336,7 @@ rpc__smb_socket_recvmsg(
 
     if (rpc__smb_buffer_length(&smb->recvbuffer) == 0)
     {
-        /* Nothing in buffer, read a complete message */
+        /* Nothing in buffer, read a complete PDU */
         do
         {
             serr = rpc__smb_socket_do_recv(sock, &count);
@@ -1319,7 +1348,7 @@ rpc__smb_socket_recvmsg(
             {
                 break;
             }
-        } while (!rpc__smb_buffer_advance_cursor(&smb->recvbuffer, NULL));
+        } while (!rpc__smb_buffer_advance_cursor(&smb->recvbuffer, &smb->received_last));
 
         /* Reset cursor back to start to begin disperal into scatter buffer */
         smb->recvbuffer.start_cursor = smb->recvbuffer.base;
@@ -1343,8 +1372,11 @@ rpc__smb_socket_recvmsg(
 
             /* Reset buffer because we have emptied it */
             smb->recvbuffer.start_cursor = smb->recvbuffer.end_cursor = smb->recvbuffer.base;
-            /* Switch into send mode */
-            rpc__smb_socket_change_state(smb, SMB_STATE_SEND);
+            /* Switch into send mode if this was the last PDU */
+            if (smb->received_last)
+            {
+                rpc__smb_socket_change_state(smb, SMB_STATE_SEND);
+            }
         }
     }
 
