@@ -297,6 +297,91 @@ unsigned32          *status;
     return;
 }
 
+PRIVATE void rpc__tower_verify
+(
+    byte_p_t            tower_octet_string,
+    unsigned32          length,
+    unsigned32          num_flrs,
+    unsigned32          *status
+)
+{
+    byte_p_t        bound = tower_octet_string + length;
+    byte_p_t        tower_floor;
+    unsigned32      i;
+    unsigned16      prot_count;
+    unsigned16      addr_count;
+
+    CODING_ERROR (status);
+
+    if (bound < tower_octet_string)
+    {
+        *status = rpc_s_not_rpc_tower;
+        return;
+    }
+
+    if (tower_octet_string + RPC_C_TOWER_FLR_COUNT_SIZE >= bound)
+    {
+        *status = rpc_s_not_rpc_tower;
+        return;
+    }
+
+    /*
+     * Initialize local to point to beginning of
+     * the canonical tower floor.
+     */
+    tower_floor = tower_octet_string + RPC_C_TOWER_FLR_COUNT_SIZE;
+
+    /*
+     * For each tower floor, starting at the desired floor,
+     * allocate the floor and initialize with the information
+     * in the tower structure.
+     */
+    for (i = 0; i < num_flrs; i++)
+    {
+        if (tower_floor + RPC_C_TOWER_FLR_LHS_COUNT_SIZE >= bound)
+        {
+            *status = rpc_s_not_rpc_tower;
+            return;
+        }
+
+        memcpy ((char *) &prot_count,
+                (char *) tower_floor,
+                RPC_C_TOWER_FLR_LHS_COUNT_SIZE);
+
+        /*
+         * Convert prot id count to host's endian representation.
+         */
+        RPC_RESOLVE_ENDIAN_INT16 (prot_count);
+
+        if (tower_floor + RPC_C_TOWER_FLR_LHS_COUNT_SIZE + prot_count >= bound)
+        {
+            *status = rpc_s_not_rpc_tower;
+            return;
+        }
+
+        /*
+         * Get the additional information count.
+         */
+        memcpy ((char *) &addr_count,
+                (char *) tower_floor + RPC_C_TOWER_FLR_LHS_COUNT_SIZE + prot_count,
+                RPC_C_TOWER_FLR_RHS_COUNT_SIZE);
+
+        /*
+         * Convert address count to host's endian representation.
+         */
+        RPC_RESOLVE_ENDIAN_INT16 (addr_count);
+
+        /*
+         * Point to the next tower floor in the tower octet string.
+         */
+        tower_floor += RPC_C_TOWER_FLR_LHS_COUNT_SIZE + prot_count +
+            RPC_C_TOWER_FLR_RHS_COUNT_SIZE + addr_count;
+    }
+
+    *status = rpc_s_ok;
+    return;
+}
+
 
 /*
 **++
@@ -716,7 +801,7 @@ unsigned32          *status;
 
         /*
          * Assume success. This way we can check the status
-         * of uuid_equal when it is called.
+         * of dce_uuid_equal when it is called.
          */
         *status = rpc_s_ok;
 
@@ -725,7 +810,7 @@ unsigned32          *status;
             for (j = 0; j < floors_to_search; j++)
             {
                 if ((master_prot_ids[k].prefix == tower_prot_ids[j].prefix) &&
-                    (uuid_equal (&(master_prot_ids[k].uuid), 
+                    (dce_uuid_equal (&(master_prot_ids[k].uuid),
                      &(tower_prot_ids[j].uuid), status)))
                 {
                     master_prot_ids[k].prefix = 0;
@@ -733,7 +818,7 @@ unsigned32          *status;
                 }
                 
                 /*
-                 * Check status from uuid_equal.
+                 * Check status from dce_uuid_equal.
                  * Return if failure.
                  */
                 if (*status != rpc_s_ok)
@@ -951,7 +1036,7 @@ unsigned32              *status;
             /*
              * Check if a syntax id and version match.
              */
-            match = uuid_equal 
+            match = dce_uuid_equal
                     (&(tower_syntax_id.id), &(if_syntax_id->id), status);
 
             if ((match == true) &&
