@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC) 
+**      Remote Procedure Call (RPC)
 **
 **  ABSTRACT:
 **
@@ -84,7 +84,7 @@ INTERNAL void fwd_delayed _DCE_PROTOTYPE_((
  *     FWD_PKT_NOTDONE	- caller should handle packet
  *     FWD_PKT_DONE     - we handled the packet, ok to free it
  *     FWD_PKT_DELAYED  - we saved it, don't handle it, don't free it.
- * 
+ *
  * WARNING: This implementation is NOT re-enterent.
  */
 
@@ -94,8 +94,8 @@ PRIVATE unsigned32 rpc__dg_fwd_pkt (
 {
     rpc_dg_pkt_hdr_p_t  hdrp = rqe->hdrp;
     rpc_if_id_t         if_id;
-    unsigned32          rpc_prot_vers_major; 
-    unsigned32          rpc_prot_vers_minor; 
+    unsigned32          rpc_prot_vers_major;
+    unsigned32          rpc_prot_vers_minor;
     rpc_addr_p_t        fwd_addr;
     rpc_fwd_action_t    fwd_action;
     unsigned32          st;
@@ -117,12 +117,12 @@ PRIVATE unsigned32 rpc__dg_fwd_pkt (
      * NCS 1.5.1 RPC_C_DG_PT_ACK and RPC_C_DG_PT_REJECT pkts do not have
      * a valid interface id so we just assume that they must be for the
      * forwarder server (i.e. they couldn't have ever been forwarded).
-     * Also, a pkt with interface id nil is not valid so don't try to 
+     * Also, a pkt with interface id nil is not valid so don't try to
      * forward it.
      */
 
-    if (RPC_DG_HDR_INQ_PTYPE(hdrp) == RPC_C_DG_PT_ACK || 
-        RPC_DG_HDR_INQ_PTYPE(hdrp) == RPC_C_DG_PT_REJECT || 
+    if (RPC_DG_HDR_INQ_PTYPE(hdrp) == RPC_C_DG_PT_ACK ||
+        RPC_DG_HDR_INQ_PTYPE(hdrp) == RPC_C_DG_PT_REJECT ||
         UUID_IS_NIL(&hdrp->if_id, &st))
     {
         return (FWD_PKT_NOTDONE);
@@ -157,9 +157,9 @@ PRIVATE unsigned32 rpc__dg_fwd_pkt (
     if (st != rpc_s_ok)
     {
         RPC_DBG_GPRINTF(
-            ("(rpc__dg_fwd_pkt) fwd map function returned error (st=%08lx, ptype=%s) [%s]\n", 
+            ("(rpc__dg_fwd_pkt) fwd map function returned error (st=%08lx, ptype=%s) [%s]\n",
             st,
-            rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)), 
+            rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)),
             rpc__dg_act_seq_string(hdrp)));
         return (FWD_PKT_NOTDONE);
     }
@@ -171,9 +171,9 @@ PRIVATE unsigned32 rpc__dg_fwd_pkt (
     switch (fwd_action)
     {
         case rpc_e_fwd_drop:
-            RPC_DBG_PRINTF(rpc_e_dbg_general, 10, 
-                ("(rpc__dg_forward_pkt) dropping (ptype=%s) [%s]\n", 
-                rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)), 
+            RPC_DBG_PRINTF(rpc_e_dbg_general, 10,
+                ("(rpc__dg_forward_pkt) dropping (ptype=%s) [%s]\n",
+                rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)),
                 rpc__dg_act_seq_string(hdrp)));
             return (FWD_PKT_NOTDONE);
 
@@ -208,15 +208,15 @@ INTERNAL void fwd_reject (
     rpc_dg_recvq_elt_p_t rqe)
 {
 
-    RPC_DBG_PRINTF(rpc_e_dbg_general, 10, 
-        ("(fwd_forward) rejecting (ptype=%s) [%s]\n", 
-        rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(rqe->hdrp)), 
+    RPC_DBG_PRINTF(rpc_e_dbg_general, 10,
+        ("(fwd_forward) rejecting (ptype=%s) [%s]\n",
+        rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(rqe->hdrp)),
         rpc__dg_act_seq_string(rqe->hdrp)));
 
     if (! RPC_DG_HDR_FLAG_IS_SET(rqe->hdrp, RPC_C_DG_PF_BROADCAST))
     {
         rpc__dg_xmit_error_body_pkt(
-            sp->sock, (rpc_addr_p_t) &rqe->from, rqe->hdrp, RPC_C_DG_PT_REJECT, 
+            sp->sock, (rpc_addr_p_t) &rqe->from, rqe->hdrp, RPC_C_DG_PT_REJECT,
             nca_s_unk_if);      /* !!! status could be better */
     }
 }
@@ -240,9 +240,9 @@ INTERNAL void fwd_forward (
     boolean             b;
     unsigned32          st;
 
-    RPC_DBG_PRINTF(rpc_e_dbg_general, 10, 
-        ("(fwd_forward) forwarding (ptype=%s) [%s]\n", 
-        rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)), 
+    RPC_DBG_PRINTF(rpc_e_dbg_general, 10,
+        ("(fwd_forward) forwarding (ptype=%s) [%s]\n",
+        rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)),
         rpc__dg_act_seq_string(hdrp)));
 
 
@@ -305,7 +305,7 @@ INTERNAL void fwd_forward (
      * in a pkt once and the address gets too large, this forwarding
      * scheme becomes undesireable, there won't be much (any) room left
      * for data.
-     * 
+     *
      * Mark the pkt as forwarded and send it (them).  Note that we're
      * setting bits in the original packet header (via "rhdrp"), NOT
      * the potentially byte-swapped header (via "hdrp").
@@ -326,7 +326,7 @@ INTERNAL void fwd_forward (
         unsigned8 orig_len_b1 = rhdrp->hdr[RPC_C_DG_RPHO_LEN + 1];
 
         /*
-         * The first piece just contains the sender's address 
+         * The first piece just contains the sender's address
          */
 
         rhdrp->hdr[RPC_C_DG_RPHO_FLAGS2] |= RPC_C_DG_PF2_FORWARDED_2;
@@ -356,7 +356,7 @@ INTERNAL void fwd_forward (
 
 #else
 
-#error "No code for MISPACKED_HDR!" 
+#error "No code for MISPACKED_HDR!"
 
 #endif
 
@@ -377,7 +377,7 @@ INTERNAL pkt_list_element_t           *delayed_pkt_head = NULL;
 
 /*
  * I N I T _ F W D _ L I S T _ M U T E X
- * 
+ *
  * Routine to init the fwd_list mutex
  */
 PRIVATE void rpc__dg_fwd_init(void)
@@ -424,12 +424,12 @@ INTERNAL void fwd_delayed (
 
 /*
  * R P C _ _ S E R V E R _ F W D _ R E S O L V E _ D E A L Y E D
- * 
+ *
  * Remove specified packet from the list of delayed packets
  * and do what we are told with it
  */
 PRIVATE void rpc__server_fwd_resolve_delayed(
-    uuid_p_t             actuuid,
+    dce_uuid_p_t             actuuid,
     rpc_addr_p_t	fwd_addr,
     rpc_fwd_action_t	*fwd_action,
     unsigned32		*status)
@@ -450,7 +450,7 @@ PRIVATE void rpc__server_fwd_resolve_delayed(
     while (ep != NULL)
     {
         hdrp = ep->rqe->hdrp;
-        if (uuid_equal(&(hdrp->actuid), actuuid, &st) && (st == rpc_s_ok))
+        if (dce_uuid_equal(&(hdrp->actuid), actuuid, &st) && (st == rpc_s_ok))
         {
             /* found - remove it from the list */
             rqe = ep->rqe;
@@ -483,9 +483,9 @@ PRIVATE void rpc__server_fwd_resolve_delayed(
     switch (*fwd_action)
     {
         case rpc_e_fwd_drop:
-            RPC_DBG_PRINTF(rpc_e_dbg_general, 10, 
-               ("(rpc__server_fwd_resolve_delayed) dropping (ptype=%s) [%s]\n", 
-                rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(rqe->hdrp)), 
+            RPC_DBG_PRINTF(rpc_e_dbg_general, 10,
+               ("(rpc__server_fwd_resolve_delayed) dropping (ptype=%s) [%s]\n",
+                rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(rqe->hdrp)),
                 rpc__dg_act_seq_string(rqe->hdrp)));
             break;
 
