@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC) 
+**      Remote Procedure Call (RPC)
 **
 **  ABSTRACT:
 **
@@ -57,7 +57,7 @@ GLOBAL rpc_dg_stats_t rpc_g_dg_stats = RPC_DG_STATS_INITIALIZER;
 
 GLOBAL unsigned32 rpc_g_dg_server_boot_time;
 
-GLOBAL uuid_t rpc_g_dg_my_cas_uuid;
+GLOBAL dce_uuid_t rpc_g_dg_my_cas_uuid;
 
 GLOBAL rpc_dg_sock_pool_t rpc_g_dg_sock_pool;
 /* ======= */
