@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC) 
+**      Remote Procedure Call (RPC)
 **
 **  ABSTRACT:
 **
@@ -88,7 +88,7 @@ INTERNAL unsigned16 pkt_log_index;
  * No lock requirements.
  */
 
-INTERNAL char act_seq_string_buff[64]; 
+INTERNAL char act_seq_string_buff[64];
 
 #ifndef NO_SPRINTF
 #  define RPC__DG_ACT_SEQ_SPRINTF   sprintf
@@ -154,17 +154,17 @@ unsigned32 lossy_action;    /* (0)drop, (1)?, (2)rexmit, (3)normal */
     memmove( &pp->hdr, hdrp, sizeof(rpc_dg_raw_pkt_hdr_t));
 
 #ifndef MISPACKED_HDR
-    /* b_c_o_p_y_ (bodyp, ((char *) pp->body), 
+    /* b_c_o_p_y_ (bodyp, ((char *) pp->body),
           MIN(MAX_LOGGED_PKT_BODY_LEN, ((rpc_dg_pkt_hdr_p_t) hdrp)->len));*/
 
-    memmove( pp->body, bodyp, 
+    memmove( pp->body, bodyp,
           MIN(MAX_LOGGED_PKT_BODY_LEN, ((rpc_dg_pkt_hdr_p_t) hdrp)->len)) ;
 #else
 
-/* b_c_o_p_y(bodyp, ((char *) pp->body), 
+/* b_c_o_p_y(bodyp, ((char *) pp->body),
        MIN(MAX_LOGGED_PKT_BODY_LEN, ...extracted from raw hdr... hdrp->len));*/
 
-    memmove( pp->body, bodyp, 
+    memmove( pp->body, bodyp,
           MIN(MAX_LOGGED_PKT_BODY_LEN, ...extracted from raw hdr... hdrp->len));
 #endif
 
@@ -220,25 +220,25 @@ PRIVATE void rpc__dg_plog_dump(void)
         if (p->timestamp == 0)
             break;
 
-        uuid_to_string(&hdrp->object, &obj, &st);
-        uuid_to_string(&hdrp->if_id, &iface, &st);
-        uuid_to_string(&hdrp->actuid, &act, &st);
+        dce_uuid_to_string(&hdrp->object, &obj, &st);
+        dce_uuid_to_string(&hdrp->if_id, &iface, &st);
+        dce_uuid_to_string(&hdrp->actuid, &act, &st);
 
         RPC_DBG_PRINTF(rpc_e_dbg_dg_pktlog, 1,
 	    ("%08x %c%c%1u %-4.4s %02x %02x %08x/%04x/%04x %04x %04x %4d %s/%02u/%03u %s %9u %s %02x%02x%02x %02x\n",
                 p->timestamp,
                 (hdrp->_rpc_vers & 0x80) ? 'R' : lossy_action[p->lossy_action],
                 ((i + 1) % RPC_C_DG_PKT_LOG_SIZE == pkt_log_index) ? '*' : ' ',
-                hdrp->_rpc_vers & 0x7f, 
+                hdrp->_rpc_vers & 0x7f,
                 rpc__dg_pkt_name(RPC_DG_HDR_INQ_PTYPE(hdrp)),
                 hdrp->flags, hdrp->flags2,
                 hdrp->seq, hdrp->fragnum,
                 hdrp->serial_hi << 8 | hdrp->serial_lo,
                 hdrp->ihint, hdrp->ahint,
-                hdrp->len, 
-                iface, hdrp->if_vers, hdrp->opnum, act, 
-                hdrp->server_boot, obj, 
-                hdrp->drep[0], hdrp->drep[1], hdrp->drep[2], 
+                hdrp->len,
+                iface, hdrp->if_vers, hdrp->opnum, act,
+                hdrp->server_boot, obj,
+                hdrp->drep[0], hdrp->drep[1], hdrp->drep[2],
                 hdrp->auth_proto) );
 
         rpc_string_free(&obj, &st);
@@ -283,7 +283,7 @@ rpc_dg_pkt_hdr_p_t hdrp;
 
 
 /*
- * R P C _ _ D G _ P K T _ N A M E  
+ * R P C _ _ D G _ P K T _ N A M E
  *
  * Return the string name for a type of packet.  This can't simply be
  * a variable because of the vagaries of global libraries.
@@ -331,7 +331,7 @@ rpc_dg_ptype_t ptype;
  * R P C _ _ D G _ X M I T _ P K T
  *
  * Send the packet described by the iov to the specified addr using the
- * specified socket.  
+ * specified socket.
  *
  * This is intended as a utility routine for non-critical processing.
  *
@@ -344,7 +344,7 @@ rpc_dg_ptype_t ptype;
  * at iov[0].len -- if it's the size of a raw packet header, iov[0].base
  * points to a raw packet header and should be left alone; otherwise
  * it's the non-raw flavor and must be compressed.
- * 
+ *
  * The output boolean is set according to the success of sendmsg.
  */
 
@@ -390,9 +390,9 @@ boolean *b;
 
     *b = true;
     RPC_DG_SOCKET_SENDMSG_OOL(sock, iov, iovlen, addr, &sentcc, &serr);
-    if (RPC_SOCKET_IS_ERR(serr) || sentcc != sendcc) 
+    if (RPC_SOCKET_IS_ERR(serr) || sentcc != sendcc)
     {
-        RPC_DBG_GPRINTF(("(rpc__dg_xmit_pkt) sendmsg failed, sendcc = %d, sentcc = %d, error = %d\n", 
+        RPC_DBG_GPRINTF(("(rpc__dg_xmit_pkt) sendmsg failed, sendcc = %d, sentcc = %d, error = %d\n",
             sendcc, sentcc, RPC_SOCKET_ETOI(serr)));
         *b = false;
     }
@@ -621,7 +621,7 @@ PRIVATE void rpc__dg_stats_print(void)
         RPC_DBG_PRINTF(rpc_e_dbg_stats, 1,
 	    ("(%02u) %-10s                   %9lu             %9lu\n",
                 i, rpc__dg_pkt_name(i),
-                rpc_g_dg_stats.pstats[i].sent, 
+                rpc_g_dg_stats.pstats[i].sent,
                 rpc_g_dg_stats.pstats[i].rcvd) );
     }
 }
@@ -631,14 +631,14 @@ PRIVATE void rpc__dg_stats_print(void)
 /*
  * R P C _ _ D G _ U U I D _ H A S H
  *
- * A significantly faster (though likely not as good) version of uuid_hash() 
+ * A significantly faster (though likely not as good) version of dce_uuid_hash()
  * (this is the NCS 1.5.1 implementation).
  */
 
 PRIVATE unsigned16 rpc__dg_uuid_hash
 #ifdef _DCE_PROTO_
 (
-    uuid_p_t uuid
+    dce_uuid_p_t uuid
 )
 #else
 (uuid)
