@@ -41,7 +41,7 @@
 
 #include <cn.h>
 
-#include <gssapi/gssapi.h>
+#include <gssapi/gssapi_ext.h>
 #include <krb5.h>
 
 typedef struct
