@@ -1,5 +1,5 @@
 /*
- * 
+ *
  * (c) Copyright 1989 OPEN SOFTWARE FOUNDATION, INC.
  * (c) Copyright 1989 HEWLETT-PACKARD COMPANY
  * (c) Copyright 1989 DIGITAL EQUIPMENT CORPORATION
@@ -16,7 +16,7 @@
  * Packard Company, nor Digital Equipment Corporation makes any
  * representations about the suitability of this software for any
  * purpose.
- * 
+ *
  */
 /*
  */
@@ -28,7 +28,7 @@
 **
 **  FACILITY:
 **
-**      Remote Procedure Call (RPC) 
+**      Remote Procedure Call (RPC)
 **
 **  ABSTRACT:
 **
@@ -46,6 +46,7 @@
 #include <com.h>
 #include <comnaf.h>
 #include <ipnaf.h>
+#include <comsoc_bsd.h>
 
 #undef __USE_GNU
 
@@ -157,7 +158,7 @@ INTERNAL boolean addr_compare _DCE_PROTOTYPE_ ((
         rpc_addr_p_t                 /*addr2*/,
         unsigned32                  * /*status*/
     ));
-      
+
 INTERNAL void inq_max_pth_unfrag_tpdu _DCE_PROTOTYPE_ ((
         rpc_addr_p_t                 /*rpc_addr*/,
         rpc_network_if_id_t          /*iftype*/,
@@ -178,7 +179,7 @@ INTERNAL void set_pkt_nodelay _DCE_PROTOTYPE_ ((
         rpc_socket_t                 /*desc*/,
         unsigned32                  * /*status*/
     ));
-              
+
 INTERNAL boolean is_connect_closed _DCE_PROTOTYPE_ ((
         rpc_socket_t                 /*desc*/,
         unsigned32                  * /*status*/
@@ -233,7 +234,7 @@ INTERNAL void inq_max_frag_size _DCE_PROTOTYPE_ ((
 **  SCOPE:              PRIVATE - EPV declared in ipnaf.h
 **
 **  DESCRIPTION:
-**      
+**
 **  Internet Address Family Initialization routine, rpc__ip_init, is
 **  calld only once, by the Communications Service initialization
 **  procedure,  at the time RPC is initialized.  If the Communications
@@ -273,7 +274,7 @@ INTERNAL void inq_max_frag_size _DCE_PROTOTYPE_ ((
 **/
 
 #include <comp.h>
-PRIVATE void rpc__module_init_func(void)
+PRIVATE void rpc__ip_naf_init_func(void)
 {
 	static rpc_naf_id_elt_t naf[1] = {
 		{
@@ -284,42 +285,46 @@ PRIVATE void rpc__module_init_func(void)
 		}
 	};
 	static rpc_tower_prot_ids_t prot_ids[2] = {
-		{ RPC_C_PROTSEQ_ID_NCADG_IP_UDP,   3, 
+		{ RPC_C_PROTSEQ_ID_NCADG_IP_UDP,   3,
 			{ {0x0A,   { 0, 0, 0, 0, 0, {0} }},       /* DG */
 				{0x08,   { 0, 0, 0, 0, 0, {0} }}, /* port */
 				{0x09,   { 0, 0, 0, 0, 0, {0} }}, /* IP addr */
 				{0x00,   { 0, 0, 0, 0, 0, {0} }}
-			} 
+			}
 		},
 
-		{ RPC_C_PROTSEQ_ID_NCACN_IP_TCP,   3, 
+		{ RPC_C_PROTSEQ_ID_NCACN_IP_TCP,   3,
 			{ {0x0B,   { 0, 0, 0, 0, 0, {0} }},       /* CN */
 				{0x07,   { 0, 0, 0, 0, 0, {0} }}, /* port */
 				{0x09,   { 0, 0, 0, 0, 0, {0} }}, /* IP addr */
-				{0x00,   { 0, 0, 0, 0, 0, {0} }} 
+				{0x00,   { 0, 0, 0, 0, 0, {0} }}
 			}
 		}
 	};
 	static rpc_protseq_id_elt_t seq_ids[2] = {
     {                                   /* Connection-RPC / IP / TCP */
         0,
+        1, /* Uses endpoint mapper */
         RPC_C_PROTSEQ_ID_NCACN_IP_TCP,
         RPC_C_PROTOCOL_ID_NCACN,
         RPC_C_NAF_ID_IP,
         RPC_C_NETWORK_PROTOCOL_ID_TCP,
         RPC_C_NETWORK_IF_ID_STREAM,
         RPC_PROTSEQ_NCACN_IP_TCP,
-        (rpc_port_restriction_list_p_t) NULL
+        (rpc_port_restriction_list_p_t) NULL,
+        &rpc_g_bsd_socket_vtbl
     },
     {                                   /* Datagram-RPC / IP / UDP */
         0,
+        1, /* Uses endpoint mapper */
         RPC_C_PROTSEQ_ID_NCADG_IP_UDP,
         RPC_C_PROTOCOL_ID_NCADG,
         RPC_C_NAF_ID_IP,
         RPC_C_NETWORK_PROTOCOL_ID_UDP,
         RPC_C_NETWORK_IF_ID_DGRAM,
         RPC_PROTSEQ_NCADG_IP_UDP,
-        (rpc_port_restriction_list_p_t) NULL
+        (rpc_port_restriction_list_p_t) NULL,
+        &rpc_g_bsd_socket_vtbl
     }
 
 	};
@@ -328,19 +333,19 @@ PRIVATE void rpc__module_init_func(void)
 	rpc__register_naf_id(naf, 1);
 }
 
-PRIVATE void  rpc__ip_init 
+PRIVATE void  rpc__ip_init
 #ifdef _DCE_PROTO_
 (
     rpc_naf_epv_p_t         *naf_epv,
     unsigned32              *status
 )
 #else
-(naf_epv, status)  
+(naf_epv, status)
 rpc_naf_epv_p_t         *naf_epv;
 unsigned32              *status;
 #endif
 {
-    /*  
+    /*
      * The Internal Entry Point Vectors for the Internet Protocol Family
      * Extension service routines.  At RPC startup time, the IP init routine,
      * rpc__ip_init, is responsible for inserting  a pointer to this EPV into
@@ -374,7 +379,7 @@ unsigned32              *status;
         set_port_restriction,
         get_next_restricted_port,
         inq_max_frag_size
-    };      
+    };
     unsigned32 lstatus;
 
     rpc__ip_init_local_addr_vec (&lstatus);
@@ -395,7 +400,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Create a copy of an RPC address. Allocate memory for a variable
 **  length RPC address, for the IP service.  Insert the Internet
 **  address and endpoint along with the overall length of the allocated
@@ -423,7 +428,7 @@ unsigned32              *status;
 **
 **      rpc_addr        The address of a pointer to an RPC address -
 **                      returned with the address of the memory
-**                      allocated by this routine. 
+**                      allocated by this routine.
 **
 **  OUTPUTS:
 **
@@ -447,7 +452,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_alloc 
+INTERNAL void addr_alloc
 #ifdef _DCE_PROTO_
 (
     rpc_protseq_id_t        rpc_protseq_id,
@@ -470,7 +475,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-    
+
     /*
      * allocate memory for the new RPC address
      */
@@ -501,7 +506,7 @@ unsigned32              *status;
     (*rpc_addr)->rpc_protseq_id = rpc_protseq_id;
     (*rpc_addr)->len = sizeof (struct sockaddr_in);
     (*rpc_addr)->sa.family = naf_id;
-    
+
     /*
      * set the endpoint in the RPC addr
      */
@@ -513,7 +518,7 @@ unsigned32              *status;
      */
     addr_set_netaddr (netaddr, rpc_addr, status);
     if (*status != rpc_s_ok) return;
-    
+
     *status = rpc_s_ok;
 }
 
@@ -525,7 +530,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Obtain the length from the source RPC address.  Allocate memory for a
 **  new, destination  RPC address. Do a byte copy from the surce address
 **  to the destination address.
@@ -534,13 +539,13 @@ unsigned32              *status;
 **
 **     src_rpc_addr     The address of a pointer to an RPC address to be
 **                      copied.  It must be the correct format for Internet
-**                      Protocol. 
+**                      Protocol.
 **
 **  INPUTS/OUTPUTS:
 **
 **     dst_rpc_addr     The address of a pointer to an RPC address -returned
 **                      with the address of the memory allocated by
-**                      this routine. 
+**                      this routine.
 **
 **  OUTPUTS:
 **
@@ -565,13 +570,13 @@ unsigned32              *status;
 **  SIDE EFFECTS:
 **
 **           In the event, the addres of a of memory segment contained in
-**           rpc_addr, is not valid or the length isn't as long as is 
+**           rpc_addr, is not valid or the length isn't as long as is
 **           indicated, a memory fault may result.
 **
 **--
 **/
 
-INTERNAL void addr_copy 
+INTERNAL void addr_copy
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            src_rpc_addr,
@@ -586,7 +591,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-    
+
     /*
      * if the source RPC address looks valid - IP family ok
      */
@@ -632,7 +637,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Free the memory for the RPC address pointed to by the argument
 **  address pointer rpc_addr.  Null the address pointer.  The memory
 **  must have been previously allocated by RPC_MEM_ALLC.
@@ -657,14 +662,14 @@ unsigned32              *status;
 **
 **  FUNCTION VALUE:     none
 **
-**  SIDE EFFECTS:       
+**  SIDE EFFECTS:
 **           In the event, the segment of memory refered to by pointer
 **           rpc_addr, is allocated by means other than RPC_MEM_ALLOC,
 **           unpredictable results will occur when this routine is called.
 **--
 **/
 
-INTERNAL void addr_free 
+INTERNAL void addr_free
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            *rpc_addr,
@@ -677,7 +682,7 @@ unsigned32              *status;
 #endif
 {
     CODING_ERROR (status);
-    
+
     /*
      * free memory of RPC addr
      */
@@ -699,7 +704,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **    Receive the null terminated ascii character string,  rpc_endpoint
 **    and convert to the Internet Protocol byte order format.  Insert
 **    into the RPC address, pointed to by argument rpc_addr.  The only
@@ -721,14 +726,14 @@ unsigned32              *status;
 **
 **          rpc_s_ok                 The call was successful.
 **
-**          rpc_s_invalid_naf_id  Argument, endpoint contains an 
+**          rpc_s_invalid_naf_id  Argument, endpoint contains an
 **                                  unauthorized pointer value.
-**          rpc_s_invalid_endpoint_format Endpoint Argument can not be 
+**          rpc_s_invalid_endpoint_format Endpoint Argument can not be
 **                                  converted (not numeric).
 **
 **  IMPLICIT INPUTS:
 **
-**      A NULL, (first byte NULL), endpoint string is an indicator to 
+**      A NULL, (first byte NULL), endpoint string is an indicator to
 **      the routine to delete the endpoint from the RPC address. indicated.
 **
 **  IMPLICIT OUTPUTS:   none
@@ -740,7 +745,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_set_endpoint 
+INTERNAL void addr_set_endpoint
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       endpoint,
@@ -758,9 +763,9 @@ unsigned32              *status;
     int                 ep;
     int                 ret;
 
-    
+
     CODING_ERROR (status);
-    
+
     /*
      * check to see if this is a request to remove the endpoint
      */
@@ -771,7 +776,7 @@ unsigned32              *status;
         return;
     }
 
-    if ((strspn ((char *)endpoint, "0123456789")) 
+    if ((strspn ((char *)endpoint, "0123456789"))
          != (strlen ((char *)endpoint)))
     {
         *status = rpc_s_invalid_endpoint_format;
@@ -803,7 +808,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **    From the RPC address indicated by arg., rpc_addr, examine the
 **    endpoint.  Convert the endopint value to a NULL terminated asci
 **    character string to be returned in the memory segment pointed to
@@ -846,7 +851,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_endpoint 
+INTERNAL void addr_inq_endpoint
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr,
@@ -875,7 +880,7 @@ unsigned32              *status;
     /*
      * if no endpoint present, return null string. Otherwise,
      * return the endpoint in Internet "dot" notation.
-     */    
+     */
     if (ep  ==  0)
     {
         RPC_MEM_ALLOC(
@@ -897,7 +902,7 @@ unsigned32              *status;
         RPC__IP_ENDPOINT_SPRINTF((char *) *endpoint, "%u", ep);
     }
 
-    *status = rpc_s_ok;    
+    *status = rpc_s_ok;
 }
 
 /*
@@ -908,7 +913,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **    Receive the null terminated ascii character string, netaddr,
 **    and convert to the Internet Protocol Network Address format.  Insert
 **    into the RPC address, indicated by argument rpc_addr.
@@ -944,7 +949,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_set_netaddr 
+INTERNAL void addr_set_netaddr
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       netaddr,
@@ -971,7 +976,7 @@ unsigned32              *status;
     ATTRIBUTE_UNUSED int                 herr;
 #endif /* GETHOSTBYNAME_R_ARGS == 3 */
 
-    CODING_ERROR (status);  
+    CODING_ERROR (status);
 
     /*
      * check to see if this is a request to remove the netaddr
@@ -1018,8 +1023,7 @@ unsigned32              *status;
     if (gethostbyname_r((char *)netaddr, &hbuf, &hdbuf) != 0)
 #else
     /* As a last resort, fall back on gethostbyname */
-    he = gethostbyname((char *)netaddr);
-    if (he != NULL)
+    if((he = gethostbyname((char *)netaddr)) == NULL)
 #endif /* GETHOSTBYNAME_R_ARGS */
     {
         *status = rpc_s_inval_net_addr;
@@ -1045,7 +1049,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **    From the RPC address indicated by arg., rpc_addr, examine the
 **    IP network address.  Convert the network address from its network
 **    format  to a NULL terminated ascii character string in IP dot
@@ -1080,7 +1084,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_netaddr 
+INTERNAL void addr_inq_netaddr
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr,
@@ -1098,10 +1102,10 @@ unsigned32              *status;
 
     rpc_ip_addr_p_t     ip_addr = (rpc_ip_addr_p_t) rpc_addr;
     unsigned8           *p;
-    
-    
+
+
     CODING_ERROR (status);
-    
+
     RPC_MEM_ALLOC(
         *netaddr,
         unsigned_char_p_t,
@@ -1132,7 +1136,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **     Receive a NULL terminated network options string and insert
 **     into the RPC address indicated by art., rpc_addr.
 **
@@ -1166,7 +1170,7 @@ unsigned32              *status;
 **/
 
 
-INTERNAL void addr_set_options 
+INTERNAL void addr_set_options
 #ifdef _DCE_PROTO_
 (
     unsigned_char_p_t       network_options ATTRIBUTE_UNUSED,
@@ -1191,7 +1195,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **      Extract the network options from the RPC address pointed to
 **      by rpc_addr and convert to a NULL terminated string placed
 **      in a buffer indicated by the options arg.
@@ -1227,7 +1231,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void addr_inq_options 
+INTERNAL void addr_inq_options
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr ATTRIBUTE_UNUSED,
@@ -1260,7 +1264,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1272,7 +1276,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      max_tsdu
 **
@@ -1289,7 +1293,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_tsdu 
+INTERNAL void inq_max_tsdu
 #ifdef _DCE_PROTO_
 (
     rpc_naf_id_t            naf_id ATTRIBUTE_UNUSED,
@@ -1308,7 +1312,7 @@ unsigned32              *status;
 #endif
 {
     if (iftype == RPC_C_NETWORK_IF_ID_DGRAM &&
-        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)    
+        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)
     {
         *max_tsdu = RPC_C_IP_UDP_MAX_TSDU;
     }
@@ -1336,7 +1340,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Determine if two address are equal.
 **
 **  INPUTS:
@@ -1347,7 +1351,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1355,7 +1359,7 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:    
+**  FUNCTION VALUE:
 **
 **      return          Boolean; true if address are the same.
 **
@@ -1364,10 +1368,10 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL boolean addr_compare 
+INTERNAL boolean addr_compare
 #ifdef _DCE_PROTO_
 (
-    rpc_addr_p_t            addr1, 
+    rpc_addr_p_t            addr1,
     rpc_addr_p_t            addr2,
     unsigned32              *status ATTRIBUTE_UNUSED
 )
@@ -1402,7 +1406,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1414,7 +1418,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      max_tpdu
 **
@@ -1431,7 +1435,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_pth_unfrag_tpdu 
+INTERNAL void inq_max_pth_unfrag_tpdu
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t            rpc_addr ATTRIBUTE_UNUSED,
@@ -1450,7 +1454,7 @@ unsigned32              *status;
 #endif
 {
     if (iftype == RPC_C_NETWORK_IF_ID_DGRAM &&
-        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)    
+        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)
     {
         *max_tpdu = RPC_C_IP_UDP_MAX_PTH_UNFRG_TPDU;
     }
@@ -1478,7 +1482,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -1490,7 +1494,7 @@ unsigned32              *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      max_tpdu
 **
@@ -1507,7 +1511,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void inq_max_loc_unfrag_tpdu 
+INTERNAL void inq_max_loc_unfrag_tpdu
 #ifdef _DCE_PROTO_
 (
     rpc_naf_id_t            naf_id ATTRIBUTE_UNUSED,
@@ -1526,7 +1530,7 @@ unsigned32              *status;
 #endif
 {
     if (iftype == RPC_C_NETWORK_IF_ID_DGRAM &&
-        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)    
+        protocol == RPC_C_NETWORK_PROTOCOL_ID_UDP)
     {
         *max_tpdu = RPC_C_IP_UDP_MAX_LOC_UNFRG_TPDU;
     }
@@ -1554,7 +1558,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  This routine is responsible for "reverse-engineering" the parameters to
 **  the original socket call that was made to create the socket "desc".
 **
@@ -1585,7 +1589,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void desc_inq_network 
+INTERNAL void desc_inq_network
 #ifdef _DCE_PROTO_
 (
     rpc_socket_t              desc,
@@ -1602,13 +1606,13 @@ unsigned32                *status;
 #endif
 {
     rpc_socket_error_t serr;
-    
+
     CODING_ERROR (status);
 
     /*
      * Get the socket type.
      */
-     
+
     serr = rpc__socket_get_if_id (desc, socket_type);
     if (RPC_SOCKET_IS_ERR (serr))
     {
@@ -1655,7 +1659,7 @@ unsigned32                *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      desc            The network descriptor to apply the nodelay
@@ -1663,7 +1667,7 @@ unsigned32                *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1678,28 +1682,24 @@ unsigned32                *status;
 **--
 **/
 
-INTERNAL void set_pkt_nodelay 
-#ifdef _DCE_PROTO_
+INTERNAL void set_pkt_nodelay
 (
-    rpc_socket_t            desc,
+    rpc_socket_t            sock,
     unsigned32              *status
 )
-#else
-(desc, status)
-rpc_socket_t            desc;
-unsigned32              *status;
-#endif
 {
     int                 err;
     int                 delay = 1;
+    /* FIXME: this really ought to become a new socket vtbl entry point */
+    int                 desc = rpc__socket_get_select_desc(sock);
 
     /*
      * Assume this is a TCP socket and corresponding connection. If
      * not the setsockopt will fail.
      */
-    if ((err = setsockopt (desc, 
-                           IPPROTO_TCP, 
-                           TCP_NODELAY, 
+    if ((err = setsockopt (desc,
+                           IPPROTO_TCP,
+                           TCP_NODELAY,
                            (char *) &delay,
                            sizeof (delay))) < 0)
     {
@@ -1726,14 +1726,14 @@ unsigned32              *status;
 **      sequenced sockets the routine is a no-op. "true" is returned
 **      because zero bytes received on a stream socket does mean the
 **      connection is closed.
-**      
+**
 **  INPUTS:
 **
 **      desc            The network descriptor representing the connection.
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      status          A value indicating the status of the routine.
 **
@@ -1741,7 +1741,7 @@ unsigned32              *status;
 **
 **  IMPLICIT OUTPUTS:   none
 **
-**  FUNCTION VALUE:     
+**  FUNCTION VALUE:
 **
 **      boolean         true if the connection is closed, false otherwise.
 **
@@ -1750,7 +1750,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL boolean is_connect_closed 
+INTERNAL boolean is_connect_closed
 #ifdef _DCE_PROTO_
 (
     rpc_socket_t            desc ATTRIBUTE_UNUSED,
@@ -1775,7 +1775,7 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Creates the lower tower floors from an RPC addr.
 **
 **  INPUTS:
@@ -1801,7 +1801,7 @@ unsigned32              *status;
 **--
 **/
 
-INTERNAL void tower_flrs_from_addr 
+INTERNAL void tower_flrs_from_addr
 #ifdef _DCE_PROTO_
 (
     rpc_addr_p_t       rpc_addr,
@@ -1832,7 +1832,7 @@ unsigned32         *status;
      * their own memory allocation. In that case, twr_* allocates
      * memory for returning the lower towers, and we must copy from
      * twr_* memory into our (rpc) memory. After the copy, we free
-     * the twr_* allocated memory. 
+     * the twr_* allocated memory.
      *
      * For now, twr_* routines also use RPC_MEM_ALLOC and RPC_MEM_FREE,
      * so we'll skip the extra copy.
@@ -1849,11 +1849,11 @@ unsigned32         *status;
     /*
      * Convert sockaddr to lower tower floors.
      */
-    twr_ip_lower_flrs_from_sa (net_prot_id, (sockaddr_t *) &(rpc_addr->sa), 
+    twr_ip_lower_flrs_from_sa (net_prot_id, (sockaddr_t *) &(rpc_addr->sa),
 #if 0
-        &temp_lower_flrs, 
+        &temp_lower_flrs,
 #else
-        lower_flrs, 
+        lower_flrs,
 #endif
         status);
 
@@ -1867,18 +1867,18 @@ unsigned32         *status;
      * Allocate a tower structure to hold the wire (and nameservice)
      * representation of the lower tower floors returned from twr_*().
      *
-     * The size includes the sizof twr_t + length of the tower floors 
+     * The size includes the sizof twr_t + length of the tower floors
      * returned from twr_ip_lower_flrs_from_sa - 1 (for tower_octet_string[0].
      */
     RPC_MEM_ALLOC (
-        *lower_flrs, 
-        twr_p_t, 
+        *lower_flrs,
+        twr_p_t,
         sizeof(twr_t) + temp_lower_flrs->tower_length - 1,
-        RPC_C_MEM_TOWER, 
+        RPC_C_MEM_TOWER,
         RPC_C_MEM_WAITOK );
 
     /*
-     * Set the tower length to the length of the tower flrs returnd from 
+     * Set the tower length to the length of the tower flrs returnd from
      * twr_ip_lower_flrs_from_sa.
      */
     (*lower_flrs)->tower_length = temp_lower_flrs->tower_length;
@@ -1886,7 +1886,7 @@ unsigned32         *status;
     /*
      * Copy the lower tower floors to the tower octet string.
      */
-    memcpy ((*lower_flrs)->tower_octet_string, 
+    memcpy ((*lower_flrs)->tower_octet_string,
         temp_lower_flrs->tower_octet_string,
         temp_lower_flrs->tower_length);
 
@@ -1909,7 +1909,7 @@ unsigned32         *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  Creates an RPC addr from a protocol tower.
 **
 **  INPUTS:
@@ -1938,7 +1938,7 @@ unsigned32         *status;
 **--
 **/
 
-INTERNAL void tower_flrs_to_addr 
+INTERNAL void tower_flrs_to_addr
 #ifdef _DCE_PROTO_
 (
     byte_p_t           tower_octet_string,
@@ -1984,7 +1984,7 @@ unsigned32         *status;
     rpc__naf_addr_from_sa (sa, sa_len, rpc_addr, status);
 
     /*
-     * Always free the twr_ip_lower_flrs_to_sa allocated memory - regardless 
+     * Always free the twr_ip_lower_flrs_to_sa allocated memory - regardless
      * of the status from rpc__naf_addr_from_sa.
      */
     RPC_MEM_FREE (sa, RPC_C_MEM_SOCKADDR);
@@ -2006,7 +2006,7 @@ unsigned32         *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **    Receive a socket descriptor which is queried to obtain family,
 **    remote endpoint and remote network address.  If this information appears valid
 **    for an DECnet IV address,  space is allocated for an RPC address which
@@ -2053,7 +2053,7 @@ unsigned32         *status;
 **--
 **/
 
-INTERNAL void desc_inq_peer_addr 
+INTERNAL void desc_inq_peer_addr
 #ifdef _DCE_PROTO_
 (
     rpc_protseq_id_t        protseq_id,
@@ -2070,7 +2070,7 @@ unsigned32              *status;
 #endif
 {
     rpc_socket_error_t  serr;
-    
+
     CODING_ERROR (status);
 
 
@@ -2082,7 +2082,7 @@ unsigned32              *status;
                    sizeof (rpc_ip_addr_t),
                    RPC_C_MEM_RPC_ADDR,
                    RPC_C_MEM_WAITOK);
-    
+
     /*
      * successful malloc
      */
@@ -2127,21 +2127,21 @@ unsigned32              *status;
 **  SCOPE:              INTERNAL
 **
 **  DESCRIPTION:
-**      
+**
 **  Builds an rpc_port_restriction_list_t and glues it into the
-**  rpc_protseq_id_elt_t in the rpc_g_protseq_id[] list.  
-**  
+**  rpc_protseq_id_elt_t in the rpc_g_protseq_id[] list.
+**
 **  INPUTS:
 **
 **      protseq_id
 **                      The protocol sequence id to set port restriction
-**                      on. 
+**                      on.
 **      n_elements
 **                      The number of port ranges passed in.
 **
 **      first_port_name_list
 **                      An array of pointers to strings containing the
-**                      lower bound port names. 
+**                      lower bound port names.
 **
 **      last_port_name_list
 **                      An array of pointers to strings containing the
@@ -2174,7 +2174,7 @@ INTERNAL void set_port_restriction
      unsigned_char_p_t           *last_port_name_list,
      unsigned32                  *status
 )
-#else 
+#else
 (protseq_id, n_elements, first_port_name_list, last_port_name_list, status)
 rpc_protseq_id_t            protseq_id;
 unsigned32                  n_elements;
@@ -2191,7 +2191,7 @@ unsigned32                  *status;
 
     CODING_ERROR (status);
 
-    /* 
+    /*
      * It is only meaningful to do this once per protocol sequence.
      */
 
@@ -2201,11 +2201,11 @@ unsigned32                  *status;
         return;
     }
 
-    /* 
+    /*
      * Allocate the port_restriction_list.
      */
 
-    RPC_MEM_ALLOC 
+    RPC_MEM_ALLOC
         (list_p,
          rpc_port_restriction_list_p_t,
          sizeof (rpc_port_restriction_list_t),
@@ -2217,12 +2217,12 @@ unsigned32                  *status;
         *status = rpc_s_no_memory;
         return;
     }
-                   
-    /* 
+
+    /*
      * Allocate the port_range_element vector.
      */
 
-    RPC_MEM_ALLOC (range_elements,        
+    RPC_MEM_ALLOC (range_elements,
                    rpc_port_range_element_p_t,
                    sizeof (rpc_port_range_element_t) * n_elements,
                    RPC_C_MEM_PORT_RANGE_ELEMENTS,
@@ -2234,15 +2234,15 @@ unsigned32                  *status;
         return;
     }
 
-    /* 
-     * Initialize the rpc_port_restriction_list 
+    /*
+     * Initialize the rpc_port_restriction_list
      */
 
     list_p -> n_tries = 0;
     list_p -> n_elements = n_elements;
     list_p -> range_elements = range_elements;
 
-    /* 
+    /*
      * Loop and initialize the range element list.
      */
 
@@ -2253,7 +2253,7 @@ unsigned32                  *status;
              ((char *) first_port_name_list[i], "%lu", &low)
              != 1)       ||
             (RPC__IP_ENDPOINT_SSCANF
-             ((char *) last_port_name_list[i], "%lu", &high) 
+             ((char *) last_port_name_list[i], "%lu", &high)
              != 1)       ||
             (low > high))
         {
@@ -2268,22 +2268,22 @@ unsigned32                  *status;
 	range_elements[i].low = (unsigned32) low;
 	range_elements[i].high = (unsigned32) high;
 
-        list_p -> n_tries += 
+        list_p -> n_tries +=
             range_elements[i].high - range_elements[i].low + 1;
     }                                   /* for i */
 
-    /* 
+    /*
      * Randomly choose a starting range and a port within the range.
      */
 
     list_p -> current_range_element = RPC_RANDOM_GET (0, n_elements - 1);
     i = list_p -> current_range_element;
 
-    list_p -> current_port_in_range = 
+    list_p -> current_port_in_range =
         RPC_RANDOM_GET (range_elements[i].low, range_elements[i].high);
 
-    /* 
-     * Everything was successful.  Wire the port_restriction_list into the 
+    /*
+     * Everything was successful.  Wire the port_restriction_list into the
      * protseq descriptor table.
      */
 
@@ -2301,15 +2301,15 @@ unsigned32                  *status;
 **  SCOPE:              INTERNAL
 **
 **  DESCRIPTION:
-**      
+**
 **  Returns the next restricted port in a sequence.  There is no guarantee
 **  that the port is available, that is up to bind() to determine.
-**  
+**
 **  INPUTS:
 **
 **      protseq_id
-**                      The protocol sequence id to get the port on. 
-**      
+**                      The protocol sequence id to get the port on.
+**
 **  INPUTS/OUTPUTS:     none
 **
 **  OUTPUTS:
@@ -2349,7 +2349,7 @@ unsigned32                 *status;
 
     CODING_ERROR (status);
 
-    /* 
+    /*
      * Validate that this protocol sequence has a port restriction.
      */
 
@@ -2357,7 +2357,7 @@ unsigned32                 *status;
 
     if (list_p == NULL)
     {
-        /* 
+        /*
          * Return an error to tell the caller that there is no range
          * restriction on this protocol sequence.
          */
@@ -2366,7 +2366,7 @@ unsigned32                 *status;
         return;
     }
 
-    /* 
+    /*
      * Alloc a string and return to caller.
      */
 
@@ -2381,35 +2381,35 @@ unsigned32                 *status;
     RPC__IP_ENDPOINT_SPRINTF
         ((char *) *port_name, "%lu", (unsigned long) list_p -> current_port_in_range);
 
-    /* 
-     * Increment to the next restricted port number.  Handle wrapping 
+    /*
+     * Increment to the next restricted port number.  Handle wrapping
      * beyond end of this range.
      */
-    
-    range_p = (rpc_port_range_element_p_t) list_p -> range_elements + 
+
+    range_p = (rpc_port_range_element_p_t) list_p -> range_elements +
         list_p -> current_range_element;
 
     if (++ (list_p -> current_port_in_range) > range_p -> high)
     {
-        /* 
+        /*
          * Advance to next range and wraparound as needed.
          */
-        
+
         list_p -> current_range_element =
-            (++ list_p -> current_range_element) % (list_p -> n_elements);
+            (list_p -> current_range_element + 1) % (list_p -> n_elements);
 
         range_p = (rpc_port_range_element_p_t) list_p -> range_elements +
             list_p -> current_range_element;
 
-        /* 
+        /*
          * Set next port in new range to the lowest in that range.
          */
 
         list_p -> current_port_in_range = range_p -> low;
 
     }                                   /* wrapped to end of range */
-        
-    /* 
+
+    /*
      * Success
      */
 
@@ -2424,7 +2424,7 @@ unsigned32                 *status;
 **  SCOPE:              INTERNAL - declared locally
 **
 **  DESCRIPTION:
-**      
+**
 **  INPUTS:
 **
 **      naf_id          Network Address Family ID serves
@@ -2436,7 +2436,7 @@ unsigned32                 *status;
 **
 **  INPUTS/OUTPUTS:     none
 **
-**  OUTPUTS:                        
+**  OUTPUTS:
 **
 **      max_tpdu
 **
@@ -2476,7 +2476,7 @@ unsigned32   *status;
     if (RPC_PROTSEQ_INQ_NET_IF_ID(rpc_addr->rpc_protseq_id)
           != RPC_C_NETWORK_IF_ID_DGRAM
         || RPC_PROTSEQ_INQ_NET_PROT_ID(rpc_addr->rpc_protseq_id)
-            != RPC_C_NETWORK_PROTOCOL_ID_UDP)    
+            != RPC_C_NETWORK_PROTOCOL_ID_UDP)
     {
         assert(false);      /* !!! */
     }
