@@ -78,7 +78,7 @@ GLOBAL ept_v3_0_epv_t ept_v3_0_mgr_epv =
 
 #ifndef REMOTE_ENDPOINT_ACCESS
 
-INTERNAL int is_remote_handle( handle_t h, error_status_t *st )
+INTERNAL int is_unpriv_handle( handle_t h, error_status_t *st )
 {
 
     error_status_t status,status1;
@@ -88,6 +88,35 @@ INTERNAL int is_remote_handle( handle_t h, error_status_t *st )
     unsigned32 i;
     static unsigned_char_p_t *local_netaddr = NULL;
     static unsigned32 addr_count = 0;
+    unsigned32 prot_seq = 0;
+    rpc_transport_info_handle_t info;
+    unsigned32 uid = (unsigned32) -1;
+    unsigned32 gid = (unsigned32) -1;
+
+    rpc_binding_inq_prot_seq(h, &prot_seq, &status);
+
+    if (! STATUS_OK(&status))
+    {
+        *st = status;
+        return(1);
+    }
+
+    if (prot_seq == rpc_c_protseq_id_ncalrpc)
+    {
+        rpc_binding_inq_transport_info(h, &info, &status);
+
+        if (! STATUS_OK(&status))
+        {
+            *st = status;
+            return(1);
+        }
+
+        rpc_lrpc_transport_info_inq_peer_eid(info, &uid, &gid);
+
+        *st = rpc_s_ok;
+
+        return (uid != 0);
+    }
 
 /* Get client network address from binding handle (client_netaddr) */
 
@@ -275,7 +304,7 @@ error_status_t      *status;
         return;
 
 #ifndef REMOTE_ENDPOINT_ACCESS
-    if ( is_remote_handle(h,&tmp_st) )
+    if ( is_unpriv_handle(h,&tmp_st) )
     {
         *status = ept_s_cant_perform_op;
         return;
@@ -323,7 +352,7 @@ error_status_t      *status;
         return;
 
 #ifndef REMOTE_ENDPOINT_ACCESS
-    if ( is_remote_handle(h,&tmp_st) )
+    if ( is_unpriv_handle(h,&tmp_st) )
     {
         *status = ept_s_cant_perform_op;
         return;
@@ -346,7 +375,7 @@ PRIVATE void ept_lookup(h, inquiry_type, object, interface, vers_option, entry_h
                       max_ents, num_ents, entries, status)
 handle_t            h;
 unsigned32          inquiry_type;
-uuid_p_t            object;
+dce_uuid_p_t            object;
 rpc_if_id_p_t       interface;
 unsigned32          vers_option;
 ept_lookup_handle_t *entry_handle;
@@ -374,7 +403,7 @@ error_status_t      *status;
 PRIVATE void ept_map(h, object, map_tower, entry_handle,
                         max_towers, num_towers, towers, status)
 handle_t            h;
-uuid_p_t            object;
+dce_uuid_p_t            object;
 twr_p_t             map_tower;
 ept_lookup_handle_t *entry_handle;
 unsigned32          max_towers;
@@ -430,7 +459,7 @@ error_status_t      *status;
 
 PRIVATE void ept_inq_object(h, object, status)
 handle_t            h;
-uuid_t              *object;
+dce_uuid_t              *object;
 error_status_t      *status;
 {
     epdb_handle_t epdb;
@@ -445,7 +474,7 @@ error_status_t      *status;
 PRIVATE void ept_mgmt_delete(h, object_speced, object, tower, status)
 handle_t            h;
 boolean32           object_speced;
-uuid_p_t            object;
+dce_uuid_p_t            object;
 twr_p_t             tower;
 error_status_t      *status;
 {
@@ -457,7 +486,7 @@ error_status_t      *status;
         return;
 
 #ifndef REMOTE_ENDPOINT_ACCESS
-    if ( is_remote_handle(h,&tmp_st) )
+    if ( is_unpriv_handle(h,&tmp_st) )
     {
         *status = ept_s_cant_perform_op;
         return;
