@@ -32,22 +32,22 @@
 #include "djdistroinfo.h"
 #include "djsshconf.h"
 #include "djpamconf.h"
-#include "djtimemgmt.h"
 #include "djcli.h"
 #include "djfirewall.h"
+#include "djauditing.h"
 #include "ctprocutils.h"
 #include "lwexc.h"
 
-#define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
+#define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
-CENTERROR ParseUInt(PCSTR in, unsigned int *out)
+DWORD ParseUInt(PCSTR in, unsigned int *out)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     PSTR endPtr;
     *out = strtoul(in, &endPtr, 10);
     if(*endPtr != '\0')
     {
-        GCE(ceError = CENTERROR_INVALID_OPTION_VALUE);
+        GCE(ceError = ERROR_INVALID_PARAMETER);
     }
 
 cleanup:
@@ -62,15 +62,20 @@ ShowUsage()
     fprintf(stdout, "  where options are:\n\n");
     fprintf(stdout, "    --help                                     Display this help information.\n");
     fprintf(stdout, "    --help-internal                            Display help for debug commands\n");
-    fprintf(stdout, "    --log {.|path}                             Log to a file (or \".\" to log\n"
+    fprintf(stdout, "    --logfile {.|path}                         Log to a file (or \".\" to log\n"
                     "                                               to console).\n");
     fprintf(stdout, "    --loglevel {error|warning|info|verbose}    Adjusts how much logging is\n"
-                    "                                               produced by domainjoin.\n\n");
+                    "                                               produced by domainjoin.\n");
+    fprintf(stdout, "    --nodcerpcd                                prevents dcerpcd from being\n"
+                    "                                               started, along with any daemons\n"
+                    "                                               that depend on it.\n");
+    fprintf(stdout, "\n");
     fprintf(stdout, "  and commands are:\n\n");
     fprintf(stdout, "    query\n");
     fprintf(stdout, "    setname <computer name>\n");
     fprintf(stdout, "    join [--notimesync] [--enable <module> --disable <module> ...] [--ou <organizationalUnit>] <domain name> <user name> [<password>]\n");
     fprintf(stdout, "    join [--advanced] --preview [--ou <organizationalUnit>] <domain name>\n");
+    fprintf(stdout, "    join [--assumeDefaultDomain] [--userDomainPrefix <short domain name>] [--ou <organizationalUnit>] <domain name>\n");
     fprintf(stdout, "    join [--ou <organizationalUnit>] --details <module> <domain name>\n");
     fprintf(stdout, "    leave [--enable <module> --disable <module> ...] [user name] [password]\n");
     fprintf(stdout, "    leave [--advanced] --preview [user name] [password]\n");
@@ -88,35 +93,47 @@ ShowUsageInternal()
 
     fprintf(stdout, "  Internal debug commands:\n");
     fprintf(stdout, "    fixfqdn\n");
-    fprintf(stdout, "    configure pam [--testprefix <dir>] { --enable | --disable }\n");
-    fprintf(stdout, "    configure nsswitch [--testprefix <dir>] { --enable | --disable }\n");
-    fprintf(stdout, "    configure ssh [--testprefix <dir>] { --enable | --disable }\n");
-    fprintf(stdout, "    configure krb5 [--testprefix <dir>] [--long <longdomain>] [--short <shortdomain>] { --enable | --disable }\n");
-    fprintf(stdout, "    configure firewall [--testprefix <dir>] { --enable | --disable }\n");
+    fprintf(stdout, "    configure { --enable | --disable } pam [--testprefix <dir>]\n");
+    fprintf(stdout, "    configure { --enable | --disable } nsswitch [--testprefix <dir>]\n");
+    fprintf(stdout, "    configure { --enable | --disable } ssh [--testprefix <dir>]\n");
+    fprintf(stdout, "    configure { --enable | --disable } [--testprefix <dir>] [--long <longdomain>] [--short <shortdomain>] krb5\n");
+    fprintf(stdout, "    configure { --enable | --disable } firewall [--testprefix <dir>]\n");
+    fprintf(stdout, "    configure { --enable | --disable } eventfwdd\n");
+    fprintf(stdout, "    configure { --enable | --disable } reapsysld\n");
     fprintf(stdout, "    get_os_type\n");
     fprintf(stdout, "    get_arch\n");
     fprintf(stdout, "    get_distro\n");
     fprintf(stdout, "    get_distro_version\n");
-    fprintf(stdout, "    sync_time <domain controller> <allowed drift>\n");
     fprintf(stdout, "    raise_error <error code | error name | 0xhex error code>\n");
     fprintf(stdout, "\n");
 }
 
 static
-CENTERROR
+DWORD
 FillMissingPassword(
     PCSTR username,
     PSTR* ppszPassword
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     PSTR pszPassword = NULL;
+    PCSTR pszEnvPassword = NULL;
 
-    fprintf(stdout, "%s's password: ",username);
-    fflush(stdout);
-    ceError = GetPassword(&pszPassword);
-    BAIL_ON_CENTERIS_ERROR(ceError);
-    fprintf(stdout, "\n");
+    pszEnvPassword = getenv("PASSWORD");
+    if (pszEnvPassword == NULL)
+    {
+        fprintf(stdout, "%s's password: ",username);
+        fflush(stdout);
+        ceError = GetPassword(&pszPassword);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+        fprintf(stdout, "\n");
+    }
+    else
+    {
+        DJ_LOG_WARNING("Retrieved password from envionmental variable");
+        ceError = CTStrdup(pszEnvPassword, &pszPassword);
+        BAIL_ON_CENTERIS_ERROR(ceError);
+    }
 
     if (!IsNullOrEmptyString(pszPassword)) {
         *ppszPassword = pszPassword;
@@ -156,12 +173,12 @@ void PrintWarning(const JoinProcessOptions *options, const char *title, const ch
 {
     PSTR      wrapped = NULL;
     int columns;
-    if(!CENTERROR_IS_OK(CTGetTerminalWidth(fileno(stdout), &columns)))
+    if(CTGetTerminalWidth(fileno(stdout), &columns))
         columns = -1;
 
-    //This function doesn't return a CENTERROR, so we have to recover as much
+    //This function doesn't return a DWORD, so we have to recover as much
     //as possible.
-    if(CENTERROR_IS_OK(CTWordWrap(message, &wrapped, 4, columns)))
+    if(!CTWordWrap(message, &wrapped, 4, columns))
         fprintf(stdout, "Warning: %s\n%s\n\n", title, wrapped);
     else
         fprintf(stdout, "Warning: %s\n%s\n\n", title, message);
@@ -283,6 +300,7 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
     DynamicArray enableModules, disableModules;
     DynamicArray detailModules;
     size_t i;
+    int passwordIndex = -1;
     PSTR moduleDetails = NULL;
     PSTR wrapped = NULL;
 
@@ -298,7 +316,9 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
         else if(!strcmp(argv[0], "--preview"))
             preview = TRUE;
         else if(!strcmp(argv[0], "--ignore-firewall-ntp"))
-            options.ignoreFirewallNtp = TRUE;
+        {
+            printf("Warning: --ignore-firewall-ntp is deprecated. This behavior is now default.\n");
+        }
         else if(!strcmp(argv[0], "--ignore-pam"))
             options.ignorePam = TRUE;
         else if(!strcmp(argv[0], "--notimesync"))
@@ -310,7 +330,7 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
         }
         else if(argc < 2)
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         else if(!strcmp(argv[0], "--enable"))
@@ -333,14 +353,29 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
         }
         else if(!strcmp(argv[0], "--ou"))
         {
+            DJ_LOG_INFO("Domainjoin invoked with option --ou %s", argv[1]);
             CT_SAFE_FREE_STRING(options.ouName);
             LW_CLEANUP_CTERR(exc, CTStrdup(argv[1], &options.ouName));
             argv++;
             argc--;
         }
+        else if(!strcmp(argv[0], "--assumeDefaultDomain"))
+        {
+            DJ_LOG_INFO("Domainjoin invoked with option --assumeDefaultDomain");
+            options.assumeDefaultDomain = TRUE;
+        }
+        else if(!strcmp(argv[0], "--userDomainPrefix"))
+        {
+            DJ_LOG_INFO("Domainjoin invoked with option --userDomainPrefix %s", argv[1]);
+            options.assumeDefaultDomain = TRUE;
+            CT_SAFE_FREE_STRING(options.userDomainPrefix);
+            LW_CLEANUP_CTERR(exc, CTStrdup(argv[1], &options.userDomainPrefix));
+            argv++;
+            argc--;
+        }
         else
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         argv++;
@@ -348,17 +383,26 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
     }
 
     if(argc == 3)
+    {
         LW_CLEANUP_CTERR(exc, CTStrdup(argv[2], &options.password));
+        passwordIndex = 2;
+    }
     // The join username is not required in preview or details mode.
     else if(argc == 1 && (preview || detailModules.size != 0) )
         ;
     else if(argc != 2)
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
     options.joiningDomain = TRUE;
 
+    DJ_LOG_INFO("Domainjoin invoked with %d arg(s) to the join command:", argc);
+    for(i = 0; i < argc; i++)
+    {
+        DJ_LOG_INFO("    [%s]", i == passwordIndex ? "<password>" : argv[i]);
+    }
+
     LW_CLEANUP_CTERR(exc, CTStrdup(
         argv[0], &options.domainName));
     if(argc > 1)
@@ -378,7 +422,7 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
                     &enableModules, i, sizeof(PCSTR));
         if(CTArrayFindString(&disableModules, module) != -1)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_OPTION_VALUE, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
             goto cleanup;
         }
         LW_TRY(exc, DJEnableModule(&options, module, TRUE, &LW_EXC));
@@ -390,7 +434,7 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
                     &disableModules, i, sizeof(PCSTR));
         if(CTArrayFindString(&enableModules, module) != -1)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_OPTION_VALUE, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
             goto cleanup;
         }
         LW_TRY(exc, DJEnableModule(&options, module, FALSE, &LW_EXC));
@@ -403,7 +447,7 @@ void DoJoin(int argc, char **argv, int columns, LWException **exc)
         ModuleState *state = DJGetModuleStateByName(&options, module);
         if(state == NULL)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_PARAMETER, "Unable to find module.", "Please check the spelling of '%s'. This module cannot be found", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Unable to find module.", "Please check the spelling of '%s'. This module cannot be found", module);
             goto cleanup;
         }
         PrintModuleState(state);
@@ -468,6 +512,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
     size_t i;
     PSTR moduleDetails = NULL;
     PSTR wrapped = NULL;
+    int passwordIndex = -1;
 
     DJZeroJoinProcessOptions(&options);
     memset(&enableModules, 0, sizeof(enableModules));
@@ -482,7 +527,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
             preview = TRUE;
         else if(argc < 2)
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         else if(!strcmp(argv[0], "--enable"))
@@ -505,7 +550,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
         }
         else
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         argv++;
@@ -513,14 +558,23 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
     }
 
     if(argc == 2)
+    {
         LW_CLEANUP_CTERR(exc, CTStrdup(argv[1], &options.password));
+        passwordIndex = 1;
+    }
     else if(argc > 2)
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
     options.joiningDomain = FALSE;
 
+    DJ_LOG_INFO("Domainjoin invoked with %d arg(s) to the leave command:", argc);
+    for(i = 0; i < argc; i++)
+    {
+        DJ_LOG_INFO("    [%s]", i == passwordIndex ? "<password>" : argv[i]);
+    }
+
     if(argc > 0)
     {
         LW_CLEANUP_CTERR(exc, CTStrdup(argv[0], &options.username));
@@ -538,7 +592,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
                     &enableModules, i, sizeof(PCSTR));
         if(CTArrayFindString(&disableModules, module) != -1)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_OPTION_VALUE, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
             goto cleanup;
         }
         LW_TRY(exc, DJEnableModule(&options, module, TRUE, &LW_EXC));
@@ -550,7 +604,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
                     &disableModules, i, sizeof(PCSTR));
         if(CTArrayFindString(&enableModules, module) != -1)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_OPTION_VALUE, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Module already specified", "The module '%s' is listed as being disabled and enabled", module);
             goto cleanup;
         }
         LW_TRY(exc, DJEnableModule(&options, module, FALSE, &LW_EXC));
@@ -563,7 +617,7 @@ void DoLeaveNew(int argc, char **argv, int columns, LWException **exc)
         ModuleState *state = DJGetModuleStateByName(&options, module);
         if(state == NULL)
         {
-            LW_RAISE_EX(exc, CENTERROR_INVALID_PARAMETER, "Unable to find module.", "Please check the spelling of '%s'. This module cannot be found", module);
+            LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Unable to find module.", "Please check the spelling of '%s'. This module cannot be found", module);
             goto cleanup;
         }
         PrintModuleState(state);
@@ -618,6 +672,7 @@ cleanup:
     CT_SAFE_FREE_STRING(wrapped);
 }
 
+#ifndef ENABLE_MINIMAL
 void DoConfigure(int argc, char **argv, LWException **exc)
 {
     EnableType dwEnable = ENABLE_TYPE_AUTO;
@@ -634,7 +689,7 @@ void DoConfigure(int argc, char **argv, LWException **exc)
             dwEnable = ENABLE_TYPE_DISABLE;
         else if(argc < 2)
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         else if(!strcmp(argv[0], "--testprefix"))
@@ -657,7 +712,7 @@ void DoConfigure(int argc, char **argv, LWException **exc)
         }
         else
         {
-            LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+            LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
             goto cleanup;
         }
         argv++;
@@ -666,7 +721,7 @@ void DoConfigure(int argc, char **argv, LWException **exc)
 
     if(argc < 1)
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
 
@@ -680,24 +735,27 @@ void DoConfigure(int argc, char **argv, LWException **exc)
     else if(!strcmp(argv[0], "krb5"))
         LW_CLEANUP_CTERR(exc, DJModifyKrb5Conf(testPrefix,
             GetEnableBoolean(dwEnable), longDomain, shortDomain, NULL));
-    else if(!strcmp(argv[0], "firewall"))
-        LW_CLEANUP_CTERR(exc, DJConfigureFirewallForAuth(testPrefix, GetEnableBoolean(dwEnable)));
+    else if(!strcmp(argv[0], "eventfwdd"))
+        LW_CLEANUP_CTERR(exc, DJConfigureEventFwd(testPrefix, GetEnableBoolean(dwEnable)));
+    else if(!strcmp(argv[0], "reapsysld"))
+        LW_CLEANUP_CTERR(exc, DJConfigureReapSyslog(testPrefix, GetEnableBoolean(dwEnable)));
     else
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
     fprintf(stdout, "SUCCESS\n");
 cleanup:
     ;
 }
+#endif
 
 void DoGetDistroInfo(int argc, char **argv, LWException **exc)
 {
     PSTR str = NULL;
     PCSTR requestType = argv[0];
     PCSTR testPrefix = NULL;
-    DistroInfo distro;
+    DistroInfo distro = {0};
 
     argc--;
     argv++;
@@ -710,7 +768,7 @@ void DoGetDistroInfo(int argc, char **argv, LWException **exc)
     }
     if(argc > 0)
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
 
@@ -733,7 +791,7 @@ void DoGetDistroInfo(int argc, char **argv, LWException **exc)
     }
     else
     {
-        LW_RAISE(exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
     fprintf(stdout, "%s\n", str);
@@ -756,11 +814,12 @@ int main(
     DWORD dwLogLevel;
     BOOLEAN showHelp = FALSE;
     BOOLEAN showInternalHelp = FALSE;
+    BOOLEAN bEnableDcerpcd = TRUE;
     int remainingArgs = argc;
     char **argPos = argv;
     int i;
 
-    if(!CENTERROR_IS_OK(CTGetTerminalWidth(fileno(stdout), &columns)))
+    if(CTGetTerminalWidth(fileno(stdout), &columns))
         columns = -1;
 
     /* Skip the program name */
@@ -780,7 +839,7 @@ int main(
         //All options after this point take an argument
         else if(remainingArgs < 2)
             showHelp = TRUE;
-        else if(!strcmp(argPos[0], "--log"))
+        else if(!strcmp(argPos[0], "--logfile"))
         {
             pszLogFilePath = (++argPos)[0];
             remainingArgs--;
@@ -790,6 +849,8 @@ int main(
             logLevel = (++argPos)[0];
             remainingArgs--;
         }
+        else if (!strcmp(argPos[0], "--nodcerpcd"))
+            bEnableDcerpcd = FALSE;
         else
             break;
         remainingArgs--;
@@ -818,7 +879,7 @@ int main(
     else if (!strcasecmp(logLevel, "verbose"))
         dwLogLevel = LOG_LEVEL_VERBOSE;
     else {
-        LW_CLEANUP_CTERR(&exc, CENTERROR_DOMAINJOIN_INVALID_LOG_LEVEL);
+        LW_CLEANUP_CTERR(&exc, LW_ERROR_INVALID_LOG_LEVEL);
     }
 
     if (bNoLog) {
@@ -827,21 +888,33 @@ int main(
                !strcmp(pszLogFilePath, ".")) {
         LW_CLEANUP_CTERR(&exc, dj_init_logging_to_console(dwLogLevel));
     } else {
-        CENTERROR ceError = dj_init_logging_to_file(dwLogLevel, pszLogFilePath);
-        if(ceError == CENTERROR_ACCESS_DENIED)
+        DWORD ceError = dj_init_logging_to_file(dwLogLevel, pszLogFilePath);
+        if(ceError == ERROR_ACCESS_DENIED)
         {
-            fprintf(stderr, "Warning: insufficient permissions to log to %s. To enable logging, please specify a different filename with --log <file>.\n",
+            fprintf(stderr, "Warning: insufficient permissions to log to %s. To enable logging, please specify a different filename with --logfile <file>.\n",
                     pszLogFilePath);
-            ceError = CENTERROR_SUCCESS;
+            ceError = ERROR_SUCCESS;
             LW_CLEANUP_CTERR(&exc, dj_disable_logging());
         }
         LW_CLEANUP_CTERR(&exc, ceError);
     }
 
-    DJ_LOG_INFO("Domainjoin invoked with %d arg(s):", argc);
-    for(i = 0; i < argc; i++)
+    if (!strcmp(argPos[0], "join") || !strcmp(argPos[0], "leave"))
     {
-        DJ_LOG_INFO("    [%s]", argv[i]);
+        DJ_LOG_INFO("Domainjoin invoked with the %s command (remaining arguments will be printed later):", argPos[0]);
+        // Only print up to the 'join' part
+        for (i = 0; i <= argPos - argv; i++)
+        {
+            DJ_LOG_INFO("    [%s]", argv[i]);
+        }
+    }
+    else
+    {
+        DJ_LOG_INFO("Domainjoin invoked with %d arg(s):", argc);
+        for (i = 0; i < argc; i++)
+        {
+            DJ_LOG_INFO("    [%s]", argv[i]);
+        }
     }
 
     if(!strcmp(argPos[0], "setname"))
@@ -853,48 +926,38 @@ int main(
             goto cleanup;
         }
         //Needed so that DJGetMachineSid does not call winbind
-        LW_TRY(&exc, DJNetInitialize(&LW_EXC));
+        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DJSetComputerName(argPos[0], NULL, &LW_EXC));
     }
-    else if(!strcmp(argPos[0], "sync_time"))
-    {
-        unsigned int allowedDrift;
-        argPos++;
-        if(--remainingArgs != 2)
-        {
-            ShowUsage();
-            goto cleanup;
-        }
-        LW_CLEANUP_CTERR(&exc, ParseUInt(argPos[1], &allowedDrift));
-        LW_CLEANUP_CTERR(&exc, DJSyncTimeToDC(argPos[0], allowedDrift));
-    }
     else if(!strcmp(argPos[0], "join"))
     {
         argPos++;
         remainingArgs--;
-        LW_TRY(&exc, DJNetInitialize(&LW_EXC));
+        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoJoin(remainingArgs, argPos, columns, &LW_EXC));
     }
     else if(!strcmp(argPos[0], "leave"))
     {
         argPos++;
         remainingArgs--;
-        LW_TRY(&exc, DJNetInitialize(&LW_EXC));
+        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoLeaveNew(remainingArgs, argPos, columns, &LW_EXC));
     }
     else if(!strcmp(argPos[0], "query"))
     {
-        LW_TRY(&exc, DJNetInitialize(&LW_EXC));
+        LW_TRY(&exc, DJNetInitialize(bEnableDcerpcd, &LW_EXC));
         LW_TRY(&exc, DoQuery(&LW_EXC));
     }
     else if(!strcmp(argPos[0], "fixfqdn"))
         LW_TRY(&exc, DoFixFqdn(&LW_EXC));
+#ifndef ENABLE_MINIMAL
     else if(!strcmp(argPos[0], "configure"))
     {
         argPos++;
         remainingArgs--;
         LW_TRY(&exc, DoConfigure(remainingArgs, argPos, &LW_EXC));
     }
+#endif
     else if(!strcmp(argPos[0], "get_os_type") ||
         !strcmp(argPos[0], "get_arch") ||
         !strcmp(argPos[0], "get_distro") ||
@@ -902,30 +965,15 @@ int main(
     {
         LW_TRY(&exc, DoGetDistroInfo(remainingArgs, argPos, &LW_EXC));
     }
-    else if(!strcmp(argPos[0], "raise_error"))
-    {
-        CENTERROR ceError;
-        argPos++;
-        if(--remainingArgs != 1)
-        {
-            ShowUsage();
-            goto cleanup;
-        }
-        if (isdigit((int) *argPos[0]))
-            ceError = (CENTERROR) strtoul(argPos[0], NULL, 0);
-        else
-            ceError = (CENTERROR) CTErrorFromName(argPos[0]);
-        LW_CLEANUP_CTERR(&exc, ceError);
-    }
     else
     {
-        LW_RAISE(&exc, CENTERROR_DOMAINJOIN_SHOW_USAGE);
+        LW_RAISE(&exc, LW_ERROR_SHOW_USAGE);
         goto cleanup;
     }
 
 cleanup:
 
-    if (!LW_IS_OK(exc) && exc->code == CENTERROR_DOMAINJOIN_SHOW_USAGE)
+    if (!LW_IS_OK(exc) && exc->code == LW_ERROR_SHOW_USAGE)
     {
         ShowUsage();
         LWHandle(&exc);
