@@ -33,14 +33,14 @@
 #include "djdistroinfo.h"
 #include "djdaemonmgr.h"
 
-#define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
+#define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
 static PSTR pszChkConfigPath = "/sbin/chkconfig";
 static PSTR pszUpdateRcDFilePath = "/usr/sbin/update-rc.d";
 
 #define DAEMON_BINARY_SEARCH_PATH "/usr/local/sbin:/usr/local/bin:/usr/dt/bin:/opt/dce/sbin:/usr/sbin:/usr/bin:/sbin:/bin:" BINDIR ":" SBINDIR
 
-CENTERROR
+DWORD
 GetInitScriptDir(PSTR *store)
 {
 #if defined(_AIX)
@@ -71,7 +71,7 @@ DJGetDaemonStatus(
     long status = 0;
     PPROCINFO pProcInfo = NULL;
     BOOLEAN daemon_installed = FALSE;
-    CENTERROR ceError;
+    DWORD ceError;
 
     if(pszDaemonName[0] == '/')
         LW_CLEANUP_CTERR(exc, CTStrdup(pszDaemonName, &prefixedPath));
@@ -85,7 +85,7 @@ DJGetDaemonStatus(
     LW_CLEANUP_CTERR(exc, CTCheckFileExists(prefixedPath, &daemon_installed));
 
     if (!daemon_installed) {
-        LW_CLEANUP_CTERR(exc, CENTERROR_DOMAINJOIN_MISSING_DAEMON);
+        LW_CLEANUP_CTERR(exc, ERROR_SERVICE_NOT_FOUND);
     }
 
     /* AIX has /etc/rc.dt. When '/etc/rc.dt status' is run, it tries to
@@ -102,7 +102,17 @@ DJGetDaemonStatus(
          */
         status = 1;
     }
-    else
+#ifdef __LWI_FREEBSD7__
+    /* FreeBSD 7.x, running the script nscd w/ status option returns 0 (and
+     * prints nothing) even when nscd is not running.
+     */
+    else if (!strcmp(prefixedPath, "/etc/rc.d/nscd"))
+    {
+        status = 1;
+    }
+#endif
+
+    if (!status)
     {
         LW_CLEANUP_CTERR(exc, CTShell("%script status >/dev/null 2>&1; echo $? >%statusBuffer",
                     CTSHELL_STRING(script, prefixedPath),
@@ -149,7 +159,7 @@ DJGetDaemonStatus(
 
         ceError = CTFindFileInPath(daemonBaseName,
                 DAEMON_BINARY_SEARCH_PATH, &daemonPath);
-        if(ceError == CENTERROR_FILE_NOT_FOUND)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
             CT_SAFE_FREE_STRING(altDaemonName);
             LW_CLEANUP_CTERR(exc, CTStrdup(daemonBaseName, &altDaemonName));
@@ -157,7 +167,7 @@ DJGetDaemonStatus(
             ceError = CTFindFileInPath(altDaemonName,
                     DAEMON_BINARY_SEARCH_PATH, &daemonPath);
         }
-        if(ceError == CENTERROR_FILE_NOT_FOUND)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
             CT_SAFE_FREE_STRING(altDaemonName);
             LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&altDaemonName,
@@ -165,16 +175,16 @@ DJGetDaemonStatus(
             ceError = CTFindFileInPath(altDaemonName,
                     DAEMON_BINARY_SEARCH_PATH, &daemonPath);
         }
-        if(ceError == CENTERROR_FILE_NOT_FOUND)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
-            LW_RAISE_EX(exc, CENTERROR_FILE_NOT_FOUND, "Cannot find daemon binary", "Querying the daemon init script with '%s status' returned code 1. This either means the init script does not support the status option, or the daemon died leaving a pid file. This program is instead attempting to see if the program is running using the ps command, however the daemon binary (%s) that goes along with the init script cannot be found.", prefixedPath, daemonBaseName);
+            LW_RAISE_EX(exc, ERROR_FILE_NOT_FOUND, "Cannot find daemon binary", "Querying the daemon init script with '%s status' returned code 1. This either means the init script does not support the status option, or the daemon died leaving a pid file. This program is instead attempting to see if the program is running using the ps command, however the daemon binary (%s) that goes along with the init script cannot be found.", prefixedPath, daemonBaseName);
             goto cleanup;
         }
         LW_CLEANUP_CTERR(exc, ceError);
 
         DJ_LOG_VERBOSE("Found %s", daemonPath);
         ceError = CTGetPidOfCmdLine(NULL, daemonPath, NULL, 0, &daemonPid, NULL);
-        if(ceError == CENTERROR_NO_SUCH_PROCESS || ceError == CENTERROR_NOT_IMPLEMENTED)
+        if(ceError == ERROR_PROC_NOT_FOUND || ceError == ERROR_NOT_SUPPORTED)
         {
             //Nope, couldn't find the daemon running
             *pbStarted = FALSE;
@@ -187,7 +197,7 @@ DJGetDaemonStatus(
         }
     }
     else {
-        LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_UNEXPECTED_ERRCODE, "Non-standard return code from init script", "According to http://forgeftp.novell.com/library/SUSE%20Package%20Conventions/spc_init_scripts.html, init scripts should return 0, 1, 2, 3, or 4. However, '%s status' returned %d.", status, prefixedPath);
+        LW_RAISE_EX(exc, ERROR_INVALID_STATE, "Non-standard return code from init script", "According to http://forgeftp.novell.com/library/SUSE%20Package%20Conventions/spc_init_scripts.html, init scripts should return 0, 1, 2, 3, or 4. However, '%s status' returned %d.", status, prefixedPath);
         goto cleanup;
     }
 
@@ -199,12 +209,13 @@ cleanup:
     CT_SAFE_FREE_STRING(initDir);
     CT_SAFE_FREE_STRING(daemonPath);
     CT_SAFE_FREE_STRING(altDaemonName);
+    CT_SAFE_FREE_STRING(statusBuffer);
 }
 
 static void FindDaemonScript(PCSTR name, PSTR *path, LWException **exc)
 {
     PSTR altName = NULL;
-    CENTERROR ceError;
+    DWORD ceError;
     const char *searchPath = "/etc/init.d:/etc/rc.d/init.d:/sbin/init.d:/etc/rc.d";
     BOOLEAN fileExists = FALSE;
 
@@ -215,7 +226,7 @@ static void FindDaemonScript(PCSTR name, PSTR *path, LWException **exc)
         LW_CLEANUP_CTERR(exc, CTCheckFileOrLinkExists(name, &fileExists));
         if (!fileExists)
         {
-            LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_MISSING_DAEMON,
+            LW_RAISE_EX(exc, ERROR_SERVICE_NOT_FOUND,
                     "Unable to find daemon",
                     "The '%s' daemon could not be found.",
                     name);
@@ -225,23 +236,23 @@ static void FindDaemonScript(PCSTR name, PSTR *path, LWException **exc)
     else
     {
         ceError = CTFindFileInPath(name, searchPath, path);
-        if(ceError == CENTERROR_FILE_NOT_FOUND)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
             CT_SAFE_FREE_STRING(altName);
             LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&altName,
                         "%s.rc", name));
             ceError = CTFindFileInPath(altName, searchPath, path);
         }
-        if(ceError == CENTERROR_FILE_NOT_FOUND && !strcmp(name, "dtlogin"))
+        if(ceError == ERROR_FILE_NOT_FOUND && !strcmp(name, "dtlogin"))
         {
             CT_SAFE_FREE_STRING(altName);
             LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&altName,
                         "rc.dt"));
             ceError = CTFindFileInPath(altName, searchPath, path);
         }
-        if(ceError == CENTERROR_FILE_NOT_FOUND)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
-            LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_MISSING_DAEMON,
+            LW_RAISE_EX(exc, ERROR_SERVICE_NOT_FOUND,
                     "Unable to find daemon",
                     "The '%s' daemon could not be found in the search path '%s'. It could not be found with the alternative name '%s' either.",
                     name, searchPath, altName);
@@ -263,12 +274,13 @@ DJStartStopDaemon(
 {
     PSTR* ppszArgs = NULL;
     DWORD nArgs = 4;
-    long status = 0;
+    LONG status = 0;
     PPROCINFO pProcInfo = NULL;
     BOOLEAN bStarted = FALSE;
     PSTR pszDaemonPath = NULL;
     int retry;
 
+#if !defined(__LWI_MACOSX__)
     LW_TRY(exc, FindDaemonScript(pszDaemonName, &pszDaemonPath, &LW_EXC));
 
     if (bStatus) {
@@ -310,14 +322,31 @@ DJStartStopDaemon(
 
         if(bStatus)
         {
-            LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_INCORRECT_STATUS, "Unable to start daemon", "An attempt was made to start the '%s' daemon, but querying its status revealed that it did not start. Try running '%s start; %s status' to diagnose the issue", pszDaemonPath, pszDaemonPath, pszDaemonPath);
+            if (!strcmp(pszDaemonPath, "/sbin/init.d/dtlogin.rc"))
+            {
+                // dtlogin on HP-UX does not have a status option
+                LW_RAISE_EX(exc, ERROR_INVALID_STATE, "Unable to start daemon", "An attempt was made to start the '%s' daemon, but querying its status revealed that it did not start. Try running '%s start; ps -ef | grep dtlogin' to diagnose the issue", pszDaemonPath, pszDaemonPath);
+            }
+            else
+            {
+                LW_RAISE_EX(exc, ERROR_INVALID_STATE, "Unable to start daemon", "An attempt was made to start the '%s' daemon, but querying its status revealed that it did not start. Try running '%s start; %s status' to diagnose the issue", pszDaemonPath, pszDaemonPath, pszDaemonPath);
+            }
         }
         else
         {
-            LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_INCORRECT_STATUS, "Unable to stop daemon", "An attempt was made to stop the '%s' daemon, but querying its status revealed that it did not stop. Try running '%s stop; %s status' to diagnose the issue", pszDaemonPath, pszDaemonPath, pszDaemonPath);
+            if (!strcmp(pszDaemonPath, "/sbin/init.d/dtlogin.rc"))
+            {
+                // dtlogin on HP-UX does not have a status option
+                LW_RAISE_EX(exc, ERROR_INVALID_STATE, "Unable to stop daemon", "An attempt was made to stop the '%s' daemon, but querying its status revealed that it did not stop. Try running '%s stop; ps -ef | grep dtlogin' to diagnose the issue", pszDaemonPath, pszDaemonPath);
+            }
+            else
+            {
+                LW_RAISE_EX(exc, ERROR_INVALID_STATE, "Unable to stop daemon", "An attempt was made to stop the '%s' daemon, but querying its status revealed that it did not stop. Try running '%s stop; %s status' to diagnose the issue", pszDaemonPath, pszDaemonPath, pszDaemonPath);
+            }
         }
         goto cleanup;
     }
+#endif
 
 cleanup:
 
@@ -359,24 +388,18 @@ DJDoUpdateRcD(
     }
 
     CTCaptureOutputToExc(command, exc);
-    if(exc != NULL && *exc != NULL &&
-            (*exc)->code == CENTERROR_COMMAND_FAILED)
-    {
-        // Put in a more specific error code
-        (*exc)->code = CENTERROR_DOMAINJOIN_UPDATERCD_FAILED;
-    }
 
 cleanup:
     CT_SAFE_FREE_STRING(command);
 }
 
-CENTERROR
+DWORD
 DJDoChkConfig(
     PCSTR pszDaemonName,
     BOOLEAN bStatus
     )
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     CHAR szDaemonPath[PATH_MAX+1];
     PSTR* ppszArgs = NULL;
     DWORD nArgs = 4;
@@ -421,7 +444,7 @@ DJDoChkConfig(
     BAIL_ON_CENTERIS_ERROR(ceError);
 
     if (status != 0) {
-        ceError = CENTERROR_DOMAINJOIN_CHKCONFIG_FAILED;
+        ceError = ERROR_BAD_COMMAND;
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -436,7 +459,7 @@ DJDoChkConfig(
 
     fp = fopen(szDaemonPath, "r");
     if (fp == NULL) {
-        ceError = CTMapSystemError(errno);
+        ceError = LwMapErrnoToLwError(errno);
         BAIL_ON_CENTERIS_ERROR(ceError);
     }
 
@@ -446,7 +469,7 @@ DJDoChkConfig(
             if (feof(fp))
                 break;
             else {
-                ceError = CTMapSystemError(errno);
+                ceError = LwMapErrnoToLwError(errno);
                 BAIL_ON_CENTERIS_ERROR(ceError);
             }
 
@@ -469,7 +492,7 @@ DJDoChkConfig(
                 BAIL_ON_CENTERIS_ERROR(ceError);
 
                 if (status != 0) {
-                    ceError = CENTERROR_DOMAINJOIN_CHKCONFIG_FAILED;
+                    ceError = ERROR_BAD_COMMAND;
                     BAIL_ON_CENTERIS_ERROR(ceError);
                 }
 
@@ -567,6 +590,9 @@ DJConfigureForDaemonRestart(
     DistroInfo distro;
     PSTR symlinkTarget = NULL;
     PSTR symlinkName = NULL;
+    long status = 0;
+    PSTR statusBuffer = NULL;
+    PSTR outputBuffer = NULL;
 
     memset(&distro, 0, sizeof(distro));
 
@@ -574,13 +600,13 @@ DJConfigureForDaemonRestart(
     LW_CLEANUP_CTERR(exc, CTCheckFileExists(pszChkConfigPath, &bFileExists));
 
     if (bFileExists) {
-        CENTERROR ceError;
+        DWORD ceError;
 
         DJ_LOG_VERBOSE("Found '%s'", pszChkConfigPath);
         ceError = DJDoChkConfig(pszDaemonName, bStatus);
-        if(ceError == CENTERROR_DOMAINJOIN_CHKCONFIG_FAILED)
+        if(ceError == ERROR_BAD_COMMAND)
         {
-            LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_CHKCONFIG_FAILED, "chkconfig failed", "An error occurred while using chkconfig to process the '%s' daemon. This daemon was being %s the list of processes to start on reboot.", pszDaemonName, bStatus? "added to" : "removed from");
+            LW_RAISE_EX(exc, ERROR_BAD_COMMAND, "chkconfig failed", "An error occurred while using chkconfig to process the '%s' daemon. This daemon was being %s the list of processes to start on reboot.", pszDaemonName, bStatus? "added to" : "removed from");
             goto cleanup;
         }
         LW_CLEANUP_CTERR(exc, ceError);
@@ -673,13 +699,13 @@ DJConfigureForDaemonRestart(
             LW_TRY(exc, FindDaemonScript(pszDaemonName,
                         &symlinkTarget, &LW_EXC));
             LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
-                    "/sbin/rc1.d/K%02d%s", stopPriority, relativeName));
+                    "/sbin/rc1.d/K%03d%s", stopPriority, relativeName));
             LW_TRY(exc,
                     DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
             CT_SAFE_FREE_STRING(symlinkName);
 
             LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
-                    "/sbin/rc2.d/S%02d%s", startPriority, relativeName));
+                    "/sbin/rc2.d/S%03d%s", startPriority, relativeName));
             LW_TRY(exc,
                     DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
         }
@@ -697,11 +723,75 @@ DJConfigureForDaemonRestart(
                         directories, pszDaemonName, &LW_EXC));
         }
     }
+    else if(distro.os == OS_SUNOS)
+    {
+        BOOLEAN bUsesSVCS = FALSE;
+
+        {
+            /* On older Solaris system, in order for a daemon to start in multi-user mode,
+             * a start symlink must be put in /etc/rc2.d, and a stop symlink
+             * must be put in /etc/rc0.d, /etc/rc1.d, /etc/rcS.d.
+             */
+            if(bStatus)
+            {
+                PCSTR relativeName = strrchr(pszDaemonName, '/');
+
+                if (relativeName == NULL)
+                {
+                    relativeName = pszDaemonName;
+                }
+                else
+                {
+                    relativeName++;
+                }
+
+                LW_TRY(exc, FindDaemonScript(pszDaemonName,
+                            &symlinkTarget, &LW_EXC));
+                LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
+                        "/etc/rc0.d/K%02d%s", stopPriority, relativeName));
+                LW_TRY(exc,
+                        DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
+                CT_SAFE_FREE_STRING(symlinkName);
+
+                LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
+                        "/etc/rc1.d/K%02d%s", stopPriority, relativeName));
+                LW_TRY(exc,
+                        DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
+                CT_SAFE_FREE_STRING(symlinkName);
+
+                LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
+                        "/etc/rcS.d/K%02d%s", stopPriority, relativeName));
+                LW_TRY(exc,
+                        DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
+                CT_SAFE_FREE_STRING(symlinkName);
+
+                LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&symlinkName,
+                        "/etc/rc2.d/S%02d%s", startPriority, relativeName));
+                LW_TRY(exc,
+                        DJOverwriteSymlink(symlinkTarget, symlinkName, &LW_EXC));
+            }
+            else
+            {
+                /* Remove any symlinks to the daemon if they exist */
+                PCSTR directories[] = {
+                    "/etc/rc0.d",
+                    "/etc/rc1.d",
+                    "/etc/rc2.d",
+                    "/etc/rc3.d",
+                    "/etc/rcS.d",
+                    NULL };
+                LW_TRY(exc, DJRemoveDaemonLinksInDirectories(
+                            directories, pszDaemonName, &LW_EXC));
+            }
+        }
+    }
 
 done:
 cleanup:
     CT_SAFE_FREE_STRING(symlinkTarget);
     CT_SAFE_FREE_STRING(symlinkName);
+    CT_SAFE_FREE_STRING(statusBuffer);
+    CT_SAFE_FREE_STRING(outputBuffer);
 }
 
 void
@@ -714,35 +804,25 @@ DJManageDaemonDescription(
     LWException **exc
     )
 {
-    BOOLEAN bStarted = FALSE;
     PSTR daemonPath = NULL;
 
     *description = NULL;
 
-    LW_TRY(exc, DJGetDaemonStatus(pszName, &bStarted, &LW_EXC));
-
-    // if we got this far, we have validated the existence of the
-    // daemon and we have figured out if its started or stopped
-
-    // if we are already in the desired state, do nothing.
-    if (bStarted != bStatus) {
-
-        CT_SAFE_FREE_STRING(daemonPath);
-        LW_TRY(exc, FindDaemonScript(pszName, &daemonPath, &LW_EXC));
-        if(bStatus)
-        {
-            LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(description,
-                    "Start %s by running '%s start'.\n"
-                    "Create symlinks for %s so that it starts at reboot.\n",
-                    pszName, daemonPath, pszName));
-        }
-        else
-        {
-            LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(description,
-                    "Stop %s by running '%s stop'.\n"
-                    "Remove symlinks for %s so that it no longer starts at reboot.\n",
-                    pszName, daemonPath, pszName));
-        }
+    CT_SAFE_FREE_STRING(daemonPath);
+    LW_TRY(exc, FindDaemonScript(pszName, &daemonPath, &LW_EXC));
+    if(bStatus)
+    {
+        LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(description,
+                "Start %s by running '%s start'.\n"
+                "Create symlinks for %s so that it starts at reboot.\n",
+                pszName, daemonPath, pszName));
+    }
+    else
+    {
+        LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(description,
+                "Stop %s by running '%s stop'.\n"
+                "Remove symlinks for %s so that it no longer starts at reboot.\n",
+                pszName, daemonPath, pszName));
     }
 
 cleanup:
@@ -760,31 +840,30 @@ DJManageDaemon(
 {
     BOOLEAN bStarted = FALSE;
 
-    // check our current state prior to doing anything.  notice that
-    // we are using the private version so that if we fail, our inner
-    // exception will be the one that was tossed due to the failure.
+#if !defined(__LWI_MACOSX__)
+    // Check for the existence of the daemon prior to doing anything.
+    // notice that we are using the private version so that if we fail,
+    // our inner exception will be the one that was tossed due to the failure.
     LW_TRY(exc, DJGetDaemonStatus(pszName, &bStarted, &LW_EXC));
 
-    // if we got this far, we have validated the existence of the
-    // daemon and we have figured out if its started or stopped
-
-    // if we are already in the desired state, do nothing.
-    if (bStarted != bStatus) {
-
-        LW_TRY(exc, DJStartStopDaemon(pszName, bStatus, &LW_EXC));
-
-    }
-
+    // Verify that daemon is configured as needed for running after restart,
+    // this also sets up any service control manager settings. On newer
+    // Solaris systems, the service control manager will start/stop the
+    // daemon as a result of the service configuration. It is okay to have
+    // this function called more than once for a given daemon, if it is already
+    // running or stopped it will remain the same with the same service pid.
     LW_TRY(exc, DJConfigureForDaemonRestart(pszName, bStatus, startPriority, stopPriority, &LW_EXC));
 
+    // Reverify the daemon status now, it may be started or stopped as needed by the step above.
+    LW_TRY(exc, DJGetDaemonStatus(pszName, &bStarted, &LW_EXC));
+
+    // If we are already in the desired state, do nothing.
+    if (bStarted != bStatus) {
+        LW_TRY(exc, DJStartStopDaemon(pszName, bStatus, &LW_EXC));
+    }
+#endif
+
 cleanup:
     ;
 }
 
-struct _DaemonList daemonList[] = {    
-    { "lsassd", {"likewise-open", NULL}, TRUE, 93, 10 },
-    { "gpagentd", {NULL}, FALSE, 94, 9 },
-    { "lwmgmtd", {NULL}, FALSE, 95, 8 },
-    { NULL, {NULL}, FALSE, 0, 0 },
-};
-
