@@ -40,12 +40,12 @@
 #include <unistd.h>
 #endif
 
-#define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
+#define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
-CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
+DWORD DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
 {
     BOOLEAN exists;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     struct utsname unameStruct;
     char *path = NULL;
     PSTR fileContents = NULL;
@@ -61,7 +61,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
     //indicates success. In fact, Solaris 8 returns 1, while Linux returns
     //0.
     if(uname(&unameStruct) < 0)
-        return CTMapSystemError(errno);
+        return LwMapErrnoToLwError(errno);
 
     //Check for os override file
     if(testPrefix == NULL)
@@ -73,7 +73,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
     if(exists)
     {
         GCE(ceError = CTReadFile(
-                path, &fileContents, NULL));
+                path, SIZE_MAX, &fileContents, NULL));
     }
     if(fileContents != NULL)
     {
@@ -93,7 +93,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
     if(exists)
     {
         GCE(ceError = CTReadFile(
-                path, &distroString, NULL));
+                path, SIZE_MAX, &distroString, NULL));
     }
     CT_SAFE_FREE_STRING(path);
 
@@ -112,8 +112,22 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
             const char *searchFile;
             const char *matchRegex;
             int versionMatchNum;
+            int updateMatchNum;
             BOOLEAN compareCase;
         } const distroSearch[] = {
+            {
+                DISTRO_ESX,
+                "/etc/vmware-release",
+                /*
+                # The format of the line is something like:
+                #   VMware ESX 4.0 (Kandinsky)
+                */
+                "^[[:space:]]*VMware ESX ([[:digit:]]+(\\.[[:digit:]]+)?)"
+                "( \\(\\S+\\))?",
+                1,
+                -1,
+                1
+            },
             {
                 DISTRO_RHEL,
                 "/etc/redhat-release",
@@ -123,13 +137,16 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 #   Red Hat Linux Advanced Server release 2.1AS (Pensacola)
                 #   Red Hat Enterprise Linux Client release 5 (Tikanga)
                 #   Red Hat Enterprise Linux Server release 5.3 (Tikanga)
+                #   Red Hat Enterprise Linux ES release 4 (Nahant Update 8)
+                #   Red Hat Enterprise Linux Workstation release 6.0 (Santiago)
                 # In addition, Oracle Linux reports itself as:
                 #   Enterprise Linux Enterprise Linux AS release 4 (October Update 5)
                 */
                 //Find a matching distro name
-                "^[[:space:]]*((Red Hat)|(Enterprise Linux)) ((Enterprise Linux)|(Linux (Advanced|Enterprise) Server))[[:space:]]+(AS |ES |Client |Server )?"
-                "release ([[:digit:]]+(\\.[[:digit:]]+)?(AS|ES)?)",
+                "^[[:space:]]*((Red Hat)|(Enterprise Linux)) ((Enterprise Linux)|(Linux (Advanced|Enterprise) Server))[[:space:]]+(AS |ES |Client |Server |Workstation )?"
+                "release ([[:digit:]]+(\\.[[:digit:]]+)?(AS|ES)?) (\\(\\S+ Update ([[:digit:]]+)\\))?",
                 9,
+                13,
                 1
             },
             {
@@ -141,6 +158,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 */
                 "^[[:space:]]*Red Hat Linux release ([[:digit:]]+(\\.[[:digit:]]+)?)",
                 1,
+                -1,
                 1
             },
             {
@@ -152,6 +170,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 */
                 "^[[:space:]]*Fedora (Core )?release (\\S+)",
                 2,
+                -1,
                 1
             },
             {
@@ -163,6 +182,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 */
                 "^[[:space:]]*CentOS release ([[:digit:]]+(\\.[[:digit:]]+)?)",
                 1,
+                -1,
                 1
             },
             {
@@ -170,6 +190,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 "/etc/SuSE-release",
                 "^[[:space:]]*SUSE LINUX ([[:digit:]]+\\.[[:digit:]]+)[[:space:]]+",
                 1,
+                -1,
                 0
             },
             {
@@ -177,6 +198,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 "/etc/SuSE-release",
                 "^[[:space:]]*openSUSE ([[:digit:]]+\\.[[:digit:]]+)[[:space:]]+",
                 1,
+                -1,
                 0
             },
             {
@@ -184,6 +206,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 "/etc/SuSE-release",
                 "^[[:space:]]*SUSE LINUX Enterprise Server ([[:digit:]]+( SP[[:digit:]]+)?)",
                 1,
+                -1,
                 0
             },
             {
@@ -191,6 +214,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 "/etc/SuSE-release",
                 "^[[:space:]]*SUSE LINUX Enterprise Desktop ([[:digit:]]+( SP[[:digit:]]+)?)",
                 1,
+                -1,
                 0
             },
             {
@@ -204,6 +228,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 "^[[:space:]]*DISTRIB_ID[[:space:]]*=[[:space:]]*Ubuntu[[:space:]]*\n"
                 "(.*\n)?DISTRIB_RELEASE[[:space:]]*=[[:space:]]*(\\S+)[[:space:]]*(\n.*)?$",
                 2,
+                -1,
                 1
             },
             {
@@ -216,11 +241,12 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 */
                 "^[[:space:]]*(\\S+)[[:space:]]*$",
                 1,
+                -1,
                 1
             },
         };
         int i;
-        regmatch_t matches[10];
+        regmatch_t matches[20];
         info->distro = DISTRO_UNKNOWN;
         for(i = 0; info->distro == DISTRO_UNKNOWN; i++)
         {
@@ -238,10 +264,10 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 if(!distroSearch[i].compareCase)
                     flags |= REG_ICASE;
 
-                GCE(ceError = CTReadFile(path, &fileContents, NULL));
+                GCE(ceError = CTReadFile(path, SIZE_MAX, &fileContents, NULL));
                 if(regcomp(&rx, distroSearch[i].matchRegex, flags) != 0)
                 {
-                    GCE(ceError = CENTERROR_REGEX_COMPILE_FAILED);
+                    GCE(ceError = LW_ERROR_REGEX_COMPILE_FAILED);
                 }
                 rxAlloced = TRUE;
                 if(regexec(&rx, fileContents,
@@ -250,10 +276,25 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
                 {
                     //This is the correct distro
                     regmatch_t *ver = &matches[distroSearch[i].versionMatchNum];
+                    regmatch_t *update = &matches[distroSearch[i].updateMatchNum];
                     info->distro = distroSearch[i].matchDistro;
-                    GCE(ceError = CTStrndup(fileContents + ver->rm_so,
+                    if (distroSearch[i].updateMatchNum == -1 ||
+                            update->rm_so == -1)
+                    {
+                        GCE(ceError = CTStrndup(fileContents + ver->rm_so,
+                                    ver->rm_eo - ver->rm_so,
+                                    &info->version));
+                    }
+                    else
+                    {
+                        GCE(ceError = CTAllocateStringPrintf(
+                                &info->version,
+                                "%.*s.%.*s",
                                 ver->rm_eo - ver->rm_so,
-                                &info->version));
+                                fileContents + ver->rm_so,
+                                update->rm_eo - update->rm_so,
+                                fileContents + update->rm_so));
+                    }
                 }
                 regfree(&rx);
                 rxAlloced = FALSE;
@@ -343,7 +384,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
     if(exists)
     {
         GCE(ceError = CTReadFile(
-                path, &fileContents, NULL));
+                path, SIZE_MAX, &fileContents, NULL));
     }
     if(fileContents != NULL)
     {
@@ -449,7 +490,7 @@ CENTERROR DJGetDistroInfo(const char *testPrefix, DistroInfo *info)
     if(exists)
     {
         GCE(ceError = CTReadFile(
-                path, &fileContents, NULL));
+                path, SIZE_MAX, &fileContents, NULL));
         info->arch = DJGetArchFromString(fileContents);
         CT_SAFE_FREE_STRING(fileContents);
     }
@@ -461,7 +502,7 @@ cleanup:
     CT_SAFE_FREE_STRING(distroString);
     if(rxAlloced)
         regfree(&rx);
-    if(!CENTERROR_IS_OK(ceError))
+    if(ceError)
     {
         DJFreeDistroInfo(info);
     }
@@ -495,7 +536,7 @@ OSType DJGetOSFromString(const char *str)
     return OS_UNKNOWN;
 }
 
-CENTERROR DJGetOSString(OSType type, char **result)
+DWORD DJGetOSString(OSType type, char **result)
 {
     int i;
     for(i = 0; i < sizeof(osList)/sizeof(osList[0]); i++)
@@ -528,6 +569,7 @@ struct
     { DISTRO_UBUNTU, "Ubuntu" },
     { DISTRO_DEBIAN, "Debian" },
     { DISTRO_FREEBSD, "FreeBSD" },
+    { DISTRO_ESX, "VMware ESX" },
 };
 
 DistroType DJGetDistroFromString(const char *str)
@@ -541,7 +583,7 @@ DistroType DJGetDistroFromString(const char *str)
     return DISTRO_UNKNOWN;
 }
 
-CENTERROR DJGetDistroString(DistroType type, char **result)
+DWORD DJGetDistroString(DistroType type, char **result)
 {
     int i;
     for(i = 0; i < sizeof(distroList)/sizeof(distroList[0]); i++)
@@ -583,7 +625,7 @@ ArchType DJGetArchFromString(const char * str)
     return ARCH_UNKNOWN;
 }
 
-CENTERROR DJGetArchString(ArchType type, char **result)
+DWORD DJGetArchString(ArchType type, char **result)
 {
     int i;
     for(i = 0; i < sizeof(archList)/sizeof(archList[0]); i++)
@@ -600,13 +642,21 @@ void DJFreeDistroInfo(DistroInfo *info)
         CT_SAFE_FREE_STRING(info->version);
 }
 
-CENTERROR DJGetLikewiseVersion(PSTR *version, PSTR *build, PSTR *revision)
+DWORD
+DJGetLikewiseVersion(
+    PSTR *product,
+    PSTR *version,
+    PSTR *build,
+    PSTR *revision
+    )
 {
     FILE *versionFile = NULL;
     PSTR line = NULL; 
     BOOLEAN isEndOfFile = FALSE;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
+    BOOLEAN bIsEnterprise = FALSE;
 
+    PSTR _product = NULL;
     PSTR _version = NULL;
     PSTR _build = NULL;
     PSTR _revision = NULL;
@@ -618,7 +668,15 @@ CENTERROR DJGetLikewiseVersion(PSTR *version, PSTR *build, PSTR *revision)
 #ifdef MINIMAL_JOIN
     GCE(ceError = CTOpenFile(LOCALSTATEDIR "/VERSION", "r", &versionFile));
 #else
-    GCE(ceError = CTOpenFile(PREFIXDIR "/data/VERSION", "r", &versionFile));
+    ceError = CTOpenFile(PREFIXDIR "/data/ENTERPRISE_VERSION", "r", &versionFile);
+    if (ceError == 0)
+    {
+        bIsEnterprise = TRUE;
+    }
+    else
+    {
+        GCE(ceError = CTOpenFile(PREFIXDIR "/data/VERSION", "r", &versionFile));
+    }
 #endif
 
     while (TRUE)
@@ -644,6 +702,14 @@ CENTERROR DJGetLikewiseVersion(PSTR *version, PSTR *build, PSTR *revision)
         }
     }
 
+    if (bIsEnterprise)
+    {
+        GCE(ceError = CTStrdup("Enterprise", &_product));
+    }
+    else
+    {
+        GCE(ceError = CTStrdup("Open", &_product));
+    }
     if (_version == NULL)
     {
         GCE(ceError = CTStrdup("unknown", &_version));
@@ -659,9 +725,11 @@ CENTERROR DJGetLikewiseVersion(PSTR *version, PSTR *build, PSTR *revision)
 
     GCE(ceError = CTSafeCloseFile(&versionFile));
 
+    *product = _product;
     *version = _version;
     *build = _build;
     *revision = _revision;
+    _product = NULL;
     _version = NULL;
     _build = NULL;
     _revision = NULL;
