@@ -44,7 +44,7 @@
 #include "djconfig_mac.h"
 #include "djddns.h"
 
-#define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
+#define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
 //Make sure all modules are included in both lists, even if the module
 //doesn't apply to start or stop
@@ -116,6 +116,7 @@ void DJFreeJoinProcessOptions(JoinProcessOptions *options)
     CT_SAFE_FREE_STRING(options->shortDomainName);
     CT_SAFE_FREE_STRING(options->computerName);
     CT_SAFE_FREE_STRING(options->ouName);
+    CT_SAFE_FREE_STRING(options->userDomainPrefix);
     CT_SAFE_FREE_STRING(options->username);
     CT_SAFE_FREE_STRING(options->password);
 
@@ -175,7 +176,7 @@ void NormalizeUsername(PSTR *username, PCSTR domainName, LWException **exc)
 
     if(IsNullOrEmptyString(domainName))
     {
-        LW_RAISE_EX(exc, CENTERROR_INVALID_PARAMETER, "Unable to determine user domain", "The domain that '%s' belongs to could not be automatically determined. Please pass the user name in user@domain.com syntax.", *username);
+        LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Unable to determine user domain", "The domain that '%s' belongs to could not be automatically determined. Please pass the user name in user@domain.com syntax.", *username);
         goto cleanup;
     }
 
@@ -239,14 +240,14 @@ void DJInitModuleStates(JoinProcessOptions *options, LWException **exc)
             case CannotConfigure:
                 arrayState->runModule = FALSE;
                 break;
-            case ApplePluginInUse:
-                LW_RAISE_EX(exc, CENTERROR_INVALID_OPERATION, "Apple AD Directory Plugin in use.", "The configuration of module '%s' detected that the computer is already joined to Active Directory with the built in Apple AD plugin. To use Likewise, please first unbind your Mac from Active Directory by using the Directory Utility of your system.\n", state.module->longName);
-                goto cleanup;
             case SufficientlyConfigured:
             case NotConfigured:
                 break;
+            case ApplePluginInUse:
+                LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Apple AD Directory Plugin in use.", "The configuration of module '%s' detected that the computer is already joined to Active Directory with the built in Apple AD plugin. To use Likewise, please first unbind your Mac from Active Directory by using the Directory Utility of your system.\n", state.module->shortName);
+                goto cleanup;
             default:
-                LW_RAISE_EX(exc, CENTERROR_INVALID_OPERATION, "Invalid module state", "The configuration of module '%s' returned an invalid configuration state.\n", state.module->longName);
+                LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Invalid module state", "The configuration of module '%s' returned an invalid configuration state.\n", state.module->shortName);
                 goto cleanup;
         }
     }
@@ -269,13 +270,13 @@ void DJCheckRequiredEnabled(const JoinProcessOptions *options, LWException **exc
         {
             case CannotConfigure:
                 LW_TRY(exc, exceptionMessage = state->module->GetChangeDescription(options, &LW_EXC));
-                LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_MODULE_NOT_ENABLED, "Manual configuration required", "The configuration stage '%s' cannot be completed automatically. Please manually perform the following steps and rerun the domain join:\n\n%s", state->module->longName, exceptionMessage);
+                LW_RAISE_EX(exc, LW_ERROR_MODULE_NOT_ENABLED, "Manual configuration required", "The configuration stage '%s' cannot be completed automatically. Please manually perform the following steps and rerun the domain join:\n\n%s", state->module->longName, exceptionMessage);
                 goto cleanup;
             case NotConfigured:
                 if(!state->runModule)
                 {
                     LW_TRY(exc, exceptionMessage = state->module->GetChangeDescription(options, &LW_EXC));
-                    LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_MODULE_NOT_ENABLED, "Required configuration stage not enabled", "The configuration of module '%s' is required. Please either allow this configuration stage to be performed automatically (by passing '--enable %s'), or manually perform these configuration steps and rerun the domain join:\n\n%s", state->module->longName, state->module->shortName, exceptionMessage);
+                    LW_RAISE_EX(exc, LW_ERROR_MODULE_NOT_ENABLED, "Required configuration stage not enabled", "The configuration of module '%s' is required. Please either allow this configuration stage to be performed automatically (by passing '--enable %s'), or manually perform these configuration steps and rerun the domain join:\n\n%s", state->module->longName, state->module->shortName, exceptionMessage);
                     goto cleanup;
                 }
                 break;
@@ -286,10 +287,16 @@ void DJCheckRequiredEnabled(const JoinProcessOptions *options, LWException **exc
                 if(state->runModule)
                 {
                     //The UI shouldn't even let this happen.
-                    LW_RAISE_EX(exc, CENTERROR_DOMAINJOIN_MODULE_ALREADY_DONE, "Invalid module enabled", "Running module '%s' is not valid at this time because it is already configured. Please disable it and try again.", state->module->longName);
+                    LW_RAISE_EX(exc, LW_ERROR_MODULE_ALREADY_DONE, "Invalid module enabled", "Running module '%s' is not valid at this time because it is already configured. Please disable it and try again.", state->module->longName);
                     goto cleanup;
                 }
                 break;
+            case ApplePluginInUse:
+                LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Apple AD Directory Plugin in use.", "The configuration of module '%s' detected that the computer is already joined to Active Directory with the built in Apple AD plugin. To use Likewise, please first unbind your Mac from Active Directory by using the Directory Utility of your system.\n", state->module->shortName);
+                goto cleanup;
+            default:
+                LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Invalid module state", "The configuration of module '%s' returned an invalid configuration state.\n", state->module->shortName);
+                goto cleanup;
         }
     }
 
@@ -331,12 +338,21 @@ void DJRunJoinProcess(JoinProcessOptions *options, LWException **exc)
                             &exceptionTitle,
                             "A resumable error occurred while processing the '%s' module",
                             state->module->shortName));
-                    options->warningCallback(options, exceptionTitle, exceptionMessage);
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options, exceptionTitle, exceptionMessage);
+                    }
                     DJLogException(LOG_LEVEL_WARNING, moduleException);
                     LWHandle(&moduleException);
                     CT_SAFE_FREE_STRING(exceptionMessage);
                     CT_SAFE_FREE_STRING(exceptionTitle);
                     break;
+                case ApplePluginInUse:
+                    LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Apple AD Directory Plugin in use.", "The configuration of module '%s' detected that the computer is already joined to Active Directory with the built in Apple AD plugin. To use Likewise, please first unbind your Mac from Active Directory by using the Directory Utility of your system.\n", state->module->shortName);
+                    goto cleanup;
+                default:
+                    LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Invalid module state", "The configuration of module '%s' returned an invalid configuration state.\n", state->module->shortName);
+                    goto cleanup;
             }
         }
         else
@@ -350,7 +366,7 @@ void DJRunJoinProcess(JoinProcessOptions *options, LWException **exc)
                 case NotConfigured:
                 case CannotConfigure:
                     LW_RAISE_EX(exc,
-                            CENTERROR_DOMAINJOIN_MODULE_NOT_CONFIGURED, "Module not configured",
+                            ERROR_CAN_NOT_COMPLETE, "Module not configured",
                             "Even though the configuration of '%s' was executed, the configuration is not complete. Please contact Likewise support.",
                             state->module->shortName);
                     goto cleanup;
@@ -359,9 +375,20 @@ void DJRunJoinProcess(JoinProcessOptions *options, LWException **exc)
                             CTAllocateStringPrintf(&exceptionMessage,
                             "Even though the configuration of '%s' was executed, the configuration did not fully complete. Please contact Likewise support.",
                             state->module->shortName));
-                    options->warningCallback(options, "A resumable error occurred while processing a module", exceptionMessage);
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options,
+                                                 "A resumable error occurred while processing a module",
+                                                 exceptionMessage);
+                    }
                     CT_SAFE_FREE_STRING(exceptionMessage);
                     break;
+                case ApplePluginInUse:
+                    LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Apple AD Directory Plugin in use.", "The configuration of module '%s' detected that the computer is already joined to Active Directory with the built in Apple AD plugin. To use Likewise, please first unbind your Mac from Active Directory by using the Directory Utility of your system.\n", state->module->shortName);
+                    goto cleanup;
+                default:
+                    LW_RAISE_EX(exc, ERROR_INVALID_OPERATION, "Invalid module state", "The configuration of module '%s' returned an invalid configuration state.\n", state->module->shortName);
+                    goto cleanup;
             }
         }
     }
@@ -377,7 +404,7 @@ void DJEnableModule(JoinProcessOptions *options, PCSTR shortName, BOOLEAN enable
     ModuleState *state = DJGetModuleStateByName(options, shortName);
     if(!state)
     {
-        LW_RAISE_EX(exc, CENTERROR_INVALID_PARAMETER, "Unable to enable/disable module", "Unable to enable/disable module '%s'. This module could not be found. Please check the name and try again. Keep in mind that some domainjoin modules may not be applicable on all platforms.", shortName);
+        LW_RAISE_EX(exc, ERROR_INVALID_PARAMETER, "Unable to enable/disable module", "Unable to enable/disable module '%s'. This module could not be found. Please check the name and try again. Keep in mind that some domainjoin modules may not be applicable on all platforms.", shortName);
         return;
     }
     state->runModule = enable;
