@@ -38,7 +38,7 @@
 #define NSSWITCH_CONF_PATH "/etc/nsswitch.conf"
 #define NSSWITCH_LWIDEFAULTS "/etc/nsswitch.lwi_defaults"
 
-#define GCE(x) GOTO_CLEANUP_ON_CENTERROR((x))
+#define GCE(x) GOTO_CLEANUP_ON_DWORD((x))
 
 typedef struct
 {
@@ -93,9 +93,9 @@ static void FreeNsswitchConfContents(NsswitchConf *conf)
 }
 
 /* Get the printed form of a line from the parsed form by concatenating all of the strings together */
-static CENTERROR GetPrintedLine(DynamicArray *dest, NsswitchConf *conf, int line)
+static DWORD GetPrintedLine(DynamicArray *dest, NsswitchConf *conf, int line)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     size_t len = 0;
     char *pos;
     int i;
@@ -132,9 +132,9 @@ cleanup:
     return ceError;
 }
 
-static CENTERROR AddFormattedLine(NsswitchConf *conf, const char *filename, const char *linestr, const char **endptr)
+static DWORD AddFormattedLine(NsswitchConf *conf, const char *filename, const char *linestr, const char **endptr)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchEntry lineObj;
     const char *pos = linestr;
     const char *token_start = NULL;
@@ -154,7 +154,7 @@ static CENTERROR AddFormattedLine(NsswitchConf *conf, const char *filename, cons
     /* Create an array of the modules for this entry */
     while(strchr("\r\n;#", *pos) == NULL)
     {
-        GCE(ceError = CTReadToken(&pos, &token, "=: \t", ";#\r\n", ""));
+        GCE(ceError = CTReadToken(&pos, &token, ", \t", ";#\r\n", ""));
         GCE(ceError = CTArrayAppend(&lineObj.modules, sizeof(CTParseToken), &token, 1));
         memset(&token, 0, sizeof(token));
     }
@@ -179,9 +179,9 @@ cleanup:
     return ceError;
 }
 
-static CENTERROR ReadNsswitchFile(NsswitchConf *conf, const char *rootPrefix, const char *filename)
+static DWORD ReadNsswitchFile(NsswitchConf *conf, const char *rootPrefix, const char *filename)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     FILE *file = NULL;
     PSTR buffer = NULL;
     char *fullPath = NULL;
@@ -198,7 +198,7 @@ static CENTERROR ReadNsswitchFile(NsswitchConf *conf, const char *rootPrefix, co
     if(!exists)
     {
         DJ_LOG_INFO("File %s does not exist", fullPath);
-        ceError = CENTERROR_INVALID_FILENAME;
+        ceError = ERROR_FILE_NOT_FOUND;
         goto cleanup;
     }
 
@@ -222,26 +222,33 @@ cleanup:
     if(file != NULL)
         CTCloseFile(file);
     CT_SAFE_FREE_STRING(fullPath);
-    if(!CENTERROR_IS_OK(ceError))
+    if(ceError)
         FreeNsswitchConfContents(conf);
     return ceError;
 }
 
-static CENTERROR WriteNsswitchConfiguration(const char *rootPrefix, NsswitchConf *conf)
+static DWORD WriteNsswitchConfiguration(const char *rootPrefix, NsswitchConf *conf)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     DynamicArray printedLine;
     int i;
     char *tempName = NULL;
     char *finalName = NULL;
+    char *prefixedPath = NULL;
     FILE *file = NULL;
     memset(&printedLine, 0, sizeof(printedLine));
 
-    DJ_LOG_INFO("Writing nsswitch configuration for %s", conf->filename);
+    GCE(ceError = CTAllocateStringPrintf(&prefixedPath, "%s%s", rootPrefix, conf->filename));
+
+    GCE(ceError = CTGetFileTempPath(
+                        prefixedPath,
+                        &finalName,
+                        &tempName));
+
+    DJ_LOG_INFO("Writing nsswitch configuration for %s", finalName);
 
-    GCE(ceError = CTAllocateStringPrintf(&tempName, "%s%s.new", rootPrefix, conf->filename));
     ceError = CTOpenFile(tempName, "w", &file);
-    if(!CENTERROR_IS_OK(ceError))
+    if(ceError)
     {
         DJ_LOG_ERROR("Unable to open '%s' for writing", tempName);
         GCE(ceError);
@@ -256,13 +263,9 @@ static CENTERROR WriteNsswitchConfiguration(const char *rootPrefix, NsswitchConf
     GCE(ceError = CTCloseFile(file));
     file = NULL;
 
-    /* Make sure to set perms */
-
-    GCE(ceError = CTChangePermissions(tempName, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH));
-
-    GCE(ceError = CTAllocateStringPrintf(&finalName, "%s%s", rootPrefix, conf->filename));
-    GCE(ceError = CTBackupFile(finalName));
-    GCE(ceError = CTMoveFile(tempName, finalName));
+    GCE(ceError = CTSafeReplaceFile(
+            finalName,
+            tempName));
     DJ_LOG_INFO("File moved into place");
 
 cleanup:
@@ -271,6 +274,7 @@ cleanup:
     CTArrayFree(&printedLine);
     CT_SAFE_FREE_STRING(tempName);
     CT_SAFE_FREE_STRING(finalName);
+    CT_SAFE_FREE_STRING(prefixedPath);
     return ceError;
 }
 
@@ -290,10 +294,10 @@ static int FindEntry(const NsswitchConf *conf, int startLine, const char *name)
     return -1;
 }
 
-static CENTERROR AddEntry(NsswitchConf *conf, const DistroInfo *distro,
+static DWORD AddEntry(NsswitchConf *conf, const DistroInfo *distro,
         int *addedIndex, const char *name)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     int line = -1;
     NsswitchEntry lineObj;
     const NsswitchEntry *copy;
@@ -373,10 +377,10 @@ static int FindModuleOnLine(const NsswitchConf *conf, int line, const char *name
     return -1;
 }
 
-static CENTERROR InsertModule(NsswitchConf *conf, const DistroInfo *distro,
+static DWORD InsertModule(NsswitchConf *conf, const DistroInfo *distro,
         int line, int insertIndex, const char *name)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchEntry *lineObj = (NsswitchEntry *)GetEntry(conf, line);
     CTParseToken *beforeModule = NULL, *afterModule = NULL;
     CTParseToken addModule;
@@ -439,10 +443,10 @@ cleanup:
     return ceError;
 }
 
-static CENTERROR RemoveModule(NsswitchConf *conf, 
+static DWORD RemoveModule(NsswitchConf *conf,
         int line, int moduleIndex)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchEntry *lineObj = (NsswitchEntry *)GetEntry(conf, line);
     CTParseToken *beforeModule = NULL, *afterModule = NULL;
     CTParseToken *removeModule;
@@ -471,14 +475,14 @@ cleanup:
     return ceError;
 }
 
-static CENTERROR RemoveLine(NsswitchConf *conf, int line)
+static DWORD RemoveLine(NsswitchConf *conf, int line)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchEntry *lineObj = (NsswitchEntry *)GetEntry(conf, line);
 
     if(lineObj == NULL)
     {
-        GCE(ceError = CENTERROR_INVALID_PARAMETER);
+        GCE(ceError = ERROR_INVALID_PARAMETER);
     }
 
     FreeNsswitchEntryContents(lineObj);
@@ -490,19 +494,19 @@ cleanup:
     return ceError;
 }
 
-CENTERROR
+DWORD
 UnConfigureNameServiceSwitch()
 {
     return DJConfigureNameServiceSwitch(NULL, FALSE);
 }
 
-CENTERROR
+DWORD
 ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
         BOOLEAN allowFileCreate)
 {
     PSTR copyDestPath = NULL;
     PSTR defaultFilePath = NULL;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     BOOLEAN bFileExists = FALSE;
     memset(conf, 0, sizeof(*conf));
     
@@ -511,10 +515,10 @@ ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
     {
         bFileExists = TRUE;
         ceError = ReadNsswitchFile(conf, testPrefix, "/etc/nsswitch.conf");
-        if(ceError == CENTERROR_INVALID_FILENAME)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
             bFileExists = FALSE;
-            ceError = CENTERROR_SUCCESS;
+            ceError = ERROR_SUCCESS;
         }
         GCE(ceError);
     }
@@ -523,10 +527,10 @@ ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
     {
         bFileExists = TRUE;
         ceError = ReadNsswitchFile(conf, testPrefix, "/etc/netsvc.conf");
-        if(ceError == CENTERROR_INVALID_FILENAME)
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
             bFileExists = FALSE;
-            ceError = CENTERROR_SUCCESS;
+            ceError = ERROR_SUCCESS;
         }
         GCE(ceError);
     }
@@ -566,7 +570,7 @@ ReadNsswitchConf(NsswitchConf *conf, const char *testPrefix,
 
     if(!bFileExists)
     {
-        GCE(ceError = CENTERROR_INVALID_FILENAME);
+        GCE(ceError = ERROR_FILE_NOT_FOUND);
     }
 
 cleanup:
@@ -576,16 +580,16 @@ cleanup:
     return ceError;
 }
 
-static CENTERROR UsingLsass(BOOLEAN *using)
+static DWORD UsingLsass(BOOLEAN *using)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     ceError = CTFindFileInPath("lsassd", BINDIR ":" SBINDIR, NULL);
-    if(ceError == CENTERROR_FILE_NOT_FOUND)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
         *using = FALSE;
-        ceError = CENTERROR_SUCCESS;
+        ceError = ERROR_SUCCESS;
     }
-    else if(CENTERROR_IS_OK(ceError))
+    else if(!ceError)
         *using = TRUE;
     return ceError;
 }
@@ -731,10 +735,10 @@ cleanup:
     return result;
 }
 
-CENTERROR
+DWORD
 UpdateNsswitchConf(NsswitchConf *conf, BOOLEAN enable)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     DistroInfo distro;
     int line;
     int lwiIndex;
@@ -833,19 +837,19 @@ cleanup:
     return ceError;
 }
 
-CENTERROR
+DWORD
 DJConfigureNameServiceSwitch(const char *testPrefix, BOOLEAN enable)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchConf conf;
 
     if(testPrefix == NULL)
         testPrefix = "";
 
     ceError = ReadNsswitchConf(&conf, testPrefix, TRUE);
-    if(ceError == CENTERROR_INVALID_FILENAME)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
-        ceError = CENTERROR_SUCCESS;
+        ceError = ERROR_SUCCESS;
         DJ_LOG_WARNING("Warning: Could not find nsswitch file");
         goto cleanup;
     }
@@ -898,10 +902,10 @@ const char *GetNameOfHostsByDns(const NsswitchConf *conf, const DistroInfo *dist
 
 //Does the platform-specific equivalent of this in nsswitch.conf:
 // hosts: files dns
-CENTERROR
+DWORD
 DJConfigureHostsEntry(const char *testPrefix)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     NsswitchConf conf;
     DistroInfo distro;
     int line;
@@ -955,10 +959,10 @@ cleanup:
     return ceError;
 }
 
-CENTERROR
+DWORD
 ConfigureNameServiceSwitch()
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     DistroInfo distro;
     GCE(ceError = DJGetDistroInfo(NULL, &distro));
     GCE(ceError = DJConfigureNameServiceSwitch(NULL, TRUE));
@@ -967,9 +971,6 @@ ConfigureNameServiceSwitch()
      * hosts file after checking DNS. This means that our FQDN setting code
      * won't necessarily take effect. So instead we rewrite the line to put
      * local in front.
-     *
-     * TODO: this code should be moved to where the FQDN is set so that it
-     * is not run when the --nohosts option is given.
      */
     if(distro.os == OS_AIX)
     {
@@ -983,17 +984,17 @@ cleanup:
 
 #define APPARMOR_NSSWITCH "/etc/apparmor.d/abstractions/nameservice"
 
-static CENTERROR
+static DWORD
 HasApparmor(BOOLEAN *hasApparmor)
 {
     return CTCheckFileOrLinkExists(APPARMOR_NSSWITCH,
                 hasApparmor);
 }
 
-static CENTERROR
+static DWORD
 IsApparmorConfigured(BOOLEAN *configured)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     BOOLEAN hasApparmor;
 
     *configured = FALSE;
@@ -1021,7 +1022,7 @@ cleanup:
 
 static void ConfigureApparmor(BOOLEAN enable, LWException **exc)
 {
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     BOOLEAN hasApparmor;
     BOOLEAN configured;
     BOOLEAN usingMr;
@@ -1029,6 +1030,8 @@ static void ConfigureApparmor(BOOLEAN enable, LWException **exc)
     PCSTR addString;
     PSTR restartPath = NULL;
     PSTR restartCommand = NULL;
+    char *tempName = NULL;
+    char *finalName = NULL;
 
     LW_CLEANUP_CTERR(exc, IsApparmorConfigured(&configured));
     if(configured == enable)
@@ -1039,7 +1042,12 @@ static void ConfigureApparmor(BOOLEAN enable, LWException **exc)
     if(!hasApparmor)
         goto cleanup;
 
-    LW_CLEANUP_CTERR(exc, CTCheckFileHoldsPattern(APPARMOR_NSSWITCH,
+    GCE(ceError = CTGetFileTempPath(
+                        APPARMOR_NSSWITCH,
+                        &finalName,
+                        &tempName));
+
+    LW_CLEANUP_CTERR(exc, CTCheckFileHoldsPattern(finalName,
                 "mr,", &usingMr));
 
     if(usingMr)
@@ -1047,46 +1055,46 @@ static void ConfigureApparmor(BOOLEAN enable, LWException **exc)
 PREFIXDIR "/lib/*.so*            mr,\n"
 PREFIXDIR "/lib64/*.so*          mr,\n"
 "/tmp/.lwidentity/pipe              rw,\n"
-LOCALSTATEDIR "/lib/likewise/.lsasd  rw,\n"
+LOCALSTATEDIR "/lib/likewise/.lsassd  rw,\n"
 LOCALSTATEDIR "/tmp/.lsaclient_*              rw,\n";
     else
         addString =
 PREFIXDIR "/lib/*.so*            r,\n"
 PREFIXDIR "/lib64/*.so*          r,\n"
 "/tmp/.lwidentity/pipe              rw,\n"
-LOCALSTATEDIR "/lib/likewise/.lsasd  rw,\n"
+LOCALSTATEDIR "/lib/likewise/.lsassd  rw,\n"
 LOCALSTATEDIR "/tmp/.lsaclient_*              rw,\n";
 
 
     if(enable)
     {
-        LW_CLEANUP_CTERR(exc, CTCopyFileWithOriginalPerms(APPARMOR_NSSWITCH, APPARMOR_NSSWITCH ".new"));
-        LW_CLEANUP_CTERR(exc, CTOpenFile(APPARMOR_NSSWITCH ".new", "a", &file));
+        LW_CLEANUP_CTERR(exc, CTCopyFileWithOriginalPerms(finalName, tempName));
+        LW_CLEANUP_CTERR(exc, CTOpenFile(tempName, "a", &file));
         LW_CLEANUP_CTERR(exc, CTFilePrintf(file, "# likewise\n%s# end likewise\n",
                     addString));
 
         CTSafeCloseFile(&file);
 
-        LW_CLEANUP_CTERR(exc, CTSafeReplaceFile(APPARMOR_NSSWITCH, APPARMOR_NSSWITCH ".new"));
+        LW_CLEANUP_CTERR(exc, CTSafeReplaceFile(finalName, tempName));
     }
     else
     {
-        LW_CLEANUP_CTERR(exc, CTRunSedOnFile(APPARMOR_NSSWITCH, APPARMOR_NSSWITCH, FALSE, "/^[ \t]*#[ \t]*likewise[ \t]*$/,/^[ \t]*#[ \t]*end likewise[ \t]*$/d"));
-        LW_CLEANUP_CTERR(exc, CTRunSedOnFile(APPARMOR_NSSWITCH, APPARMOR_NSSWITCH, FALSE, "/^[ \t]*#[ \t]*centeris[ \t]*$/,/^[ \t]*#[ \t]*end centeris[ \t]*$/d"));
+        LW_CLEANUP_CTERR(exc, CTRunSedOnFile(finalName, finalName, FALSE, "/^[ \t]*#[ \t]*likewise[ \t]*$/,/^[ \t]*#[ \t]*end likewise[ \t]*$/d"));
+        LW_CLEANUP_CTERR(exc, CTRunSedOnFile(finalName, finalName, FALSE, "/^[ \t]*#[ \t]*centeris[ \t]*$/,/^[ \t]*#[ \t]*end centeris[ \t]*$/d"));
     }
 
 
     ceError = CTFindFileInPath("rcapparmor", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", &restartPath);
-    if(ceError == CENTERROR_FILE_NOT_FOUND)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
         ceError = CTFindFileInPath("apparmor", "/etc/init.d/apparmor", &restartPath);
     }
     
-    if(ceError == CENTERROR_FILE_NOT_FOUND)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
-        ceError = CENTERROR_SUCCESS;
+        ceError = ERROR_SUCCESS;
     }
-    else if(CENTERROR_IS_OK(ceError))
+    else if(!ceError)
     {
         LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&restartCommand,
                     "%s restart", restartPath));
@@ -1098,16 +1106,18 @@ cleanup:
     if(file != NULL)
     {
         CTCloseFile(file);
-        CTRemoveFile(APPARMOR_NSSWITCH ".new");
+        CTRemoveFile(tempName);
     }
     CT_SAFE_FREE_STRING(restartPath);
     CT_SAFE_FREE_STRING(restartCommand);
+    CT_SAFE_FREE_STRING(tempName);
+    CT_SAFE_FREE_STRING(finalName);
 }
 
-static CENTERROR UnsuportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
+DWORD UnsupportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
 {
     BOOLEAN hasSeLinux;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
     PSTR output = NULL;
     DistroInfo distro;
 
@@ -1123,15 +1133,15 @@ static CENTERROR UnsuportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
         goto cleanup;
 
     ceError = CTRunCommand("/usr/sbin/selinuxenabled >/dev/null 2>&1");
-    if(ceError == CENTERROR_COMMAND_FAILED)
+    if(ceError == ERROR_BAD_COMMAND)
     {
         //selinux is not enabled
-        ceError = CENTERROR_SUCCESS;
+        ceError = ERROR_SUCCESS;
         goto cleanup;
     }
     GCE(ceError);
 
-    GCE(ceError = CTCaptureOutputWithStderr("/usr/sbin/getenforce", TRUE, &output));
+    GCE(ceError = CTCaptureOutput("/usr/sbin/getenforce", &output));
     CTStripWhitespace(output);
     if(!strcmp(output, "Permissive"))
     {
@@ -1153,7 +1163,7 @@ static CENTERROR UnsuportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
             }
             break;
         case DISTRO_FEDORA:
-            if(distro.version[0] < '6')
+            if(atol(distro.version) < 6)
             {
                 DJ_LOG_INFO("Safe version of Fedora");
                 goto cleanup;
@@ -1165,7 +1175,7 @@ static CENTERROR UnsuportedSeLinuxEnabled(BOOLEAN *hasBadSeLinux)
     *hasBadSeLinux = TRUE;
 
 cleanup:
-    if(!CENTERROR_IS_OK(ceError))
+    if(ceError)
         *hasBadSeLinux = TRUE;
 
     CT_SAFE_FREE_STRING(output);
@@ -1179,55 +1189,55 @@ static QueryResult QueryNsswitch(const JoinProcessOptions *options, LWException
     QueryResult result = FullyConfigured;
     BOOLEAN configured;
     BOOLEAN exists;
-    BOOLEAN hasApparmor;
     BOOLEAN hasBadSeLinux;
     NsswitchConf conf;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
 
     memset(&conf, 0, sizeof(conf));
 
-    ceError = ReadNsswitchConf(&conf, NULL, FALSE);
-    if(ceError == CENTERROR_INVALID_FILENAME)
+    if (options->joiningDomain)
     {
-        ceError = CENTERROR_SUCCESS;
-        DJ_LOG_WARNING("Warning: Could not find nsswitch file");
-        goto cleanup;
-    }
-    LW_CLEANUP_CTERR(exc, ceError);
-
-    if(options->joiningDomain)
-    {
-        LW_TRY(exc, result = RemoveCompat(&conf, NULL, &LW_EXC));
-        if(result == CannotConfigure || result == NotConfigured)
-            goto cleanup;
-    }
-
-    LW_CLEANUP_CTERR(exc, UpdateNsswitchConf(&conf, options->joiningDomain));
-    if(conf.modified)
-    {
-        LW_CLEANUP_CTERR(exc, UnsuportedSeLinuxEnabled(&hasBadSeLinux));
-        if(hasBadSeLinux)
-            result = CannotConfigure;
-        else
-            result = NotConfigured;
-        goto cleanup;
-    }
-
-    LW_CLEANUP_CTERR(exc, DJHasMethodsCfg(&exists));
-    if(exists)
-    {
-        LW_CLEANUP_CTERR(exc, DJIsMethodsCfgConfigured(&configured));
-        if((configured != FALSE) != (options->joiningDomain != FALSE))
+        ceError = ReadNsswitchConf(&conf, NULL, FALSE);
+        if(ceError == ERROR_FILE_NOT_FOUND)
         {
-            result = NotConfigured;
+            ceError = ERROR_SUCCESS;
+            DJ_LOG_WARNING("Warning: Could not find nsswitch file");
             goto cleanup;
         }
-    }
+        LW_CLEANUP_CTERR(exc, ceError);
+
+        LW_TRY(exc, result = RemoveCompat(&conf, NULL, &LW_EXC));
+        if(result == CannotConfigure || result == NotConfigured)
+        {
+            goto cleanup;
+        }
+
+        LW_CLEANUP_CTERR(exc, UpdateNsswitchConf(&conf, TRUE));
+        if(conf.modified)
+        {
+            LW_CLEANUP_CTERR(exc, UnsupportedSeLinuxEnabled(&hasBadSeLinux));
+            if(hasBadSeLinux)
+                result = CannotConfigure;
+            else
+                result = NotConfigured;
+            goto cleanup;
+        }
+
+        LW_CLEANUP_CTERR(exc, DJHasMethodsCfg(&exists));
+
+        if(exists)
+        {
+            LW_CLEANUP_CTERR(exc, DJIsMethodsCfgConfigured(&configured));
+
+            if(!configured)
+            {
+                result = NotConfigured;
+                goto cleanup;
+            }
+        }
+
+        LW_CLEANUP_CTERR(exc, IsApparmorConfigured(&configured));
 
-    LW_CLEANUP_CTERR(exc, IsApparmorConfigured(&configured));
-    LW_CLEANUP_CTERR(exc, HasApparmor(&hasApparmor));
-    if(options->joiningDomain)
-    {
         if(!configured)
         {
             result = NotConfigured;
@@ -1236,15 +1246,24 @@ static QueryResult QueryNsswitch(const JoinProcessOptions *options, LWException
     }
     else
     {
-        if(hasApparmor && configured)
+        LW_CLEANUP_CTERR(exc, DJHasMethodsCfg(&exists));
+
+        if(exists)
         {
-            result = SufficientlyConfigured;
-            goto cleanup;
+            LW_CLEANUP_CTERR(exc, DJIsMethodsCfgConfigured(&configured));
+
+            if(configured)
+            {
+                result = NotConfigured;
+                goto cleanup;
+            }
         }
     }
 
 cleanup:
+
     FreeNsswitchConfContents(&conf);
+
     return result;
 }
 
@@ -1258,7 +1277,7 @@ static void RestartDtloginIfRunning(JoinProcessOptions *options, LWException **e
     memset(&distro, 0, sizeof(distro));
 
     DJGetDaemonStatus("dtlogin", &doRestart, &inner);
-    if(!LW_IS_OK(inner) && inner->code == CENTERROR_DOMAINJOIN_MISSING_DAEMON)
+    if(!LW_IS_OK(inner) && inner->code == ERROR_SERVICE_NOT_FOUND)
     {
         LW_HANDLE(&inner);
         goto cleanup;
@@ -1281,29 +1300,41 @@ static void RestartDtloginIfRunning(JoinProcessOptions *options, LWException **e
                 LW_CLEANUP_CTERR(exc, DJGetDistroInfo(NULL, &distro));
                 if(distro.os == OS_SUNOS)
                 {
-                    options->warningCallback(options, "Unable to restart dtlogin",
-                        "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
-                        "/etc/init.d/dtlogin stop\n"
-                        "/etc/init.d/dtlogin start");
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options, "Unable to restart dtlogin",
+                            "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
+                            "/etc/init.d/dtlogin stop\n"
+                            "/etc/init.d/dtlogin start");
+                    }
                 }
                 else if(distro.os == OS_HPUX)
                 {
-                    options->warningCallback(options, "Unable to restart dtlogin",
-                        "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
-                        "/sbin/init.d/dtlogin.rc stop\n"
-                        "/sbin/init.d/dtlogin.rc start");
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options, "Unable to restart dtlogin",
+                            "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
+                            "/sbin/init.d/dtlogin.rc stop\n"
+                            "/sbin/init.d/dtlogin.rc start");
+                    }
                 }
                 else if(distro.os == OS_AIX)
                 {
-                    options->warningCallback(options, "Unable to restart dtlogin",
-                        "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
-                        "kill `cat /var/dt/Xpid`\n"
-                        "/etc/rc.dt");
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options, "Unable to restart dtlogin",
+                            "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please run these commands as root, outside of an Xwindows session:\n\n"
+                            "kill `cat /var/dt/Xpid`\n"
+                            "/etc/rc.dt");
+                    }
                 }
                 else
                 {
-                    options->warningCallback(options, "Unable to restart dtlogin",
-                        "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please restart dtlogin.");
+                    if (options->warningCallback != NULL)
+                    {
+                        options->warningCallback(options, "Unable to restart dtlogin",
+                            "The dtlogin process needs to be restarted for domain users to interactively login graphically, but it cannot be restarted at this time because a user is currently logged in. After the user exits, please restart dtlogin.");
+                    }
                 }
             }
         }
@@ -1322,7 +1353,7 @@ static void DoNsswitch(JoinProcessOptions *options, LWException **exc)
 {
     LWException *restartException = NULL;
     NsswitchConf conf;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
 
     memset(&conf, 0, sizeof(conf));
 
@@ -1334,10 +1365,13 @@ static void DoNsswitch(JoinProcessOptions *options, LWException **exc)
         LW_CLEANUP_CTERR(exc, DJUnconfigMethodsConfigFile());
 
     ceError = ReadNsswitchConf(&conf, "", TRUE);
-    if(ceError == CENTERROR_INVALID_FILENAME)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
-        ceError = CENTERROR_SUCCESS;
-        options->warningCallback(options, "Could not find file", "Could not find nsswitch file");
+        ceError = ERROR_SUCCESS;
+        if (options->warningCallback != NULL)
+        {
+            options->warningCallback(options, "Could not find file", "Could not find nsswitch file");
+        }
         goto cleanup;
     }
     LW_CLEANUP_CTERR(exc, ceError);
@@ -1356,16 +1390,30 @@ static void DoNsswitch(JoinProcessOptions *options, LWException **exc)
 
     CTCaptureOutputToExc( SCRIPTDIR "/ConfigureLogin nsswitch_restart",
             &restartException);
-    if(restartException != NULL && restartException->code == CENTERROR_COMMAND_FAILED)
+    if(restartException != NULL && restartException->code == ERROR_BAD_COMMAND)
     {
-        options->warningCallback(options, "Unable to restart services after nsswitch modification", restartException->longMsg);
+        if (options->warningCallback)
+        {
+            options->warningCallback(options, "Some services require manual restart", restartException->longMsg);
+        }
         LW_HANDLE(&restartException);
     }
     LW_CLEANUP(exc, restartException);
 
     LW_TRY(exc, RestartDtloginIfRunning(options, &LW_EXC));
 
+    if (options->warningCallback != NULL)
+    {
+        options->warningCallback(options,
+                                 "System restart required",
+                                 "Your system has been configured to authenticate to "
+                                 "Active Directory for the first time.  It is recommended "
+                                 "that you restart your system to ensure that all applications "
+                                 "recognize the new settings.");
+    }
+
 cleanup:
+
     FreeNsswitchConfContents(&conf);
 }
 
@@ -1377,11 +1425,11 @@ static PSTR GetNsswitchDescription(const JoinProcessOptions *options, LWExceptio
     QueryResult compatResult = FullyConfigured;
     PSTR compatDescription = NULL;
     NsswitchConf conf;
-    CENTERROR ceError = CENTERROR_SUCCESS;
+    DWORD ceError = ERROR_SUCCESS;
 
     memset(&conf, 0, sizeof(conf));
 
-    LW_CLEANUP_CTERR(exc, UnsuportedSeLinuxEnabled(&hasBadSeLinux));
+    LW_CLEANUP_CTERR(exc, UnsupportedSeLinuxEnabled(&hasBadSeLinux));
     if(hasBadSeLinux)
     {
         LW_CLEANUP_CTERR(exc, CTAllocateStringPrintf(&ret,
@@ -1390,10 +1438,13 @@ static PSTR GetNsswitchDescription(const JoinProcessOptions *options, LWExceptio
     }
 
     ceError = ReadNsswitchConf(&conf, "", TRUE);
-    if(ceError == CENTERROR_INVALID_FILENAME)
+    if(ceError == ERROR_FILE_NOT_FOUND)
     {
-        ceError = CENTERROR_SUCCESS;
-        options->warningCallback(options, "Could not find file", "Could not find nsswitch file");
+        ceError = ERROR_SUCCESS;
+        if (options->warningCallback != NULL)
+        {
+            options->warningCallback(options, "Could not find file", "Could not find nsswitch file");
+        }
         goto cleanup;
     }
     LW_CLEANUP_CTERR(exc, ceError);
