@@ -55,7 +55,8 @@ LWIIsLocalHost(
     struct addrinfo* remoteInfo = NULL;
     PCSTR            pcszLocalHost = NULL;
     PCSTR            pcszRemoteHost = NULL;
-    
+    CHAR             canonNameLocal[NI_MAXHOST] = "";
+    CHAR             canonNameRemote[NI_MAXHOST] = "";
 
     memset(localHost, 0, sizeof(localHost));
 
@@ -76,9 +77,14 @@ LWIIsLocalHost(
         }
         else
         {
-            pcszLocalHost = localInfo->ai_canonname ?
-                            localInfo->ai_canonname :
-                            localHost;
+            dwError = getnameinfo(localInfo->ai_addr, localInfo->ai_addrlen, canonNameLocal, NI_MAXHOST, NULL, 0, 0);
+
+            if(dwError || !canonNameLocal[0]) {
+                pcszLocalHost = localHost;
+            }
+            else {
+                pcszLocalHost = canonNameLocal;
+            }
         }
 
         dwError = getaddrinfo(hostname, NULL, NULL, &remoteInfo);
@@ -88,12 +94,17 @@ LWIIsLocalHost(
         }
         else
         {
-            pcszRemoteHost = remoteInfo->ai_canonname ?
-                             remoteInfo->ai_canonname :
-                             hostname;
+            dwError = getnameinfo(remoteInfo->ai_addr, remoteInfo->ai_addrlen, canonNameRemote, NI_MAXHOST, NULL, 0, 0);
+
+            if(dwError || !canonNameRemote[0]) {
+                pcszRemoteHost = hostname;
+            }
+            else {
+                pcszRemoteHost = canonNameRemote;
+            }
         }
 
-        if ( !strcasecmp(pcszLocalHost,pcszRemoteHost) )
+        if ( !strcasecmp(pcszLocalHost, pcszRemoteHost) )
         {
             bResult = TRUE;
         }
@@ -119,7 +130,7 @@ LWICreateEventLogRpcBinding(
     handle_t *   event_binding
     )
 {
-    unsigned int rpcstatus = 0;
+    DWORD winerror = 0;
     DWORD dwError = 0;
     const char * protocol;
     const char * endpoint;
@@ -130,12 +141,12 @@ LWICreateEventLogRpcBinding(
     handle_t eventBinding_local = 0;
     BOOLEAN bLocalHost = FALSE;
 
-    if ( (hostname == NULL) || ((geteuid() == 0) && LWIIsLocalHost(hostname)) )
+    if (hostname == NULL || LWIIsLocalHost(hostname))
     {
         /* If no host is specified, connect to the local host over ncalrpc */
         bLocalHost = TRUE;
         protocol = "ncalrpc";
-        endpoint = CACHEDIR "/rpc/socket";
+        endpoint = CACHEDIR "/.eventlog";
     }
     else
     {
@@ -147,8 +158,8 @@ LWICreateEventLogRpcBinding(
     EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() hostname=%s, *event_binding=%.16X\n",
                     hostname, *event_binding);
 
-    RPC_STRING_BINDING_COMPOSE((char*) protocol, (char*) hostname, (char*) endpoint, &pszBindingString, &rpcstatus);
-    BAIL_ON_DCE_ERROR(dwError, rpcstatus);
+    RPC_STRING_BINDING_COMPOSE((char*) protocol, (char*) hostname, (char*) endpoint, &pszBindingString, &winerror);
+    BAIL_ON_DCE_ERROR(dwError, winerror);
 
     if (pszBindingString == NULL || *pszBindingString == '\0') {
         BAIL_ON_DCE_ERROR(dwError, RPC_S_INVALID_STRING_BINDING);
@@ -157,8 +168,8 @@ LWICreateEventLogRpcBinding(
     EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() pszBindingString=%s, running rbfsb\n",
                     pszBindingString);
 
-    RPC_BINDING_FROM_STRING_BINDING(pszBindingString, &eventBinding_local, &rpcstatus);
-    BAIL_ON_DCE_ERROR(dwError, rpcstatus);
+    RPC_BINDING_FROM_STRING_BINDING(pszBindingString, &eventBinding_local, &winerror);
+    BAIL_ON_DCE_ERROR(dwError, winerror);
 
     EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() eventBinding_local=%.16X, finished rbfsb\n",
                     eventBinding_local);
@@ -174,30 +185,28 @@ LWICreateEventLogRpcBinding(
         
         ret = snprintf(hostPrincipal, hostPrincipalSize, "host/%s", hostname);
         if (ret < 0 || ret >= hostPrincipalSize) {
-            BAIL_ON_EVT_ERROR(EVT_ERROR_INSUFFICIENT_BUFFER);
+            BAIL_ON_EVT_ERROR(ERROR_INSUFFICIENT_BUFFER);
         }
         
         EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() using host principal [%s]\n",
                         hostPrincipal);
         
-        RPC_BINDING_SET_AUTH_INFO(eventBinding_local,
+        winerror = RpcBindingSetAuthInfo(eventBinding_local,
                                   (unsigned char*)hostPrincipal,
                                   rpc_c_protect_level_pkt_privacy,
                                   rpc_c_authn_gss_negotiate,
                                   NULL,
-                                  rpc_c_authz_name,
-                                  &rpcstatus);
-        BAIL_ON_DCE_ERROR(dwError, rpcstatus);
+                                  rpc_c_authz_name);
+        BAIL_ON_DCE_ERROR(dwError, winerror);
         
         EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() eventBinding_local=%.16X, auth info set"
-                        "rpcstatus=0x%08x\n", eventBinding_local, rpcstatus);
+                        "winerror=0x%08x\n", eventBinding_local, winerror);
         
     }
 
     *event_binding = eventBinding_local;
 
-    EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() finished successfully\n",
-                    rpcstatus);
+    EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() finished successfully\n");
 
 cleanup:
     if (hostPrincipal)
@@ -214,8 +223,8 @@ cleanup:
     return dwError;
 
 error:
-    EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() label error: rpcstatus=%d\n",
-                    rpcstatus);
+    EVT_LOG_VERBOSE("client::eventlogbinding.c: CreateEventLogRpcBinding() label error: winerror=%d\n",
+                    winerror);
 
     goto cleanup;
 }
