@@ -42,14 +42,20 @@
 #include "includes.h"
 
 
-
-void
+VOID
 LWIFreeEventRecord(
     PEVENT_LOG_RECORD pEventRecord
     )
 {
-    EVT_LOG_VERBOSE("client::eventlog.c FreeEventRecord(pEventRecord=%.16X)\n", pEventRecord);
+    LWIFreeEventRecordList(1, pEventRecord);
+}
 
+VOID
+LWIFreeEventRecordContents(
+    PEVENT_LOG_RECORD pEventRecord
+    )
+{
+    EVT_SAFE_FREE_STRING(pEventRecord->pszEventTableCategoryId);
     EVT_SAFE_FREE_STRING(pEventRecord->pszEventType);
     EVT_SAFE_FREE_STRING(pEventRecord->pszEventSource);
     EVT_SAFE_FREE_STRING(pEventRecord->pszEventCategory);
@@ -57,11 +63,22 @@ LWIFreeEventRecord(
     EVT_SAFE_FREE_STRING(pEventRecord->pszComputer);
     EVT_SAFE_FREE_STRING(pEventRecord->pszDescription);
     EVT_SAFE_FREE_MEMORY(pEventRecord->pszData);
+}
 
-    EVTFreeMemory(pEventRecord);
+VOID
+LWIFreeEventRecordList(
+    DWORD dwRecords,
+    PEVENT_LOG_RECORD pEventRecordList
+    )
+{
+    DWORD dwIndex = 0;
 
-    EVT_LOG_VERBOSE("client::eventlog.c FreeEventRecord() Finished\n", pEventRecord);
+    for (dwIndex = 0; dwIndex < dwRecords; dwIndex++)
+    {
+        LWIFreeEventRecordContents(&pEventRecordList[dwIndex]);
+    }
 
+    EVTFreeMemory(pEventRecordList);
 }
 
 void
@@ -73,13 +90,8 @@ LWIFreeEventLogHandle(
     PEVENT_LOG_HANDLE pEventLogHandle = (PEVENT_LOG_HANDLE) hEventLog;
     PEVENT_LOG_RECORD pEventRecord = &(pEventLogHandle->defaultEventLogRecord);
 
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszEventType);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszEventSource);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszEventCategory);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszUser);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszComputer);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszDescription);
-    EVT_SAFE_FREE_MEMORY(pEventRecord->pszData);
+    LWIFreeEventLogRpcBinding(pEventLogHandle->bindingHandle);
+    LWIFreeEventRecordContents(pEventRecord);
     EVT_SAFE_FREE_MEMORY(pEventLogHandle);
 }
 
@@ -138,7 +150,7 @@ LWIOpenEventLog(
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_RPC_BINDING);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY
 
@@ -152,17 +164,17 @@ LWIOpenEventLog(
     }
     CATCH (rpc_x_auth_method)
     {
-        dwError = EVT_ERROR_ACCESS_DENIED;
+        dwError = ERROR_ACCESS_DENIED;
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_OPEN);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY
 
     BAIL_ON_EVT_ERROR(dwError);
 
-    pEventLogHandle->bindingHandle = (ULONG) eventBindingLocal;
+    pEventLogHandle->bindingHandle = eventBindingLocal;
     *phEventLog = (HANDLE)pEventLogHandle;
 
 
@@ -172,7 +184,7 @@ cleanup:
 error:
     switch(dwError)
     {
-        case EVT_ERROR_ACCESS_DENIED:
+        case ERROR_ACCESS_DENIED:
             EVT_LOG_ERROR("Failed to open event log. Access is denied.\n");
             break;
         default:
@@ -185,6 +197,11 @@ error:
         LWIFreeEventLogHandle((HANDLE)pEventLogHandle);
     }
 
+    if (eventBindingLocal)
+    {
+        LWIFreeEventLogRpcBinding(eventBindingLocal);
+    }
+
     *phEventLog = (HANDLE) NULL;
 
     goto cleanup;
@@ -273,12 +290,12 @@ LWICloseEventLog(
     TRY
     {
         dwError = RpcLWICloseEventLog(
-            (handle_t)(ULONG) pEventLogHandle->bindingHandle
+            (handle_t) pEventLogHandle->bindingHandle
             );
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_CLOSE);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
@@ -315,7 +332,7 @@ LWIReadEventLog(
     PEVENT_LOG_HANDLE pEventLogHandle = (PEVENT_LOG_HANDLE) hEventLog;
 
     if (sqlFilter == NULL) {
-        dwError = EVT_ERROR_INTERNAL;
+        dwError = ERROR_INTERNAL_ERROR;
         BAIL_ON_EVT_ERROR(dwError);
     }
 
@@ -329,7 +346,7 @@ LWIReadEventLog(
     TRY
     {
         dwError = RpcLWIReadEventLog(
-                    (handle_t)(ULONG) pEventLogHandle->bindingHandle,
+                    (handle_t) pEventLogHandle->bindingHandle,
                     dwLastRecordId, 
                     nRecordsPerPage,
                     (idl_char*)sqlFilterChar, 
@@ -338,7 +355,7 @@ LWIReadEventLog(
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_READ);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
@@ -346,6 +363,7 @@ LWIReadEventLog(
 
 
 cleanup:
+    EVT_SAFE_FREE_STRING(sqlFilterChar);
     return dwError;
 
 error:
@@ -368,7 +386,7 @@ LWICountEventLog(
 
     if (sqlFilter == NULL) {
         EVT_LOG_VERBOSE("CountEventLog(): sqlFilter == NULL\n");
-        dwError = EVT_ERROR_INTERNAL;
+        dwError = ERROR_INTERNAL_ERROR;
         BAIL_ON_EVT_ERROR(dwError);
     }
 
@@ -379,19 +397,20 @@ LWICountEventLog(
     TRY
     {
         dwError = RpcLWIEventLogCount(
-                    (handle_t)(ULONG) pEventLogHandle->bindingHandle,
+                    (handle_t) pEventLogHandle->bindingHandle,
                     (idl_char*)sqlFilterChar,
                     (unsigned32*)pdwNumMatched);
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_COUNT);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
     BAIL_ON_EVT_ERROR(dwError);
 
 cleanup:
+    EVT_SAFE_FREE_STRING(sqlFilterChar);
     return dwError;
 
 error:
@@ -554,6 +573,40 @@ error:
 }
 
 
+DWORD
+LWIWriteEventLogRecords(
+    HANDLE hEventLog,
+    DWORD cRecords,
+    PEVENT_LOG_RECORD pEventRecords
+    )
+{
+    volatile DWORD dwError = 0;
+    PEVENT_LOG_HANDLE pEventLogHandle = (PEVENT_LOG_HANDLE) hEventLog;
+
+    TRY
+    {
+        dwError = RpcLWIWriteEventLogRecords(
+                        (handle_t) pEventLogHandle->bindingHandle,
+                        cRecords,
+                        pEventRecords);
+    }
+    CATCH_ALL
+    {
+        dwError = EVTGetRpcError(THIS_CATCH);
+    }
+    ENDTRY;
+
+    BAIL_ON_EVT_ERROR(dwError);
+
+cleanup:
+
+    return dwError;
+
+error:
+    EVT_LOG_ERROR("Failed to write event log. Error code [%d]\n", dwError);
+    goto cleanup;
+}
+
 DWORD
 LWIWriteEventLogBase(
     HANDLE hEventLog,
@@ -638,12 +691,14 @@ LWIWriteEventLogBase(
 
     TRY
     {
-        dwError = RpcLWIWriteEventLog( (handle_t)(ULONG) pEventLogHandle->bindingHandle,
-						   eventRecordLocal);
+        dwError = RpcLWIWriteEventLogRecords(
+                        (handle_t) pEventLogHandle->bindingHandle,
+                        1,
+                        &eventRecordLocal);
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_WRITE);        
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
@@ -658,8 +713,6 @@ error:
     goto cleanup;
 }
 
-
-
 DWORD
 LWIWriteEventLog(
     HANDLE hEventLog,
@@ -709,12 +762,12 @@ LWIDeleteFromEventLog(
 
     TRY
     {
-        dwError = RpcLWIDeleteFromEventLog( (handle_t)(ULONG) pEventLogHandle->bindingHandle,
+        dwError = RpcLWIDeleteFromEventLog( (handle_t) pEventLogHandle->bindingHandle,
                                             (idl_char*)sqlFilterChar);
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_DELETE);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
@@ -739,11 +792,11 @@ LWIClearEventLog(
 
     TRY
     {
-        dwError = RpcLWIClearEventLog( (handle_t)(ULONG) pEventLogHandle->bindingHandle);
+        dwError = RpcLWIClearEventLog( (handle_t) pEventLogHandle->bindingHandle);
     }
     CATCH_ALL
     {
-        dwError = EVTGetRpcError(THIS_CATCH, EVT_ERROR_RPC_EXCEPTION_UPON_CLEAR);
+        dwError = EVTGetRpcError(THIS_CATCH);
     }
     ENDTRY;
 
@@ -760,22 +813,11 @@ error:
 
 DWORD
 EVTGetRpcError(
-    dcethread_exc* exCatch,
-    DWORD dwEVTError
+    dcethread_exc* exCatch
     )
 {
     DWORD dwError = 0;
-
-#ifdef _WIN32
-    dwError = dwEVTError;
-#else
     dwError = dcethread_exc_getstatus (exCatch);
-    if(!dwError)
-    {
-        dwError = dwEVTError;
-    }
-#endif //!_WIN32
-
-    return dwError;
+    return LwNtStatusToWin32Error(LwRpcStatusToNtStatus(dwError));
 }
 
