@@ -30,6 +30,10 @@
   #include <errno.h>
   #include <netdb.h>
   #include <ctype.h>
+  #include <libxml/tree.h>
+  #include <libxml/parser.h>
+  #include <libxml/xpath.h>
+  #include <libxml/xpathInternals.h>
 
 #if HAVE_LIMITS_H
   #include <limits.h>
@@ -51,16 +55,14 @@
   #include <sys/stat.h>
   #include <dirent.h>
   #include <pwd.h>
-  #include <compat/dcerpc.h>
-  #include <dce/dcethread.h>
-  #include <compat/rpcstatus.h>
+  #include <lwrpcrt/lwrpcrt.h>
+  #include <lw/winerror.h>
   #include <dlfcn.h>
   #include <dce/rpcexc.h>
+  #include <dce/lrpc.h>
 
 #if HAVE_WC16STR_H
   #include <wc16str.h>
 #endif
 
 #endif
-
-
