@@ -94,10 +94,10 @@
                              ORDER BY EventRecordId ASC  \
                              LIMIT %ld OFFSET %ld"
 
-#define DB_QUERY_COUNT_ALL  "SELECT COUNT(EventRecordId)  \
+#define DB_QUERY_COUNT_ALL  "SELECT COUNT(*)  \
                              FROM     lwievents"
 
-#define DB_QUERY_COUNT      "SELECT COUNT(EventRecordId)  \
+#define DB_QUERY_COUNT      "SELECT COUNT(*)  \
                              FROM     lwievents           \
                              WHERE  (%s)"
 
@@ -135,13 +135,16 @@
                                         ?10)"
 
 //Delete the record over 'n' entries
-#define DB_QUERY_DELETE_AT_LIMIT    "DELETE FROM     lwievents      \
-                                     WHERE  EventRecordId NOT IN (  \
-                                       SELECT EventRecordId         \
-                                       FROM     lwievents           \
-                                       ORDER BY EventRecordId DESC  \
-                                       LIMIT %lu                    \
-                                     )"
+#define DB_QUERY_DELETE_AT_LIMIT    "DELETE FROM lwievents \
+                                    WHERE EventRecordId IN ( \
+                                        SELECT EventRecordId \
+                                        FROM lwievents \
+                                        ORDER BY EventRecordId ASC \
+                                        LIMIT max( \
+                                            (SELECT count(*) FROM lwievents) - \
+                                            %lu, 0 \
+                                        ) \
+                                    )"
 
 //Delete the first 'n' entries
 #define DB_QUERY_DELETE_COUNT       "DELETE FROM     lwievents      \
@@ -645,8 +648,7 @@ SrvWriteEventLog(
 static
 DWORD
 SrvMaintainDB(
-    HANDLE hDB,
-    PBOOLEAN pbSafeInsert
+    HANDLE hDB
     )
 {
     DWORD dwError = 0;
@@ -717,18 +719,12 @@ SrvMaintainDB(
         BAIL_ON_EVT_ERROR(dwError);
     }
 
-    //Safe to insert record,set bSafeInsert to TRUE
-    *pbSafeInsert = TRUE;
-
     EVT_LOG_VERBOSE("Pruned DB returning");
 
 cleanup:
-
-	return dwError;
+    return dwError;
 
 error:
-
-    *pbSafeInsert = FALSE;
     goto cleanup;
 }
 
@@ -740,34 +736,43 @@ SrvWriteToDB(
     )
 {
     DWORD dwError = 0;
-    BOOLEAN bSafeInsert = TRUE;
     BOOLEAN bRemoveAsNeeded = FALSE;
+    BOOLEAN bInLock = FALSE;
 
-    dwError = EVTGetRemoveAsNeeded(&bRemoveAsNeeded);
+    //Write the eventlog
+    dwError = SrvWriteEventLog(
+                    hDB,
+                    cRecords,
+                    pEventRecords);
     BAIL_ON_EVT_ERROR(dwError);
 
-    if (bSafeInsert)
+    ENTER_RW_WRITER_LOCK;
+    bInLock = TRUE;
+    gdwNewEventCount += cRecords;
+    if (gdwNewEventCount >= EVT_MAINTAIN_EVENT_COUNT)
     {
-        //Write the eventlog
-        dwError = SrvWriteEventLog(
-                        hDB,
-                        cRecords,
-                        pEventRecords);
+        dwError = EVTGetRemoveAsNeeded(&bRemoveAsNeeded);
         BAIL_ON_EVT_ERROR(dwError);
+
+        gdwNewEventCount = 0;
     }
+    LEAVE_RW_WRITER_LOCK;
+    bInLock = FALSE;
 
     if (bRemoveAsNeeded)
     {
         //Trim the DB
         EVT_LOG_VERBOSE("Going to trim database .....");
-        dwError = SrvMaintainDB(hDB,&bSafeInsert);
+        dwError = SrvMaintainDB(hDB);
         BAIL_ON_EVT_ERROR(dwError);
     }
 
 error:
-
+    if (bInLock)
+    {
+        LEAVE_RW_WRITER_LOCK;
+    }
     return dwError;
-
 }
 
 
@@ -1071,7 +1076,7 @@ SrvQueryEventLog(
 
     pContext = (PEVENTLOG_CONTEXT)((long)hDB);
     if (!pContext->pDbHandle) {
-        dwError = EVT_ERROR_INVALID_DB_HANDLE;
+        dwError = ERROR_INVALID_HANDLE;
         BAIL_ON_EVT_ERROR(dwError);
     }
 
@@ -1143,18 +1148,6 @@ BuildEventLogRecordList(
 
         for (iCol = 0; iCol < nCols; iCol++)
         {
-
-            if (iRow < (nRows -1) || iCol < (nCols - 1))
-            {
-                EVT_LOG_DEBUG("server::evtdb.c BuildEventLogRecordList(r%d, c%d): CURR:%s  NEXT:%s\n",
-                    iRow, iCol, ppszResult[iVal], ppszResult[iVal+1]);
-            }
-
-            else {
-                EVT_LOG_DEBUG("server::evtdb.c BuildEventLogRecordList(r%d, c%d): CURR:%s\n",
-                    iRow, iCol, ppszResult[iVal]);
-            }
-
             switch(iCol)
             {
                 case EventTableCategoryId:
