@@ -57,6 +57,9 @@ typedef struct {
     pthread_mutex_t lock;
     /* Should start as daemon */
     DWORD dwStartAsDaemon;
+    /* Log to syslog if this is true or starting as a daemon without a log
+     * file. */
+    BOOLEAN bLogToSyslog;
     /* How much logging do you want? */
     DWORD dwLogLevel;
     /* log file path */
@@ -81,17 +84,27 @@ typedef struct {
     DWORD dwMaxAge;
     /* Purge the records at the interval */
     DWORD dwPurgeInterval;
+    /* Flag to prune database*/
+    BOOLEAN bRemoveAsNeeded;
 
-    /* Who is allowed to read events */
-    EVTALLOWEDDATA  pAllowReadTo;
-    /* Who is allowed to write events */
-    EVTALLOWEDDATA  pAllowWriteTo;
-    /* Who is allowed to delete events */
-    EVTALLOWEDDATA  pAllowDeleteTo;
+    /* Who is allowed to read, write, and delete events. The security
+     * descriptor is set when all of the users/groups can be resolved. */
+    PSTR pszAllowReadTo;
+    PSTR pszAllowWriteTo;
+    PSTR pszAllowDeleteTo;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pAccess;
 } EVTSERVERINFO, *PEVTSERVERINFO;
 
-EVTSERVERINFO gServerInfo;
+extern EVTSERVERINFO gServerInfo;
 
+#define EVENTLOG_READ_RECORD    1
+#define EVENTLOG_WRITE_RECORD   2
+#define EVENTLOG_DELETE_RECORD  4
+
+VOID
+EVTFreeSecurityDescriptor(
+    PSECURITY_DESCRIPTOR_ABSOLUTE pDescriptor
+    );
 
 DWORD
 EVTGetConfigPath(
@@ -108,11 +121,22 @@ EVTGetPrefixPath(
     PSTR* ppszPath
     );
 
+DWORD
+EVTCheckAllowed(
+    PACCESS_TOKEN pUserToken,
+    ACCESS_MASK dwAccessMask
+    );
+
 DWORD
 EVTGetMaxRecords(
     DWORD* pdwMaxRecords
     );
 
+DWORD
+EVTGetRemoveAsNeeded(
+    PBOOLEAN pbRemoveAsNeeded
+    );
+
 DWORD
 EVTGetMaxAge(
     DWORD* pdwMaxLogSize
@@ -128,21 +152,6 @@ EVTGetRemoveEventsFlag(
     PBOOLEAN pbRemoveEvents
     );
 
-DWORD
-EVTGetAllowReadToLocked(
-    PEVTALLOWEDDATA * ppAllowReadTo
-    );
-
-DWORD
-EVTGetAllowWriteToLocked(
-    PEVTALLOWEDDATA * ppAllowWriteTo
-    );
-
-DWORD
-EVTGetAllowDeleteToLocked(
-    PEVTALLOWEDDATA * ppAllowDeleteTo
-    );
-
 void
 EVTUnlockServerInfo();
 
