@@ -53,15 +53,6 @@ ParseEvents(
     PFNEventRecordProcessor eventRecordProcessor
     );
 
-DWORD
-ParseAndAddEvents(
-    PEVENT_LOG_HANDLE pEventLogHandle,
-    PSTR   pszFilePath,
-    PSTR   pszEventTableCategoryId,
-    BOOLEAN bEventTableCategoryIdInCSV,
-    PFNEventRecordProcessor eventRecordProcessor
-    );
-
 DWORD
 PrintEventRecords(
     FILE* output, 
@@ -81,6 +72,7 @@ PrintEventRecordsTable(
 DWORD
 ReadAndExportEvents(
     PEVENT_LOG_HANDLE pEventLogHandle,
+    PCWSTR pwszSqlFilter,
     FILE* fpExport
     );
 
