@@ -86,7 +86,10 @@ LsaOpenServer(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwError = MAP_LWMSG_ERROR(lwmsg_assoc_establish(pContext->pAssoc));
+    dwError = MAP_LWMSG_ERROR(lwmsg_assoc_connect(pContext->pAssoc, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = MAP_LWMSG_ERROR(lwmsg_assoc_get_session(pContext->pAssoc, &pContext->pSession));
     BAIL_ON_LSA_ERROR(dwError);
 
     *phConnection = (HANDLE)pContext;
@@ -143,6 +146,30 @@ LsaCloseServer(
     return dwError;
 }
 
+DWORD
+LsaDropServer(
+    HANDLE hConnection
+    )
+{
+    DWORD dwError = 0;
+    PLSA_CLIENT_CONNECTION_CONTEXT pContext =
+                     (PLSA_CLIENT_CONNECTION_CONTEXT)hConnection;
+
+    if (pContext->pAssoc)
+    {
+        lwmsg_assoc_delete(pContext->pAssoc);
+    }
+
+    if (pContext->pProtocol)
+    {
+        lwmsg_protocol_delete(pContext->pProtocol);
+    }
+
+    LwFreeMemory(pContext);
+
+    return dwError;
+}
+
 DWORD
 LsaIpcAcquireCall(
     HANDLE hServer,
@@ -161,15 +188,15 @@ error:
 }
 
 DWORD
-LsaIpcUnregisterHandle(
-    LWMsgCall* pCall,
+LsaIpcReleaseHandle(
+    HANDLE hServer,
     PVOID pHandle
     )
 {
     DWORD dwError = 0;
-    LWMsgSession* pSession = lwmsg_call_get_session(pCall);
+    PLSA_CLIENT_CONNECTION_CONTEXT pContext = hServer;
 
-    dwError = MAP_LWMSG_ERROR(lwmsg_session_unregister_handle(pSession, pHandle));
+    dwError = MAP_LWMSG_ERROR(lwmsg_session_release_handle(pContext->pSession, pHandle));
     BAIL_ON_LSA_ERROR(dwError);
 
 error:
@@ -343,14 +370,13 @@ error:
 }
 
 DWORD
-LsaTransactAuthenticateUser(
+LsaTransactAuthenticateUserPam(
     HANDLE hServer,
-    PCSTR  pszLoginName,
-    PCSTR  pszPassword
+    PLSA_AUTH_USER_PAM_PARAMS pParams,
+    PLSA_AUTH_USER_PAM_INFO* ppPamAuthInfo
     )
 {
     DWORD dwError = 0;
-    LSA_IPC_AUTH_USER_REQ authUserReq;
     PLSA_IPC_ERROR pError = NULL;
 
     LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
@@ -360,21 +386,33 @@ LsaTransactAuthenticateUser(
     dwError = LsaIpcAcquireCall(hServer, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
-    authUserReq.pszLoginName = pszLoginName;
-    authUserReq.pszPassword = pszPassword;
-
-    in.tag = LSA_Q_AUTH_USER;
-    in.data = &authUserReq;
+    in.tag = LSA_Q_AUTH_USER_PAM;
+    in.data = pParams;
 
     dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
     BAIL_ON_LSA_ERROR(dwError);
 
     switch (out.tag)
     {
-        case LSA_R_AUTH_USER_SUCCESS:
+        case LSA_R_AUTH_USER_PAM_SUCCESS:
+            if (ppPamAuthInfo)
+            {
+                *ppPamAuthInfo = out.data;
+                out.data = NULL;
+            }
             break;
-        case LSA_R_AUTH_USER_FAILURE:
+        case LSA_R_AUTH_USER_PAM_FAILURE:
             pError = (PLSA_IPC_ERROR) out.data;
+            if (pParams->dwFlags & LSA_AUTH_USER_PAM_FLAG_RETURN_MESSAGE &&
+                    ppPamAuthInfo)
+            {
+                dwError = LwAllocateMemory(sizeof(*ppPamAuthInfo),
+                                            (PVOID)ppPamAuthInfo);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                (*ppPamAuthInfo)->pszMessage = pError->pszErrorMessage;
+                pError->pszErrorMessage = NULL;
+            }
             dwError = pError->dwError;
             BAIL_ON_LSA_ERROR(dwError);
             break;
@@ -1243,6 +1281,90 @@ error:
     goto cleanup;
 }
 
+LW_DWORD
+LsaTransactFindGroupAndExpandedMembers(
+    LW_IN LW_HANDLE hLsa,
+    LW_PCSTR pszTargetProvider,
+    LW_IN LSA_FIND_FLAGS FindFlags,
+    LW_IN LSA_QUERY_TYPE QueryType,
+    LW_IN LSA_QUERY_ITEM QueryItem,
+    LW_OUT PLSA_SECURITY_OBJECT* ppGroupObject,
+    LW_OUT LW_PDWORD pdwMemberObjectCount,
+    LW_OUT PLSA_SECURITY_OBJECT** pppMemberObjects
+    )
+{
+    DWORD dwError = 0;
+    LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ req = {0};
+    PLSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES pRes = NULL;
+    PLSA_IPC_ERROR pError = NULL;
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+
+    dwError = LsaIpcAcquireCall(hLsa, &pCall);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    req.pszTargetProvider = pszTargetProvider;
+    req.FindFlags = FindFlags;
+    req.QueryType = QueryType;
+    req.QueryItem = QueryItem;
+
+    switch(QueryType)
+    {
+    case LSA_QUERY_TYPE_BY_UNIX_ID:
+        req.IpcQueryType = LSA2_IPC_QUERY_DWORDS;
+        break;
+    default:
+        req.IpcQueryType = LSA2_IPC_QUERY_STRINGS;
+        break;
+    }
+
+    in.tag = LSA2_Q_FIND_GROUP_AND_EXPANDED_MEMBERS;
+    in.data = &req;
+
+    dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out, NULL, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    switch (out.tag)
+    {
+    case LSA2_R_FIND_GROUP_AND_EXPANDED_MEMBERS:
+        pRes = out.data;
+        *ppGroupObject = pRes->pGroup;
+        *pdwMemberObjectCount = pRes->dwMemberObjectCount;
+        *pppMemberObjects = pRes->ppMemberObjects;
+
+        pRes->pGroup = NULL;
+        pRes->ppMemberObjects = NULL;
+        break;
+    case LSA2_R_ERROR:
+        pError = (PLSA_IPC_ERROR) out.data;
+        dwError = pError->dwError;
+        BAIL_ON_LSA_ERROR(dwError);
+        break;
+    default:
+        dwError = LW_ERROR_INTERNAL;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+cleanup:
+
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    return dwError;
+
+error:
+
+   *ppGroupObject = NULL;
+   *pdwMemberObjectCount = 0;
+   *pppMemberObjects = NULL;
+
+    goto cleanup;
+}
+
 DWORD
 LsaTransactCloseEnum(
     IN HANDLE hLsa,
@@ -1254,13 +1376,10 @@ LsaTransactCloseEnum(
     LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
     LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
-    LWMsgSession* pSession = NULL;
 
     dwError = LsaIpcAcquireCall(hLsa, &pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
-    pSession = lwmsg_call_get_session(pCall);
-
     in.tag = LSA2_Q_CLOSE_ENUM;
     in.data = hEnum;
 
@@ -1270,7 +1389,6 @@ LsaTransactCloseEnum(
     switch (out.tag)
     {
     case LSA2_R_CLOSE_ENUM:
-        lwmsg_session_unregister_handle(pSession, hEnum);
         break;
     case LSA2_R_ERROR:
         pError = (PLSA_IPC_ERROR) out.data;
@@ -1282,6 +1400,65 @@ LsaTransactCloseEnum(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+cleanup:
+
+    /* Release handle no matter what */
+    LsaIpcReleaseHandle(hLsa, hEnum);
+
+    if (pCall)
+    {
+        lwmsg_call_destroy_params(pCall, &out);
+        lwmsg_call_release(pCall);
+    }
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
+
+DWORD
+LsaTransactGetSmartCardUserObject(
+    HANDLE hServer,
+    PLSA_SECURITY_OBJECT* ppObject,
+    PSTR* ppszSmartCardReader
+    )
+{
+    DWORD dwError = 0;
+    PLSA_IPC_ERROR pError = NULL;
+    LWMsgParams in = LWMSG_PARAMS_INITIALIZER;
+    LWMsgParams out = LWMSG_PARAMS_INITIALIZER;
+    LWMsgCall* pCall = NULL;
+    LSA2_IPC_GET_SMART_CARD_USER_RES* getSmartCardUserResponse;
+
+    dwError = LsaIpcAcquireCall(hServer, &pCall);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    in.tag = LSA2_Q_GET_SMARTCARD_USER_OBJECT;
+
+    dwError = MAP_LWMSG_ERROR(lwmsg_call_dispatch(pCall, &in, &out,
+                NULL, NULL));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    switch (out.tag)
+    {
+    case LSA2_R_GET_SMARTCARD_USER_OBJECT:
+        getSmartCardUserResponse = out.data;
+        *ppObject = getSmartCardUserResponse->pObject;
+        *ppszSmartCardReader = getSmartCardUserResponse->pszSmartCardReader;
+        getSmartCardUserResponse->pObject = NULL;
+        getSmartCardUserResponse->pszSmartCardReader = NULL;
+        break;
+    case LSA2_R_ERROR:
+        pError = (PLSA_IPC_ERROR) out.data;
+        BAIL_WITH_LSA_ERROR(pError->dwError);
+        break;
+    default:
+        BAIL_WITH_LSA_ERROR(LW_ERROR_INTERNAL);
+    }
+
 cleanup:
 
     if (pCall)
