@@ -49,54 +49,55 @@
 
 static LWMsgClient* gpClient = NULL;
 static LWMsgProtocol* gpProtocol = NULL;
-
-#if defined(__LWI_SOLARIS__) || defined (__LWI_AIX__)
-static pthread_once_t gOnceControl = {PTHREAD_ONCE_INIT};
-#else
-static pthread_once_t gOnceControl = PTHREAD_ONCE_INIT;
-#endif
+static LWMsgSession* gpSession = NULL;
+static pthread_mutex_t gLock = PTHREAD_MUTEX_INITIALIZER;
 
 static
-VOID
-__NtlmInitialize(VOID)
+DWORD
+NtlmInitialize(
+    VOID
+    )
 {
-    const DWORD dwMaxConnections = 10;
     DWORD dwError = LW_ERROR_SUCCESS;
 
-    dwError = LwMapLwmsgStatusToLwError(lwmsg_protocol_new(NULL, &gpProtocol));
-    BAIL_ON_LSA_ERROR(dwError);
+    pthread_mutex_lock(&gLock);
 
-    dwError = LwMapLwmsgStatusToLwError(
-        lwmsg_protocol_add_protocol_spec(gpProtocol, NtlmIpcGetProtocolSpec()));
-    BAIL_ON_LSA_ERROR(dwError);
+    if (!gpProtocol)
+    {
+        dwError = LwMapLwmsgStatusToLwError(lwmsg_protocol_new(NULL, &gpProtocol));
+        BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMapLwmsgStatusToLwError(
-        lwmsg_client_new(NULL, gpProtocol, &gpClient));
-    BAIL_ON_LSA_ERROR(dwError);
+        dwError = LwMapLwmsgStatusToLwError(
+            lwmsg_protocol_add_protocol_spec(gpProtocol, NtlmIpcGetProtocolSpec()));
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
-    dwError = LwMapLwmsgStatusToLwError(
-        lwmsg_client_set_max_concurrent(gpClient, dwMaxConnections));
-    BAIL_ON_LSA_ERROR(dwError);
+    if (!gpClient)
+    {
+        dwError = LwMapLwmsgStatusToLwError(
+            lwmsg_client_new(NULL, gpProtocol, &gpClient));
+        BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LwMapLwmsgStatusToLwError(
-        lwmsg_client_set_endpoint(
-            gpClient,
-            LWMSG_CONNECTION_MODE_LOCAL,
-            CACHEDIR "/" NTLM_SERVER_FILENAME));
-    BAIL_ON_LSA_ERROR(dwError);
+        dwError = LwMapLwmsgStatusToLwError(
+            lwmsg_client_set_endpoint(
+                gpClient,
+                LWMSG_CONNECTION_MODE_LOCAL,
+                CACHEDIR "/" NTLM_SERVER_FILENAME));
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
-    return;
+    if (!gpSession)
+    {
+        dwError = LwMapLwmsgStatusToLwError(
+            lwmsg_peer_connect(gpClient, &gpSession));
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
 error:
 
-    abort();
-}
+    pthread_mutex_unlock(&gLock);
 
-static
-VOID
-NtlmInitialize()
-{
-    pthread_once(&gOnceControl, __NtlmInitialize);
+    return dwError;
 }
 
 static
@@ -107,6 +108,9 @@ NtlmIpcAcquireCall(
 {
     DWORD dwError = LW_ERROR_SUCCESS;
 
+    dwError = NtlmInitialize();
+    BAIL_ON_LSA_ERROR(dwError);
+
     dwError = LwMapLwmsgStatusToLwError(
         lwmsg_client_acquire_call(
             gpClient,
@@ -119,16 +123,14 @@ error:
 }
 
 DWORD
-NtlmIpcUnregisterHandle(
-    LWMsgCall* pCall,
+NtlmIpcReleaseHandle(
     PVOID pHandle
     )
 {
     DWORD dwError = 0;
-    LWMsgSession* pSession = lwmsg_call_get_session(pCall);
 
     dwError = MAP_LWMSG_ERROR(
-        lwmsg_session_unregister_handle(pSession, pHandle));
+        lwmsg_session_release_handle(gpSession, pHandle));
     BAIL_ON_LSA_ERROR(dwError);
 
 error:
@@ -156,8 +158,6 @@ NtlmTransactAcceptSecurityContext(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -202,8 +202,11 @@ NtlmTransactAcceptSecurityContext(
                     FALSE);
             BAIL_ON_LSA_ERROR(dwError);
 
-            *phNewContext = pResultList->hNewContext;
-            pResultList->hNewContext = NULL;
+            if (phNewContext)
+            {
+                *phNewContext = pResultList->hNewContext;
+                pResultList->hNewContext = NULL;
+            }
 
             *pfContextAttr = pResultList->fContextAttr;
             *ptsTimeStamp = pResultList->tsTimeStamp;
@@ -221,6 +224,12 @@ NtlmTransactAcceptSecurityContext(
     }
 
 cleanup:
+    if (phContext && *phContext)
+    {
+        NtlmIpcReleaseHandle(*phContext);
+        *phContext = NULL;
+    }
+
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &Out);
@@ -230,6 +239,19 @@ cleanup:
     return dwError;
 
 error:
+    if (phNewContext)
+    {
+        if (phContext)
+        {
+            *phNewContext = *phContext;
+            *phContext = NULL;
+        }
+        else
+        {
+            *phNewContext = NULL;
+        }
+    }
+
     *pfContextAttr = 0;
     *ptsTimeStamp = 0;
 
@@ -254,8 +276,6 @@ NtlmTransactAcquireCredentialsHandle(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -327,8 +347,6 @@ NtlmTransactDecryptMessage(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -399,8 +417,6 @@ NtlmTransactDeleteSecurityContext(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -421,8 +437,6 @@ NtlmTransactDeleteSecurityContext(
     switch (Out.tag)
     {
         case NTLM_R_DELETE_SEC_CTXT_SUCCESS:
-            dwError = NtlmIpcUnregisterHandle(pCall, *phContext);
-            BAIL_ON_LSA_ERROR(dwError);
             break;
         case NTLM_R_GENERIC_FAILURE:
             pError = (PNTLM_IPC_ERROR) Out.data;
@@ -435,6 +449,9 @@ NtlmTransactDeleteSecurityContext(
     }
 
 cleanup:
+
+    NtlmIpcReleaseHandle(*phContext);
+
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &Out);
@@ -462,8 +479,6 @@ NtlmTransactEncryptMessage(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -536,8 +551,6 @@ NtlmTransactExportSecurityContext(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -607,8 +620,6 @@ NtlmTransactFreeCredentialsHandle(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -629,8 +640,6 @@ NtlmTransactFreeCredentialsHandle(
     switch (Out.tag)
     {
         case NTLM_R_FREE_CREDS_SUCCESS:
-            dwError = NtlmIpcUnregisterHandle(pCall, *phCredential);
-            BAIL_ON_LSA_ERROR(dwError);
             break;
         case NTLM_R_GENERIC_FAILURE:
             pError = (PNTLM_IPC_ERROR) Out.data;
@@ -643,6 +652,9 @@ NtlmTransactFreeCredentialsHandle(
     }
 
 cleanup:
+
+    NtlmIpcReleaseHandle(*phCredential);
+
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &Out);
@@ -670,8 +682,6 @@ NtlmTransactImportSecurityContext(
     LWMsgParams Out= LWMSG_PARAMS_INITIALIZER;
     LWMsgCall* pCall = NULL;
 
-    NtlmInitialize();
-
     dwError = NtlmIpcAcquireCall(&pCall);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -832,6 +842,12 @@ NtlmTransactInitializeSecurityContext(
     }
 
 cleanup:
+    if (phContext && *phContext)
+    {
+        NtlmIpcReleaseHandle(*phContext);
+        *phContext = NULL;
+    }
+
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &Out);
@@ -841,7 +857,19 @@ cleanup:
     return dwError;
 
 error:
-    *phNewContext = NULL;
+    if (phNewContext)
+    {
+        if (phContext)
+        {
+            *phNewContext = *phContext;
+            *phContext = NULL;
+        }
+        else
+        {
+            *phNewContext = NULL;
+        }
+    }
+
     *pfContextAttr = 0;
     *ptsExpiry = 0;
     memset(pOutput, 0, sizeof(SecBufferDesc));
