@@ -47,8 +47,6 @@
  *
  */
 
-#define LWMSG_SPEC_META
-
 #include "ipc.h"
 
 static LWMsgTypeSpec gLsaIPCErrorSpec[] =
@@ -60,6 +58,12 @@ static LWMsgTypeSpec gLsaIPCErrorSpec[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gLsaStringSpec[] =
+{
+    LWMSG_PSTR,
+    LWMSG_TYPE_END
+};
+
 LWMsgTypeSpec gLsaGroupInfo0Spec[] =
 {
     LWMSG_STRUCT_BEGIN(LSA_GROUP_INFO_0),
@@ -394,7 +398,6 @@ static LWMsgTypeSpec gLsaMetricPack0Spec[] =
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_0, failedOpenSession),
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_0, failedCloseSession),
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_0, failedChangePassword),
-    LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_0, unauthorizedAccesses),
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
@@ -419,7 +422,6 @@ static LWMsgTypeSpec gLsaMetricPack1Spec[] =
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_1, failedCloseSession),
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_1, successfulChangePassword),
     LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_1, failedChangePassword),
-    LWMSG_MEMBER_UINT64(LSA_METRIC_PACK_1, unauthorizedAccesses),
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
@@ -649,6 +651,28 @@ static LWMsgTypeSpec gLsaIPCAuthUserReqSpec[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gLsaAuthUserPamParamsSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LSA_AUTH_USER_PAM_PARAMS),
+    LWMSG_MEMBER_UINT32(LSA_AUTH_USER_PAM_PARAMS, dwFlags),
+    LWMSG_MEMBER_PSTR(LSA_AUTH_USER_PAM_PARAMS, pszLoginName),
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_MEMBER_PSTR(LSA_AUTH_USER_PAM_PARAMS, pszPassword),
+    LWMSG_ATTR_SENSITIVE,
+    LWMSG_MEMBER_PSTR(LSA_AUTH_USER_PAM_PARAMS, pszPamSource),
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec gLsaAuthUserPamInfoSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LSA_AUTH_USER_PAM_INFO),
+    LWMSG_MEMBER_PSTR(LSA_AUTH_USER_PAM_INFO, pszMessage),
+    LWMSG_MEMBER_UINT8(LSA_AUTH_USER_PAM_INFO, bOnlineLogon),
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
 static LWMsgTypeSpec gLsaIPCChangePasswordReqSpec[] =
 {
     LWMSG_STRUCT_BEGIN(LSA_IPC_CHANGE_PASSWORD_REQ),
@@ -919,27 +943,18 @@ static LWMsgTypeSpec gLsaIPCProviderIoControlReqSpec[] =
     LWMSG_TYPE_END
 };
 
-static LWMsgTypeSpec gLsaIPCSetMachineSid[] =
-{
-    LWMSG_STRUCT_BEGIN(LSA_IPC_SET_MACHINE_SID),
-    LWMSG_MEMBER_PSTR(LSA_IPC_SET_MACHINE_SID, pszSid),
-    LWMSG_STRUCT_END,
-    LWMSG_TYPE_END
-};
-
-static LWMsgTypeSpec gLsaIPCSetMachineName[] =
-{
-    LWMSG_STRUCT_BEGIN(LSA_IPC_SET_MACHINE_NAME),
-    LWMSG_MEMBER_PSTR(LSA_IPC_SET_MACHINE_NAME, pszMachineName),
-    LWMSG_TYPE_END
-};
-
 static LWMsgTypeSpec gLsaIPCGetPamConfigSpec[] =
 {
     LWMSG_STRUCT_BEGIN(LSA_PAM_CONFIG),
     LWMSG_MEMBER_UINT32(LSA_PAM_CONFIG, dwLogLevel),
     LWMSG_MEMBER_UINT32(LSA_PAM_CONFIG, bLsaPamDisplayMOTD),
     LWMSG_MEMBER_PSTR(LSA_PAM_CONFIG, pszAccessDeniedMessage),
+    LWMSG_MEMBER_UINT32(LSA_PAM_CONFIG, dwNumSmartCardServices),
+    LWMSG_MEMBER_POINTER(LSA_PAM_CONFIG, ppszSmartCardServices, LWMSG_PSTR),
+    LWMSG_ATTR_LENGTH_MEMBER(LSA_PAM_CONFIG, dwNumSmartCardServices),
+    LWMSG_MEMBER_UINT32(LSA_PAM_CONFIG, dwNumSmartCardPromptGecos),
+    LWMSG_MEMBER_POINTER(LSA_PAM_CONFIG, ppszSmartCardPromptGecos, LWMSG_PSTR),
+    LWMSG_ATTR_LENGTH_MEMBER(LSA_PAM_CONFIG, dwNumSmartCardPromptGecos),
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
@@ -1161,6 +1176,40 @@ static LWMsgTypeSpec gLsa2IpcQueryMemberOfResSpec[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gLsa2IpcFindGroupAndExpandedGroupMembersReqSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ),
+    LWMSG_MEMBER_PSTR(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, pszTargetProvider),
+    LWMSG_MEMBER_UINT32(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, FindFlags),
+    LWMSG_MEMBER_UINT8(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, QueryType),
+    LWMSG_MEMBER_UINT8(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, IpcQueryType),
+    LWMSG_MEMBER_UNION_BEGIN(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, QueryItem),
+    LWMSG_MEMBER_PSTR(LSA_QUERY_ITEM, pszString),
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_ATTR_TAG(LSA2_IPC_QUERY_STRINGS),
+    LWMSG_MEMBER_UINT32(LSA_QUERY_ITEM, dwId),
+    LWMSG_ATTR_TAG(LSA2_IPC_QUERY_DWORDS),
+    LWMSG_UNION_END,
+    LWMSG_ATTR_DISCRIM(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ, IpcQueryType),
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec gLsa2IpcFindGroupAndExpandedGroupMembersResSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES),
+    LWMSG_MEMBER_POINTER(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES, pGroup, LWMSG_TYPESPEC(gLsaSecurityObjectSpec)),
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_MEMBER_UINT32(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES, dwMemberObjectCount),
+    LWMSG_MEMBER_POINTER_BEGIN(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES, ppMemberObjects),
+    LWMSG_POINTER(LWMSG_TYPESPEC(gLsaSecurityObjectSpec)),
+    LWMSG_POINTER_END,
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_ATTR_LENGTH_MEMBER(LSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES, dwMemberObjectCount),
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
 static LWMsgTypeSpec gLsa2IpcCloseEnumReqSpec[] =
 {
     LWMSG_HANDLE(LSA2_IPC_ENUM_HANDLE),
@@ -1217,11 +1266,21 @@ static LWMsgTypeSpec gLsa2IpcDeleteObjectReqSpec[] =
     LWMSG_TYPE_END
 };
 
+static LWMsgTypeSpec gLsa2IpcGetSmartCardUserObjectResSpec[] =
+{
+    LWMSG_STRUCT_BEGIN(LSA2_IPC_GET_SMART_CARD_USER_RES),
+    LWMSG_MEMBER_POINTER(LSA2_IPC_GET_SMART_CARD_USER_RES, pObject,
+            LWMSG_TYPESPEC(gLsaSecurityObjectSpec)),
+    LWMSG_MEMBER_PSTR(LSA2_IPC_GET_SMART_CARD_USER_RES, pszSmartCardReader),
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
 static LWMsgProtocolSpec gLsaIPCSpec[] =
 {
-    LWMSG_MESSAGE(LSA_Q_AUTH_USER, gLsaIPCAuthUserReqSpec),
-    LWMSG_MESSAGE(LSA_R_AUTH_USER_SUCCESS, NULL),
-    LWMSG_MESSAGE(LSA_R_AUTH_USER_FAILURE, gLsaIPCErrorSpec),
+    LWMSG_MESSAGE(LSA_Q_AUTH_USER_PAM, gLsaAuthUserPamParamsSpec),
+    LWMSG_MESSAGE(LSA_R_AUTH_USER_PAM_SUCCESS, gLsaAuthUserPamInfoSpec),
+    LWMSG_MESSAGE(LSA_R_AUTH_USER_PAM_FAILURE, gLsaIPCErrorSpec),
     LWMSG_MESSAGE(LSA_Q_AUTH_USER_EX, gLsaIPCAuthUserExReqSpec),
     LWMSG_MESSAGE(LSA_R_AUTH_USER_EX_SUCCESS, gLsaAuthUserInfoPtrSpec),
     LWMSG_MESSAGE(LSA_R_AUTH_USER_EX_FAILURE, gLsaIPCErrorSpec),
@@ -1283,12 +1342,6 @@ static LWMsgProtocolSpec gLsaIPCSpec[] =
     LWMSG_MESSAGE(LSA_R_PROVIDER_IO_CONTROL_SUCCESS, NULL),
     LWMSG_MESSAGE(LSA_R_PROVIDER_IO_CONTROL_SUCCESS_DATA, gLsaIPCDataBlobSpec),
     LWMSG_MESSAGE(LSA_R_PROVIDER_IO_CONTROL_FAILURE, gLsaIPCErrorSpec),
-    LWMSG_MESSAGE(LSA_Q_SET_MACHINE_SID, gLsaIPCSetMachineSid),
-    LWMSG_MESSAGE(LSA_R_SET_MACHINE_SID_SUCCESS, NULL),
-    LWMSG_MESSAGE(LSA_R_SET_MACHINE_SID_FAILURE, gLsaIPCErrorSpec),
-    LWMSG_MESSAGE(LSA_Q_SET_MACHINE_NAME, gLsaIPCSetMachineName),
-    LWMSG_MESSAGE(LSA_R_SET_MACHINE_NAME_SUCCESS, NULL),
-    LWMSG_MESSAGE(LSA_R_SET_MACHINE_NAME_FAILURE, gLsaIPCErrorSpec),
     LWMSG_MESSAGE(LSA_Q_GET_PAM_CONFIG, NULL),
     LWMSG_MESSAGE(LSA_R_GET_PAM_CONFIG_SUCCESS, gLsaIPCGetPamConfigSpec),
     LWMSG_MESSAGE(LSA_R_GET_PAM_CONFIG_FAILURE, gLsaIPCErrorSpec),
@@ -1307,6 +1360,9 @@ static LWMsgProtocolSpec gLsaIPCSpec[] =
     LWMSG_MESSAGE(LSA2_R_CLOSE_ENUM, NULL),
     LWMSG_MESSAGE(LSA2_Q_QUERY_MEMBER_OF, gLsa2IpcQueryMemberOfReqSpec),
     LWMSG_MESSAGE(LSA2_R_QUERY_MEMBER_OF, gLsa2IpcQueryMemberOfResSpec),
+    LWMSG_MESSAGE(LSA2_Q_FIND_GROUP_AND_EXPANDED_MEMBERS, gLsa2IpcFindGroupAndExpandedGroupMembersReqSpec),
+    LWMSG_MESSAGE(LSA2_R_FIND_GROUP_AND_EXPANDED_MEMBERS, gLsa2IpcFindGroupAndExpandedGroupMembersResSpec),
+    LWMSG_MESSAGE(LSA2_R_QUERY_MEMBER_OF, gLsa2IpcQueryMemberOfResSpec),
     LWMSG_MESSAGE(LSA2_Q_ADD_GROUP, gLsa2IpcAddGroupReqSpec),
     LWMSG_MESSAGE(LSA2_R_ADD_GROUP, NULL),
     LWMSG_MESSAGE(LSA2_Q_MODIFY_GROUP, gLsa2IpcModifyGroupReqSpec),
@@ -1317,6 +1373,8 @@ static LWMsgProtocolSpec gLsaIPCSpec[] =
     LWMSG_MESSAGE(LSA2_R_ADD_USER, NULL),
     LWMSG_MESSAGE(LSA2_Q_MODIFY_USER, gLsa2IpcModifyUserReqSpec),
     LWMSG_MESSAGE(LSA2_R_MODIFY_USER, NULL),
+    LWMSG_MESSAGE(LSA2_Q_GET_SMARTCARD_USER_OBJECT, NULL),
+    LWMSG_MESSAGE(LSA2_R_GET_SMARTCARD_USER_OBJECT, gLsa2IpcGetSmartCardUserObjectResSpec),
     LWMSG_PROTOCOL_END
 };
 
