@@ -133,12 +133,15 @@ typedef LW_DWORD LSA_DM_STATE_FLAGS, *PLSA_DM_STATE_FLAGS;
 #define LSA_DM_STATE_FLAG_FORCE_OFFLINE          0x00000002
 /// Whether globally offline due to media sense.
 #define LSA_DM_STATE_FLAG_MEDIA_SENSE_OFFLINE    0x00000004
+/// Whether to ignore trusts.
+#define LSA_DM_STATE_FLAG_IGNORE_ALL_TRUSTS      0x00000008
 
 #define LSA_DM_STATE_FLAGS_VALID_MASK \
     ( \
         LSA_DM_STATE_FLAG_OFFLINE_ENABLED | \
         LSA_DM_STATE_FLAG_FORCE_OFFLINE | \
         LSA_DM_STATE_FLAG_MEDIA_SENSE_OFFLINE | \
+        LSA_DM_STATE_FLAG_IGNORE_ALL_TRUSTS | \
         0 \
     )
 
@@ -194,8 +197,10 @@ typedef LW_DWORD LSA_NIS_MAP_QUERY_FLAGS;
 
 typedef LW_DWORD LSA_FIND_FLAGS, *PLSA_FIND_FLAGS;
 
-#define LSA_FIND_FLAGS_NSS   0x00000001
-#define LSA_FIND_FLAGS_LOCAL 0x00000002
+#define LSA_FIND_FLAGS_NSS        0x00000001
+#define LSA_FIND_FLAGS_LOCAL      0x00000002
+// Only supported by the LsaFindObjects function
+#define LSA_FIND_FLAGS_CACHE_ONLY 0x00000004
 
 typedef struct __LW_LSA_DATA_BLOB
 {
@@ -545,7 +550,6 @@ typedef struct __LSA_METRIC_PACK_0
     LW_UINT64 failedOpenSession;
     LW_UINT64 failedCloseSession;
     LW_UINT64 failedChangePassword;
-    LW_UINT64 unauthorizedAccesses;
 } LSA_METRIC_PACK_0, *PLSA_METRIC_PACK_0;
 
 typedef struct __LSA_METRIC_PACK_1
@@ -567,7 +571,6 @@ typedef struct __LSA_METRIC_PACK_1
     LW_UINT64 failedCloseSession;
     LW_UINT64 successfulChangePassword;
     LW_UINT64 failedChangePassword;
-    LW_UINT64 unauthorizedAccesses;
 } LSA_METRIC_PACK_1, *PLSA_METRIC_PACK_1;
 
 typedef struct __LSA_METRIC_PACK
@@ -705,6 +708,23 @@ typedef struct __LSA_AUTH_USER_PARAMS
     } pass;
 } LSA_AUTH_USER_PARAMS, *PLSA_AUTH_USER_PARAMS;
 
+typedef struct __LSA_AUTH_USER_PAM_PARAMS
+{
+    LW_DWORD dwFlags;
+    LW_PCSTR pszLoginName;
+    LW_PCSTR pszPassword;
+    LW_PCSTR pszPamSource;
+} LSA_AUTH_USER_PAM_PARAMS, *PLSA_AUTH_USER_PAM_PARAMS;
+
+#define LSA_AUTH_USER_PAM_FLAG_RETURN_MESSAGE   0x00000001
+#define LSA_AUTH_USER_PAM_FLAG_SMART_CARD       0x00000002
+
+typedef struct __LSA_AUTH_USER_PAM_INFO
+{
+    LW_PSTR    pszMessage;
+    LW_BOOLEAN bOnlineLogon;
+} LSA_AUTH_USER_PAM_INFO, *PLSA_AUTH_USER_PAM_INFO;
+
 #define LSA_MAX_SID_SUB_AUTHORITIES  15
 
 typedef struct __LSA_SID_ATTRIB
@@ -781,6 +801,10 @@ typedef struct _LSA_PAM_CONFIG
     LW_DWORD dwLogLevel;
     LW_BOOLEAN bLsaPamDisplayMOTD;
     LW_PSTR pszAccessDeniedMessage;
+    LW_DWORD dwNumSmartCardServices;
+    LW_PSTR *ppszSmartCardServices;
+    LW_DWORD dwNumSmartCardPromptGecos;
+    LW_PSTR *ppszSmartCardPromptGecos;
 } LSA_PAM_CONFIG, *PLSA_PAM_CONFIG;
 
 /**
@@ -1380,7 +1404,8 @@ LW_DWORD
 LsaAuthenticateUser(
     LW_HANDLE hLsaConnection,
     LW_PCSTR pszLoginName,
-    LW_PCSTR pszPassword
+    LW_PCSTR pszPassword,
+    LW_PSTR* ppszMessage
     );
 
 LW_DWORD
@@ -1400,6 +1425,18 @@ LsaAuthenticateUserEx(
     LW_OUT PLSA_AUTH_USER_INFO* ppUserInfo
     );
 
+LW_DWORD
+LsaAuthenticateUserPam(
+    LW_IN LW_HANDLE hLsaConnection,
+    LW_IN LSA_AUTH_USER_PAM_PARAMS* pParams,
+    LW_OUT PLSA_AUTH_USER_PAM_INFO* ppPamAuthInfo
+    );
+
+LW_VOID
+LsaFreeAuthUserPamInfo(
+    LW_IN LW_OUT PLSA_AUTH_USER_PAM_INFO pPamAuthInfo
+    );
+
 LW_DWORD
 LsaValidateUser(
     LW_HANDLE hLsaConnection,
@@ -1476,6 +1513,11 @@ LsaFreeStatus(
     PLSASTATUS pLsaStatus
     );
 
+DWORD
+LsaReadVersionFile(
+    PLSA_VERSION pVersion
+    );
+
 /**
  * @ingroup connection
  * @brief Closes connection to LSASS server
@@ -1491,6 +1533,24 @@ LsaCloseServer(
     LW_HANDLE hConnection
     );
 
+/**
+ * @ingroup connection
+ * @brief Frees a connection to LSASS server
+ *
+ * This frees the local resources associated with a connection, but does not
+ * explicitly notify the server. The server will discover the connection is
+ * closed when it tries to read from the socket. In general, LsaCloseServer is
+ * a cleaner way to close a connection.
+ *
+ * @param[in,out] hConnection the connection handle to close
+ * @retval LW_ERROR_SUCCESS success
+ * @retval EINVAL the handle was invalid
+ */
+LW_DWORD
+LsaDropServer(
+    LW_HANDLE hConnection
+    );
+
 LW_DWORD
 LsaGetErrorMessageForLoggingEvent(
     LW_DWORD dwError,
@@ -1601,22 +1661,29 @@ LsaFreePamConfig(
     LW_IN PLSA_PAM_CONFIG pPamConfig
     );
 
+enum _LSA_QUERY_TYPE
+{
+	LSA_QUERY_TYPE_UNDEFINED     = 0,
+	LSA_QUERY_TYPE_BY_DN         = 1,
+	LSA_QUERY_TYPE_BY_SID        = 2,
+	LSA_QUERY_TYPE_BY_NT4        = 3,
+	LSA_QUERY_TYPE_BY_UPN        = 4,
+	LSA_QUERY_TYPE_BY_ALIAS      = 5,
+	LSA_QUERY_TYPE_BY_UNIX_ID    = 6,
+	LSA_QUERY_TYPE_BY_NAME       = 7
+};
 typedef LW_UINT8 LSA_QUERY_TYPE, *PLSA_QUERY_TYPE;
-#define LSA_QUERY_TYPE_UNDEFINED      0
-#define LSA_QUERY_TYPE_BY_DN          1
-#define LSA_QUERY_TYPE_BY_SID         2
-#define LSA_QUERY_TYPE_BY_NT4         3
-#define LSA_QUERY_TYPE_BY_UPN         4
-#define LSA_QUERY_TYPE_BY_ALIAS       5
-#define LSA_QUERY_TYPE_BY_UNIX_ID     6
-#define LSA_QUERY_TYPE_BY_NAME        7
 
+enum _LSA_OBJECT_TYPE
+{
+    LSA_OBJECT_TYPE_UNDEFINED = 0,
+    LSA_OBJECT_TYPE_GROUP     = 1,
+    LSA_OBJECT_TYPE_USER      = 2,
+    LSA_OBJECT_TYPE_DOMAIN    = 3,
+    LSA_OBJECT_TYPE_COMPUTER  = 4
+};
 typedef LW_UINT8 LSA_OBJECT_TYPE, *PLSA_OBJECT_TYPE;
-#define LSA_OBJECT_TYPE_UNDEFINED 0
-#define LSA_OBJECT_TYPE_USER      2
-#define LSA_OBJECT_TYPE_GROUP     1
-#define LSA_OBJECT_TYPE_COMPUTER  4
-#define LSA_OBJECT_TYPE_DOMAIN    3
+
 
 typedef struct __LSA_SECURITY_OBJECT_VERSION_INFO
 {
@@ -1883,6 +1950,18 @@ LsaQueryExpandedGroupMembers(
     LW_OUT PLSA_SECURITY_OBJECT** pppMembers
     );
 
+LW_DWORD
+LsaFindGroupAndExpandedMembers(
+    LW_IN LW_HANDLE hLsa,
+    LW_PCSTR pszTargetProvider,
+    LW_IN LSA_FIND_FLAGS FindFlags,
+    LW_IN LSA_QUERY_TYPE QueryType,
+    LW_IN LSA_QUERY_ITEM QueryItem,
+    LW_OUT PLSA_SECURITY_OBJECT* ppGroupObject,
+    LW_OUT LW_PDWORD pdwMemberObjectCount,
+    LW_OUT PLSA_SECURITY_OBJECT** pppMemberObjects
+    );
+
 LW_DWORD
 LsaModifyUser2(
     LW_HANDLE hLsaConnection,
@@ -1918,6 +1997,13 @@ LsaAddUser2(
     PLSA_USER_ADD_INFO pUserAddInfo
     );
 
+LW_DWORD
+LsaGetSmartCardUserObject(
+    LW_IN LW_HANDLE hLsaConnection,
+    LW_OUT PLSA_SECURITY_OBJECT* ppObject,
+    LW_OUT LW_PSTR* ppszSmartCardReader
+    );
+
 #endif /* __LSA_H__ */
 
 
