@@ -1,6 +1,34 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
+
+/*
+ * Copyright Likewise Software    2004-2008
+ * All rights reserved.
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the license, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.  You should have received a copy
+ * of the GNU Lesser General Public License along with this program.  If
+ * not, see <http://www.gnu.org/licenses/>.
+ *
+ * LIKEWISE SOFTWARE MAKES THIS SOFTWARE AVAILABLE UNDER OTHER LICENSING
+ * TERMS AS WELL.  IF YOU HAVE ENTERED INTO A SEPARATE LICENSE AGREEMENT
+ * WITH LIKEWISE SOFTWARE, THEN YOU MAY ELECT TO USE THE SOFTWARE UNDER THE
+ * TERMS OF THAT SOFTWARE LICENSE AGREEMENT INSTEAD OF THE TERMS OF THE GNU
+ * LESSER GENERAL PUBLIC LICENSE, NOTWITHSTANDING THE ABOVE NOTICE.  IF YOU
+ * HAVE QUESTIONS, OR WISH TO REQUEST A COPY OF THE ALTERNATE LICENSING
+ * TERMS OFFERED BY LIKEWISE SOFTWARE, PLEASE CONTACT LIKEWISE SOFTWARE AT
+ * license@likewisesoftware.com
+ */
+
+
 
 /*
  * Copyright (C) Likewise Software. All rights reserved.
@@ -20,7 +48,11 @@
 #ifndef __LSA_UTILS_H__
 #define __LSA_UTILS_H__
 
+#include "lsaipc.h"
 #include "lsalist.h"
+#include <lw/ntstatus.h>
+#include <lw/winerror.h>
+#include <lw/security-types.h>
 
 #ifndef LW_ENDIAN_SWAP16
 
@@ -50,57 +82,48 @@
 
 #ifndef WIN32
 
+#ifndef BAIL_ON_NT_STATUS
+#define BAIL_ON_NT_STATUS(err) \
+    do { \
+        if ((err) != STATUS_SUCCESS) { \
+            LSA_LOG_DEBUG("Error at %s:%d [code: %X]", \
+                          __FILE__, __LINE__, (err));  \
+		    goto error; \
+	    } \
+    } while (0);
+#endif
+
 #define BAIL_ON_LSA_ERROR(dwError) \
     if (dwError) {\
-       LSA_LOG_DEBUG("Error at %s:%d [code: %d]", __FILE__, __LINE__, dwError); \
+       LSA_LOG_DEBUG("Error code: %u (symbol: %s)", dwError, LSA_SAFE_LOG_STRING(LwWin32ExtErrorToName(dwError))); \
        goto error;                 \
     }
 
+#define BAIL_ON_LWMSG_ERROR(pAssoc, dwError) \
+    do { \
+        if (dwError) \
+        { \
+           (dwError) = LsaTranslateLwMsgError(pAssoc, dwError, __FILE__, __LINE__); \
+           goto error; \
+        } \
+    } while (0)
+
 #define BAIL_WITH_LSA_ERROR(_newerror_) \
     do {dwError = (_newerror_); BAIL_ON_LSA_ERROR(dwError);} while (0)
 
 #endif
 
-#define LSA_SAFE_FREE_STRING(str) \
-        do {                      \
-           if (str) {             \
-              LsaFreeString(str); \
-              (str) = NULL;       \
-           }                      \
-        } while(0);
-
-#define LSA_SAFE_CLEAR_FREE_STRING(str)       \
-        do {                                  \
-           if (str) {                         \
-              if (*str) {                     \
-                 memset(str, 0, strlen(str)); \
-              }                               \
-              LsaFreeString(str);             \
-              (str) = NULL;                   \
-           }                                  \
-        } while(0);
-
-#define LSA_SAFE_FREE_MEMORY(mem) \
-        do {                      \
-           if (mem) {             \
-              LsaFreeMemory(mem); \
-              (mem) = NULL;       \
-           }                      \
-        } while(0);
-
-#define LSA_SAFE_FREE_STRING_ARRAY(ppszArray)               \
-        do {                                                \
-           if (ppszArray) {                                 \
-               LsaFreeNullTerminatedStringArray(ppszArray); \
-               (ppszArray) = NULL;                          \
-           }                                                \
-        } while (0);
-
-#define IsNullOrEmptyString(str) (!(str) || !(*(str)))
-
 /*
  * Logging
  */
+
+typedef LW_VOID (*PFN_LSA_LOG_MESSAGE)(
+    LW_HANDLE hLog,
+    LsaLogLevel logLevel,
+    LW_PCSTR pszFormat,
+    va_list msgList
+    );
+
 #if defined(LW_ENABLE_THREADS)
 
 extern pthread_mutex_t gLogLock;
@@ -110,8 +133,8 @@ extern pthread_mutex_t gLogLock;
 
 #define _LSA_LOG_WITH_THREAD(Level, Format, ...) \
     _LSA_LOG_MESSAGE(Level, \
-                     "0x%x:" Format, \
-                     (unsigned int)pthread_self(), \
+                     "0x%lx:" Format, \
+                     (unsigned long)pthread_self(), \
                      ## __VA_ARGS__)
 
 #else
@@ -179,6 +202,21 @@ extern PFN_LSA_LOG_MESSAGE gpfnLogger;
 #define LSA_LOG_DEBUG(szFmt, ...) \
     _LSA_LOG_IF(LSA_LOG_LEVEL_DEBUG, szFmt, ## __VA_ARGS__)
 
+#define LSA_LOG_TRACE(szFmt, ...) \
+    _LSA_LOG_IF(LSA_LOG_LEVEL_TRACE, szFmt, ## __VA_ARGS__)
+
+#define LSA_LOG_ERROR_API_FAILED(hServer, dwError, szFmt, ...) \
+    LSA_LOG_ERROR("Failed to " szFmt " -> error = %u, symbol = %s, client pid = %ld", \
+         ## __VA_ARGS__, \
+         dwError, \
+         LSA_SAFE_LOG_STRING(LwWin32ExtErrorToName(dwError)), \
+         (long)(hServer? ((PLSA_SRV_API_STATE)hServer)->peerPID : getpid()))
+
+#define LSA_LOG_VERBOSE_ENTRY_NOT_FOUND(hServer, dwError, szFmt, ...) \
+    LSA_LOG_VERBOSE("Failed to " szFmt " -> error = no such entry, client pid = %ld", \
+         ## __VA_ARGS__, \
+         (long)(hServer? ((PLSA_SRV_API_STATE)hServer)->peerPID : getpid()))
+
 // Like assert() but also calls LSA_LOG.
 #define _LSA_ASSERT(Expression, Action) \
     do { \
@@ -190,7 +228,7 @@ extern PFN_LSA_LOG_MESSAGE gpfnLogger;
     } while (0)
 #define _LSA_ASSERT_OR_BAIL(Expression, dwError, Action) \
     _LSA_ASSERT(Expression, \
-                (dwError) = LSA_ERROR_INTERNAL; \
+                (dwError) = LW_ERROR_INTERNAL; \
                 Action ; \
                 BAIL_ON_LSA_ERROR(dwError))
 #ifdef NDEBUG
@@ -207,53 +245,6 @@ extern PFN_LSA_LOG_MESSAGE gpfnLogger;
 #define LSA_IS_XOR(Expression1, Expression2) \
     (!!(Expression1) ^ !!(Expression2))
 
-#define LSA_SAFE_FREE_LOGIN_NAME_INFO(pLoginNameInfo) \
-    do { \
-        if (pLoginNameInfo) \
-        { \
-            LsaFreeNameInfo(pLoginNameInfo); \
-            (pLoginNameInfo) = NULL; \
-        } \
-    } while(0);
-
-#if defined(LW_ENABLE_THREADS)
-
-extern pthread_mutex_t gTraceLock;
-
-#define LSA_LOCK_TRACER   pthread_mutex_lock(&gTraceLock)
-#define LSA_UNLOCK_TRACER pthread_mutex_unlock(&gTraceLock)
-
-#else
-
-#define LSA_LOCK_TRACER
-#define LSA_UNLOCK_TRACER
-
-#endif
-
-#define LSA_TRACE_BEGIN_FUNCTION(traceFlagArray, dwNumFlags)  \
-    do {                                                      \
-        LSA_LOCK_TRACER;                                      \
-        if (LsaTraceIsAllowed(traceFlagArray, dwNumFlags)) {  \
-            LSA_LOG_ALWAYS("Begin %s() %s:%d",                \
-                           __FUNCTION__,                      \
-                           __FILE__,                          \
-                           __LINE__);                         \
-        }                                                     \
-        LSA_UNLOCK_TRACER;                                    \
-    } while (0)
-
-#define LSA_TRACE_END_FUNCTION(traceFlagArray, dwNumFlags)   \
-    do {                                                     \
-        LSA_LOCK_TRACER;                                     \
-        if (LsaTraceIsAllowed(traceFlagArray, dwNumFlags)) { \
-            LSA_LOG_ALWAYS("End %s() %s:%d",                 \
-                           __FUNCTION__,                     \
-                           __FILE__,                         \
-                           __LINE__);                        \
-        }                                                    \
-        LSA_UNLOCK_TRACER;                                   \
-    } while (0)
-
 #if defined(LW_ENABLE_THREADS)
 
 extern pthread_mutex_t gTraceLock;
@@ -313,20 +304,20 @@ typedef struct _DNS_FQDN
 #define SEC_BUFFER_S_CONVERT(_sb_,_sbs_) do{(_sb_)->length = (_sbs_)->length;(_sb_)->maxLength=(_sbs_)->maxLength;(_sb_)->buffer = (_sbs_)->buffer;} while (0)
 
 #define BAIL_ON_INVALID_STRING(pszParam)          \
-        if (IsNullOrEmptyString(pszParam)) {      \
-           dwError = LSA_ERROR_INVALID_PARAMETER; \
+        if (LW_IS_NULL_OR_EMPTY_STR(pszParam)) {      \
+           dwError = LW_ERROR_INVALID_PARAMETER; \
            BAIL_ON_LSA_ERROR(dwError);            \
         }
 
 #define BAIL_ON_INVALID_HANDLE(hParam)            \
         if (hParam == (HANDLE)NULL) {             \
-           dwError = LSA_ERROR_INVALID_PARAMETER; \
+           dwError = LW_ERROR_INVALID_PARAMETER; \
            BAIL_ON_LSA_ERROR(dwError);            \
         }
 
 #define BAIL_ON_INVALID_POINTER(p)                \
         if (NULL == p) {                          \
-           dwError = LSA_ERROR_INVALID_PARAMETER; \
+           dwError = LW_ERROR_INVALID_PARAMETER; \
            BAIL_ON_LSA_ERROR(dwError);            \
         }
 
@@ -334,42 +325,14 @@ typedef struct _DNS_FQDN
 //S-<revision>-<authority>-<domain_computer_id>-<relative ID>
 //example: S-1-5-32-546 (Guests)
 //See http://support.microsoft.com/kb/243330
-
+/*
 #define WELLKNOWN_SID_DOMAIN_USER_GROUP_RID 513
 
 typedef struct __LSA_SECURITY_IDENTIFIER {
     UCHAR* pucSidBytes;  //byte representation of multi-byte Security Identification Descriptor
     DWORD dwByteLength;
 } LSA_SECURITY_IDENTIFIER, *PLSA_SECURITY_IDENTIFIER;
-
-/*
- * Config parsing callbacks
- */
-typedef DWORD (*PFNCONFIG_START_SECTION)(
-                        PCSTR    pszSectionName,
-                        PVOID    pData,
-                        PBOOLEAN pbSkipSection,
-                        PBOOLEAN pbContinue
-                        );
-
-typedef DWORD (*PFNCONFIG_COMMENT)(
-                        PCSTR    pszComment,
-                        PVOID    pData,
-                        PBOOLEAN pbContinue
-                        );
-
-typedef DWORD (*PFNCONFIG_NAME_VALUE_PAIR)(
-                        PCSTR    pszName,
-                        PCSTR    pszValue,
-                        PVOID    pData,
-                        PBOOLEAN pbContinue
-                        );
-
-typedef DWORD (*PFNCONFIG_END_SECTION)(
-                        PCSTR pszSectionName,
-                        PVOID    pData,
-                        PBOOLEAN pbContinue
-                        );
+*/
 
 typedef struct __LSA_BIT_VECTOR
 {
@@ -439,21 +402,6 @@ typedef struct _SEC_BUFFER_RELATIVE {
     ULONG  offset;
 } SEC_BUFFER_RELATIVE, *PSEC_BUFFER_RELATIVE;
 
-typedef struct _SEC_BUFFER {
-    USHORT length;
-    USHORT maxLength;
-    PBYTE  buffer;
-} SEC_BUFFER, *PSEC_BUFFER;
-
-/* static buffer secbufer */
-#define S_BUFLEN 24
-
-typedef struct _SEC_BUFFER_S {
-    USHORT length;
-    USHORT maxLength;
-    BYTE buffer[S_BUFLEN];
-} SEC_BUFFER_S, *PSEC_BUFFER_S;
-
 typedef struct _OID {
     DWORD length;
     PVOID *elements;
@@ -476,42 +424,6 @@ typedef struct _LSA_STRING_BUFFER
 typedef struct passwd * passwd_ptr_t;
 typedef struct group  * group_ptr_t;
 
-typedef enum
-{
-    NameType_NT4 = 0,
-    NameType_UPN = 1,
-    NameType_Alias
-} ADLogInNameType;
-
-typedef struct __LSA_LOGIN_NAME_INFO
-{
-    ADLogInNameType nameType;
-    PSTR  pszDomainNetBiosName;
-    PSTR  pszFullDomainName;
-    PSTR  pszName;
-    PSTR  pszObjectSid;
-} LSA_LOGIN_NAME_INFO, *PLSA_LOGIN_NAME_INFO;
-
-typedef struct __LSADATACOORDINATES {
-    DWORD offset;
-    DWORD length;
-} LSADATACOORDINATES, *PLSADATACOORDINATES;
-
-typedef struct __LSACREDENTIALHEADER {
-    LSADATACOORDINATES login;
-    LSADATACOORDINATES passwd;
-    LSADATACOORDINATES old_passwd;
-} LSACREDENTIALHEADER, *PLSACREDENTIALHEADER;
-
-typedef struct __LSAERRORRECORDHEADER {
-    DWORD              errorCode;
-    LSADATACOORDINATES message;
-} LSAERRORRECORDHEADER, *PLSAERRORRECORDHEADER;
-
-typedef struct __LSASESSIONHEADER {
-    LSADATACOORDINATES login;
-} LSASESSIONHEADER, *PLSASESSIONHEADER;
-
 typedef enum
 {
     LsaMetricSuccessfulAuthentications    =  0,
@@ -541,6 +453,36 @@ typedef struct __LSA_NIS_NICKNAME
     PSTR pszMapName;
 } LSA_NIS_NICKNAME, *PLSA_NIS_NICKNAME;
 
+typedef DWORD (*PFLSA_CACHE_HASH) (PVOID pKey, DWORD dwIndex, PVOID pData);
+typedef BOOLEAN (*PFLSA_CACHE_EQUAL) (PVOID pKey1, PVOID pKey2, DWORD dwIndex, PVOID pData);
+typedef PVOID (*PFLSA_CACHE_GETKEY) (PVOID pEntry, DWORD dwIndex, PVOID pData);
+typedef DWORD (*PFLSA_CACHE_MISS) (PVOID pKey, DWORD dwIndex, PVOID pData, PVOID* ppEntry);
+typedef DWORD (*PFLSA_CACHE_KICK) (PVOID pEntry, PVOID pData);
+
+typedef struct __LSA_CACHE_ENTRY
+{
+    DWORD dwRefCount;
+} LSA_CACHE_ENTRY, *PLSA_CACHE_ENTRY;
+
+typedef struct __LSA_CACHE
+{
+    DWORD dwNumKeys;
+    DWORD dwNumBuckets;
+    PVOID* ppEntries;
+    PFLSA_CACHE_HASH pfHash;
+    PFLSA_CACHE_EQUAL pfEqual;
+    PFLSA_CACHE_GETKEY pfGetKey;
+    PFLSA_CACHE_MISS pfMiss;
+    PFLSA_CACHE_KICK pfKick;
+    PVOID pData;
+    DWORD dwNumHashMisses;
+    DWORD dwNumFullMisses;
+    DWORD dwNumKicks;
+    DWORD dwNumUsedBuckets;
+    DWORD dwNumCollisions;
+} LSA_CACHE, *PLSA_CACHE;
+
+
 #if !defined(HAVE_STRTOLL)
 
 long long int
@@ -577,19 +519,6 @@ void
 LsaFreeStringBufferContents(
         LSA_STRING_BUFFER *pBuffer);
 
-DWORD
-LsaAllocateMemory(
-    DWORD dwSize,
-    PVOID * ppMemory
-    );
-
-DWORD
-LsaReallocMemory(
-    PVOID  pMemory,
-    PVOID * ppNewMemory,
-    DWORD dwSize
-    );
-
 DWORD
 LsaAppendAndFreePtrs(
     IN OUT PDWORD pdwDestCount,
@@ -598,125 +527,6 @@ LsaAppendAndFreePtrs(
     IN OUT PVOID** pppAppendPtrArray
     );
 
-void
-LsaFreeMemory(
-    PVOID pMemory
-    );
-
-DWORD
-LsaAllocateString(
-    PCSTR pszInputString,
-    PSTR *ppszOutputString
-    );
-
-
-void
-LsaFreeString(
-    PSTR pszString
-    );
-
-void
-LsaStripWhitespace(
-    PSTR pszString,
-    BOOLEAN bLeading,
-    BOOLEAN bTrailing
-    );
-
-DWORD
-LsaStrIsAllSpace(
-    PCSTR pszString,
-    PBOOLEAN pbIsAllSpace
-    );
-
-void
-LsaStrToUpper(
-    PSTR pszString
-    );
-
-void
-LsaStrnToUpper(
-    PSTR  pszString,
-    DWORD dwLen
-    );
-
-void
-LsaStrToLower(
-    PSTR pszString
-    );
-
-void
-LsaStrnToLower(
-    PSTR  pszString,
-    DWORD dwLen
-    );
-
-DWORD
-LsaEscapeString(
-    PSTR pszOrig,
-    PSTR * ppszEscapedString
-    );
-
-DWORD
-LsaStrndup(
-    PCSTR pszInputString,
-    size_t size,
-    PSTR * ppszOutputString
-    );
-
-DWORD
-LsaAllocateStringPrintf(
-    PSTR* ppszOutputString,
-    PCSTR pszFormat,
-    ...
-    );
-
-DWORD
-LsaAllocateStringPrintfV(
-    PSTR*   ppszOutputString,
-    PCSTR   pszFormat,
-    va_list args
-    );
-
-VOID
-LsaStrCharReplace(
-    PSTR pszStr,
-    CHAR oldCh,
-    CHAR newCh);
-
-// If pszInputString == NULL, then *ppszOutputString = NULL
-DWORD
-LsaStrDupOrNull(
-    PCSTR pszInputString,
-    PSTR *ppszOutputString
-    );
-
-
-// If pszInputString == NULL, return "", else return pszInputString
-// The return value does not need to be freed.
-PCSTR
-LsaEmptyStrForNull(
-    PCSTR pszInputString
-    );
-
-
-void
-LsaStrChr(
-    PCSTR pszInputString,
-    CHAR c,
-    PSTR *pszOutputString
-    );
-
-void
-LsaFreeStringArray(
-    PSTR * ppStringArray,
-    DWORD dwCount
-    );
-
-void
-LsaFreeNullTerminatedStringArray(
-    PSTR * ppStringArray
-    );
-
 VOID
 LsaPrincipalNonRealmToLower(
     IN OUT PSTR pszPrincipal
@@ -786,6 +596,11 @@ LsaDLinkedListForEach(
     PVOID                pUserData
     );
 
+DWORD
+LsaDLinkedListLength(
+    PDLINKEDLIST pList
+    );
+
 VOID
 LsaDLinkedListFree(
     PDLINKEDLIST pList
@@ -834,22 +649,34 @@ LsaHashCreate(
     LSA_HASH_TABLE** ppResult
     );
 
+size_t
+LsaHashGetKeyCount(
+    PLSA_HASH_TABLE pTable
+    );
+
+void
+LsaHashRemoveAll(
+        LSA_HASH_TABLE* pResult);
+
 void
 LsaHashSafeFree(
-        LSA_HASH_TABLE** ppResult);
+    LSA_HASH_TABLE** ppResult
+    );
 
 DWORD
 LsaHashSetValue(
-        LSA_HASH_TABLE *pTable,
-        PVOID  pKey,
-        PVOID  pValue);
+    LSA_HASH_TABLE *pTable,
+    PVOID  pKey,
+    PVOID  pValue
+    );
 
-//Returns ENOENT if pKey is not in the table
+//Returns ERROR_NOT_FOUND if pKey is not in the table
 DWORD
 LsaHashGetValue(
-        LSA_HASH_TABLE *pTable,
-        PCVOID  pKey,
-        PVOID* ppValue);
+    LSA_HASH_TABLE *pTable,
+    PCVOID  pKey,
+    PVOID* ppValue
+    );
 
 BOOLEAN
 LsaHashExists(
@@ -866,33 +693,49 @@ LsaHashCopy(
 //Invalidates all iterators
 DWORD
 LsaHashResize(
-        LSA_HASH_TABLE *pTable,
-        size_t sTableSize);
+    LSA_HASH_TABLE *pTable,
+    size_t sTableSize
+    );
 
 DWORD
 LsaHashGetIterator(
-        LSA_HASH_TABLE *pTable,
-        LSA_HASH_ITERATOR *pIterator);
+    LSA_HASH_TABLE *pTable,
+    LSA_HASH_ITERATOR *pIterator
+    );
 
 // returns NULL after passing the last entry
 LSA_HASH_ENTRY *
 LsaHashNext(
-        LSA_HASH_ITERATOR *pIterator
-        );
+    LSA_HASH_ITERATOR *pIterator
+    );
 
 DWORD
 LsaHashRemoveKey(
-        LSA_HASH_TABLE *pTable,
-        PVOID  pKey);
+    LSA_HASH_TABLE *pTable,
+    PVOID  pKey
+    );
 
 int
 LsaHashCaselessStringCompare(
-        PCVOID str1,
-        PCVOID str2);
+    PCVOID str1,
+    PCVOID str2
+    );
 
 size_t
-LsaHashCaselessString(
-        PCVOID str);
+LsaHashCaselessStringHash(
+    PCVOID str
+    );
+
+int
+LsaHashPVoidCompare(
+    IN PCVOID pvData1,
+    IN PCVOID pvData2
+    );
+
+size_t
+LsaHashPVoidHash(
+    IN PCVOID pvData
+    );
 
 VOID
 LsaHashFreeStringKey(
@@ -916,6 +759,18 @@ LsaCheckSockExists(
     PBOOLEAN pbFileExists
     );
 
+DWORD
+LsaCheckLinkExists(
+    PSTR pszPath,
+    PBOOLEAN pbFileExists
+    );
+
+DWORD
+LsaCheckFileOrLinkExists(
+    PSTR pszPath,
+    PBOOLEAN pbExists
+    );
+
 DWORD
 LsaMoveFile(
     PCSTR pszSrcPath,
@@ -974,7 +829,7 @@ LsaCheckDirectoryExists(
 
 DWORD
 LsaCreateDirectory(
-    PSTR pszPath,
+    PCSTR pszPath,
     mode_t dwFileMode
     );
 
@@ -1040,14 +895,6 @@ LsaGetLibDirPath(
     PSTR* ppszPath
     );
 
-DWORD
-LsaCoalesceGroupInfoList(
-    PVOID** pppGroupInfoList,
-    PDWORD  pdwNumGroupsFound,
-    PVOID** pppGroupInfoList_accumulate,
-    PDWORD  pdwTotalNumGroupsFound
-    );
-
 DWORD
 LsaValidateGroupInfoLevel(
     DWORD dwGroupInfoLevel
@@ -1064,14 +911,6 @@ LsaValidateGroupInfo(
     DWORD dwGroupInfoLevel
     );
 
-DWORD
-LsaCoalesceNSSArtefactInfoList(
-    PVOID** pppNSSArtefactInfoList,
-    PDWORD  pdwNumNSSArtefactsFound,
-    PVOID** pppNSSArtefactInfoList_accumulate,
-    PDWORD  pdwTotalNumNSSArtefactsFound
-    );
-
 DWORD
 LsaValidateUserInfo(
     PVOID pUserInfo,
@@ -1083,41 +922,16 @@ LsaValidateUserInfoLevel(
     DWORD dwUserInfoLevel
     );
 
-DWORD
-LsaCoalesceUserInfoList(
-    PVOID** pppUserInfoList,
-    PDWORD  pdwNumUsersFound,
-    PVOID** pppUserInfoList_accumulate,
-    PDWORD  pdwTotalNumUsersFound
-    );
-
 DWORD
 LsaValidateUserName(
     PCSTR pszName
     );
 
-#define LSA_DOMAIN_SEPARATOR_DEFAULT    '\\'
-
-CHAR
-LsaGetDomainSeparator(
-    VOID
-    );
-
 DWORD
-LsaSetDomainSeparator(
-    CHAR chValue
-    );
-
-DWORD
-LsaCrackDomainQualifiedName(
-    PCSTR pszId,
-    PCSTR pszDefaultDomain,
-    PLSA_LOGIN_NAME_INFO* ppNameInfo
-    );
-
-void
-LsaFreeNameInfo(
-    PLSA_LOGIN_NAME_INFO pNameInfo
+LsaAllocateGroupInfo(
+    PVOID *ppDstGroupInfo,
+    DWORD dwLevel,
+    PVOID pSrcGroupInfo
     );
 
 DWORD
@@ -1175,25 +989,124 @@ LsaModifyUser_SetPasswordMustExpire(
     );
 
 DWORD
-LsaModifyUser_SetAccountExpiryDate(
+LsaModifyUser_SetExpiryDate(
     PLSA_USER_MOD_INFO pUserModInfo,
     PCSTR pszDate
     );
 
+DWORD
+LsaModifyUser_SetPrimaryGroup(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszGid
+    );
+
+DWORD
+LsaModifyUser_SetHomedir(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszHomedir
+    );
+
+DWORD
+LsaModifyUser_SetShell(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszShell
+    );
+
+DWORD
+LsaModifyUser_SetGecos(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszGecos
+    );
+
+DWORD
+LsaModifyUser_SetPassword(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszPassword
+    );
+
+DWORD
+LsaModifyUser_SetNtPasswordHash(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszHash
+    );
+
+DWORD
+LsaModifyUser_SetLmPasswordHash(
+    PLSA_USER_MOD_INFO pUserModInfo,
+    PCSTR pszHash
+    );
+
 void
 LsaFreeUserModInfo(
     PLSA_USER_MOD_INFO pUserModInfo
     );
 
+void
+LsaFreeUserModInfo2(
+    PLSA_USER_MOD_INFO_2 pUserModInfo
+    );
+
+void
+LsaFreeUserAddInfo(
+    PLSA_USER_ADD_INFO pUserAddInfo
+    );
+
 DWORD
-LsaParseConfigFile(
-    PCSTR                     pszFilePath,
-    DWORD                     dwOptions,
-    PFNCONFIG_START_SECTION   pfnStartSectionHandler,
-    PFNCONFIG_COMMENT         pfnCommentHandler,
-    PFNCONFIG_NAME_VALUE_PAIR pfnNameValuePairHandler,
-    PFNCONFIG_END_SECTION     pfnEndSectionHandler,
-    PVOID                     pData
+LsaBuildGroupModInfo(
+    gid_t gid,
+    PLSA_GROUP_MOD_INFO *ppGroupModInfo
+    );
+
+DWORD
+LsaModifyGroup_AddMembers(
+    PLSA_GROUP_MOD_INFO pGroupModInfo,
+    PVOID pData
+    );
+
+DWORD
+LsaModifyGroup_RemoveMembers(
+    PLSA_GROUP_MOD_INFO pGroupModInfo,
+    PVOID pData
+    );
+
+void
+LsaFreeGroupModInfo(
+    PLSA_GROUP_MOD_INFO pGroupModInfo
+    );
+
+void
+LsaFreeGroupModInfo2(
+    PLSA_GROUP_MOD_INFO_2 pGroupModInfo
+    );
+
+void
+LsaFreeGroupAddInfo(
+    PLSA_GROUP_ADD_INFO pGroupAddInfo
+    );
+
+void
+LsaFreeIpcNameSidsList(
+    PLSA_FIND_NAMES_BY_SIDS pNameSidsList
+    );
+
+void
+LsaFreeIpcUserInfoList(
+    PLSA_USER_INFO_LIST pUserIpcInfoList
+    );
+
+VOID
+LsaSrvFreeIpcMetriPack(
+    PLSA_METRIC_PACK pMetricPack
+    );
+
+void
+LsaFreeIpcGroupInfoList(
+    PLSA_GROUP_INFO_LIST pGroupIpcInfoList
+    );
+
+void
+LsaFreeIpcNssArtefactInfoList(
+    PLSA_NSS_ARTEFACT_INFO_LIST pNssArtefactIpcInfoList
     );
 
 DWORD
@@ -1201,6 +1114,30 @@ LsaDnsGetHostInfo(
     PSTR* ppszHostname
     );
 
+DWORD
+LsaWc16sHash(
+    PCWSTR  pwszStr,
+    PDWORD  pdwResult
+    );
+
+DWORD
+LsaStrHash(
+    PCSTR   pszStr,
+    PDWORD  pdwResult
+    );
+
+DWORD
+LsaHashToWc16s(
+    DWORD   dwHash,
+    PWSTR  *ppwszHashStr
+    );
+
+DWORD
+LsaHashToStr(
+    DWORD   dwHash,
+    PSTR   *ppszHashStr
+    );
+
 VOID
 LsaDnsFreeFQDNList(
     PDNS_FQDN* ppFQDNList,
@@ -1292,90 +1229,6 @@ LsaTraceShutdown(
     VOID
     );
 
-DWORD
-LsaAllocSecurityIdentifierFromBinary(
-    UCHAR* sidBytes,
-    DWORD dwSidBytesLength,
-    PLSA_SECURITY_IDENTIFIER* ppSecurityIdentifier
-    );
-
-DWORD
-LsaAllocSecurityIdentifierFromString(
-    PCSTR pszSidString,
-    PLSA_SECURITY_IDENTIFIER* ppSecurityIdentifier
-    );
-
-VOID
-LsaFreeSecurityIdentifier(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier
-    );
-
-DWORD
-LsaGetSecurityIdentifierRid(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    PDWORD pdwRid
-    );
-
-DWORD
-LsaSetSecurityIdentifierRid(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    DWORD dwRid
-    );
-
-DWORD
-LsaGetSecurityIdentifierHashedRid(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    PDWORD dwHashedRid
-    );
-
-// The UID is a DWORD constructued using a non-cryptographic hash
-// of the User's domain SID and user RID.
-DWORD
-LsaHashSecurityIdentifierToId(
-    IN PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    OUT PDWORD pdwId
-    );
-
-DWORD
-LsaHashSidStringToId(
-    IN PCSTR pszSidString,
-    OUT PDWORD pdwId
-    );
-
-DWORD
-LsaGetSecurityIdentifierBinary(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    UCHAR** ppucSidBytes,
-    DWORD* pdwSidBytesLength
-    );
-
-DWORD
-LsaGetSecurityIdentifierString(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    PSTR* ppszSidStr
-    );
-
-DWORD
-LsaSidBytesToString(
-    UCHAR* pucSidBytes,
-    DWORD dwSidBytesLength,
-    PSTR* pszSidString
-    );
-
-DWORD
-LsaGetDomainSecurityIdentifier(
-    PLSA_SECURITY_IDENTIFIER pSecurityIdentifier,
-    PLSA_SECURITY_IDENTIFIER* ppDomainSID
-    );
-
-DWORD
-LsaHexStrToByteArray(
-    IN PCSTR pszHexString,
-    IN OPTIONAL DWORD* pdwHexStringLength,
-    OUT UCHAR** ppucByteArray,
-    OUT DWORD* pdwByteArrayLength
-    );
-
 DWORD
 LsaByteArrayToHexStr(
     IN UCHAR* pucByteArray,
@@ -1383,19 +1236,6 @@ LsaByteArrayToHexStr(
     OUT PSTR* ppszHexString
     );
 
-DWORD
-LsaByteArrayToLdapFormatHexStr(
-    IN UCHAR* pucByteArray,
-    IN DWORD dwByteArrayLength,
-    OUT PSTR* ppszHexString
-    );
-
-DWORD
-LsaSidStrToLdapFormatHexStr(
-    IN PCSTR pszSid,
-    OUT PSTR* ppszHexSid
-    );
-
 int
 LsaStrError(
     int errnum,
@@ -1403,136 +1243,6 @@ LsaStrError(
     size_t buflen
     );
 
-DWORD
-LsaMarshalCredentials(
-    PCSTR  pszLoginName,
-    PCSTR  pszPassword,
-    PCSTR  pszOldPassword,
-    PSTR   pszBuffer,
-    PDWORD pdwBufLen
-    );
-
-DWORD
-LsaUnmarshalCredentials(
-    PCSTR pszMsgBuf,
-    DWORD dwMsgLen,
-    PSTR* ppszLoginName,
-    PSTR* ppszPassword,
-    PSTR* ppszOldPassword
-    );
-
-DWORD
-LsaMarshalError(
-    DWORD errorCode,
-    PCSTR pszErrorMessage,
-    PSTR  pszBuf,
-    PDWORD pdwBufLen
-    );
-
-DWORD
-LsaUnmarshalError(
-    PCSTR  pszMsgBuf,
-    DWORD  dwMsgLen,
-    PDWORD pdwError,
-    PSTR*  ppszError
-    );
-
-DWORD
-LsaMarshalGSSMakeAuthMsgQ(
-    PSEC_BUFFER     credentials,
-    PSEC_BUFFER_S   serverChallenge,
-    PSEC_BUFFER     targetInfo,
-    ULONG           negotiateFlags,
-    PSTR            pszMsg,
-    DWORD *         pdwMsgLen
-    );
-
-DWORD
-LsaUnMarshalGSSMakeAuthMsgQ(
-    PCSTR           pszMsg,
-    DWORD           dwMsgLen,
-    PSEC_BUFFER     credentials,
-    PSEC_BUFFER_S   serverChallenge,
-    PSEC_BUFFER     targetInfo,
-    ULONG  *        negotiateFlags
-    );
-
-DWORD
-LsaMarshalGSSMakeAuthMsgR(
-    DWORD           msgError,
-    PSEC_BUFFER     authenticateMessage,
-    PSEC_BUFFER_S   sessionKey,
-    PSTR            pszMsg,
-    DWORD *         pdwMsgLen
-    );
-
-DWORD
-LsaUnMarshalGSSMakeAuthMsgR(
-    PCSTR       pszMsg,
-    DWORD       dwMsgLen,
-    DWORD *     dwMsgError,
-    PSEC_BUFFER authenticateMessage,
-    PSEC_BUFFER_S baseSessionKey
-    );
-
-DWORD
-LsaMarshalGSSCheckAuthMsgQ(
-    ULONG           negotiateFlags,
-    PSEC_BUFFER_S   serverChallenge,
-    PSEC_BUFFER     targetInfo,
-    PSEC_BUFFER     authenticateMessage,
-    PSTR            pszMsg,
-    DWORD *         pdwMsgLen
-    );
-
-DWORD
-LsaUnMarshalGSSCheckAuthMsgQ(
-    PCSTR           pszMsg,
-    DWORD           dwMsgLen,
-    ULONG  *        negotiateFlags,
-    PSEC_BUFFER_S   serverChallenge,
-    PSEC_BUFFER     targetInfo,
-    PSEC_BUFFER     authenticateMessage
-    );
-
-
-DWORD
-LsaMarshalGSSCheckAuthMsgR(
-    DWORD           msgError,
-    PSEC_BUFFER_S   baseSessionKey,
-    PSTR            pszMsg,
-    DWORD *         pdwMsgLen
-    );
-
-
-DWORD
-LsaUnMarshalGSSCheckAuthMsgR(
-    PCSTR           pszMsg,
-    DWORD           dwMsgLen,
-    DWORD          *msgError,
-    PSEC_BUFFER_S   baseSessionKey
-    );
-
-DWORD
-LsaMarshalSession(
-    PCSTR  pszLoginId,
-    PSTR   pszBuffer,
-    PDWORD pdwBufLen
-    );
-
-DWORD
-LsaUnmarshalSession(
-    PCSTR pszMsgBuf,
-    DWORD dwMsgLen,
-    PSTR* ppszLoginId
-    );
-
-DWORD
-LsaMarshalConfigurationRefreshRequest(
-    PSTR   pszBuffer,
-    PDWORD pdwBufLen
-    );
-
 VOID
 LsaFreeDCInfo(
     PLSA_DC_INFO pDCInfo
@@ -1571,6 +1281,125 @@ LsaNISFreeNicknameList(
     PDLINKEDLIST pNicknameList
     );
 
+DWORD
+LsaCacheNew(
+    DWORD dwNumKeys,
+    DWORD dwNumBuckets,
+    PFLSA_CACHE_HASH pfHash,
+    PFLSA_CACHE_EQUAL pfEqual,
+    PFLSA_CACHE_GETKEY pfGetKey,
+    PFLSA_CACHE_MISS pfMiss,
+    PFLSA_CACHE_KICK pfKick,
+    PVOID pData,
+    PLSA_CACHE* ppCache
+    );
+
+DWORD
+LsaCacheInsert(
+    PLSA_CACHE pCache,
+    PVOID pEntry
+    );
+
+DWORD
+LsaCacheRemove(
+    PLSA_CACHE pCache,
+    PVOID pEntry
+    );
+
+DWORD
+LsaCacheLookup(
+    PLSA_CACHE pCache,
+    PVOID pkey,
+    DWORD dwIndex,
+    PVOID* ppEntry
+    );
+
+DWORD
+LsaCacheFlush(
+    PLSA_CACHE pCache
+    );
+
+VOID
+LsaCacheDelete(
+    PLSA_CACHE pCache
+    );
+
+DWORD
+LsaTranslateLwMsgError(
+        LWMsgAssoc *pAssoc,
+        DWORD dwMsgError,
+        const char *file,
+        int line
+        );
+
+DWORD
+LsaNtStatusToLsaError(
+    IN NTSTATUS Status
+    );
+
+NTSTATUS
+LsaLsaErrorToNtStatus(
+    IN DWORD LsaError
+    );
+
+DWORD
+LsaAllocateCStringFromSid(
+    OUT PSTR* ppszStringSid,
+    IN PSID pSid
+    );
+
+DWORD
+LsaAllocateSidFromCString(
+    OUT PSID* ppSid,
+    IN PCSTR pszStringSid
+    );
+
+DWORD
+LsaAllocateSidAppendRid(
+    OUT PSID* ppSid,
+    IN PSID pDomainSid,
+    IN ULONG Rid
+    );
+
+VOID
+LsaUtilFreeSecurityObject(
+    PLSA_SECURITY_OBJECT pObject
+    );
+
+VOID
+LsaUtilFreeSecurityObjectList(
+    DWORD dwCount,
+    PLSA_SECURITY_OBJECT* ppObjectList
+    );
+
+DWORD
+LsaUtilAllocatePamConfig(
+    OUT PLSA_PAM_CONFIG *ppConfig
+    );
+
+DWORD
+LsaUtilInitializePamConfig(
+    OUT PLSA_PAM_CONFIG pConfig
+    );
+
+VOID
+LsaUtilFreePamConfigContents(
+    IN PLSA_PAM_CONFIG pConfig
+    );
+
+VOID
+LsaUtilFreePamConfig(
+    IN PLSA_PAM_CONFIG pConfig
+    );
+
 #endif /* __LSA_UTILS_H__ */
 
 
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
