@@ -76,7 +76,7 @@ pam_sm_chauthtok(
     if (!(flags & PAM_UPDATE_AUTHTOK) &&
         !(flags & PAM_PRELIM_CHECK))
     {
-        dwError = PAM_AUTHTOK_ERR;
+        dwError = LsaPamUnmapErrorCode(PAM_AUTHTOK_ERR);
         BAIL_ON_LSA_ERROR(dwError);
     }
 
@@ -142,10 +142,12 @@ cleanup:
     }
     else
     {
-        iPamError = LsaPamMapErrorCode(dwError, pPamContext);
+        iPamError = LsaPamOpenPamFilterChauthtok(
+                                    LsaPamMapErrorCode(dwError, pPamContext));
     }
 #else
-    iPamError = LsaPamMapErrorCode(dwError, pPamContext);
+    iPamError = LsaPamOpenPamFilterChauthtok(
+                                LsaPamMapErrorCode(dwError, pPamContext));
 #endif
 
     LSA_LOG_PAM_DEBUG("pam_sm_chauthtok::returning pam error code %d", iPamError);
@@ -153,12 +155,19 @@ cleanup:
 
 error:
 
-    LSA_LOG_PAM_ERROR("pam_sm_chauthtok failed [error code: %d]", dwError);
+    if (dwError ==  LW_ERROR_NO_SUCH_USER)
+    {
+        LSA_LOG_PAM_DEBUG("pam_sm_chauthtok failed since the user could not be found [error code: %u]", dwError);
+    }
+    else
+    {
+        LSA_LOG_PAM_ERROR("pam_sm_chauthtok failed [error code: %u]", dwError);
+    }
 
     goto cleanup;
 }
 
-int
+DWORD
 LsaPamCheckCurrentPassword(
     pam_handle_t* pamh,
     PPAMCONTEXT pPamContext
@@ -169,6 +178,7 @@ LsaPamCheckCurrentPassword(
     BOOLEAN bCheckOldPassword = TRUE;
     PSTR    pszOldPassword = NULL;
     PSTR    pszLoginId = NULL;
+    PSTR pszMessage = NULL;
 
     LSA_LOG_PAM_DEBUG("LsaPamCheckCurrentPassword::begin");
 
@@ -199,7 +209,15 @@ LsaPamCheckCurrentPassword(
         dwError = LsaAuthenticateUser(
                           hLsaConnection,
                           pszLoginId,
-                          pszOldPassword);
+                          pszOldPassword,
+                          &pszMessage);
+        if (pszMessage)
+        {
+            LsaPamConverse(pamh,
+                           pszMessage,
+                           PAM_TEXT_INFO,
+                           NULL);
+        }
         BAIL_ON_LSA_ERROR(dwError);
      }
 
@@ -212,6 +230,7 @@ cleanup:
 
     LW_SAFE_FREE_STRING(pszLoginId);
     LW_SAFE_CLEAR_FREE_STRING(pszOldPassword);
+    LW_SAFE_FREE_STRING(pszMessage);
 
     LSA_LOG_PAM_DEBUG("LsaPamCheckCurrentPassword::end");
 
@@ -219,14 +238,14 @@ cleanup:
 
 error:
 
-    LSA_LOG_PAM_ERROR("LsaPamCheckCurrentPassword failed [login:%s][error code: %d]",
+    LSA_LOG_PAM_ERROR("LsaPamCheckCurrentPassword failed [login:%s][error code: %u]",
                       LSA_SAFE_LOG_STRING(pszLoginId),
                       dwError);
 
     goto cleanup;
 }
 
-int
+DWORD
 LsaPamMustCheckCurrentPassword(
     HANDLE   hLsaConnection,
     PCSTR    pszLoginId,
@@ -275,9 +294,18 @@ error:
 
     *pbCheckOldPassword = TRUE;
 
-    LSA_LOG_PAM_ERROR("LsaPamMustCheckCurrentPassword failed [login:%s][error code: %d]",
-                      LSA_SAFE_LOG_STRING(pszLoginId),
-                      dwError);
+    if (dwError == LW_ERROR_NO_SUCH_USER)
+    {
+        LSA_LOG_PAM_DEBUG("LsaPamMustCheckCurrentPassword failed since the user could not be found [login:%s][error code: %u]",
+                          LSA_SAFE_LOG_STRING(pszLoginId),
+                          dwError);
+    }
+    else
+    {
+        LSA_LOG_PAM_ERROR("LsaPamMustCheckCurrentPassword failed [login:%s][error code: %u]",
+                          LSA_SAFE_LOG_STRING(pszLoginId),
+                          dwError);
+    }
 
     goto cleanup;
 }
@@ -293,7 +321,6 @@ LsaPamUpdatePassword(
     PSTR   pszPassword = NULL;
     PSTR   pszLoginId = NULL;
     HANDLE hLsaConnection = (HANDLE)NULL;
-    BOOLEAN bCheckOldPassword = FALSE;
 
     LSA_LOG_PAM_DEBUG("LsaPamUpdatePassword::begin");
 
@@ -319,20 +346,11 @@ LsaPamUpdatePassword(
     dwError = LsaOpenServer(&hLsaConnection);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaPamMustCheckCurrentPassword(
-                         hLsaConnection,
-                         pszLoginId,
-                         &bCheckOldPassword);
-    BAIL_ON_LSA_ERROR(dwError);
-
-    if (bCheckOldPassword)
-    {
-	dwError = LsaPamGetOldPassword(
+    dwError = LsaPamGetOldPassword(
                    pamh,
                    pPamContext,
                    &pszOldPassword);
-	BAIL_ON_LSA_ERROR(dwError);
-    }
+    BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LsaPamGetNewPassword(
                    pamh,
@@ -340,23 +358,12 @@ LsaPamUpdatePassword(
                    &pszPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (bCheckOldPassword)
-    {
-        dwError = LsaChangePassword(
-                       hLsaConnection,
-                       pszLoginId,
-                       pszPassword,
-                       pszOldPassword);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
-    else
-    {
-        dwError = LsaSetPassword(
-                       hLsaConnection,
-                       pszLoginId,
-                       pszPassword);
-        BAIL_ON_LSA_ERROR(dwError);
-    }
+    dwError = LsaChangePassword(
+                   hLsaConnection,
+                   pszLoginId,
+                   pszPassword,
+                   pszOldPassword);
+    BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
 
@@ -396,14 +403,23 @@ error:
             NULL);
     }
 
-    LSA_LOG_PAM_ERROR("LsaPamUpdatePassword failed [login:%s][error code: %d]",
-                      LSA_SAFE_LOG_STRING(pszLoginId),
-                      dwError);
+    if (dwError == LW_ERROR_NO_SUCH_USER)
+    {
+        LSA_LOG_PAM_DEBUG("LsaPamUpdatePassword failed since the user could not be found [login:%s][error code: %u]",
+                          LSA_SAFE_LOG_STRING(pszLoginId),
+                          dwError);
+    }
+    else
+    {
+        LSA_LOG_PAM_ERROR("LsaPamUpdatePassword failed [login:%s][error code: %u]",
+                          LSA_SAFE_LOG_STRING(pszLoginId),
+                          dwError);
+    }
 
     goto cleanup;
 }
 
-int
+DWORD
 LsaPamGetCurrentPassword(
     pam_handle_t* pamh,
     PPAMCONTEXT   pPamContext,
@@ -414,6 +430,7 @@ LsaPamGetCurrentPassword(
     PSTR pszPassword = NULL;
     BOOLEAN bPrompt = TRUE;
     PPAMOPTIONS pPamOptions = &pPamContext->pamOptions;
+    int iPamError = 0;
 
     LSA_LOG_PAM_DEBUG("LsaPamGetCurrentPassword::begin");
 
@@ -421,13 +438,14 @@ LsaPamGetCurrentPassword(
         pPamOptions->bUseFirstPass) {
         PCSTR pszItem = NULL;
 
-        dwError = pam_get_item(
+        iPamError = pam_get_item(
                         pamh,
                         PAM_AUTHTOK,
                         (PAM_GET_ITEM_TYPE)&pszItem);
-        if (dwError != PAM_SUCCESS)
+        dwError = LsaPamUnmapErrorCode(iPamError);
+        if (dwError)
         {
-            if (dwError == PAM_BAD_ITEM)
+            if (dwError == LsaPamUnmapErrorCode(PAM_BAD_ITEM))
             {
                 if (pPamOptions->bUseFirstPass)
                 {
@@ -452,7 +470,7 @@ LsaPamGetCurrentPassword(
         }
         else if (pPamOptions->bUseFirstPass)
         {
-            dwError = PAM_BAD_ITEM;
+            dwError = LsaPamUnmapErrorCode(PAM_BAD_ITEM);
             BAIL_ON_LSA_ERROR(dwError);
         }
     }
@@ -466,10 +484,11 @@ LsaPamGetCurrentPassword(
                         &pszPassword);
        BAIL_ON_LSA_ERROR(dwError);
 
-       dwError = pam_set_item(
+       iPamError = pam_set_item(
                        pamh,
                        PAM_AUTHTOK,
                        (const void*) pszPassword);
+       dwError = LsaPamUnmapErrorCode(iPamError);
        BAIL_ON_LSA_ERROR(dwError);
 
     }
@@ -480,7 +499,7 @@ cleanup:
 
     LSA_LOG_PAM_DEBUG("LsaPamGetCurrentPassword::end");
 
-    return LsaPamMapErrorCode(dwError, pPamContext);
+    return dwError;
 
 error:
 
@@ -488,22 +507,23 @@ error:
 
     *ppszPassword = NULL;
 
-    LSA_LOG_PAM_ERROR("LsaPamGetCurrentPassword failed [error code: %d]", dwError);
+    LSA_LOG_PAM_ERROR("LsaPamGetCurrentPassword failed [error code: %u]", dwError);
 
     goto cleanup;
 }
 
-int
+DWORD
 LsaPamGetOldPassword(
     pam_handle_t* pamh,
     PPAMCONTEXT pPamContext,
     PSTR* ppszPassword
     )
 {
-    int dwError = 0;
+    DWORD dwError = 0;
     PSTR pszPassword = NULL;
     BOOLEAN bPrompt = TRUE;
     PPAMOPTIONS pPamOptions = &pPamContext->pamOptions;
+    int iPamError = 0;
 
     LSA_LOG_PAM_DEBUG("LsaPamGetOldPassword::begin");
 
@@ -519,17 +539,19 @@ LsaPamGetOldPassword(
 
         /* HP-UX likes to clear PAM_OLDAUTHTOK between the two phases
            of chauthtok, so we also grab our saved version in this case */
-        dwError = pam_get_data(
+        iPamError = pam_get_data(
             pamh,
             PAM_LSASS_OLDAUTHTOK,
             (PAM_GET_DATA_TYPE)&pszItem);
+        dwError = LsaPamUnmapErrorCode(iPamError);
 #else
-        dwError = pam_get_item(
+        iPamError = pam_get_item(
             pamh,
             PAM_OLDAUTHTOK,
             (PAM_GET_ITEM_TYPE)&pszItem);
+        dwError = LsaPamUnmapErrorCode(iPamError);
 #endif
-        if (dwError == PAM_BAD_ITEM ||
+        if (dwError == LsaPamUnmapErrorCode(PAM_BAD_ITEM) ||
             pszItem == NULL)
         {
             if (pPamOptions->bUseFirstPass)
@@ -543,7 +565,7 @@ LsaPamGetOldPassword(
                 dwError = 0;
             }
         }
-        else if (dwError != PAM_SUCCESS)
+        else if (dwError)
         {
            BAIL_ON_LSA_ERROR(dwError);
         }
@@ -566,15 +588,19 @@ LsaPamGetOldPassword(
                             &pszPassword);
        BAIL_ON_LSA_ERROR(dwError);
 
-       dwError = pam_set_item(
+       iPamError = pam_set_item(
                        pamh,
                        PAM_OLDAUTHTOK,
                        (const void*) pszPassword);
+       dwError = LsaPamUnmapErrorCode(iPamError);
        BAIL_ON_LSA_ERROR(dwError);
 
-#ifdef __LWI_HP_UX__
+#if defined(__LWI_SOLARIS__) || defined(__LWI_HP_UX__)
        /* HP-UX clears PAM_OLDAUTHTOK between the two phases of chauthtok, so
           save a copy of the old password where we can find it later */
+
+       /* For Solaris, we read PAM_LSASS_OLDAUTHTOK instead of
+          PAM_OLDAUTHTOK. */
        dwError = LsaPamSetDataString(pamh, PAM_LSASS_OLDAUTHTOK, pszPassword);
        BAIL_ON_LSA_ERROR(dwError);
 #endif
@@ -586,7 +612,7 @@ cleanup:
 
     LSA_LOG_PAM_DEBUG("LsaPamGetOldPassword::end");
 
-    return LsaPamMapErrorCode(dwError, pPamContext);
+    return dwError;
 
 error:
 
@@ -594,25 +620,26 @@ error:
 
     *ppszPassword = NULL;
 
-    LSA_LOG_PAM_ERROR("LsaPamGetOldPassword failed [error code: %d]", dwError);
+    LSA_LOG_PAM_ERROR("LsaPamGetOldPassword failed [error code: %u]", dwError);
 
     goto cleanup;
 }
 
-int
+DWORD
 LsaPamGetNewPassword(
     pam_handle_t* pamh,
     PPAMCONTEXT   pPamContext,
     PSTR*         ppszPassword
     )
 {
-    int dwError = PAM_SUCCESS;
+    DWORD dwError = 0;
     PSTR pszPassword_1 = NULL;
     PSTR pszPassword_2 = NULL;
     DWORD dwLen_1 = 0;
     DWORD dwLen_2 = 0;
     BOOLEAN bPrompt = TRUE;
     PPAMOPTIONS pPamOptions = &pPamContext->pamOptions;
+    int iPamError = 0;
 
     LSA_LOG_PAM_DEBUG("LsaPamGetNewPassword::begin");
 
@@ -620,10 +647,11 @@ LsaPamGetNewPassword(
 
         PCSTR pszItem = NULL;
 
-        dwError = pam_get_item(
+        iPamError = pam_get_item(
                     pamh,
                     PAM_AUTHTOK,
                     (PAM_GET_ITEM_TYPE)&pszItem);
+        dwError = LsaPamUnmapErrorCode(iPamError);
         BAIL_ON_LSA_ERROR(dwError);
 
         if (!LW_IS_NULL_OR_EMPTY_STR(pszItem)) {
@@ -661,7 +689,7 @@ LsaPamGetNewPassword(
            (strcmp(pszPassword_1, pszPassword_2) != 0))
        {
            LsaPamConverse(pamh, "Passwords do not match", PAM_ERROR_MSG, NULL);
-           dwError = PAM_SUCCESS;
+           dwError = 0;
 
            LW_SAFE_CLEAR_FREE_STRING(pszPassword_1);
            LW_SAFE_CLEAR_FREE_STRING(pszPassword_2);
@@ -669,10 +697,11 @@ LsaPamGetNewPassword(
        }
        else
        {
-           dwError = pam_set_item(
+           iPamError = pam_set_item(
                        pamh,
                        PAM_AUTHTOK,
                        (const void*) pszPassword_1);
+           dwError = LsaPamUnmapErrorCode(iPamError);
            BAIL_ON_LSA_ERROR(dwError);
 
            bPrompt = FALSE;
@@ -687,7 +716,7 @@ cleanup:
 
     LSA_LOG_PAM_DEBUG("LsaPamGetNewPassword::end");
 
-    return LsaPamMapErrorCode(dwError, pPamContext);
+    return dwError;
 
 error:
 
@@ -695,7 +724,7 @@ error:
 
     *ppszPassword = NULL;
 
-    LSA_LOG_PAM_ERROR("LsaPamGetNewPassword failed [error code: %d]", dwError);
+    LSA_LOG_PAM_ERROR("LsaPamGetNewPassword failed [error code: %u]", dwError);
 
     goto cleanup;
 }
