@@ -615,7 +615,7 @@ static GSS_MECH_CONFIG gNtlmMech =
     NULL, //ntlm_gss_unwrap_aead,
     ntlm_gss_wrap_iov,
     ntlm_gss_unwrap_iov,
-    NULL, //ntlm_gss_wrap_iov_length,
+    ntlm_gss_wrap_iov_length,
     NULL, //ntlm_gss_complete_auth_token,
     NULL, //ntlm_gss_inquire_context2
     .gss_get_name_attribute = ntlm_gss_get_name_attribute
@@ -841,7 +841,7 @@ ntlm_gss_init_sec_context(
     SecBuffer OutputToken = {0};
     NTLM_CRED_HANDLE CredHandle = NULL;
     TimeStamp Expiry = 0;
-    DWORD dwNtlmFlags = NTLM_FLAG_NEGOTIATE_DEFAULT;
+    DWORD dwNtlmFlags = 0;
     DWORD dwOutNtlmFlags = 0;
     OM_uint32 RetFlags = 0;
 
@@ -865,7 +865,11 @@ ntlm_gss_init_sec_context(
     }
 
     // if no credentials are passed in, create default creds
-    if (GSS_C_NO_CREDENTIAL == InitiatorCredHandle)
+    if (nReqFlags & GSS_C_ANON_FLAG)
+    {
+        CredHandle = NULL;
+    }
+    else if (GSS_C_NO_CREDENTIAL == InitiatorCredHandle)
     {
         MinorStatus = NtlmClientAcquireCredentialsHandle(
             NULL,
@@ -882,7 +886,7 @@ ntlm_gss_init_sec_context(
     {
         CredHandle = ((PNTLM_GSS_CREDS)InitiatorCredHandle)->
             CredHandle;
-        if(CredHandle == NULL)
+        if (CredHandle == NULL)
         {
             // This means ntlm_gss_acquire_cred was called, but not default
             // credentials were available. The user should have called
@@ -895,11 +899,19 @@ ntlm_gss_init_sec_context(
         }
     }
 
-    // NTLM supports only signing and sealing (both at the same time)
-    if ((nReqFlags & GSS_C_INTEG_FLAG) ||
-        (nReqFlags & GSS_C_CONF_FLAG))
+    // The server will ignore these flags and always perform signing and
+    // sealing, but they are translated here for the sake of consistency.
+    if (nReqFlags & GSS_C_INTEG_FLAG)
     {
-        dwNtlmFlags |= (NTLM_FLAG_SIGN | NTLM_FLAG_SEAL);
+        dwNtlmFlags |= ISC_REQ_INTEGRITY;
+    }
+    if (nReqFlags & GSS_C_CONF_FLAG)
+    {
+        dwNtlmFlags |= ISC_REQ_CONFIDENTIALITY;
+    }
+    if (nReqFlags & GSS_C_ANON_FLAG)
+    {
+        dwNtlmFlags |= ISC_REQ_NULL_SESSION;
     }
 
     MinorStatus = NtlmClientInitializeSecurityContext(
@@ -926,15 +938,18 @@ ntlm_gss_init_sec_context(
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
 
-    if (dwOutNtlmFlags & NTLM_FLAG_SIGN)
+    if (dwOutNtlmFlags & ISC_RET_INTEGRITY)
     {
         RetFlags |= GSS_C_INTEG_FLAG;
     }
-
-    if (dwOutNtlmFlags & NTLM_FLAG_SEAL)
+    if (dwOutNtlmFlags & ISC_RET_CONFIDENTIALITY)
     {
         RetFlags |= GSS_C_CONF_FLAG;
     }
+    if (dwOutNtlmFlags & ISC_RET_NULL_SESSION)
+    {
+        RetFlags |= GSS_C_ANON_FLAG;
+    }
 
 cleanup:
     *pMinorStatus = MinorStatus;
@@ -997,7 +1012,8 @@ ntlm_gss_accept_sec_context(
     OM_uint32 MajorStatus = GSS_S_COMPLETE;
     OM_uint32 MinorStatus = LW_ERROR_SUCCESS;
     DWORD dwRetFlags = 0;
-    DWORD dwFinalFlags = 0;
+    DWORD dwReqFlags = 0;
+    DWORD dwGssRetFlags = 0;
     SecBufferDesc InputBuffer = {0};
     SecBufferDesc OutputBuffer = {0};
     SecBuffer InputToken = {0};
@@ -1029,7 +1045,18 @@ ntlm_gss_accept_sec_context(
     }
     if (pRetFlags)
     {
-        dwRetFlags = *pRetFlags;
+        if (*pRetFlags & GSS_C_INTEG_FLAG)
+        {
+            dwReqFlags |= ASC_REQ_INTEGRITY;
+        }
+        if (*pRetFlags & GSS_C_CONF_FLAG)
+        {
+            dwReqFlags |= ASC_REQ_CONFIDENTIALITY;
+        }
+        if (*pRetFlags & GSS_C_ANON_FLAG)
+        {
+            dwReqFlags |= ASC_REQ_NULL_SESSION;
+        }
     }
 
     if (AcceptorCredHandle)
@@ -1051,31 +1078,6 @@ ntlm_gss_accept_sec_context(
         PassAcceptorCredHandle = LocalCreds;
     }
 
-#if 0
-    // convert
-    switch (dwRetFlags)
-    {
-    case GSS_C_DELEG_FLAG:
-        break;
-    case GSS_C_MUTUAL_FLAG:
-        break;
-    case GSS_C_REPLAY_FLAG:
-        break;
-    case GSS_C_SEQUENCE_FLAG:
-        break;
-    case GSS_C_CONF_FLAG:
-        break;
-    case GSS_C_INTEG_FLAG:
-        break;
-    case GSS_C_ANON_FLAG:
-        break;
-    case GSS_C_PROT_READY_FLAG:
-        break;
-    case GSS_C_TRANS_FLAG:
-        break;
-    }
-#endif
-
     memset(pOutputToken, 0, sizeof(*pOutputToken));
 
     InputBuffer.cBuffers = 1;
@@ -1092,11 +1094,11 @@ ntlm_gss_accept_sec_context(
         &((PNTLM_GSS_CREDS)PassAcceptorCredHandle)->CredHandle,
         (PNTLM_CONTEXT_HANDLE)pContextHandle,
         &InputBuffer,
-        dwFinalFlags,
+        dwReqFlags,
         NTLM_NATIVE_DATA_REP,
         &NewCtxtHandle,
         &OutputBuffer,
-        pTimeRec,
+        &dwRetFlags,
         &tsExpiry);
 
     if (MinorStatus == LW_WARNING_CONTINUE_NEEDED)
@@ -1107,9 +1109,6 @@ ntlm_gss_accept_sec_context(
     {
         BAIL_ON_LSA_ERROR(MinorStatus);
 
-        dwRetFlags |= GSS_C_CONF_FLAG;
-        dwRetFlags |= GSS_C_INTEG_FLAG;
-
         MajorStatus = ntlm_gss_inquire_context(
                           &MinorStatus,
                           (gss_ctx_id_t)NewCtxtHandle,
@@ -1152,7 +1151,19 @@ cleanup:
 
     if (pRetFlags)
     {
-        *pRetFlags = dwRetFlags;
+        if (dwRetFlags & ASC_RET_INTEGRITY)
+        {
+            dwGssRetFlags |= GSS_C_INTEG_FLAG;
+        }
+        if (dwRetFlags & ASC_RET_CONFIDENTIALITY)
+        {
+            dwGssRetFlags |= GSS_C_CONF_FLAG;
+        }
+        if (dwRetFlags & ASC_RET_NULL_SESSION)
+        {
+            dwGssRetFlags |= GSS_C_ANON_FLAG;
+        }
+        *pRetFlags = dwGssRetFlags;
     }
 
     if (pTimeRec)
@@ -1490,27 +1501,32 @@ ntlm_gss_wrap_iov(
     OM_uint32 MinorStatus = LW_ERROR_SUCCESS;
     NTLM_CONTEXT_HANDLE ContextHandle = (NTLM_CONTEXT_HANDLE)GssCtxtHandle;
     SecBufferDesc Message = {0};
-    SecBuffer NtlmBuffer[2];
+    SecBuffer *NtlmBuffer = NULL;
     SecPkgContext_Sizes Sizes = {0};
     INT nEncrypted = 0;
+    DWORD dwIndex = 0;
+    BOOLEAN bFoundHeader = FALSE;
 
-    Message.cBuffers = 2;
+    if (cBuffers < 2)
+    {
+        MinorStatus = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(MinorStatus);
+    }
+
+    MinorStatus = LwAllocateMemory(
+        cBuffers * sizeof(SecBuffer),
+        OUT_PPVOID(&NtlmBuffer));
+    BAIL_ON_LSA_ERROR(MinorStatus);
+
+    Message.cBuffers = cBuffers;
     Message.pBuffers = NtlmBuffer;
 
-    memset(NtlmBuffer, 0, sizeof(SecBuffer) * Message.cBuffers);
-
     if (Qop != GSS_C_QOP_DEFAULT)
     {
         MajorStatus = GSS_S_BAD_QOP;
         BAIL_ON_LSA_ERROR(MajorStatus);
     }
 
-    if (cBuffers != 2)
-    {
-        MinorStatus = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(MinorStatus);
-    }
-
     // Since every encrypted message gets a signature, we need to get the size
     MinorStatus = NtlmClientQueryContextAttributes(
         &ContextHandle,
@@ -1519,37 +1535,75 @@ ntlm_gss_wrap_iov(
         );
     BAIL_ON_LSA_ERROR(MinorStatus);
 
-    if (GSS_IOV_BUFFER_TYPE(pBuffers[0].type) != GSS_IOV_BUFFER_TYPE_HEADER)
+    for (dwIndex = 0 ; dwIndex < cBuffers ; dwIndex++ )
+    {
+        if (GSS_IOV_BUFFER_FLAGS(pBuffers[dwIndex].type) & GSS_IOV_BUFFER_FLAG_ALLOCATE)
+        {
+            switch (GSS_IOV_BUFFER_TYPE(pBuffers[dwIndex].type))
+            {
+                case GSS_IOV_BUFFER_TYPE_HEADER:
+                    pBuffers[dwIndex].buffer.length = Sizes.cbMaxSignature;
+                    bFoundHeader = TRUE;
+                    break;
+                case GSS_IOV_BUFFER_TYPE_PADDING:
+                    pBuffers[dwIndex].buffer.length = 0;
+                    break;
+                default:
+                    MinorStatus = LW_ERROR_INVALID_PARAMETER;
+                    BAIL_ON_LSA_ERROR(MinorStatus);
+                    break;
+            }
+
+            if (pBuffers[dwIndex].buffer.length == 0)
+            {
+                pBuffers[dwIndex].buffer.value = NULL;
+            }
+            else
+            {
+                MinorStatus = LwAllocateMemory(
+                    pBuffers[dwIndex].buffer.length,
+                    OUT_PPVOID(&pBuffers[dwIndex].buffer.value));
+                BAIL_ON_LSA_ERROR(MinorStatus);
+
+                pBuffers[dwIndex].type |= GSS_IOV_BUFFER_FLAG_ALLOCATED;
+            }
+        }
+        else
+        {
+            switch (GSS_IOV_BUFFER_TYPE(pBuffers[dwIndex].type))
+            {
+                case GSS_IOV_BUFFER_TYPE_HEADER:
+                    NtlmBuffer[dwIndex].BufferType = SECBUFFER_TOKEN;
+                    bFoundHeader = TRUE;
+                    break;
+                case GSS_IOV_BUFFER_TYPE_DATA:
+                    NtlmBuffer[dwIndex].BufferType = SECBUFFER_DATA;
+                    break;
+                case GSS_IOV_BUFFER_TYPE_SIGN_ONLY:
+                    NtlmBuffer[dwIndex].BufferType = SECBUFFER_DATA |
+                                                     SECBUFFER_READONLY_WITH_CHECKSUM;
+                    break;
+                case GSS_IOV_BUFFER_TYPE_PADDING:
+                    NtlmBuffer[dwIndex].BufferType = SECBUFFER_PADDING;
+                    pBuffers[dwIndex].buffer.length = 0;
+                    break;
+                default:
+                    MinorStatus = LW_ERROR_INVALID_PARAMETER;
+                    BAIL_ON_LSA_ERROR(MinorStatus);
+                    break;
+            }
+        }
+
+        NtlmBuffer[dwIndex].cbBuffer = pBuffers[dwIndex].buffer.length;
+        NtlmBuffer[dwIndex].pvBuffer = pBuffers[dwIndex].buffer.value;
+    }
+
+    if (!bFoundHeader)
     {
         MinorStatus = LW_ERROR_INVALID_PARAMETER;
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
 
-    if (GSS_IOV_BUFFER_FLAGS(pBuffers[0].type) & GSS_IOV_BUFFER_FLAG_ALLOCATE)
-    {
-        pBuffers[0].buffer.length = Sizes.cbMaxSignature;
-        MinorStatus = LwAllocateMemory(
-            pBuffers[0].buffer.length,
-            OUT_PPVOID(&pBuffers[0].buffer.value));
-        BAIL_ON_LSA_ERROR(MinorStatus);
-
-        pBuffers[0].type |= GSS_IOV_BUFFER_FLAG_ALLOCATED;
-    }
-
-    NtlmBuffer[0].BufferType = SECBUFFER_TOKEN;
-    NtlmBuffer[0].cbBuffer = pBuffers[0].buffer.length;
-    NtlmBuffer[0].pvBuffer = pBuffers[0].buffer.value;
-
-    if (pBuffers[1].type != GSS_IOV_BUFFER_TYPE_DATA)
-    {
-        MinorStatus = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(MinorStatus);
-    }
-
-    NtlmBuffer[1].BufferType = SECBUFFER_DATA;
-    NtlmBuffer[1].cbBuffer = pBuffers[1].buffer.length;
-    NtlmBuffer[1].pvBuffer = pBuffers[1].buffer.value;
-
     MinorStatus = NtlmClientEncryptMessage(
         &ContextHandle,
         nEncrypt ? TRUE : FALSE,
@@ -1565,6 +1619,8 @@ ntlm_gss_wrap_iov(
 
 cleanup:
 
+    LW_SAFE_FREE_MEMORY(NtlmBuffer);
+
     if (pEncrypted)
     {
         *pEncrypted = nEncrypted;
@@ -1574,6 +1630,16 @@ cleanup:
     return MajorStatus;
 
 error:
+
+    for (dwIndex = 0 ; dwIndex < cBuffers ; dwIndex++ )
+    {
+        if (GSS_IOV_BUFFER_FLAGS(pBuffers[dwIndex].type) & GSS_IOV_BUFFER_FLAG_ALLOCATED)
+        {
+            LW_SAFE_FREE_MEMORY(pBuffers[dwIndex].buffer.value);
+            pBuffers[dwIndex].type |= ~GSS_IOV_BUFFER_FLAG_ALLOCATED;
+        }
+    }
+
     nEncrypted = 0;
 
     if (MajorStatus == GSS_S_COMPLETE)
@@ -1583,6 +1649,80 @@ error:
     goto cleanup;
 }
 
+OM_uint32
+ntlm_gss_wrap_iov_length(
+    OM_uint32* pMinorStatus,
+    gss_ctx_id_t GssCtxtHandle,
+    INT nEncrypt,
+    gss_qop_t Qop,
+    PINT pEncrypted,
+    gss_iov_buffer_desc* pBuffers,
+    int cBuffers
+    )
+{
+    OM_uint32 MajorStatus = GSS_S_COMPLETE;
+    OM_uint32 MinorStatus = LW_ERROR_SUCCESS;
+    NTLM_CONTEXT_HANDLE ContextHandle = (NTLM_CONTEXT_HANDLE)GssCtxtHandle;
+    SecPkgContext_Sizes Sizes = {0};
+    DWORD dwIndex = 0;
+    BOOLEAN bFoundHeader = FALSE;
+
+    if (cBuffers < 2)
+    {
+        MinorStatus = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(MinorStatus);
+    }
+
+    if (Qop != GSS_C_QOP_DEFAULT)
+    {
+        MajorStatus = GSS_S_BAD_QOP;
+        BAIL_ON_LSA_ERROR(MajorStatus);
+    }
+
+    // Since every encrypted message gets a signature, we need to get the size
+    MinorStatus = NtlmClientQueryContextAttributes(
+        &ContextHandle,
+        SECPKG_ATTR_SIZES,
+        &Sizes
+        );
+    BAIL_ON_LSA_ERROR(MinorStatus);
+
+    for (dwIndex = 0 ; dwIndex < cBuffers ; dwIndex++ )
+    {
+        switch (GSS_IOV_BUFFER_TYPE(pBuffers[dwIndex].type))
+        {
+            case GSS_IOV_BUFFER_TYPE_HEADER:
+                pBuffers[dwIndex].buffer.length = Sizes.cbMaxSignature;
+                pBuffers[dwIndex].buffer.value = NULL;
+                bFoundHeader = TRUE;
+                break;
+            case GSS_IOV_BUFFER_TYPE_PADDING:
+                pBuffers[dwIndex].buffer.length = 0;
+                pBuffers[dwIndex].buffer.value = NULL;
+                break;
+        }
+    }
+
+    if (!bFoundHeader)
+    {
+        MinorStatus = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(MinorStatus);
+    }
+
+cleanup:
+
+    *pMinorStatus = MinorStatus;
+    return MajorStatus;
+
+error:
+
+    if (MajorStatus == GSS_S_COMPLETE)
+    {
+        MajorStatus = GSS_S_FAILURE;
+    }
+    goto cleanup;
+}
+
 OM_uint32
 ntlm_gss_unwrap(
     OM_uint32* pMinorStatus,
@@ -1662,7 +1802,7 @@ ntlm_gss_unwrap(
     NtlmBuffer[1].cbBuffer = dwBufferSize;
     NtlmBuffer[1].pvBuffer = pBuffer;
 
-    if (dwNtlmFlags & (NTLM_FLAG_SEAL | NTLM_FLAG_SIGN))
+    if (dwNtlmFlags & (ISC_RET_CONFIDENTIALITY | ISC_RET_INTEGRITY))
     {
         MinorStatus = NtlmClientDecryptMessage(
             &ContextHandle,
@@ -1671,7 +1811,7 @@ ntlm_gss_unwrap(
             &bEncrypted
             );
     }
-    else if (dwNtlmFlags & NTLM_FLAG_SIGN)
+    else if (dwNtlmFlags & ISC_RET_INTEGRITY)
     {
         MinorStatus = NtlmClientVerifySignature(
             &ContextHandle,
@@ -1730,40 +1870,59 @@ ntlm_gss_unwrap_iov(
     OM_uint32 MinorStatus = LW_ERROR_SUCCESS;
     NTLM_CONTEXT_HANDLE ContextHandle = (NTLM_CONTEXT_HANDLE)GssCtxtHandle;
     SecBufferDesc Message;
-    SecBuffer NtlmBuffer[2];
+    SecBuffer *NtlmBuffer = NULL;
     BOOLEAN bEncrypted = FALSE;
+    DWORD dwIndex = 0;
+    BOOLEAN bFoundHeader = FALSE;
 
-    Message.cBuffers = 2;
+    if (cBuffers < 2)
+    {
+        MinorStatus = LW_ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(MinorStatus);
+    }
+
+    MinorStatus = LwAllocateMemory(
+        cBuffers * sizeof(SecBuffer),
+        OUT_PPVOID(&NtlmBuffer));
+    BAIL_ON_LSA_ERROR(MinorStatus);
+
+    Message.cBuffers = cBuffers;
     Message.pBuffers = NtlmBuffer;
 
-    memset(NtlmBuffer, 0, sizeof(SecBuffer) * Message.cBuffers);
+    for (dwIndex = 0 ; dwIndex < cBuffers ; dwIndex++)
+    {
+        switch (GSS_IOV_BUFFER_TYPE(pBuffers[dwIndex].type))
+        {
+            case GSS_IOV_BUFFER_TYPE_HEADER:
+                NtlmBuffer[dwIndex].BufferType = SECBUFFER_TOKEN;
+                bFoundHeader = TRUE;
+                break;
+            case GSS_IOV_BUFFER_TYPE_DATA:
+                NtlmBuffer[dwIndex].BufferType = SECBUFFER_DATA;
+                break;
+            case GSS_IOV_BUFFER_TYPE_SIGN_ONLY:
+                NtlmBuffer[dwIndex].BufferType = SECBUFFER_DATA |
+                                                 SECBUFFER_READONLY_WITH_CHECKSUM;
+                break;
+            case GSS_IOV_BUFFER_TYPE_PADDING:
+                NtlmBuffer[dwIndex].BufferType = SECBUFFER_PADDING;
+                break;
+            default:
+                MinorStatus = LW_ERROR_INVALID_PARAMETER;
+                BAIL_ON_LSA_ERROR(MinorStatus);
+                break;
+        }
 
-    if (cBuffers != 2)
+        NtlmBuffer[dwIndex].cbBuffer = pBuffers[dwIndex].buffer.length;
+        NtlmBuffer[dwIndex].pvBuffer = pBuffers[dwIndex].buffer.value;
+    }
+
+    if (!bFoundHeader)
     {
         MinorStatus = LW_ERROR_INVALID_PARAMETER;
         BAIL_ON_LSA_ERROR(MinorStatus);
     }
 
-    if (pBuffers[0].type != GSS_IOV_BUFFER_TYPE_HEADER)
-    {
-        MinorStatus = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(MinorStatus);
-    }
-
-    NtlmBuffer[0].BufferType = SECBUFFER_TOKEN;
-    NtlmBuffer[0].cbBuffer = pBuffers[0].buffer.length;
-    NtlmBuffer[0].pvBuffer = pBuffers[0].buffer.value;
-
-    if (pBuffers[1].type != GSS_IOV_BUFFER_TYPE_DATA)
-    {
-        MinorStatus = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(MinorStatus);
-    }
-
-    NtlmBuffer[1].BufferType = SECBUFFER_DATA;
-    NtlmBuffer[1].cbBuffer = pBuffers[1].buffer.length;
-    NtlmBuffer[1].pvBuffer = pBuffers[1].buffer.value;
-
     MinorStatus = NtlmClientDecryptMessage(
         &ContextHandle,
         &Message,
@@ -1774,6 +1933,8 @@ ntlm_gss_unwrap_iov(
 
 cleanup:
 
+    LW_SAFE_FREE_MEMORY(NtlmBuffer);
+
     if (pEncrypted)
     {
         *pEncrypted = bEncrypted;
@@ -2116,12 +2277,12 @@ ntlm_gss_inquire_context(
             &dwNtlmFlags);
         BAIL_ON_LSA_ERROR(MinorStatus);
 
-        if (dwNtlmFlags & NTLM_FLAG_SIGN)
+        if (dwNtlmFlags & ISC_RET_INTEGRITY)
         {
             *pCtxtFlags |= GSS_C_INTEG_FLAG;
         }
 
-        if (dwNtlmFlags & NTLM_FLAG_SEAL)
+        if (dwNtlmFlags & ISC_RET_CONFIDENTIALITY)
         {
             *pCtxtFlags |= GSS_C_CONF_FLAG;
         }
