@@ -43,13 +43,15 @@
  * Authors: Danilo Almeida (dalmeida@likewise.com)
  */
 
-#include <lwmapsecurity/lwmapsecurity-plugin.h>
 #include <lw/base.h>
-#include <lsa/lsa.h>
+#include <lw/mapsecurity-plugin.h>
 #include <lw/rtlgoto.h>
-#include <lw/safeint.h>
+#include <lsa/lsa.h>
 #include <uuid/uuid.h>
-#include <lwrpc/krb5pac.h>
+#include <lwio/lwio.h>
+#include <lw/rpc/samr.h>
+#include <lw/rpc/netlogon.h>
+#include <lw/rpc/krb5pac.h>
 #include <gssapi/gssapi.h>
 #include <gssapi/gssapi_ext.h>
 #include <sys/types.h>
@@ -74,9 +76,10 @@ typedef struct _LW_MAP_SECURITY_PLUGIN_CONTEXT {
 
 typedef UCHAR LSA_MAP_SECURITY_OBJECT_INFO_FLAGS, *PLSA_MAP_SECURITY_OBJECT_INFO_FLAGS;
 
-#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_IS_USER   0x01
-#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID 0x02
-#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID 0x04
+#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_IS_USER          0x01
+#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID        0x02
+#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID        0x04
+#define LSA_MAP_SECURITY_OBJECT_INFO_FLAG_ACCOUNT_DISABLED 0x08
 
 typedef struct _LSA_MAP_SECURITY_OBJECT_INFO {
     LSA_MAP_SECURITY_OBJECT_INFO_FLAGS Flags;
@@ -285,6 +288,11 @@ LsaMapSecurityResolveObjectInfo(
         SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID);
         SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID);
 
+        if (ppObjects[0]->userInfo.bAccountDisabled)
+        {
+            SetFlag(objectInfo.Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_ACCOUNT_DISABLED);
+        }
+
         objectInfo.Uid = ppObjects[0]->userInfo.uid;
         objectInfo.Gid = ppObjects[0]->userInfo.gid;
 
@@ -476,6 +484,8 @@ LsaMapSecurityResolveObjectInfoFromPac(
                     1,
                     QueryList,
                     &ppObjects);
+    status = LwWin32ErrorToNtStatus(dwError);
+    GOTO_CLEANUP_ON_STATUS(status);
 
     LsaMapSecurityCloseConnection(Context, &hConnection);
 
@@ -597,6 +607,33 @@ cleanup:
     return status;
 }
 
+static
+NTSTATUS
+LsaMapSecurityGetSidFromName(
+    IN PLW_MAP_SECURITY_PLUGIN_CONTEXT Context,
+    OUT PSID* Sid,
+    IN BOOLEAN IsUser,
+    IN PCSTR pszName
+    )
+{
+    NTSTATUS status = STATUS_SUCCESS;
+    LSA_MAP_SECURITY_OBJECT_INFO objectInfo = { 0 };
+    PSID sid = NULL;
+
+    status = LsaMapSecurityResolveObjectInfo(Context, IsUser, pszName, NULL, &objectInfo);
+    GOTO_CLEANUP_ON_STATUS(status);
+
+    sid = objectInfo.Sid;
+    objectInfo.Sid = NULL;
+
+cleanup:
+    LsaMapSecurityFreeObjectInfo(&objectInfo);
+
+    *Sid = sid;
+
+    return status;
+}
+
 static
 VOID
 LsaMapSecurityFreeAccessTokenCreateInformation(
@@ -797,6 +834,14 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfo(
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
     LSA_QUERY_LIST QueryList;
 
+    if (IsSetFlag(
+            pObjectInfo->Flags,
+            LSA_MAP_SECURITY_OBJECT_INFO_FLAG_ACCOUNT_DISABLED))
+    {
+        status = STATUS_ACCOUNT_DISABLED;
+        GOTO_CLEANUP_ON_STATUS(status);
+    }
+
     status = RtlAllocateCStringFromSid(&pszSid, pObjectInfo->Sid);
     GOTO_CLEANUP_ON_STATUS(status);
 
@@ -1182,29 +1227,21 @@ LsaMapSecurityGetAccessTokenCreateInformationFromObjectInfoAndGroups(
     // TOKEN_UNIX
     //
 
-    if (IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID))
+    if (IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_UID) &&
+        IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID))
     {
         createInformation->Unix->Uid = pObjectInfo->Uid;
-    }
-    else
-    {
-        status = STATUS_NO_SUCH_USER;
-        GOTO_CLEANUP();
-    }
-
-    if (IsSetFlag(pObjectInfo->Flags, LSA_MAP_SECURITY_OBJECT_INFO_FLAG_VALID_GID))
-    {
         createInformation->Unix->Gid = pObjectInfo->Gid;
+
+        // TODO-Should the token even have a umask, and if so,
+        // where should it come from?
+        createInformation->Unix->Umask = 0022;
     }
     else
     {
-        status = STATUS_NO_SUCH_GROUP;
-        GOTO_CLEANUP();
+        createInformation->Unix = NULL;
     }
 
-    // TODO-Should the token even have a umask, and if so,
-    // where should it come from?
-    createInformation->Unix->Umask = 0022;
 
     //
     // TOKEN_USER
@@ -1356,7 +1393,6 @@ LsaMapSecurityGetPacInfoFromGssContext(
     NTSTATUS status = STATUS_SUCCESS;
     OM_uint32 minorStatus = 0;
     OM_uint32 majorStatus = 0;
-    error_status_t dceStatus = 0;
     gss_name_t srcName = GSS_C_NO_NAME;
     gss_buffer_desc logonInfoUrn =
     {
@@ -1395,25 +1431,25 @@ LsaMapSecurityGetPacInfoFromGssContext(
         &more);
     if (majorStatus != GSS_S_COMPLETE)
     {
-        status = STATUS_UNSUCCESSFUL;
+        // Try to convert the minorStatus as a Win32 error.  At worst we'll end
+        // up with a squashed error code if it is not a valid Win32 error. But to
+        // be fair we were always returning STATUS_UNSUCCESSFUL here before.
+
+        status = LwWin32ErrorToNtStatus(minorStatus);
         BAIL_ON_NT_STATUS(status);
     }
 
     if (pacData.value == NULL)
     {
-        status = STATUS_UNSUCCESSFUL;
+        status = STATUS_INVALID_USER_BUFFER;
         BAIL_ON_NT_STATUS(status);
     }
 
-    dceStatus = DecodePacLogonInfo(
+    status = DecodePacLogonInfo(
         pacData.value,
         pacData.length,
         &pPac);
-    if (dceStatus)
-    {
-        status = STATUS_UNSUCCESSFUL;
-        BAIL_ON_NT_STATUS(status);
-    }
+    BAIL_ON_NT_STATUS(status);
 
     *ppPac = pPac;
 
@@ -1507,16 +1543,16 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
     BAIL_ON_NT_STATUS(status);
 
     status = RTL_ALLOCATE(&ppInputSids, PSID, sizeof(PSID) *
-                          (pPac->info3.base.groups.count +
-                           pPac->res_groups.count +
+                          (pPac->info3.base.groups.dwCount +
+                           pPac->res_groups.dwCount +
                            pPac->info3.sidcount));
     BAIL_ON_NT_STATUS(status);
 
-    for (dwIndex = 0; dwIndex < pPac->info3.base.groups.count; dwIndex++)
+    for (dwIndex = 0; dwIndex < pPac->info3.base.groups.dwCount; dwIndex++)
     {
         status = LsaMapSecurityConstructSid(
             pPac->info3.base.domain_sid,
-            pPac->info3.base.groups.rids[dwIndex].rid,
+            pPac->info3.base.groups.pRids[dwIndex].dwRid,
             &pSid);
         BAIL_ON_NT_STATUS(status);
 
@@ -1524,11 +1560,11 @@ LsaMapSecurityGetAccessTokenCreateInformationFromGssContext(
         pSid = NULL;
     }
 
-    for (dwIndex = 0; dwIndex < pPac->res_groups.count; dwIndex++)
+    for (dwIndex = 0; dwIndex < pPac->res_groups.dwCount; dwIndex++)
     {
         status = LsaMapSecurityConstructSid(
             pPac->res_group_dom_sid,
-            pPac->res_groups.rids[dwIndex].rid,
+            pPac->res_groups.pRids[dwIndex].dwRid,
             &pSid);
         BAIL_ON_NT_STATUS(status);
 
@@ -1597,6 +1633,7 @@ static LW_MAP_SECURITY_PLUGIN_INTERFACE gLsaMapSecurityPluginInterface = {
     .FreeContext = LsaMapSecurityFreeContext,
     .GetIdFromSid = LsaMapSecurityGetIdFromSid,
     .GetSidFromId = LsaMapSecurityGetSidFromId,
+    .GetSidFromName = LsaMapSecurityGetSidFromName,
     .DuplicateSid = LsaMapSecurityDuplicateSid,
     .FreeSid = LsaMapSecurityFreeSid,
     .GetAccessTokenCreateInformationFromUid = LsaMapSecurityGetAccessTokenCreateInformationFromUid,
