@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -46,36 +46,74 @@
  */
 
 #include "config.h"
-#include "lsasystem.h"
 
-#include <uuid/uuid.h>
+#include "lsasystem.h"
+#include <lsa/lsa.h>
+#include <lsa/lsa2.h>
+#include <lwmsg/lwmsg.h>
+
+#include <dce/rpc.h>
+#include <dce/dcethread.h>
+#include <lw/base.h>
+#include <reg/reg.h>
 
 #include <eventlog.h>
 
 #include "lsadef.h"
-#include "lsa/lsa.h"
-
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
-#include "lsaunistr.h"
 #include "lsalog_r.h"
+#include "lsaregdef.h"
+#include "lwmapsecurity-lsass.h"
 
 #include "lsasrvutils.h"
 #include "lsaserver.h"
-#include "lsaprovider.h"
+#include <lsa/provider.h>
+#include "lsarpcsrv.h"
+#include "rpcctl.h"
+#include "directory.h"
 
 #include "structs_p.h"
 #include "auth_p.h"
 #include "auth_provider_p.h"
+#include "rpc_server_p.h"
+#include "event_p.h"
 #include "externs_p.h"
-#include "groups_p.h"
-#include "artefacts_p.h"
 #include "session_p.h"
 #include "state_p.h"
-#include "users_p.h"
 #include "metrics_p.h"
 #include "status_p.h"
 #include "config_p.h"
-#include "event_p.h"
 
-#include "ntlmgsssrv.h"
+#include "lsasrvapi.h"
+#include "lsasrvapi2.h"
 
+#include "lsaipc.h"
+
+#include "ipc_error_p.h"
+#include "externs_p.h"
+
+#include <lsarpcsrv.h>
+
+#include "rpcctl-register.h"
+#include "externs.h"
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
