@@ -50,12 +50,13 @@
 #define __AUTH_P_H__
 
 DWORD
-LsaSrvAuthenticateUser(
+LsaSrvAuthenticateUserEx(
     HANDLE hServer,
-    PCSTR  pszLoginId,
-    PCSTR  pszPassword
+    PLSA_AUTH_USER_PARAMS pUserParms,
+    PLSA_AUTH_USER_INFO *ppUserInfo
     );
 
+
 DWORD
 LsaSrvValidateUser(
     HANDLE hServer,
