@@ -52,38 +52,16 @@ LsaSrvRefreshConfiguration(
     HANDLE hServer
     );
 
-DWORD
-LsaSrvApiGetConfigFilePath(
-    PSTR* ppszConfigFilePath
-    );
-
 DWORD
 LsaSrvApiInitConfig(
     PLSA_SRV_API_CONFIG pConfig
     );
 
 DWORD
-LsaSrvApiReadConfig(
-    PCSTR pszConfigFilePath,
+LsaSrvApiReadRegistry(
     PLSA_SRV_API_CONFIG pConfig
     );
 
-DWORD
-LsaSrvApiConfigStartSection(
-    PCSTR    pszSectionName,
-    PVOID    pData,
-    PBOOLEAN pbSkipSection,
-    PBOOLEAN pbContinue
-    );
-
-DWORD
-LsaSrvApiConfigNameValuePair(
-    PCSTR    pszName,
-    PCSTR    pszValue,
-    PVOID    pData,
-    PBOOLEAN pbContinue
-    );
-
 DWORD
 LsaSrvApiTransferConfigContents(
     PLSA_SRV_API_CONFIG pSrc,
@@ -100,19 +78,28 @@ LsaSrvEventlogEnabled(
     VOID
     );
 
+BOOLEAN
+LsaSrvLogInvalidPasswords(
+    VOID
+    );
+
 VOID
 LsaSrvEnableEventlog(
     BOOLEAN bValue
     );
 
-BOOLEAN
-LsaSrvShouldLogNetworkConnectionEvents(
-    VOID
+DWORD
+LsaSrvSetMachineSid(
+    HANDLE hServer,
+    PCSTR pszSid
     );
 
-VOID
-LsaSrvSetLogNetworkConnectionEvents(
-    BOOLEAN bValue
+
+DWORD
+LsaSrvSetMachineName(
+    HANDLE hServer,
+    PCSTR  pszNewMachineName
     );
 
+
 #endif /* __CONFIG_P_H__ */
