@@ -51,19 +51,30 @@ time_t gServerStartTime = 0;
 PLSA_AUTH_PROVIDER gpAuthProviderList = NULL;
 pthread_rwlock_t gpAuthProviderList_rwlock;
 
+PLSA_RPC_SERVER gpRpcServerList = NULL;
+pthread_rwlock_t gpRpcServerList_rwlock;
+
+pthread_t gRpcSrvWorker;
+
 pthread_rwlock_t gPerfCounters_rwlock;
 UINT64 gPerfCounters[LsaMetricSentinel];
 
 pthread_mutex_t    gAPIConfigLock     = PTHREAD_MUTEX_INITIALIZER;
-PSTR               gpszConfigFilePath = NULL;
 LSA_SRV_API_CONFIG gAPIConfig = {0};
 
+PLW_MAP_SECURITY_CONTEXT gpLsaSecCtx;
+
+// Do not directly access this. Use gEventLogState
+EVENT_LOG_RECORD_QUEUE gEventLogQueues[2] = { {0}, {0} };
+EVENTLOG_THREAD_STATE gEventLogState = { (pthread_t)(size_t)-1, PTHREAD_COND_INITIALIZER, 0, PTHREAD_MUTEX_INITIALIZER, &gEventLogQueues[0] };
+
 DWORD
 LsaSrvApiInit(
-    PCSTR pszConfigFilePath
+    PLSA_STATIC_PROVIDER pStaticProviders
     )
 {
     DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
     LSA_SRV_API_CONFIG apiConfig = {0};
 
     gServerStartTime = time(NULL);
@@ -74,12 +85,12 @@ LsaSrvApiInit(
 
     pthread_rwlock_init(&gpAuthProviderList_rwlock, NULL);
 
+    pthread_rwlock_init(&gpRpcServerList_rwlock, NULL);
+
     dwError = LsaSrvApiInitConfig(&gAPIConfig);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaSrvApiReadConfig(
-                    pszConfigFilePath,
-                    &apiConfig);
+    dwError = LsaSrvApiReadRegistry(&apiConfig);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LsaSrvApiTransferConfigContents(
@@ -87,19 +98,32 @@ LsaSrvApiInit(
                     &gAPIConfig);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaSrvInitAuthProviders(pszConfigFilePath);
+    /*
+     * We must initialize lwmapsecurity before the auth providers
+     * because the pstore calls LwMapSecurityInitialize which
+     * causes us to use the external plugin
+     */
+    LwMapSecurityUseInternalPlugin(MapSecurityPluginCreateContext);
+
+    ntStatus = LwMapSecurityCreateContext(&gpLsaSecCtx);
+    if (ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaAllocateString(
-                    pszConfigFilePath,
-                    &gpszConfigFilePath);
+    dwError = LsaSrvInitAuthProviders(pStaticProviders);
     BAIL_ON_LSA_ERROR(dwError);
 
+    dwError = LsaSrvInitRpcServers();
+    BAIL_ON_LSA_ERROR(dwError);
+
+
 cleanup:
 
     LsaSrvApiFreeConfigContents(&apiConfig);
 
-    return 0;
+    return dwError;
 
 error:
 
@@ -113,13 +137,15 @@ LsaSrvApiShutdown(
 {
     LsaSrvFreeAuthProviders();
 
-    pthread_mutex_lock(&gAPIConfigLock);
+    LsaSrvFreeRpcServers();
 
-    LSA_SAFE_FREE_STRING(gpszConfigFilePath);
+    pthread_mutex_lock(&gAPIConfigLock);
 
     LsaSrvApiFreeConfigContents(&gAPIConfig);
 
     pthread_mutex_unlock(&gAPIConfigLock);
 
     return 0;
+
+
 }
