@@ -47,6 +47,40 @@
 
 #include "api.h"
 
+static
+VOID
+LsaFreeSecurityObjectPrivateAttrs(
+    PLSA_SECURITY_OBJECT pObject
+    )
+{
+    if (pObject)
+    {
+        if (pObject->type == LSA_OBJECT_TYPE_USER)
+        {
+            LW_SAFE_FREE_MEMORY(pObject->userInfo.pLmHash);
+            LW_SAFE_FREE_MEMORY(pObject->userInfo.pNtHash);
+        }
+    }
+}
+
+static
+VOID
+LsaFreeSecurityObjectListPrivateAttrs(
+    DWORD dwCount,
+    PLSA_SECURITY_OBJECT* ppObjectList
+    )
+{
+    DWORD dwIndex = 0;
+
+    if (ppObjectList)
+    {
+        for (dwIndex = 0; dwIndex < dwCount; dwIndex++)
+        {
+            LsaFreeSecurityObjectPrivateAttrs(ppObjectList[dwIndex]);
+        }
+    }
+}
+
 static
 DWORD
 LsaSrvIpcRegisterHandle(
@@ -377,7 +411,7 @@ error:
 
 
 static LWMsgStatus
-LsaSrvIpcAuthenticateUser(
+LsaSrvIpcAuthenticateUserPam(
     LWMsgCall* pCall,
     const LWMsgParams* pIn,
     LWMsgParams* pOut,
@@ -385,25 +419,36 @@ LsaSrvIpcAuthenticateUser(
     )
 {
     DWORD dwError = 0;
-    PLSA_IPC_AUTH_USER_REQ pReq = pIn->data;
+    PLSA_AUTH_USER_PAM_PARAMS pParams = pIn->data;
+    PLSA_AUTH_USER_PAM_INFO pInfo = NULL;
     PLSA_IPC_ERROR pError = NULL;
 
-    dwError = LsaSrvAuthenticateUser(
+    // The following call with *always* allocate pInfo
+
+    dwError = LsaSrvAuthenticateUserPam(
                         LsaSrvIpcGetSessionData(pCall),
-                        pReq->pszLoginName,
-                        pReq->pszPassword);
+                        pParams,
+                        &pInfo);
 
     if (!dwError)
     {
-        pOut->tag = LSA_R_AUTH_USER_SUCCESS;
-        pOut->data = NULL;
+        pOut->tag = LSA_R_AUTH_USER_PAM_SUCCESS;
+        pOut->data = pInfo;
+        pInfo = NULL;
     }
     else
     {
-        dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
+        dwError = LsaSrvIpcCreateError(
+                        dwError,
+                        pInfo ? pInfo->pszMessage : NULL,
+                        &pError);
         BAIL_ON_LSA_ERROR(dwError);
+        if (pInfo)
+        {
+            pInfo->pszMessage = NULL;
+        }
 
-        pOut->tag = LSA_R_AUTH_USER_FAILURE;
+        pOut->tag = LSA_R_AUTH_USER_PAM_FAILURE;
         pOut->data = pError;
     }
 
@@ -411,6 +456,10 @@ cleanup:
     return MAP_LW_ERROR_IPC(dwError);
 
 error:
+    if (pInfo)
+    {
+        LsaFreeAuthUserPamInfo(pInfo);
+    }
     goto cleanup;
 }
 
@@ -674,75 +723,6 @@ error:
 }
 
 
-static LWMsgStatus
-LsaSrvIpcSetMachineSid(
-    LWMsgCall *pCall,
-    const LWMsgParams *pIn,
-    LWMsgParams *pOut,
-    void *data
-    )
-{
-    DWORD dwError = 0;
-    PLSA_IPC_ERROR pError = NULL;
-    PLSA_IPC_SET_MACHINE_SID pReq = pIn->data;
-
-    dwError = LsaSrvSetMachineSid(LsaSrvIpcGetSessionData(pCall), pReq->pszSid);
-    if (!dwError)
-    {
-        pOut->tag  = LSA_R_SET_MACHINE_SID_SUCCESS;
-        pOut->data = NULL;
-    }
-    else
-    {
-        dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
-        BAIL_ON_LSA_ERROR(dwError);
-
-        pOut->tag    = LSA_R_SET_MACHINE_SID_FAILURE;
-        pOut->data = pError;
-    }
-
-cleanup:
-    return MAP_LW_ERROR_IPC(dwError);
-
-error:
-    goto cleanup;
-}
-
-static LWMsgStatus
-LsaSrvIpcSetMachineName(
-    LWMsgCall *pCall,
-    const LWMsgParams *pIn,
-    LWMsgParams *pOut,
-    void *data
-    )
-{
-    DWORD dwError = 0;
-    PLSA_IPC_ERROR pError = NULL;
-    PLSA_IPC_SET_MACHINE_NAME pReq = pIn->data;
-
-    dwError = LsaSrvSetMachineName(LsaSrvIpcGetSessionData(pCall),
-				   pReq->pszMachineName);
-    if (!dwError)
-    {
-        pOut->tag  = LSA_R_SET_MACHINE_NAME_SUCCESS;
-        pOut->data = NULL;
-    }
-    else
-    {
-        dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
-        BAIL_ON_LSA_ERROR(dwError);
-
-        pOut->tag  = LSA_R_SET_MACHINE_NAME_FAILURE;
-        pOut->data = pError;
-    }
-
-cleanup:
-    return MAP_LW_ERROR_IPC(dwError);
-
-error:
-    goto cleanup;
-}
-
 static LWMsgStatus
 LsaSrvIpcAddGroup2(
     LWMsgCall* pCall,
@@ -1510,6 +1490,10 @@ LsaSrvIpcFindObjects(
 
     if (!dwError)
     {
+        LsaFreeSecurityObjectListPrivateAttrs(
+            pReq->dwCount,
+            ppObjects);
+
         dwError = LwAllocateMemory(sizeof(*pRes), OUT_PPVOID(&pRes));
         BAIL_ON_LSA_ERROR(dwError);
 
@@ -1621,6 +1605,10 @@ LsaSrvIpcEnumObjects(
 
     if (!dwError)
     {
+        LsaFreeSecurityObjectListPrivateAttrs(
+            dwObjectsCount,
+            ppObjects);
+
         dwError = LwAllocateMemory(sizeof(*pRes), OUT_PPVOID(&pRes));
         BAIL_ON_LSA_ERROR(dwError);
 
@@ -1823,6 +1811,105 @@ error:
     goto cleanup;
 }
 
+static LWMsgStatus
+LsaSrvIpcFindGroupAndExpandedMembers(
+    LWMsgCall* pCall,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
+    void* data
+    )
+{
+    DWORD dwError = 0;
+    PLSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_REQ pReq = pIn->data;
+    PLSA2_IPC_FIND_GROUP_AND_EXPANDED_MEMBERS_RES pRes = NULL;
+    PLSA_SECURITY_OBJECT pGroupObject = NULL;
+    DWORD dwMemberObjectCount = 0;
+    PLSA_SECURITY_OBJECT* ppMemberObjects = NULL;
+    PLSA_IPC_ERROR pError = NULL;
+
+    switch (pReq->QueryType)
+    {
+    case LSA_QUERY_TYPE_BY_UNIX_ID:
+        if (pReq->IpcQueryType != LSA2_IPC_QUERY_DWORDS)
+        {
+            dwError = LW_ERROR_INTERNAL;
+        }
+        break;
+    case LSA_QUERY_TYPE_BY_DN:
+    case LSA_QUERY_TYPE_BY_SID:
+    case LSA_QUERY_TYPE_BY_NT4:
+    case LSA_QUERY_TYPE_BY_ALIAS:
+    case LSA_QUERY_TYPE_BY_UPN:
+    case LSA_QUERY_TYPE_BY_NAME:
+        if (pReq->IpcQueryType != LSA2_IPC_QUERY_STRINGS)
+        {
+            dwError = LW_ERROR_INTERNAL;
+        }
+        break;
+    default:
+        dwError = LW_ERROR_INTERNAL;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    if (!dwError)
+    {
+        dwError = LsaSrvFindGroupAndExpandedMembers(
+            LsaSrvIpcGetSessionData(pCall),
+            pReq->pszTargetProvider,
+            pReq->FindFlags,
+            pReq->QueryType,
+            pReq->QueryItem,
+            &pGroupObject,
+            &dwMemberObjectCount,
+            &ppMemberObjects);
+    }
+
+    if (!dwError)
+    {
+        LsaFreeSecurityObjectListPrivateAttrs(
+            dwMemberObjectCount,
+            ppMemberObjects);
+
+        dwError = LwAllocateMemory(sizeof(*pRes), OUT_PPVOID(&pRes));
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pRes->pGroup = pGroupObject;
+        pRes->dwMemberObjectCount = dwMemberObjectCount;
+        pRes->ppMemberObjects = ppMemberObjects;
+
+        pGroupObject = NULL;
+        ppMemberObjects = NULL;
+
+        pOut->tag = LSA2_R_FIND_GROUP_AND_EXPANDED_MEMBERS;
+        pOut->data = pRes;
+    }
+    else
+    {
+        dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pOut->tag = LSA2_R_ERROR;
+        pOut->data = pError;
+    }
+
+cleanup:
+
+    if (pGroupObject)
+    {
+        LsaUtilFreeSecurityObject(pGroupObject);
+    }
+    if (ppMemberObjects)
+    {
+        LsaUtilFreeSecurityObjectList(dwMemberObjectCount, ppMemberObjects);
+    }
+
+    return MAP_LW_ERROR_IPC(dwError);
+
+error:
+
+    goto cleanup;
+}
+
 static LWMsgStatus
 LsaSrvIpcCloseEnum(
     LWMsgCall* pCall,
@@ -1857,9 +1944,66 @@ error:
     goto cleanup;
 }
 
+static LWMsgStatus
+LsaSrvIpcGetSmartCardUserObject(
+    LWMsgCall* pCall,
+    const LWMsgParams* pIn,
+    LWMsgParams* pOut,
+    void* data
+    )
+{
+    DWORD dwError = 0;
+    PLSA2_IPC_GET_SMART_CARD_USER_RES pRes = NULL;
+    PLSA_SECURITY_OBJECT pObject = NULL;
+    PSTR pszSmartCardReader = NULL;
+
+    dwError = LsaSrvGetSmartCardUserObject(
+        LsaSrvIpcGetSessionData(pCall),
+        &pObject, &pszSmartCardReader);
+
+    if (!dwError)
+    {
+        dwError = LwAllocateMemory(sizeof(*pRes), OUT_PPVOID(&pRes));
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pRes->pObject = pObject;
+        pObject = NULL;
+        pRes->pszSmartCardReader = pszSmartCardReader;
+        pszSmartCardReader = NULL;
+
+        pOut->tag = LSA2_R_GET_SMARTCARD_USER_OBJECT;
+        pOut->data = pRes;
+    }
+    else
+    {
+        PLSA_IPC_ERROR pError = NULL;
+
+        dwError = LsaSrvIpcCreateError(dwError, NULL, &pError);
+        BAIL_ON_LSA_ERROR(dwError);
+
+        pOut->tag = LSA2_R_ERROR;
+        pOut->data = pError;
+    }
+
+cleanup:
+
+    if (pObject)
+    {
+        LsaUtilFreeSecurityObject(pObject);
+    }
+
+    LW_SAFE_FREE_STRING(pszSmartCardReader);
+
+    return MAP_LW_ERROR_IPC(dwError);
+
+error:
+
+    goto cleanup;
+}
+
 static LWMsgDispatchSpec gMessageHandlers[] =
 {
-    LWMSG_DISPATCH_BLOCK(LSA_Q_AUTH_USER, LsaSrvIpcAuthenticateUser),
+    LWMSG_DISPATCH_BLOCK(LSA_Q_AUTH_USER_PAM, LsaSrvIpcAuthenticateUserPam),
     LWMSG_DISPATCH_BLOCK(LSA_Q_AUTH_USER_EX, LsaSrvIpcAuthenticateUserEx),
     LWMSG_DISPATCH_BLOCK(LSA_Q_VALIDATE_USER, LsaSrvIpcValidateUser),
     LWMSG_DISPATCH_BLOCK(LSA_Q_CHANGE_PASSWORD, LsaSrvIpcChangePassword),
@@ -1880,8 +2024,6 @@ static LWMsgDispatchSpec gMessageHandlers[] =
     LWMSG_DISPATCH_BLOCK(LSA_Q_GET_TRACE_INFO, LsaSrvIpcGetTraceInfo),
     LWMSG_DISPATCH_BLOCK(LSA_Q_ENUM_TRACE_INFO, LsaSrvIpcEnumTraceInfo),
     LWMSG_DISPATCH_BLOCK(LSA_Q_PROVIDER_IO_CONTROL, LsaSrvIpcProviderIoControl),
-    LWMSG_DISPATCH_BLOCK(LSA_Q_SET_MACHINE_SID, LsaSrvIpcSetMachineSid),
-    LWMSG_DISPATCH_BLOCK(LSA_Q_SET_MACHINE_NAME, LsaSrvIpcSetMachineName),
     LWMSG_DISPATCH_BLOCK(LSA_Q_GET_PAM_CONFIG, LsaSrvIpcGetPamConfig),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_FIND_OBJECTS, LsaSrvIpcFindObjects),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_OPEN_ENUM_OBJECTS, LsaSrvIpcOpenEnumObjects),
@@ -1889,12 +2031,14 @@ static LWMsgDispatchSpec gMessageHandlers[] =
     LWMSG_DISPATCH_BLOCK(LSA2_Q_OPEN_ENUM_MEMBERS, LsaSrvIpcOpenEnumMembers),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_ENUM_MEMBERS, LsaSrvIpcEnumMembers),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_QUERY_MEMBER_OF, LsaSrvIpcQueryMemberOf),
+    LWMSG_DISPATCH_BLOCK(LSA2_Q_FIND_GROUP_AND_EXPANDED_MEMBERS, LsaSrvIpcFindGroupAndExpandedMembers),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_CLOSE_ENUM, LsaSrvIpcCloseEnum),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_MODIFY_USER, LsaSrvIpcModifyUser2),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_ADD_GROUP, LsaSrvIpcAddGroup2),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_MODIFY_GROUP, LsaSrvIpcModifyGroup2),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_DELETE_OBJECT, LsaSrvIpcDeleteObject),
     LWMSG_DISPATCH_BLOCK(LSA2_Q_ADD_USER, LsaSrvIpcAddUser2),
+    LWMSG_DISPATCH_BLOCK(LSA2_Q_GET_SMARTCARD_USER_OBJECT, LsaSrvIpcGetSmartCardUserObject),
     LWMSG_DISPATCH_END
 };
 
