@@ -61,9 +61,11 @@ LsaSrvOpenSession(
 
     LSA_TRACE_BEGIN_FUNCTION(dwTraceFlags, sizeof(dwTraceFlags)/sizeof(dwTraceFlags[0]));
 
+    BAIL_ON_INVALID_STRING(pszLoginId);
+
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
-    dwError = LSA_ERROR_NOT_HANDLED;
+    dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList;
          pProvider;
@@ -72,21 +74,25 @@ LsaSrvOpenSession(
         dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable->pfnOpenSession(
+        dwError = pProvider->pFnTable2->pfnOpenSession(
                                         hProvider,
                                         pszLoginId);
         if (!dwError) {
-
-           if (LsaSrvEventlogEnabled()){
-                  LsaSrvWriteLoginSuccessEvent(hServer,
-                                               pszLoginId,
-                                               dwError);
-           }
-
+            if (LsaSrvEventlogEnabled())
+            {
+                LsaSrvWriteLoginSuccessEvent(
+                    hServer,
+                    pProvider->pszName,
+                    pszLoginId,
+                    NULL,
+                    0,
+                    LSASS_EVENT_LOGON_PHASE_CREATE_SESSION,
+                    dwError);
+            }
            break;
 
-        } else if ((dwError == LSA_ERROR_NOT_HANDLED) ||
-                   (dwError == LSA_ERROR_NO_SUCH_USER)) {
+        } else if ((dwError == LW_ERROR_NOT_HANDLED) ||
+                   (dwError == LW_ERROR_NO_SUCH_USER)) {
 
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = (HANDLE)NULL;
@@ -94,9 +100,18 @@ LsaSrvOpenSession(
             continue;
 
         } else {
-
+            if (LsaSrvEventlogEnabled())
+            {
+                LsaSrvWriteLoginFailedEvent(
+                    hServer,
+                    pProvider->pszName,
+                    pszLoginId,
+                    NULL,
+                    0,
+                    LSASS_EVENT_LOGON_PHASE_CREATE_SESSION,
+                    dwError);
+            }
             BAIL_ON_LSA_ERROR(dwError);
-
         }
     }
 
@@ -122,12 +137,7 @@ cleanup:
     return dwError;
 
 error:
-
-    if (LsaSrvEventlogEnabled()){
-            LsaSrvWriteLoginFailedEvent(hServer,
-                                        pszLoginId,
-                                        dwError);
-    }
+    LSA_LOG_ERROR_API_FAILED(hServer, dwError, "open session for user (name = '%s')", LSA_SAFE_LOG_STRING(pszLoginId));
 
     goto cleanup;
 }
@@ -148,7 +158,7 @@ LsaSrvCloseSession(
 
     ENTER_AUTH_PROVIDER_LIST_READER_LOCK(bInLock);
 
-    dwError = LSA_ERROR_NOT_HANDLED;
+    dwError = LW_ERROR_NOT_HANDLED;
 
     for (pProvider = gpAuthProviderList;
          pProvider;
@@ -157,21 +167,23 @@ LsaSrvCloseSession(
         dwError = LsaSrvOpenProvider(hServer, pProvider, &hProvider);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = pProvider->pFnTable->pfnCloseSession(
+        dwError = pProvider->pFnTable2->pfnCloseSession(
                                 hProvider,
                                 pszLoginId);
         if (!dwError) {
 
-            if (LsaSrvEventlogEnabled()){
+            if (LsaSrvEventlogEnabled())
+            {
                     LsaSrvWriteLogoutSuccessEvent(hServer,
-                                                  pszLoginId,
-                                                  dwError);
+                                                  pProvider->pszName,
+                                                  LSASS_EVENT_LOGON_PHASE_CREATE_SESSION,
+                                                  pszLoginId);
             }
 
            break;
 
-        } else if ((dwError == LSA_ERROR_NOT_HANDLED) ||
-                   (dwError == LSA_ERROR_NO_SUCH_USER)) {
+        } else if ((dwError == LW_ERROR_NOT_HANDLED) ||
+                   (dwError == LW_ERROR_NO_SUCH_USER)) {
 
             LsaSrvCloseProvider(pProvider, hProvider);
             hProvider = (HANDLE)NULL;
@@ -207,12 +219,7 @@ cleanup:
     return dwError;
 
 error:
-
-    if (LsaSrvEventlogEnabled()){
-            LsaSrvWriteLogoutFailedEvent(hServer,
-                                         pszLoginId,
-                                         dwError);
-    }
+    LSA_LOG_ERROR_API_FAILED(hServer, dwError, "close session for user (name = '%s')", LSA_SAFE_LOG_STRING(pszLoginId));
 
     goto cleanup;
 }
