@@ -49,48 +49,42 @@
 
 #include "auth_provider_p.h"
 
-typedef struct __LSA_SRV_PROVIDER_STATE {
-
+typedef struct __LSA_SRV_PROVIDER_STATE
+{
     PLSA_AUTH_PROVIDER pProvider;
     HANDLE hProvider;
     HANDLE hResume;
 
-    struct __LSA_SRV_PROVIDER_STATE *pNext;
-
+    struct __LSA_SRV_PROVIDER_STATE* pNext;
 } LSA_SRV_PROVIDER_STATE, *PLSA_SRV_PROVIDER_STATE;
 
-typedef struct __LSA_SRV_RECORD_ENUM_STATE {
-
-    PSTR    pszGUID;
-    DWORD   dwInfoLevel;
-    DWORD   dwNumMaxRecords;
-    DWORD   dwMapFlags;
-    PSTR    pszMapName;
-
-    BOOLEAN bInLock;
-
+typedef struct __LSA_SRV_ENUM_STATE
+{
+    DWORD dwNumMaxRecords;
+    DWORD dwInfoLevel;
+    DWORD dwMapFlags;
+    BOOLEAN bCheckOnline;
+    LSA_FIND_FLAGS FindFlags;
+    PSTR  pszMapName;
     PLSA_SRV_PROVIDER_STATE pProviderStateList;
     PLSA_SRV_PROVIDER_STATE pCurProviderState;
 
-    struct __LSA_SRV_RECORD_ENUM_STATE * pNext;
-
-} LSA_SRV_RECORD_ENUM_STATE, *PLSA_SRV_RECORD_ENUM_STATE;
+    BOOLEAN bInLock;
+} LSA_SRV_ENUM_STATE, *PLSA_SRV_ENUM_STATE;
 
 typedef struct __LSA_SRV_API_STATE
 {
     uid_t  peerUID;
     gid_t  peerGID;
-
-    HANDLE hEventLog;
-
-    PLSA_SRV_RECORD_ENUM_STATE pUserEnumStateList;
-
-    PLSA_SRV_RECORD_ENUM_STATE pGroupEnumStateList;
-
-    PLSA_SRV_RECORD_ENUM_STATE pNSSArtefactEnumStateList;
-
+    gid_t  peerPID;
 } LSA_SRV_API_STATE, *PLSA_SRV_API_STATE;
 
+DWORD
+LsaSrvFindProviderByName(
+    IN PCSTR pszProvider,
+    OUT PLSA_AUTH_PROVIDER* ppProvider
+    );
+
 DWORD
 LsaSrvOpenProvider(
     HANDLE  hServer,
@@ -104,6 +98,16 @@ LsaSrvCloseProvider(
     HANDLE hProvider
     );
 
+DWORD
+LsaSrvCreateNSSArtefactEnumState(
+    HANDLE  hServer,
+    PCSTR   pszMapName,
+    LSA_NIS_MAP_QUERY_FLAGS dwFlags,
+    DWORD   dwNSSArtefactInfoLevel,
+    DWORD   dwMaxNumArtefacts,
+    PLSA_SRV_ENUM_STATE* ppEnumState
+    );
+
 VOID
 LsaSrvFreeProviderStateList(
     PLSA_SRV_PROVIDER_STATE pStateList
@@ -114,68 +118,10 @@ LsaSrvReverseProviderStateList(
     PLSA_SRV_PROVIDER_STATE pStateList
     );
 
-DWORD
-LsaSrvAddUserEnumState(
-    HANDLE  hServer,
-    DWORD   dwUserInfoLevel,
-    DWORD   dwMaxNumUsers,
-    PLSA_SRV_RECORD_ENUM_STATE* ppEnumState
-    );
-
-PLSA_SRV_RECORD_ENUM_STATE
-LsaSrvFindUserEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
 VOID
-LsaSrvFreeUserEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
+LsaSrvFreeEnumState(
+    PLSA_SRV_ENUM_STATE pState
     );
 
-DWORD
-LsaSrvAddGroupEnumState(
-    HANDLE  hServer,
-    DWORD   dwGroupInfoLevel,
-    DWORD   dwMaxNumGroups,
-    PLSA_SRV_RECORD_ENUM_STATE* ppEnumState
-    );
-
-PLSA_SRV_RECORD_ENUM_STATE
-LsaSrvFindGroupEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
-VOID
-LsaSrvFreeGroupEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
-DWORD
-LsaSrvAddNSSArtefactEnumState(
-    HANDLE  hServer,
-    PCSTR   pszMapName,
-    LSA_NIS_MAP_QUERY_FLAGS dwFlags,
-    DWORD   dwNSSArtefactInfoLevel,
-    DWORD   dwMaxNumArtefacts,
-    PLSA_SRV_RECORD_ENUM_STATE* ppEnumState
-    );
-
-PLSA_SRV_RECORD_ENUM_STATE
-LsaSrvFindNSSArtefactEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
-VOID
-LsaSrvFreeNSSArtefactEnumState(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
-
 #endif /* __STATE_P_H__ */
 
