@@ -84,8 +84,16 @@ LocalSyncDomainInfo(
     DWORD dwNewLockoutThreshold = LOCAL_LOCKOUT_THRESHOLD;
     DWORD dwNewLockoutDuration = LOCAL_LOCKOUT_DURATION;
     DWORD dwNewLockoutWindow = LOCAL_LOCKOUT_WINDOW;
+    DWORD dwMinUid = LOWEST_UID;;
+    DWORD dwNewMinUid = LOWEST_UID;;
+    DWORD dwMaxUid = HIGHEST_UID;;
+    DWORD dwNewMaxUid = HIGHEST_UID;;
+    DWORD dwMinGid = LOWEST_GID;;
+    DWORD dwNewMinGid = LOWEST_GID;;
+    DWORD dwMaxGid = HIGHEST_GID;;
+    DWORD dwNewMaxGid = HIGHEST_GID;;
     HANDLE hDirectory = NULL;
-    PSTR pszFilterFmt = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d";
+    PSTR pszFilterFmt = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
     DWORD dwDomainObjectClass = LOCAL_OBJECT_CLASS_DOMAIN;
     PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
@@ -135,7 +143,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewMinPasswordAge
+            &dwNewMinPasswordAge,
+            NULL
         },
         {
             "MaxPasswordAge",
@@ -144,7 +153,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewMaxPasswordAge
+            &dwNewMaxPasswordAge,
+            NULL
         },
         {
             "MinPasswordLength",
@@ -153,7 +163,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewMinPasswordLength
+            &dwNewMinPasswordLength,
+            NULL
         },
         {
             "PasswordPromptTime",
@@ -162,7 +173,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewPasswordPromptTime
+            &dwNewPasswordPromptTime,
+            NULL
         },
         {
             "LockoutThreshold",
@@ -171,7 +183,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewLockoutThreshold
+            &dwNewLockoutThreshold,
+            NULL
         },
         {
             "LockoutDuration",
@@ -180,7 +193,8 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewLockoutDuration
+            &dwNewLockoutDuration,
+            NULL
         },
         {
             "LockoutWindow",
@@ -189,7 +203,48 @@ LocalSyncDomainInfo(
             0,
             MAXDWORD,
             NULL,
-            &dwNewLockoutWindow
+            &dwNewLockoutWindow,
+            NULL
+        },
+        {
+            "MaxUid",
+            TRUE,
+            LsaTypeDword,
+            0,
+            MAXDWORD,
+            NULL,
+            &dwNewMaxUid,
+            NULL
+        },
+        {
+            "MaxGid",
+            TRUE,
+            LsaTypeDword,
+            0,
+            MAXDWORD,
+            NULL,
+            &dwNewMaxGid,
+            NULL
+        },
+        {
+            "MinUid",
+            TRUE,
+            LsaTypeDword,
+            0,
+            MAXDWORD,
+            NULL,
+            &dwNewMinUid,
+            NULL
+        },
+        {
+            "MinGid",
+            TRUE,
+            LsaTypeDword,
+            0,
+            MAXDWORD,
+            NULL,
+            &dwNewMinGid,
+            NULL
         }
     };
 
@@ -338,7 +393,7 @@ LocalSyncDomainInfo(
                     dwDomainObjectClass);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -532,6 +587,34 @@ LocalSyncDomainInfo(
         llLockoutWindow = llNewLockoutWindow;
     }
 
+    /* MaxUid */
+    if (dwMaxUid != dwNewMaxUid)
+    {
+        // Currently not stored in domain database, so we'll just update the global for now
+        dwMaxUid = dwNewMaxUid;
+    }
+
+    /* MaxGid */
+    if (dwMaxGid != dwNewMaxGid)
+    {
+        // Currently not stored in domain database, so we'll just update the global for now
+        dwMaxGid = dwNewMaxGid;
+    }
+
+    /* MinUid */
+    if (dwMinUid != dwNewMinUid)
+    {
+        // Currently not stored in domain database, so we'll just update the global for now
+        dwMinUid = dwNewMinUid;
+    }
+
+    /* MinGid */
+    if (dwMinGid != dwNewMinGid)
+    {
+        // Currently not stored in domain database, so we'll just update the global for now
+        dwMinGid = dwNewMinGid;
+    }
+
     /*
      * Update domain object with current SAM configuration if
      * there was any change found
@@ -574,13 +657,25 @@ LocalSyncDomainInfo(
     pGlobals->dwLockoutThreshold   = dwLockoutThreshold;
     pGlobals->llLockoutDuration    = llLockoutDuration;
     pGlobals->llLockoutWindow      = llLockoutWindow;
+    pGlobals->dwMaxUid             = dwMaxUid;
+    pGlobals->dwMaxGid             = dwMaxGid;
+    pGlobals->dwMinUid             = dwMinUid;
+    pGlobals->dwMinGid             = dwMinGid;
 
 cleanup:
+    if (hDirectory)
+    {
+        DirectoryClose(hDirectory);
+    }
+
     if (pEntries)
     {
         DirectoryFreeEntries(pEntries, dwNumEntries);
     }
 
+    LW_SAFE_FREE_MEMORY(pszFilter);
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
     {
@@ -609,7 +704,7 @@ LocalGetDomainInfo(
     DWORD  dwError = 0;
     HANDLE hDirectory  = NULL;
     DWORD  objectClass = LOCAL_OBJECT_CLASS_DOMAIN;
-    PCSTR  pszFilterTemplate = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d";
+    PCSTR  pszFilterTemplate = LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
     PSTR   pszFilter = NULL;
     PWSTR  pwszFilter = NULL;
     wchar16_t wszAttrNameDomain[]      = LOCAL_DIR_ATTR_DOMAIN;
@@ -645,7 +740,7 @@ LocalGetDomainInfo(
                     objectClass);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -805,7 +900,7 @@ LocalGetSingleStringAttrValue(
 
     if (pAttrs[0].data.pwszStringValue)
     {
-        dwError = LsaWc16sToMbs(
+        dwError = LwWc16sToMbs(
                         pAttrs[0].data.pwszStringValue,
                         &pszValue);
         BAIL_ON_LSA_ERROR(dwError);
@@ -857,6 +952,403 @@ error:
 }
 
 
+DWORD
+LocalDirSetDomainName(
+    IN PCSTR  pszNewDomainName
+    )
+{
+    const wchar_t wszDomainFilterFmt[] = L"%ws=%d";
+    const DWORD dwInt32StrSize = 10;
+
+    DWORD dwError = 0;
+    PWSTR pwszNewDomainName = NULL;
+    PWSTR pwszDomainObjectDN = NULL;
+    HANDLE hDirectory = (HANDLE)NULL;
+    PWSTR pwszDomainFilter = NULL;
+    DWORD dwDomainFilterLen = 0;
+    PWSTR pwszBase = NULL;
+    ULONG ulScope = 0;
+    ULONG ulAttributesOnly = 0;
+    PDIRECTORY_ENTRY pDomainEntries = NULL;
+    PDIRECTORY_ENTRY pDomainEntry = NULL;
+    DWORD dwNumDomainEntries = 0;
+    DWORD iMod = 0;
+    PWSTR pwszUserDN = NULL;
+    PWSTR pwszCredentials = NULL;
+    ULONG ulMethod = 0;
+    BOOLEAN bLocked = FALSE;
+
+    WCHAR wszAttrObjectClass[] = DIRECTORY_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrObjectDN[] = DIRECTORY_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrDomainName[] = DIRECTORY_ATTR_DOMAIN_NAME;
+    WCHAR wszAttrNetbiosName[] = DIRECTORY_ATTR_NETBIOS_NAME;
+    WCHAR wszAttrCommonName[] = DIRECTORY_ATTR_COMMON_NAME;
+    WCHAR wszAttrSamAccountName[] = DIRECTORY_ATTR_SAM_ACCOUNT_NAME;
+
+    PWSTR wszAttributes[] = {
+        &wszAttrObjectDN[0],
+        NULL
+    };
+
+    enum AttrValueIndex {
+        ATTR_IDX_DOMAIN_NAME  = 0,
+        ATTR_IDX_NETBIOS_NAME,
+        ATTR_IDX_COMMON_NAME,
+        ATTR_IDX_SAM_ACCOUNT_NAME,
+        ATTR_IDX_SENTINEL
+    };
+
+    ATTRIBUTE_VALUE AttrValues[] = {
+        {   /* ATTR_IDX_DOMAIN_NAME */
+            .Type = DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+            .data.pwszStringValue = NULL
+        },
+        {   /* ATTR_IDX_NETBIOS_NAME */
+            .Type = DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+            .data.pwszStringValue = NULL
+        },
+        {   /* ATTR_IDX_COMMON_NAME */
+            .Type = DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+            .data.pwszStringValue = NULL
+        },
+        {   /* ATTR_IDX_SAM_ACCOUNT_NAME */
+            .Type = DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+            .data.pwszStringValue = NULL
+        },
+    };
+
+    DIRECTORY_MOD modDomainName = {
+        DIR_MOD_FLAGS_REPLACE,
+        &wszAttrDomainName[0],
+        1,
+        &AttrValues[ATTR_IDX_DOMAIN_NAME]
+    };
+
+    DIRECTORY_MOD modNetbiosName = {
+        DIR_MOD_FLAGS_REPLACE,
+        &wszAttrNetbiosName[0],
+        1,
+        &AttrValues[ATTR_IDX_NETBIOS_NAME]
+    };
+
+    DIRECTORY_MOD modCommonName = {
+        DIR_MOD_FLAGS_REPLACE,
+        &wszAttrCommonName[0],
+        1,
+        &AttrValues[ATTR_IDX_COMMON_NAME]
+    };
+
+    DIRECTORY_MOD modSamAccountName = {
+        DIR_MOD_FLAGS_REPLACE,
+        &wszAttrSamAccountName[0],
+        1,
+        &AttrValues[ATTR_IDX_SAM_ACCOUNT_NAME]
+    };
+
+    DIRECTORY_MOD mods[ATTR_IDX_SENTINEL + 1];
+
+    BAIL_ON_INVALID_POINTER(pszNewDomainName);
+
+    if (strlen(pszNewDomainName) > 15)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    /*
+     * Freeze local provider to prevent from changing
+     * database contents in the background
+     */
+    LOCAL_WRLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    dwError = LwMbsToWc16s(pszNewDomainName,
+                           &pwszNewDomainName);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectoryOpen(&hDirectory);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /*
+     * Search and modify MACHINE domain object
+     */
+
+    dwDomainFilterLen = ((sizeof(wszAttrObjectClass)/sizeof(WCHAR) - 1) +
+                         dwInt32StrSize +
+                         (sizeof(wszDomainFilterFmt)/
+                          sizeof(wszDomainFilterFmt[0])));
+    dwError = LwAllocateMemory(dwDomainFilterLen * sizeof(WCHAR),
+                               (PVOID*)&pwszDomainFilter);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (sw16printfw(pwszDomainFilter, dwDomainFilterLen, wszDomainFilterFmt,
+                    &wszAttrObjectClass[0],
+                    DIR_OBJECT_CLASS_DOMAIN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectorySearch(hDirectory,
+                              pwszBase,
+                              ulScope,
+                              pwszDomainFilter,
+                              wszAttributes,
+                              ulAttributesOnly,
+                              &pDomainEntries,
+                              &dwNumDomainEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumDomainEntries != 1)
+    {
+        dwError = LW_ERROR_SAM_DATABASE_ERROR;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    pDomainEntry = &(pDomainEntries[0]);
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                                pDomainEntry,
+                                wszAttrObjectDN,
+                                DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                                &pwszDomainObjectDN);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    iMod = 0;
+    memset(&mods[0], 0, sizeof(mods));
+
+    AttrValues[ATTR_IDX_DOMAIN_NAME].data.pwszStringValue  = pwszNewDomainName;
+    AttrValues[ATTR_IDX_NETBIOS_NAME].data.pwszStringValue = pwszNewDomainName;
+    AttrValues[ATTR_IDX_COMMON_NAME].data.pwszStringValue  = pwszNewDomainName;
+    AttrValues[ATTR_IDX_SAM_ACCOUNT_NAME].data.pwszStringValue
+        = pwszNewDomainName;
+
+    mods[iMod++] = modDomainName;
+    mods[iMod++] = modNetbiosName;
+    mods[iMod++] = modCommonName;
+    mods[iMod++] = modSamAccountName;
+
+    dwError = DirectoryModifyObject(hDirectory,
+                                    pwszDomainObjectDN,
+                                    mods);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /*
+     * Reload local domain info
+     */
+
+    LW_SAFE_FREE_STRING(gLPGlobals.pszLocalDomain);
+    LW_SAFE_FREE_STRING(gLPGlobals.pszNetBIOSName);
+    LW_SAFE_FREE_MEMORY(gLPGlobals.pLocalDomainSID);
+
+    dwError = LocalGetDomainInfo(
+                    pwszUserDN,
+                    pwszCredentials,
+                    ulMethod,
+                    &gLPGlobals.pszLocalDomain,
+                    &gLPGlobals.pszNetBIOSName,
+                    &gLPGlobals.pLocalDomainSID,
+                    &gLPGlobals.llMaxPwdAge,
+                    &gLPGlobals.llPwdChangeTime);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (pDomainEntries)
+    {
+        DirectoryFreeEntries(pDomainEntries,
+                             dwNumDomainEntries);
+    }
+
+    if (hDirectory)
+    {
+        DirectoryClose(hDirectory);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszNewDomainName);
+    LW_SAFE_FREE_MEMORY(pwszDomainFilter);
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
+DWORD
+LocalDirSetDomainSid(
+    IN PCSTR  pszSid
+    )
+{
+    const wchar_t wszDomainFilterFmt[] = L"%ws=%d";
+    const DWORD dwInt32StrSize = 10;
+
+    DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    PWSTR pwszNewDomainSid = NULL;
+    PSID pNewDomainSid = NULL;
+    HANDLE hDirectory = (HANDLE)NULL;
+    PWSTR pwszDomainFilter = NULL;
+    DWORD dwDomainFilterLen = 0;
+    PWSTR pwszBase = NULL;
+    ULONG ulScope = 0;
+    ULONG ulAttributesOnly = 0;
+    PDIRECTORY_ENTRY pDomainEntries = NULL;
+    PDIRECTORY_ENTRY pDomainEntry = NULL;
+    DWORD dwNumDomainEntries = 0;
+    PWSTR pwszDomainObjectDN = NULL;
+    PWSTR pwszUserDN = NULL;
+    PWSTR pwszCredentials = NULL;
+    ULONG ulMethod = 0;
+    BOOLEAN bLocked = FALSE;
+
+    WCHAR wszAttrObjectDN[] = DIRECTORY_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrObjectClass[] = DIRECTORY_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrObjectSID[] = DIRECTORY_ATTR_OBJECT_SID;
+
+    PWSTR wszAttributes[] = {
+        &wszAttrObjectDN[0],
+        NULL
+    };
+
+    enum AttrValueIndex {
+        ATTR_IDX_OBJECT_SID = 0,
+        ATTR_IDX_SENTINEL
+    };
+
+    ATTRIBUTE_VALUE AttrValues[] = {
+        {
+            .Type = DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+            .data.pwszStringValue = NULL
+        }
+    };
+
+    DIRECTORY_MOD modObjectSID = {
+        DIR_MOD_FLAGS_REPLACE,
+        &wszAttrObjectSID[0],
+        1,
+        &AttrValues[ATTR_IDX_OBJECT_SID]
+    };
+
+    DIRECTORY_MOD mods[ATTR_IDX_SENTINEL + 1];
+
+
+    /*
+     * Freeze local provider to prevent from changing
+     * database contents in the background
+     */
+    LOCAL_WRLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    dwError = LwMbsToWc16s(pszSid,
+                           &pwszNewDomainSid);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlAllocateSidFromWC16String(&pNewDomainSid,
+                                            pwszNewDomainSid);
+    if (ntStatus != STATUS_SUCCESS)
+    {
+        dwError = ERROR_INVALID_PARAMETER;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectoryOpen(&hDirectory);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwDomainFilterLen = ((sizeof(wszAttrObjectClass)/sizeof(WCHAR) - 1) +
+                         dwInt32StrSize +
+                         (sizeof(wszDomainFilterFmt)/
+                          sizeof(wszDomainFilterFmt[0])));
+    dwError = LwAllocateMemory(dwDomainFilterLen * sizeof(WCHAR),
+                               (PVOID*)&pwszDomainFilter);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (sw16printfw(pwszDomainFilter, dwDomainFilterLen, wszDomainFilterFmt,
+                    &wszAttrObjectClass[0],
+                    DIR_OBJECT_CLASS_DOMAIN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectorySearch(hDirectory,
+                              pwszBase,
+                              ulScope,
+                              pwszDomainFilter,
+                              wszAttributes,
+                              ulAttributesOnly,
+                              &pDomainEntries,
+                              &dwNumDomainEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumDomainEntries != 1)
+    {
+        dwError = LW_ERROR_SAM_DATABASE_ERROR;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    pDomainEntry = &(pDomainEntries[0]);
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                                pDomainEntry,
+                                wszAttrObjectDN,
+                                DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                                &pwszDomainObjectDN);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    memset(&mods[0], 0, sizeof(mods));
+
+    AttrValues[ATTR_IDX_OBJECT_SID].data.pwszStringValue = pwszNewDomainSid;
+    mods[0] = modObjectSID;
+
+    dwError = DirectoryModifyObject(hDirectory,
+                                    pwszDomainObjectDN,
+                                    mods);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /*
+     * Reload local domain info
+     */
+
+    LW_SAFE_FREE_STRING(gLPGlobals.pszLocalDomain);
+    LW_SAFE_FREE_STRING(gLPGlobals.pszNetBIOSName);
+    LW_SAFE_FREE_MEMORY(gLPGlobals.pLocalDomainSID);
+
+    dwError = LocalGetDomainInfo(
+                    pwszUserDN,
+                    pwszCredentials,
+                    ulMethod,
+                    &gLPGlobals.pszLocalDomain,
+                    &gLPGlobals.pszNetBIOSName,
+                    &gLPGlobals.pLocalDomainSID,
+                    &gLPGlobals.llMaxPwdAge,
+                    &gLPGlobals.llPwdChangeTime);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (pDomainEntries)
+    {
+        DirectoryFreeEntries(pDomainEntries,
+                             dwNumDomainEntries);
+    }
+
+    if (hDirectory)
+    {
+        DirectoryClose(hDirectory);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszDomainFilter);
+    LW_SAFE_FREE_MEMORY(pwszNewDomainSid);
+    RTL_FREE(&pNewDomainSid);
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
 /*
 local variables:
 mode: c
