@@ -86,6 +86,7 @@ LocalDirAddGroup(
     )
 {
     DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
     BOOLEAN bEventlogEnabled = FALSE;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
@@ -127,12 +128,16 @@ LocalDirAddGroup(
                 .data.ulValue = pGroupInfo->gid
         }
     };
-    WCHAR wszAttrObjectClass[]    = LOCAL_DIR_ATTR_OBJECT_CLASS;
+
+    WCHAR wszAttrObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrDistinguishedName[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrObjectSID[] = LOCAL_DIR_ATTR_OBJECT_SID;
     WCHAR wszAttrSamAccountName[] = LOCAL_DIR_ATTR_SAM_ACCOUNT_NAME;
-    WCHAR wszAttrCommonName[]     = LOCAL_DIR_ATTR_COMMON_NAME;
-    WCHAR wszAttrDomain[]         = LOCAL_DIR_ATTR_DOMAIN;
-    WCHAR wszAttrNameNetBIOSDomain[]  = LOCAL_DIR_ATTR_NETBIOS_NAME;
+    WCHAR wszAttrCommonName[] = LOCAL_DIR_ATTR_COMMON_NAME;
+    WCHAR wszAttrDomain[] = LOCAL_DIR_ATTR_DOMAIN;
+    WCHAR wszAttrNameNetBIOSDomain[] = LOCAL_DIR_ATTR_NETBIOS_NAME;
     WCHAR wszAttrNameGID[] = LOCAL_DIR_ATTR_GID;
+
     DIRECTORY_MOD modObjectClass =
     {
         DIR_MOD_FLAGS_ADD,
@@ -180,42 +185,73 @@ LocalDirAddGroup(
     PWSTR pwszSamAccountName = NULL;
     PWSTR pwszDomain = NULL;
     PWSTR pwszNetBIOSDomain = NULL;
+    size_t sGroupDNLen = 0;
+    DWORD dwFilterLen = 0;
+    PWSTR pwszFilter = NULL;
+    wchar_t wszFilterFmt[] = L"%ws = '%ws'";
+    PWSTR pwszBase = NULL;
+    DWORD dwScope = 0;
+
+    PWSTR wszGroupAttrs[] = {
+        wszAttrDistinguishedName,
+        wszAttrObjectSID,
+        NULL
+    };
+
+    PDIRECTORY_ENTRY pGroup = NULL;
+    DWORD dwNumEntries = 0;
+    PWSTR pwszGroupSID = NULL;
+    PSID pGroupSID = NULL;
+    DWORD dwGroupRID = 0;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
+    BOOLEAN bLocked = FALSE;
 
     memset(&mods[0], 0, sizeof(mods));
 
     BAIL_ON_INVALID_STRING(pGroupInfo->pszName);
 
-    if (pGroupInfo->gid) {
+    if (pGroupInfo->gid)
+    {
         dwError = LocalDirValidateGID(pGroupInfo->gid);
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwError = LocalCrackDomainQualifiedName(
+    dwError = LsaSrvCrackDomainQualifiedName(
                     pGroupInfo->pszName,
                     &pLoginInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (!LocalServicesDomain(pLoginInfo->pszDomainNetBiosName))
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (!pLoginInfo->pszDomain)
+    {
+        dwError = LwAllocateString(
+                        gLPGlobals.pszNetBIOSName,
+                        &pLoginInfo->pszDomain);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    if (!LocalServicesDomain(pLoginInfo->pszDomain))
     {
         dwError = LW_ERROR_NOT_HANDLED;
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwError = LsaMbsToWc16s(
-                    pLoginInfo->pszFullDomainName,
+    dwError = LwMbsToWc16s(
+                    pLoginInfo->pszDomain,
                     &pwszDomain);
     BAIL_ON_LSA_ERROR(dwError);
 
     attrValues[LOCAL_DAG1_IDX_DOMAIN].data.pwszStringValue = pwszDomain;
 
-    dwError = LsaMbsToWc16s(
-                    pLoginInfo->pszDomainNetBiosName,
+    dwError = LwMbsToWc16s(
+                    pLoginInfo->pszDomain,
                     &pwszNetBIOSDomain);
     BAIL_ON_LSA_ERROR(dwError);
 
     attrValues[LOCAL_DAG1_IDX_NETBIOS_DOMAIN].data.pwszStringValue = pwszNetBIOSDomain;
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pGroupInfo->pszName,
                     &pwszSamAccountName);
     BAIL_ON_LSA_ERROR(dwError);
@@ -244,6 +280,69 @@ LocalDirAddGroup(
                     mods);
     BAIL_ON_LSA_ERROR(dwError);
 
+    dwError = LwWc16sLen(pwszGroupDN,
+                         &sGroupDNLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwFilterLen = (sizeof(wszAttrDistinguishedName) - 2) +
+                  (sGroupDNLen * sizeof(WCHAR)) +
+                   sizeof(wszFilterFmt);
+
+    dwError = LwAllocateMemory(
+                    dwFilterLen,
+                    OUT_PPVOID(&pwszFilter));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (sw16printfw(pwszFilter, dwFilterLen/sizeof(WCHAR), wszFilterFmt,
+                    wszAttrDistinguishedName, pwszGroupDN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectorySearch(
+                    pContext->hDirectory,
+                    pwszBase,
+                    dwScope,
+                    pwszFilter,
+                    wszGroupAttrs,
+                    0,
+                    &pGroup,
+                    &dwNumEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /*
+     * Set default security descriptor on the account
+     */
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                    pGroup,
+                    wszAttrObjectSID,
+                    DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                    &pwszGroupSID);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlAllocateSidFromWC16String(&pGroupSID,
+                                            pwszGroupSID);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = RtlGetRidSid(&dwGroupRID,
+                            pGroupSID);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    dwError = LocalDirCreateNewAccountSecurityDescriptor(
+                    gLPGlobals.pLocalDomainSID,
+                    dwGroupRID,
+                    DIR_OBJECT_CLASS_LOCAL_GROUP,
+                    &pSecDesc);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectorySetEntrySecurityDescriptor(
+                                  pContext->hDirectory,
+                                  pwszGroupDN,
+                                  pSecDesc);
+    BAIL_ON_LSA_ERROR(dwError);
+
     if (pGroupInfo->ppszMemberSids)
     {
         dwError = LocalAddMembersToGroup(
@@ -264,16 +363,32 @@ LocalDirAddGroup(
     }
 
 cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
 
     if (pLoginInfo)
     {
-        LsaFreeNameInfo(pLoginInfo);
+        LsaSrvFreeNameInfo(pLoginInfo);
+    }
+
+    if (pGroup)
+    {
+        DirectoryFreeEntries(pGroup, dwNumEntries);
     }
 
     LW_SAFE_FREE_MEMORY(pwszGroupDN);
     LW_SAFE_FREE_MEMORY(pwszSamAccountName);
     LW_SAFE_FREE_MEMORY(pwszDomain);
     LW_SAFE_FREE_MEMORY(pwszNetBIOSDomain);
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+    RTL_FREE(&pGroupSID);
+
+    LocalDirFreeSecurityDescriptor(&pSecDesc);
+
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
 
     return dwError;
 
@@ -289,11 +404,23 @@ LocalDirValidateGID(
     )
 {
     DWORD dwError = 0;
+    DWORD dwGidMin = LOWEST_GID;
+    DWORD dwGidMax = HIGHEST_GID;
+    DWORD dwGid = (DWORD) gid;
+
+    dwError = LocalCfgGetMinGid(&dwGidMin);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LocalCfgGetMaxGid(&dwGidMax);
+    BAIL_ON_LSA_ERROR(dwError);
 
     /* Check whether group gid is within permitted range */
-    if (gid < LOWEST_GID) {
-        dwError = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(dwError);
+    if (dwGid < dwGidMin) {
+        BAIL_WITH_LSA_ERROR(LW_ERROR_GID_TOO_LOW);
+    }
+
+    if (dwGid > dwGidMax) {
+        BAIL_WITH_LSA_ERROR(LW_ERROR_GID_TOO_HIGH);
     }
 
 cleanup:
@@ -324,7 +451,7 @@ LocalDirModifyGroup(
     DWORD dwObjectClassLocalUser = LOCAL_OBJECT_CLASS_USER;
     PWSTR pwszBase = NULL;
     ULONG ulScope = 0;
-    wchar_t wszFilterFmtSidOnly[] = L"(%ws=%d OR %ws=%d) AND %ws=\'%ws\'";
+    wchar_t wszFilterFmtSidOnly[] = L"(%ws=%u OR %ws=%u) AND %ws=\'%ws\'";
     PWSTR pwszFilter = NULL;
     DWORD dwFilterLen = 0;
     WCHAR wszAttrObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
@@ -427,7 +554,7 @@ LocalDirModifyGroup(
 
             bForeignSid = !bIsLocalOrBuiltinSid;
 
-            dwError = LsaMbsToWc16s(pszSID, &pwszSID);
+            dwError = LwMbsToWc16s(pszSID, &pwszSID);
             BAIL_ON_LSA_ERROR(dwError);
 
             dwFilterLen = (sizeof(wszAttrObjectClass) - 2) +
@@ -538,7 +665,7 @@ LocalDirModifyGroup(
         {
             pszSID = pGroupModInfo->ppszRemoveMembers[i];
 
-            dwError = LsaMbsToWc16s(pszSID, &pwszSID);
+            dwError = LwMbsToWc16s(pszSID, &pwszSID);
             BAIL_ON_LSA_ERROR(dwError);
 
             dwFilterLen = (sizeof(wszAttrObjectClass) - 1) +
@@ -658,10 +785,13 @@ LocalDirCheckLocalOrBuiltinSid(
     BOOLEAN bIsLocalOrBuiltinSid = FALSE;
     PSID pSid = NULL;
     PSID pBuiltinSid = NULL;
+    BOOLEAN bLocked = FALSE;
 
     dwError = LsaAllocateSidFromCString(&pSid, pszSid);
     BAIL_ON_LSA_ERROR(dwError);
 
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
     if (RtlIsPrefixSid(gLPGlobals.pLocalDomainSID, pSid))
     {
         bIsLocalOrBuiltinSid = TRUE;
@@ -680,6 +810,8 @@ LocalDirCheckLocalOrBuiltinSid(
     bIsLocalOrBuiltinSid = FALSE;
 
 cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
     LW_SAFE_FREE_MEMORY(pBuiltinSid);
     LW_SAFE_FREE_MEMORY(pSid);
 
@@ -762,7 +894,7 @@ LocalDirCreateForeignPrincipalDN(
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
     PWSTR pwszBase = NULL;
     ULONG ulScope = 0;
-    wchar_t wszFilterFmt[] = L"%ws=%d";
+    wchar_t wszFilterFmt[] = L"%ws=%u";
     DWORD dwFilterLen = 0;
     PWSTR pwszFilter = NULL;
     WCHAR wszAttrObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
@@ -875,17 +1007,86 @@ LocalDirDeleteGroup(
     PWSTR  pwszGroupDN
     )
 {
+    const wchar_t wszFilterTemplate[] = L"%ws=%u AND %ws='%ws'";
     DWORD dwError = 0;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
+    WCHAR wszAttrNameDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrNameObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrNameObjectSid[] = LOCAL_DIR_ATTR_OBJECT_SID;
+    PWSTR pwszFilter = NULL;
+    PDIRECTORY_ENTRY pEntry = NULL;
+    DWORD dwNumEntries = 0;
+    PSID pGroupSid = NULL;
+    PSID pDomainSid = NULL;
+
+    PWSTR pwszAttributes[] = {
+        wszAttrNameObjectClass,
+        wszAttrNameObjectSid,
+        NULL
+    };
+
+    dwError = LwAllocateWc16sPrintfW(
+                    &pwszFilter,
+                    wszFilterTemplate,
+                    wszAttrNameObjectClass, LOCAL_OBJECT_CLASS_GROUP,
+                    wszAttrNameDN, pwszGroupDN);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectorySearch(
+                    pContext->hDirectory,
+                    NULL,
+                    0,
+                    pwszFilter,
+                    pwszAttributes,
+                    FALSE,
+                    &pEntry,
+                    &dwNumEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumEntries == 0)
+    {
+        dwError = ERROR_NO_SUCH_GROUP;
+    }
+    else if (dwNumEntries != 1)
+    {
+        dwError = LW_ERROR_DATA_ERROR;
+    }
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LocalMarshalAttrToSid(
+                    pEntry,
+                    &wszAttrNameObjectSid[0],
+                    &pGroupSid);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    pDomainSid = gLPGlobals.pLocalDomainSID;
+
+    if (LocalDirIsBuiltinAccount(
+                    pDomainSid,
+                    pGroupSid))
+    {
+        dwError = ERROR_SPECIAL_ACCOUNT;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     dwError = DirectoryDeleteObject(
                     pContext->hDirectory,
                     pwszGroupDN);
     BAIL_ON_LSA_ERROR(dwError);
 
-error:
+cleanup:
+    if (pEntry)
+    {
+        DirectoryFreeEntries(pEntry, dwNumEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+    LW_SAFE_FREE_MEMORY(pGroupSid);
 
     return dwError;
+
+error:
+    goto cleanup;
 }
 
 VOID
