@@ -48,6 +48,30 @@
 
 #include "includes.h"
 
+static
+DWORD
+LocalSetDomainName(
+    IN HANDLE  hProvider,
+    IN uid_t   peerUID,
+    IN gid_t   peerGID,
+    IN DWORD   dwInputBufferSize,
+    IN PVOID   pInputBuffer,
+    OUT DWORD* pdwOutputBufferSize,
+    OUT PVOID* ppOutputBuffer
+    );
+
+static
+DWORD
+LocalSetDomainSid(
+    IN HANDLE  hProvider,
+    IN uid_t   peerUID,
+    IN gid_t   peerGID,
+    IN DWORD   dwInputBufferSize,
+    IN PVOID   pInputBuffer,
+    OUT DWORD* pdwOutputBufferSize,
+    OUT PVOID* ppOutputBuffer
+    );
+
 static
 DWORD
 LocalInitializeProvider(
@@ -56,6 +80,7 @@ LocalInitializeProvider(
     )
 {
     DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
     LOCAL_CONFIG config;
     BOOLEAN bEventLogEnabled = FALSE;
     PWSTR   pwszUserDN = NULL;
@@ -64,9 +89,16 @@ LocalInitializeProvider(
 
     memset(&config, 0, sizeof(config));
 
+    pthread_mutex_init(&gLPGlobals.cfgMutex, NULL);
+
     dwError = LocalCfgInitialize(&config);
     BAIL_ON_LSA_ERROR(dwError);
 
+    pthread_rwlock_init(&gLPGlobals.rwlock, NULL);
+
+    ntStatus = LwMapSecurityCreateContext(&gLPGlobals.pSecCtx);
+    BAIL_ON_NT_STATUS(ntStatus);
+
     dwError = LocalSyncDomainInfo(
                     pwszUserDN,
                     pwszCredentials,
@@ -94,6 +126,11 @@ LocalInitializeProvider(
     *ppFunctionTable = &gLocalProviderAPITable2;
 
 cleanup:
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
 
     return dwError;
 
@@ -106,6 +143,8 @@ error:
 
     LocalCfgFreeContents(&config);
 
+    LwMapSecurityFreeContext(&gLPGlobals.pSecCtx);
+
     *ppszProviderName = NULL;
     *ppFunctionTable = NULL;
 
@@ -200,10 +239,10 @@ LocalServicesDomain(
 }
 
 DWORD
-LocalAuthenticateUser(
+LocalAuthenticateUserPam(
     HANDLE hProvider,
-    PCSTR  pszLoginId,
-    PCSTR  pszPassword
+    LSA_AUTH_USER_PAM_PARAMS* pParams,
+    PLSA_AUTH_USER_PAM_INFO* ppPamAuthInfo
     )
 {
     DWORD dwError = 0;
@@ -213,15 +252,27 @@ LocalAuthenticateUser(
     PWSTR pwszPassword = NULL;
     PLSA_SECURITY_OBJECT pObject = NULL;
     DWORD dwBadPasswordCount = 0;
+    PLSA_AUTH_USER_PAM_INFO pPamAuthInfo = NULL;
+    BOOLEAN bUserIsGuest = FALSE;
+
+    if (ppPamAuthInfo)
+    {
+        *ppPamAuthInfo = NULL;
+    }
 
     dwError = LocalCheckForQueryAccess(hProvider);
     BAIL_ON_LSA_ERROR(dwError);
 
+    dwError = LwAllocateMemory(
+                    sizeof(*pPamAuthInfo),
+                    (PVOID*)&pPamAuthInfo);
+    BAIL_ON_LSA_ERROR(dwError);
+
     dwError = LocalDirFindObjectByGenericName(
         hProvider,
         0,
         LSA_OBJECT_TYPE_USER,
-        pszLoginId,
+        pParams->pszLoginName,
         &pObject);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -232,6 +283,15 @@ LocalAuthenticateUser(
 
     /* Check for disable, expired, etc..  accounts */
 
+    dwError = LocalCheckIsGuest(pObject, &bUserIsGuest);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (bUserIsGuest)
+    {
+        dwError = LW_ERROR_LOGON_FAILURE;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
     dwError = LocalCheckAccountFlags(pObject);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -242,10 +302,10 @@ LocalAuthenticateUser(
                   &dwBadPasswordCount);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (pszPassword)
+    if (pParams->pszPassword)
     {
-        dwError = LsaMbsToWc16s(
-                        pszPassword,
+        dwError = LwMbsToWc16s(
+                        pParams->pszPassword,
                         &pwszPassword);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -270,6 +330,11 @@ LocalAuthenticateUser(
     }
     BAIL_ON_LSA_ERROR(dwError);
 
+    if (ppPamAuthInfo)
+    {
+        *ppPamAuthInfo = pPamAuthInfo;
+    }
+
 cleanup:
 
     LsaUtilFreeSecurityObject(pObject);
@@ -281,11 +346,15 @@ cleanup:
     {
         dwError = dwUpdateError;
     }
-
     return dwError;
 
 error:
 
+    if (pPamAuthInfo)
+    {
+        LsaFreeAuthUserPamInfo(pPamAuthInfo);
+    }
+
     goto cleanup;
 }
 
@@ -326,7 +395,7 @@ LocalValidateUser(
     {
         LSA_LOG_DEBUG(
                 "Failed to find user '%s' while validating login "
-                "[error code:%d]",
+                "[error code:%u]",
                 pszLoginId,
                 dwError);
         dwError = LW_ERROR_NOT_HANDLED;
@@ -405,12 +474,12 @@ LocalChangePassword(
                     pszPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                         (pszPassword ? pszPassword : ""),
                         &pwszNewPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                         (pszOldPassword ? pszOldPassword : ""),
                         &pwszOldPassword);
     BAIL_ON_LSA_ERROR(dwError);
@@ -469,7 +538,7 @@ LocalSetPassword(
                     pszPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                         (pszPassword ? pszPassword : ""),
                         &pwszNewPassword);
     BAIL_ON_LSA_ERROR(dwError);
@@ -853,6 +922,7 @@ LocalGetStatus(
 {
     DWORD dwError = 0;
     PLSA_AUTH_PROVIDER_STATUS pProviderStatus = NULL;
+    BOOLEAN bInLock = FALSE;
 
     dwError = LwAllocateMemory(
                    sizeof(LSA_AUTH_PROVIDER_STATUS),
@@ -867,10 +937,19 @@ LocalGetStatus(
     pProviderStatus->mode = LSA_PROVIDER_MODE_LOCAL_SYSTEM;
     pProviderStatus->status = LSA_AUTH_PROVIDER_STATUS_ONLINE;
 
+    LOCAL_RDLOCK_RWLOCK(bInLock, &gLPGlobals.rwlock);
+
+    dwError = LwAllocateString(
+                    gLPGlobals.pszLocalDomain,
+                    &pProviderStatus->pszDomain);
+    BAIL_ON_LSA_ERROR(dwError);
+
     *ppProviderStatus = pProviderStatus;
 
 cleanup:
 
+    LOCAL_UNLOCK_RWLOCK(bInLock, &gLPGlobals.rwlock);
+
     return dwError;
 
 error:
@@ -897,20 +976,20 @@ LocalRefreshConfiguration(
     dwError = LocalCfgReadRegistry(&config);
     BAIL_ON_LSA_ERROR(dwError);
 
-    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.mutex);
+    LOCAL_LOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
 
     dwError = LocalCfgTransferContents(
                     &config,
                     &gLPGlobals.cfg);
     BAIL_ON_LSA_ERROR(dwError);
 
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.mutex);
+    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
 
     LocalEventLogConfigReload();
 
 cleanup:
 
-    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.mutex);
+    LOCAL_UNLOCK_MUTEX(bInLock, &gLPGlobals.cfgMutex);
 
     return dwError;
 
@@ -948,15 +1027,40 @@ LocalIoControl(
     )
 {
     DWORD dwError = 0;
+    PVOID pOutputBuffer = NULL;
+    DWORD dwOutputBufferSize = 0;
 
     switch (dwIoControlCode)
     {
+    case LSA_LOCAL_IO_SETDOMAINNAME:
+        dwError = LocalSetDomainName(hProvider,
+                                     peerUID,
+                                     peerGID,
+                                     dwInputBufferSize,
+                                     pInputBuffer,
+                                     NULL,
+                                     NULL);
+        break;
+
+    case LSA_LOCAL_IO_SETDOMAINSID:
+        dwError = LocalSetDomainSid(hProvider,
+                                    peerUID,
+                                    peerGID,
+                                    dwInputBufferSize,
+                                    pInputBuffer,
+                                    NULL,
+                                    NULL);
+        break;
+
     default:
         dwError = LW_ERROR_NOT_HANDLED;
         break;
     }
     BAIL_ON_LSA_ERROR(dwError);
 
+    *pdwOutputBufferSize = dwOutputBufferSize;
+    *ppOutputBuffer      = pOutputBuffer;
+
 cleanup:
     return dwError;
 
@@ -967,6 +1071,66 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+LocalSetDomainName(
+    IN HANDLE  hProvider,
+    IN uid_t   peerUID,
+    IN gid_t   peerGID,
+    IN DWORD   dwInputBufferSize,
+    IN PVOID   pInputBuffer,
+    OUT DWORD* pdwOutputBufferSize,
+    OUT PVOID* ppOutputBuffer
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+
+    if (peerUID)
+    {
+        dwError = LW_ERROR_ACCESS_DENIED;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = LocalDirSetDomainName((PSTR)pInputBuffer);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+static
+DWORD
+LocalSetDomainSid(
+    IN HANDLE  hProvider,
+    IN uid_t   peerUID,
+    IN gid_t   peerGID,
+    IN DWORD   dwInputBufferSize,
+    IN PVOID   pInputBuffer,
+    OUT DWORD* pdwOutputBufferSize,
+    OUT PVOID* ppOutputBuffer
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+
+    if (peerUID)
+    {
+        dwError = LW_ERROR_ACCESS_DENIED;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = LocalDirSetDomainSid((PSTR)pInputBuffer);
+    BAIL_ON_LSA_ERROR(dwError);
+
+cleanup:
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
 DWORD
 LocalFindObjects(
     IN HANDLE hProvider,
@@ -1088,15 +1252,46 @@ LocalQueryMemberOf(
         pppszGroupSids);
 }
 
+DWORD
+LocalGetSmartCardUserObject(
+    IN HANDLE hProvider,
+    OUT PLSA_SECURITY_OBJECT* ppObject,
+    OUT PSTR* ppszSmartCardReader
+    )
+{
+    *ppObject = NULL;
+    *ppszSmartCardReader = NULL;
+
+    return LW_ERROR_NOT_HANDLED;
+}
+
 DWORD
 LocalShutdownProvider(
     VOID
     )
 {
+    DWORD dwError = ERROR_SUCCESS;
+    BOOLEAN bLocked = FALSE;
+    BOOLEAN bCfgLocked = FALSE;
+
+    LOCAL_WRLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    LwMapSecurityFreeContext(&gLPGlobals.pSecCtx);
+
     LW_SAFE_FREE_STRING(gLPGlobals.pszLocalDomain);
     LW_SAFE_FREE_STRING(gLPGlobals.pszNetBIOSName);
+    RTL_FREE(&gLPGlobals.pLocalDomainSID);
 
-    return 0;
+    LOCAL_LOCK_MUTEX(bCfgLocked, &gLPGlobals.cfgMutex);
+
+    LocalCfgFreeContents(&gLPGlobals.cfg);
+
+    LOCAL_UNLOCK_MUTEX(bCfgLocked, &gLPGlobals.cfgMutex);
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    pthread_rwlock_destroy(&gLPGlobals.rwlock);
+
+    return dwError;
 }
 
 DWORD
@@ -1108,6 +1303,7 @@ LsaInitializeProvider2(
     return LocalInitializeProvider(ppszProviderName, ppFunctionTable);
 }
 
+
 /*
 local variables:
 mode: c
