@@ -87,7 +87,7 @@ LocalFindObjectByName(
                     pszDomainName);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -172,7 +172,7 @@ LocalDirResolveUserObjectPrimaryGroupSid(
         wszAttrNameObjectSID,
         NULL
     };
-    PCSTR pszTemplate = LOCAL_DB_DIR_ATTR_GID " = %d";
+    PCSTR pszTemplate = LOCAL_DB_DIR_ATTR_GID " = %u";
     PSTR pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
@@ -189,7 +189,7 @@ LocalDirResolveUserObjectPrimaryGroupSid(
         pUserObject->userInfo.gid);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
         pszFilter,
         &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -296,8 +296,8 @@ LocalDirFindObjectsInternal(
         LOCAL_DB_DIR_ATTR_NETBIOS_NAME " = \"%s\"" \
         " AND " LOCAL_DB_DIR_ATTR_SAM_ACCOUNT_NAME " = \"%s\"%s";
     PCSTR pszFilterTemplateString = "%s = \"%s\"%s";
-    PCSTR pszFilterTemplateDword = "%s = %d%s";
-    PCSTR pszFilterTemplateType = " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d";
+    PCSTR pszFilterTemplateDword = "%s = %u%s";
+    PCSTR pszFilterTemplateType = " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
     PCSTR pszFilterBy = NULL;
     PSTR pszFilterType = NULL;
     PSTR  pszFilter = NULL;
@@ -305,6 +305,7 @@ LocalDirFindObjectsInternal(
     DWORD dwObjectClass = LOCAL_OBJECT_CLASS_UNKNOWN;
     DWORD dwIndex = 0;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
+    BOOLEAN bLocked = FALSE;
 
     /* FIXME: support generic queries */
     switch (ObjectType)
@@ -371,20 +372,27 @@ LocalDirFindObjectsInternal(
         {
         case LSA_QUERY_TYPE_BY_ALIAS:
         case LSA_QUERY_TYPE_BY_NT4:
-            dwError = LocalCrackDomainQualifiedName(
+            dwError = LsaSrvCrackDomainQualifiedName(
                 QueryList.ppszStrings[dwIndex],
                 &pLoginInfo);
             BAIL_ON_LSA_ERROR(dwError);
 
-            if (!pLoginInfo->pszDomainNetBiosName)
+            if (!pLoginInfo->pszDomain)
             {
-                continue;
+                LOCAL_LOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
+
+                dwError = LwAllocateString(
+                                gLPGlobals.pszNetBIOSName,
+                                &pLoginInfo->pszDomain);
+                BAIL_ON_LSA_ERROR(dwError);
+
+                LOCAL_UNLOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
             }
 
             dwError = LwAllocateStringPrintf(
                 &pszFilter,
                 pszFilterTemplateQualified,
-                pLoginInfo->pszDomainNetBiosName,
+                pLoginInfo->pszDomain,
                 pLoginInfo->pszName,
                 pszFilterType ? pszFilterType : "");
             BAIL_ON_LSA_ERROR(dwError);
@@ -414,7 +422,7 @@ LocalDirFindObjectsInternal(
             BAIL_ON_LSA_ERROR(dwError);
         }
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
             pszFilter,
             &pwszFilter);
         BAIL_ON_LSA_ERROR(dwError);
@@ -479,13 +487,13 @@ LocalDirFindObjectsInternal(
 
         if (pLoginInfo)
         {
-            LsaFreeNameInfo(pLoginInfo);
+            LsaSrvFreeNameInfo(pLoginInfo);
             pLoginInfo = NULL;
         }
     }
 
 cleanup:
-
+    LOCAL_UNLOCK_MUTEX(bLocked, &gLPGlobals.cfgMutex);
     LW_SAFE_FREE_STRING(pszFilterType);
     LW_SAFE_FREE_STRING(pszFilter);
     LW_SAFE_FREE_MEMORY(pwszFilter);
@@ -497,7 +505,7 @@ cleanup:
 
     if (pLoginInfo)
     {
-        LsaFreeNameInfo(pLoginInfo);
+        LsaSrvFreeNameInfo(pLoginInfo);
     }
 
     return dwError;
@@ -650,7 +658,7 @@ LocalDirOpenEnumObjects(
     {
         dwError = LwAllocateStringPrintf(
             &pszTypeFilter,
-            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d",
+            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u",
             dwObjectClass);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -658,8 +666,8 @@ LocalDirOpenEnumObjects(
     {
         dwError = LwAllocateStringPrintf(
             &pszTypeFilter,
-            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d OR "
-            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d",
+            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u OR "
+            LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u",
             LOCAL_OBJECT_CLASS_USER,
             LOCAL_OBJECT_CLASS_GROUP);
         BAIL_ON_LSA_ERROR(dwError);
@@ -680,7 +688,7 @@ LocalDirOpenEnumObjects(
         pszTypeFilter = NULL;
     }
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
         pszFilter,
         &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -843,7 +851,7 @@ LocalDirOpenEnumMembers(
         pszSid);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
         pszFilter,
         &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1028,7 +1036,7 @@ LocalDirQueryMemberOfDN(
             pGroupHash,
             pszGroupSid,
             OUT_PPVOID(&pszPreviousGroupSid));
-        if (dwError == ENOENT)
+        if (dwError == ERROR_NOT_FOUND)
         {
             dwError = LsaHashSetValue(
                 pGroupHash,
@@ -1098,7 +1106,7 @@ LocalDirQueryMemberOfInternal(
         pszSid);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
         pszFilter,
             &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1267,7 +1275,7 @@ LocalDirFindObjectByGenericName(
 
     BAIL_ON_INVALID_HANDLE(hProvider);
 
-    dwError = LocalCrackDomainQualifiedName(
+    dwError = LsaSrvCrackDomainQualifiedName(
         pszName,
         &pLoginInfo);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1323,7 +1331,7 @@ cleanup:
 
     if (pLoginInfo)
     {
-        LsaFreeNameInfo(pLoginInfo);
+        LsaSrvFreeNameInfo(pLoginInfo);
     }
 
     LsaUtilFreeSecurityObjectList(1, ppObjects);
