@@ -79,7 +79,7 @@ LocalDirGetUserInfoFlags(
     DWORD dwNumAttrs = (sizeof(pwszAttrs)/sizeof(pwszAttrs[0])) - 1;
     PCSTR pszFilterTemplate =
                         LOCAL_DB_DIR_ATTR_UID " = %u" \
-                        " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %d";
+                        " AND " LOCAL_DB_DIR_ATTR_OBJECT_CLASS " = %u";
     PSTR  pszFilter = NULL;
     PWSTR pwszFilter = NULL;
     PDIRECTORY_ENTRY pEntry = NULL;
@@ -94,7 +94,7 @@ LocalDirGetUserInfoFlags(
                     LOCAL_OBJECT_CLASS_USER);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -160,7 +160,8 @@ LocalDirAddUser(
     PLSA_USER_ADD_INFO pUserInfo
     )
 {
-    DWORD dwError = 0;
+    DWORD dwError = ERROR_SUCCESS;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
     BOOLEAN bEventlogEnabled = FALSE;
     PLSA_LOGIN_NAME_INFO pLoginInfo = NULL;
@@ -323,6 +324,7 @@ LocalDirAddUser(
     };
     DIRECTORY_MOD mods[LOCAL_DAU0_IDX_SENTINEL + 1];
 
+    BOOLEAN bLocked = 0;
     DWORD iMod = 0;
     PWSTR pwszSamAccountName = NULL;
     PWSTR pwszGecos = NULL;
@@ -343,6 +345,10 @@ LocalDirAddUser(
     PDIRECTORY_ENTRY pMember = NULL;
     DWORD dwNumEntries = 0;
     BOOLEAN bUserAdded = FALSE;
+    PWSTR pwszUserSID = NULL;
+    PSID pUserSID = NULL;
+    DWORD dwUserRID = 0;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     gid_t gid = 0;
     PSID pGroupSID = NULL;
     PLSA_SECURITY_OBJECT* ppObjects = NULL;
@@ -366,12 +372,22 @@ LocalDirAddUser(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
-    dwError = LocalCrackDomainQualifiedName(
+    dwError = LsaSrvCrackDomainQualifiedName(
                     pUserInfo->pszName,
                     &pLoginInfo);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (!LocalServicesDomain(pLoginInfo->pszDomainNetBiosName))
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (!pLoginInfo->pszDomain)
+    {
+        dwError = LwAllocateString(
+                        gLPGlobals.pszNetBIOSName,
+                        &pLoginInfo->pszDomain);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    if (!LocalServicesDomain(pLoginInfo->pszDomain))
     {
         dwError = LW_ERROR_NOT_HANDLED;
         BAIL_ON_LSA_ERROR(dwError);
@@ -417,22 +433,22 @@ LocalDirAddUser(
         RTL_FREE(&pGroupSID);
     }
 
-    dwError = LsaMbsToWc16s(
-                    pLoginInfo->pszFullDomainName,
+    dwError = LwMbsToWc16s(
+                    pLoginInfo->pszDomain,
                     &pwszDomain);
     BAIL_ON_LSA_ERROR(dwError);
 
     attrValues[LOCAL_DAU0_IDX_DOMAIN].data.pwszStringValue = pwszDomain;
 
-    dwError = LsaMbsToWc16s(
-                    pLoginInfo->pszDomainNetBiosName,
+    dwError = LwMbsToWc16s(
+                    pLoginInfo->pszDomain,
                     &pwszNetBIOSDomain);
     BAIL_ON_LSA_ERROR(dwError);
 
     attrValues[LOCAL_DAU0_IDX_NETBIOS_DOMAIN].data.pwszStringValue = pwszNetBIOSDomain;
 
-    dwError = LsaMbsToWc16s(
-                    pUserInfo->pszName,
+    dwError = LwMbsToWc16s(
+                    pLoginInfo->pszName,
                     &pwszSamAccountName);
     BAIL_ON_LSA_ERROR(dwError);
 
@@ -447,7 +463,7 @@ LocalDirAddUser(
 
     if (!LW_IS_NULL_OR_EMPTY_STR(pUserInfo->pszGecos))
     {
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                     pUserInfo->pszGecos,
                     &pwszGecos);
         BAIL_ON_LSA_ERROR(dwError);
@@ -457,7 +473,7 @@ LocalDirAddUser(
 
     if (!LW_IS_NULL_OR_EMPTY_STR(pUserInfo->pszHomedir))
     {
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                     pUserInfo->pszHomedir,
                     &pwszHomedir);
         BAIL_ON_LSA_ERROR(dwError);
@@ -465,12 +481,12 @@ LocalDirAddUser(
     else
     {
         dwError = LocalBuildHomeDirPathFromTemplate(
-                        pUserInfo->pszName,
-                        pLoginInfo->pszDomainNetBiosName,
+                        pLoginInfo->pszName,
+                        pLoginInfo->pszDomain,
                         &pszHomedir);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszHomedir,
                         &pwszHomedir);
         BAIL_ON_LSA_ERROR(dwError);
@@ -480,7 +496,7 @@ LocalDirAddUser(
 
     if (!LW_IS_NULL_OR_EMPTY_STR(pUserInfo->pszShell))
     {
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                     pUserInfo->pszShell,
                     &pwszShell);
         BAIL_ON_LSA_ERROR(dwError);
@@ -490,8 +506,8 @@ LocalDirAddUser(
         dwError = LocalCfgGetDefaultShell(&pszShell);
         BAIL_ON_LSA_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
-                    pUserInfo->pszShell,
+        dwError = LwMbsToWc16s(
+                    pszShell,
                     &pwszShell);
         BAIL_ON_LSA_ERROR(dwError);
     }
@@ -540,8 +556,12 @@ LocalDirAddUser(
                     (PVOID*)&pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
 
-    sw16printfw(pwszFilter, dwFilterLen/sizeof(WCHAR), wszFilterFmt,
-                wszAttrDistinguishedName, pwszUserDN);
+    if (sw16printfw(pwszFilter, dwFilterLen/sizeof(WCHAR), wszFilterFmt,
+                    wszAttrDistinguishedName, pwszUserDN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     dwError = DirectorySearch(
                     pContext->hDirectory,
@@ -554,7 +574,42 @@ LocalDirAddUser(
                     &dwNumEntries);
     BAIL_ON_LSA_ERROR(dwError);
 
-    if (!pwszGroupDN) {
+    /*
+     * Set default security descriptor on the account
+     */
+    dwError = DirectoryGetEntryAttrValueByName(
+                    pMember,
+                    wszAttrObjectSID,
+                    DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                    &pwszUserSID);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlAllocateSidFromWC16String(&pUserSID,
+                                            pwszUserSID);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = RtlGetRidSid(&dwUserRID,
+                            pUserSID);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    dwError = LocalDirCreateNewAccountSecurityDescriptor(
+                    gLPGlobals.pLocalDomainSID,
+                    dwUserRID,
+                    DIR_OBJECT_CLASS_USER,
+                    &pSecDesc);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectorySetEntrySecurityDescriptor(
+                                  pContext->hDirectory,
+                                  pwszUserDN,
+                                  pSecDesc);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /*
+     * Add user to primary group
+     */
+    if (!pwszGroupDN)
+    {
         dwError = DirectoryGetEntryAttrValueByName(
                         pMember,
                         wszAttrGID,
@@ -607,12 +662,16 @@ LocalDirAddUser(
                     pMember);
     BAIL_ON_LSA_ERROR(dwError);
 
+    /*
+     * Set user password
+     */
+
     if (pUserInfo->pszPassword)
     {
         pszPassword = pUserInfo->pszPassword;
     }
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszPassword,
                     &pwszPassword);
     BAIL_ON_LSA_ERROR(dwError);
@@ -633,12 +692,13 @@ LocalDirAddUser(
     }
 
 cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
 
     LsaUtilFreeSecurityObjectList(1, ppObjects);
 
     if (pLoginInfo)
     {
-        LsaFreeNameInfo(pLoginInfo);
+        LsaSrvFreeNameInfo(pLoginInfo);
     }
 
     LW_SAFE_FREE_MEMORY(pwszUserDN);
@@ -653,6 +713,7 @@ cleanup:
     LW_SAFE_FREE_MEMORY(pwszDomain);
     LW_SAFE_FREE_MEMORY(pwszNetBIOSDomain);
     RTL_FREE(&pGroupSID);
+    RTL_FREE(&pUserSID);
 
     LW_SAFE_FREE_MEMORY(pwszFilter);
 
@@ -661,6 +722,14 @@ cleanup:
         DirectoryFreeEntries(pMember, dwNumEntries);
     }
 
+    LocalDirFreeSecurityDescriptor(&pSecDesc);
+
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
+
     return dwError;
 
 error:
@@ -674,7 +743,7 @@ error:
                         pwszUserDN);
         if (dwError2)
         {
-            LSA_LOG_ERROR("Failed to remove user [code: %d]", dwError2);
+            LSA_LOG_ERROR("Failed to remove user [code: %u]", dwError2);
         }
     }
 
@@ -688,11 +757,23 @@ LocalDirValidateUID(
     )
 {
     DWORD dwError = 0;
+    DWORD dwUidMin = LOWEST_UID;
+    DWORD dwUidMax = HIGHEST_UID;
+    DWORD dwUid = (DWORD) uid;
+
+    dwError = LocalCfgGetMinUid(&dwUidMin);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LocalCfgGetMaxUid(&dwUidMax);
+    BAIL_ON_LSA_ERROR(dwError);
 
     /* Check whether account uid is within permitted range */
-    if (uid < LOWEST_UID) {
-        dwError = LW_ERROR_INVALID_PARAMETER;
-        BAIL_ON_LSA_ERROR(dwError);
+    if (dwUid < dwUidMin) {
+        BAIL_WITH_LSA_ERROR(LW_ERROR_UID_TOO_LOW);
+    }
+
+    if (dwUid > dwUidMax) {
+        BAIL_WITH_LSA_ERROR(LW_ERROR_UID_TOO_HIGH);
     }
 
 cleanup:
@@ -715,7 +796,7 @@ LocalDirModifyUser(
     DWORD   dwUserInfoFlags = 0;
     DWORD   dwOrigUserInfoFlags = 0;
     WCHAR   wszAttrDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
-    DIRECTORY_MOD mods[9] = {{0}};
+    DIRECTORY_MOD mods[10] = {{0}};
 
     ATTRIBUTE_VALUE UserAttrVal[] = {
         {
@@ -745,6 +826,7 @@ LocalDirModifyUser(
     };
 
     WCHAR wszAttrNameDn[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrNameLastPasswordSet[] = LOCAL_DIR_ATTR_PASSWORD_LAST_SET;
     WCHAR wszAttrNameUserInfoFlags[] = LOCAL_DIR_ATTR_ACCOUNT_FLAGS;
     WCHAR wszAttrNameAccountExpiry[] = LOCAL_DIR_ATTR_ACCOUNT_EXPIRY;
     WCHAR wszAttrNameNtHash [] = LOCAL_DIR_ATTR_NT_HASH;
@@ -753,6 +835,7 @@ LocalDirModifyUser(
     WCHAR wszAttrNameShell [] = LOCAL_DIR_ATTR_SHELL;
     WCHAR wszAttrNameGecos [] = LOCAL_DIR_ATTR_GECOS;
     WCHAR wszAttrNameHomedir [] = LOCAL_DIR_ATTR_HOME_DIR;
+    ATTRIBUTE_VALUE avLastPasswordChange = {0};
     ATTRIBUTE_VALUE avUserInfoFlags = {0};
     ATTRIBUTE_VALUE avAccountExpiry = {0};
     ATTRIBUTE_VALUE avNtHash = {0};
@@ -839,7 +922,14 @@ LocalDirModifyUser(
 
     if (pUserModInfo->actions.bSetChangePasswordOnNextLogon)
     {
-        dwUserInfoFlags |= LOCAL_ACB_PW_EXPIRED;
+        mods[dwNumMods].ulOperationFlags = DIR_MOD_FLAGS_REPLACE;
+        mods[dwNumMods].pwszAttrName = &wszAttrNameLastPasswordSet[0];
+        mods[dwNumMods].ulNumValues = 1;
+        avLastPasswordChange.Type = DIRECTORY_ATTR_TYPE_LARGE_INTEGER;
+        avLastPasswordChange.data.llValue = 0;
+        mods[dwNumMods].pAttrValues = &avLastPasswordChange;
+
+        dwNumMods++;
     }
 
     if (pUserModInfo->actions.bUnlockUser)
@@ -865,7 +955,7 @@ LocalDirModifyUser(
 
         if (strptime(pUserModInfo->pszExpiryDate, "%Y-%m-%d", &tmbuf) == NULL)
         {
-            dwError = errno;
+            dwError = LwMapErrnoToLwError(errno);
             BAIL_ON_LSA_ERROR(dwError);
         }
 
@@ -1037,7 +1127,7 @@ LocalDirModifyUser(
         BAIL_ON_LSA_ERROR(dwError);
 
         LsaUtilFreeSecurityObject(pObject);
-        ppObjects = NULL;
+        pObject = NULL;
 
         dwError = DirectoryRemoveFromGroup(
             pContext->hDirectory,
@@ -1073,7 +1163,7 @@ LocalDirModifyUser(
 
     if (pUserModInfo->actions.bSetPassword)
     {
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pUserModInfo->pszPassword ? pUserModInfo->pszPassword : "",
                         &pwszPassword);
         BAIL_ON_LSA_ERROR(dwError);
@@ -1114,17 +1204,91 @@ LocalDirDeleteUser(
     PWSTR  pwszUserDN
     )
 {
+    const wchar_t wszFilterTemplate[] = L"%ws=%u AND %ws='%ws'";
     DWORD dwError = 0;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
+    WCHAR wszAttrNameDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrNameObjectClass[] = LOCAL_DIR_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrNameObjectSid[] = LOCAL_DIR_ATTR_OBJECT_SID;
+    PWSTR pwszFilter = NULL;
+    PDIRECTORY_ENTRY pEntry = NULL;
+    DWORD dwNumEntries = 0;
+    PSID pUserSid = NULL;
+    PSID pDomainSid = NULL;
+    BOOLEAN bLocked = FALSE;
+
+    PWSTR pwszAttributes[] = {
+        wszAttrNameObjectClass,
+        wszAttrNameObjectSid,
+        NULL
+    };
+
+    dwError = LwAllocateWc16sPrintfW(
+                    &pwszFilter,
+                    wszFilterTemplate,
+                    wszAttrNameObjectClass, LOCAL_OBJECT_CLASS_USER,
+                    wszAttrNameDN, pwszUserDN);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectorySearch(
+                    pContext->hDirectory,
+                    NULL,
+                    0,
+                    pwszFilter,
+                    pwszAttributes,
+                    FALSE,
+                    &pEntry,
+                    &dwNumEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumEntries == 0)
+    {
+        dwError = ERROR_NO_SUCH_USER;
+    }
+    else if (dwNumEntries != 1)
+    {
+        dwError = LW_ERROR_DATA_ERROR;
+    }
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LocalMarshalAttrToSid(
+                    pEntry,
+                    &wszAttrNameObjectSid[0],
+                    &pUserSid);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    pDomainSid = gLPGlobals.pLocalDomainSID;
+
+    if (LocalDirIsBuiltinAccount(
+                    pDomainSid,
+                    pUserSid))
+    {
+        dwError = ERROR_SPECIAL_ACCOUNT;
+        BAIL_ON_LSA_ERROR(dwError);
+    }
 
     dwError = DirectoryDeleteObject(
                     pContext->hDirectory,
                     pwszUserDN);
     BAIL_ON_LSA_ERROR(dwError);
 
-error:
+cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (pEntry)
+    {
+        DirectoryFreeEntries(pEntry, dwNumEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+    LW_SAFE_FREE_MEMORY(pUserSid);
 
     return dwError;
+
+error:
+    goto cleanup;
 }
 
 DWORD
@@ -1135,8 +1299,118 @@ LocalDirChangePassword(
     PWSTR  pwszNewPassword
     )
 {
+    const wchar_t wszUserFilter[] = L"%ws = '%ws'";
     DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
     PLOCAL_PROVIDER_CONTEXT pContext = (PLOCAL_PROVIDER_CONTEXT)hProvider;
+    BOOLEAN bLocked = FALSE;
+    PACCESS_TOKEN pUserToken = NULL;
+    PWSTR pwszBase = NULL;
+    DWORD dwScope = 0;
+    PWSTR pwszFilter = NULL;
+    DWORD dwFilterLen = 0;
+    size_t sUserDnLen = 0;
+    WCHAR wszAttrDN[] = LOCAL_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrUid[] = LOCAL_DIR_ATTR_UID;
+    WCHAR wszAttrGid[] = LOCAL_DIR_ATTR_GID;
+    WCHAR wszAttrSecurityDescriptor[] = LOCAL_DIR_ATTR_SECURITY_DESCRIPTOR;
+
+    PWSTR wszAttributes[] = {
+        wszAttrUid,
+        wszAttrGid,
+        wszAttrSecurityDescriptor,
+        NULL
+    };
+
+    PDIRECTORY_ENTRY pUserEntry = NULL;
+    DWORD dwNumEntries = 0;
+    DWORD dwUid = 0;
+    DWORD dwGid = 0;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
+    GENERIC_MAPPING GenericMapping = {0};
+    DWORD dwAccessGranted = 0;
+
+    dwError = LwWc16sLen(pwszUserDN, &sUserDnLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwFilterLen = (sizeof(wszAttrDN)/sizeof(wszAttrDN[0]) +
+                   sUserDnLen +
+                   sizeof(wszUserFilter));
+
+    dwError = LwAllocateMemory(sizeof(pwszFilter[0]) * dwFilterLen,
+                               OUT_PPVOID(&pwszFilter));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (sw16printfw(pwszFilter, dwFilterLen,
+                    wszUserFilter,
+                    wszAttrDN, pwszUserDN) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_LSA_ERROR(dwError);
+    }
+
+    dwError = DirectorySearch(
+                    pContext->hDirectory,
+                    pwszBase,
+                    dwScope,
+                    pwszFilter,
+                    wszAttributes,
+                    FALSE,
+                    &pUserEntry,
+                    &dwNumEntries);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (dwNumEntries == 0)
+    {
+        dwError = LW_ERROR_NO_SUCH_USER;
+    }
+    else if (dwNumEntries != 1)
+    {
+        dwError = LW_ERROR_DATA_ERROR;
+    }
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                    pUserEntry,
+                    wszAttrUid,
+                    DIRECTORY_ATTR_TYPE_INTEGER,
+                    &dwUid);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectoryGetEntryAttrValueByName(
+                    pUserEntry,
+                    wszAttrUid,
+                    DIRECTORY_ATTR_TYPE_INTEGER,
+                    &dwGid);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = DirectoryGetEntrySecurityDescriptor(
+                    pUserEntry,
+                    &pSecDesc);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    LOCAL_RDLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    /*
+     * Check if user has right to change the password first
+     */
+    ntStatus = LwMapSecurityCreateAccessTokenFromUidGid(
+                    gLPGlobals.pSecCtx,
+                    &pUserToken,
+                    dwUid,
+                    dwGid);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    if (!RtlAccessCheck(pSecDesc,
+                        pUserToken,
+                        USER_ACCESS_CHANGE_PASSWORD,
+                        0,
+                        &GenericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
 
     dwError = DirectoryChangePassword(
                     pContext->hDirectory,
@@ -1145,11 +1419,33 @@ LocalDirChangePassword(
                     pwszNewPassword);
     BAIL_ON_LSA_ERROR(dwError);
 
-error:
+cleanup:
+    LOCAL_UNLOCK_RWLOCK(bLocked, &gLPGlobals.rwlock);
+
+    if (pUserEntry)
+    {
+        DirectoryFreeEntries(pUserEntry, dwNumEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszFilter);
+
+    DirectoryFreeEntrySecurityDescriptor(&pSecDesc);
+
+    RtlReleaseAccessToken(&pUserToken);
+
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
 
     return dwError;
+
+error:
+    goto cleanup;
 }
 
+
 DWORD
 LocalDirSetPassword(
     HANDLE hProvider,
@@ -1176,7 +1472,7 @@ LocalDirSetPassword(
     }
     else
     {
-        dwError = EACCES;
+        dwError = LW_ERROR_ACCESS_DENIED;
         BAIL_ON_LSA_ERROR(dwError);
     }
 
@@ -1324,6 +1620,50 @@ error:
     goto cleanup;
 }
 
+DWORD
+LocalCheckIsGuest(
+    PLSA_SECURITY_OBJECT pObject,
+    PBOOLEAN pbUserIsGuest
+    )
+{
+    DWORD dwError = LW_ERROR_SUCCESS;
+    PSID pUserSid = NULL;
+    ULONG ulRid = 0;
+    BOOLEAN bUserIsGuest = FALSE;
+
+    dwError = LwNtStatusToWin32Error(
+                  RtlAllocateSidFromCString(
+                      &pUserSid,
+                      pObject->pszObjectSid));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = LwNtStatusToWin32Error(
+                  RtlGetRidSid(
+                      &ulRid,
+                      pUserSid));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    if (ulRid == DOMAIN_USER_RID_GUEST)
+    {
+        bUserIsGuest = TRUE;
+    }
+
+cleanup:
+
+    if (pUserSid)
+    {
+        LW_RTL_FREE(&pUserSid);
+    }
+
+    *pbUserIsGuest = bUserIsGuest;
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
 DWORD
 LocalCheckAccountFlags(
     PLSA_SECURITY_OBJECT pObject
@@ -1431,7 +1771,7 @@ LocalGetUserLogonInfo(
                     pszUserDn);
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1865,6 +2205,7 @@ error:
     goto cleanup;
 }
 
+
 /*
 local variables:
 mode: c
