@@ -3,7 +3,7 @@
  * -*- mode: c, c-basic-offset: 4 -*- */
 
 /*
- * Copyright Likewise Software    2004-2008
+ * Copyright Likewise Software
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,6 +47,8 @@
 #ifndef __LSASQLITE_H__
 #define __LSASQLITE_H__
 
+#include <lw/security-types.h>
+
 #define SQLITE3_SAFE_FREE_STRING(x) \
     if ((x) != NULL) \
     { \
@@ -58,7 +60,7 @@
     do { \
         if (dwError) \
         { \
-           LSA_LOG_DEBUG("Sqlite3 error '%s' (code = %d)", \
+           LSA_LOG_DEBUG("Sqlite3 error '%s' (code = %u)", \
                          LSA_SAFE_LOG_STRING(pszError), dwError); \
            goto error;                               \
         } \
