@@ -47,9 +47,11 @@
 #ifndef __LSASRVAPI_H__
 #define __LSASRVAPI_H__
 
+#include <lsa/provider.h>
+
 DWORD
 LsaSrvApiInit(
-    PCSTR pszConfigFilePath
+    PLSA_STATIC_PROVIDER pStaticProviders
     );
 
 DWORD
@@ -61,21 +63,38 @@ DWORD
 LsaSrvOpenServer(
     uid_t peerUID,
     gid_t peerGID,
+    pid_t peerPID,
     PHANDLE phServer
     );
 
+VOID
+LsaSrvGetClientId(
+    HANDLE hServer,
+    uid_t* pUid,
+    gid_t* pGid,
+    pid_t* pPid
+    );
+
 void
 LsaSrvCloseServer(
     HANDLE hServer
     );
 
 DWORD
-LsaSrvAuthenticateUser(
+LsaSrvAuthenticateUserPam(
     HANDLE hServer,
-    PCSTR  pszLoginId,
-    PCSTR  pszPassword
+    PLSA_AUTH_USER_PAM_PARAMS pParams,
+    PLSA_AUTH_USER_PAM_INFO* ppPamAuthInfo
     );
 
+DWORD
+LsaSrvAuthenticateUserEx(
+    HANDLE hServer,
+    PLSA_AUTH_USER_PARAMS pUserParms,
+    PLSA_AUTH_USER_INFO *ppUserInfo
+    );
+
+
 DWORD
 LsaSrvValidateUser(
     HANDLE hServer,
@@ -99,125 +118,45 @@ LsaSrvChangePassword(
     );
 
 DWORD
-LsaSrvFindGroupByName(
-    IN HANDLE hServer,
-    IN PCSTR pszGroup,
-    IN LSA_FIND_FLAGS FindFlags,
-    IN DWORD dwGroupInfoLevel,
-    OUT PVOID* ppGroupInfo
-    );
-
-DWORD
-LsaSrvFindGroupById(
-    IN HANDLE hServer,
-    IN gid_t gid,
-    IN LSA_FIND_FLAGS FindFlags,
-    IN DWORD dwGroupInfoLevel,
-    OUT PVOID* ppGroupInfo
-    );
-
-DWORD
-LsaSrvGetGroupsForUser(
-    IN HANDLE hServer,
-    IN uid_t uid,
-    IN LSA_FIND_FLAGS FindFlags,
-    IN DWORD dwGroupInfoLevel,
-    OUT PDWORD pdwGroupsFound,
-    OUT PVOID** pppGroupInfoList
-    );
-
-DWORD
-LsaSrvFindUserByName(
-    HANDLE  hServer,
-    PCSTR   pszUsername,
-    DWORD   dwUserInfoLevel,
-    PVOID*  ppUserInfo
-    );
-
-DWORD
-LsaSrvAddGroup(
+LsaSrvSetPassword(
     HANDLE hServer,
-    DWORD  dwGroupInfoLevel,
-    PVOID  pGroupInfo
+    PCSTR  pszLoginId,
+    PCSTR  pszPassword
     );
 
 DWORD
-LsaSrvDeleteGroup(
+LsaSrvAddGroup2(
     HANDLE hServer,
-    gid_t  gid
+    PCSTR pszTargetProvider,
+    PLSA_GROUP_ADD_INFO pGroupAddInfo
     );
 
 DWORD
-LsaSrvAddUser(
+LsaSrvModifyGroup2(
     HANDLE hServer,
-    DWORD  dwUserInfoLevel,
-    PVOID  pUserInfo
+    PCSTR pszTargetProvider,
+    PLSA_GROUP_MOD_INFO_2 pGroupModInfo
     );
 
 DWORD
-LsaSrvModifyUser(
+LsaSrvDeleteObject(
     HANDLE hServer,
-    PLSA_USER_MOD_INFO pUserModInfo
+    PCSTR pszTargetProvider,
+    PCSTR pszSid
     );
 
 DWORD
-LsaSrvDeleteUser(
+LsaSrvAddUser2(
     HANDLE hServer,
-    uid_t  uid
+    PCSTR pszTargetProvider,
+    PLSA_USER_ADD_INFO pUserAddInfo
     );
 
 DWORD
-LsaSrvFindUserById(
+LsaSrvModifyUser2(
     HANDLE hServer,
-    uid_t  uid,
-    DWORD  dwUserInfoLevel,
-    PVOID* ppUserInfo
-    );
-
-DWORD
-LsaSrvBeginEnumUsers(
-    HANDLE hServer,
-    DWORD  dwUserInfoLevel,
-    DWORD  dwNumMaxUsers,
-    PSTR*  ppszGUID
-    );
-
-DWORD
-LsaSrvEnumUsers(
-    HANDLE  hServer,
-    PCSTR   pszGUID,
-    PDWORD  pdwUserInfoLevel,
-    PVOID** pppUserInfoList,
-    PDWORD  pdwNumUsersFound
-    );
-
-DWORD
-LsaSrvEndEnumUsers(
-    HANDLE hServer,
-    PCSTR  pszGUID
-    );
-
-DWORD
-LsaSrvBeginEnumGroups(
-    HANDLE hServer,
-    DWORD  dwGroupInfoLevel,
-    DWORD  dwNumMaxGroups,
-    PSTR*  ppszGUID
-    );
-
-DWORD
-LsaSrvEnumGroups(
-    HANDLE  hServer,
-    PCSTR   pszGUID,
-    PDWORD  pdwGroupInfoLevel,
-    PVOID** pppGroupInfoList,
-    PDWORD  pdwNumGroupsFound
-    );
-
-DWORD
-LsaSrvEndEnumGroups(
-    HANDLE hServer,
-    PCSTR  pszGUID
+    PCSTR pszTargetProvider,
+    PLSA_USER_MOD_INFO_2 pUserModInfo
     );
 
 DWORD
@@ -237,13 +176,13 @@ LsaSrvBeginEnumNSSArtefacts(
     LSA_NIS_MAP_QUERY_FLAGS dwFlags,
     DWORD  dwGroupInfoLevel,
     DWORD  dwNumMaxGroups,
-    PSTR*  ppszGUID
+    PHANDLE phState
     );
 
 DWORD
 LsaSrvEnumNSSArtefacts(
     HANDLE  hServer,
-    PCSTR   pszGUID,
+    HANDLE  hState,
     PDWORD  pdwGroupInfoLevel,
     PVOID** pppGroupInfoList,
     PDWORD  pdwNumGroupsFound
@@ -252,7 +191,7 @@ LsaSrvEnumNSSArtefacts(
 DWORD
 LsaSrvEndEnumNSSArtefacts(
     HANDLE hServer,
-    PCSTR  pszGUID
+    HANDLE hState
     );
 
 DWORD
@@ -281,17 +220,6 @@ LsaSrvCloseSession(
     PCSTR  pszLoginId
     );
 
-DWORD
-LsaSrvGetNamesBySidList(
-    IN HANDLE hServer,
-    IN size_t sCount,
-    IN PSTR* ppszSidList,
-    OUT PSTR** pppszDomainNames,
-    OUT PSTR** pppszSamAccounts,
-    OUT ADAccountType **ppTypes,
-    OUT CHAR* pchDomainSeparator
-    );
-
 DWORD
 LsaSrvGetLogInfo(
     HANDLE hServer,
@@ -343,5 +271,44 @@ LsaSrvRefreshConfiguration(
     HANDLE hServer
     );
 
+//
+// For targeting specific providers
+//
+
+DWORD
+LsaSrvProviderIoControl(
+    IN HANDLE  hServer,
+    IN PCSTR   pszProvider,
+    IN DWORD   dwIoControlCode,
+    IN DWORD   dwInputBufferSize,
+    IN PVOID   pInputBuffer,
+    OUT DWORD* pdwOutputBufferSize,
+    OUT PVOID* ppOutputBuffer
+    );
+
+//
+// Server-Side only APIs for calling specific providers
+//
+
+DWORD
+LsaSrvProviderServicesDomain(
+    IN PCSTR pszProvider,
+    IN PCSTR pszDomainName,
+    OUT PBOOLEAN pbServicesDomain
+    );
+
+DWORD
+LsaSrvGetPamConfig(
+    IN HANDLE hServer,
+    OUT PLSA_PAM_CONFIG *ppPamConfig
+    );
+
+VOID
+LsaSrvFreePamConfig(
+    IN PLSA_PAM_CONFIG pConfig
+    );
+
+#define LSA_PROVIDER_TAG_LOCAL "lsa-local-provider"
+
 #endif /* __LSASRVAPI_H__ */
 
