@@ -135,7 +135,7 @@ lsassd_main(
     BAIL_ON_LSA_ERROR(dwError);
 
     if (atexit(LsaSrvExitHandler) < 0) {
-       dwError = errno;
+       dwError = LwMapErrnoToLwError(errno);
        BAIL_ON_LSA_ERROR(dwError);
     }
 
@@ -157,6 +157,9 @@ lsassd_main(
     dwError = LsaSrvStartupPreCheck();
     BAIL_ON_LSA_ERROR(dwError);
 
+    dwError = LsaSrvStartEventLoggingThread();
+    BAIL_ON_LSA_ERROR(dwError);
+
 #ifdef ENABLE_PIDFILE
     LsaSrvCreatePIDFile();
 #endif
@@ -168,16 +171,18 @@ lsassd_main(
         BAIL_ON_LSA_ERROR(dwError);
     }
 
+    /* Start NTLM IPC server before we initialize providers
+       because the providers might end up attempting to use
+       NTLM via gss-api */
+    dwError = NtlmSrvStartListenThread();
+    BAIL_ON_LSA_ERROR(dwError);
+
     dwError = LsaSrvInitialize();
     BAIL_ON_LSA_ERROR(dwError);
 
-    //dwError = LsaSrvStartListenThread(&listenerThreadId, &pListenerThreadId);
     dwError = LsaSrvStartListenThread();
     BAIL_ON_LSA_ERROR(dwError);
 
-    dwError = NtlmSrvStartListenThread();
-    BAIL_ON_LSA_ERROR(dwError);
-
     if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
     {
         notifyFd = atoi(pszSmNotify);
@@ -189,8 +194,9 @@ lsassd_main(
 
         if (ret < 0)
         {
-            LSA_LOG_ERROR("Could not notify service manager: %s (%i)", strerror(errno), errno);
             dwError = LwMapErrnoToLwError(errno);
+            LSA_LOG_ERROR("Could not notify service manager: %s (%i)",
+                    strerror(errno), errno);
             BAIL_ON_LSA_ERROR(dwError);
         }
 
@@ -223,6 +229,8 @@ cleanup:
 
     LSA_LOG_INFO("LSA Service exiting...");
 
+    dwError = LsaSrvStopEventLoggingThread();
+
     LsaSrvSetProcessExitCode(dwError);
 
     LsaShutdownLogging_r();
@@ -237,7 +245,7 @@ cleanup:
 
 error:
 
-    LSA_LOG_ERROR("LSA Process exiting due to error [Code:%d]", dwError);
+    LSA_LOG_ERROR("LSA Process exiting due to error [Code:%u]", dwError);
 
     LsaSrvLogProcessFailureEvent(dwError);
 
@@ -276,7 +284,7 @@ LsaSrvStartupPreCheck(
     if (iter >= STARTUP_PRE_CHECK_WAIT)
     {
         dwError = LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
-        LSA_LOG_ERROR("LSA start up pre-check failed to get updated hostname after %d seconds of waiting [Code:%d]",
+        LSA_LOG_ERROR("LSA start up pre-check failed to get updated hostname after %u seconds of waiting [Code:%u]",
                       STARTUP_PRE_CHECK_WAIT*10,
                       dwError);
         BAIL_ON_LSA_ERROR(dwError);
@@ -302,7 +310,7 @@ LsaSrvStartupPreCheck(
     if (iter >= STARTUP_NETLOGON_WAIT)
     {
         dwError = LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
-        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use NetLogonD after %d seconds of waiting [Code:%d]",
+        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use NetLogonD after %u seconds of waiting [Code:%u]",
                       STARTUP_NETLOGON_WAIT,
                       dwError);
         BAIL_ON_LSA_ERROR(dwError);
@@ -327,7 +335,7 @@ LsaSrvStartupPreCheck(
     if (iter >= STARTUP_LWIO_WAIT)
     {
         dwError = LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
-        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use LwIoD after %d seconds of waiting [Code:%d]",
+        LSA_LOG_ERROR("LSA start up pre-check failed to be able to use LwIoD after %u seconds of waiting [Code:%u]",
                       STARTUP_LWIO_WAIT,
                       dwError);
         BAIL_ON_LSA_ERROR(dwError);
@@ -347,7 +355,7 @@ cleanup:
 
 error:
 
-    LSA_LOG_ERROR("LSA Process exiting due to error checking hostname at startup [Code:%d]", dwError);
+    LSA_LOG_ERROR("LSA Process exiting due to error checking hostname at startup [Code:%u]", dwError);
 
     goto cleanup;
 }
@@ -361,7 +369,7 @@ LsaSrvVerifyNetLogonStatus(
     PSTR pszDomain = NULL;
 
     dwError = LWNetGetCurrentDomain(&pszDomain);
-    LSA_LOG_INFO("LsaSrvVerifyNetLogonStatus call to LWNet API returned %d", dwError);
+    LSA_LOG_INFO("LsaSrvVerifyNetLogonStatus call to LWNet API returned %u", dwError);
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
@@ -393,13 +401,13 @@ LsaSrvVerifyLwIoStatus(
     PIO_CONTEXT pContext = NULL;
 
     dwError = LwIoOpenContext(&pContext);
-    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %d", dwError);
+    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %u", dwError);
     BAIL_ON_LSA_ERROR(dwError);
 
     dwError = LwIoGetLogInfo(
                   (HANDLE) pContext,
                   &pLogInfo);
-    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %d", dwError);
+    LSA_LOG_INFO("LsaSrvVerifyLwIoStatus call to LwIo API returned %u", dwError);
     BAIL_ON_LSA_ERROR(dwError);
 
 cleanup:
@@ -438,7 +446,7 @@ LsaSrvRaiseMaxFiles(
 
     if (maxFiles.rlim_cur < dwMaxFiles)
     {
-        LSA_LOG_INFO("Raising soft max fd limit from %d to %d",
+        LSA_LOG_INFO("Raising soft max fd limit from %lu to %u",
                 maxFiles.rlim_cur,
                 dwMaxFiles);
         maxFiles.rlim_cur = dwMaxFiles;
@@ -446,7 +454,7 @@ LsaSrvRaiseMaxFiles(
     }
     if (maxFiles.rlim_max < dwMaxFiles)
     {
-        LSA_LOG_INFO("Raising hard max fd limit from %d to %d",
+        LSA_LOG_INFO("Raising hard max fd limit from %lu to %u",
                 maxFiles.rlim_max,
                 dwMaxFiles);
         maxFiles.rlim_max = dwMaxFiles;
@@ -457,8 +465,8 @@ LsaSrvRaiseMaxFiles(
     {
         if (setrlimit(RLIMIT_NOFILE, &maxFiles) < 0)
         {
-            LSA_LOG_ERROR("Raising max fd limit failed");
             dwError = LwMapErrnoToLwError(errno);
+            LSA_LOG_ERROR("Raising max fd limit failed");
             BAIL_ON_LSA_ERROR(dwError);
         }
     }
@@ -712,10 +720,10 @@ LsaSrvExitHandler(
     if (dwExitCode) {
        fp = fopen(szErrCodeFilePath, "w");
        if (fp == NULL) {
-          dwError = errno;
+          dwError = LwMapErrnoToLwError(errno);
           BAIL_ON_LSA_ERROR(dwError);
        }
-       fprintf(fp, "%d\n", dwExitCode);
+       fprintf(fp, "%u\n", dwExitCode);
     }
 
 error:
