@@ -160,7 +160,7 @@ LsaSrvGetDomainByName(
     dwError = LsaHashGetValue(pPolCtx->pDomains,
                               (PVOID)pKey,
                               OUT_PPVOID(&pEntry));
-    if (dwError == ERROR_FILE_NOT_FOUND)
+    if (dwError == ERROR_NOT_FOUND)
     {
         ntStatus = STATUS_NO_SUCH_DOMAIN;
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
@@ -220,7 +220,7 @@ LsaSrvGetDomainBySid(
     dwError = LsaHashGetValue(pPolCtx->pDomains,
                               (PVOID)pKey,
                               OUT_PPVOID(&pEntry));
-    if (dwError == ERROR_FILE_NOT_FOUND)
+    if (dwError == ERROR_NOT_FOUND)
     {
         ntStatus = STATUS_NO_SUCH_DOMAIN;
         BAIL_ON_NTSTATUS_ERROR(ntStatus);
@@ -371,7 +371,7 @@ LsaSrvDomainEntryDestroy(
         pEntry->hPolicy)
     {
         LsaClose(pEntry->hLsaBinding, pEntry->hPolicy);
-        FreeLsaBinding(&pEntry->hLsaBinding);
+        LsaFreeBinding(&pEntry->hLsaBinding);
     }
 
     LsaSrvDomainEntryFree(&pEntry);
@@ -640,12 +640,10 @@ LsaSrvConnectDomainByName(
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
-    RPCSTATUS rpcStatus = 0;
-    PSTR pszDcName = NULL;
     PIO_CREDS pCreds = NULL;
-    handle_t hLsaBinding = NULL;
+    LSA_BINDING hLsaBinding = NULL;
     POLICY_HANDLE hDcPolicy = NULL;
-    handle_t hNetrBinding = NULL;
+    NETR_BINDING hNetrBinding = NULL;
     NetrDomainTrust *pTrusts = NULL;
     DWORD dwNumTrusts = 0;
     DWORD i = 0;
@@ -655,10 +653,6 @@ LsaSrvConnectDomainByName(
     DOMAIN_ENTRY DomEntry = {0};
     PDOMAIN_ENTRY pDomEntry = NULL;
 
-    dwError = LwWc16sToMbs(pPolCtx->pwszDcName,
-                           &pszDcName);
-    BAIL_ON_LSA_ERROR(dwError);
-
     ntStatus = LsaSrvGetSystemCreds(&pCreds);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
@@ -680,14 +674,10 @@ LsaSrvConnectDomainByName(
         /*
          * Connect to a DC of the domain we're member of
          */
-        rpcStatus = InitLsaBindingDefault(&hLsaBinding,
-                                          pszDcName,
-                                          pCreds);
-        if (rpcStatus)
-        {
-            ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-            BAIL_ON_NTSTATUS_ERROR(ntStatus);
-        }
+        ntStatus = LsaInitBindingDefault(&hLsaBinding,
+                                         pPolCtx->pwszDcName,
+                                         pCreds);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = LsaOpenPolicy2(hLsaBinding,
                                   pPolCtx->pwszDcName,
@@ -718,15 +708,11 @@ LsaSrvConnectDomainByName(
          * domains and make a connection if so.
          */
 
-        rpcStatus = InitNetlogonBindingDefault(&hNetrBinding,
-                                               pszDcName,
-                                               pCreds,
-                                               FALSE);
-        if (rpcStatus)
-        {
-            ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-            BAIL_ON_NTSTATUS_ERROR(ntStatus);
-        }
+        ntStatus = NetrInitBindingDefault(&hNetrBinding,
+                                           pPolCtx->pwszDcName,
+                                           pCreds,
+                                           FALSE);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = DsrEnumerateDomainTrusts(hNetrBinding,
                                             pPolCtx->pwszDcName,
@@ -765,14 +751,10 @@ LsaSrvConnectDomainByName(
                                                &pszTrustedDcName);
             BAIL_ON_LSA_ERROR(dwError);
 
-            rpcStatus = InitLsaBindingDefault(&hLsaBinding,
-                                              pszTrustedDcName,
-                                              pCreds);
-            if (rpcStatus)
-            {
-                ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-                BAIL_ON_NTSTATUS_ERROR(ntStatus);
-            }
+            ntStatus = LsaInitBindingDefault(&hLsaBinding,
+                                             pTrust->dns_name,
+                                             pCreds);
+            BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
             ntStatus = LsaOpenPolicy2(hLsaBinding,
                                       pTrust->dns_name,
@@ -818,7 +800,7 @@ LsaSrvConnectDomainByName(
     *ppDomEntry = pDomEntry;
 
 cleanup:
-    FreeNetlogonBinding(&hNetrBinding);
+    NetrFreeBinding(&hNetrBinding);
 
     if (pTrusts)
     {
@@ -832,7 +814,6 @@ cleanup:
 
     LWNetFreeString(pszTrustedDcName);
 
-    LW_SAFE_FREE_MEMORY(pszDcName);
     LW_SAFE_FREE_MEMORY(pszTrustedDcName);
 
     if (ntStatus == STATUS_SUCCESS &&
@@ -860,12 +841,10 @@ LsaSrvConnectDomainBySid(
 
     DWORD dwError = ERROR_SUCCESS;
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    RPCSTATUS rpcStatus = 0;
-    PSTR pszDcName = NULL;
     PIO_CREDS pCreds = NULL;
-    handle_t hLsaBinding= NULL;
+    LSA_BINDING hLsaBinding= NULL;
     POLICY_HANDLE hDcPolicy = NULL;
-    handle_t hNetrBinding = NULL;
+    NETR_BINDING hNetrBinding = NULL;
     NetrDomainTrust *pTrusts = NULL;
     DWORD dwNumTrusts = 0;
     DWORD i = 0;
@@ -875,10 +854,6 @@ LsaSrvConnectDomainBySid(
     DOMAIN_ENTRY DomEntry = {0};
     PDOMAIN_ENTRY pDomEntry = NULL;
 
-    dwError = LwWc16sToMbs(pPolCtx->pwszDcName,
-                           &pszDcName);
-    BAIL_ON_LSA_ERROR(dwError);
-
     ntStatus = LsaSrvGetSystemCreds(&pCreds);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
@@ -900,14 +875,10 @@ LsaSrvConnectDomainBySid(
         /*
          * Connect to a DC of the domain we're member of
          */
-        rpcStatus = InitLsaBindingDefault(&hLsaBinding,
-                                          pszDcName,
-                                          pCreds);
-        if (rpcStatus)
-        {
-            ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-            BAIL_ON_NTSTATUS_ERROR(ntStatus);
-        }
+        ntStatus = LsaInitBindingDefault(&hLsaBinding,
+                                         pPolCtx->pwszDcName,
+                                         pCreds);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = LsaOpenPolicy2(hLsaBinding,
                                   pPolCtx->pwszDcName,
@@ -938,15 +909,11 @@ LsaSrvConnectDomainBySid(
          * domains and make a connection if so.
          */
 
-        rpcStatus = InitNetlogonBindingDefault(&hNetrBinding,
-                                               pszDcName,
-                                               pCreds,
-                                               FALSE);
-        if (rpcStatus)
-        {
-            ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-            BAIL_ON_NTSTATUS_ERROR(ntStatus);
-        }
+        ntStatus = NetrInitBindingDefault(&hNetrBinding,
+                                          pPolCtx->pwszDcName,
+                                          pCreds,
+                                          FALSE);
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
         ntStatus = DsrEnumerateDomainTrusts(hNetrBinding,
                                             pPolCtx->pwszDcName,
@@ -985,14 +952,10 @@ LsaSrvConnectDomainBySid(
                                                &pszTrustedDcName);
             BAIL_ON_LSA_ERROR(dwError);
 
-            rpcStatus = InitLsaBindingDefault(&hLsaBinding,
-                                              pszTrustedDcName,
-                                              pCreds);
-            if (rpcStatus)
-            {
-                ntStatus = LwRpcStatusToNtStatus(rpcStatus);
-                BAIL_ON_NTSTATUS_ERROR(ntStatus);
-            }
+            ntStatus = LsaInitBindingDefault(&hLsaBinding,
+                                             pTrust->dns_name,
+                                             pCreds);
+            BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
             ntStatus = LsaOpenPolicy2(hLsaBinding,
                                       pTrust->dns_name,
@@ -1038,7 +1001,7 @@ LsaSrvConnectDomainBySid(
     *ppDomEntry = pDomEntry;
 
 cleanup:
-    FreeNetlogonBinding(&hNetrBinding);
+    NetrFreeBinding(&hNetrBinding);
 
     if (pTrusts)
     {
@@ -1050,10 +1013,10 @@ cleanup:
         LwIoDeleteCreds(pCreds);
     }
 
-    LWNetFreeString(pszTrustedDcName);
-
-    LW_SAFE_FREE_MEMORY(pszDcName);
-    LW_SAFE_FREE_MEMORY(pszTrustedDcName);
+    if (pszTrustedDcName)
+    {
+        LWNetFreeString(pszTrustedDcName);
+    }
 
     if (ntStatus == STATUS_SUCCESS &&
         dwError != ERROR_SUCCESS)
