@@ -29,7 +29,17 @@
  */
 
 /*
- * Abstract: Samr wrapper functions called from DCE/RPC stubs (rpc server library)
+ * Copyright (C) Likewise Software. All rights reserved.
+ *
+ * Module Name:
+ *
+ *        samr.c
+ *
+ * Abstract:
+ *
+ *        Remote Procedure Call (RPC) Server Interface
+ *
+ *        Samr rpc server stub functions
  *
  * Authors: Rafal Szczesniak (rafal@likewise.com)
  */
@@ -37,7 +47,7 @@
 #include "includes.h"
 
 
-NTSTATUS __SamrConnect(
+NTSTATUS srv_SamrConnect(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ wchar16_t *system_name,
     /* [in] */ UINT32 access_mask,
@@ -54,22 +64,20 @@ NTSTATUS __SamrConnect(
 }
 
 
-NTSTATUS __SamrClose(
+NTSTATUS srv_SamrClose(
     /* [in] */ handle_t IDL_handle,
-    /* [in,context_handle] */ void *hIn,
-    /* [out,context_handle] */ void **hOut
+    /* [in,context_handle] */ void **phInOut
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvClose(IDL_handle,
-                          hIn,
-                          hOut);
+                          phInOut);
     return status;
 }
 
 
-NTSTATUS __SamrSetSecurity(
+NTSTATUS srv_SamrSetSecurity(
     /* [in] */ handle_t IDL_handle,
     /* [in,context_handle] */ void *hObject,
     /* [in] */ UINT32 security_info,
@@ -86,7 +94,7 @@ NTSTATUS __SamrSetSecurity(
 }
 
 
-NTSTATUS __SamrQuerySecurity(
+NTSTATUS srv_SamrQuerySecurity(
     /* [in] */ handle_t IDL_handle,
     /* [in,context_handle] */ void *hObject,
     /* [in] */ UINT32 security_info,
@@ -103,7 +111,7 @@ NTSTATUS __SamrQuerySecurity(
 }
 
 
-NTSTATUS _samr_Function04(
+NTSTATUS srv_samr_Function04(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -112,10 +120,10 @@ NTSTATUS _samr_Function04(
 }
 
 
-NTSTATUS __SamrLookupDomain(
+NTSTATUS srv_SamrLookupDomain(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ CONNECT_HANDLE hConn,
-    /* [in] */ UnicodeString *domain_name,
+    /* [in] */ UNICODE_STRING *domain_name,
     /* [out] */ SID **sid
     )
 {
@@ -129,12 +137,12 @@ NTSTATUS __SamrLookupDomain(
 }
 
 
-NTSTATUS __SamrEnumDomains(
+NTSTATUS srv_SamrEnumDomains(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ CONNECT_HANDLE hConn,
     /* [in, out] */ UINT32 *resume,
     /* [in] */ UINT32 size,
-    /* [out] */ EntryArray **domains,
+    /* [out] */ ENTRY_ARRAY **domains,
     /* [out] */ UINT32 *num_entries
     )
 {
@@ -150,7 +158,7 @@ NTSTATUS __SamrEnumDomains(
 }
 
 
-NTSTATUS __SamrOpenDomain(
+NTSTATUS srv_SamrOpenDomain(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ CONNECT_HANDLE hConn,
     /* [in] */ UINT32 access_mask,
@@ -169,7 +177,7 @@ NTSTATUS __SamrOpenDomain(
 }
 
 
-NTSTATUS __SamrQueryDomainInfo(
+NTSTATUS srv_SamrQueryDomainInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT16 level,
@@ -186,7 +194,7 @@ NTSTATUS __SamrQueryDomainInfo(
 }
 
 
-NTSTATUS _samr_Function09(
+NTSTATUS srv_samr_Function09(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -195,28 +203,33 @@ NTSTATUS _samr_Function09(
 }
 
 
-NTSTATUS _samr_Function0a(
-    /* [in] */ handle_t IDL_handle
-    )
-{
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
-    return status;
-}
-
-
-NTSTATUS _samr_Function0b(
-    /* [in] */ handle_t IDL_handle
-    )
-{
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
-    return status;
-}
-
-
-NTSTATUS __SamrCreateUser(
+NTSTATUS srv_SamrCreateDomGroup(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
-    /* [in] */ UnicodeString *account_name,
+    /* [in] */ UNICODE_STRING *group_name,
+    /* [in] */ UINT32 access_mask,
+    /* [out] */ ACCOUNT_HANDLE *hGroup,
+    /* [out] */ UINT32 *rid
+    )
+{
+    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    return status;
+}
+
+
+NTSTATUS srv_samr_Function0b(
+    /* [in] */ handle_t IDL_handle
+    )
+{
+    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    return status;
+}
+
+
+NTSTATUS srv_SamrCreateUser(
+    /* [in] */ handle_t IDL_handle,
+    /* [in] */ DOMAIN_HANDLE hDomain,
+    /* [in] */ UNICODE_STRING *account_name,
     /* [in] */ UINT32 access_mask,
     /* [out] */ ACCOUNT_HANDLE *hUser,
     /* [out] */ UINT32 *rid
@@ -234,13 +247,13 @@ NTSTATUS __SamrCreateUser(
 }
 
 
-NTSTATUS __SamrEnumDomainUsers(
+NTSTATUS srv_SamrEnumDomainUsers(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in, out] */ UINT32 *resume,
     /* [in] */ UINT32 account_flags,
     /* [in] */ UINT32 max_size,
-    /* [out] */ RidNameArray **names,
+    /* [out] */ RID_NAME_ARRAY **names,
     /* [out] */ UINT32 *num_entries
     )
 {
@@ -257,10 +270,10 @@ NTSTATUS __SamrEnumDomainUsers(
 }
 
 
-NTSTATUS __SamrCreateDomAlias(
+NTSTATUS srv_SamrCreateDomAlias(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
-    /* [in] */ UnicodeString *alias_name,
+    /* [in] */ UNICODE_STRING *alias_name,
     /* [in] */ UINT32 access_mask,
     /* [out] */ ACCOUNT_HANDLE *hAlias,
     /* [out] */ UINT32 *rid
@@ -278,12 +291,12 @@ NTSTATUS __SamrCreateDomAlias(
 }
 
 
-NTSTATUS __SamrEnumDomainAliases(
+NTSTATUS srv_SamrEnumDomainAliases(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in, out] */ UINT32 *resume,
     /* [in] */ UINT32 account_flags,
-    /* [out] */ RidNameArray **names,
+    /* [out] */ RID_NAME_ARRAY **names,
     /* [out] */ UINT32 *num_entries
     )
 {
@@ -299,11 +312,11 @@ NTSTATUS __SamrEnumDomainAliases(
 }
 
 
-NTSTATUS __SamrGetAliasMembership(
+NTSTATUS srv_SamrGetAliasMembership(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
-    /* [in] */ SidArray *sids,
-    /* [out] */ Ids *rids
+    /* [in] */ SID_ARRAY *sids,
+    /* [out] */ IDS *rids
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -316,13 +329,13 @@ NTSTATUS __SamrGetAliasMembership(
 }
 
 
-NTSTATUS __SamrLookupNames(
+NTSTATUS srv_SamrLookupNames(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT32 num_names,
-    /* [in] */ UnicodeString *names,
-    /* [out] */ Ids *ids,
-    /* [out] */ Ids *types
+    /* [in] */ UNICODE_STRING *names,
+    /* [out] */ IDS *ids,
+    /* [out] */ IDS *types
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -337,13 +350,13 @@ NTSTATUS __SamrLookupNames(
 }
 
 
-NTSTATUS __SamrLookupRids(
+NTSTATUS srv_SamrLookupRids(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT32 num_rids,
     /* [in] */ UINT32 *rids,
-    /* [out] */ UnicodeStringArray *names,
-    /* [out] */ Ids *types
+    /* [out] */ UNICODE_STRING_ARRAY *names,
+    /* [out] */ IDS *types
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -358,7 +371,20 @@ NTSTATUS __SamrLookupRids(
 }
 
 
-NTSTATUS _samr_Function13(
+NTSTATUS srv_SamrOpenGroup(
+    /* [in] */ handle_t IDL_handle,
+    /* [in] */ DOMAIN_HANDLE hDomain,
+    /* [in] */ UINT32 access_mask,
+    /* [in] */ UINT32 rid,
+    /* [out] */ ACCOUNT_HANDLE *hGroup
+)
+{
+    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    return status;
+}
+
+
+NTSTATUS srv_samr_Function14(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -367,7 +393,7 @@ NTSTATUS _samr_Function13(
 }
 
 
-NTSTATUS _samr_Function14(
+NTSTATUS srv_samr_Function15(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -376,7 +402,7 @@ NTSTATUS _samr_Function14(
 }
 
 
-NTSTATUS _samr_Function15(
+NTSTATUS srv_samr_Function16(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -385,7 +411,18 @@ NTSTATUS _samr_Function15(
 }
 
 
-NTSTATUS _samr_Function16(
+NTSTATUS srv_SamrDeleteDomGroup(
+    /* [in] */ handle_t IDL_handle,
+    /* [in] */ ACCOUNT_HANDLE hGroupIn,
+    /* [out] */ ACCOUNT_HANDLE *hGroupOut
+)
+{
+    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    return status;
+}
+
+
+NTSTATUS srv_samr_Function18(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -394,7 +431,7 @@ NTSTATUS _samr_Function16(
 }
 
 
-NTSTATUS _samr_Function17(
+NTSTATUS srv_samr_Function19(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -403,7 +440,7 @@ NTSTATUS _samr_Function17(
 }
 
 
-NTSTATUS _samr_Function18(
+NTSTATUS srv_samr_Function1a(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -412,25 +449,7 @@ NTSTATUS _samr_Function18(
 }
 
 
-NTSTATUS _samr_Function19(
-    /* [in] */ handle_t IDL_handle
-    )
-{
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
-    return status;
-}
-
-
-NTSTATUS _samr_Function1a(
-    /* [in] */ handle_t IDL_handle
-    )
-{
-    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
-    return status;
-}
-
-
-NTSTATUS __SamrOpenAlias(
+NTSTATUS srv_SamrOpenAlias(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT32 access_mask,
@@ -449,7 +468,7 @@ NTSTATUS __SamrOpenAlias(
 }
 
 
-NTSTATUS __SamrQueryAliasInfo(
+NTSTATUS srv_SamrQueryAliasInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hAlias,
     /* [in] */ UINT16 level,
@@ -466,7 +485,7 @@ NTSTATUS __SamrQueryAliasInfo(
 }
 
 
-NTSTATUS __SamrSetAliasInfo(
+NTSTATUS srv_SamrSetAliasInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hAlias,
     /* [in] */ UINT16 level,
@@ -483,22 +502,20 @@ NTSTATUS __SamrSetAliasInfo(
 }
 
 
-NTSTATUS __SamrDeleteDomAlias(
+NTSTATUS srv_SamrDeleteDomAlias(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ ACCOUNT_HANDLE hAliasIn,
-    /* [out] */ ACCOUNT_HANDLE *hAliasOut
+    /* [in,out] */ ACCOUNT_HANDLE *phAlias
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvDeleteDomAlias(IDL_handle,
-                                   hAliasIn,
-                                   hAliasOut);
+                                   phAlias);
     return status;
 }
 
 
-NTSTATUS __SamrAddAliasMember(
+NTSTATUS srv_SamrAddAliasMember(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hAlias,
     /* [in] */ SID *sid
@@ -513,7 +530,7 @@ NTSTATUS __SamrAddAliasMember(
 }
 
 
-NTSTATUS __SamrDeleteAliasMember(
+NTSTATUS srv_SamrDeleteAliasMember(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hAlias,
     /* [in] */ SID *sid
@@ -528,10 +545,10 @@ NTSTATUS __SamrDeleteAliasMember(
 }
 
 
-NTSTATUS __SamrGetMembersInAlias(
+NTSTATUS srv_SamrGetMembersInAlias(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hAlias,
-    /* [out] */ SidArray *sids
+    /* [out] */ SID_ARRAY *sids
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -543,7 +560,7 @@ NTSTATUS __SamrGetMembersInAlias(
 }
 
 
-NTSTATUS __SamrOpenUser(
+NTSTATUS srv_SamrOpenUser(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT32 access_mask,
@@ -562,22 +579,20 @@ NTSTATUS __SamrOpenUser(
 }
 
 
-NTSTATUS __SamrDeleteUser(
+NTSTATUS srv_SamrDeleteUser(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ ACCOUNT_HANDLE hUserIn,
-    /* [in] */ ACCOUNT_HANDLE *phUserOut
+    /* [in] */ ACCOUNT_HANDLE *phUser
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvDeleteUser(IDL_handle,
-                               hUserIn,
-                               phUserOut);
+                               phUser);
     return status;
 }
 
 
-NTSTATUS __SamrQueryUserInfo(
+NTSTATUS srv_SamrQueryUserInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hUser,
     /* [in] */ UINT16 level,
@@ -594,7 +609,7 @@ NTSTATUS __SamrQueryUserInfo(
 }
 
 
-NTSTATUS __SamrSetUserInfo(
+NTSTATUS srv_SamrSetUserInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hUser,
     /* [in] */ UINT16 level,
@@ -611,7 +626,7 @@ NTSTATUS __SamrSetUserInfo(
 }
 
 
-NTSTATUS _samr_Function26(
+NTSTATUS srv_samr_Function26(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -620,10 +635,10 @@ NTSTATUS _samr_Function26(
 }
 
 
-NTSTATUS __SamrGetUserGroups(
+NTSTATUS srv_SamrGetUserGroups(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hUser,
-    /* [out] */ RidWithAttributeArray **rids
+    /* [out] */ RID_WITH_ATTRIBUTE_ARRAY **rids
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -635,7 +650,7 @@ NTSTATUS __SamrGetUserGroups(
 }
 
 
-NTSTATUS __SamrQueryDisplayInfo(
+NTSTATUS srv_SamrQueryDisplayInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ UINT16 level,
@@ -662,7 +677,7 @@ NTSTATUS __SamrQueryDisplayInfo(
 }
 
 
-NTSTATUS _samr_Function29(
+NTSTATUS srv_samr_Function29(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -671,7 +686,7 @@ NTSTATUS _samr_Function29(
 }
 
 
-NTSTATUS _samr_Function2a(
+NTSTATUS srv_samr_Function2a(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -680,7 +695,7 @@ NTSTATUS _samr_Function2a(
 }
 
 
-NTSTATUS _samr_Function2b(
+NTSTATUS srv_samr_Function2b(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -689,7 +704,7 @@ NTSTATUS _samr_Function2b(
 }
 
 
-NTSTATUS __SamrGetUserPwInfo(
+NTSTATUS srv_SamrGetUserPwInfo(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hUser,
     /* [out] */ PwInfo *info
@@ -704,7 +719,7 @@ NTSTATUS __SamrGetUserPwInfo(
 }
 
 
-NTSTATUS __SamrRemoveMemberFromForeignDomain(
+NTSTATUS srv_SamrRemoveMemberFromForeignDomain(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
     /* [in] */ PSID sid
@@ -719,7 +734,7 @@ NTSTATUS __SamrRemoveMemberFromForeignDomain(
 }
 
 
-NTSTATUS _samr_Function2e(
+NTSTATUS srv_samr_Function2e(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -728,7 +743,7 @@ NTSTATUS _samr_Function2e(
 }
 
 
-NTSTATUS _samr_Function2f(
+NTSTATUS srv_samr_Function2f(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -737,7 +752,7 @@ NTSTATUS _samr_Function2f(
 }
 
 
-NTSTATUS _samr_Function30(
+NTSTATUS srv_samr_Function30(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -746,7 +761,7 @@ NTSTATUS _samr_Function30(
 }
 
 
-NTSTATUS _samr_Function31(
+NTSTATUS srv_samr_Function31(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -755,10 +770,10 @@ NTSTATUS _samr_Function31(
 }
 
 
-NTSTATUS __SamrCreateUser2(
+NTSTATUS srv_SamrCreateUser2(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ DOMAIN_HANDLE hDomain,
-    /* [in] */ UnicodeStringEx *account_name,
+    /* [in] */ UNICODE_STRING *account_name,
     /* [in] */ UINT32 account_flags,
     /* [in] */ UINT32 access_mask,
     /* [out] */ ACCOUNT_HANDLE *hUser,
@@ -780,7 +795,7 @@ NTSTATUS __SamrCreateUser2(
 }
 
 
-NTSTATUS _samr_Function33(
+NTSTATUS srv_samr_Function33(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -789,7 +804,7 @@ NTSTATUS _samr_Function33(
 }
 
 
-NTSTATUS _samr_Function34(
+NTSTATUS srv_samr_Function34(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -798,7 +813,7 @@ NTSTATUS _samr_Function34(
 }
 
 
-NTSTATUS _samr_Function35(
+NTSTATUS srv_samr_Function35(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -807,7 +822,7 @@ NTSTATUS _samr_Function35(
 }
 
 
-NTSTATUS _samr_Function36(
+NTSTATUS srv_samr_Function36(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -816,10 +831,10 @@ NTSTATUS _samr_Function36(
 }
 
 
-NTSTATUS __SamrChangePasswordUser2(
+NTSTATUS srv_SamrChangePasswordUser2(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ UnicodeString *server,
-    /* [in] */ UnicodeString *account_name,
+    /* [in] */ UNICODE_STRING *server,
+    /* [in] */ UNICODE_STRING *account_name,
     /* [in] */ CryptPassword *nt_password,
     /* [in] */ HashPassword *nt_verifier,
     /* [in] */ UINT8 lm_change,
@@ -841,7 +856,7 @@ NTSTATUS __SamrChangePasswordUser2(
 }
 
 
-NTSTATUS _samr_Function38(
+NTSTATUS srv_samr_Function38(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -850,9 +865,8 @@ NTSTATUS _samr_Function38(
 }
 
 
-NTSTATUS __SamrConnect2(
+NTSTATUS srv_SamrConnect2(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ UINT32 size,
     /* [in] */ wchar16_t *system_name,
     /* [in] */ UINT32 access_mask,
     /* [out] */ CONNECT_HANDLE *hConn
@@ -861,7 +875,6 @@ NTSTATUS __SamrConnect2(
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvConnect2(IDL_handle,
-                             size,
                              system_name,
                              access_mask,
                              hConn);
@@ -869,7 +882,7 @@ NTSTATUS __SamrConnect2(
 }
 
 
-NTSTATUS __SamrSetUserInfo2(
+NTSTATUS srv_SamrSetUserInfo2(
     /* [in] */ handle_t IDL_handle,
     /* [in] */ ACCOUNT_HANDLE hUser,
     /* [in] */ UINT16 level,
@@ -886,7 +899,7 @@ NTSTATUS __SamrSetUserInfo2(
 }
 
 
-NTSTATUS _samr_Function3a(
+NTSTATUS srv_samr_Function3a(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -895,7 +908,7 @@ NTSTATUS _samr_Function3a(
 }
 
 
-NTSTATUS _samr_Function3b(
+NTSTATUS srv_samr_Function3b(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -904,7 +917,7 @@ NTSTATUS _samr_Function3b(
 }
 
 
-NTSTATUS _samr_Function3c(
+NTSTATUS srv_samr_Function3c(
     /* [in] */ handle_t IDL_handle
     )
 {
@@ -913,9 +926,8 @@ NTSTATUS _samr_Function3c(
 }
 
 
-NTSTATUS __SamrConnect3(
+NTSTATUS srv_SamrConnect3(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ UINT32 size,
     /* [in] */ wchar16_t *system_name,
     /* [in] */ UINT32 unknown,
     /* [in] */ UINT32 access_mask,
@@ -925,7 +937,6 @@ NTSTATUS __SamrConnect3(
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvConnect3(IDL_handle,
-                             size,
                              system_name,
                              unknown,
                              access_mask,
@@ -934,9 +945,8 @@ NTSTATUS __SamrConnect3(
 }
 
 
-NTSTATUS __SamrConnect4(
+NTSTATUS srv_SamrConnect4(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ UINT32 size,
     /* [in] */ wchar16_t *system_name,
     /* [in] */ UINT32 client_access,
     /* [in] */ UINT32 access_mask,
@@ -946,7 +956,6 @@ NTSTATUS __SamrConnect4(
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvConnect4(IDL_handle,
-                             size,
                              system_name,
                              client_access,
                              access_mask,
@@ -955,9 +964,17 @@ NTSTATUS __SamrConnect4(
 }
 
 
-NTSTATUS __SamrConnect5(
+NTSTATUS srv_samr_Function3f(
+    /* [in] */ handle_t IDL_handle
+    )
+{
+    NTSTATUS status = STATUS_NOT_IMPLEMENTED;
+    return status;
+}
+
+
+NTSTATUS srv_SamrConnect5(
     /* [in] */ handle_t IDL_handle,
-    /* [in] */ UINT32 size,
     /* [in] */ wchar16_t *system_name,
     /* [in] */ UINT32 access_mask,
     /* [in] */ UINT32 level_in,
@@ -970,7 +987,6 @@ NTSTATUS __SamrConnect5(
     NTSTATUS status = STATUS_SUCCESS;
 
     status = SamrSrvConnect5(IDL_handle,
-                             size,
                              system_name,
                              access_mask,
                              level_in,
