@@ -47,6 +47,21 @@
 #include "includes.h"
 
 
+static
+DWORD
+SamrSrvCreateServerDacl(
+    PACL *ppDacl
+    );
+
+
+static
+DWORD
+SamrSrvCreateDacl(
+    PACL *ppDacl,
+    PACCESS_LIST pList
+    );
+
+
 static
 NTSTATUS
 SamrSrvCreateLocalUserDacl(
@@ -67,6 +82,287 @@ SamrSrvCreateLocalGroupDacl(
     );
 
 
+DWORD
+SamrSrvInitServerSecurityDescriptor(
+    PSECURITY_DESCRIPTOR_ABSOLUTE *ppSecDesc
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
+    PSID pOwnerSid = NULL;
+    PSID pGroupSid = NULL;
+    PACL pDacl = NULL;
+
+    BAIL_ON_INVALID_PTR(ppSecDesc);
+
+    dwError = LwAllocateMemory(SECURITY_DESCRIPTOR_ABSOLUTE_MIN_SIZE,
+                               OUT_PPVOID(&pSecDesc));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlCreateSecurityDescriptorAbsolute(
+                                    pSecDesc,
+                                    SECURITY_DESCRIPTOR_REVISION);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    dwError = LwCreateWellKnownSid(WinLocalSystemSid,
+                                   NULL,
+                                   &pOwnerSid,
+                                   NULL);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlSetOwnerSecurityDescriptor(
+                                    pSecDesc,
+                                    pOwnerSid,
+                                    FALSE);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pGroupSid,
+                                   NULL);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlSetGroupSecurityDescriptor(
+                                    pSecDesc,
+                                    pGroupSid,
+                                    FALSE);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    dwError = SamrSrvCreateServerDacl(&pDacl);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlSetDaclSecurityDescriptor(
+                                    pSecDesc,
+                                    TRUE,
+                                    pDacl,
+                                    FALSE);
+
+    *ppSecDesc = pSecDesc;
+
+cleanup:
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
+
+    return dwError;
+
+error:
+    LW_SAFE_FREE_MEMORY(pSecDesc);
+
+    *ppSecDesc = NULL;
+    goto cleanup;
+}
+
+
+static
+DWORD
+SamrSrvCreateServerDacl(
+    PACL *ppDacl
+    )
+{
+    ACCESS_MASK SystemAccessMask = STANDARD_RIGHTS_REQUIRED |
+                                   SAMR_ACCESS_CONNECT_TO_SERVER |
+                                   SAMR_ACCESS_SHUTDOWN_SERVER |
+                                   SAMR_ACCESS_INITIALIZE_SERVER |
+                                   SAMR_ACCESS_CREATE_DOMAIN |
+                                   SAMR_ACCESS_ENUM_DOMAINS |
+                                   SAMR_ACCESS_OPEN_DOMAIN;
+
+    ACCESS_MASK AdminAccessMask = STANDARD_RIGHTS_REQUIRED |
+                                  SAMR_ACCESS_CONNECT_TO_SERVER |
+                                  SAMR_ACCESS_SHUTDOWN_SERVER |
+                                  SAMR_ACCESS_INITIALIZE_SERVER |
+                                  SAMR_ACCESS_ENUM_DOMAINS |
+                                  SAMR_ACCESS_OPEN_DOMAIN;
+
+    ACCESS_MASK AuthenticatedAccessMask = STANDARD_RIGHTS_READ |
+                                          SAMR_ACCESS_CONNECT_TO_SERVER |
+                                          SAMR_ACCESS_ENUM_DOMAINS |
+                                          SAMR_ACCESS_OPEN_DOMAIN;
+
+    ACCESS_MASK WorldAccessMask = STANDARD_RIGHTS_READ;
+
+    DWORD dwError = ERROR_SUCCESS;
+    DWORD dwSystemSidLen = 0;
+    DWORD dwBuiltinAdminsSidLen = 0;
+    DWORD dwAuthenticatedSidLen = 0;
+    DWORD dwWorldSidLen = 0;
+    PSID pSystemSid = NULL;
+    PSID pBuiltinAdminsSid = NULL;
+    PSID pAuthenticatedSid = NULL;
+    PSID pWorldSid = NULL;
+    PACL pDacl = NULL;
+
+    ACCESS_LIST AccessList[] = {
+        {
+            .ppSid        = &pSystemSid,
+            .AccessMask   = SystemAccessMask,
+            .ulAccessType = ACCESS_ALLOWED_ACE_TYPE
+        },
+        {
+            .ppSid        = &pBuiltinAdminsSid,
+            .AccessMask   = AdminAccessMask,
+            .ulAccessType = ACCESS_ALLOWED_ACE_TYPE
+        },
+        {
+            .ppSid        = &pAuthenticatedSid,
+            .AccessMask   = AuthenticatedAccessMask,
+            .ulAccessType = ACCESS_ALLOWED_ACE_TYPE
+        },
+        {
+            .ppSid        = &pWorldSid,
+            .AccessMask   = WorldAccessMask,
+            .ulAccessType = ACCESS_ALLOWED_ACE_TYPE
+        },
+        {
+            .ppSid        = NULL,
+            .AccessMask   = 0,
+            .ulAccessType = 0
+        }
+    };
+
+    /* create local system sid */
+    dwError = LwCreateWellKnownSid(WinLocalSystemSid,
+                                   NULL,
+                                   &pSystemSid,
+                                   &dwSystemSidLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /* create administrators sid */
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   &dwBuiltinAdminsSidLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /* create authenticated users sid */
+    dwError = LwCreateWellKnownSid(WinAuthenticatedUserSid,
+                                   NULL,
+                                   &pAuthenticatedSid,
+                                   &dwAuthenticatedSidLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    /* create world (everyone) sid */
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   &dwWorldSidLen);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    dwError = SamrSrvCreateDacl(&pDacl,
+                                AccessList);
+    BAIL_ON_LSA_ERROR(dwError);
+
+    *ppDacl = pDacl;
+
+cleanup:
+    LW_SAFE_FREE_MEMORY(pSystemSid);
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pAuthenticatedSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
+
+    return dwError;
+
+error:
+    *ppDacl = NULL;
+
+    goto cleanup;
+}
+
+
+static
+DWORD
+SamrSrvCreateDacl(
+    PACL *ppDacl,
+    PACCESS_LIST pList
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    DWORD dwDaclSize = 0;
+    PACL pDacl = NULL;
+    DWORD i = 0;
+    ULONG ulSidSize = 0;
+
+    dwDaclSize += ACL_HEADER_SIZE;
+
+    for (i = 0; pList[i].ppSid && (*pList[i].ppSid); i++)
+    {
+        ulSidSize = RtlLengthSid(*(pList[i].ppSid));
+
+        if (pList[i].ulAccessType == ACCESS_ALLOWED_ACE_TYPE)
+        {
+            dwDaclSize += ulSidSize + sizeof(ACCESS_ALLOWED_ACE);
+        }
+        else if (pList[i].ulAccessType == ACCESS_DENIED_ACE_TYPE)
+        {
+            dwDaclSize += ulSidSize + sizeof(ACCESS_DENIED_ACE);
+        }
+    }
+
+    dwError = LwAllocateMemory(dwDaclSize,
+                               OUT_PPVOID(&pDacl));
+    BAIL_ON_LSA_ERROR(dwError);
+
+    ntStatus = RtlCreateAcl(pDacl, dwDaclSize, ACL_REVISION);
+    BAIL_ON_NTSTATUS_ERROR(ntStatus);
+
+    for (i = 0; pList[i].ppSid && (*pList[i].ppSid); i++)
+    {
+        if (pList[i].ulAccessType == ACCESS_ALLOWED_ACE_TYPE)
+        {
+            ntStatus = RtlAddAccessAllowedAceEx(pDacl,
+                                                ACL_REVISION,
+                                                0,
+                                                pList[i].AccessMask,
+                                                *(pList[i].ppSid));
+        }
+        else if (pList[i].ulAccessType == ACCESS_DENIED_ACE_TYPE)
+        {
+            ntStatus = RtlAddAccessDeniedAceEx(pDacl,
+                                               ACL_REVISION,
+                                               0,
+                                               pList[i].AccessMask,
+                                               *(pList[i].ppSid));
+        }
+
+        BAIL_ON_NTSTATUS_ERROR(ntStatus);
+    }
+
+    *ppDacl = pDacl;
+
+cleanup:
+    if (dwError == ERROR_SUCCESS &&
+        ntStatus != STATUS_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+    }
+
+    return dwError;
+
+error:
+    LW_SAFE_FREE_MEMORY(pDacl);
+    *ppDacl = NULL;
+
+    goto cleanup;
+}
+
+
+DWORD
+SamrSrvDestroyServerSecurityDescriptor(
+    PSECURITY_DESCRIPTOR_ABSOLUTE *ppSecDesc
+    )
+{
+    DWORD dwError = ERROR_SUCCESS;
+
+    SamrSrvFreeSecurityDescriptor(ppSecDesc);
+    return dwError;
+}
+
+
 NTSTATUS
 SamrSrvCreateNewAccountSecurityDescriptor(
     PSID                           pDomainSid,
@@ -144,7 +440,7 @@ SamrSrvCreateNewAccountSecurityDescriptor(
                                                dwObjectClass,
                                                &pDacl);
     }
-    BAIL_ON_NTSTATUS_ERROR(dwError);
+    BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
     ntStatus = RtlSetDaclSecurityDescriptor(
                                     pSecDesc,
@@ -156,9 +452,17 @@ SamrSrvCreateNewAccountSecurityDescriptor(
     *ppSecDesc = pSecDesc;
 
 cleanup:
+    if (ntStatus == STATUS_SUCCESS &&
+        dwError != ERROR_SUCCESS)
+    {
+        ntStatus = LwWin32ErrorToNtStatus(dwError);
+    }
+
     return ntStatus;
 
 error:
+    SamrSrvFreeSecurityDescriptor(&pSecDesc);
+
     goto cleanup;
 }
 
@@ -274,22 +578,32 @@ SamrSrvCreateLocalUserDacl(
                                    NULL);
     BAIL_ON_LSA_ERROR(dwError);
 
-    ntStatus = SamrSrvCreateDacl(&pDacl,
-                                 AccessList);
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
+    dwError = SamrSrvCreateDacl(&pDacl,
+                                AccessList);
+    BAIL_ON_LSA_ERROR(dwError);
 
     *ppDacl = pDacl;
 
 cleanup:
-    return ntStatus;
-
-error:
     LW_SAFE_FREE_MEMORY(pAdminSid);
     LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
     LW_SAFE_FREE_MEMORY(pAccountSid);
     LW_SAFE_FREE_MEMORY(pWorldSid);
 
-    *ppDacl = NULL;
+    if (ntStatus == STATUS_SUCCESS &&
+        dwError != ERROR_SUCCESS)
+    {
+        ntStatus = LwWin32ErrorToNtStatus(dwError);
+    }
+
+    return ntStatus;
+
+error:
+    if (ppDacl)
+    {
+        *ppDacl = NULL;
+    }
+
     goto cleanup;
 }
 
@@ -314,7 +628,6 @@ SamrSrvCreateLocalGroupDacl(
                                 ALIAS_ACCESS_GET_MEMBERS |
                                 ALIAS_ACCESS_LOOKUP_INFO;
 
-    NTSTATUS ntStatus = STATUS_SUCCESS;
     DWORD dwError = ERROR_SUCCESS;
     PSID pAdminSid = NULL;
     PSID pBuiltinAdminsSid = NULL;
@@ -362,97 +675,24 @@ SamrSrvCreateLocalGroupDacl(
                                    NULL);
     BAIL_ON_LSA_ERROR(dwError);
 
-    ntStatus = SamrSrvCreateDacl(&pDacl,
-                                 AccessList);
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
+    dwError = SamrSrvCreateDacl(&pDacl,
+                                AccessList);
+    BAIL_ON_LSA_ERROR(dwError);
 
     *ppDacl = pDacl;
 
 cleanup:
-    return ntStatus;
-
-error:
     LW_SAFE_FREE_MEMORY(pAdminSid);
     LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
     LW_SAFE_FREE_MEMORY(pWorldSid);
 
-    *ppDacl = NULL;
-    goto cleanup;
-}
-
-
-NTSTATUS
-SamrSrvCreateDacl(
-    PACL *ppDacl,
-    PACCESS_LIST pList
-    )
-{
-    DWORD dwError = ERROR_SUCCESS;
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    DWORD dwDaclSize = 0;
-    PACL pDacl = NULL;
-    DWORD i = 0;
-    ULONG ulSidSize = 0;
-
-    dwDaclSize += ACL_HEADER_SIZE;
-
-    for (i = 0; pList[i].ppSid && (*pList[i].ppSid); i++)
-    {
-        ulSidSize = RtlLengthSid(*(pList[i].ppSid));
-
-        if (pList[i].ulAccessType == ACCESS_ALLOWED_ACE_TYPE)
-        {
-            dwDaclSize += ulSidSize + sizeof(ACCESS_ALLOWED_ACE);
-        }
-        else if (pList[i].ulAccessType == ACCESS_DENIED_ACE_TYPE)
-        {
-            dwDaclSize += ulSidSize + sizeof(ACCESS_DENIED_ACE);
-        }
-    }
-
-    dwError = LwAllocateMemory(dwDaclSize,
-                               OUT_PPVOID(&pDacl));
-    BAIL_ON_LSA_ERROR(dwError);
-
-    ntStatus = RtlCreateAcl(pDacl, dwDaclSize, ACL_REVISION);
-    BAIL_ON_NTSTATUS_ERROR(ntStatus);
-
-    for (i = 0; pList[i].ppSid && (*pList[i].ppSid); i++)
-    {
-        if (pList[i].ulAccessType == ACCESS_ALLOWED_ACE_TYPE)
-        {
-            ntStatus = RtlAddAccessAllowedAceEx(pDacl,
-                                                ACL_REVISION,
-                                                0,
-                                                pList[i].AccessMask,
-                                                *(pList[i].ppSid));
-        }
-        else if (pList[i].ulAccessType == ACCESS_DENIED_ACE_TYPE)
-        {
-            ntStatus = RtlAddAccessDeniedAceEx(pDacl,
-                                               ACL_REVISION,
-                                               0,
-                                               pList[i].AccessMask,
-                                               *(pList[i].ppSid));
-        }
-
-        BAIL_ON_NTSTATUS_ERROR(ntStatus);
-    }
-
-    *ppDacl = pDacl;
-
-cleanup:
-    if (ntStatus == STATUS_SUCCESS &&
-        dwError != ERROR_SUCCESS)
-    {
-        dwError = LwNtStatusToWin32Error(ntStatus);
-    }
-
     return dwError;
 
 error:
-    LW_SAFE_FREE_MEMORY(pDacl);
-    *ppDacl = NULL;
+    if (ppDacl)
+    {
+        *ppDacl = NULL;
+    }
 
     goto cleanup;
 }
@@ -506,19 +746,22 @@ SamrSrvFreeSecurityDescriptor(
                                             &bSaclDefaulted);
     BAIL_ON_NTSTATUS_ERROR(ntStatus);
 
+error:
     LW_SAFE_FREE_MEMORY(pOwnerSid);
     LW_SAFE_FREE_MEMORY(pPrimaryGroupSid);
-    LW_SAFE_FREE_MEMORY(pDacl);
-    LW_SAFE_FREE_MEMORY(pSacl);
+
+    if (bDaclPresent)
+    {
+        LW_SAFE_FREE_MEMORY(pDacl);
+    }
+
+    if (bSaclPresent)
+    {
+        LW_SAFE_FREE_MEMORY(pSacl);
+    }
 
     LW_SAFE_FREE_MEMORY(pSecDesc);
     *ppSecDesc = NULL;
-
-cleanup:
-    return;
-
-error:
-    goto cleanup;
 }
 
 
