@@ -76,6 +76,14 @@ SamDbInsertObjectToDatabase(
     DIRECTORY_MOD          modifications[]
     );
 
+static
+DWORD
+SamDbValidateIds(
+    PSAM_DIRECTORY_CONTEXT pDirectoryContext,
+    DWORD                  objectClass,
+    PDIRECTORY_MOD         pMods
+    );
+
 static
 DWORD
 SamDbBuildAddObjectQuery(
@@ -361,6 +369,15 @@ SamDbInsertObjectToDatabase(
                     &pObjectClassMapInfo);
     BAIL_ON_SAMDB_ERROR(dwError);
 
+    if (objectClass == SAMDB_OBJECT_CLASS_USER ||
+        objectClass == SAMDB_OBJECT_CLASS_LOCAL_GROUP)
+    {
+        dwError = SamDbValidateIds(pDirectoryContext,
+                                   objectClass,
+                                   modifications);
+        BAIL_ON_SAMDB_ERROR(dwError);
+    }
+
     dwError = SamDbBuildAddObjectQuery(
                     pDirectoryContext,
                     pObjectClassMapInfo,
@@ -431,6 +448,78 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+SamDbValidateIds(
+    PSAM_DIRECTORY_CONTEXT pDirectoryContext,
+    DWORD                  objectClass,
+    PDIRECTORY_MOD         pMods
+    )
+{
+    DWORD dwError = 0;
+    DWORD iMod = 0;
+    PDIRECTORY_MOD pMod = &(pMods[iMod]);
+    WCHAR wszAttrObjectSid[] = SAM_DB_DIR_ATTR_OBJECT_SID;
+    WCHAR wszAttrUid[] = SAM_DB_DIR_ATTR_UID;
+    WCHAR wszAttrGid[] = SAM_DB_DIR_ATTR_GID;
+    PSTR pszSid = NULL;
+    DWORD dwUID = 0;
+    DWORD dwGID = 0;
+
+    while (pMod->pwszAttrName)
+    {
+        if (!wc16scasecmp(pMod->pwszAttrName, wszAttrObjectSid))
+        {
+            if (pMod->pAttrValues[0].Type == DIRECTORY_ATTR_TYPE_UNICODE_STRING)
+            {
+                dwError = LwWc16sToMbs(
+                                 pMod->pAttrValues[0].data.pwszStringValue,
+                                 &pszSid);
+            }
+            else if (pMod->pAttrValues[0].Type == DIRECTORY_ATTR_TYPE_ANSI_STRING)
+            {
+                dwError = LwAllocateString(
+                                 pMod->pAttrValues[0].data.pszStringValue,
+                                 &pszSid);
+            }
+            BAIL_ON_LSA_ERROR(dwError);
+
+            dwError = SamDbCheckAvailableSID(pDirectoryContext,
+                                             pszSid);
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+        else if (objectClass == SAMDB_OBJECT_CLASS_USER &&
+                 !wc16scasecmp(pMod->pwszAttrName, wszAttrUid))
+        {
+            dwUID = pMod->pAttrValues[0].data.ulValue;
+
+            dwError = SamDbCheckAvailableUID(pDirectoryContext,
+                                             dwUID);
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+        else if (objectClass == SAMDB_OBJECT_CLASS_LOCAL_GROUP &&
+                 !wc16scasecmp(pMod->pwszAttrName, wszAttrGid))
+        {
+            dwGID = pMod->pAttrValues[0].data.ulValue;
+
+            dwError = SamDbCheckAvailableGID(pDirectoryContext,
+                                             dwGID);
+            BAIL_ON_LSA_ERROR(dwError);
+        }
+
+        pMod = &(pMods[++iMod]);
+    }
+
+cleanup:
+    LW_SAFE_FREE_MEMORY(pszSid);
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
 #define SAMDB_ADD_OBJECT_QUERY_ROWID     "rowid"
 #define SAMDB_ADD_OBJECT_QUERY_SEPARATOR ","
 #define SAMDB_ADD_OBJECT_QUERY_PREFIX    "INSERT INTO " SAM_DB_OBJECTS_TABLE "("
@@ -492,7 +581,7 @@ SamDbBuildAddObjectQuery(
             dwColValuesLen += sizeof(SAMDB_ADD_OBJECT_QUERY_SEPARATOR) - 1;
         }
 
-        sprintf(szBuf, "\?%d", ++iCol);
+        sprintf(szBuf, "\?%u", ++iCol);
 
         dwColValuesLen += strlen(szBuf);
     }
@@ -560,7 +649,7 @@ SamDbBuildAddObjectQuery(
             dwColValuesLen += sizeof(SAMDB_ADD_OBJECT_QUERY_SEPARATOR)  - 1;
         }
 
-        sprintf(szBuf, "\?%d", ++iCol);
+        sprintf(szBuf, "\?%u", ++iCol);
 
         pszCursor = &szBuf[0];
         while (pszCursor && *pszCursor)
@@ -1023,7 +1112,7 @@ SamDbAddGenerateParentDN(
 
     if (pwszParentDN)
     {
-        dwError = LsaWc16sToMbs(
+        dwError = LwWc16sToMbs(
                         pwszParentDN,
                         &pAttrValue->data.pszStringValue);
         BAIL_ON_SAMDB_ERROR(dwError);
@@ -1073,7 +1162,7 @@ SamDbAddGenerateDN(
 
     if (pwszDN)
     {
-        dwError = LsaWc16sToMbs(
+        dwError = LwWc16sToMbs(
                         pwszDN,
                         &pAttrValue->data.pszStringValue);
         BAIL_ON_SAMDB_ERROR(dwError);
@@ -1123,7 +1212,7 @@ SamDbAddGenerateDomain(
 
     if (pwszDomainName)
     {
-        dwError = LsaWc16sToMbs(
+        dwError = LwWc16sToMbs(
                         pwszDomainName,
                         &pAttrValue->data.pszStringValue);
         BAIL_ON_SAMDB_ERROR(dwError);
@@ -1401,7 +1490,7 @@ SamDbAddConvertUnicodeAttrValues(
 
                 if (pSrcValue->data.pwszStringValue)
                 {
-                    dwError = LsaWc16sToMbs(
+                    dwError = LwWc16sToMbs(
                                     pSrcValue->data.pwszStringValue,
                                     &pTgtValue->data.pszStringValue);
                     BAIL_ON_SAMDB_ERROR(dwError);
@@ -1439,6 +1528,8 @@ SamDbAddConvertUnicodeAttrValues(
             case DIRECTORY_ATTR_TYPE_OCTET_STREAM:
             case DIRECTORY_ATTR_TYPE_NT_SECURITY_DESCRIPTOR:
 
+                pTgtValue->Type = DIRECTORY_ATTR_TYPE_OCTET_STREAM;
+
                 dwError = DirectoryAllocateMemory(
                             sizeof(OCTET_STRING),
                             (PVOID*)&pTgtValue->data.pOctetString);
@@ -1668,9 +1759,9 @@ SamDbFindDomainSID(
     PSTR  pszDomainName = NULL;
     SAMDB_OBJECT_CLASS objectClass = SAMDB_OBJECT_CLASS_DOMAIN;
     PCSTR pszQueryClause = SAM_DB_COL_DOMAIN       " = \"%s\"" \
-                           "   AND " SAM_DB_COL_OBJECT_CLASS " = %d;";
+                           "   AND " SAM_DB_COL_OBJECT_CLASS " = %u;";
 
-    dwError = LsaWc16sToMbs(
+    dwError = LwWc16sToMbs(
                     pwszDomainName,
                     &pszDomainName);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -1682,7 +1773,7 @@ SamDbFindDomainSID(
                     objectClass);
     BAIL_ON_SAMDB_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszFilter,
                     &pwszFilter);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -1717,7 +1808,7 @@ SamDbFindDomainSID(
         BAIL_ON_SAMDB_ERROR(dwError);
     }
 
-    dwError = LsaWc16sToMbs(
+    dwError = LwWc16sToMbs(
                     pDirEntries[0].pAttributes[0].pValues[0].data.pwszStringValue,
                     &pszDomainSID);
     BAIL_ON_SAMDB_ERROR(dwError);
