@@ -125,6 +125,23 @@ SamDbSetupLocalGroupMemberships(
     HANDLE hDirectory
     );
 
+static
+DWORD
+SamDbFixAcls(
+    HANDLE hDirectory
+    );
+
+static
+DWORD
+SamDbFixLocalAccounts(
+    HANDLE hDirectory
+    );
+
+static
+VOID
+SamDbFreeAbsoluteSecurityDescriptor(
+    IN OUT PSECURITY_DESCRIPTOR_ABSOLUTE *ppSecDesc
+    );
 
 DWORD
 DirectoryInitializeProvider(
@@ -226,6 +243,7 @@ SamDbInit(
     )
 {
     DWORD dwError = 0;
+    HANDLE hDirectory1 = (HANDLE)NULL;
     HANDLE hDirectory = (HANDLE)NULL;
     PSAM_DIRECTORY_CONTEXT pDirectory = NULL;
     PCSTR  pszDbDirPath = SAM_DB_DIR;
@@ -240,7 +258,17 @@ SamDbInit(
     // TODO: Implement an upgrade scenario
     if (bExists)
     {
-       goto cleanup;
+        dwError = SamDbOpen(&hDirectory1);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        //Fix ACLs
+        dwError = SamDbFixAcls(hDirectory1);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        dwError = SamDbFixLocalAccounts(hDirectory1);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        goto cleanup;
     }
 
     dwError = LsaCheckDirectoryExists(
@@ -284,6 +312,10 @@ SamDbInit(
     BAIL_ON_SAMDB_ERROR(dwError);
 
 cleanup:
+    if (hDirectory1)
+    {
+        SamDbClose(hDirectory1);
+    }
 
     if (hDirectory)
     {
@@ -323,7 +355,7 @@ cleanup:
 
 error:
 
-    SAMDB_LOG_DEBUG("Sqlite3 Error (code: %d): %s",
+    SAMDB_LOG_DEBUG("Sqlite3 Error (code: %u): %s",
                 dwError,
                 LSA_SAFE_LOG_STRING(pszError));
 
@@ -343,17 +375,53 @@ SamDbAddDefaultEntries(
 {
     DWORD  dwError = 0;
     PSTR   pszHostname = NULL;
+    size_t sHostnameLen = 0;
+    PSTR   pszHostnameLower = NULL;
+    DWORD  dwHostnameHash = 0;
+    PSTR   pszHashStr = NULL;
+    size_t sHashStrLen = 0;
     PSTR   pszDomainDN = NULL;
-    CHAR   szNetBIOSName[16];
+    CHAR   szDomainName[16] = {0};
     PSID   pMachineSid = NULL;
 
     dwError = LsaDnsGetHostInfo(&pszHostname);
     BAIL_ON_SAMDB_ERROR(dwError);
 
     LwStrToUpper(pszHostname);
+    sHostnameLen = strlen(pszHostname);
 
-    memset(szNetBIOSName, 0, sizeof(szNetBIOSName));
-    strncpy(szNetBIOSName, pszHostname, 15);
+    if (sHostnameLen < 16)
+    {
+        strncpy(szDomainName, pszHostname, sHostnameLen);
+    }
+    else
+    {
+        /*
+         * The hostname is too long for NetBIOS max size
+         * so format the machine domain name based on the hostname
+         * and its hash (separated with '-').
+         */
+        dwError = LwAllocateString(pszHostname, &pszHostnameLower);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        LwStrToLower(pszHostnameLower);
+
+        dwError = LsaStrHash(pszHostnameLower, &dwHostnameHash);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        dwError = LsaHashToStr(dwHostnameHash, &pszHashStr);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        sHashStrLen = strlen(pszHashStr);
+
+        strncpy(szDomainName,
+                pszHostname,
+                sizeof(szDomainName) - sHashStrLen - 1);
+        szDomainName[sizeof(szDomainName) - sHashStrLen - 2] = '-';
+        strncpy(&szDomainName[sizeof(szDomainName) - sHashStrLen - 1],
+                pszHashStr,
+                sHashStrLen);
+    }
 
     dwError = SamDbInitConfig(hDirectory);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -361,14 +429,14 @@ SamDbAddDefaultEntries(
     dwError = LwAllocateStringPrintf(
                     &pszDomainDN,
                     "DC=%s",
-                    pszHostname);
+                    &szDomainName[0]);
     BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbAddMachineDomain(
                     hDirectory,
                     pszDomainDN,
-                    pszHostname,
-                    &szNetBIOSName[0],
+                    &szDomainName[0],
+                    &szDomainName[0],
                     &pMachineSid);
     BAIL_ON_SAMDB_ERROR(dwError);
 
@@ -385,7 +453,7 @@ SamDbAddDefaultEntries(
     dwError = SamDbAddLocalAccounts(
                     hDirectory,
                     pszDomainDN,
-                    szNetBIOSName,
+                    &szDomainName[0],
                     pMachineSid);
     BAIL_ON_SAMDB_ERROR(dwError);
 
@@ -394,9 +462,10 @@ SamDbAddDefaultEntries(
     BAIL_ON_SAMDB_ERROR(dwError);
 
 cleanup:
-
     DIRECTORY_FREE_STRING(pszHostname);
     DIRECTORY_FREE_STRING(pszDomainDN);
+    DIRECTORY_FREE_STRING(pszHashStr);
+    DIRECTORY_FREE_STRING(pszHostnameLower);
 
     return dwError;
 
@@ -562,7 +631,7 @@ SamDbAddLocalDomain(
 
     memset(mods, 0, sizeof(mods));
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszDomainDN,
                     &pwszObjectDN);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -574,7 +643,7 @@ SamDbAddLocalDomain(
     avObjectClass.data.ulValue = SAMDB_OBJECT_CLASS_DOMAIN;
     mods[iMod].pAttrValues = &avObjectClass;
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszMachineSID,
                     &pwszMachineSID);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -586,7 +655,7 @@ SamDbAddLocalDomain(
     avMachineSID.data.pwszStringValue = pwszMachineSID;
     mods[iMod].pAttrValues = &avMachineSID;
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszDomainName,
                     &pwszDomainName);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -612,7 +681,7 @@ SamDbAddLocalDomain(
     avSamAccountName.data.pwszStringValue = pwszDomainName;
     mods[iMod].pAttrValues = &avSamAccountName;
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszNetBIOSName,
                     &pwszNetBIOSName);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -709,6 +778,7 @@ cleanup:
     DIRECTORY_FREE_MEMORY(pwszDomainName);
     DIRECTORY_FREE_MEMORY(pwszNetBIOSName);
     RTL_FREE(&pMachineSID);
+    DIRECTORY_FREE_MEMORY(pSecDesc);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -770,22 +840,22 @@ SamDbAddContainer(
                     pszDomainDN);
     BAIL_ON_SAMDB_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszObjectDN,
                     &pwszObjectDN);
     BAIL_ON_SAMDB_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszName,
                     &pwszDomainName);
     BAIL_ON_SAMDB_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszNetBIOSName,
                     &pwszNetBIOSName);
     BAIL_ON_SAMDB_ERROR(dwError);
 
-    dwError = LsaMbsToWc16s(
+    dwError = LwMbsToWc16s(
                     pszSID,
                     &pwszSID);
     BAIL_ON_SAMDB_ERROR(dwError);
@@ -863,6 +933,7 @@ cleanup:
     DIRECTORY_FREE_MEMORY(pwszDomainName);
     DIRECTORY_FREE_MEMORY(pwszNetBIOSName);
     RTL_FREE(&pBuiltinSID);
+    DIRECTORY_FREE_MEMORY(pSecDesc)
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -1009,32 +1080,32 @@ SamDbAddBuiltinAccounts(
                         pszDomainDN);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszObjectDN,
                         &pwszObjectDN);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszName,
                         &pwszSamAccountName);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszSID,
                         &pwszSID);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszDomainName,
                         &pwszDomainName);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszNetBIOSDomain,
                         &pwszNetBIOSDomain);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszDescription,
                         &pwszDescription);
         BAIL_ON_SAMDB_ERROR(dwError);
@@ -1137,6 +1208,7 @@ SamDbAddBuiltinAccounts(
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszDomainName);
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszNetBIOSDomain);
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszDescription);
+        DIRECTORY_FREE_MEMORY_AND_RESET(pSecDesc);
 
         RTL_FREE(&pAccountSid);
     }
@@ -1150,6 +1222,7 @@ cleanup:
     DIRECTORY_FREE_MEMORY(pwszDomainName);
     DIRECTORY_FREE_MEMORY(pwszNetBIOSDomain);
     DIRECTORY_FREE_MEMORY(pwszDescription);
+    DIRECTORY_FREE_MEMORY(pSecDesc);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -1193,7 +1266,7 @@ SamDbAddLocalAccounts(
                                 "computer/domain",
             .pszShell         = SAM_DB_DEFAULT_ADMINISTRATOR_SHELL,
             .pszHomedir       = SAM_DB_DEFAULT_ADMINISTRATOR_HOMEDIR,
-            .flags            = SAMDB_ACB_NORMAL,
+            .flags            = SAMDB_ACB_NORMAL | SAMDB_ACB_DISABLED,
             .objectClass      = SAMDB_OBJECT_CLASS_USER
         },
         {
@@ -1319,7 +1392,7 @@ SamDbAddLocalAccounts(
                         pszDomainDN);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszObjectDN,
                         &pwszObjectDN);
         BAIL_ON_SAMDB_ERROR(dwError);
@@ -1332,35 +1405,35 @@ SamDbAddLocalAccounts(
             BAIL_ON_SAMDB_ERROR(dwError);
         }
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszName,
                         &pwszSamAccountName);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszDomain,
                         &pwszDomain);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszNetBIOSDomain,
                         &pwszNetBIOSDomain);
         BAIL_ON_SAMDB_ERROR(dwError);
 
-        dwError = LsaMbsToWc16s(
+        dwError = LwMbsToWc16s(
                         pszDescription,
                         &pwszDescription);
         BAIL_ON_SAMDB_ERROR(dwError);
 
         if (pszShell) {
-            dwError = LsaMbsToWc16s(
+            dwError = LwMbsToWc16s(
                             pszShell,
                             &pwszShell);
             BAIL_ON_SAMDB_ERROR(dwError);
         }
 
         if (pszHomedir) {
-            dwError = LsaMbsToWc16s(
+            dwError = LwMbsToWc16s(
                             pszHomedir,
                             &pwszHomedir);
             BAIL_ON_SAMDB_ERROR(dwError);
@@ -1510,13 +1583,9 @@ SamDbAddLocalAccounts(
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszHomedir);
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszDomain);
         DIRECTORY_FREE_MEMORY_AND_RESET(pwszNetBIOSDomain);
-
-        if (pwszSID) {
-            RTL_FREE(&pwszSID);
-            pwszSID = NULL;
-        }
-
+        DIRECTORY_FREE_MEMORY_AND_RESET(pSecDesc);
         DIRECTORY_FREE_MEMORY_AND_RESET(pAccountSid);
+        RTL_FREE(&pwszSID);
     }
 
 cleanup:
@@ -1529,12 +1598,9 @@ cleanup:
     DIRECTORY_FREE_MEMORY(pwszHomedir);
     DIRECTORY_FREE_MEMORY(pwszDomain);
     DIRECTORY_FREE_MEMORY(pwszNetBIOSDomain);
-
-    if (pwszSID) {
-        RTL_FREE(&pwszSID);
-    }
-
-    LW_SAFE_FREE_MEMORY(pAccountSid);
+    DIRECTORY_FREE_MEMORY(pSecDesc);
+    DIRECTORY_FREE_MEMORY(pAccountSid);
+    RTL_FREE(&pwszSID);
 
     return dwError;
 
@@ -1682,10 +1748,14 @@ SamDbSetupLocalGroupMemberships(
                                            OUT_PPVOID(&pwszMemberFilter));
                 BAIL_ON_SAMDB_ERROR(dwError);
 
-                sw16printfw(pwszMemberFilter, dwMemberFilterLen,
-                            wszMemberFilterFmt,
-                            wszAttrSamAccountName,
-                            pwszMemberName);
+                if (sw16printfw(pwszMemberFilter, dwMemberFilterLen,
+                                wszMemberFilterFmt,
+                                wszAttrSamAccountName,
+                                pwszMemberName) < 0)
+                {
+                    dwError = LwErrnoToWin32Error(errno);
+                    BAIL_ON_SAMDB_ERROR(dwError);
+                }
 
                 dwError = SamDbSearchObject(hDirectory,
                                             pwszBase,
@@ -1715,8 +1785,15 @@ SamDbSetupLocalGroupMemberships(
                                               .pValues[0].data.pwszStringValue);
                         DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
                                               .pValues[0].data.pwszStringValue);
+                        DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[0]
+                                              .pValues);
+                        DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
+                                              .pValues);
                         DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[0]
                                               .pwszName);
+                        DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
+                                              .pwszName);
+                        DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes);
                     }
 
                     DIRECTORY_FREE_MEMORY(pMemberEntries);
@@ -1734,8 +1811,11 @@ SamDbSetupLocalGroupMemberships(
             {
                 DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
                                       .pValues[0].data.pwszStringValue);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
+                                      .pValues);
                 DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
                                       .pwszName);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes);
             }
 
             DIRECTORY_FREE_MEMORY(pGroupEntries);
@@ -1751,10 +1831,13 @@ cleanup:
     {
         for (i = 0; i < dwGroupEntriesNum; i++)
         {
-            DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
-                                  .pValues[0].data.pwszStringValue);
-            DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
-                                  .pwszName);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
+                                      .pValues[0].data.pwszStringValue);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
+                                      .pValues);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes[0]
+                                      .pwszName);
+                DIRECTORY_FREE_MEMORY(pGroupEntries[i].pAttributes);
         }
 
         DIRECTORY_FREE_MEMORY(pGroupEntries);
@@ -1768,8 +1851,15 @@ cleanup:
                                   .pValues[0].data.pwszStringValue);
             DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
                                   .pValues[0].data.pwszStringValue);
+            DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[0]
+                                  .pValues);
+            DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
+                                  .pValues);
             DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[0]
                                   .pwszName);
+            DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes[1]
+                                  .pwszName);
+            DIRECTORY_FREE_MEMORY(pMemberEntries[i].pAttributes);
         }
 
         DIRECTORY_FREE_MEMORY(pMemberEntries);
@@ -1787,6 +1877,443 @@ error:
 }
 
 
+static
+DWORD
+SamDbFixAcls(
+    HANDLE hDirectory
+    )
+{
+
+    DWORD dwError = 0;
+    NTSTATUS status = 0;
+    const wchar_t wszAnyObjectFilterFmt[] = L"%ws>0";
+    const DWORD dwInt32StrSize = 10;
+    WCHAR wszAttrRecordId[] = DIRECTORY_ATTR_RECORD_ID;
+    WCHAR wszAttrObjectDN[] = DIRECTORY_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrSecurityDescriptor[] = DIRECTORY_ATTR_SECURITY_DESCRIPTOR;
+    DWORD dwAnyObjectFilterLen = 0;
+    PWSTR pwszAnyObjectFilter = NULL;
+    ULONG ulScope = 0;
+    DWORD iEntry = 0;
+    ULONG ulAttributesOnly = 0;
+    PWSTR pwszBase = NULL;
+    PWSTR wszAttributes[] = {
+        &wszAttrObjectDN[0],
+        &wszAttrSecurityDescriptor[0],
+        NULL
+    };
+    PDIRECTORY_ENTRY pObjectEntries = NULL;
+    DWORD dwNumObjectEntries = 0;
+    PDIRECTORY_ENTRY pObjectEntry = NULL;
+    PWSTR pwszAccountObjectDN = NULL;
+    POCTET_STRING pSecDescBlob = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pCurrentSecDesc = NULL;
+
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDescAbs = NULL;
+    ULONG ulSecDescAbsLen = 0;
+    PSECURITY_DESCRIPTOR_RELATIVE pSecDescRelNew = NULL;
+    ULONG ulSecDescRelNewLen = 1024;
+    PACL pDacl = NULL;
+    ULONG ulDaclLen = 0;
+    PACL pSacl = NULL;
+    ULONG ulSaclLen = 0;
+    PSID pOwner = NULL;
+    ULONG ulOwnerLen = 0;
+    PSID pGroup = NULL;
+    ULONG ulGroupLen = 0;
+    PSID pSamGroup = NULL;
+    BOOLEAN bIsGroupDefaulted = FALSE;
+    PSID pGroupSid = NULL;
+
+
+    enum AttrValueIndex {
+        ATTR_VAL_IDX_SEC_DESC = 0,
+        ATTR_VAL_IDX_SENTINEL
+    };
+
+    ATTRIBUTE_VALUE AttrValues[] = {
+        {   /* ATTR_VAL_IDX_SEC_DESC */
+            .Type = DIRECTORY_ATTR_TYPE_OCTET_STREAM,
+            .data.pOctetString = NULL
+        }
+    };
+
+    DIRECTORY_MOD ModSecDesc = {
+        DIR_MOD_FLAGS_REPLACE,
+        wszAttrSecurityDescriptor,
+        1,
+        &AttrValues[ATTR_VAL_IDX_SEC_DESC]
+    };
+
+    DIRECTORY_MOD Mods[ATTR_VAL_IDX_SENTINEL + 1];
+    memset(&Mods, 0, sizeof(Mods));
+
+    OCTET_STRING NewSecDescBlob = {0};
+    DWORD iMod = 0;
+
+
+    dwAnyObjectFilterLen = ((sizeof(wszAttrRecordId)/sizeof(WCHAR) - 1) +
+                            dwInt32StrSize +
+                            (sizeof(wszAnyObjectFilterFmt)/
+                             sizeof(wszAnyObjectFilterFmt[0])));
+    dwError = LwAllocateMemory(dwAnyObjectFilterLen * sizeof(WCHAR),
+                                (PVOID*)&pwszAnyObjectFilter);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
+    sw16printfw(pwszAnyObjectFilter, dwAnyObjectFilterLen,
+                wszAnyObjectFilterFmt,
+                &wszAttrRecordId[0]);
+
+    dwError = SamDbSearchObject(hDirectory,
+                               pwszBase,
+                               ulScope,
+                               pwszAnyObjectFilter,
+                               wszAttributes,
+                               ulAttributesOnly,
+                               &pObjectEntries,
+                               &dwNumObjectEntries);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
+    for (iEntry = 0; iEntry < dwNumObjectEntries; iEntry++)
+    {
+        pObjectEntry = &(pObjectEntries[iEntry]);
+
+        dwError = DirectoryGetEntryAttrValueByName(
+                                    pObjectEntry,
+                                    wszAttrObjectDN,
+                                    DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                                    &pwszAccountObjectDN);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        dwError = DirectoryGetEntryAttrValueByName(
+                                  pObjectEntry,
+                                  wszAttrSecurityDescriptor,
+                                  DIRECTORY_ATTR_TYPE_NT_SECURITY_DESCRIPTOR,
+                                  &pSecDescBlob);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        pCurrentSecDesc  = (PSECURITY_DESCRIPTOR_RELATIVE)pSecDescBlob->pBytes;
+
+        if (!pCurrentSecDesc)
+            continue;
+
+        // Get sizes
+        status = RtlSelfRelativeToAbsoluteSD(pCurrentSecDesc,
+                                             pSecDescAbs, &ulSecDescAbsLen,
+                                             pDacl, &ulDaclLen,
+                                             pSacl, &ulSaclLen,
+                                             pOwner, &ulOwnerLen,
+                                             pGroup, &ulGroupLen);
+        if (status == STATUS_BUFFER_TOO_SMALL)
+        {
+            status = STATUS_SUCCESS;
+        }
+        BAIL_ON_NT_STATUS(status);
+
+        status = LW_RTL_ALLOCATE(&pSecDescAbs, VOID, ulSecDescAbsLen);
+        BAIL_ON_NT_STATUS(status);
+
+        if (ulOwnerLen)
+        {
+            status = LW_RTL_ALLOCATE(&pOwner, SID, ulOwnerLen);
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (ulGroupLen)
+        {
+            status = LW_RTL_ALLOCATE(&pGroup, SID, ulGroupLen);
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (ulDaclLen)
+        {
+            status = LW_RTL_ALLOCATE(&pDacl, VOID, ulDaclLen);
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        if (ulSaclLen)
+        {
+            status = LW_RTL_ALLOCATE(&pSacl, VOID, ulSaclLen);
+            BAIL_ON_NT_STATUS(status);
+        }
+
+        status = RtlSelfRelativeToAbsoluteSD(pCurrentSecDesc,
+                                             pSecDescAbs, &ulSecDescAbsLen,
+                                             pDacl, &ulDaclLen,
+                                             pSacl, &ulSaclLen,
+                                             pOwner, &ulOwnerLen,
+                                             pGroup, &ulGroupLen);
+        BAIL_ON_NT_STATUS(status);
+
+        // Check whether group part is set, if NOT set, set it
+        status = RtlGetGroupSecurityDescriptor(pSecDescAbs,
+                                               &pSamGroup,
+                                               &bIsGroupDefaulted);
+        BAIL_ON_NT_STATUS(status);
+
+        if (!pSamGroup)
+        {
+
+            status = RtlAllocateSidFromCString(&pGroupSid, "S-1-5-32-544");
+            BAIL_ON_NT_STATUS(status);
+
+            status = RtlSetGroupSecurityDescriptor(
+                         pSecDescAbs,
+                         pGroupSid,
+                         FALSE);
+            BAIL_ON_NT_STATUS(status);
+
+            // convert absolute back to relative before write to SamDb
+            do
+            {
+                status = LwReallocMemory(pSecDescRelNew,
+                                            (PVOID*)&pSecDescRelNew,
+                                            ulSecDescRelNewLen);
+                BAIL_ON_NT_STATUS(status);
+
+                memset(pSecDescRelNew, 0, ulSecDescRelNewLen);
+
+                status = RtlAbsoluteToSelfRelativeSD(pSecDescAbs,
+                                                     pSecDescRelNew,
+                                                     &ulSecDescRelNewLen);
+                if (STATUS_BUFFER_TOO_SMALL  == status)
+                {
+                    ulSecDescRelNewLen *= 2;
+                }
+                else
+                {
+                    BAIL_ON_NT_STATUS(status);
+                }
+            }
+            while((status != STATUS_SUCCESS) &&
+                  (ulSecDescRelNewLen <= SECURITY_DESCRIPTOR_RELATIVE_MAX_SIZE));
+
+            // Write the fixed ACL back to registry
+            NewSecDescBlob.pBytes     = (PBYTE)pSecDescRelNew;
+            NewSecDescBlob.ulNumBytes = ulSecDescRelNewLen;
+
+            AttrValues[ATTR_VAL_IDX_SEC_DESC].data.pOctetString = &NewSecDescBlob;
+            Mods[iMod++] = ModSecDesc;
+
+            dwError = SamDbModifyObject(hDirectory,
+                                        pwszAccountObjectDN,
+                                        Mods);
+            BAIL_ON_SAMDB_ERROR(dwError);
+        }
+        BAIL_ON_NT_STATUS(status);
+
+        iMod = 0;
+        memset(&Mods, 0, sizeof(Mods));
+        memset(&NewSecDescBlob, 0, sizeof(NewSecDescBlob));
+        LW_SAFE_FREE_MEMORY(pSecDescRelNew);
+        ulSecDescRelNewLen = 1024;
+        SamDbFreeAbsoluteSecurityDescriptor(&pSecDescAbs);
+        ulSecDescAbsLen = 0;
+        ulDaclLen = 0;
+        ulSaclLen = 0;
+        ulOwnerLen = 0;
+        ulGroupLen = 0;
+    }
+
+cleanup:
+
+    if (pObjectEntries)
+    {
+        DirectoryFreeEntries(pObjectEntries, dwNumObjectEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszAnyObjectFilter);
+    LW_SAFE_FREE_MEMORY(pSecDescRelNew);
+    SamDbFreeAbsoluteSecurityDescriptor(&pSecDescAbs);
+
+    if (status != STATUS_SUCCESS &&
+        dwError == ERROR_SUCCESS)
+    {
+        dwError = LwNtStatusToWin32Error(status);
+    }
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
+
+static
+DWORD
+SamDbFixLocalAccounts(
+    HANDLE hDirectory
+    )
+{
+
+    DWORD dwError = 0;
+    const wchar_t wszUserObjectFilterFmt[] = L"%ws = %u";
+    const DWORD dwInt32StrSize = 10;
+    WCHAR wszAttrObjectClass[] = SAM_DB_DIR_ATTR_OBJECT_CLASS;
+    WCHAR wszAttrObjectDN[] = SAM_DB_DIR_ATTR_DISTINGUISHED_NAME;
+    WCHAR wszAttrAccountFlags[] = SAM_DB_DIR_ATTR_ACCOUNT_FLAGS;
+    WCHAR wszAttrNtHash[] = SAM_DB_DIR_ATTR_NT_HASH;
+    DWORD dwUserObjectFilterLen = 0;
+    PWSTR pwszUserObjectFilter = NULL;
+    ULONG ulScope = 0;
+    ULONG ulAttributesOnly = 0;
+    PWSTR pwszBase = NULL;
+    PWSTR wszAttributes[] = {
+        &wszAttrObjectDN[0],
+        &wszAttrAccountFlags[0],
+        &wszAttrNtHash[0],
+        NULL
+    };
+
+    PDIRECTORY_ENTRY pUserEntries = NULL;
+    DWORD dwNumUserEntries = 0;
+    PDIRECTORY_ENTRY pUserEntry = NULL;
+    DWORD iEntry = 0;
+    PWSTR pwszUserObjectDN = NULL;
+    DWORD dwAccountFlags = 0;
+    POCTET_STRING pNtHash = NULL;
+    DWORD iMod = 0;
+
+    enum AttrValueIndex {
+        ATTR_VAL_IDX_ACCOUNT_FLAGS = 0,
+        ATTR_VAL_IDX_SENTINEL
+    };
+
+    ATTRIBUTE_VALUE AttrValues[] = {
+        {   /* ATTR_VAL_IDX_ACCOUNT_FLAGS */
+            .Type = DIRECTORY_ATTR_TYPE_LARGE_INTEGER,
+            .data.ulValue = 0
+        }
+    };
+
+    DIRECTORY_MOD ModAccountFlags = {
+        DIR_MOD_FLAGS_REPLACE,
+        wszAttrAccountFlags,
+        1,
+        &AttrValues[ATTR_VAL_IDX_ACCOUNT_FLAGS]
+    };
+
+    DIRECTORY_MOD Mods[ATTR_VAL_IDX_SENTINEL + 1];
+    memset(&Mods, 0, sizeof(Mods));
+
+    dwUserObjectFilterLen = (sizeof(wszAttrObjectClass)/sizeof(wszAttrObjectClass[0]) +
+                             dwInt32StrSize +
+                             sizeof(wszUserObjectFilterFmt));
+    dwError = LwAllocateMemory(dwUserObjectFilterLen * sizeof(WCHAR),
+                               OUT_PPVOID(&pwszUserObjectFilter));
+    BAIL_ON_SAMDB_ERROR(dwError);
+
+    if (sw16printfw(pwszUserObjectFilter, dwUserObjectFilterLen,
+                    wszUserObjectFilterFmt,
+                    &wszAttrObjectClass[0], SAMDB_OBJECT_CLASS_USER) < 0)
+    {
+        dwError = LwErrnoToWin32Error(errno);
+        BAIL_ON_SAMDB_ERROR(dwError);
+    }
+
+    dwError = SamDbSearchObject(hDirectory,
+                                pwszBase,
+                                ulScope,
+                                pwszUserObjectFilter,
+                                wszAttributes,
+                                ulAttributesOnly,
+                                &pUserEntries,
+                                &dwNumUserEntries);
+    BAIL_ON_SAMDB_ERROR(dwError);
+
+    for (iEntry = 0; iEntry < dwNumUserEntries; iEntry++)
+    {
+        pUserEntry = &(pUserEntries[iEntry]);
+
+        dwError = DirectoryGetEntryAttrValueByName(
+                                    pUserEntry,
+                                    wszAttrObjectDN,
+                                    DIRECTORY_ATTR_TYPE_UNICODE_STRING,
+                                    &pwszUserObjectDN);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        dwError = DirectoryGetEntryAttrValueByName(
+                                    pUserEntry,
+                                    wszAttrAccountFlags,
+                                    DIRECTORY_ATTR_TYPE_INTEGER,
+                                    &dwAccountFlags);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        dwError = DirectoryGetEntryAttrValueByName(
+                                    pUserEntry,
+                                    wszAttrNtHash,
+                                    DIRECTORY_ATTR_TYPE_OCTET_STREAM,
+                                    &pNtHash);
+        BAIL_ON_SAMDB_ERROR(dwError);
+
+        if ((pNtHash == NULL || pNtHash->ulNumBytes == 0) &&
+            !(dwAccountFlags & SAMDB_ACB_DISABLED))
+        {
+            dwAccountFlags |= SAMDB_ACB_DISABLED;
+
+            AttrValues[ATTR_VAL_IDX_ACCOUNT_FLAGS].data.ulValue = dwAccountFlags;
+
+            Mods[iMod++] = ModAccountFlags;
+
+            dwError = SamDbModifyObject(hDirectory,
+                                        pwszUserObjectDN,
+                                        Mods);
+            BAIL_ON_SAMDB_ERROR(dwError);
+        }
+    }
+
+cleanup:
+    if (pUserEntries)
+    {
+        DirectoryFreeEntries(pUserEntries, dwNumUserEntries);
+    }
+
+    LW_SAFE_FREE_MEMORY(pwszUserObjectFilter);
+
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
+
+static
+VOID
+SamDbFreeAbsoluteSecurityDescriptor(
+    IN OUT PSECURITY_DESCRIPTOR_ABSOLUTE *ppSecDesc
+    )
+{
+    PSID pOwner = NULL;
+    PSID pGroup = NULL;
+    PACL pDacl = NULL;
+    PACL pSacl = NULL;
+    BOOLEAN bDefaulted = FALSE;
+    BOOLEAN bPresent = FALSE;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
+
+    if ((ppSecDesc == NULL) || (*ppSecDesc == NULL)) {
+        return;
+    }
+
+    pSecDesc = *ppSecDesc;
+
+    RtlGetOwnerSecurityDescriptor(pSecDesc, &pOwner, &bDefaulted);
+    RtlGetGroupSecurityDescriptor(pSecDesc, &pGroup, &bDefaulted);
+    RtlGetDaclSecurityDescriptor(pSecDesc, &bPresent, &pDacl, &bDefaulted);
+    RtlGetSaclSecurityDescriptor(pSecDesc, &bPresent, &pSacl, &bDefaulted);
+
+    LW_RTL_FREE(&pSecDesc);
+    LW_RTL_FREE(&pOwner);
+    LW_RTL_FREE(&pGroup);
+    LW_RTL_FREE(&pDacl);
+    LW_RTL_FREE(&pSacl);
+
+    *ppSecDesc = NULL;
+
+    return;
+}
+
 /*
 local variables:
 mode: c
