@@ -127,6 +127,7 @@ SamDbCreateLocalDomainSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -184,10 +185,13 @@ SamDbCreateLocalDomainSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -235,6 +239,7 @@ SamDbCreateBuiltinDomainSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -292,10 +297,13 @@ SamDbCreateBuiltinDomainSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -343,6 +351,7 @@ SamDbCreateLocalUserSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -398,10 +407,13 @@ SamDbCreateLocalUserSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -449,6 +461,7 @@ SamDbCreateLocalGroupSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -504,10 +517,13 @@ SamDbCreateLocalGroupSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -555,6 +571,7 @@ SamDbCreateBuiltinGroupSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -610,10 +627,13 @@ SamDbCreateBuiltinGroupSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -661,6 +681,7 @@ SamDbCreateNewLocalAccountSecDesc(
     PSID pGroupSid = NULL;
     PSECURITY_DESCRIPTOR_ABSOLUTE pSecDesc = NULL;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRel = NULL;
+    PSECURITY_DESCRIPTOR_RELATIVE pNewSecDescRel = NULL;
     ULONG ulSecDescLen = 1024;
     PACL pDacl = NULL;
 
@@ -716,10 +737,13 @@ SamDbCreateNewLocalAccountSecDesc(
     do
     {
         dwError = LwReallocMemory(pSecDescRel,
-                                  OUT_PPVOID(&pSecDescRel),
+                                  OUT_PPVOID(&pNewSecDescRel),
                                   ulSecDescLen);
         BAIL_ON_SAMDB_ERROR(dwError);
 
+        pSecDescRel    = pNewSecDescRel;
+        pNewSecDescRel = NULL;
+
         ntStatus = RtlAbsoluteToSelfRelativeSD(
                                     pSecDesc,
                                     pSecDescRel,
@@ -739,6 +763,9 @@ cleanup:
     LW_SAFE_FREE_MEMORY(pSecDesc);
     LW_SAFE_FREE_MEMORY(pDacl);
 
+    LW_SAFE_FREE_MEMORY(pOwnerSid);
+    LW_SAFE_FREE_MEMORY(pGroupSid);
+
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
     {
@@ -829,13 +856,17 @@ SamDbCreateLocalDomainDacl(
                                pLocalAdminSid,
                                DOMAIN_USER_RID_ADMIN);
 
-    ntStatus = RtlAllocateSidFromCString(&pBuiltinAdminsSid,
-                                         "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
-    ntStatus = RtlAllocateSidFromCString(&pWorldSid,
-                                         "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbCreateDacl(&pDacl, AccessList);
     BAIL_ON_LSA_ERROR(dwError);
@@ -843,9 +874,9 @@ SamDbCreateLocalDomainDacl(
     *ppDacl = pDacl;
 
 cleanup:
-
-    RTL_FREE(&pBuiltinAdminsSid);
-    RTL_FREE(&pWorldSid);
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
+    LW_SAFE_FREE_MEMORY(pLocalAdminSid);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -914,13 +945,17 @@ SamDbCreateBuiltinDomainDacl(
         }
     };
 
-    ntStatus = RtlAllocateSidFromCString(&pBuiltinAdminsSid,
-                                         "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
-    ntStatus = RtlAllocateSidFromCString(&pWorldSid,
-                                         "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbCreateDacl(&pDacl, AccessList);
     BAIL_ON_LSA_ERROR(dwError);
@@ -928,8 +963,8 @@ SamDbCreateBuiltinDomainDacl(
     *ppDacl = pDacl;
 
 cleanup:
-    RTL_FREE(&pBuiltinAdminsSid);
-    RTL_FREE(&pWorldSid);
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -1030,6 +1065,9 @@ SamDbCreateLocalUserDacl(
     *ppDacl = pDacl;
 
 cleanup:
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
+
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
     {
@@ -1060,7 +1098,7 @@ SamDbCreateLocalGroupDacl(
                                   ALIAS_ACCESS_LOOKUP_INFO |
                                   ALIAS_ACCESS_SET_INFO;
 
-    ACCESS_MASK AllAccessMask = STANDARD_RIGHTS_REQUIRED |
+    ACCESS_MASK AllAccessMask = STANDARD_RIGHTS_READ |
                                 ALIAS_ACCESS_GET_MEMBERS |
                                 ALIAS_ACCESS_LOOKUP_INFO;
 
@@ -1110,13 +1148,17 @@ SamDbCreateLocalGroupDacl(
                                pLocalAdminSid,
                                DOMAIN_USER_RID_ADMIN);
 
-    ntStatus = RtlAllocateSidFromCString(&pBuiltinAdminsSid,
-                                         "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
-    ntStatus = RtlAllocateSidFromCString(&pWorldSid,
-                                         "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbCreateDacl(&pDacl, AccessList);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1124,8 +1166,9 @@ SamDbCreateLocalGroupDacl(
     *ppDacl = pDacl;
 
 cleanup:
-    RTL_FREE(&pBuiltinAdminsSid);
-    RTL_FREE(&pWorldSid);
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
+    LW_SAFE_FREE_MEMORY(pLocalAdminSid);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
@@ -1157,7 +1200,7 @@ SamDbCreateBuiltinGroupDacl(
                                   ALIAS_ACCESS_LOOKUP_INFO |
                                   ALIAS_ACCESS_SET_INFO;
 
-    ACCESS_MASK AllAccessMask = STANDARD_RIGHTS_REQUIRED |
+    ACCESS_MASK AllAccessMask = STANDARD_RIGHTS_READ |
                                 ALIAS_ACCESS_GET_MEMBERS |
                                 ALIAS_ACCESS_LOOKUP_INFO;
 
@@ -1185,13 +1228,17 @@ SamDbCreateBuiltinGroupDacl(
         }
     };
 
-    ntStatus = RtlAllocateSidFromCString(&pBuiltinAdminsSid,
-                                         "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
-    ntStatus = RtlAllocateSidFromCString(&pWorldSid,
-                                         "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbCreateDacl(&pDacl, AccessList);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1199,6 +1246,9 @@ SamDbCreateBuiltinGroupDacl(
     *ppDacl = pDacl;
 
 cleanup:
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
+
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
     {
@@ -1274,13 +1324,17 @@ SamDbCreateNewLocalAccountDacl(
         }
     };
 
-    ntStatus = RtlAllocateSidFromCString(&pBuiltinAdminsSid,
-                                         "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinBuiltinAdministratorsSid,
+                                   NULL,
+                                   &pBuiltinAdminsSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
-    ntStatus = RtlAllocateSidFromCString(&pWorldSid,
-                                         "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
+    dwError = LwCreateWellKnownSid(WinWorldSid,
+                                   NULL,
+                                   &pWorldSid,
+                                   NULL);
+    BAIL_ON_SAMDB_ERROR(dwError);
 
     dwError = SamDbCreateDacl(&pDacl, AccessList);
     BAIL_ON_LSA_ERROR(dwError);
@@ -1288,9 +1342,8 @@ SamDbCreateNewLocalAccountDacl(
     *ppDacl = pDacl;
 
 cleanup:
-
-    RTL_FREE(&pBuiltinAdminsSid);
-    RTL_FREE(&pWorldSid);
+    LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
+    LW_SAFE_FREE_MEMORY(pWorldSid);
 
     if (dwError == ERROR_SUCCESS &&
         ntStatus != STATUS_SUCCESS)
