@@ -47,6 +47,9 @@
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
 #include "lsaclient.h"
 
@@ -75,7 +78,7 @@ main(
        LsaCloseServer(hLsaConnection);
     }
 
-    LSA_LOG_ERROR("Failed communication with the LSA Agent. Error code [%d]\n", dwError);
+    LSA_LOG_ERROR("Failed communication with the LSA Agent. Error code [%u]\n", dwError);
 
     return (dwError);
 }
