@@ -51,6 +51,9 @@
 #include "lsasystem.h"
 #include "lsadef.h"
 #include "lsa/lsa.h"
+#include "lwmem.h"
+#include "lwstr.h"
+#include "lwsecurityidentifier.h"
 #include "lsautils.h"
 
 static
@@ -123,7 +126,7 @@ main(
 
     if (getgrgid_r(gid, &grp, szBuf, sizeof(szBuf), &pGroup) < 0) {
         fprintf(stderr, "Failed to lookup group [gid:%u]", (unsigned int)gid);
-        dwError = errno;
+        dwError = LwMapErrnoToLwError(errno);
         BAIL_ON_LSA_ERROR(dwError);
     }
 
@@ -133,9 +136,9 @@ main(
     
     if(pGroup)
     {
-        printf("Name:     %s\n", IsNullOrEmptyString(pGroup->gr_name) ? "<null>" : pGroup->gr_name);
+        printf("Name:     %s\n", LW_IS_NULL_OR_EMPTY_STR(pGroup->gr_name) ? "<null>" : pGroup->gr_name);
         printf("Gid:      %u\n", (unsigned int)pGroup->gr_gid);
-        printf("Passwd:   %s\n", IsNullOrEmptyString(pGroup->gr_passwd) ? "<null>" : pGroup->gr_passwd);
+        printf("Passwd:   %s\n", LW_IS_NULL_OR_EMPTY_STR(pGroup->gr_passwd) ? "<null>" : pGroup->gr_passwd);
         printf("Members: ");
         ppszMembers = pGroup->gr_mem;
         while (ppszMembers && *ppszMembers)
