@@ -94,23 +94,23 @@ main(
                     pszConfigFilePath);
     BAIL_ON_LSA_ERROR(dwError);
     
-    if (IsNullOrEmptyString(gpszProviderLibPath))
+    if (LW_IS_NULL_OR_EMPTY_STR(gpszProviderLibPath))
     {
         fprintf(stderr, "Error: The stress config file does not have a valid provider library path\n");
-        dwError = EINVAL;
+        dwError = LW_ERROR_INVALID_PARAMETER;
         BAIL_ON_LSA_ERROR(dwError);
     }
     
-    if (IsNullOrEmptyString(gpszProviderConfigFilePath))
+    if (LW_IS_NULL_OR_EMPTY_STR(gpszProviderConfigFilePath))
     {
         fprintf(stderr, "Error: The stress config file does not have a valid provider config file path\n");
-        dwError = EINVAL;
+        dwError = LW_ERROR_INVALID_PARAMETER;
         BAIL_ON_LSA_ERROR(dwError);
     }
     
     gpAuthProvider->pszProviderLibpath = gpszProviderLibPath;
     
-    dwError = LWNetExtendEnvironmentForKrb5Affinity(FALSE);
+    dwError = LWNetExtendEnvironmentForKrb5Affinity(TRUE);
     BAIL_ON_LSA_ERROR(dwError);
     
     dwError = LADSBlockSelectedSignals();
@@ -138,13 +138,13 @@ cleanup:
 
     LADSShutdownGlobals();
 
-    LSA_SAFE_FREE_STRING(pszConfigFilePath);
+    LW_SAFE_FREE_STRING(pszConfigFilePath);
 
     return (dwError);
 
 error:
 
-    LSA_LOG_ERROR("Failed in stress-testing ad-provider [error code: %d]", dwError);
+    LSA_LOG_ERROR("Failed in stress-testing ad-provider [error code: %u]", dwError);
 
     goto cleanup;
 }
@@ -205,27 +205,27 @@ LADSShutdownGlobals(
                     DWORD i = 0;
                     for (i = 0; i < gLADSStressData[iType].dwNumItems; i++)
                     {
-                        LSA_SAFE_FREE_STRING(gLADSStressData[iType].data.ppszNames[i]);
+                        LW_SAFE_FREE_STRING(gLADSStressData[iType].data.ppszNames[i]);
                     }
-                    LsaFreeMemory(gLADSStressData[iType].data.ppszNames);
+                    LwFreeMemory(gLADSStressData[iType].data.ppszNames);
                 }
                 break;
                 
             case LADS_FIND_USER_BY_ID:
                 
-                LSA_SAFE_FREE_MEMORY(gLADSStressData[iType].data.pUidArray);
+                LW_SAFE_FREE_MEMORY(gLADSStressData[iType].data.pUidArray);
 
             case LADS_FIND_GROUP_BY_ID:
                 
-                LSA_SAFE_FREE_MEMORY(gLADSStressData[iType].data.pGidArray);
+                LW_SAFE_FREE_MEMORY(gLADSStressData[iType].data.pGidArray);
 
             default:
                 break;
         }
     }
     
-    LSA_SAFE_FREE_STRING(gpszProviderLibPath);
-    LSA_SAFE_FREE_STRING(gpszProviderConfigFilePath);
+    LW_SAFE_FREE_STRING(gpszProviderLibPath);
+    LW_SAFE_FREE_STRING(gpszProviderConfigFilePath);
     
     return dwError;
 }
@@ -286,7 +286,7 @@ LADSParseArgs(
            }
            case ConfigFile:
                
-               dwError = LsaAllocateString(
+               dwError = LwAllocateString(
                                pszArg,
                                &pszConfigFilePath);
                BAIL_ON_LSA_ERROR(dwError);
@@ -331,7 +331,7 @@ LADSParseArgs(
         }
     } while ((parse_mode != Done) && (iArg < argc));
     
-    if (IsNullOrEmptyString(pszConfigFilePath))
+    if (LW_IS_NULL_OR_EMPTY_STR(pszConfigFilePath))
     {
         ShowUsage();
         exit(1);
@@ -348,7 +348,7 @@ error:
 
     *ppszConfigFilePath = NULL;
 
-    LSA_SAFE_FREE_STRING(pszConfigFilePath);
+    LW_SAFE_FREE_STRING(pszConfigFilePath);
     
     goto cleanup;
 }
@@ -409,7 +409,7 @@ LADSHandleSignals(
     action.sa_handler = LADSInterruptHandler;
 
     sysRet = sigaction(SIGINT, &action, NULL);
-    dwError = (sysRet != 0) ? errno : 0;
+    dwError = (sysRet != 0) ? LwMapErrnoToLwError(errno) : 0;
     BAIL_ON_LSA_ERROR(dwError);
 
     // Unblock SIGINT
@@ -470,74 +470,61 @@ error:
 DWORD
 LADSInitAuthProvider(
     PCSTR pszConfigFilePath,
-    PLSA_AUTH_PROVIDER pProvider
+    PTEST_AUTH_PROVIDER pProvider
     )
 {
     DWORD dwError = 0;
     PFNINITIALIZEPROVIDER pfnInitProvider = NULL;
-    PCSTR  pszError = NULL;
+    PCSTR pszError = NULL;
     PSTR pszProviderLibpath = NULL;
-    
-    if (IsNullOrEmptyString(pProvider->pszProviderLibpath)) {
-        dwError = ENOENT;
+
+    if (LW_IS_NULL_OR_EMPTY_STR(pProvider->pszProviderLibpath))
+    {
+        dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
         BAIL_ON_LSA_ERROR(dwError);
     }
     
     pszProviderLibpath = pProvider->pszProviderLibpath;
     
     dlerror();
-        
     pProvider->pLibHandle = dlopen(pszProviderLibpath, RTLD_NOW | RTLD_GLOBAL);
-    if (pProvider->pLibHandle == NULL) {
-       LSA_LOG_ERROR("Failed to open auth provider at path [%s]", pszProviderLibpath);
+    if (!pProvider->pLibHandle)
+    {
+        LSA_LOG_ERROR("Failed to open auth provider at path '%s'", pszProviderLibpath);
 
-       pszError = dlerror();
-       if (!IsNullOrEmptyString(pszError)) {
-          LSA_LOG_ERROR("%s", pszError);
-       }
-            
-       dwError = LSA_ERROR_INVALID_AUTH_PROVIDER;
-       BAIL_ON_LSA_ERROR(dwError);
+        pszError = dlerror();
+        if (!LW_IS_NULL_OR_EMPTY_STR(pszError))
+        {
+            LSA_LOG_ERROR("%s", pszError);
+        }
+
+        dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
+        BAIL_ON_LSA_ERROR(dwError);
     }
-        
+
     dlerror();
     pfnInitProvider = (PFNINITIALIZEPROVIDER)dlsym(
-                                        pProvider->pLibHandle,
-                                        LSA_SYMBOL_NAME_INITIALIZE_PROVIDER);
-    if (pfnInitProvider == NULL) {
-       LSA_LOG_ERROR("Ignoring invalid auth provider at path [%s]", pszProviderLibpath);
-            
-       pszError = dlerror();
-       if (!IsNullOrEmptyString(pszError)) {
-          LSA_LOG_ERROR("%s", pszError);
-       }
-            
-       dwError = LSA_ERROR_INVALID_AUTH_PROVIDER;
-       BAIL_ON_LSA_ERROR(dwError);
-    }
-        
-    dlerror();
-    pProvider->pFnShutdown = (PFNSHUTDOWNPROVIDER)dlsym(
-                                        pProvider->pLibHandle,
-                                        LSA_SYMBOL_NAME_SHUTDOWN_PROVIDER);
-    if (pProvider->pFnShutdown == NULL) {
-       LSA_LOG_ERROR("Ignoring invalid auth provider at path [%s]", pszProviderLibpath);
-            
-       pszError = dlerror();
-       if (!IsNullOrEmptyString(pszError)) {
-          LSA_LOG_ERROR("%s", pszError);
-       }
-            
-       dwError = LSA_ERROR_INVALID_AUTH_PROVIDER;
-       BAIL_ON_LSA_ERROR(dwError);
+        pProvider->pLibHandle,
+        LSA_SYMBOL_NAME_INITIALIZE_PROVIDER);
+    if (!pfnInitProvider)
+    {
+        LSA_LOG_ERROR("Ignoring invalid auth provider at path '%s'", pszProviderLibpath);
+
+        pszError = dlerror();
+        if (!LW_IS_NULL_OR_EMPTY_STR(pszError))
+        {
+            LSA_LOG_ERROR("%s", pszError);
+        }
+
+        dwError = LW_ERROR_INVALID_AUTH_PROVIDER;
+        BAIL_ON_LSA_ERROR(dwError);
     }
         
     dwError = pfnInitProvider(
-                    pszConfigFilePath,
                     &pProvider->pszName,
                     &pProvider->pFnTable);
     BAIL_ON_LSA_ERROR(dwError);
-        
+
     dwError = LADSValidateProvider(pProvider);
     BAIL_ON_LSA_ERROR(dwError);
     
@@ -552,15 +539,16 @@ error:
 
 DWORD
 LADSValidateProvider(
-    PLSA_AUTH_PROVIDER pProvider
+    PTEST_AUTH_PROVIDER pProvider
     )
 {
     if (!pProvider ||
         !pProvider->pFnTable ||
+        !pProvider->pFnTable->pfnShutdownProvider ||
         !pProvider->pFnTable->pfnOpenHandle ||
         !pProvider->pFnTable->pfnCloseHandle ||
         !pProvider->pFnTable->pfnServicesDomain ||
-        !pProvider->pFnTable->pfnAuthenticateUser ||
+        !pProvider->pFnTable->pfnAuthenticateUserPam ||
         !pProvider->pFnTable->pfnValidateUser ||
         !pProvider->pFnTable->pfnLookupUserByName ||
         !pProvider->pFnTable->pfnLookupUserById ||
@@ -589,7 +577,7 @@ LADSValidateProvider(
         !pProvider->pFnTable->pfnFreeStatus
         )
     {
-        return LSA_ERROR_INVALID_AUTH_PROVIDER;
+        return LW_ERROR_INVALID_AUTH_PROVIDER;
     }
     
     return 0;
@@ -615,7 +603,7 @@ LADSStartWorkers(
            continue;
         }
 
-        dwError = LsaAllocateMemory(
+        dwError = LwAllocateMemory(
                       sizeof(pthread_t) * gLADSStressData[iType].dwNumThreads,
                       (PVOID*)&gLADSStressData[iType].pThreadArray);
         BAIL_ON_LSA_ERROR(dwError);
@@ -708,7 +696,7 @@ LADSStopWorkers(
             }
         }
 
-        LsaFreeMemory(gLADSStressData[iType].pThreadArray);
+        LwFreeMemory(gLADSStressData[iType].pThreadArray);
     }
 
     return dwError;
@@ -716,22 +704,22 @@ LADSStopWorkers(
 
 DWORD
 LADSShutdownAuthProvider(
-    PLSA_AUTH_PROVIDER pProvider
+    PTEST_AUTH_PROVIDER pProvider
     )
 {
-    if (pProvider->pFnShutdown) {
-    
-        pProvider->pFnShutdown(
-               pProvider->pszName,
-               pProvider->pFnTable
-               );
-   
+    if (pProvider)
+    {
+        if (pProvider->pFnTable && pProvider->pFnTable->pfnShutdownProvider)
+        {
+            pProvider->pFnTable->pfnShutdownProvider();
+        }
+
+        if (pProvider->pLibHandle)
+        {
+            dlclose(pProvider->pLibHandle);
+        }
     }
 
-    if (pProvider->pLibHandle) {
-        dlclose(pProvider->pLibHandle);
-    }
-    
     return 0;
 }
 
