@@ -85,7 +85,7 @@ static const struct krb5err_lwerr krb5err_lwerr_map[] = {
 
     LWERR(KRB5KDC_ERR_SVC_UNAVAILABLE, LW_ERROR_KRB5KDC_ERR_SVC_UNAVAILABLE),
 
-    LWERR(KRB5KRB_AP_ERR_BAD_INTEGRITY, LW_ERROR_KRB5KRB_AP_ERR_BAD_INTEGRITY),
+    LWERR(KRB5KRB_AP_ERR_BAD_INTEGRITY, LW_ERROR_PASSWORD_MISMATCH),
     LWERR(KRB5KRB_AP_ERR_TKT_EXPIRED, LW_ERROR_KRB5KRB_AP_ERR_TKT_EXPIRED),
     LWERR(KRB5KRB_AP_ERR_TKT_NYV, LW_ERROR_KRB5KRB_AP_ERR_TKT_NYV),
     LWERR(KRB5KRB_AP_ERR_REPEAT, LW_ERROR_KRB5KRB_AP_ERR_REPEAT),
