@@ -386,6 +386,10 @@
 #define LW_ERROR_LDAP_CANNOT_CANCEL                        40352
 #define LW_ERROR_LDAP_ASSERTION_FAILED                     40353
 
+#define LW_ERROR_UID_TOO_LOW                               40400
+#define LW_ERROR_UID_TOO_HIGH                              40401
+#define LW_ERROR_GID_TOO_LOW                               40402
+#define LW_ERROR_GID_TOO_HIGH                              40403
 
 /* range 40500 - 40600 are reserved for GSS specific errors */
 
@@ -407,6 +411,11 @@
 #define LW_ERROR_CSV_NO_SUCH_FIELD                         40603
 #define LW_ERROR_TEST_SKIPPED                              40604
 
+/*Range 40700 - 41200 is reserved for registry specific error*/
+
+#define _LW_ERROR_REG_BASE                                 40700
+#define _LW_ERROR_REG_MAX                                  41200
+
 /* Range 41201 - 41700 is reserved for service manager errors */
 #define LW_ERROR_INVALID_SERVICE_TRANSITION                41201
 #define LW_ERROR_SERVICE_DEPENDENCY_UNMET                  41202
@@ -414,7 +423,7 @@
 #define LW_ERROR_NO_SUCH_SERVICE                           41204
 #define LW_ERROR_DEPENDENT_SERVICE_STILL_RUNNING           41205
 
-/* Range 41701 - 42500 is reserved for Kerberos errors.
+/* Range 41701 - 42499 is reserved for Kerberos errors.
  * Using the same spacing as the original error tables
  * to facilitate adding new errors.
  */
@@ -724,6 +733,28 @@
 #define LW_ERROR_KRB5_PROF_BAD_INTEGER                     42180
 #define LW_ERROR_KRB5_PROF_MAGIC_FILE_DATA                 42181
 
+/* Range 42500 - 42550 is reserved for domain join */
+
+#define LW_ERROR_INVALID_OU                                42500
+#define LW_ERROR_BAD_LICENSE_KEY                           42501
+#define LW_ERROR_MODULE_NOT_ENABLED                        42502
+#define LW_ERROR_MODULE_ALREADY_DONE                       42503
+#define LW_ERROR_PAM_MISSING_SERVICE                       42504
+#define LW_ERROR_PAM_BAD_CONF                              42505
+#define LW_ERROR_SHOW_USAGE                                42506
+#define LW_ERROR_DOMAINJOIN_WARNING                        42507
+#define LW_ERROR_FAILED_ADMIN_PRIVS                        42508
+
+//
+// 49900-49999 reserved for internal PAM error code mapping
+//
+
+#define _LW_ERROR_PAM_BASE                                 49900
+#define _LW_ERROR_PAM_MAX                                  49999
+
+
+LW_BEGIN_EXTERN_C
+
 size_t
 LwGetErrorString(
     LW_IN LW_DWORD dwError,
@@ -761,4 +792,7 @@ LwWin32ExtErrorToDescription(
     LW_WINERROR winerr
     );
 
+LW_END_EXTERN_C
+
+
 #endif /* __LW_ERROR_H__ */
