@@ -443,6 +443,12 @@ LwRpcStatusToNtStatus(
             return LW_STATUS_TOO_MANY_ADDRESSES;
         case LW_ERROR_RPC_S_INVALID_CREDENTIALS:
             return LW_STATUS_ACCESS_DENIED;
+        case LW_ERROR_RPC_S_AUTH_TKT_EXPIRED:
+            return LW_STATUS_KDC_CERT_REVOKED;
+        case LW_ERROR_RPC_S_AUTH_SKEW:
+            return LW_STATUS_TIME_DIFFERENCE_AT_DC;
+        case LW_ERROR_RPC_S_NO_NAME_MAPPING:
+            return LW_STATUS_BAD_NETWORK_NAME;
 
         case LW_ERROR_RPC_S_MOD:
         case LW_ERROR_RPC_S_OP_RNG_ERROR:
@@ -585,7 +591,6 @@ LwRpcStatusToNtStatus(
         case LW_ERROR_RPC_S_NO_MORE_MEMBERS:
         case LW_ERROR_RPC_S_NO_MORE_INTERFACES:
         case LW_ERROR_RPC_S_INVALID_NAME_SERVICE:
-        case LW_ERROR_RPC_S_NO_NAME_MAPPING:
         case LW_ERROR_RPC_S_PROFILE_NOT_FOUND:
         case LW_ERROR_RPC_S_NOT_FOUND:
         case LW_ERROR_RPC_S_NO_UPDATES:
@@ -661,12 +666,10 @@ LwRpcStatusToNtStatus(
         case LW_ERROR_RPC_S_CALL_ID_NOT_FOUND:
         case LW_ERROR_RPC_S_KEY_ID_NOT_FOUND:
         case LW_ERROR_RPC_S_AUTH_BAD_INTEGRITY:
-        case LW_ERROR_RPC_S_AUTH_TKT_EXPIRED:
         case LW_ERROR_RPC_S_AUTH_TKT_NYV:
         case LW_ERROR_RPC_S_AUTH_REPEAT:
         case LW_ERROR_RPC_S_AUTH_NOT_US:
         case LW_ERROR_RPC_S_AUTH_BADMATCH:
-        case LW_ERROR_RPC_S_AUTH_SKEW:
         case LW_ERROR_RPC_S_AUTH_BADADDR:
         case LW_ERROR_RPC_S_AUTH_BADVERSION:
         case LW_ERROR_RPC_S_AUTH_MSG_TYPE:
