@@ -39,14 +39,8 @@
 #include "includes.h"
 #include "threadpool-epoll.h"
 
-/* FIXME: Make these tweakable on the thread pool itself */
-
 /* Maximum events to read on each epoll_wait() */
 #define MAX_EVENTS 500
-/* Number of event-processing threads */
-#define EVENT_THREAD_COUNT 8
-/* Number of work item-processing threads */
-#define WORK_THREAD_COUNT 8
 /* Maximum number of ticks (task function invocations) to
    process each iteration of the event loop */
 #define MAX_TICKS 1000
@@ -558,8 +552,8 @@ EventLoop(
     RING runnable;
     RING waiting;
     CLOCK clock = {0};
-    LONG64 llNow;
-    LONG64 llNextDeadline;
+    LONG64 llNow = 0;
+    LONG64 llNextDeadline = 0;
     struct epoll_event events[MAX_EVENTS];
     int ready = 0;
     BOOLEAN bShutdown = FALSE;
@@ -575,7 +569,7 @@ EventLoop(
         status = ClockGetMonotonicTime(&clock, &llNow);
         GOTO_ERROR_ON_STATUS(status);
 
-        /* Schedule any timed tasks that are have reached their deadline */
+        /* Schedule any timed tasks that have reached their deadline */
         ScheduleTimedTasks(
             &timed,
             llNow,
@@ -661,58 +655,6 @@ EventThread(
     return NULL;
 }
 
-static
-NTSTATUS
-WorkLoop(
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-    PRING pRing = NULL;
-    PWORK_ITEM pItem = NULL;
-
-    LOCK_POOL(pThread->pPool);
-
-    for(;;)
-    {
-        while (!pThread->pPool->bShutdown && RingIsEmpty(&pThread->pPool->WorkItems))
-        {
-            pthread_cond_wait(&pThread->pPool->Event, &pThread->pPool->Lock);
-        }
-
-        if (pThread->pPool->bShutdown)
-        {
-            break;
-        }
-
-        RingDequeue(&pThread->pPool->WorkItems, &pRing);
-
-        UNLOCK_POOL(pThread->pPool);
-
-        pItem = LW_STRUCT_FROM_FIELD(pRing, WORK_ITEM, Ring);
-
-        pItem->pfnFunc(pItem->pContext);
-        RtlMemoryFree(pItem);
-
-        LOCK_POOL(pThread->pPool);
-    }
-
-    UNLOCK_POOL(pThread->pPool);
-
-    return status;
-}
-
-static
-PVOID
-WorkThread(
-    PVOID pContext
-    )
-{
-    WorkLoop((PWORK_ITEM_THREAD) pContext);
-
-    return NULL;
-}
-
 NTSTATUS
 LwRtlCreateTask(
     PLW_THREAD_POOL pPool,
@@ -728,6 +670,11 @@ LwRtlCreateTask(
     ULONG ulMinLoad = 0xFFFFFFFF;
     ULONG ulIndex = 0;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTask(pPool->pDelegate, ppTask, pGroup, pfnFunc, pContext);
+    }
+
     status = LW_RTL_ALLOCATE_AUTO(&pTask);
     GOTO_ERROR_ON_STATUS(status);
 
@@ -783,6 +730,11 @@ LwRtlCreateTaskGroup(
     NTSTATUS status = STATUS_SUCCESS;
     PLW_TASK_GROUP pGroup = NULL;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTaskGroup(pPool->pDelegate, ppGroup);
+    }
+
     status = LW_RTL_ALLOCATE_AUTO(&pGroup);
     GOTO_ERROR_ON_STATUS(status);
 
@@ -1080,6 +1032,7 @@ static
 NTSTATUS
 InitEventThread(
     PEPOLL_POOL pPool,
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs,
     PEPOLL_THREAD pThread,
     LONG lCpu
     )
@@ -1110,12 +1063,17 @@ InitEventThread(
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    SetCloseOnExec(pThread->SignalFds[0]);
+    SetCloseOnExec(pThread->SignalFds[1]);
+
     if ((pThread->EpollFd = epoll_create(MAX_EVENTS)) < 0)
     {
         status = LwErrnoToNtStatus(errno);
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    SetCloseOnExec(pThread->EpollFd);
+
     /* Add signal fd to epoll set */
     event.events = EPOLLIN;
     event.data.ptr = NULL;
@@ -1140,6 +1098,13 @@ InitEventThread(
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    if (pAttrs && pAttrs->ulTaskThreadStackSize)
+    {
+        status = LwErrnoToNtStatus(
+            pthread_attr_setstacksize(&threadAttr, pAttrs->ulTaskThreadStackSize));
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
     status = LwErrnoToNtStatus(
         pthread_create(
             &pThread->Thread,
@@ -1183,32 +1148,10 @@ DestroyEventThread(
     }
 }
 
-static
-NTSTATUS
-InitWorkThread(
-    PEPOLL_POOL pPool,
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-
-    pThread->pPool = pPool;
-
-    status = pthread_create(
-        &pThread->Thread,
-        NULL,
-        WorkThread,
-        pThread);
-    GOTO_ERROR_ON_STATUS(status);
-
-error:
-
-    return status;
-}
-
 NTSTATUS
 LwRtlCreateThreadPool(
-    PLW_THREAD_POOL* ppPool
+    PLW_THREAD_POOL* ppPool,
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -1219,8 +1162,6 @@ LwRtlCreateThreadPool(
     status = LW_RTL_ALLOCATE_AUTO(&pPool);
     GOTO_ERROR_ON_STATUS(status);
 
-    RingInit(&pPool->WorkItems);
-
     status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
@@ -1231,44 +1172,47 @@ LwRtlCreateThreadPool(
 
     if (numCpus < 0)
     {
-        pPool->ulEventThreadCount = EVENT_THREAD_COUNT;
+        numCpus = 1;
+    }
+
+    if (GetDelegateAttr(pAttrs))
+    {
+        status = AcquireDelegatePool(&pPool->pDelegate);
+        GOTO_ERROR_ON_STATUS(status);
     }
     else
     {
-        pPool->ulEventThreadCount = (ULONG) numCpus;
+        pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
+
+        if (pPool->ulEventThreadCount)
+        {
+            status = LW_RTL_ALLOCATE_ARRAY_AUTO(
+                &pPool->pEventThreads,
+                pPool->ulEventThreadCount);
+            GOTO_ERROR_ON_STATUS(status);
+
+            for (i = 0; i < pPool->ulEventThreadCount; i++)
+            {
+                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
+                GOTO_ERROR_ON_STATUS(status);
+            }
+        }
     }
 
-    status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-        &pPool->pEventThreads,
-        pPool->ulEventThreadCount);
+    status = InitWorkThreads(&pPool->WorkThreads, pAttrs, numCpus);
     GOTO_ERROR_ON_STATUS(status);
 
-    for (i = 0; i < pPool->ulEventThreadCount; i++)
-    {
-        status = InitEventThread(pPool, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
-        GOTO_ERROR_ON_STATUS(status);
-    }
-
-    pPool->ulWorkThreadCount = WORK_THREAD_COUNT;
-
-    status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-        &pPool->pWorkThreads,
-        pPool->ulWorkThreadCount);
-    GOTO_ERROR_ON_STATUS(status);
-
-    for (i = 0; i < pPool->ulWorkThreadCount; i++)
-    {
-        status = InitWorkThread(pPool, &pPool->pWorkThreads[i]);
-        GOTO_ERROR_ON_STATUS(status);
-    }
-
-    pPool->ulRefCount = 1;
-
     *ppPool = pPool;
 
-error:
+cleanup:
 
     return status;
+
+error:
+
+    LwRtlFreeThreadPool(&pPool);
+
+    goto cleanup;
 }
 
 VOID
@@ -1303,18 +1247,15 @@ LwRtlFreeThreadPool(
             RtlMemoryFree(pPool->pEventThreads);
         }
 
-        if (pPool->pWorkThreads)
+        if (pPool->pDelegate)
         {
-            for (i = 0; i < pPool->ulWorkThreadCount; i++)
-            {
-                pthread_join(pPool->pWorkThreads[i].Thread, NULL);
-            }
-
-            RtlMemoryFree(pPool->pWorkThreads);
+            ReleaseDelegatePool(&pPool->pDelegate);
         }
 
         pthread_mutex_destroy(&pPool->Lock);
 
+        DestroyWorkThreads(&pPool->WorkThreads);
+
         RtlMemoryFree(pPool);
 
         *ppPool = NULL;
@@ -1329,32 +1270,8 @@ LwRtlQueueWorkItem(
     LW_WORK_ITEM_FLAGS Flags
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
-    PWORK_ITEM pItem = NULL;
-
-    status = LW_RTL_ALLOCATE_AUTO(&pItem);
-    GOTO_ERROR_ON_STATUS(status);
-
-    RingInit(&pItem->Ring);
-    pItem->pfnFunc = pfnFunc;
-    pItem->pContext = pContext;
-
-    LOCK_POOL(pPool);
-
-    RingEnqueue(&pPool->WorkItems, &pItem->Ring);
-    pthread_cond_signal(&pPool->Event);
-    pItem = NULL;
-
-error:
-
-    UNLOCK_POOL(pPool);
-
-    if (pItem)
-    {
-        RtlMemoryFree(pItem);
-    }
-
-    return status;
+    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
 }
 
 
+
