@@ -755,58 +755,6 @@ EventThread(
     return NULL;
 }
 
-static
-NTSTATUS
-WorkLoop(
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-    PRING pRing = NULL;
-    PWORK_ITEM pItem = NULL;
-
-    LOCK_POOL(pThread->pPool);
-
-    for(;;)
-    {
-        while (!pThread->pPool->bShutdown && RingIsEmpty(&pThread->pPool->WorkItems))
-        {
-            pthread_cond_wait(&pThread->pPool->Event, &pThread->pPool->Lock);
-        }
-
-        if (pThread->pPool->bShutdown)
-        {
-            break;
-        }
-
-        RingDequeue(&pThread->pPool->WorkItems, &pRing);
-
-        UNLOCK_POOL(pThread->pPool);
-
-        pItem = LW_STRUCT_FROM_FIELD(pRing, WORK_ITEM, Ring);
-
-        pItem->pfnFunc(pItem->pContext);
-        RtlMemoryFree(pItem);
-
-        LOCK_POOL(pThread->pPool);
-    }
-
-    UNLOCK_POOL(pThread->pPool);
-
-    return status;
-}
-
-static
-PVOID
-WorkThread(
-    PVOID pContext
-    )
-{
-    WorkLoop((PWORK_ITEM_THREAD) pContext);
-
-    return NULL;
-}
-
 NTSTATUS
 LwRtlCreateTask(
     PLW_THREAD_POOL pPool,
@@ -822,6 +770,11 @@ LwRtlCreateTask(
     ULONG ulMinLoad = 0xFFFFFFFF;
     ULONG ulIndex = 0;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTask(pPool->pDelegate, ppTask, pGroup, pfnFunc, pContext);
+    }
+
     status = LW_RTL_ALLOCATE_AUTO(&pTask);
     GOTO_ERROR_ON_STATUS(status);
 
@@ -877,6 +830,11 @@ LwRtlCreateTaskGroup(
     NTSTATUS status = STATUS_SUCCESS;
     PLW_TASK_GROUP pGroup = NULL;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTaskGroup(pPool->pDelegate, ppGroup);
+    }
+
     status = LW_RTL_ALLOCATE_AUTO(&pGroup);
     GOTO_ERROR_ON_STATUS(status);
 
@@ -1159,6 +1117,7 @@ static
 NTSTATUS
 InitEventThread(
     PKQUEUE_POOL pPool,
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs,
     PKQUEUE_THREAD pThread,
     LONG lCpu
     )
@@ -1189,12 +1148,17 @@ InitEventThread(
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    SetCloseOnExec(pThread->SignalFds[0]);
+    SetCloseOnExec(pThread->SignalFds[1]);
+
     if ((pThread->KqueueFd = kqueue()) < 0)
     {
         status = LwErrnoToNtStatus(errno);
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    SetCloseOnExec(pThread->KqueueFd);
+
     /* Add signal fd to kqueue set */
     EV_SET(&event, pThread->SignalFds[0], EVFILT_READ, EV_ADD, 0, 0, NULL);
 
@@ -1219,6 +1183,13 @@ InitEventThread(
         GOTO_ERROR_ON_STATUS(status);
     }
 
+    if (pAttrs && pAttrs->ulTaskThreadStackSize)
+    {
+        status = LwErrnoToNtStatus(
+            pthread_attr_setstacksize(&threadAttr, pAttrs->ulTaskThreadStackSize));
+        GOTO_ERROR_ON_STATUS(status);
+    }
+
     status = LwErrnoToNtStatus(
         pthread_create(
             &pThread->Thread,
@@ -1262,32 +1233,10 @@ DestroyEventThread(
     }
 }
 
-static
-NTSTATUS
-InitWorkThread(
-    PKQUEUE_POOL pPool,
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-
-    pThread->pPool = pPool;
-
-    status = pthread_create(
-        &pThread->Thread,
-        NULL,
-        WorkThread,
-        pThread);
-    GOTO_ERROR_ON_STATUS(status);
-
-error:
-
-    return status;
-}
-
 NTSTATUS
 LwRtlCreateThreadPool(
-    PLW_THREAD_POOL* ppPool
+    PLW_THREAD_POOL* ppPool,
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
@@ -1298,8 +1247,6 @@ LwRtlCreateThreadPool(
     status = LW_RTL_ALLOCATE_AUTO(&pPool);
     GOTO_ERROR_ON_STATUS(status);
 
-    RingInit(&pPool->WorkItems);
-
     status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL));
     GOTO_ERROR_ON_STATUS(status);
 
@@ -1310,44 +1257,47 @@ LwRtlCreateThreadPool(
 
     if (numCpus < 0)
     {
-        pPool->ulEventThreadCount = EVENT_THREAD_COUNT;
+        numCpus = 1;
+    }
+
+    if (GetDelegateAttr(pAttrs))
+    {
+        status = AcquireDelegatePool(&pPool->pDelegate);
+        GOTO_ERROR_ON_STATUS(status);
     }
     else
     {
-        pPool->ulEventThreadCount = (ULONG) numCpus;
+        pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
+
+        if (pPool->ulEventThreadCount)
+        {
+            status = LW_RTL_ALLOCATE_ARRAY_AUTO(
+                &pPool->pEventThreads,
+                pPool->ulEventThreadCount);
+            GOTO_ERROR_ON_STATUS(status);
+
+            for (i = 0; i < pPool->ulEventThreadCount; i++)
+            {
+                status = InitEventThread(pPool, pAttrs, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
+                GOTO_ERROR_ON_STATUS(status);
+            }
+        }
     }
 
-    status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-        &pPool->pEventThreads,
-        pPool->ulEventThreadCount);
+    status = InitWorkThreads(&pPool->WorkThreads, pAttrs, numCpus);
     GOTO_ERROR_ON_STATUS(status);
 
-    for (i = 0; i < pPool->ulEventThreadCount; i++)
-    {
-        status = InitEventThread(pPool, &pPool->pEventThreads[i], numCpus >= 0 ? i % numCpus : -1);
-        GOTO_ERROR_ON_STATUS(status);
-    }
-
-    pPool->ulWorkThreadCount = WORK_THREAD_COUNT;
-
-    status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-        &pPool->pWorkThreads,
-        pPool->ulWorkThreadCount);
-    GOTO_ERROR_ON_STATUS(status);
-
-    for (i = 0; i < pPool->ulWorkThreadCount; i++)
-    {
-        status = InitWorkThread(pPool, &pPool->pWorkThreads[i]);
-        GOTO_ERROR_ON_STATUS(status);
-    }
-
-    pPool->ulRefCount = 1;
-
     *ppPool = pPool;
 
-error:
+cleanup:
 
     return status;
+
+error:
+
+    LwRtlFreeThreadPool(&pPool);
+
+    goto cleanup;
 }
 
 VOID
@@ -1382,18 +1332,15 @@ LwRtlFreeThreadPool(
             RtlMemoryFree(pPool->pEventThreads);
         }
 
-        if (pPool->pWorkThreads)
+        if (pPool->pDelegate)
         {
-            for (i = 0; i < pPool->ulWorkThreadCount; i++)
-            {
-                pthread_join(pPool->pWorkThreads[i].Thread, NULL);
-            }
-
-            RtlMemoryFree(pPool->pWorkThreads);
+            ReleaseDelegatePool(&pPool->pDelegate);
         }
 
         pthread_mutex_destroy(&pPool->Lock);
 
+        DestroyWorkThreads(&pPool->WorkThreads);
+
         RtlMemoryFree(pPool);
 
         *ppPool = NULL;
@@ -1408,32 +1355,7 @@ LwRtlQueueWorkItem(
     LW_WORK_ITEM_FLAGS Flags
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
-    PWORK_ITEM pItem = NULL;
-
-    status = LW_RTL_ALLOCATE_AUTO(&pItem);
-    GOTO_ERROR_ON_STATUS(status);
-
-    RingInit(&pItem->Ring);
-    pItem->pfnFunc = pfnFunc;
-    pItem->pContext = pContext;
-
-    LOCK_POOL(pPool);
-
-    RingEnqueue(&pPool->WorkItems, &pItem->Ring);
-    pthread_cond_signal(&pPool->Event);
-    pItem = NULL;
-
-error:
-
-    UNLOCK_POOL(pPool);
-
-    if (pItem)
-    {
-        RtlMemoryFree(pItem);
-    }
-
-    return status;
+    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
 }
 
 
