@@ -525,58 +525,6 @@ EventThread(
     return NULL;
 }
 
-static
-NTSTATUS
-WorkLoop(
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-    PRING pRing = NULL;
-    PWORK_ITEM pItem = NULL;
-
-    LOCK_POOL(pThread->pPool);
-
-    for(;;)
-    {
-        while (!pThread->pPool->bShutdown && RingIsEmpty(&pThread->pPool->WorkItems))
-        {
-            pthread_cond_wait(&pThread->pPool->Event, &pThread->pPool->Lock);
-        }
-
-        if (pThread->pPool->bShutdown)
-        {
-            break;
-        }
-
-        RingDequeue(&pThread->pPool->WorkItems, &pRing);
-
-        UNLOCK_POOL(pThread->pPool);
-
-        pItem = LW_STRUCT_FROM_FIELD(pRing, WORK_ITEM, Ring);
-
-        pItem->pfnFunc(pItem->pContext);
-        RtlMemoryFree(pItem);
-
-        LOCK_POOL(pThread->pPool);
-    }
-
-    UNLOCK_POOL(pThread->pPool);
-
-    return status;
-}
-
-static
-PVOID
-WorkThread(
-    PVOID pContext
-    )
-{
-    WorkLoop((PWORK_ITEM_THREAD) pContext);
-
-    return NULL;
-}
-
 NTSTATUS
 LwRtlCreateTask(
     PLW_THREAD_POOL pPool,
@@ -589,6 +537,11 @@ LwRtlCreateTask(
     NTSTATUS status = STATUS_SUCCESS;
     PSELECT_TASK pTask = NULL;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTask(pPool->pDelegate, ppTask, pGroup, pfnFunc, pContext);
+    }
+
     GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pTask));
 
     RingInit(&pTask->GroupRing);
@@ -637,6 +590,11 @@ LwRtlCreateTaskGroup(
     NTSTATUS status = STATUS_SUCCESS;
     PLW_TASK_GROUP pGroup = NULL;
 
+    if (pPool->pDelegate)
+    {
+        return LwRtlCreateTaskGroup(pPool->pDelegate, ppGroup);
+    }
+
     GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pGroup));
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pGroup->Lock, NULL)));
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pGroup->Event, NULL)));
@@ -896,10 +854,18 @@ LwRtlWaitTaskGroup(
 static
 NTSTATUS
 SelectThreadInit(
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs,
     PSELECT_THREAD pThread
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
+    pthread_attr_t pthreadAttr;
+    BOOLEAN bAttrInit = FALSE;
+
+    status = LwErrnoToNtStatus(pthread_attr_init(&pthreadAttr));
+    GOTO_ERROR_ON_STATUS(status);
+
+    bAttrInit = TRUE;
 
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pThread->Lock, NULL)));
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pThread->Event, NULL)));
@@ -909,16 +875,31 @@ SelectThreadInit(
         GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(errno));
     }
 
+    SetCloseOnExec(pThread->SignalFds[0]);
+    SetCloseOnExec(pThread->SignalFds[1]);
+
     RingInit(&pThread->Tasks);
 
-    GOTO_ERROR_ON_STATUS(status = pthread_create(
-                      &pThread->Thread,
-                      NULL,
-                      EventThread,
-                      pThread));
+    if (pAttrs && pAttrs->ulTaskThreadStackSize)
+    {
+        GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(
+                                 pthread_attr_setstacksize(&pthreadAttr, pAttrs->ulTaskThreadStackSize)));
+    }
+
+    GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(
+                             pthread_create(
+                                 &pThread->Thread,
+                                 &pthreadAttr,
+                                 EventThread,
+                                 pThread)));
 
 error:
 
+    if (bAttrInit)
+    {
+        pthread_attr_destroy(&pthreadAttr);
+    }
+
     return status;
 }
 
@@ -932,65 +913,58 @@ SelectThreadDestroy(
     pthread_cond_destroy(&pThread->Event);
 }
 
-static
-NTSTATUS
-WorkThreadInit(
-    PSELECT_POOL pPool,
-    PWORK_ITEM_THREAD pThread
-    )
-{
-    NTSTATUS status = STATUS_SUCCESS;
-
-    pThread->pPool = pPool;
-
-    GOTO_ERROR_ON_STATUS(status = pthread_create(
-                      &pThread->Thread,
-                      NULL,
-                      WorkThread,
-                      pThread));
-
-error:
-
-    return status;
-}
-
 NTSTATUS
 LwRtlCreateThreadPool(
-    PLW_THREAD_POOL* ppPool
+    PLW_THREAD_POOL* ppPool,
+    PLW_THREAD_POOL_ATTRIBUTES pAttrs
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
     PLW_THREAD_POOL pPool = NULL;
     int i = 0;
+    int numCpus = 0;
+
+#if defined(_SC_NPROCESSORS_ONLN)
+    numCpus = sysconf(_SC_NPROCESSORS_ONLN);
+
+    if (numCpus < 0)
+    {
+        numCpus = 1;
+    }
+#else
+    numCpus = 1;
+#endif
+
 
     GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pPool));
 
-    RingInit(&pPool->WorkItems);
-
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_mutex_init(&pPool->Lock, NULL)));
     GOTO_ERROR_ON_STATUS(status = LwErrnoToNtStatus(pthread_cond_init(&pPool->Event, NULL)));
 
-    pPool->ulEventThreadCount = EVENT_THREAD_COUNT;
-    pPool->ulNextEventThread = 0;
-    GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-                             &pPool->pEventThreads,
-                             pPool->ulEventThreadCount));
-    for (i = 0; i < pPool->ulEventThreadCount; i++)
+    if (GetDelegateAttr(pAttrs))
     {
-        GOTO_ERROR_ON_STATUS(status = SelectThreadInit(&pPool->pEventThreads[i]));
+        status = AcquireDelegatePool(&pPool->pDelegate);
+        GOTO_ERROR_ON_STATUS(status);
+    }
+    else
+    {
+        pPool->ulEventThreadCount = GetTaskThreadsAttr(pAttrs, numCpus);
+        pPool->ulNextEventThread = 0;
+
+        if (pPool->ulEventThreadCount)
+        {
+            GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_ARRAY_AUTO(
+                                     &pPool->pEventThreads,
+                                     pPool->ulEventThreadCount));
+            for (i = 0; i < pPool->ulEventThreadCount; i++)
+            {
+                GOTO_ERROR_ON_STATUS(status = SelectThreadInit(pAttrs, &pPool->pEventThreads[i]));
+            }
+        }
     }
 
-    pPool->ulWorkThreadCount = WORK_THREAD_COUNT;
-    GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_ARRAY_AUTO(
-                             &pPool->pWorkThreads,
-                             pPool->ulWorkThreadCount));
-
-    for (i = 0; i < pPool->ulWorkThreadCount; i++)
-    {
-        GOTO_ERROR_ON_STATUS(status = WorkThreadInit(pPool, &pPool->pWorkThreads[i]));
-    }
-
-    pPool->ulRefCount = 1;
+    status = InitWorkThreads(&pPool->WorkThreads, pAttrs, numCpus);
+    GOTO_ERROR_ON_STATUS(status);
 
     *ppPool = pPool;
 
@@ -1031,18 +1005,15 @@ LwRtlFreeThreadPool(
             RtlMemoryFree(pPool->pEventThreads);
         }
 
-        if (pPool->pWorkThreads)
+        if (pPool->pDelegate)
         {
-            for (i = 0; i < pPool->ulWorkThreadCount; i++)
-            {
-                pthread_join(pPool->pWorkThreads[i].Thread, NULL);
-            }
-
-            RtlMemoryFree(pPool->pWorkThreads);
+            ReleaseDelegatePool(&pPool->pDelegate);
         }
 
         pthread_mutex_destroy(&pPool->Lock);
 
+        DestroyWorkThreads(&pPool->WorkThreads);
+
         RtlMemoryFree(pPool);
 
         *ppPool = NULL;
@@ -1057,30 +1028,7 @@ LwRtlQueueWorkItem(
     LW_WORK_ITEM_FLAGS Flags
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
-    PWORK_ITEM pItem = NULL;
-
-    GOTO_ERROR_ON_STATUS(status = LW_RTL_ALLOCATE_AUTO(&pItem));
-    RingInit(&pItem->Ring);
-    pItem->pfnFunc = pfnFunc;
-    pItem->pContext = pContext;
-
-    LOCK_POOL(pPool);
-
-    RingEnqueue(&pPool->WorkItems, &pItem->Ring);
-    pthread_cond_signal(&pPool->Event);
-    pItem = NULL;
-
-error:
-
-    UNLOCK_POOL(pPool);
-
-    if (pItem)
-    {
-        RtlMemoryFree(pItem);
-    }
-
-    return status;
+    return QueueWorkItem(&pPool->WorkThreads, pfnFunc, pContext, Flags);
 }
 
 
