@@ -383,12 +383,12 @@ LwIoAssertionFailedFormat(
 #define LWIO_ASSERT_MSG(Expression, Message) \
     ((Expression) ? \
      TRUE : \
-     LwIoAssertionFailed(#Expression, Message, __FUNCTION__, __FILE__, __LINE__), FALSE)
+     (LwIoAssertionFailed(#Expression, Message, __FUNCTION__, __FILE__, __LINE__), FALSE))
 
 #define LWIO_ASSERT_FORMAT(Expression, Format, ...) \
     ((Expression) ? \
      TRUE : \
-     LwIoAssertionFailedFormat(#Expression, Format, __FUNCTION__, __FILE__, __LINE__, ## __VA_ARGS__), FALSE)
+     (LwIoAssertionFailedFormat(#Expression, Format, __FUNCTION__, __FILE__, __LINE__, ## __VA_ARGS__), FALSE))
 
 #define LWIO_ASSERT(Expression) \
     LWIO_ASSERT_MSG(Expression, NULL)
@@ -714,8 +714,6 @@ SMBFreeNullTerminatedStringArray(
     PSTR * ppStringArray
     );
 
-#if defined(UNICODE)
-
 DWORD
 SMBAllocateStringW(
     PWSTR  pwszInputString,
@@ -747,13 +745,20 @@ SMBWc16sLen(
     size_t*    psLen
     );
 
-DWORD
+int
 SMBWc16sCmp(
     PCWSTR  pwszFirst,
     PCWSTR  pwszSecond
     );
 
-DWORD
+int
+SMBWc16snCmp(
+    PCWSTR pwszFirst,
+    PCWSTR pwszSecond,
+    size_t sLen
+    );
+
+int
 SMBWc16sCaseCmp(
     PCWSTR  pwszFirst,
     PCWSTR  pwszSecond
@@ -765,8 +770,6 @@ SMBWc16sDup(
     PWSTR* pwszOutput
     );
 
-#endif /* defined(UNICODE) */
-
 DWORD
 SMBDLinkedListPrepend(
     PSMBDLINKEDLIST* ppList,
