@@ -42,21 +42,19 @@
  * Authors: Danilo Almeida (dalmeida@likewisesoftware.com)
  */
 
-#define LWMSG_SPEC_META
-
 #include "includes.h"
 #include "ntipcmsg.h"
 #include "goto.h"
 #include "ntlogmacros.h"
 
 #define _LWMSG_MEMBER_BOOLEAN(Type, Field) \
-    LWMSG_MEMBER_UINT8(Type, Field)
+    LWMSG_MEMBER_TYPESPEC(Type, Field, gNtIpcTypeSpecBoolean)
 
 #define _LWMSG_MEMBER_BUFFER(Type, BufferField, LengthField) \
     LWMSG_MEMBER_POINTER_BEGIN(Type, BufferField), \
     LWMSG_UINT8(BYTE), \
     LWMSG_POINTER_END, \
-    LWMSG_ATTR_SENSITIVE, \
+    LWMSG_ATTR_ENCODING("hex+ascii"), \
     LWMSG_ATTR_LENGTH_MEMBER(Type, LengthField)
 
 #define _LWMSG_MEMBER_OPTIONAL_ARRAY(Type, PointerField, CountField, ElementSpec) \
@@ -90,6 +88,16 @@
 #define _LWMSG_MEMBER_IO_FILE_HANDLE_OUT(Type, Field) \
     _LWMSG_MEMBER_HANDLE_OUT(Type, Field, IO_FILE_HANDLE)
 
+static
+LWMsgTypeSpec gNtIpcTypeSpecBoolean[] =
+{
+    LWMSG_ENUM_BEGIN(BOOLEAN, 1, LWMSG_UNSIGNED),
+    LWMSG_ENUM_NAMED_VALUE("FALSE", FALSE),
+    LWMSG_ENUM_NAMED_VALUE("TRUE", TRUE),
+    LWMSG_ENUM_END,
+    LWMSG_TYPE_END
+};
+
 static
 LWMsgTypeSpec gNtIpcTypeSpecIoFileName[] =
 {
@@ -539,3 +547,22 @@ NtIpcUnregisterFileHandle(
             FileHandle));
     return status;
 }
+
+
+NTSTATUS
+NtIpcReleaseFileHandle(
+    IN LWMsgCall* pCall,
+    IN IO_FILE_HANDLE FileHandle
+    )
+{
+    NTSTATUS status = 0;
+    LWMsgSession* pSession = NULL;
+
+    pSession = lwmsg_call_get_session(pCall);
+
+    status = NtIpcLWMsgStatusToNtStatus(
+        lwmsg_session_release_handle(
+            pSession,
+            FileHandle));
+    return status;
+}
