@@ -118,11 +118,20 @@ static LWMsgTypeSpec gShareInfo502Spec[] =
 };
 
 
-#define SHARE_INFO_LEVEL_0   0
-#define SHARE_INFO_LEVEL_1   1
-#define SHARE_INFO_LEVEL_2   2
-#define SHARE_INFO_LEVEL_501 501
-#define SHARE_INFO_LEVEL_502 502
+static LWMsgTypeSpec gShareInfo1005Spec[] =
+{
+    LWMSG_STRUCT_BEGIN(SHARE_INFO_1005),
+    LWMSG_MEMBER_UINT32(SHARE_INFO_1005, shi1005_flags),
+    LWMSG_STRUCT_END
+};
+
+
+#define SHARE_INFO_LEVEL_0     0
+#define SHARE_INFO_LEVEL_1     1
+#define SHARE_INFO_LEVEL_2     2
+#define SHARE_INFO_LEVEL_501   501
+#define SHARE_INFO_LEVEL_502   502
+#define SHARE_INFO_LEVEL_1005  1005
 
 
 static LWMsgTypeSpec gShareInfoUnionSpec[] =
@@ -138,6 +147,8 @@ static LWMsgTypeSpec gShareInfoUnionSpec[] =
     LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_501),
     LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p502, LWMSG_TYPESPEC(gShareInfo502Spec)),
     LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_502),
+    LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p1005, LWMSG_TYPESPEC(gShareInfo1005Spec)),
+    LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_1005),
     LWMSG_UNION_END,
     LWMSG_TYPE_END
 };
@@ -185,6 +196,9 @@ static LWMsgTypeSpec gShareInfoEnumParamsSpec[] =
     LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p502, LWMSG_TYPESPEC(gShareInfo502Spec)),
     LWMSG_ATTR_LENGTH_MEMBER(SHARE_INFO_ENUM_PARAMS, dwNumEntries),
     LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_502),
+    LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p1005, LWMSG_TYPESPEC(gShareInfo1005Spec)),
+    LWMSG_ATTR_LENGTH_MEMBER(SHARE_INFO_ENUM_PARAMS, dwNumEntries),
+    LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_1005),
     LWMSG_UNION_END,
     LWMSG_ATTR_DISCRIM(SHARE_INFO_ENUM_PARAMS, dwInfoLevel),
     LWMSG_STRUCT_END,
@@ -220,102 +234,14 @@ static LWMsgTypeSpec gShareInfoGetParamsSpec[] =
     LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_501),
     LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p502, LWMSG_TYPESPEC(gShareInfo502Spec)),
     LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_502),
+    LWMSG_MEMBER_POINTER(SHARE_INFO_UNION, p1005, LWMSG_TYPESPEC(gShareInfo1005Spec)),
+    LWMSG_ATTR_TAG(SHARE_INFO_LEVEL_1005),
     LWMSG_UNION_END,
     LWMSG_ATTR_DISCRIM(SHARE_INFO_GETINFO_PARAMS, dwInfoLevel),
     LWMSG_STRUCT_END,
     LWMSG_TYPE_END
 };
 
-#if defined(sun) || defined(_AIX)
-#  define ONCE_INIT {PTHREAD_ONCE_INIT}
-#else
-#  define ONCE_INIT PTHREAD_ONCE_INIT
-#endif
-
-static pthread_once_t gShareMarshalContextControl = ONCE_INIT;
-static LWMsgContext* gpMarshalContext = NULL;
-static LWMsgStatus gMarshalContextStatus = LWMSG_STATUS_SUCCESS;
-
-static
-void
-LwShareInfoInitMarshalContext(void)
-{
-    gMarshalContextStatus = lwmsg_context_new(NULL, &gpMarshalContext);
-}
-
-static
-LW_NTSTATUS
-LwShareInfoGetMarshalContext(
-    LWMsgContext** ppMarshalContext
-    )
-{
-    NTSTATUS Status = STATUS_SUCCESS;
-    int err = 0;
-
-    err = pthread_once(&gShareMarshalContextControl, LwShareInfoInitMarshalContext);
-    if (err)
-    {
-        Status = LwErrnoToNtStatus(err);
-        BAIL_ON_NT_STATUS(Status);
-    }
-
-    Status = MAP_LWMSG_STATUS(gMarshalContextStatus);
-    BAIL_ON_NT_STATUS(Status);
-
-    *ppMarshalContext = gpMarshalContext;
-
-cleanup:
-
-    return Status;
-
-error:
-
-    *ppMarshalContext = NULL;
-
-    goto cleanup;
-}
-
-static
-LW_NTSTATUS
-LwShareInfoAcquireDataContext(
-    LWMsgDataContext** ppDataContext
-    )
-{
-    NTSTATUS Status = STATUS_SUCCESS;
-    LWMsgContext* pContext;
-    LWMsgDataContext* pDataContext;
-
-    Status = LwShareInfoGetMarshalContext(&pContext);
-    BAIL_ON_NT_STATUS(Status);
-
-    Status = MAP_LWMSG_STATUS(lwmsg_data_context_new(pContext, &pDataContext));
-    BAIL_ON_NT_STATUS(Status);
-
-    *ppDataContext = pDataContext;
-
-cleanup:
-
-    return Status;
-
-error:
-
-    *ppDataContext = NULL;
-
-    goto cleanup;
-}
-
-static
-void
-LwShareInfoReleaseDataContext(
-    LWMsgDataContext* pDataContext
-    )
-{
-    if (pDataContext)
-    {
-        lwmsg_data_context_delete(pDataContext);
-    }
-}
-
 LW_NTSTATUS
 LwShareInfoMarshalAddParameters(
     PSHARE_INFO_ADD_PARAMS pParams,
@@ -328,7 +254,7 @@ LwShareInfoMarshalAddParameters(
     size_t szBufferSize = 0;
     LWMsgDataContext* pDataContext = NULL;
 
-    Status = LwShareInfoAcquireDataContext(&pDataContext);
+    Status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(Status);
 
     Status = MAP_LWMSG_STATUS(
@@ -345,7 +271,7 @@ LwShareInfoMarshalAddParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
 
@@ -372,7 +298,7 @@ LwShareInfoUnmarshalAddParameters(
     PSHARE_INFO_ADD_PARAMS pParams = NULL;
     LWMsgDataContext* pDataContext = NULL;
 
-    Status = LwShareInfoAcquireDataContext(&pDataContext);
+    Status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(Status);
 
     Status = MAP_LWMSG_STATUS(
@@ -388,7 +314,7 @@ LwShareInfoUnmarshalAddParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
 
@@ -417,7 +343,7 @@ LwShareInfoMarshalDeleteParameters(
     size_t ulBufferSize = 0;
     LWMsgDataContext* pDataContext = NULL;
 
-    Status = LwShareInfoAcquireDataContext(&pDataContext);
+    Status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(Status);
 
     Status = MAP_LWMSG_STATUS(
@@ -434,7 +360,7 @@ LwShareInfoMarshalDeleteParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
 
@@ -462,7 +388,7 @@ LwShareInfoUnmarshalDeleteParameters(
     PSHARE_INFO_DELETE_PARAMS pParams = NULL;
     LWMsgDataContext* pDataContext = NULL;
 
-    Status = LwShareInfoAcquireDataContext(&pDataContext);
+    Status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(Status);
 
     Status = MAP_LWMSG_STATUS(
@@ -478,7 +404,7 @@ LwShareInfoUnmarshalDeleteParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
 
@@ -507,7 +433,7 @@ LwShareInfoMarshalEnumParameters(
     size_t ulBufferSize = 0;
     LWMsgDataContext* pDataContext = NULL;
 
-    status = LwShareInfoAcquireDataContext(&pDataContext);
+    status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(status);
 
     status = MAP_LWMSG_STATUS(
@@ -524,7 +450,7 @@ LwShareInfoMarshalEnumParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return status;
 
@@ -552,7 +478,7 @@ LwShareInfoUnmarshalEnumParameters(
     PSHARE_INFO_ENUM_PARAMS pParams = NULL;
     LWMsgDataContext* pDataContext = NULL;
 
-    Status = LwShareInfoAcquireDataContext(&pDataContext);
+    Status = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(Status);
 
     Status = MAP_LWMSG_STATUS(
@@ -568,7 +494,7 @@ LwShareInfoUnmarshalEnumParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return Status;
 
@@ -597,7 +523,7 @@ LwShareInfoMarshalSetParameters(
     size_t ulBufferSize = 0;
     LWMsgDataContext* pDataContext = NULL;
 
-    ntStatus = LwShareInfoAcquireDataContext(&pDataContext);
+    ntStatus = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = MAP_LWMSG_STATUS(
@@ -614,7 +540,7 @@ LwShareInfoMarshalSetParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return ntStatus;
 
@@ -642,7 +568,7 @@ LwShareInfoUnmarshalSetParameters(
     LWMsgDataContext* pDataContext = NULL;
 
 
-    ntStatus = LwShareInfoAcquireDataContext(&pDataContext);
+    ntStatus = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = MAP_LWMSG_STATUS(
@@ -658,7 +584,7 @@ LwShareInfoUnmarshalSetParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return ntStatus;
 
@@ -687,7 +613,7 @@ LwShareInfoMarshalGetParameters(
     size_t ulBufferSize = 0;
     LWMsgDataContext* pDataContext = NULL;
 
-    ntStatus = LwShareInfoAcquireDataContext(&pDataContext);
+    ntStatus = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = MAP_LWMSG_STATUS(
@@ -704,7 +630,7 @@ LwShareInfoMarshalGetParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return ntStatus;
 
@@ -733,7 +659,7 @@ LwShareInfoUnmarshalGetParameters(
     PSHARE_INFO_GETINFO_PARAMS pParams = NULL;
     LWMsgDataContext* pDataContext = NULL;
 
-    ntStatus = LwShareInfoAcquireDataContext(&pDataContext);
+    ntStatus = LwSrvInfoAcquireDataContext(&pDataContext);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = MAP_LWMSG_STATUS(
@@ -749,7 +675,7 @@ LwShareInfoUnmarshalGetParameters(
 
 cleanup:
 
-    LwShareInfoReleaseDataContext(pDataContext);
+    LwSrvInfoReleaseDataContext(pDataContext);
 
     return ntStatus;
 
