@@ -46,6 +46,11 @@
 
 #include "pvfs.h"
 
+static
+NTSTATUS
+PvfsSetMaximalAccessMask(
+    PIRP pIrp
+    );
 
 /*****************************************************************************
  Main entry to the Create() routine for driver.  Splits work based on the
@@ -68,9 +73,12 @@ PvfsCreate(
                                         FILE_NON_DIRECTORY_FILE);
 
 
-    /* Check to see if this is a Device Create() */
+    /* Check to see if this is a Device Create (i.e. NULL RootFileHandle
+       and empty Filename) */
 
-    if (*pIrp->Args.Create.FileName.FileName == '\0')
+    if (!pIrp->Args.Create.FileName.RootFileHandle &&
+        pIrp->Args.Create.FileName.FileName &&
+        *pIrp->Args.Create.FileName.FileName == '\0')
     {
         ntError = PvfsCreateDevice(pIrpContext);
         BAIL_ON_NT_STATUS(ntError);
@@ -82,6 +90,12 @@ PvfsCreate(
 
     CreateOptions = pIrp->Args.Create.CreateOptions;
 
+    if (((CreateOptions & ~FILE_CREATE_OPTIONS_VALID) != 0))
+    {
+        ntError = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntError);
+    }
+
     if ((CreateOptions & FileDirCombo) == FileDirCombo)
     {
         ntError = STATUS_INVALID_PARAMETER;
@@ -132,6 +146,9 @@ PvfsCreate(
     }
     BAIL_ON_NT_STATUS(ntError);
 
+    ntError = PvfsSetMaximalAccessMask(pIrp);
+    BAIL_ON_NT_STATUS(ntError);
+
 cleanup:
     RtlCStringFree(&pszFilename);
     RtlCStringFree(&pszDiskFilename);
@@ -146,6 +163,14 @@ error:
 /*****************************************************************************
  ****************************************************************************/
 
+static
+VOID
+PvfsCleanupFailedCreate(
+    int fd,
+    PCSTR pszPath,
+    BOOLEAN bWasCreated
+    );
+
 NTSTATUS
 PvfsCreateFileDoSysOpen(
     IN PVOID pContext
@@ -161,6 +186,18 @@ PvfsCreateFileDoSysOpen(
     FILE_CREATE_RESULT CreateResult = 0;
     PIO_SECURITY_CONTEXT_PROCESS_INFORMATION pProcess = NULL;
 
+    pProcess = IoSecurityGetProcessInfo(pSecCtx);
+
+    /* Fail any create that requires setting the security but doesn't
+       have the Unix uid/gid information */
+
+    if ((pCreateContext->SetPropertyFlags & PVFS_SET_PROP_SECURITY) &&
+        (pProcess == NULL))
+    {
+        ntError = STATUS_ACCESS_DENIED;
+        BAIL_ON_NT_STATUS(ntError);
+    }
+
     ntError = PvfsEnforceShareMode(
                    pCreateContext->pFcb,
                    Args.ShareAccess,
@@ -172,7 +209,15 @@ PvfsCreateFileDoSysOpen(
     ntError = MapPosixOpenFlags(&unixFlags, pCreateContext->GrantedAccess, Args);
     BAIL_ON_NT_STATUS(ntError);
 
-    ntError = PvfsSysOpen(&fd, pCreateContext->pszDiskFilename, unixFlags, 0700);
+    do
+    {
+        ntError = PvfsSysOpen(
+                      &fd,
+                      pCreateContext->pszDiskFilename,
+                      unixFlags,
+                      (mode_t)gPvfsDriverConfig.CreateFileMode);
+
+    } while (ntError == STATUS_MORE_PROCESSING_REQUIRED);
     BAIL_ON_NT_STATUS(ntError);
 
     /* Perform preallocation is requested */
@@ -227,8 +272,6 @@ PvfsCreateFileDoSysOpen(
     {
         /* Unix Security */
 
-        pProcess = IoSecurityGetProcessInfo(pSecCtx);
-
         ntError = PvfsSysChown(
                       pCreateContext->pCcb,
                       pProcess->Uid,
@@ -264,6 +307,8 @@ PvfsCreateFileDoSysOpen(
     ntError = PvfsStoreCCB(pIrp->FileHandle, pCreateContext->pCcb);
     BAIL_ON_NT_STATUS(ntError);
 
+    PvfsInitializeZctSupport(pCreateContext->pCcb, pIrp->FileHandle);
+
     CreateResult = PvfsSetCreateResult(
                        Args.CreateDisposition,
                        pCreateContext->bFileExisted,
@@ -296,9 +341,19 @@ error:
 
     if (fd != -1)
     {
-        close(fd);
-    }
+        PSTR pszRemovePath = NULL;
 
+        /* Pick up where we started the pathname */
+
+        pszRemovePath = pCreateContext->pszDiskFilename ?
+                        pCreateContext->pszDiskFilename :
+                        pCreateContext->pCcb->pszFilename;
+
+        PvfsCleanupFailedCreate(
+            fd,
+            pszRemovePath,
+            !pCreateContext->bFileExisted);
+    }
 
     goto cleanup;
 }
@@ -323,6 +378,20 @@ PvfsCreateDirDoSysOpen(
     IO_MATCH_FILE_SPEC FileSpec = {0};
     WCHAR wszPattern[2] = {L'*', 0x0 };
     PIO_SECURITY_CONTEXT_PROCESS_INFORMATION pProcess = NULL;
+    PBOOLEAN pbEnableAbe = NULL;
+    ULONG ulEcpSize = 0;
+
+    pProcess = IoSecurityGetProcessInfo(pSecCtx);
+
+    /* Fail any create that requires setting the security but doesn't
+       have the Unix uid/gid information */
+
+    if ((pCreateContext->SetPropertyFlags & PVFS_SET_PROP_SECURITY) &&
+        (pProcess == NULL))
+    {
+        ntError = STATUS_ACCESS_DENIED;
+        BAIL_ON_NT_STATUS(ntError);
+    }
 
     /* Do the open() */
 
@@ -334,7 +403,9 @@ PvfsCreateDirDoSysOpen(
 
     if (!pCreateContext->bFileExisted)
     {
-        ntError = PvfsSysMkDir(pCreateContext->pszDiskFilename, 0700);
+        ntError = PvfsSysMkDir(
+                      pCreateContext->pszDiskFilename,
+                      (mode_t)gPvfsDriverConfig.CreateDirectoryMode);
         BAIL_ON_NT_STATUS(ntError);
     }
 
@@ -348,7 +419,36 @@ PvfsCreateDirDoSysOpen(
     pCreateContext->pCcb->pszFilename = pCreateContext->pszDiskFilename;
     pCreateContext->pszDiskFilename = NULL;
 
-    ntError = PvfsSysOpen(&fd, pCreateContext->pCcb->pszFilename, 0, 0);
+    ntError = IoRtlEcpListFind(
+                  pIrp->Args.Create.EcpList,
+                  SRV_ECP_TYPE_ABE,
+                  OUT_PPVOID(&pbEnableAbe),
+                  &ulEcpSize);
+    if (ntError != STATUS_NOT_FOUND)
+    {
+        BAIL_ON_NT_STATUS(ntError);
+
+        if (ulEcpSize != sizeof(BOOLEAN))
+        {
+            ntError = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(ntError);
+        }
+
+        if (*pbEnableAbe)
+        {
+            pCreateContext->pCcb->EcpFlags |= PVFS_ECP_ENABLE_ABE;
+        }
+    }
+
+    do
+    {
+        ntError = PvfsSysOpen(
+                      &fd,
+                      pCreateContext->pCcb->pszFilename,
+                      0,
+                      0);
+
+    } while (ntError == STATUS_MORE_PROCESSING_REQUIRED);
     BAIL_ON_NT_STATUS(ntError);
 
     /* Save our state */
@@ -368,8 +468,6 @@ PvfsCreateDirDoSysOpen(
     {
         /* Unix Security */
 
-        pProcess = IoSecurityGetProcessInfo(pSecCtx);
-
         ntError = PvfsSysChown(
                       pCreateContext->pCcb,
                       pProcess->Uid,
@@ -414,6 +512,8 @@ PvfsCreateDirDoSysOpen(
     ntError = PvfsStoreCCB(pIrp->FileHandle, pCreateContext->pCcb);
     BAIL_ON_NT_STATUS(ntError);
 
+    PvfsInitializeZctSupport(pCreateContext->pCcb, pIrp->FileHandle);
+
     CreateResult = PvfsSetCreateResult(
                        Args.CreateDisposition,
                        pCreateContext->bFileExisted,
@@ -446,9 +546,19 @@ error:
 
     if (fd != -1)
     {
-        close(fd);
-    }
+        PSTR pszRemovePath = NULL;
 
+        /* Pick up where we started the pathname */
+
+        pszRemovePath = pCreateContext->pszDiskFilename ?
+                        pCreateContext->pszDiskFilename :
+                        pCreateContext->pCcb->pszFilename;
+
+        PvfsCleanupFailedCreate(
+            fd,
+            pszRemovePath,
+            !pCreateContext->bFileExisted);
+    }
 
     goto cleanup;
 }
@@ -735,6 +845,86 @@ error:
 }
 
 
+/*****************************************************************************
+ ****************************************************************************/
+
+static
+VOID
+PvfsCleanupFailedCreate(
+    int fd,
+    PCSTR pszPath,
+    BOOLEAN bWasCreated
+    )
+{
+    NTSTATUS ntError = STATUS_SUCCESS;
+    PVFS_STAT Stat1 = {0};
+    PVFS_STAT Stat2 = {0};
+
+    /* Remove the file if it was created and the Dev/Inode matches our fd */
+
+    if (bWasCreated && (pszPath != NULL))
+    {
+        ntError = PvfsSysFstat(fd, &Stat1);
+        if (ntError == STATUS_SUCCESS)
+        {
+            ntError = PvfsSysStat(pszPath, &Stat2);
+            if (ntError == STATUS_SUCCESS)
+            {
+                if ((Stat1.s_dev == Stat1.s_dev) &&
+                    (Stat2.s_ino == Stat2.s_ino))
+                {
+                    ntError = PvfsSysRemove(pszPath);
+                }
+            }
+        }
+    }
+
+    ntError = PvfsSysClose(fd);
+
+    return;
+}
+
+/*****************************************************************************
+ ****************************************************************************/
+
+static
+NTSTATUS
+PvfsSetMaximalAccessMask(
+    PIRP pIrp
+    )
+{
+    NTSTATUS ntError = STATUS_SUCCESS;
+    PACCESS_MASK pulMaxAccessMask = NULL;
+    ULONG       ulEcpSize = 0;
+
+    ntError = IoRtlEcpListFind(
+                  pIrp->Args.Create.EcpList,
+                  SRV_ECP_TYPE_MAX_ACCESS,
+                  OUT_PPVOID(&pulMaxAccessMask),
+                  &ulEcpSize);
+    if (ntError != STATUS_NOT_FOUND)
+    {
+        BAIL_ON_NT_STATUS(ntError);
+
+        if (ulEcpSize != sizeof(ACCESS_MASK))
+        {
+            ntError = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(ntError);
+        }
+
+        // TODO: Fill in the correct maximal access
+        *pulMaxAccessMask = 0x1F01FF;
+    }
+    else
+    {
+        ntError = STATUS_SUCCESS;
+    }
+
+error:
+
+    return ntError;
+}
+
 
 /*
 local variables:
