@@ -102,10 +102,17 @@ PvfsDestroyFCBTable(
     VOID
     )
 {
-    /* Need to add tree traversal here and remove
-       data */
+    BOOLEAN bLocked = FALSE;
 
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bLocked, &gFcbTable.rwLock);
+
+    /* Need a traversal to close all open CCBs first.  Then free the
+       RBTree */
     LwRtlRBTreeFree(gFcbTable.pFcbTree);
+    gFcbTable.pFcbTree = NULL;
+
+    LWIO_UNLOCK_RWMUTEX(bLocked, &gFcbTable.rwLock);
+
     pthread_rwlock_destroy(&gFcbTable.rwLock);
 
     PVFS_ZERO_MEMORY(&gFcbTable);
@@ -150,10 +157,7 @@ PvfsFreeFCB(
 
         PVFS_FREE(&pFcb);
 
-#ifdef _PVFS_DEVELOPER_DEBUG
         InterlockedDecrement(&gPvfsFcbCount);
-#endif
-
     }
 
     return;
@@ -265,9 +269,7 @@ PvfsAllocateFCB(
 
     *ppFcb = pFcb;
 
-#ifdef _PVFS_DEVELOPER_DEBUG
-        InterlockedIncrement(&gPvfsFcbCount);
-#endif
+    InterlockedIncrement(&gPvfsFcbCount);
 
     ntError = STATUS_SUCCESS;
 
@@ -324,9 +326,11 @@ PvfsReferenceFCB(
  ******************************************************/
 
 
-static NTSTATUS
-PvfsSetLastWriteTime(
-    PPVFS_FCB pFcb
+static
+NTSTATUS
+PvfsFlushLastWriteTime(
+    PPVFS_FCB pFcb,
+    LONG64 LastWriteTime
     )
 {
     NTSTATUS ntError = STATUS_UNSUCCESSFUL;
@@ -342,12 +346,10 @@ PvfsSetLastWriteTime(
     BAIL_ON_NT_STATUS(ntError);
 
     ntError = PvfsSysUtime(pFcb->pszFilename,
-                           pFcb->LastWriteTime,
+                           LastWriteTime,
                            LastAccessTime);
     BAIL_ON_NT_STATUS(ntError);
 
-    pFcb->LastWriteTime = 0;
-
 cleanup:
     return ntError;
 
@@ -365,7 +367,7 @@ PvfsExecuteDeleteOnClose(
 
     /* Always reset the delete-on-close state to be safe */
 
-    pFcb->bDeleteOnClose = FALSE;
+    PvfsFcbSetPendingDelete(pFcb, FALSE);
 
     /* Verify we are deleting the file we think we are */
 
@@ -401,7 +403,6 @@ PvfsReleaseFCB(
     BOOLEAN bFcbControlLocked = FALSE;
     PVFS_STAT Stat = {0};
     PPVFS_FCB pFcb = NULL;
-    BOOLEAN bDeleteLocked = FALSE;
 
     /* Do housekeeping such as setting the last write time or
        honoring DeletOnClose when the CCB handle count reaches
@@ -423,10 +424,12 @@ PvfsReleaseFCB(
         ntError = PvfsSysStat(pFcb->pszFilename, &Stat);
         if (ntError == STATUS_SUCCESS)
         {
-            if (pFcb->LastWriteTime != 0)
+            LONG64 LastWriteTime = PvfsClearLastWriteTimeFCB(pFcb);
+
+            if (LastWriteTime != 0)
             {
 
-                ntError = PvfsSetLastWriteTime(pFcb);
+                ntError = PvfsFlushLastWriteTime(pFcb, LastWriteTime);
 
                 if (ntError == STATUS_SUCCESS)
                 {
@@ -438,8 +441,7 @@ PvfsReleaseFCB(
                 }
             }
 
-            LWIO_LOCK_MUTEX(bDeleteLocked, &pFcb->ControlBlock);
-            if (pFcb->bDeleteOnClose)
+            if (PvfsFcbIsPendingDelete(pFcb))
             {
                 ntError = PvfsExecuteDeleteOnClose(pFcb);
 
@@ -448,6 +450,7 @@ PvfsReleaseFCB(
                    any future opens. */
 
                 LWIO_LOCK_RWMUTEX_EXCLUSIVE(bTableLocked, &gFcbTable.rwLock);
+                LWIO_LOCK_MUTEX(bFcbControlLocked, &pFcb->ControlBlock);
 
                 if (!pFcb->bRemoved)
                 {
@@ -455,8 +458,8 @@ PvfsReleaseFCB(
                     pFcb->bRemoved = TRUE;
                 }
 
+                LWIO_UNLOCK_MUTEX(bFcbControlLocked, &pFcb->ControlBlock);
                 LWIO_UNLOCK_RWMUTEX(bTableLocked, &gFcbTable.rwLock);
-                LWIO_UNLOCK_MUTEX(bDeleteLocked, &pFcb->ControlBlock);
 
                 if (ntError == STATUS_SUCCESS)
                 {
@@ -473,7 +476,6 @@ PvfsReleaseFCB(
                         pFcb->pszFilename);
                 }
             }
-            LWIO_UNLOCK_MUTEX(bDeleteLocked, &pFcb->ControlBlock);
         }
     }
 
@@ -1104,13 +1106,16 @@ PvfsScheduleCancelPendingOp(
 {
     NTSTATUS ntError = STATUS_UNSUCCESSFUL;
     PPVFS_WORK_CONTEXT pWorkCtx = NULL;
+    PPVFS_IRP_CONTEXT pIrpCtx = NULL;
 
     BAIL_ON_INVALID_PTR(pIrpContext->pFcb, ntError);
 
+    pIrpCtx = PvfsReferenceIrpContext(pIrpContext);
+
     ntError = PvfsCreateWorkContext(
                   &pWorkCtx,
                   FALSE,
-                  pIrpContext,
+                  pIrpCtx,
                   (PPVFS_WORK_CONTEXT_CALLBACK)PvfsOplockCleanPendingOpQueue,
                   (PPVFS_WORK_CONTEXT_FREE_CTX)PvfsOplockCleanPendingOpFree);
     BAIL_ON_NT_STATUS(ntError);
@@ -1118,14 +1123,17 @@ PvfsScheduleCancelPendingOp(
     ntError = PvfsAddWorkItem(gpPvfsInternalWorkQueue, (PVOID)pWorkCtx);
     BAIL_ON_NT_STATUS(ntError);
 
-    pWorkCtx = NULL;
-
 cleanup:
-    PVFS_FREE(&pWorkCtx);
-
     return ntError;
 
 error:
+    if (pIrpCtx)
+    {
+        PvfsReleaseIrpContext(&pIrpCtx);
+    }
+
+    PvfsFreeWorkContext(&pWorkCtx);
+
     goto cleanup;
 }
 
@@ -1176,6 +1184,11 @@ cleanup:
         PvfsReleaseFCB(&pFcb);
     }
 
+    if (pIrpCtx)
+    {
+        PvfsReleaseIrpContext(&pIrpCtx);
+    }
+
     return ntError;
 
 error:
@@ -1280,6 +1293,7 @@ PvfsRenameFCB(
     PPVFS_FCB pOldParentFcb = NULL;
     PPVFS_FCB pExistingTargetFcb = NULL;
     BOOLEAN bExistingFcbLocked = FALSE;
+    BOOLEAN bTargetFcbControl = FALSE;
 
     LWIO_LOCK_RWMUTEX_EXCLUSIVE(bTableLocked, &gFcbTable.rwLock);
 
@@ -1323,7 +1337,13 @@ PvfsRenameFCB(
                Another reason to use the dev/inode pair instead if
                we could solve the "Create New File" issue.  */
 
+            LWIO_LOCK_MUTEX(bTargetFcbControl, &pFcb->ControlBlock);
             ntError = PvfsRemoveFCB(pFcb);
+            if (ntError == STATUS_SUCCESS)
+            {
+                pFcb->bRemoved = TRUE;
+            }
+            LWIO_UNLOCK_MUTEX(bTargetFcbControl, &pFcb->ControlBlock);
             BAIL_ON_NT_STATUS(ntError);
 
             PVFS_FREE(&pFcb->pszFilename);
@@ -1344,7 +1364,13 @@ PvfsRenameFCB(
             }
             LWIO_UNLOCK_RWMUTEX(bParentLinkLocked, &pFcb->rwParent);
 
+            LWIO_LOCK_MUTEX(bTargetFcbControl, &pFcb->ControlBlock);
             ntError = PvfsAddFCB(pFcb);
+            if (ntError == STATUS_SUCCESS)
+            {
+                pFcb->bRemoved = FALSE;
+            }
+            LWIO_UNLOCK_MUTEX(bTargetFcbControl, &pFcb->ControlBlock);
             BAIL_ON_NT_STATUS(ntError);
 
         LWIO_UNLOCK_RWMUTEX(bFcbLocked, &pFcb->rwFileName);
@@ -1445,6 +1471,44 @@ PvfsGetParentFCB(
     return pParent;
 }
 
+/*****************************************************************************
+ ****************************************************************************/
+
+LONG64
+PvfsClearLastWriteTimeFCB(
+    PPVFS_FCB pFcb
+    )
+{
+    BOOLEAN bLocked = FALSE;
+    LONG64 LastWriteTime = 0;
+
+    LWIO_LOCK_MUTEX(bLocked, &pFcb->ControlBlock);
+    LastWriteTime = pFcb->LastWriteTime;
+    pFcb->LastWriteTime = 0;
+    LWIO_UNLOCK_MUTEX(bLocked, &pFcb->ControlBlock);
+
+    return LastWriteTime;
+}
+
+/*****************************************************************************
+ ****************************************************************************/
+
+VOID
+PvfsSetLastWriteTimeFCB(
+    PPVFS_FCB pFcb,
+    LONG64 LastWriteTime
+    )
+{
+    BOOLEAN bLocked = FALSE;
+
+    LWIO_LOCK_MUTEX(bLocked, &pFcb->ControlBlock);
+    pFcb->LastWriteTime = LastWriteTime;
+    LWIO_UNLOCK_MUTEX(bLocked, &pFcb->ControlBlock);
+}
+
+
+
+
 /*
 local variables:
 mode: c
