@@ -108,6 +108,12 @@ PvfsOplockRequest(
         BAIL_ON_NT_STATUS(ntError);
     }
 
+    if (!gPvfsDriverConfig.EnableOplocks)
+    {
+        ntError = STATUS_OPLOCK_NOT_GRANTED;
+        BAIL_ON_NT_STATUS(ntError);
+    }
+
     /* Verify the oplock request type */
 
     switch(pOplockRequest->OplockRequestType)
@@ -1256,10 +1262,7 @@ PvfsScheduleCancelOplock(
     ntError = PvfsAddWorkItem(gpPvfsInternalWorkQueue, (PVOID)pWorkCtx);
     BAIL_ON_NT_STATUS(ntError);
 
-    pWorkCtx = NULL;
-
 cleanup:
-    PvfsFreeWorkContext(&pWorkCtx);
 
     return ntError;
 
@@ -1269,6 +1272,8 @@ error:
         PvfsReleaseIrpContext(&pIrpCtx);
     }
 
+    PvfsFreeWorkContext(&pWorkCtx);
+
     goto cleanup;
 }
 
@@ -1353,7 +1358,6 @@ cleanup:
         PvfsReleaseIrpContext(&pIrpCtx);
     }
 
-
     return ntError;
 
 error:
@@ -1374,54 +1378,6 @@ PvfsOplockCleanOplockFree(
 }
 
 
-/*****************************************************************************
- ****************************************************************************/
-
-NTSTATUS
-PvfsOplockCloseFile(
-    PPVFS_FCB pFcb,
-    PPVFS_CCB pCcb
-    )
-{
-    NTSTATUS ntError = STATUS_SUCCESS;
-    BOOLEAN bLocked = FALSE;
-    PPVFS_OPLOCK_RECORD pOplock = NULL;
-    PLW_LIST_LINKS pOplockLink = NULL;
-    PLW_LIST_LINKS pNextLink = NULL;
-
-    LWIO_LOCK_MUTEX(bLocked, &pFcb->mutexOplock);
-
-    pOplockLink = PvfsListTraverse(pFcb->pOplockList, NULL);
-
-    while (pOplockLink)
-    {
-        pOplock = LW_STRUCT_FROM_FIELD(
-                      pOplockLink,
-                      PVFS_OPLOCK_RECORD,
-                      OplockList);
-
-        pNextLink = PvfsListTraverse(pFcb->pOplockList, pOplockLink);
-
-        if (pOplock->pCcb == pCcb)
-        {
-            PvfsListRemoveItem(pFcb->pOplockList, pOplockLink);
-
-            pOplock->pIrpContext->pIrp->IoStatusBlock.Status = STATUS_FILE_CLOSED;
-
-            PvfsAsyncIrpComplete(pOplock->pIrpContext);
-
-            PvfsFreeOplockRecord(&pOplock);
-        }
-
-        pOplockLink = pNextLink;
-    }
-
-    LWIO_UNLOCK_MUTEX(bLocked, &pFcb->mutexOplock);
-
-    return ntError;
-}
-
-
 /*****************************************************************************
  ****************************************************************************/
 
