@@ -528,31 +528,34 @@ SMBSocketReceiveAndUnmarshall(
     NTSTATUS ntStatus = 0;
     uint32_t readLen = 0;
 
-    while (pPacket->bufferUsed < sizeof(NETBIOS_HEADER))
+    if (pPacket->bufferUsed < sizeof(NETBIOS_HEADER))
     {
-        ntStatus = SMBSocketRead(
-            pSocket,
-            pPacket->pRawBuffer + pPacket->bufferUsed,
-            sizeof(NETBIOS_HEADER) - pPacket->bufferUsed,
-            &readLen);
-        BAIL_ON_NT_STATUS(ntStatus);
-
-        if (readLen == 0)
+        while (pPacket->bufferUsed < sizeof(NETBIOS_HEADER))
         {
-            ntStatus = STATUS_END_OF_FILE;
+            ntStatus = SMBSocketRead(
+                pSocket,
+                pPacket->pRawBuffer + pPacket->bufferUsed,
+                sizeof(NETBIOS_HEADER) - pPacket->bufferUsed,
+                &readLen);
             BAIL_ON_NT_STATUS(ntStatus);
+
+            if (readLen == 0)
+            {
+                ntStatus = STATUS_END_OF_FILE;
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
+            pPacket->bufferUsed += readLen;
         }
 
-        pPacket->bufferUsed += readLen;
-    }
+        pPacket->pNetBIOSHeader = (NETBIOS_HEADER *) pPacket->pRawBuffer;
+        pPacket->pNetBIOSHeader->len = htonl(pPacket->pNetBIOSHeader->len);
 
-    pPacket->pNetBIOSHeader = (NETBIOS_HEADER *) pPacket->pRawBuffer;
-    pPacket->pNetBIOSHeader->len = htonl(pPacket->pNetBIOSHeader->len);
-
-    if ((uint64_t) pPacket->pNetBIOSHeader->len + (uint64_t) pPacket->bufferUsed > (uint64_t) pPacket->bufferLen)
-    {
-        ntStatus = STATUS_INVALID_NETWORK_RESPONSE;
-        BAIL_ON_NT_STATUS(ntStatus);
+        if ((uint64_t) pPacket->pNetBIOSHeader->len + sizeof(NETBIOS_HEADER) > (uint64_t) pPacket->bufferLen)
+        {
+            ntStatus = STATUS_INVALID_NETWORK_RESPONSE;
+            BAIL_ON_NT_STATUS(ntStatus);
+        }
     }
 
     while (pPacket->bufferUsed < pPacket->pNetBIOSHeader->len + sizeof(NETBIOS_HEADER))
@@ -575,10 +578,23 @@ SMBSocketReceiveAndUnmarshall(
 
     pPacket->pSMBHeader = (SMB_HEADER *) (pPacket->pRawBuffer + sizeof(NETBIOS_HEADER));
 
+    if ((PBYTE) pPacket->pSMBHeader + sizeof(SMB_HEADER) >= pPacket->pRawBuffer + pPacket->bufferUsed)
+    {
+           ntStatus = STATUS_INVALID_NETWORK_RESPONSE;
+           BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     if (SMBIsAndXCommand(SMB_LTOH8(pPacket->pSMBHeader->command)))
     {
         pPacket->pAndXHeader = (ANDX_HEADER *)
             (pPacket->pRawBuffer + sizeof(SMB_HEADER) + sizeof(NETBIOS_HEADER));
+
+        if (pPacket->pSMBHeader->error == 0 &&
+            (PBYTE) pPacket->pAndXHeader + sizeof(ANDX_HEADER) >= pPacket->pRawBuffer + pPacket->bufferUsed)
+        {
+            ntStatus = STATUS_INVALID_NETWORK_RESPONSE;
+            BAIL_ON_NT_STATUS(ntStatus);
+        }
     }
 
     pPacket->pParams = pPacket->pAndXHeader ?
@@ -604,8 +620,8 @@ SMBSocketReaderMain(
     NTSTATUS ntStatus = 0;
     PSMB_SOCKET pSocket = (PSMB_SOCKET) pContext;
     BOOLEAN bInLock = FALSE;
-    socklen_t len = 0;
-    long err = 0;
+    int err = 0;
+    SOCKLEN_T len = sizeof(err);
     static const LONG64 llConnectTimeout = 10 * 1000000000ll; // 10 sec
 
     if (WakeMask & LW_TASK_EVENT_CANCEL)
@@ -623,16 +639,26 @@ SMBSocketReaderMain(
 
     if (pSocket->state < RDR_SOCKET_STATE_NEGOTIATING)
     {
-        /* See if we are done connecting */
-        if (getsockopt(pSocket->fd, SOL_SOCKET, SO_ERROR, &err, &len) < 0)
+        /* If the socket is writable, we can safely call getsockopt() */
+        if (WakeMask & LW_TASK_EVENT_FD_WRITABLE)
         {
-            ntStatus = LwErrnoToNtStatus(errno);
-            BAIL_ON_NT_STATUS(ntStatus);
+            /* Get result of connect() */
+            if (getsockopt(pSocket->fd, SOL_SOCKET, SO_ERROR, &err, &len) < 0)
+            {
+                ntStatus = LwErrnoToNtStatus(errno);
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+        }
+        else
+        {
+            /* The connect() is still in progress */
+            err = EINPROGRESS;
         }
 
         switch (err)
         {
         case 0:
+            /* Notify thread initiating connect that it can now move on to negotiate */
             pSocket->state = RDR_SOCKET_STATE_NEGOTIATING;
             pthread_cond_broadcast(&pSocket->event);
             break;
@@ -708,7 +734,8 @@ error:
 
     if (ntStatus != STATUS_PENDING)
     {
-        SMBSocketInvalidate(pSocket, ntStatus);
+        LWIO_LOCK_MUTEX(bInLock, &pSocket->mutex);
+        SMBSocketInvalidate_InLock(pSocket, ntStatus);
         *pWaitMask = LW_TASK_EVENT_COMPLETE;
     }
 
@@ -734,6 +761,16 @@ SMBSocketFindAndSignalResponse(
     uint16_t uid = SMB_LTOH16(pPacket->pSMBHeader->uid);
     uint16_t tid = SMB_LTOH16(pPacket->pSMBHeader->tid);
     uint16_t mid = SMB_LTOH16(pPacket->pSMBHeader->mid);
+    ULONG ulFlags = pPacket->pSMBHeader->flags;
+
+    /* If the packet is not a response, drop it on the floor */
+    if (!(ulFlags & FLAG_RESPONSE))
+    {
+        SMBPacketRelease(
+            pSocket->hPacketAllocator,
+            pPacket);
+        goto cleanup;
+    }
 
     /* If any intermediate object has an error status, then the
        waiting thread has (or will soon) be awoken with an error
@@ -912,6 +949,12 @@ SMBSocketConnect(
             {
                 continue;
             }
+#endif
+#ifdef EAFNOSUPPORT
+            if (errno == EAFNOSUPPORT)
+            {
+                continue;
+            }
 #endif
             ntStatus = ErrnoToNtStatus(errno);
             BAIL_ON_NT_STATUS(ntStatus);
@@ -934,7 +977,7 @@ SMBSocketConnect(
     }
     BAIL_ON_NT_STATUS(ntStatus);
 
-    if (connect(fd, ai->ai_addr, ai->ai_addrlen) && errno != EINPROGRESS)
+    if (connect(fd, pCursor->ai_addr, pCursor->ai_addrlen) && errno != EINPROGRESS)
     {
         ntStatus = LwErrnoToNtStatus(errno);
         BAIL_ON_NT_STATUS(ntStatus);
@@ -944,7 +987,6 @@ SMBSocketConnect(
 
     pSocket->fd = fd;
     fd = -1;
-    memcpy(&pSocket->address, &ai->ai_addr, ai->ai_addrlen);
 
     /* Let the task wait for the connect() to complete before proceeding */
     LwRtlWakeTask(pSocket->pTask);
@@ -1001,7 +1043,15 @@ SMBSocketRead(
             case EINTR:
                 continue;
             case EAGAIN:
-                ntStatus = STATUS_PENDING;
+                if (totalRead == 0)
+                {
+                    ntStatus = STATUS_PENDING;
+                }
+                else
+                {
+                    /* Return a successful partial read */
+                    goto cleanup;
+                }
                 break;
             default:
                 ntStatus = ErrnoToNtStatus(errno);
