@@ -191,6 +191,7 @@ SMBGSSContextBuild(
         .length = GSS_MECH_NTLM_LEN,
         .elements = GSS_MECH_NTLM
     };
+    size_t sCopyServerChars = 0;
 
     dwError = LwRtlCStringAllocateFromWC16String(&pszServerName, pwszServerName);
     BAIL_ON_LWIO_ERROR(dwError);
@@ -218,11 +219,26 @@ SMBGSSContextBuild(
 
             SMBStrToUpper(pszDomainName);
 
+            sCopyServerChars = strlen(pszServerName);
+            if (sCopyServerChars > 0 &&
+                    pszServerName[sCopyServerChars - 1] == '.')
+            {
+                // Strip the trailing dot
+                sCopyServerChars --;
+            }
+
+            if (sCopyServerChars > INT_MAX)
+            {
+                dwError = STATUS_INTEGER_OVERFLOW;
+                BAIL_ON_LWIO_ERROR(dwError);
+            }
+
             dwError = SMBAllocateStringPrintf(
-                &pszTargetName,
-                "cifs/%s@%s",
-                pszServerName,
-                pszDomainName);
+                            &pszTargetName,
+                            "cifs/%.*s@%s",
+                            (int)sCopyServerChars,
+                            pszServerName,
+                            pszDomainName);
             BAIL_ON_LWIO_ERROR(dwError);
 
             inputNameBuffer.value = pszTargetName;
@@ -261,12 +277,17 @@ SMBGSSContextBuild(
                 usernameBuffer.value = pszUsername;
                 usernameBuffer.length = strlen(pszUsername);
 
-                dwMajorStatus = gss_import_name(
-                    (OM_uint32 *)&dwMinorStatus,
-                    &usernameBuffer,
-                    GSS_C_NT_USER_NAME,
-                    &pUsername);
-                BAIL_ON_SEC_ERROR(dwMajorStatus);
+                // If "" is passed in, that means to use anonymous
+                // authentication. gss_import_name fails on "" though
+                if (usernameBuffer.length)
+                {
+                    dwMajorStatus = gss_import_name(
+                        (OM_uint32 *)&dwMinorStatus,
+                        &usernameBuffer,
+                        GSS_C_NT_USER_NAME,
+                        &pUsername);
+                    BAIL_ON_SEC_ERROR(dwMajorStatus);
+                }
             }
 
             desiredMechs.count = 1;
@@ -430,13 +451,19 @@ SMBGSSContextNegotiate(
             break;
 
         case GSS_S_FAILURE:
-            if (dwMinorStatus == (DWORD) KRB5KRB_AP_ERR_SKEW)
+            switch (dwMinorStatus)
             {
-                dwError = LWIO_ERROR_CLOCK_SKEW;
-            }
-            else
-            {
-                dwError = LWIO_ERROR_GSS;
+                case ((DWORD) KRB5KRB_AP_ERR_SKEW):
+                    dwError = LWIO_ERROR_CLOCK_SKEW;
+                    break;
+                case ((DWORD) KRB5KDC_ERR_TGT_REVOKED):
+                    dwError = LW_STATUS_KDC_CERT_REVOKED;
+                    break;
+                case ((DWORD) KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN):
+                    dwError = STATUS_ACCESS_DENIED;
+                    break;
+                default:
+                    dwError = LWIO_ERROR_GSS;
             }
             BAIL_ON_LWIO_ERROR(dwError);
             break;
@@ -637,6 +664,7 @@ SMBGetServerDomain(
     char* pDot = NULL;
     PSTR pszDomain = NULL;
     PSTR pszNormal = NULL;
+    size_t sDomain = 0;
 
     pDot = strchr(pszServerName, '.');
 
@@ -654,6 +682,13 @@ SMBGetServerDomain(
     dwError = SMBAllocateString(strchr(pszNormal, '.') + 1, &pszDomain);
     BAIL_ON_LWIO_ERROR(dwError);
 
+    sDomain = strlen(pszDomain);
+    if (sDomain > 0 && pszDomain[sDomain - 1] == '.')
+    {
+        // Strip the trailing dot off
+        pszDomain[sDomain - 1] = 0;
+    }
+
     *ppszDomain = pszDomain;
 
 cleanup:
@@ -852,14 +887,14 @@ smb_display_status_1(
 #endif
                 LWIO_LOG_VERBOSE("GSS-API error calling %s: %d (%s)\n",
                         pszId, code,
-                        (char *)msg.value);
+                        SMB_SAFE_LOG_STRING((char *)msg.value));
                 break;
 
             default:
 
                 LWIO_LOG_ERROR("GSS-API error calling %s: %d (%s)\n",
                         pszId, code,
-                        (char *)msg.value);
+                        SMB_SAFE_LOG_STRING((char *)msg.value));
         }
 
         (void) gss_release_buffer(&min_stat, &msg);
