@@ -50,6 +50,48 @@
 
 #include "includes.h"
 
+static
+NTSTATUS
+SrvShareUpdateInfo0(
+    PSRV_SHARE_INFO pShareInfo,
+    PSHARE_INFO_0 pInfo0
+    );
+
+static
+NTSTATUS
+SrvShareUpdateInfo1(
+    PSRV_SHARE_INFO pShareInfo,
+    PSHARE_INFO_1 pInfo1
+    );
+
+static
+NTSTATUS
+SrvShareUpdateInfo2(
+    PSRV_SHARE_INFO pShareInfo,
+    PSHARE_INFO_2 pInfo2
+    );
+
+static
+NTSTATUS
+SrvShareUpdateInfo501(
+    PSRV_SHARE_INFO pShareInfo,
+    PSHARE_INFO_501 pInfo501
+    );
+
+static
+NTSTATUS
+SrvShareUpdateInfo502(
+    PSRV_SHARE_INFO pShareInfo,
+    PSHARE_INFO_502 pInfo502
+    );
+
+static
+NTSTATUS
+SrvShareUpdateInfo1005(
+    PSRV_SHARE_INFO  pShareInfo,
+    PSHARE_INFO_1005 pInfo1005
+    );
+
 NTSTATUS
 SrvShareDevCtlAdd(
     IN     PBYTE lpInBuffer,
@@ -66,12 +108,14 @@ SrvShareDevCtlAdd(
     PSHARE_INFO_2 pShareInfo2 = NULL;
     PSHARE_INFO_501 pShareInfo501 = NULL;
     PSHARE_INFO_502 pShareInfo502 = NULL;
+    PSHARE_INFO_1005 pShareInfo1005 = NULL;
     PWSTR pwszShareName = NULL;
     ULONG ulShareType = 0;
     PWSTR pwszComment = NULL;
     PWSTR pwszPath = NULL;
     PWSTR pwszPathLocal = NULL;
     PWSTR pwszShareType = NULL;
+    ULONG ulShareFlags = 0;
     ULONG ulSecDescLen = 0;
     PBYTE pSecDesc = NULL;
 
@@ -126,6 +170,10 @@ SrvShareDevCtlAdd(
         ulSecDescLen  = pShareInfo502->shi502_reserved;
         break;
 
+    case 1005:
+        pShareInfo1005 = pAddShareInfoParams->info.p1005;
+        ulShareFlags   = pShareInfo1005->shi1005_flags;
+
     default:
         ntStatus = STATUS_INVALID_PARAMETER;
         BAIL_ON_NT_STATUS(ntStatus);
@@ -152,7 +200,8 @@ SrvShareDevCtlAdd(
                         pwszComment,
                         pSecDesc,
                         ulSecDescLen,
-                        pwszShareType);
+                        pwszShareType,
+                        ulShareFlags);
 
 error:
 
@@ -233,6 +282,7 @@ SrvShareDevCtlEnum(
     PSHARE_INFO_2 p2 = NULL;
     PSHARE_INFO_501 p501 = NULL;
     PSHARE_INFO_502 p502 = NULL;
+    PSHARE_INFO_1005 p1005 = NULL;
 
     memset(&EnumShareInfoParamsOut, 0, sizeof(EnumShareInfoParamsOut));
 
@@ -336,7 +386,7 @@ SrvShareDevCtlEnum(
                 p501[i].shi501_netname         = pShareInfo->pwszName;
                 p501[i].shi501_type            = pShareInfo->service;
                 p501[i].shi501_remark          = pShareInfo->pwszComment;
-                p501[i].shi501_flags           = 0;
+                p501[i].shi501_flags           = pShareInfo->ulFlags;
             }
 
             EnumShareInfoParamsOut.info.p501 = p501;
@@ -367,14 +417,32 @@ SrvShareDevCtlEnum(
                 BAIL_ON_NT_STATUS(ntStatus);
 
                 p502[i].shi502_password            = NULL;
-                p502[i].shi502_reserved            = 0;
-                p502[i].shi502_security_descriptor = NULL;
+                p502[i].shi502_reserved            = pShareInfo->ulSecDescLen;
+                p502[i].shi502_security_descriptor = (PBYTE) pShareInfo->pSecDesc;
             }
 
             EnumShareInfoParamsOut.info.p502 = p502;
 
             break;
 
+        case 1005:
+
+            ntStatus = SrvAllocateMemory(
+                            sizeof(*p1005) * ulNumEntries,
+                            OUT_PPVOID(&p1005));
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            for (i = 0; i < ulNumEntries; i++)
+            {
+                PSRV_SHARE_INFO pShareInfo = ppShares[i];
+
+                p1005[i].shi1005_flags = pShareInfo->ulFlags;
+            }
+
+            EnumShareInfoParamsOut.info.p1005 = p1005;
+
+            break;
+
         default:
 
             ntStatus = STATUS_INVALID_LEVEL;
@@ -398,7 +466,7 @@ SrvShareDevCtlEnum(
     }
     else
     {
-        ntStatus = STATUS_MORE_ENTRIES;
+        ntStatus = STATUS_BUFFER_TOO_SMALL;
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
@@ -456,6 +524,10 @@ cleanup:
 
         SrvFreeMemory(p502);
     }
+    if (p1005)
+    {
+        SrvFreeMemory(p1005);
+    }
     if (pBuffer)
     {
         SrvFreeMemory(pBuffer);
@@ -494,11 +566,13 @@ SrvShareDevCtlGetInfo(
     SHARE_INFO_GETINFO_PARAMS GetShareInfoParamsOut;
     PWSTR pwszShareName = NULL;
     PSRV_SHARE_INFO pShareInfo = NULL;
+    BOOLEAN         bInLock = FALSE;
     PSHARE_INFO_0 p0 = NULL;
     PSHARE_INFO_1 p1 = NULL;
     PSHARE_INFO_2 p2 = NULL;
     PSHARE_INFO_501 p501 = NULL;
     PSHARE_INFO_502 p502 = NULL;
+    PSHARE_INFO_1005 p1005 = NULL;
 
     memset(&GetShareInfoParamsOut, 0, sizeof(GetShareInfoParamsOut));
 
@@ -519,6 +593,8 @@ SrvShareDevCtlGetInfo(
                         &pShareInfo);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    LWIO_LOCK_RWMUTEX_SHARED(bInLock, &pShareInfo->mutex);
+
     switch (ulLevel)
     {
         case 0:
@@ -584,7 +660,7 @@ SrvShareDevCtlGetInfo(
             p501->shi501_netname         = pShareInfo->pwszName;
             p501->shi501_type            = pShareInfo->service;
             p501->shi501_remark          = pShareInfo->pwszComment;
-            p501->shi501_flags           = 0;
+            p501->shi501_flags           = pShareInfo->ulFlags;
 
             GetShareInfoParamsOut.Info.p501 = p501;
 
@@ -617,6 +693,18 @@ SrvShareDevCtlGetInfo(
 
             break;
 
+        case 1005:
+            ntStatus = SrvAllocateMemory(
+                            sizeof(*p1005),
+                            OUT_PPVOID(&p1005));
+            BAIL_ON_NT_STATUS(ntStatus);
+
+
+            p1005->shi1005_flags = pShareInfo->ulFlags;
+
+            GetShareInfoParamsOut.Info.p1005 = p1005;
+            break;
+
         default:
 
             ntStatus = STATUS_INVALID_LEVEL;
@@ -639,7 +727,7 @@ SrvShareDevCtlGetInfo(
     }
     else
     {
-        ntStatus = STATUS_MORE_ENTRIES;
+        ntStatus = STATUS_BUFFER_TOO_SMALL;
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
@@ -647,6 +735,8 @@ SrvShareDevCtlGetInfo(
 
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
     if (pShareInfo) {
         SrvShareReleaseInfo(pShareInfo);
     }
@@ -684,6 +774,10 @@ cleanup:
 
         SrvFreeMemory(p502);
     }
+    if (p1005)
+    {
+        SrvFreeMemory(p1005);
+    }
     if (pBuffer)
     {
         SrvFreeMemory(pBuffer);
@@ -699,52 +793,6 @@ error:
     goto cleanup;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
-static
-NTSTATUS
-SrvValidateShareNameChange(
-    PSRV_SHARE_INFO pShareInfo,
-    PWSTR pwszNewName
-    );
-
-static
-NTSTATUS
-SrvShareUpdateInfo0(
-    PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_0 pInfo0
-    );
-
-static
-NTSTATUS
-SrvShareUpdateInfo1(
-    PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_1 pInfo1
-    );
-
-static
-NTSTATUS
-SrvShareUpdateInfo2(
-    PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_2 pInfo2
-    );
-
-static
-NTSTATUS
-SrvShareUpdateInfo501(
-    PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_501 pInfo501
-    );
-
-static
-NTSTATUS
-SrvShareUpdateInfo502(
-    PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_502 pInfo502
-    );
-
 NTSTATUS
 SrvShareDevCtlSetInfo(
     IN     PBYTE lpInBuffer,
@@ -757,7 +805,6 @@ SrvShareDevCtlSetInfo(
     PLWIO_SRV_SHARE_ENTRY_LIST pShareList = &gSMBSrvGlobals.shareList;
     PSHARE_INFO_SETINFO_PARAMS pSetShareInfoParamsIn = NULL;
     PSRV_SHARE_INFO pShareInfo = NULL;
-    BOOLEAN bShareLocked = FALSE;
 
     ntStatus = LwShareInfoUnmarshalSetParameters(
                    lpInBuffer,
@@ -771,55 +818,55 @@ SrvShareDevCtlSetInfo(
                    &pShareInfo);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bShareLocked, &pShareInfo->mutex);
-
     switch (pSetShareInfoParamsIn->dwInfoLevel)
     {
-    case 0:
-        ntStatus = SrvShareUpdateInfo0(
-                       pShareInfo,
-                       pSetShareInfoParamsIn->Info.p0);
-        break;
+        case 0:
+            ntStatus = SrvShareUpdateInfo0(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p0);
+            break;
 
-    case 1:
-        ntStatus = SrvShareUpdateInfo1(
-                       pShareInfo,
-                       pSetShareInfoParamsIn->Info.p1);
-        break;
+        case 1:
+            ntStatus = SrvShareUpdateInfo1(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p1);
+            break;
 
-    case 2:
-        ntStatus = SrvShareUpdateInfo2(
-                       pShareInfo,
-                       pSetShareInfoParamsIn->Info.p2);
-        break;
+        case 2:
+            ntStatus = SrvShareUpdateInfo2(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p2);
+            break;
 
-    case 501:
-        ntStatus = SrvShareUpdateInfo501(
-                       pShareInfo,
-                       pSetShareInfoParamsIn->Info.p501);
-        break;
+        case 501:
+            ntStatus = SrvShareUpdateInfo501(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p501);
+            break;
 
-    case 502:
-        ntStatus = SrvShareUpdateInfo502(
-                       pShareInfo,
-                       pSetShareInfoParamsIn->Info.p502);
-        break;
+        case 502:
+            ntStatus = SrvShareUpdateInfo502(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p502);
+            break;
 
-    default:
-        ntStatus = STATUS_INVALID_LEVEL;
-        break;
+        case 1005:
+            ntStatus = SrvShareUpdateInfo1005(
+                           pShareInfo,
+                           pSetShareInfoParamsIn->Info.p1005);
+            break;
+
+        default:
+            ntStatus = STATUS_INVALID_LEVEL;
+            break;
     }
     BAIL_ON_NT_STATUS(ntStatus);
 
-    LWIO_UNLOCK_RWMUTEX(bShareLocked, &pShareInfo->mutex);
-
     ntStatus = SrvShareUpdate(pShareList, pShareInfo);
     BAIL_ON_NT_STATUS(ntStatus);
 
 error:
 
-    LWIO_UNLOCK_RWMUTEX(bShareLocked, &pShareInfo->mutex);
-
     if (pSetShareInfoParamsIn) {
         SrvFreeMemory(pSetShareInfoParamsIn);
     }
@@ -831,109 +878,28 @@ error:
     return ntStatus;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
-static
-NTSTATUS
-SrvValidateShareNameChange(
-    PSRV_SHARE_INFO pShareInfo,
-    PWSTR pwszNewName
-    )
-{
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    PSRV_SHARE_INFO pNewShareInfo = NULL;
-
-    if (pwszNewName == NULL)
-    {
-        ntStatus = STATUS_INVALID_PARAMETER;
-        BAIL_ON_NT_STATUS(ntStatus);
-    }
-
-    /* This needs better locking to ensure that there is no race
-       window between the new name check and the update */
-
-    ntStatus = SrvShareFindByName(
-                   &gSMBSrvGlobals.shareList,
-                   pwszNewName,
-                   &pNewShareInfo);
-    if ((ntStatus == STATUS_SUCCESS) && (pShareInfo != pNewShareInfo))
-    {
-        ntStatus = STATUS_DUPLICATE_NAME;
-        BAIL_ON_NT_STATUS(ntStatus);
-    }
-    ntStatus = STATUS_SUCCESS;
-
-cleanup:
-
-    return ntStatus;
-
-error:
-
-    goto cleanup;
-}
-
-
-/***********************************************************************
- **********************************************************************/
-
 static
 NTSTATUS
 SrvShareUpdateInfo0(
     PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_0 pInfo0
+    PSHARE_INFO_0   pInfo0
     )
 {
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    SRV_SHARE_INFO NewShareInfo = *pShareInfo;
-
-    /* Sanity Check */
-
-    ntStatus = SrvValidateShareNameChange(pShareInfo, pInfo0->shi0_netname);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    /* Setup new share info */
-
-    ntStatus = SrvAllocateStringW(pInfo0->shi0_netname, &NewShareInfo.pwszName);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    /* Struct copy to original share */
-
-    SrvFreeMemory(pShareInfo->pwszName);
-    pShareInfo->pwszName = NULL;
-
-    *pShareInfo = NewShareInfo;
-
-    ntStatus = STATUS_SUCCESS;
-
-cleanup:
-
-    return ntStatus;
-
-error:
-
-    if (NewShareInfo.pwszName)
-    {
-        SrvFreeMemory(NewShareInfo.pwszName);
-    }
-
-    goto cleanup;
+    // They share name is immutable.
+    // So, there is nothing else to update.
+    return STATUS_SUCCESS;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
 static
 NTSTATUS
 SrvShareUpdateInfo1(
     PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_1 pInfo1
+    PSHARE_INFO_1   pInfo1
     )
 {
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    SRV_SHARE_INFO NewShareInfo = *pShareInfo;
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    PWSTR    pwszComment = NULL;
+    BOOLEAN  bInLock     = FALSE;
 
     /* Sanity Check */
 
@@ -943,67 +909,47 @@ SrvShareUpdateInfo1(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    ntStatus = SrvValidateShareNameChange(pShareInfo, pInfo1->shi1_netname);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    /* Setup New Share */
-
-    ntStatus = SrvAllocateStringW(pInfo1->shi1_netname, &NewShareInfo.pwszName);
-    BAIL_ON_NT_STATUS(ntStatus);
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pShareInfo->mutex);
 
     if (pInfo1->shi1_remark)
     {
-        ntStatus = SrvAllocateStringW(pInfo1->shi1_remark, &NewShareInfo.pwszComment);
+        ntStatus = SrvAllocateStringW(pInfo1->shi1_remark, &pwszComment);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    /* Copy to the original share using struct copy */
-
-    SrvFreeMemory(pShareInfo->pwszName);
-    pShareInfo->pwszName = NULL;
-
-    if (pShareInfo->pwszComment)
+    if (pwszComment && SMBWc16sCmp(pShareInfo->pwszComment, pwszComment))
     {
         SrvFreeMemory(pShareInfo->pwszComment);
-        pShareInfo->pwszComment = NULL;
+
+        pShareInfo->pwszComment = pwszComment;
+
+        pwszComment = NULL;
     }
 
-    *pShareInfo = NewShareInfo;
-
-    ntStatus = STATUS_SUCCESS;
-
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    SRV_SAFE_FREE_MEMORY(pwszComment);
+
     return ntStatus;
 
 error:
 
-    if (NewShareInfo.pwszName)
-    {
-        SrvFreeMemory(NewShareInfo.pwszName);
-    }
-
-    if (NewShareInfo.pwszComment)
-    {
-        SrvFreeMemory(NewShareInfo.pwszComment);
-    }
-
     goto cleanup;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
 static
 NTSTATUS
 SrvShareUpdateInfo2(
     PSRV_SHARE_INFO pShareInfo,
-    PSHARE_INFO_2 pInfo2
+    PSHARE_INFO_2   pInfo2
     )
 {
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    SRV_SHARE_INFO NewShareInfo = *pShareInfo;
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    PWSTR    pwszPath    = NULL;
+    PWSTR    pwszComment = NULL;
+    BOOLEAN  bInLock     = FALSE;
 
     /* Sanity Check */
 
@@ -1013,76 +959,56 @@ SrvShareUpdateInfo2(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    if ((pInfo2->shi2_path == NULL) ||
-        (LwRtlWC16StringNumChars(pInfo2->shi2_path) == 0))
+    if (IsNullOrEmptyString(pInfo2->shi2_path))
     {
         ntStatus = STATUS_INVALID_PARAMETER;
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    ntStatus = SrvValidateShareNameChange(pShareInfo, pInfo2->shi2_netname);
+    ntStatus = SrvShareMapFromWindowsPath(pInfo2->shi2_path, &pwszPath);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    /* Setup New Share */
-
-    ntStatus = SrvAllocateStringW(pInfo2->shi2_netname, &NewShareInfo.pwszName);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    ntStatus = SrvShareMapFromWindowsPath(pInfo2->shi2_path, &NewShareInfo.pwszPath);
-    BAIL_ON_NT_STATUS(ntStatus);
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pShareInfo->mutex);
 
     if (pInfo2->shi2_remark)
     {
-        ntStatus = SrvAllocateStringW(pInfo2->shi2_remark, &NewShareInfo.pwszComment);
+        ntStatus = SrvAllocateStringW(pInfo2->shi2_remark, &pwszComment);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    /* Copy to the original share using struct copy */
-
-    SrvFreeMemory(pShareInfo->pwszName);
-    pShareInfo->pwszName = NULL;
-
-    SrvFreeMemory(pShareInfo->pwszPath);
-    pShareInfo->pwszPath = NULL;
-
-    if (pShareInfo->pwszComment)
+    if (pwszPath && SMBWc16sCmp(pShareInfo->pwszPath, pwszPath))
     {
-        SrvFreeMemory(pShareInfo->pwszComment);
-        pShareInfo->pwszComment = NULL;
+        SrvFreeMemory(pShareInfo->pwszPath);
+
+        pShareInfo->pwszPath = pwszPath;
+
+        pwszPath = NULL;
     }
 
-    *pShareInfo = NewShareInfo;
+    if (pwszComment && SMBWc16sCmp(pShareInfo->pwszComment, pwszComment))
+    {
+        SrvFreeMemory(pShareInfo->pwszComment);
 
-    ntStatus = STATUS_SUCCESS;
+        pShareInfo->pwszComment = pwszComment;
+
+        pwszComment = NULL;
+    }
 
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    SRV_SAFE_FREE_MEMORY(pwszComment);
+
+    SRV_SAFE_FREE_MEMORY(pwszPath);
+
     return ntStatus;
 
 error:
 
-    if (NewShareInfo.pwszName)
-    {
-        SrvFreeMemory(NewShareInfo.pwszName);
-    }
-
-    if (NewShareInfo.pwszPath)
-    {
-        SrvFreeMemory(NewShareInfo.pwszPath);
-    }
-
-    if (NewShareInfo.pwszComment)
-    {
-        SrvFreeMemory(NewShareInfo.pwszComment);
-    }
-
     goto cleanup;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
 static
 NTSTATUS
 SrvShareUpdateInfo501(
@@ -1090,8 +1016,9 @@ SrvShareUpdateInfo501(
     PSHARE_INFO_501 pInfo501
     )
 {
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    SRV_SHARE_INFO NewShareInfo = *pShareInfo;
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    PWSTR    pwszComment = NULL;
+    BOOLEAN  bInLock     = FALSE;
 
     /* Sanity Check */
 
@@ -1101,58 +1028,36 @@ SrvShareUpdateInfo501(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    ntStatus = SrvValidateShareNameChange(pShareInfo, pInfo501->shi501_netname);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    /* Setup New Share */
-
-    ntStatus = SrvAllocateStringW(pInfo501->shi501_netname, &NewShareInfo.pwszName);
-    BAIL_ON_NT_STATUS(ntStatus);
-
     if (pInfo501->shi501_remark)
     {
-        ntStatus = SrvAllocateStringW(pInfo501->shi501_remark, &NewShareInfo.pwszComment);
+        ntStatus = SrvAllocateStringW(pInfo501->shi501_remark, &pwszComment);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    /* Copy to the original share using struct copy */
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pShareInfo->mutex);
 
-    SrvFreeMemory(pShareInfo->pwszName);
-    pShareInfo->pwszName = NULL;
-
-    if (pShareInfo->pwszComment)
+    if (pwszComment && SMBWc16sCmp(pShareInfo->pwszComment, pwszComment))
     {
         SrvFreeMemory(pShareInfo->pwszComment);
-        pShareInfo->pwszComment = NULL;
+
+        pShareInfo->pwszComment = pwszComment;
+
+        pwszComment = NULL;
     }
 
-    *pShareInfo = NewShareInfo;
-
-    ntStatus = STATUS_SUCCESS;
-
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    SRV_SAFE_FREE_MEMORY(pwszComment);
+
     return ntStatus;
 
 error:
 
-    if (NewShareInfo.pwszName)
-    {
-        SrvFreeMemory(NewShareInfo.pwszName);
-    }
-
-    if (NewShareInfo.pwszComment)
-    {
-        SrvFreeMemory(NewShareInfo.pwszComment);
-    }
-
     goto cleanup;
 }
 
-
-/***********************************************************************
- **********************************************************************/
-
 static
 NTSTATUS
 SrvShareUpdateInfo502(
@@ -1160,8 +1065,12 @@ SrvShareUpdateInfo502(
     PSHARE_INFO_502 pInfo502
     )
 {
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    SRV_SHARE_INFO NewShareInfo = *pShareInfo;
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    PWSTR    pwszPath    = NULL;
+    PWSTR    pwszComment = NULL;
+    BOOLEAN  bInLock     = FALSE;
+    BOOLEAN  bSecDescUpdated       = FALSE;
+    PSRV_SHARE_INFO pShareInfoCopy = NULL;
 
     /* Sanity Check */
 
@@ -1171,91 +1080,108 @@ SrvShareUpdateInfo502(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    if ((pInfo502->shi502_path == NULL) ||
-        (LwRtlWC16StringNumChars(pInfo502->shi502_path) == 0))
+    if (IsNullOrEmptyString(pInfo502->shi502_path))
     {
         ntStatus = STATUS_INVALID_PARAMETER;
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    ntStatus = SrvValidateShareNameChange(pShareInfo, pInfo502->shi502_netname);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    /* Setup New Share */
-
-    ntStatus = SrvAllocateStringW(pInfo502->shi502_netname, &NewShareInfo.pwszName);
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    ntStatus = SrvShareMapFromWindowsPath(pInfo502->shi502_path, &NewShareInfo.pwszPath);
+    ntStatus = SrvShareMapFromWindowsPath(pInfo502->shi502_path, &pwszPath);
     BAIL_ON_NT_STATUS(ntStatus);
 
     if (pInfo502->shi502_remark)
     {
-        ntStatus = SrvAllocateStringW(pInfo502->shi502_remark, &NewShareInfo.pwszComment);
+        ntStatus = SrvAllocateStringW(
+                        pInfo502->shi502_remark,
+                        &pwszComment);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
+    ntStatus = SrvShareDuplicateInfo(pShareInfo, &pShareInfoCopy);
+    BAIL_ON_NT_STATUS(ntStatus);
+
     if (pInfo502->shi502_security_descriptor)
     {
-        /* NULL the SDs since we don't own them to prevent side effecting
-           the pShareInfo */
-
-        NewShareInfo.pSecDesc = NULL;
-        NewShareInfo.ulSecDescLen = 0;
-        NewShareInfo.pAbsSecDesc = NULL;
-
         ntStatus = SrvShareSetSecurity(
-                       &NewShareInfo,
+                       pShareInfoCopy,
                        (PSECURITY_DESCRIPTOR_RELATIVE)pInfo502->shi502_security_descriptor,
                        pInfo502->shi502_reserved);
         BAIL_ON_NT_STATUS(ntStatus);
+
+        bSecDescUpdated = TRUE;
     }
 
-    /* Copy to the original share using struct copy */
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pShareInfo->mutex);
 
-    SrvFreeMemory(pShareInfo->pwszName);
-    pShareInfo->pwszName = NULL;
+    if (pwszPath && SMBWc16sCmp(pShareInfo->pwszPath, pwszPath))
+    {
+        SrvFreeMemory(pShareInfo->pwszPath);
 
-    SrvFreeMemory(pShareInfo->pwszPath);
-    pShareInfo->pwszPath = NULL;
+        pShareInfo->pwszPath = pwszPath;
+        pwszPath = NULL;
+    }
 
-    if (pShareInfo->pwszComment)
+    if (pwszComment && SMBWc16sCmp(pShareInfo->pwszComment, pwszComment))
     {
         SrvFreeMemory(pShareInfo->pwszComment);
-        pShareInfo->pwszComment = NULL;
+
+        pShareInfo->pwszComment = pwszComment;
+        pwszComment = NULL;
     }
 
-    SrvShareFreeSecurity(pShareInfo);
+    if (bSecDescUpdated)
+    {
+        SrvShareFreeSecurity(pShareInfo);
 
-    *pShareInfo = NewShareInfo;
+        pShareInfo->pSecDesc = pShareInfoCopy->pSecDesc;
+        pShareInfo->ulSecDescLen = pShareInfoCopy->ulSecDescLen;
+        pShareInfoCopy->pSecDesc = NULL;
 
-    ntStatus = STATUS_SUCCESS;
+        pShareInfo->pAbsSecDesc = pShareInfoCopy->pAbsSecDesc;
+        pShareInfoCopy->pAbsSecDesc = NULL;
+    }
 
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    if (pShareInfoCopy)
+    {
+        SrvShareReleaseInfo(pShareInfoCopy);
+    }
+
+    SRV_SAFE_FREE_MEMORY(pwszPath);
+
+    SRV_SAFE_FREE_MEMORY(pwszComment);
+
     return ntStatus;
 
 error:
 
-    if (NewShareInfo.pwszName)
-    {
-        SrvFreeMemory(NewShareInfo.pwszName);
-    }
-
-    if (NewShareInfo.pwszPath)
-    {
-        SrvFreeMemory(NewShareInfo.pwszPath);
-    }
-
-    if (NewShareInfo.pwszComment)
-    {
-        SrvFreeMemory(NewShareInfo.pwszComment);
-    }
-
     goto cleanup;
 }
 
 
+static
+NTSTATUS
+SrvShareUpdateInfo1005(
+    PSRV_SHARE_INFO  pShareInfo,
+    PSHARE_INFO_1005 pInfo1005
+    )
+{
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    BOOLEAN  bInLock     = FALSE;
+
+    LWIO_LOCK_RWMUTEX_EXCLUSIVE(bInLock, &pShareInfo->mutex);
+
+    pShareInfo->ulFlags = pInfo1005->shi1005_flags;
+
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    return ntStatus;
+}
+
+
 /*
 local variables:
 mode: c
