@@ -61,8 +61,15 @@ typedef UCHAR SMB_OPLOCK_LEVEL;
 
 #define SMB_CN_MAX_BUFFER_SIZE 0x00010000
 
+typedef ULONG SMB_SESSION_FLAG;
+
+#define SMB_SESSION_FLAG_GUEST         0x00000001
+#define SMB_SESSION_FLAG_NO_ENCRYPTION 0x00000002
+
 typedef VOID (*PFN_LWIO_SRV_FREE_OPLOCK_STATE)(HANDLE hOplockState);
-typedef VOID (*PFN_LWIO_SRV_FREE_BRL_STATE)(HANDLE hByteRangeLockState);
+typedef VOID (*PFN_LWIO_SRV_CANCEL_OPLOCK_STATE)(HANDLE hOplockState);
+typedef VOID (*PFN_LWIO_SRV_FREE_BRL_STATE_LIST)(HANDLE hBRLStateList);
+typedef VOID (*PFN_LWIO_SRV_CANCEL_ASYNC_STATE)(HANDLE hAsyncState);
 typedef VOID (*PFN_LWIO_SRV_FREE_ASYNC_STATE)(HANDLE hAsyncState);
 
 typedef struct __SMB2_FID
@@ -71,6 +78,63 @@ typedef struct __SMB2_FID
     ULONG64 ullVolatileId;
 } __attribute__((__packed__)) SMB2_FID, *PSMB2_FID;
 
+typedef enum
+{
+    SRV_RESOURCE_TYPE_UNKNOWN = 0,
+    SRV_RESOURCE_TYPE_CONNECTION,
+    SRV_RESOURCE_TYPE_FILE
+} SRV_RESOURCE_TYPE;
+
+typedef struct _SRV_RESOURCE_ATTRIBUTES
+{
+    SMB_PROTOCOL_VERSION protocolVersion;
+
+    ULONG        ulConnectionResourceId;
+
+    union
+    {
+
+        USHORT   usUid;
+        ULONG64  ullUid;
+
+    } sessionId;
+
+    union
+    {
+
+        USHORT   usTid;
+        ULONG    ulTid;
+
+    } treeId;
+
+    union
+    {
+
+        PUSHORT   pFid1;
+        PSMB2_FID pFid2;
+
+    } fileId;
+
+} SRV_RESOURCE_ATTRIBUTES, *PSRV_RESOURCE_ATTRIBUTES;
+
+typedef struct _SRV_RESOURCE
+{
+    LONG                     refCount;
+
+    ULONG                    ulResourceId;
+
+    SRV_RESOURCE_TYPE        resourceType;
+
+    PSRV_RESOURCE_ATTRIBUTES pAttributes;
+
+} SRV_RESOURCE, *PSRV_RESOURCE;
+
+typedef NTSTATUS (*PFN_ENUM_RESOURCES)(
+                    PSRV_RESOURCE pResource,
+                    PVOID         pUserData,
+                    PBOOLEAN      pbContinue
+                    );
+
 typedef struct _LWIO_ASYNC_STATE
 {
     pthread_rwlock_t               mutex;
@@ -82,7 +146,9 @@ typedef struct _LWIO_ASYNC_STATE
     USHORT                         usCommand;
 
     HANDLE                         hAsyncState;
-    PFN_LWIO_SRV_FREE_ASYNC_STATE  pfnFreeAsyncState;
+
+    PFN_LWIO_SRV_FREE_ASYNC_STATE   pfnFreeAsyncState;
+    PFN_LWIO_SRV_CANCEL_ASYNC_STATE pfnCancelAsyncState;
 
 } LWIO_ASYNC_STATE, *PLWIO_ASYNC_STATE;
 
@@ -95,6 +161,9 @@ typedef struct _LWIO_SRV_FILE
 
     USHORT                  fid;
 
+    SRV_RESOURCE            resource;
+    SRV_RESOURCE_ATTRIBUTES resourceAttrs;
+
     IO_FILE_HANDLE          hFile;
     PIO_FILE_NAME           pFilename; // physical path on server
     PWSTR                   pwszFilename; // requested path
@@ -105,13 +174,18 @@ typedef struct _LWIO_SRV_FILE
     FILE_CREATE_DISPOSITION createDisposition;
     FILE_CREATE_OPTIONS     createOptions;
 
-    UCHAR                          ucCurrentOplockLevel;
+    UCHAR                   ucCurrentOplockLevel;
 
-    HANDLE                         hOplockState;
-    PFN_LWIO_SRV_FREE_OPLOCK_STATE pfnFreeOplockState;
+    ULONG                   ulPermissions;
+    ULONG                   ulNumLocks;
+
+    HANDLE                           hOplockState;
+    PFN_LWIO_SRV_CANCEL_OPLOCK_STATE pfnCancelOplockState;
+    PFN_LWIO_SRV_FREE_OPLOCK_STATE   pfnFreeOplockState;
+
+    HANDLE                           hCancellableBRLStateList;
+    PFN_LWIO_SRV_FREE_BRL_STATE_LIST pfnFreeBRLStateList;
 
-    HANDLE                         hByteRangeLockState;
-    PFN_LWIO_SRV_FREE_BRL_STATE    pfnFreeByteRangeLockState;
     ULONG64                        ullLastFailedLockOffset;
 
 } LWIO_SRV_FILE, *PLWIO_SRV_FILE;
@@ -141,6 +215,9 @@ typedef struct _LWIO_SRV_FILE_2
 
     SMB2_FID                fid;
 
+    SRV_RESOURCE            resource;
+    SRV_RESOURCE_ATTRIBUTES resourceAttrs;
+
     IO_FILE_HANDLE          hFile;
     PIO_FILE_NAME           pFilename; // physical path on server
     PWSTR                   pwszFilename; // requested path
@@ -154,7 +231,10 @@ typedef struct _LWIO_SRV_FILE_2
     LWIO_SRV_SEARCH_SPACE_2  searchSpace;
     PLWIO_SRV_SEARCH_SPACE_2 pSearchSpace;
 
-    UCHAR                          ucCurrentOplockLevel;
+    UCHAR                    ucCurrentOplockLevel;
+
+    ULONG                    ulPermissions;
+    ULONG                    ulNumLocks;
 
     HANDLE                         hOplockState;
     PFN_LWIO_SRV_FREE_OPLOCK_STATE pfnFreeOplockState;
@@ -169,13 +249,19 @@ typedef struct _LWIO_SRV_TREE
     pthread_rwlock_t* pMutex;
 
     USHORT            tid;
+    USHORT            uid;
+    ULONG             ulConnectionResourceId;
 
     PSRV_SHARE_INFO   pShareInfo;
 
+    IO_FILE_HANDLE    hFile;
+
     PLWIO_SRV_FILE    lruFile[SRV_LRU_CAPACITY];
 
     PLWRTL_RB_TREE    pFileCollection;
 
+    ULONG             ulNumOpenFiles;
+
     PLWRTL_RB_TREE    pAsyncStateCollection;
 
     USHORT            nextAvailableFid;
@@ -190,13 +276,19 @@ typedef struct _LWIO_SRV_TREE_2
     pthread_rwlock_t* pMutex;
 
     ULONG             ulTid;
+    ULONG64           ullUid;
+    ULONG             ulConnectionResourceId;
 
     PSRV_SHARE_INFO   pShareInfo;
 
+    IO_FILE_HANDLE    hFile;
+
     PLWIO_SRV_FILE_2  lruFile[SRV_LRU_CAPACITY];
 
     PLWRTL_RB_TREE    pFileCollection;
 
+    ULONG             ulNumOpenFiles;
+
     ULONG64           ullNextAvailableFid;
 
 } LWIO_SRV_TREE_2, *PLWIO_SRV_TREE_2;
@@ -205,11 +297,15 @@ typedef struct _LWIO_SRV_SESSION
 {
     LONG              refcount;
 
-    pthread_rwlock_t   mutex;
-    pthread_rwlock_t*  pMutex;
+    pthread_rwlock_t  mutex;
+    pthread_rwlock_t* pMutex;
 
     USHORT            uid;
 
+    ULONG             ulConnectionResourceId;
+
+    LONG64            llBirthTime;
+
     PLWIO_SRV_TREE    lruTree[SRV_LRU_CAPACITY];
 
     PLWRTL_RB_TREE    pTreeCollection;
@@ -218,7 +314,11 @@ typedef struct _LWIO_SRV_SESSION
 
     USHORT            nextAvailableTid;
 
-    PSTR              pszClientPrincipalName;
+    PWSTR             pwszClientPrincipalName;
+
+    LONG64            llLastActivityTime;
+
+    ULONG             ulUserFlags;
 
     PIO_CREATE_SECURITY_CONTEXT   pIoSecurityContext;
 
@@ -228,11 +328,15 @@ typedef struct _LWIO_SRV_SESSION_2
 {
     LONG              refcount;
 
-    pthread_rwlock_t   mutex;
-    pthread_rwlock_t*  pMutex;
+    pthread_rwlock_t  mutex;
+    pthread_rwlock_t* pMutex;
 
     ULONG64           ullUid;
 
+    ULONG             ulConnectionResourceId;
+
+    LONG64            llBirthTime;
+
     PLWIO_SRV_TREE_2  lruTree[SRV_LRU_CAPACITY];
 
     PLWRTL_RB_TREE    pTreeCollection;
@@ -241,7 +345,11 @@ typedef struct _LWIO_SRV_SESSION_2
 
     ULONG             ulNextAvailableTid;
 
-    PSTR              pszClientPrincipalName;
+    PWSTR             pwszClientPrincipalName;
+
+    LONG64            llLastActivityTime;
+
+    ULONG             ulUserFlags;
 
     PIO_CREATE_SECURITY_CONTEXT   pIoSecurityContext;
 
@@ -255,12 +363,21 @@ typedef enum
     LWIO_SRV_CONN_STATE_INVALID
 } LWIO_SRV_CONN_STATE;
 
+typedef enum
+{
+    SRV_ZCT_STATE_UNKNOWN = 0,
+    SRV_ZCT_STATE_NOT_ZCT,
+    SRV_ZCT_STATE_IS_ZCT
+} SRV_ZCT_STATE, *PSRV_ZCT_STATE;
+
 typedef struct _SRV_PROPERTIES
 {
     USHORT  preferredSecurityMode;
     BOOLEAN bEncryptPasswords;
     BOOLEAN bEnableSecuritySignatures;
     BOOLEAN bRequireSecuritySignatures;
+    ULONG   ulZctReadThreshold;
+    ULONG   ulZctWriteThreshold;
     USHORT  MaxMpxCount;
     USHORT  MaxNumberVCs;
     ULONG   MaxBufferSize;
@@ -284,9 +401,20 @@ typedef struct _SRV_CLIENT_PROPERITES
 
 } SRV_CLIENT_PROPERTIES, *PSRV_CLIENT_PROPERTIES;
 
-typedef VOID (*PFN_LWIO_SRV_FREE_SOCKET_HANDLE)(HANDLE hSocket);
+typedef struct _SRV_SOCKET *PLWIO_SRV_SOCKET;
+typedef VOID (*PFN_SRV_SOCKET_FREE)(PLWIO_SRV_SOCKET pSocket);
+typedef VOID (*PFN_SRV_SOCKET_DISCONNECT)(PLWIO_SRV_SOCKET pSocket);
+typedef NTSTATUS (*PFN_SRV_SOCKET_GET_ADDRESS_BYTES)(PLWIO_SRV_SOCKET pSocket, PVOID* ppAddr, PULONG pulAddrLength);
+typedef VOID (*PFN_SRV_CONNECTION_IO_COMPLETE)(PVOID pContext, NTSTATUS Status);
+struct _SRV_EXEC_CONTEXT;
 
-typedef struct _LWIO_SRV_CONNECTION
+typedef struct _SRV_CONNECTION_SOCKET_DISPATCH {
+    PFN_SRV_SOCKET_FREE pfnFree;
+    PFN_SRV_SOCKET_DISCONNECT pfnDisconnect;
+    PFN_SRV_SOCKET_GET_ADDRESS_BYTES pfnGetAddressBytes;
+} SRV_CONNECTION_SOCKET_DISPATCH, *PSRV_CONNECTION_SOCKET_DISPATCH;
+
+typedef struct _SRV_CONNECTION
 {
     LONG                refCount;
 
@@ -295,8 +423,14 @@ typedef struct _LWIO_SRV_CONNECTION
 
     LWIO_SRV_CONN_STATE  state;
 
-    HANDLE                          hSocket;
-    PFN_LWIO_SRV_FREE_SOCKET_HANDLE pfnSocketFree;
+    // Immutable for lifetime of connection.
+    SRV_RESOURCE     resource;
+    struct sockaddr  clientAddress;
+    SOCKLEN_T        clientAddrLen;
+    struct sockaddr  serverAddress;
+    SOCKLEN_T        serverAddrLen;
+    PLWIO_SRV_SOCKET pSocket;
+    PSRV_CONNECTION_SOCKET_DISPATCH pSocketDispatch;
 
     SRV_PROPERTIES        serverProperties;
     SRV_CLIENT_PROPERTIES clientProperties;
@@ -317,18 +451,25 @@ typedef struct _LWIO_SRV_CONNECTION
 
     struct
     {
-        BOOLEAN         bReadHeader;
+        BOOLEAN         bNeedHeader;
         size_t          sNumBytesToRead;
         size_t          sOffset;
         PSMB_PACKET     pRequestPacket;
+        SRV_ZCT_STATE   zctState;
+        struct _SRV_EXEC_CONTEXT* pContinueExecContext;
+        PFN_SRV_CONNECTION_IO_COMPLETE pfnZctCallback;
+        PVOID           pZctCallbackContext;
+        ULONG           ulSkipBytes;
 
     } readerState;
 
     PBYTE               pSessionKey;
     ULONG               ulSessionKeyLength;
 
+    // Server-wide state
     PSRV_HOST_INFO             pHostinfo;
     PLWIO_SRV_SHARE_ENTRY_LIST pShareList;
+    PVOID                      pProtocolTransportDriverContext;
 
     HANDLE              hGssContext;
 
@@ -344,6 +485,7 @@ typedef struct _LWIO_SRV_CONNECTION
     };
 
     PLWRTL_RB_TREE      pSessionCollection;
+    ULONG64             ullSessionCount;
 
     ULONG64             ullNextAvailableAsyncId;
 
@@ -465,7 +607,6 @@ typedef struct _SRV_EXEC_CONTEXT
     PFN_SRV_PROTOCOL_FREE_EXEC_CONTEXT pfnFreeContext;
 
     PSMB_PACKET                        pSmbResponse;
-    ULONG                              ulNumDuplicates;
 
     PSMB_PACKET                        pInterimResponse;
 
@@ -473,6 +614,8 @@ typedef struct _SRV_EXEC_CONTEXT
 
     ULONG64                            ullAsyncId;
 
+    PSRV_STAT_INFO                     pStatInfo;
+
 } SRV_EXEC_CONTEXT, *PSRV_EXEC_CONTEXT;
 
 typedef struct _SRV_ELEMENTS_STATISTICS
@@ -572,13 +715,25 @@ SrvGssNegHints(
     ULONG *pulNegHintsLength
     );
 
+ULONG64
+SrvAsyncStateBuildId(
+    ULONG  ulPid,
+    USHORT usMid
+    );
+
 NTSTATUS
 SrvAsyncStateCreate(
-    ULONG64                       ullAsyncId,
-    USHORT                        usCommand,
-    HANDLE                        hAsyncState,
-    PFN_LWIO_SRV_FREE_ASYNC_STATE pfnFreeAsyncState,
-    PLWIO_ASYNC_STATE*            ppAsyncState
+    ULONG64                         ullAsyncId,
+    USHORT                          usCommand,
+    HANDLE                          hAsyncState,
+    PFN_LWIO_SRV_CANCEL_ASYNC_STATE pfnCancelAsyncState,
+    PFN_LWIO_SRV_FREE_ASYNC_STATE   pfnFreeAsyncState,
+    PLWIO_ASYNC_STATE*              ppAsyncState
+    );
+
+VOID
+SrvAsyncStateCancel(
+    PLWIO_ASYNC_STATE pAsyncState
     );
 
 PLWIO_ASYNC_STATE
@@ -593,13 +748,17 @@ SrvAsyncStateRelease(
 
 NTSTATUS
 SrvConnectionCreate(
-    HANDLE                          hSocket,
+    const struct sockaddr*          pClientAddress,
+    SOCKLEN_T                       clientAddrLen,
+    const struct sockaddr*          pServerAddress,
+    SOCKLEN_T                       serverAddrLen,
+    PLWIO_SRV_SOCKET                pSocket,
     HANDLE                          hPacketAllocator,
     HANDLE                          hGssContext,
     PLWIO_SRV_SHARE_ENTRY_LIST      pShareList,
     PSRV_PROPERTIES                 pServerProperties,
     PSRV_HOST_INFO                  pHostinfo,
-    PFN_LWIO_SRV_FREE_SOCKET_HANDLE pfnSocketFree,
+    PSRV_CONNECTION_SOCKET_DISPATCH pSocketDispatch,
     PLWIO_SRV_CONNECTION*           ppConnection
     );
 
@@ -608,6 +767,11 @@ SrvConnectionGetProtocolVersion(
     PLWIO_SRV_CONNECTION pConnection
     );
 
+SMB_PROTOCOL_VERSION
+SrvConnectionGetProtocolVersion_inlock(
+    PLWIO_SRV_CONNECTION pConnection
+    );
+
 NTSTATUS
 SrvConnectionSetProtocolVersion(
     PLWIO_SRV_CONNECTION pConnection,
@@ -632,6 +796,13 @@ SrvConnection2CreateSession(
     PLWIO_SRV_SESSION_2* ppSession
     );
 
+NTSTATUS
+SrvConnectionFindSession_inlock(
+    PLWIO_SRV_CONNECTION pConnection,
+    USHORT               uid,
+    PLWIO_SRV_SESSION*   ppSession
+    );
+
 NTSTATUS
 SrvConnectionFindSession(
     PLWIO_SRV_CONNECTION pConnection,
@@ -639,6 +810,13 @@ SrvConnectionFindSession(
     PLWIO_SRV_SESSION*   ppSession
     );
 
+NTSTATUS
+SrvConnection2FindSession_inlock(
+    PLWIO_SRV_CONNECTION pConnection,
+    ULONG64              ullUid,
+    PLWIO_SRV_SESSION_2* ppSession
+    );
+
 NTSTATUS
 SrvConnection2FindSession(
     PLWIO_SRV_CONNECTION pConnection,
@@ -646,12 +824,27 @@ SrvConnection2FindSession(
     PLWIO_SRV_SESSION_2* ppSession
     );
 
+NTSTATUS
+SrvConnectionGetSessionCount(
+    PLWIO_SRV_CONNECTION pConnection,
+    PWSTR                pwszUsername,
+    PULONG64             pullSessionCount
+    );
+
+NTSTATUS
+SrvConnection2GetSessionCount(
+    PLWIO_SRV_CONNECTION pConnection,
+    PWSTR                pwszUsername,
+    PULONG64             pullSessionCount
+    );
+
 NTSTATUS
 SrvConnection2CreateAsyncState(
-    PLWIO_SRV_CONNECTION          pConnection,
-    USHORT                        usCommand,
-    PFN_LWIO_SRV_FREE_ASYNC_STATE pfnFreeAsyncState,
-    PLWIO_ASYNC_STATE*            ppAsyncState
+    PLWIO_SRV_CONNECTION            pConnection,
+    USHORT                          usCommand,
+    PFN_LWIO_SRV_CANCEL_ASYNC_STATE pfnCancelAsyncState,
+    PFN_LWIO_SRV_FREE_ASYNC_STATE   pfnFreeAsyncState,
+    PLWIO_ASYNC_STATE*              ppAsyncState
     );
 
 NTSTATUS
@@ -679,6 +872,12 @@ SrvConnectionGetNamedPipeSessionKey(
     PIO_ECP_LIST        pEcpList
     );
 
+NTSTATUS
+SrvConnectionDeleteSessions(
+    PLWIO_SRV_CONNECTION pConnection,
+    PWSTR                pwszUsername
+    );
+
 LWIO_SRV_CONN_STATE
 SrvConnectionGetState(
     PLWIO_SRV_CONNECTION pConnection
@@ -722,6 +921,16 @@ SrvConnectionSetState(
     LWIO_SRV_CONN_STATE  connState
     );
 
+BOOLEAN
+SrvConnectionIsSigningActive(
+    PLWIO_SRV_CONNECTION pConnection
+    );
+
+BOOLEAN
+SrvConnectionIsSigningActive_inlock(
+    PLWIO_SRV_CONNECTION pConnection
+    );
+
 NTSTATUS
 SrvSessionCreate(
     USHORT            uid,
@@ -748,6 +957,37 @@ SrvSessionCreateTree(
     PLWIO_SRV_TREE*   ppTree
     );
 
+NTSTATUS
+SrvSessionSetPrincipalName(
+    PLWIO_SRV_SESSION pSession,
+    PCSTR             pszClientPrincipal
+    );
+
+NTSTATUS
+SrvSessionCheckPrincipal(
+    PLWIO_SRV_SESSION pSession,
+    PCWSTR            pwszClientPrincipal,
+    PBOOLEAN          pbIsMatch
+    );
+
+NTSTATUS
+SrvSessionGetPrincipalName(
+    PLWIO_SRV_SESSION pSession,
+    PWSTR*            ppwszClientPrincipal
+    );
+
+VOID
+SrvSessionSetUserFlags(
+    PLWIO_SRV_SESSION pSession,
+    ULONG Flags
+    );
+
+NTSTATUS
+SrvSessionGetFileCount(
+    PLWIO_SRV_SESSION pSession,
+    PULONG64          pullFileCount
+    );
+
 PLWIO_SRV_SESSION
 SrvSessionAcquire(
     PLWIO_SRV_SESSION pSession
@@ -789,6 +1029,38 @@ SrvSession2CreateTree(
     PLWIO_SRV_TREE_2*   ppTree
     );
 
+NTSTATUS
+SrvSession2SetPrincipalName(
+    PLWIO_SRV_SESSION_2 pSession,
+    PCSTR               pszClientPrincipal
+    );
+
+NTSTATUS
+SrvSession2CheckPrincipal(
+    PLWIO_SRV_SESSION_2 pSession,
+    PCWSTR              pwszClientPrincipal,
+    PBOOLEAN            pbIsMatch
+    );
+
+NTSTATUS
+SrvSession2GetPrincipalName(
+    PLWIO_SRV_SESSION_2 pSession,
+    PWSTR*              ppwszClientPrincipal
+    );
+
+VOID
+SrvSession2SetUserFlags(
+    PLWIO_SRV_SESSION_2 pSession,
+    ULONG Flags
+    );
+
+
+NTSTATUS
+SrvSession2GetFileCount(
+    PLWIO_SRV_SESSION_2 pSession,
+    PULONG64            pullFileCount
+    );
+
 PLWIO_SRV_SESSION_2
 SrvSession2Acquire(
     PLWIO_SRV_SESSION_2 pSession
@@ -939,25 +1211,38 @@ SrvTree2Rundown(
     PLWIO_SRV_TREE_2 pTree
     );
 
+NTSTATUS
+SrvIoSecCreateSecurityContext(
+    OUT PIO_CREATE_SECURITY_CONTEXT* SecurityContext,
+    OUT PBOOLEAN pbLoggedInAsGuest,
+    IN LW_MAP_SECURITY_GSS_CONTEXT hGssContext,
+    IN PCSTR pszUsername
+    );
+
 NTSTATUS
 SrvIoCreateFile(
-    IN PSRV_SHARE_INFO pShareInfo,
-    OUT PIO_FILE_HANDLE pFileHandle,
-    IN OUT OPTIONAL PIO_ASYNC_CONTROL_BLOCK pAsyncControlBlock,
-    OUT PIO_STATUS_BLOCK pIoStatusBlock,
-    IN PIO_CREATE_SECURITY_CONTEXT pSecurityContext,
-    IN PIO_FILE_NAME pFileName,
-    IN OPTIONAL PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,
-    IN OPTIONAL PVOID pSecurityQualityOfService,
-    IN ACCESS_MASK DesiredAccess,
-    IN OPTIONAL LONG64 AllocationSize,
-    IN FILE_ATTRIBUTES FileAttributes,
-    IN FILE_SHARE_FLAGS ShareAccess,
-    IN FILE_CREATE_DISPOSITION CreateDisposition,
-    IN FILE_CREATE_OPTIONS CreateOptions,
-    IN OPTIONAL PVOID pEaBuffer,
-    IN ULONG EaLength,
-    IN OUT PIO_ECP_LIST* ppEcpList
+    PSRV_SHARE_INFO               pShareInfo,              /* IN              */
+    PIO_FILE_HANDLE               pFileHandle,             /*    OUT          */
+    PIO_ASYNC_CONTROL_BLOCK       pAsyncControlBlock,      /* IN OUT OPTIONAL */
+    PIO_STATUS_BLOCK              pIoStatusBlock,          /*    OUT          */
+    PIO_CREATE_SECURITY_CONTEXT   pSecurityContext,        /* IN              */
+    PIO_FILE_NAME                 pFileName,               /* IN              */
+    PSECURITY_DESCRIPTOR_RELATIVE pSecurityDescriptor,     /* IN     OPTIONAL */
+    PVOID                         pSecurityQOS,            /* IN     OPTIONAL */
+    ACCESS_MASK                   DesiredAccess,           /* IN              */
+    LONG64                        AllocationSize,          /* IN     OPTIONAL */
+    FILE_ATTRIBUTES               FileAttributes,          /* IN              */
+    FILE_SHARE_FLAGS              ShareAccess,             /* IN              */
+    FILE_CREATE_DISPOSITION       CreateDisposition,       /* IN              */
+    FILE_CREATE_OPTIONS           CreateOptions,           /* IN              */
+    PVOID                         pEaBuffer,               /* IN     OPTIONAL */
+    ULONG                         EaLength,                /* IN              */
+    PIO_ECP_LIST                  pEcpList                 /* IN              */
+    );
+
+NTSTATUS
+SrvIoPrepareAbeEcpList(
+    PIO_ECP_LIST pEcpList    /* IN */
     );
 
 NTSTATUS
@@ -977,9 +1262,10 @@ SrvFileCreate(
 
 NTSTATUS
 SrvFileSetOplockState(
-    PLWIO_SRV_FILE                 pFile,
-    HANDLE                         hOplockState,
-    PFN_LWIO_SRV_FREE_OPLOCK_STATE pfnReleaseOplockState
+    PLWIO_SRV_FILE                   pFile,
+    HANDLE                           hOplockState,
+    PFN_LWIO_SRV_CANCEL_OPLOCK_STATE pfnCancelOplockState,
+    PFN_LWIO_SRV_FREE_OPLOCK_STATE   pfnFreeOplockState
     );
 
 HANDLE
@@ -1014,6 +1300,16 @@ SrvFileGetLastFailedLockOffset(
     PLWIO_SRV_FILE pFile
     );
 
+VOID
+SrvFileRegisterLock(
+    PLWIO_SRV_FILE pFile
+    );
+
+VOID
+SrvFileRegisterUnlock(
+    PLWIO_SRV_FILE pFile
+    );
+
 PLWIO_SRV_FILE
 SrvFileAcquire(
     PLWIO_SRV_FILE pFile
@@ -1072,6 +1368,16 @@ SrvFile2GetOplockLevel(
     PLWIO_SRV_FILE_2 pFile
     );
 
+VOID
+SrvFile2RegisterLock(
+    PLWIO_SRV_FILE_2 pFile
+    );
+
+VOID
+SrvFile2RegisterUnlock(
+    PLWIO_SRV_FILE_2 pFile
+    );
+
 PLWIO_SRV_FILE_2
 SrvFile2Acquire(
     PLWIO_SRV_FILE_2 pFile
@@ -1103,6 +1409,7 @@ SrvFinderBuildSearchPath(
 
 NTSTATUS
 SrvFinderCreateSearchSpace(
+    IN  IO_FILE_HANDLE  hRootFileHandle,
     IN  PSRV_SHARE_INFO pShareInfo,
     IN  PIO_CREATE_SECURITY_CONTEXT pIoSecurityContext,
     IN  HANDLE         hFinderRepository,
@@ -1163,6 +1470,11 @@ SrvReleaseExecContextHandle(
    IN HANDLE hExecContext
    );
 
+PSRV_EXEC_CONTEXT
+SrvAcquireExecContext(
+   PSRV_EXEC_CONTEXT pContext
+   );
+
 VOID
 SrvReleaseExecContext(
    IN PSRV_EXEC_CONTEXT pContext
@@ -1178,11 +1490,43 @@ SrvElementsGetShareNameEcpEnabled(
     VOID
     );
 
+BOOLEAN
+SrvElementsGetClientAddressEcpEnabled(
+    VOID
+    );
+
 NTSTATUS
 SrvElementsGetStats(
     PSRV_ELEMENTS_STATISTICS pStats
     );
 
+NTSTATUS
+SrvElementsRegisterResource(
+    PSRV_RESOURCE pResource,     /* IN OUT          */
+    PULONG        pulResourceId  /* IN OUT OPTIONAL */
+    );
+
+NTSTATUS
+SrvElementsFindResource(
+    ULONG              ulResourceId,
+    SRV_RESOURCE_TYPE  resourceType,
+    PFN_ENUM_RESOURCES pfnEnumResourcesCB,
+    PVOID              pUserData
+    );
+
+NTSTATUS
+SrvElementsEnumResources(
+    SRV_RESOURCE_TYPE  resourceType,
+    PFN_ENUM_RESOURCES pfnEnumResourcesCB,
+    PVOID              pUserData
+    );
+
+NTSTATUS
+SrvElementsUnregisterResource(
+    ULONG          ulResourceId, /* IN              */
+    PSRV_RESOURCE* ppResource    /*    OUT OPTIONAL */
+    );
+
 NTSTATUS
 SrvElementsResetStats(
     VOID
@@ -1194,3 +1538,14 @@ SrvElementsShutdown(
     );
 
 #endif /* __ELEMENTSAPI_H__ */
+
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
