@@ -143,6 +143,9 @@ SrvProcessNTCreateAndX(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -197,7 +200,7 @@ SrvProcessNTCreateAndX(
                             pCreateState->pRequestHeader->createOptions,
                             NULL, /* EA Buffer */
                             0,    /* EA Length */
-                            &pCreateState->pEcpList);
+                            pCreateState->pEcpList);
             BAIL_ON_NT_STATUS(ntStatus);
 
             SrvReleaseCreateStateAsync(pCreateState); // completed synchronously
@@ -255,23 +258,28 @@ SrvProcessNTCreateAndX(
             ntStatus = pCreateState->ioStatusBlock.Status;
             BAIL_ON_NT_STATUS(ntStatus);
 
-            if (!pCreateState->pFile->hByteRangeLockState)
+            if (!pCreateState->pFile->hCancellableBRLStateList)
             {
-                PSRV_PENDING_LOCK_STATE_LIST pPendingLockStateList = NULL;
+                PSRV_BYTE_RANGE_LOCK_STATE_LIST pBRLStateList = NULL;
 
-                ntStatus = SrvCreatePendingLockStateList(&pPendingLockStateList);
+                ntStatus = SrvCreatePendingLockStateList(&pBRLStateList);
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                pCreateState->pFile->hByteRangeLockState =
-                                (HANDLE)pPendingLockStateList;
+                pCreateState->pFile->hCancellableBRLStateList =
+                                (HANDLE)pBRLStateList;
 
-                pCreateState->pFile->pfnFreeByteRangeLockState =
+                pCreateState->pFile->pfnFreeBRLStateList =
                                 &SrvFreePendingLockStateListHandle;
             }
 
             ntStatus = SrvBuildNTCreateResponse_inlock(pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvElementsRegisterResource(
+                            &pCreateState->pFile->resource,
+                            NULL);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             pCreateState->stage = SRV_CREATE_STAGE_SMB_V1_DONE;
 
             // intentional fall through
@@ -280,6 +288,19 @@ SrvProcessNTCreateAndX(
 
             pCreateState->bRemoveFileFromTree = FALSE;
 
+            if (pCreateState->pRequestHeader->desiredAccess & FILE_READ_DATA)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_READ;
+            }
+            if (pCreateState->pRequestHeader->desiredAccess & FILE_WRITE_DATA)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_WRITE;
+            }
+            if (pCreateState->ulCreateAction == FILE_CREATED)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_CREATE;
+            }
+
             pCtxSmb1->pFile = SrvFileAcquire(pCreateState->pFile);
 
             break;
@@ -517,14 +538,19 @@ SrvRequestCreateXOplocks(
 
             case STATUS_PENDING:
 
+                InterlockedIncrement(&pOplockState->refCount);
+
                 ntStatus = SrvFileSetOplockState(
                                pCreateState->pFile,
                                pOplockState,
+                               &SrvCancelOplockStateHandle,
                                &SrvReleaseOplockStateHandle);
+                if (ntStatus != STATUS_SUCCESS)
+                {
+                    InterlockedDecrement(&pOplockState->refCount);
+                }
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                InterlockedIncrement(&pOplockState->refCount);
-
                 SrvFileSetOplockLevel(
                         pCreateState->pFile,
                         pOplockCursor->oplockLevel);
@@ -740,7 +766,6 @@ SrvBuildCreateState(
     PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol   = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1       = pCtxProtocol->pSmb1Context;
     PSRV_CREATE_STATE_SMB_V1   pCreateState   = NULL;
-    BOOLEAN                    bTreeInLock    = FALSE;
 
     ntStatus = SrvAllocateMemory(
                     sizeof(SRV_CREATE_STATE_SMB_V1),
@@ -763,9 +788,6 @@ SrvBuildCreateState(
 
     if (pRequestHeader->rootDirectoryFid != 0)
     {
-        wchar16_t wszFwdSlash[]  = {'/',  0};
-        wchar16_t wszBackSlash[] = {'\\', 0};
-
         ntStatus = SrvTreeFindFile(
                         pCreateState->pTree,
                         pRequestHeader->rootDirectoryFid,
@@ -774,6 +796,9 @@ SrvBuildCreateState(
 
         if (pwszFilename && *pwszFilename)
         {
+            wchar16_t wszFwdSlash[]  = {'/',  0};
+            wchar16_t wszBackSlash[] = {'\\', 0};
+
             if ((*pwszFilename == wszFwdSlash[0]) ||
                 (*pwszFilename == wszBackSlash[0]))
             {
@@ -781,15 +806,11 @@ SrvBuildCreateState(
                 BAIL_ON_NT_STATUS(ntStatus);
             }
 
-            LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
             ntStatus = SrvBuildFilePath(
                             NULL, /* relative path */
                             pwszFilename,
                             &pCreateState->pFilename->FileName);
             BAIL_ON_NT_STATUS(ntStatus);
-
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->mutex);
         }
 
         pCreateState->pFilename->RootFileHandle =
@@ -797,15 +818,11 @@ SrvBuildCreateState(
     }
     else
     {
-        LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
-        ntStatus = SrvBuildFilePath(
-                        pCtxSmb1->pTree->pShareInfo->pwszPath,
+        ntStatus = SrvBuildTreeRelativePath(
+                        pCtxSmb1->pTree,
                         pwszFilename,
-                        &pCreateState->pFilename->FileName);
+                        pCreateState->pFilename);
         BAIL_ON_NT_STATUS(ntStatus);
-
-        LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->mutex);
     }
 
     pCreateState->pwszFilename = pwszFilename;
@@ -837,8 +854,6 @@ SrvBuildCreateState(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
     return ntStatus;
 
 error:
