@@ -99,7 +99,6 @@ SrvProcessDeleteDirectory(
     PLWIO_SRV_SESSION           pSession     = NULL;
     PLWIO_SRV_TREE              pTree        = NULL;
     BOOLEAN                     bInLock      = FALSE;
-    BOOLEAN                     bTreeInLock  = FALSE;
     PSRV_DELETEDIR_STATE_SMB_V1 pDeletedirState = NULL;
 
     pDeletedirState = (PSRV_DELETEDIR_STATE_SMB_V1)pCtxSmb1->hState;
@@ -124,6 +123,9 @@ SrvProcessDeleteDirectory(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -162,18 +164,12 @@ SrvProcessDeleteDirectory(
     {
         case SRV_DELETEDIR_STAGE_SMB_V1_INITIAL:
 
-            LWIO_LOCK_RWMUTEX_SHARED(   bTreeInLock,
-                                        &pCtxSmb1->pTree->pShareInfo->mutex);
-
-            ntStatus = SrvBuildFilePath(
-                            pCtxSmb1->pTree->pShareInfo->pwszPath,
+            ntStatus = SrvBuildTreeRelativePath(
+                            pCtxSmb1->pTree,
                             pDeletedirState->pwszPathFragment,
-                            &pDeletedirState->fileName.FileName);
+                            &pDeletedirState->fileName);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                                &pCtxSmb1->pTree->pShareInfo->mutex);
-
             pDeletedirState->stage = SRV_DELETEDIR_STAGE_SMB_V1_ATTEMPT_SET_INFO;
 
             SrvPrepareDeletedirStateAsync(pDeletedirState, pExecContext);
@@ -196,7 +192,7 @@ SrvProcessDeleteDirectory(
                             FILE_DIRECTORY_FILE,
                             NULL,
                             0,
-                            &pDeletedirState->pEcpList);
+                            pDeletedirState->pEcpList);
             BAIL_ON_NT_STATUS(ntStatus);
 
             SrvReleaseDeletedirStateAsync(pDeletedirState);
@@ -233,8 +229,6 @@ SrvProcessDeleteDirectory(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->pShareInfo->mutex);
-
     if (pTree)
     {
         SrvTreeRelease(pTree);
