@@ -105,8 +105,6 @@ SrvProcessDelete(
     PSRV_MESSAGE_SMB_V1        pSmbRequest  = &pCtxSmb1->pRequests[iMsg];
     PSRV_DELETE_STATE_SMB_V1   pDeleteState = NULL;
     BOOLEAN                    bInLock      = FALSE;
-    BOOLEAN                    bTreeInLock  = FALSE;
-    PWSTR                      pwszFilesystemPath = NULL;
 
     pDeleteState = (PSRV_DELETE_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -167,6 +165,11 @@ SrvProcessDelete(
                             &pDeleteState->pSession);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvSetStatSessionInfo(
+                            pExecContext,
+                            pDeleteState->pSession);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             ntStatus = SrvSessionFindTree_SMB_V1(
                             pCtxSmb1,
                             pDeleteState->pSession,
@@ -174,19 +177,8 @@ SrvProcessDelete(
                             &pDeleteState->pTree);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            LWIO_LOCK_RWMUTEX_SHARED(   bTreeInLock,
-                                        &pDeleteState->pTree->pShareInfo->mutex);
-
-            ntStatus = SrvAllocateStringW(
-                            pDeleteState->pTree->pShareInfo->pwszPath,
-                            &pwszFilesystemPath);
-            BAIL_ON_NT_STATUS(ntStatus);
-
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                                &pDeleteState->pTree->pShareInfo->mutex);
-
             ntStatus = SrvFinderBuildSearchPath(
-                            pwszFilesystemPath,
+                            NULL,
                             pDeleteState->pwszSearchPattern,
                             &pDeleteState->pwszFilesystemPath,
                             &pDeleteState->pwszSearchPattern2,
@@ -196,6 +188,7 @@ SrvProcessDelete(
             if (pDeleteState->bPathHasWildCards)
             {
                 ntStatus = SrvFinderCreateSearchSpace(
+                                pDeleteState->pTree->hFile,
                                 pDeleteState->pTree->pShareInfo,
                                 pDeleteState->pSession->pIoSecurityContext,
                                 pDeleteState->pSession->hFinderRepository,
@@ -247,8 +240,6 @@ SrvProcessDelete(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pDeleteState->pTree->pShareInfo->mutex);
-
     if (pDeleteState)
     {
         LWIO_UNLOCK_MUTEX(bInLock, &pDeleteState->mutex);
@@ -256,11 +247,6 @@ cleanup:
         SrvReleaseDeleteState(pDeleteState);
     }
 
-    if (pwszFilesystemPath)
-    {
-        SrvFreeMemory(pwszFilesystemPath);
-    }
-
     return ntStatus;
 
 error:
@@ -399,6 +385,8 @@ SrvDeleteFiles(
                             &pDeleteState->fileName.FileName);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            pDeleteState->fileName.RootFileHandle = pDeleteState->pTree->hFile;
+
             SrvPrepareDeleteStateAsync(pDeleteState, pExecContext);
 
             pDeleteState->bPendingCreate = TRUE;
@@ -420,7 +408,7 @@ SrvDeleteFiles(
                             pDeleteState->ulCreateOptions,
                             NULL,
                             0,
-                            &pDeleteState->pEcpList);
+                            pDeleteState->pEcpList);
             BAIL_ON_NT_STATUS(ntStatus);
 
             SrvReleaseDeleteStateAsync(pDeleteState); // completed sync
@@ -535,6 +523,8 @@ SrvDeleteSingleFile(
                         &pDeleteState->fileName.FileName);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        pDeleteState->fileName.RootFileHandle = pDeleteState->pTree->hFile;
+
         SrvPrepareDeleteStateAsync(pDeleteState, pExecContext);
 
         pDeleteState->bPendingCreate = TRUE;
@@ -556,7 +546,7 @@ SrvDeleteSingleFile(
                         pDeleteState->ulCreateOptions,
                         NULL,
                         0,
-                        &pDeleteState->pEcpList);
+                        pDeleteState->pEcpList);
         BAIL_ON_NT_STATUS(ntStatus);
 
         SrvReleaseDeleteStateAsync(pDeleteState); // completed sync
