@@ -227,6 +227,32 @@ SrvFinderMarshallIdBothDirInfoResults(
     PUSHORT           pusSearchCount
     );
 
+static
+NTSTATUS
+SrvFinderGetIdFullDirInfoSearchResults(
+    PSRV_SEARCH_SPACE pSearchSpace,
+    BOOLEAN           bReturnSingleEntry,
+    BOOLEAN           bRestartScan,
+    USHORT            usDesiredSearchCount,
+    USHORT            usMaxDataCount,
+    USHORT            usDataOffset,
+    PBYTE*            ppData,
+    PUSHORT           pusDataLen,
+    PUSHORT           pusSearchResultCount,
+    PBOOLEAN          pbEndOfSearch
+    );
+
+static
+NTSTATUS
+SrvFinderMarshallIdFullDirInfoResults(
+    PSRV_SEARCH_SPACE pSearchSpace,
+    USHORT            usBytesAvailable,
+    USHORT            usDesiredSearchCount,
+    PUSHORT           pusDataOffset,
+    PBYTE*            ppData,
+    PUSHORT           pusDataLen,
+    PUSHORT           pusSearchCount
+    );
 
 
 static
@@ -390,6 +416,22 @@ SrvFinderGetSearchResults(
 
             break;
 
+        case SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO:
+
+            ntStatus = SrvFinderGetIdFullDirInfoSearchResults(
+                            pSearchSpace,
+                            bReturnSingleEntry,
+                            bRestartScan,
+                            usDesiredSearchCount,
+                            usMaxDataCount,
+                            usDataOffset,
+                            &pData,
+                            &usDataLen,
+                            &usSearchResultCount,
+                            &bEndOfSearch);
+
+            break;
+
         case SMB_FIND_FILE_ID_BOTH_DIRECTORY_INFO:
 
             ntStatus = SrvFinderGetIdBothDirInfoSearchResults(
@@ -2303,6 +2345,363 @@ error:
     goto cleanup;
 }
 
+static
+NTSTATUS
+SrvFinderGetIdFullDirInfoSearchResults(
+    PSRV_SEARCH_SPACE pSearchSpace,
+    BOOLEAN           bReturnSingleEntry,
+    BOOLEAN           bRestartScan,
+    USHORT            usDesiredSearchCount,
+    USHORT            usMaxDataCount,
+    USHORT            usDataOffset,
+    PBYTE*            ppData,
+    PUSHORT           pusDataLen,
+    PUSHORT           pusSearchResultCount,
+    PBOOLEAN          pbEndOfSearch
+    )
+{
+    NTSTATUS ntStatus = 0;
+    PBYTE    pData = NULL;
+    USHORT   usDataLen = 0;
+    USHORT   usSearchResultCount = 0;
+    USHORT   usBytesAvailable = usMaxDataCount;
+    BOOLEAN  bEndOfSearch = FALSE;
+    PFILE_ID_FULL_DIR_INFORMATION pFileInfoCursor = NULL;
+
+    if (!pSearchSpace->pFileInfo)
+    {
+        // Keep space for 10 items
+        USHORT usBytesAllocated = (sizeof(FILE_ID_FULL_DIR_INFORMATION) + 256 * sizeof(wchar16_t)) * 10;
+
+        ntStatus = SrvAllocateMemory(
+                        usBytesAllocated,
+                        (PVOID*)&pSearchSpace->pFileInfo);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        pSearchSpace->usFileInfoLen = usBytesAllocated;
+    }
+
+    while (!bEndOfSearch && (usSearchResultCount < usDesiredSearchCount))
+    {
+        USHORT usIterSearchCount = 0;
+
+        pFileInfoCursor = (PFILE_ID_FULL_DIR_INFORMATION)pSearchSpace->pFileInfoCursor;
+        if (!pFileInfoCursor)
+        {
+            IO_MATCH_FILE_SPEC ioFileSpec;
+            IO_STATUS_BLOCK ioStatusBlock = {0};
+
+            ioFileSpec.Type = IO_MATCH_FILE_SPEC_TYPE_UNKNOWN;
+            // ioFileSpec.Options = IO_NAME_OPTION_CASE_SENSITIVE;
+            RtlUnicodeStringInit(
+                &ioFileSpec.Pattern,
+                pSearchSpace->pwszSearchPattern);
+
+            pSearchSpace->pFileInfoCursor = NULL;
+
+            do
+            {
+                memset(pSearchSpace->pFileInfo, 0x0, pSearchSpace->usFileInfoLen);
+
+                ntStatus = IoQueryDirectoryFile(
+                                pSearchSpace->hFile,
+                                NULL,
+                                &ioStatusBlock,
+                                pSearchSpace->pFileInfo,
+                                pSearchSpace->usFileInfoLen,
+                                FileIdFullDirectoryInformation,
+                                bReturnSingleEntry,
+                                &ioFileSpec,
+                                bRestartScan);
+                if (ntStatus == STATUS_SUCCESS)
+                {
+                    pSearchSpace->pFileInfoCursor = pSearchSpace->pFileInfo;
+
+                    break;
+                }
+                else if (ntStatus == STATUS_NO_MORE_MATCHES)
+                {
+                    bEndOfSearch = TRUE;
+                    ntStatus = 0;
+
+                    break;
+                }
+                else if (ntStatus == STATUS_BUFFER_TOO_SMALL)
+                {
+                    USHORT usNewSize = pSearchSpace->usFileInfoLen + 256 * sizeof(wchar16_t);
+
+                    ntStatus = SMBReallocMemory(
+                                    pSearchSpace->pFileInfo,
+                                    (PVOID*)&pSearchSpace->pFileInfo,
+                                    usNewSize);
+                    BAIL_ON_NT_STATUS(ntStatus);
+
+                    memset(pSearchSpace->pFileInfo + pSearchSpace->usFileInfoLen,
+                           0,
+                           usNewSize - pSearchSpace->usFileInfoLen);
+
+                    pSearchSpace->usFileInfoLen = usNewSize;
+
+                    continue;
+                }
+                BAIL_ON_NT_STATUS(ntStatus);
+
+            } while (TRUE);
+        }
+
+        if (pSearchSpace->pFileInfoCursor)
+        {
+            ntStatus = SrvFinderMarshallIdFullDirInfoResults(
+                            pSearchSpace,
+                            usBytesAvailable - usDataLen,
+                            usDesiredSearchCount - usSearchResultCount,
+                            &usDataOffset,
+                            &pData,
+                            &usDataLen,
+                            &usIterSearchCount);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            if (!usIterSearchCount)
+            {
+                break;
+            }
+
+            usSearchResultCount += usIterSearchCount;
+        }
+    }
+
+    if (!usSearchResultCount)
+    {
+        ntStatus = STATUS_NO_MORE_MATCHES;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    *ppData = pData;
+    *pusDataLen = usDataLen;
+    *pusSearchResultCount = usSearchResultCount;
+    *pbEndOfSearch = pSearchSpace->pFileInfoCursor ? FALSE : bEndOfSearch;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    *ppData = NULL;
+    *pusDataLen = 0;
+    *pusSearchResultCount = 0;
+    *pbEndOfSearch = FALSE;
+
+    if (pData)
+    {
+        SrvFreeMemory(pData);
+    }
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SrvFinderMarshallIdFullDirInfoResults(
+    PSRV_SEARCH_SPACE pSearchSpace,
+    USHORT            usBytesAvailable,
+    USHORT            usDesiredSearchCount,
+    PUSHORT           pusDataOffset,
+    PBYTE*            ppData,
+    PUSHORT           pusDataLen,
+    PUSHORT           pusSearchCount
+    )
+{
+    NTSTATUS ntStatus = 0;
+    USHORT   usBytesRequired = 0;
+    PSMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER pInfoHeader = NULL;
+    PFILE_ID_FULL_DIR_INFORMATION pFileInfoCursor = NULL;
+    PBYTE    pData = *ppData;
+    USHORT   usDataLen = 0;
+    USHORT   usOrigDataLen = *pusDataLen;
+    PBYTE    pDataCursor = NULL;
+    USHORT   usSearchCount = 0;
+    USHORT   iSearchCount = 0;
+    USHORT   usOffset = 0;
+    USHORT   usDataOffset = *pusDataOffset;
+
+    pFileInfoCursor = (PFILE_ID_FULL_DIR_INFORMATION)pSearchSpace->pFileInfoCursor;
+
+    while (pFileInfoCursor &&
+           (usSearchCount < usDesiredSearchCount) &&
+           (usBytesAvailable > 0))
+    {
+        USHORT usInfoBytesRequired = 0;
+
+        usInfoBytesRequired = sizeof(SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER);
+        usDataOffset += sizeof(SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER);
+
+        if (pSearchSpace->bUseLongFilenames)
+        {
+            usInfoBytesRequired += pFileInfoCursor->FileNameLength;
+            usDataOffset += pFileInfoCursor->FileNameLength;
+        }
+
+        if (!pFileInfoCursor->FileNameLength)
+        {
+            usInfoBytesRequired += sizeof(wchar16_t);
+            usDataOffset += sizeof(wchar16_t);
+        }
+
+        if (usDataOffset % 8)
+        {
+            USHORT usAlignment = (8 - (usDataOffset % 8));
+
+            usInfoBytesRequired += usAlignment;
+            usDataOffset += usAlignment;
+        }
+
+        if (usBytesAvailable < usInfoBytesRequired)
+        {
+            break;
+        }
+
+        usSearchCount++;
+        usBytesAvailable -= usInfoBytesRequired;
+        usBytesRequired += usInfoBytesRequired;
+
+        if (pFileInfoCursor->NextEntryOffset)
+        {
+            pFileInfoCursor = (PFILE_ID_FULL_DIR_INFORMATION)(((PBYTE)pFileInfoCursor) + pFileInfoCursor->NextEntryOffset);
+        }
+        else
+        {
+            pFileInfoCursor = NULL;
+        }
+    }
+
+    if (!pData)
+    {
+        ntStatus = SrvAllocateMemory(
+                        usBytesRequired,
+                        (PVOID*)&pData);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        usDataLen = usBytesRequired;
+    }
+    else
+    {
+        USHORT usNewDataLen = usOrigDataLen + usBytesRequired;
+
+        ntStatus = SMBReallocMemory(
+                        pData,
+                        (PVOID*)&pData,
+                        usNewDataLen);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        usDataLen = usNewDataLen;
+
+        // Got to fill the last offset from previous data
+        usOffset = usOrigDataLen;
+        pDataCursor = pData;
+        pInfoHeader = (PSMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER)pDataCursor;
+        while (pInfoHeader->NextEntryOffset)
+        {
+            usOffset -= pInfoHeader->NextEntryOffset;
+            pInfoHeader = (PSMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER)((PBYTE)pInfoHeader + pInfoHeader->NextEntryOffset);
+        }
+    }
+
+    usDataOffset = *pusDataOffset;
+
+    pDataCursor = pData + usOrigDataLen;
+    for (; iSearchCount < usSearchCount; iSearchCount++)
+    {
+        pFileInfoCursor = (PFILE_ID_FULL_DIR_INFORMATION)pSearchSpace->pFileInfoCursor;
+
+        if (pInfoHeader)
+        {
+            pInfoHeader->NextEntryOffset = usOffset;
+        }
+
+        pInfoHeader = (PSMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER)pDataCursor;
+
+        usOffset = 0;
+
+        pInfoHeader->FileIndex = pFileInfoCursor->FileIndex;
+        pInfoHeader->EaSize = pFileInfoCursor->EaSize;
+        pInfoHeader->CreationTime = pFileInfoCursor->CreationTime;
+        pInfoHeader->LastAccessTime = pFileInfoCursor->LastAccessTime;
+        pInfoHeader->LastWriteTime = pFileInfoCursor->LastWriteTime;
+        pInfoHeader->ChangeTime = pFileInfoCursor->ChangeTime;
+        pInfoHeader->EndOfFile = pFileInfoCursor->EndOfFile;
+        pInfoHeader->AllocationSize = pFileInfoCursor->AllocationSize;
+        pInfoHeader->FileAttributes = pFileInfoCursor->FileAttributes;
+        pInfoHeader->FileNameLength = pFileInfoCursor->FileNameLength;
+        pInfoHeader->FileId = pFileInfoCursor->FileId;
+
+        pDataCursor += sizeof(SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER);
+        usOffset += sizeof(SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER);
+        usDataOffset += sizeof(SMB_FIND_FILE_ID_FULL_DIRECTORY_INFO_HEADER);
+
+        if (pSearchSpace->bUseLongFilenames && pFileInfoCursor->FileNameLength)
+        {
+            memcpy(pDataCursor, (PBYTE)pFileInfoCursor->FileName, pFileInfoCursor->FileNameLength);
+
+            pDataCursor += pFileInfoCursor->FileNameLength;
+            usOffset += pFileInfoCursor->FileNameLength;
+            usDataOffset += pFileInfoCursor->FileNameLength;
+        }
+
+        if (!pFileInfoCursor->FileNameLength)
+        {
+            pDataCursor += sizeof(wchar16_t);
+            usOffset += sizeof(wchar16_t);
+            usDataOffset += sizeof(wchar16_t);
+        }
+
+        if (usDataOffset % 8)
+        {
+            USHORT usAlignment = (8 - (usDataOffset % 8));
+
+            pDataCursor += usAlignment;
+            usOffset += usAlignment;
+            usDataOffset += usAlignment;
+        }
+
+        if (pFileInfoCursor->NextEntryOffset)
+        {
+            pSearchSpace->pFileInfoCursor = (((PBYTE)pFileInfoCursor) + pFileInfoCursor->NextEntryOffset);
+        }
+        else
+        {
+            pSearchSpace->pFileInfoCursor = NULL;
+        }
+    }
+
+    if (pInfoHeader)
+    {
+        pInfoHeader->NextEntryOffset = 0;
+    }
+
+    *pusSearchCount = usSearchCount;
+    *pusDataLen = usDataLen;
+    *pusDataOffset = usDataOffset;
+    *ppData = pData;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    *pusSearchCount = 0;
+    *pusDataLen = 0;
+    *ppData = NULL;
+
+    if (pData)
+    {
+        SrvFreeMemory(pData);
+    }
+
+    goto cleanup;
+}
+
 
 static
 NTSTATUS
