@@ -90,6 +90,8 @@ SrvProtocolExecute_SMB_V1(
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
     PSRV_EXEC_CONTEXT_SMB_V1 pSmb1Context = NULL;
+    UCHAR    ucCurrentCommand = 0;
+    BOOLEAN  bPopStatMessage  = FALSE;
 
     if (!pExecContext->pProtocolContext->pSmb1Context)
     {
@@ -184,6 +186,19 @@ SrvProtocolExecute_SMB_V1(
                          SrvGetCommandDescription_SMB_V1(pRequest->ucCommand),
                          pRequest->ucCommand);
 
+        if (pExecContext->pStatInfo)
+        {
+            ucCurrentCommand = pRequest->ucCommand;
+
+            ntStatus = SrvStatisticsPushMessage(
+                            pExecContext->pStatInfo,
+                            ucCurrentCommand,
+                            pRequest->ulMessageSize);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            bPopStatMessage = TRUE;
+        }
+
         switch (pRequest->ucCommand)
         {
             case COM_NEGOTIATE:
@@ -457,7 +472,8 @@ SrvProtocolExecute_SMB_V1(
                     /* Terminate the chain */
                     if (pPrevResponse->pAndXHeader)
                     {
-                        pPrevResponse->pAndXHeader->andXCommand = 0xFF;
+                        pPrevResponse->pAndXHeader->andXCommand =
+                            COM_NO_ANDX_COMMAND;
                         pPrevResponse->pAndXHeader->andXOffset = 0;
                     }
 
@@ -483,6 +499,27 @@ SrvProtocolExecute_SMB_V1(
         }
 
         pExecContext->pSmbResponse->bufferUsed += pResponse->ulMessageSize;
+
+        if (bPopStatMessage)
+        {
+            NTSTATUS ntStatus2 =
+                    SrvStatisticsPopMessage(
+                            pExecContext->pStatInfo,
+                            ucCurrentCommand,
+                            (pResponse->ulMessageSize ?
+                                    pResponse->ulMessageSize :
+                                    pResponse->ulZctMessageSize),
+                            ntStatus);
+            if (ntStatus2)
+            {
+                LWIO_LOG_ERROR( "Error: Failed to notify statistics "
+                                "module on end of message processing "
+                                "[error: %u]", ntStatus2);
+            }
+
+            bPopStatMessage = FALSE;
+            ucCurrentCommand = 0;
+        }
     }
 
     ntStatus = SMBPacketMarshallFooter(pExecContext->pSmbResponse);
@@ -494,9 +531,66 @@ cleanup:
 
 error:
 
+    switch (ntStatus)
+    {
+        case STATUS_PENDING:
+
+            break;
+
+        default:
+
+            if (bPopStatMessage)
+            {
+                NTSTATUS ntStatus2 =
+                        SrvStatisticsPopMessage(
+                                pExecContext->pStatInfo,
+                                ucCurrentCommand,
+                                pExecContext->pSmbResponse->bufferUsed,
+                                ntStatus);
+                if (ntStatus2)
+                {
+                    LWIO_LOG_ERROR( "Error: Failed to notify statistics "
+                                    "module on end of message processing "
+                                    "[error: %u]", ntStatus2);
+                }
+            }
+
+            break;
+    }
+
     goto cleanup;
 }
 
+NTSTATUS
+SrvProtocolCloseFile_SMB_V1(
+    PLWIO_SRV_TREE pTree,
+    PLWIO_SRV_FILE pFile
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+
+    if (!pTree || !pFile)
+    {
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    SrvFileResetOplockState(pFile);
+
+    ntStatus = SrvTreeRemoveFile(
+                    pTree,
+                    pFile->fid);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    SrvFileCancelAsyncOperations(pTree, pFile);
+
+    SrvFileRundown(pFile);
+
+error:
+
+    return ntStatus;
+}
+
 NTSTATUS
 SrvBuildExecContext_SMB_V1(
     PLWIO_SRV_CONNECTION      pConnection,
@@ -569,7 +663,7 @@ SrvBuildExecContext_SMB_V1(
         {
             switch (pAndXHeader->andXCommand)
             {
-                case 0xFF:
+                case COM_NO_ANDX_COMMAND:
 
                     pBuffer = NULL;
 
@@ -663,7 +757,9 @@ SrvBuildExecContext_SMB_V1(
                 break;
         }
 
-        if (pMessage->pAndXHeader && pMessage->pAndXHeader->andXOffset)
+        if (pMessage->pAndXHeader &&
+            pMessage->pAndXHeader->andXOffset &&
+            pMessage->pAndXHeader->andXCommand != COM_NO_ANDX_COMMAND)
         {
             ucAndXCommand = pMessage->pAndXHeader->andXCommand;
 
@@ -870,12 +966,11 @@ error:
     goto cleanup;
 }
 
-NTSTATUS
+VOID
 SrvProtocolShutdown_SMB_V1(
     VOID
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
     BOOLEAN bInLock = FALSE;
 
     LWIO_LOCK_MUTEX(bInLock, &gProtocolGlobals_SMB_V1.mutex);
@@ -887,8 +982,6 @@ SrvProtocolShutdown_SMB_V1(
     /* Configuration shutdown should always come last as other shutdown
      * routines may rely on configuration parameters to be set */
     SrvConfigShutdown_SMB_V1();
-
-    return status;
 }
 
 static
