@@ -88,6 +88,13 @@ SrvFreeNtRenameState(
     PSRV_NT_RENAME_STATE_SMB_V1 pRenameState
     );
 
+static
+NTSTATUS
+SrvValidateStreamNewName(
+    PWSTR pwszOldName,
+    PWSTR pwszNewName
+    );
+
 NTSTATUS
 SrvProcessNtRename(
     PSRV_EXEC_CONTEXT pExecContext
@@ -100,7 +107,6 @@ SrvProcessNtRename(
     PLWIO_SRV_SESSION           pSession     = NULL;
     PLWIO_SRV_TREE              pTree        = NULL;
     PSRV_NT_RENAME_STATE_SMB_V1 pRenameState = NULL;
-    BOOLEAN                     bTreeInLock  = FALSE;
     BOOLEAN                     bInLock      = FALSE;
 
     pRenameState = (PSRV_NT_RENAME_STATE_SMB_V1)pCtxSmb1->hState;
@@ -126,6 +132,9 @@ SrvProcessNtRename(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -160,24 +169,21 @@ SrvProcessNtRename(
     {
         case SRV_NT_RENAME_STAGE_SMB_V1_INITIAL:
 
-            LWIO_LOCK_RWMUTEX_SHARED(   bTreeInLock,
-                                        &pCtxSmb1->pTree->pShareInfo->mutex);
-
-            ntStatus = SrvBuildFilePath(
-                            pCtxSmb1->pTree->pShareInfo->pwszPath,
+            ntStatus = SrvBuildTreeRelativePath(
+                            pCtxSmb1->pTree,
                             pRenameState->pwszOldName,
-                            &pRenameState->oldName.FileName);
+                            &pRenameState->oldName);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            ntStatus = SrvAllocateStringW(
-                            pTree->pShareInfo->pwszPath,
-                            &pRenameState->dirPath.FileName);
+            ntStatus = SrvBuildTreeRelativePath(
+                            pCtxSmb1->pTree,
+                            NULL,
+                            &pRenameState->dirPath);
             BAIL_ON_NT_STATUS(ntStatus);
 
             pRenameState->newName.FileName = pRenameState->pwszNewName;
 
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                                &pCtxSmb1->pTree->pShareInfo->mutex);
+            pRenameState->newName.RootFileHandle = pTree->hFile;
 
             pRenameState->stage = SRV_NT_RENAME_STAGE_SMB_V1_ATTEMPT_RENAME;
 
@@ -208,8 +214,6 @@ SrvProcessNtRename(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->pShareInfo->mutex);
-
     if (pSession)
     {
         SrvSessionRelease(pSession);
@@ -254,6 +258,36 @@ error:
     goto cleanup;
 }
 
+static
+NTSTATUS
+SrvValidateStreamNewName(
+    PWSTR pwszOldName,
+    PWSTR pwszNewName
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    wchar16_t wszColon[] = {':', 0};
+
+    while(!IsNullOrEmptyString(pwszOldName))
+    {
+        if (*pwszOldName++ == wszColon[0])
+        {
+            if (*pwszNewName != wszColon[0] ||
+                IsNullOrEmptyString((pwszNewName+1)))
+            {
+                ntStatus = STATUS_INVALID_PARAMETER;
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
+            break;
+        }
+    }
+
+error:
+
+    return ntStatus;
+}
+
 static
 NTSTATUS
 SrvBuildNtRenameState(
@@ -272,6 +306,9 @@ SrvBuildNtRenameState(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
+    ntStatus = SrvValidateStreamNewName(pwszOldName, pwszNewName);
+    BAIL_ON_NT_STATUS(ntStatus);
+
     ntStatus = SrvAllocateMemory(
                     sizeof(SRV_NT_RENAME_STATE_SMB_V1),
                     (PVOID*)&pRenameState);
@@ -343,7 +380,7 @@ SrvExecuteNtRename(
                         FILE_DIRECTORY_FILE,
                         NULL, /* EA Buffer */
                         0,    /* EA Length */
-                        &pRenameState->pDirEcpList
+                        pRenameState->pDirEcpList
                         );
         BAIL_ON_NT_STATUS(ntStatus);
 
@@ -371,7 +408,7 @@ SrvExecuteNtRename(
                         0,
                         NULL, /* EA Buffer */
                         0,    /* EA Length */
-                        &pRenameState->pFileEcpList
+                        pRenameState->pFileEcpList
                         );
         BAIL_ON_NT_STATUS(ntStatus);
 
