@@ -257,6 +257,11 @@ SrvProcessNtTransact(
                         &pNTTransactState->pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(
+                        pExecContext,
+                        pNTTransactState->pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pNTTransactState->pSession,
@@ -267,6 +272,14 @@ SrvProcessNtTransact(
 
     LWIO_LOCK_MUTEX(bInLock, &pNTTransactState->mutex);
 
+    if (pExecContext->pStatInfo)
+    {
+        ntStatus = SrvStatisticsSetSubOpcode(
+                        pExecContext->pStatInfo,
+                        pNTTransactState->pRequestHeader->usFunction);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     switch (pNTTransactState->pRequestHeader->usFunction)
     {
         case SMB_SUB_COMMAND_NT_TRANSACT_QUERY_SECURITY_DESC:
@@ -931,6 +944,8 @@ SrvProcessIOCTL(
                 BAIL_ON_NT_STATUS(ntStatus);
             }
 
+            SrvUnmarshallBoolean(&pNTTransactState->pIoctlRequest->bIsFsctl);
+
             if (pNTTransactState->pIoctlRequest->ucFlags & 0x1)
             {
                 // TODO: Apply only to DFS Share
@@ -952,6 +967,14 @@ SrvProcessIOCTL(
 
         case SRV_NTTRANSACT_STAGE_SMB_V1_ATTEMPT_IO:
 
+            if (pExecContext->pStatInfo)
+            {
+                ntStatus = SrvStatisticsSetIOCTL(
+                                pExecContext->pStatInfo,
+                                pNTTransactState->pIoctlRequest->ulFunctionCode);
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
             if (pNTTransactState->pIoctlRequest->bIsFsctl)
             {
                 ntStatus = SrvExecuteFsctl(pExecContext);
@@ -1378,6 +1401,8 @@ SrvProcessNotifyChange(
             pNTTransactState->pNotifyChangeHeader =
                 (PSMB_NOTIFY_CHANGE_HEADER)(PBYTE)pNTTransactState->pSetup;
 
+            SrvUnmarshallBoolean(&pNTTransactState->pNotifyChangeHeader->bWatchTree);
+
             ntStatus = SrvTreeFindFile_SMB_V1(
                             pCtxSmb1,
                             pNTTransactState->pTree,
@@ -1403,6 +1428,7 @@ SrvProcessNotifyChange(
                             pNotifyState->ullNotifyId,
                             SMB_SUB_COMMAND_NT_TRANSACT_NOTIFY_CHANGE,
                             pNotifyState,
+                            &SrvNotifyStateCancel,
                             &SrvNotifyStateReleaseHandle,
                             &pAsyncState);
             BAIL_ON_NT_STATUS(ntStatus);
@@ -1581,7 +1607,7 @@ SrvProcessChangeNotifyCompletion(
                     &pTree);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    ullNotifyId = SrvNotifyGetId(
+    ullNotifyId = SrvAsyncStateBuildId(
                     SMB_V1_GET_PROCESS_ID(pSmbRequest->pHeader),
                     pSmbRequest->pHeader->mid);
 
@@ -1957,7 +1983,7 @@ SrvProcessNtTransactCreate(
                             pNTTransactState->pNtTransactCreateHeader->ulCreateOptions,
                             pNTTransactState->pEA,
                             pNTTransactState->pNtTransactCreateHeader->ulEALength,
-                            &pNTTransactState->pEcpList);
+                            pNTTransactState->pEcpList);
             BAIL_ON_NT_STATUS(ntStatus);
 
             SrvReleaseNTTransactStateAsync(pNTTransactState); // completed sync
@@ -2015,23 +2041,28 @@ SrvProcessNtTransactCreate(
             ntStatus = pNTTransactState->ioStatusBlock.Status;
             BAIL_ON_NT_STATUS(ntStatus);
 
-            if (!pNTTransactState->pFile->hByteRangeLockState)
+            if (!pNTTransactState->pFile->hCancellableBRLStateList)
             {
-                PSRV_PENDING_LOCK_STATE_LIST pPendingLockStateList = NULL;
+                PSRV_BYTE_RANGE_LOCK_STATE_LIST pBRLStateList = NULL;
 
-                ntStatus = SrvCreatePendingLockStateList(&pPendingLockStateList);
+                ntStatus = SrvCreatePendingLockStateList(&pBRLStateList);
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                pNTTransactState->pFile->hByteRangeLockState =
-                                (HANDLE)pPendingLockStateList;
+                pNTTransactState->pFile->hCancellableBRLStateList =
+                                (HANDLE)pBRLStateList;
 
-                pNTTransactState->pFile->pfnFreeByteRangeLockState =
+                pNTTransactState->pFile->pfnFreeBRLStateList =
                                 &SrvFreePendingLockStateListHandle;
             }
 
             ntStatus = SrvBuildNTTransactCreateResponse(pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvElementsRegisterResource(
+                            &pNTTransactState->pFile->resource,
+                            NULL);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             pNTTransactState->stage = SRV_NTTRANSACT_STAGE_SMB_V1_DONE;
 
             // intentional fall through
@@ -2040,6 +2071,19 @@ SrvProcessNtTransactCreate(
 
             pNTTransactState->bRemoveFileFromTree = FALSE;
 
+            if (pNTTransactState->pNtTransactCreateHeader->ulDesiredAccess & FILE_READ_DATA)
+            {
+                pNTTransactState->pFile->ulPermissions |= SRV_PERM_FILE_READ;
+            }
+            if (pNTTransactState->pNtTransactCreateHeader->ulDesiredAccess & FILE_WRITE_DATA)
+            {
+                pNTTransactState->pFile->ulPermissions |= SRV_PERM_FILE_WRITE;
+            }
+            if (pNTTransactState->ulCreateAction == FILE_CREATED)
+            {
+                pNTTransactState->pFile->ulPermissions |= SRV_PERM_FILE_CREATE;
+            }
+
             pCtxSmb1->pFile = SrvFileAcquire(pNTTransactState->pFile);
 
             break;
@@ -2075,7 +2119,7 @@ SrvParseNtTransactCreateParameters(
     ULONG                        ulOffset         = 0;
     ULONG                        ulBytesAvailable = 0;
     PWSTR                        pwszFilename     = NULL;
-    BOOLEAN                      bTreeInLock      = FALSE;
+    BOOLEAN                      bShareInLock      = FALSE;
 
     pNTTransactState = (PSRV_NTTRANSACT_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -2217,7 +2261,7 @@ SrvParseNtTransactCreateParameters(
                 BAIL_ON_NT_STATUS(ntStatus);
             }
 
-            LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pNTTransactState->pTree->mutex);
+            LWIO_LOCK_RWMUTEX_SHARED(bShareInLock, &pNTTransactState->pTree->mutex);
 
             ntStatus = SrvBuildFilePath(
                             NULL, /* relative path */
@@ -2225,7 +2269,7 @@ SrvParseNtTransactCreateParameters(
                             &pNTTransactState->pFilename->FileName);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pNTTransactState->pTree->mutex);
+            LWIO_UNLOCK_RWMUTEX(bShareInLock, &pNTTransactState->pTree->mutex);
         }
 
         pNTTransactState->pFilename->RootFileHandle =
@@ -2233,15 +2277,11 @@ SrvParseNtTransactCreateParameters(
     }
     else
     {
-        LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pNTTransactState->pTree->mutex);
-
-        ntStatus = SrvBuildFilePath(
-                        pNTTransactState->pTree->pShareInfo->pwszPath,
+        ntStatus = SrvBuildTreeRelativePath(
+                        pNTTransactState->pTree,
                         pwszFilename,
-                        &pNTTransactState->pFilename->FileName);
+                        pNTTransactState->pFilename);
         BAIL_ON_NT_STATUS(ntStatus);
-
-        LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pNTTransactState->pTree->mutex);
     }
 
     /* For named pipes, we need to pipe some extra data into the npfs driver:
@@ -2267,7 +2307,9 @@ SrvParseNtTransactCreateParameters(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pNTTransactState->pTree->mutex);
+    LWIO_UNLOCK_RWMUTEX(
+            bShareInLock,
+            &pNTTransactState->pTree->pShareInfo->mutex);
 
     SRV_SAFE_FREE_MEMORY(pwszFilename);
 
@@ -2462,14 +2504,19 @@ SrvRequestNTTransactOplocks(
 
             case STATUS_PENDING:
 
+                InterlockedIncrement(&pOplockState->refCount);
+
                 ntStatus = SrvFileSetOplockState(
                                 pNTTransactState->pFile,
                                 pOplockState,
+                                &SrvCancelOplockStateHandle,
                                 &SrvReleaseOplockStateHandle);
+                if (ntStatus != STATUS_SUCCESS)
+                {
+                    InterlockedDecrement(&pOplockState->refCount);
+                }
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                InterlockedIncrement(&pOplockState->refCount);
-
                 SrvFileSetOplockLevel(
                                 pNTTransactState->pFile,
                                 pOplockCursor->oplockLevel);
