@@ -136,6 +136,9 @@ SrvProcessOpenAndX(
                             &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -190,7 +193,7 @@ SrvProcessOpenAndX(
                             pOpenState->usCreateOptions,
                             NULL, /* EA Buffer */
                             0,    /* EA Length */
-                            &pOpenState->pEcpList);
+                            pOpenState->pEcpList);
             BAIL_ON_NT_STATUS(ntStatus);
 
             SrvReleaseOpenStateAsync(pOpenState); // completed synchronously
@@ -248,34 +251,49 @@ SrvProcessOpenAndX(
             ntStatus = pOpenState->ioStatusBlock.Status;
             BAIL_ON_NT_STATUS(ntStatus);
 
-            if (!pOpenState->pFile->hByteRangeLockState)
+            if (!pOpenState->pFile->hCancellableBRLStateList)
             {
-                PSRV_PENDING_LOCK_STATE_LIST pPendingLockStateList = NULL;
+                PSRV_BYTE_RANGE_LOCK_STATE_LIST pBRLStateList = NULL;
 
-                ntStatus = SrvCreatePendingLockStateList(&pPendingLockStateList);
+                ntStatus = SrvCreatePendingLockStateList(&pBRLStateList);
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                pOpenState->pFile->hByteRangeLockState =
-                                (HANDLE)pPendingLockStateList;
+                pOpenState->pFile->hCancellableBRLStateList =
+                                (HANDLE)pBRLStateList;
 
-                pOpenState->pFile->pfnFreeByteRangeLockState =
+                pOpenState->pFile->pfnFreeBRLStateList =
                                 &SrvFreePendingLockStateListHandle;
             }
 
             ntStatus = SrvBuildOpenResponse(pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvElementsRegisterResource(
+                            &pOpenState->pFile->resource,
+                            NULL);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             pOpenState->stage = SRV_OPEN_STAGE_SMB_V1_DONE;
 
             // intentional fall through
 
         case SRV_OPEN_STAGE_SMB_V1_DONE:
 
-            ntStatus = pOpenState->ioStatusBlock.Status;
-            BAIL_ON_NT_STATUS(ntStatus);
-
             pOpenState->bRemoveFileFromTree = FALSE;
 
+            if (pOpenState->pRequestHeader->usDesiredAccess & FILE_READ_DATA)
+            {
+                pOpenState->pFile->ulPermissions |= SRV_PERM_FILE_READ;
+            }
+            if (pOpenState->pRequestHeader->usDesiredAccess & FILE_WRITE_DATA)
+            {
+                pOpenState->pFile->ulPermissions |= SRV_PERM_FILE_WRITE;
+            }
+            if (pOpenState->ulCreateAction == FILE_CREATED)
+            {
+                pOpenState->pFile->ulPermissions |= SRV_PERM_FILE_CREATE;
+            }
+
             pCtxSmb1->pFile = SrvFileAcquire(pOpenState->pFile);
 
             break;
@@ -341,7 +359,6 @@ SrvBuildOpenState(
     PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol   = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1       = pCtxProtocol->pSmb1Context;
     PSRV_OPEN_STATE_SMB_V1     pOpenState   = NULL;
-    BOOLEAN                    bTreeInLock    = FALSE;
 
     ntStatus = SrvAllocateMemory(
                     sizeof(SRV_OPEN_STATE_SMB_V1),
@@ -363,16 +380,12 @@ SrvBuildOpenState(
 
     pOpenState->pTree = SrvTreeAcquire(pCtxSmb1->pTree);
 
-    LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
-    ntStatus = SrvBuildFilePath(
-                    pCtxSmb1->pTree->pShareInfo->pwszPath,
+    ntStatus = SrvBuildTreeRelativePath(
+                    pCtxSmb1->pTree,
                     pwszFilename,
-                    &pOpenState->pFilename->FileName);
+                    pOpenState->pFilename);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
     pOpenState->pwszFilename = pwszFilename;
 
     /* For named pipes, we need to pipe some extra data into the npfs driver:
@@ -529,8 +542,6 @@ SrvBuildOpenState(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->mutex);
-
     return ntStatus;
 
 error:
@@ -728,14 +739,19 @@ SrvRequestOpenXOplocks(
                 break;
 
             case STATUS_PENDING:
+                InterlockedIncrement(&pOplockState->refCount);
+
                 ntStatus = SrvFileSetOplockState(
                                pOpenState->pFile,
                                pOplockState,
+                               &SrvCancelOplockStateHandle,
                                &SrvReleaseOplockStateHandle);
+                if (ntStatus != STATUS_SUCCESS)
+                {
+                    InterlockedDecrement(&pOplockState->refCount);
+                }
                 BAIL_ON_NT_STATUS(ntStatus);
 
-                InterlockedIncrement(&pOplockState->refCount);
-
                 SrvFileSetOplockLevel(
                         pOpenState->pFile,
                         pOplockCursor->oplockLevel);
