@@ -100,7 +100,6 @@ SrvProcessRename(
     PLWIO_SRV_SESSION          pSession     = NULL;
     PLWIO_SRV_TREE             pTree        = NULL;
     PSRV_RENAME_STATE_SMB_V1   pRenameState = NULL;
-    BOOLEAN                    bTreeInLock  = FALSE;
     BOOLEAN                    bInLock      = FALSE;
 
     pRenameState = (PSRV_RENAME_STATE_SMB_V1)pCtxSmb1->hState;
@@ -126,6 +125,9 @@ SrvProcessRename(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -160,25 +162,20 @@ SrvProcessRename(
     {
         case SRV_RENAME_STAGE_SMB_V1_INITIAL:
 
-            LWIO_LOCK_RWMUTEX_SHARED(   bTreeInLock,
-                                        &pCtxSmb1->pTree->pShareInfo->mutex);
-
-            ntStatus = SrvBuildFilePath(
-                            pCtxSmb1->pTree->pShareInfo->pwszPath,
+            ntStatus = SrvBuildTreeRelativePath(
+                            pCtxSmb1->pTree,
                             pRenameState->pwszOldName,
-                            &pRenameState->oldName.FileName);
+                            &pRenameState->oldName);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            ntStatus = SrvAllocateStringW(
-                            pTree->pShareInfo->pwszPath,
-                            &pRenameState->dirPath.FileName);
+            ntStatus = SrvBuildTreeRelativePath(
+                            pCtxSmb1->pTree,
+                            NULL,
+                            &pRenameState->dirPath);
             BAIL_ON_NT_STATUS(ntStatus);
 
             pRenameState->newName.FileName = pRenameState->pwszNewName;
 
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                                &pCtxSmb1->pTree->pShareInfo->mutex);
-
             pRenameState->stage = SRV_RENAME_STAGE_SMB_V1_ATTEMPT_RENAME;
 
             // intentional fall through
@@ -208,8 +205,6 @@ SrvProcessRename(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb1->pTree->pShareInfo->mutex);
-
     if (pSession)
     {
         SrvSessionRelease(pSession);
@@ -343,7 +338,7 @@ SrvExecuteRename(
                         FILE_DIRECTORY_FILE,
                         NULL, /* EA Buffer */
                         0,    /* EA Length */
-                        &pRenameState->pDirEcpList
+                        pRenameState->pDirEcpList
                         );
         BAIL_ON_NT_STATUS(ntStatus);
 
@@ -371,7 +366,7 @@ SrvExecuteRename(
                         0,
                         NULL, /* EA Buffer */
                         0,    /* EA Length */
-                        &pRenameState->pFileEcpList
+                        pRenameState->pFileEcpList
                         );
         BAIL_ON_NT_STATUS(ntStatus);
 
