@@ -121,7 +121,6 @@ SrvProcessSetInformation(
     PSRV_MESSAGE_SMB_V1        pSmbRequest  = &pCtxSmb1->pRequests[iMsg];
     PSRV_SET_INFO_STATE_SMB_V1 pInfoState   = NULL;
     BOOLEAN                    bInLock      = FALSE;
-    BOOLEAN                    bTreeInLock  = FALSE;
     PLWIO_SRV_SESSION          pSession     = NULL;
     PLWIO_SRV_TREE             pTree        = NULL;
 
@@ -152,6 +151,9 @@ SrvProcessSetInformation(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -192,16 +194,33 @@ SrvProcessSetInformation(
                             &pInfoState->fileBasicInfo.LastWriteTime);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pInfoState->pTree->mutex);
+            pInfoState->fileBasicInfo.FileAttributes =
+                         pInfoState->pRequestHeader->usFileAttributes;
 
-            ntStatus = SrvBuildFilePath(
-                            pInfoState->pTree->pShareInfo->pwszPath,
+            switch(pInfoState->fileBasicInfo.FileAttributes)
+            {
+                case FILE_ATTRIBUTE_NORMAL:
+
+                    pInfoState->fileBasicInfo.FileAttributes = 0;
+                    break;
+
+                case 0:
+
+                    pInfoState->fileBasicInfo.FileAttributes =
+                                                FILE_ATTRIBUTE_NORMAL;
+                    break;
+
+                default:
+
+                    break;
+            }
+
+            ntStatus = SrvBuildTreeRelativePath(
+                            pInfoState->pTree,
                             pInfoState->pwszFilename,
-                            &pInfoState->fileName.FileName);
+                            &pInfoState->fileName);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pInfoState->pTree->mutex);
-
             pInfoState->stage = SRV_SET_INFO_STAGE_SMB_V1_ATTEMPT_SET;
 
             SrvPrepareSetInfoStateAsync(pInfoState, pExecContext);
@@ -223,7 +242,7 @@ SrvProcessSetInformation(
                             0,
                             NULL, /* EA Buffer */
                             0,    /* EA Length */
-                            &pInfoState->pEcpList
+                            pInfoState->pEcpList
                             );
             BAIL_ON_NT_STATUS(ntStatus);
 
@@ -257,8 +276,6 @@ SrvProcessSetInformation(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pInfoState->pTree->mutex);
-
     if (pSession)
     {
         SrvSessionRelease(pSession);
