@@ -155,6 +155,7 @@ typedef struct _SRV_CHANGE_NOTIFY_STATE_SMB_V1
 typedef enum
 {
     SRV_TREE_CONNECT_STAGE_SMB_V1_INITIAL = 0,
+    SRV_TREE_CONNECT_STAGE_SMB_V1_CREATE_TREE_ROOT_HANDLE,
     SRV_TREE_CONNECT_STAGE_SMB_V1_ATTEMPT_QUERY_INFO,
     SRV_TREE_CONNECT_STAGE_SMB_V1_QUERY_INFO_COMPLETED,
     SRV_TREE_CONNECT_STAGE_SMB_V1_DONE
@@ -182,7 +183,6 @@ typedef struct _SRV_TREE_CONNECT_STATE_SMB_V1
     PVOID                          pSecurityDescriptor;
     PVOID                          pSecurityQOS;
     PIO_ECP_LIST                   pEcpList;
-    IO_FILE_HANDLE                 hFile;
 
     PSRV_SHARE_INFO                pShareInfo;
     PLWIO_SRV_SESSION              pSession;
@@ -196,6 +196,9 @@ typedef struct _SRV_TREE_CONNECT_STATE_SMB_V1
     ULONG                          ulMaximalShareAccessMask;
     ULONG                          ulGuestMaximalShareAccessMask;
 
+    struct sockaddr                clientAddress;
+    ULONG                          ulClientAddressLength;
+
     BOOLEAN                        bRemoveTreeFromSession;
 
 } SRV_TREE_CONNECT_STATE_SMB_V1, *PSRV_TREE_CONNECT_STATE_SMB_V1;
@@ -345,6 +348,11 @@ typedef struct _SRV_LOCK_STATE_SMB_V1
     PLOCKING_ANDX_RANGE              pUnlockRange;       // Do not free
     PLOCKING_ANDX_RANGE              pLockRange;         // Do not free
 
+    ULONG                            ulPid;
+    USHORT                           usMid;
+
+    ULONG64                          ullAsyncId;
+
     USHORT                           iUnlock;
     BOOLEAN                          bUnlockPending;
 
@@ -364,24 +372,33 @@ typedef struct _SRV_LOCK_STATE_SMB_V1
 
     PSRV_TIMER_REQUEST               pTimerRequest;
 
-    struct _SRV_LOCK_STATE_SMB_V1*   pNext;
-
 } SRV_LOCK_STATE_SMB_V1, *PSRV_LOCK_STATE_SMB_V1;
 
-typedef struct _SRV_PENDING_LOCK_STATE_LIST
+typedef struct _SRV_BYTE_RANGE_LOCK_STATE
 {
-    pthread_mutex_t        mutex;
-    pthread_mutex_t*       pMutex;
+    ULONG64 ullAsyncId;
 
-    PSRV_LOCK_STATE_SMB_V1 pLockStateHead;
-    PSRV_LOCK_STATE_SMB_V1 pLockStateTail;
+    struct _SRV_BYTE_RANGE_LOCK_STATE* pNext;
 
-} SRV_PENDING_LOCK_STATE_LIST, *PSRV_PENDING_LOCK_STATE_LIST;
+} SRV_BYTE_RANGE_LOCK_STATE, *PSRV_BYTE_RANGE_LOCK_STATE;
+
+typedef struct _SRV_BYTE_RANGE_LOCK_STATE_LIST
+{
+    pthread_mutex_t  mutex;
+    pthread_mutex_t* pMutex;
+
+    PSRV_BYTE_RANGE_LOCK_STATE pHead;
+    PSRV_BYTE_RANGE_LOCK_STATE pTail;
+
+} SRV_BYTE_RANGE_LOCK_STATE_LIST, *PSRV_BYTE_RANGE_LOCK_STATE_LIST;
 
 typedef enum
 {
     SRV_READ_STAGE_SMB_V1_INITIAL = 0,
+    SRV_READ_STAGE_SMB_V1_RESPONSE_START,
     SRV_READ_STAGE_SMB_V1_ATTEMPT_READ,
+    SRV_READ_STAGE_SMB_V1_RESPONSE_FINISH,
+    SRV_READ_STAGE_SMB_V1_ZCT_COMPLETE,
     SRV_READ_STAGE_SMB_V1_DONE
 } SRV_READ_STAGE_SMB_V1;
 
@@ -407,30 +424,47 @@ typedef struct _SRV_READ_STATE_SMB_V1
         PREAD_ANDX_REQUEST_HEADER_WC_12 pRequestHeader_WC_12;
     }; // Do not free
 
-    PLWIO_SRV_FILE            pFile;
+    PLWIO_SRV_FILE             pFile;
 
-    ULONG64                   ullBytesToRead;
-    LONG64                    llByteOffset;
+    ULONG64                    ullBytesToRead;
+    LONG64                     llByteOffset;
 
     PREAD_ANDX_RESPONSE_HEADER pResponseHeader; // Do not free
-    PBYTE pOutBuffer;
-    ULONG ulBytesAvailable;
-    ULONG ulOffset;
-    ULONG ulPackageByteCount;
-    ULONG ulTotalBytesUsed;
-    ULONG ulDataOffset;
-    ULONG ulBytesRead;
-    ULONG ulBytesToRead;
-    PBYTE pData;
-    ULONG ulKey;
-    BOOLEAN bPagedIo;
+    PBYTE                      pOutBuffer;
+    ULONG                      ulBytesAvailable;
+    ULONG                      ulOffset;
+    ULONG                      ulPackageByteCount;
+    ULONG                      ulTotalBytesUsed;
+    ULONG                      ulDataOffset;
+    ULONG                      ulBytesRead;
+    ULONG                      ulBytesToRead;
+    PBYTE                      pData;
+    ULONG                      ulKey;
+    BOOLEAN                    bPagedIo;
+    BOOLEAN                    bStartedRead;
+    PLW_ZCT_VECTOR             pZct;
+    PVOID                      pZctCompletion;
 
 } SRV_READ_STATE_SMB_V1, *PSRV_READ_STATE_SMB_V1;
 
+typedef struct _SRV_ZCT_WRITE_STATE
+{
+    ULONG                ulDataBytesMissing;
+    ULONG                ulDataBytesResident;
+    ULONG                ulSkipBytes;
+    PLW_ZCT_VECTOR       pZct;
+    PVOID                pZctCompletion;
+    PVOID                pPadding;
+    ULONG                ulPaddingSize;
+    PLWIO_SRV_CONNECTION pPausedConnection;
+} SRV_ZCT_WRITE_STATE, *PSRV_ZCT_WRITE_STATE;
+
 typedef enum
 {
     SRV_WRITEX_STAGE_SMB_V1_INITIAL = 0,
     SRV_WRITEX_STAGE_SMB_V1_ATTEMPT_WRITE,
+    SRV_WRITEX_STAGE_SMB_V1_ZCT_IO,
+    SRV_WRITEX_STAGE_SMB_V1_ZCT_COMPLETE,
     SRV_WRITEX_STAGE_SMB_V1_BUILD_RESPONSE,
     SRV_WRITEX_STAGE_SMB_V1_DONE
 } SRV_WRITEX_STAGE_SMB_V1;
@@ -451,12 +485,19 @@ typedef struct _SRV_WRITEX_STATE_SMB_V1
 
     PLWIO_SRV_FILE             pFile;
 
-    PWRITE_ANDX_REQUEST_HEADER pRequestHeader; // Do not free
+    UCHAR                      ucWordCount;
+    PWRITE_ANDX_REQUEST_HEADER_WC_12 pRequestHeader_WC_12; // Do not free
+    PWRITE_ANDX_REQUEST_HEADER_WC_14 pRequestHeader_WC_14; // Do not free
+
     PBYTE                      pData;          // Do not free
-    LONG64                     llDataOffset;
-    LONG64                     llDataLength;
-    ULONG64                    ullBytesWritten;
+    LONG64                     llOffset;
+    ULONG                      ulLength;
     ULONG                      ulKey;
+    ULONG                      ulBytesWritten;
+
+    BOOLEAN                    bStartedIo;
+
+    SRV_ZCT_WRITE_STATE        zct;
 
 } SRV_WRITEX_STATE_SMB_V1, *PSRV_WRITEX_STATE_SMB_V1;
 
@@ -464,6 +505,8 @@ typedef enum
 {
     SRV_WRITE_STAGE_SMB_V1_INITIAL = 0,
     SRV_WRITE_STAGE_SMB_V1_ATTEMPT_WRITE,
+    SRV_WRITE_STAGE_SMB_V1_ZCT_IO,
+    SRV_WRITE_STAGE_SMB_V1_ZCT_COMPLETE,
     SRV_WRITE_STAGE_SMB_V1_BUILD_RESPONSE,
     SRV_WRITE_STAGE_SMB_V1_DONE
 } SRV_WRITE_STAGE_SMB_V1;
@@ -486,11 +529,15 @@ typedef struct _SRV_WRITE_STATE_SMB_V1
 
     PWRITE_REQUEST_HEADER   pRequestHeader; // Do not free
     PBYTE                   pData;          // Do not free
-    ULONG                   ulDataOffset;
-    LONG64                  llDataOffset;
-    USHORT                  usDataLength;
-    USHORT                  usBytesWritten;
+    LONG64                  llOffset;
+    ULONG                   ulLength;
     ULONG                   ulKey;
+    ULONG                   ulBytesWritten;
+
+    BOOLEAN bStartedIo;
+
+    // ZCT information
+    SRV_ZCT_WRITE_STATE Zct;
 
     FILE_END_OF_FILE_INFORMATION  fileEofInfo;
     PFILE_END_OF_FILE_INFORMATION pFileEofInfo;
@@ -707,6 +754,8 @@ typedef struct _SRV_TRANS2_STATE_SMB_V1
     USHORT                      usBytesAllocated;
     USHORT                      usBytesUsed;
 
+    BOOLEAN                     bSetInfoAttempted;
+
 } SRV_TRANS2_STATE_SMB_V1, *PSRV_TRANS2_STATE_SMB_V1;
 
 typedef enum
@@ -1126,6 +1175,7 @@ typedef struct __SRV_MESSAGE_SMB_V1
     PANDX_HEADER pAndXHeader;
     USHORT       usHeaderSize;
     ULONG        ulMessageSize;
+    ULONG        ulZctMessageSize;
 
     ULONG        ulBytesAvailable;
 
