@@ -217,6 +217,14 @@ SrvProcessTransaction(
 
     LWIO_LOCK_MUTEX(bInLock, &pTransState->mutex);
 
+    if (pExecContext->pStatInfo)
+    {
+        ntStatus = SrvStatisticsSetSubOpcode(
+                        pExecContext->pStatInfo,
+                        *pTransState->pSetup);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     switch (*pTransState->pSetup)
     {
         case SMB_SUB_COMMAND_TRANS_SET_NAMED_PIPE_HANDLE_STATE:
@@ -331,6 +339,11 @@ SrvProcessSetNamedPipeHandleState(
                                 &pTransState->pSession);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvSetStatSessionInfo(
+                                pExecContext,
+                                pTransState->pSession);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             ntStatus = SrvSessionFindTree_SMB_V1(
                             pCtxSmb1,
                             pTransState->pSession,
@@ -549,6 +562,11 @@ SrvProcessGetNamedPipeHandleState(
                             &pTransState->pSession);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvSetStatSessionInfo(
+                            pExecContext,
+                            pTransState->pSession);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             ntStatus = SrvSessionFindTree_SMB_V1(
                             pCtxSmb1,
                             pTransState->pSession,
@@ -789,6 +807,11 @@ SrvProcessGetNamedPipeInfo(
                             &pTransState->pSession);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvSetStatSessionInfo(
+                            pExecContext,
+                            pTransState->pSession);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             ntStatus = SrvSessionFindTree_SMB_V1(
                             pCtxSmb1,
                             pTransState->pSession,
@@ -1040,6 +1063,11 @@ SrvProcessTransactNamedPipe(
                             &pTransState->pSession);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            ntStatus = SrvSetStatSessionInfo(
+                            pExecContext,
+                            pTransState->pSession);
+            BAIL_ON_NT_STATUS(ntStatus);
+
             ntStatus = SrvSessionFindTree_SMB_V1(
                             pCtxSmb1,
                             pTransState->pSession,
