@@ -176,6 +176,18 @@ SrvBuildQueryInternalInfoResponse(
     PSRV_EXEC_CONTEXT pExecContext
     );
 
+static
+NTSTATUS
+SrvQueryNetworkOpenInfo(
+    PSRV_EXEC_CONTEXT pExecContext
+    );
+
+static
+NTSTATUS
+SrvBuildQueryNetworkOpenInfoResponse(
+    PSRV_EXEC_CONTEXT pExecContext
+    );
+
 static
 NTSTATUS
 SrvQueryAccessInfo(
@@ -257,6 +269,12 @@ SrvQueryInfo(
 
             break;
 
+        case SMB_QUERY_FILE_NETWORK_OPEN_INFO :
+
+            ntStatus = SrvQueryNetworkOpenInfo(pExecContext);
+
+            break;
+
         case SMB_INFO_STANDARD :
 
             ntStatus = SrvQuerySmbStandardInfo(pExecContext);
@@ -281,7 +299,6 @@ SrvQueryInfo(
         case SMB_QUERY_FILE_POSITION_INFO :
         case SMB_QUERY_FILE_MODE_INFO :
         case SMB_QUERY_FILE_ALIGNMENT_INFO :
-        case SMB_QUERY_FILE_NETWORK_OPEN_INFO :
         case SMB_QUERY_FILE_ATTRIBUTE_TAG_INFO :
 
             ntStatus = STATUS_NOT_SUPPORTED;
@@ -367,6 +384,12 @@ SrvBuildQueryInfoResponse(
 
             break;
 
+        case SMB_QUERY_FILE_NETWORK_OPEN_INFO :
+
+            ntStatus = SrvBuildQueryNetworkOpenInfoResponse(pExecContext);
+
+            break;
+
         case SMB_INFO_STANDARD :
 
             ntStatus = SrvBuildQuerySmbStandardInfoResponse(pExecContext);
@@ -390,7 +413,6 @@ SrvBuildQueryInfoResponse(
         case SMB_QUERY_FILE_POSITION_INFO :
         case SMB_QUERY_FILE_MODE_INFO :
         case SMB_QUERY_FILE_ALIGNMENT_INFO :
-        case SMB_QUERY_FILE_NETWORK_OPEN_INFO :
         case SMB_QUERY_FILE_ATTRIBUTE_TAG_INFO :
 
             ntStatus = STATUS_NOT_SUPPORTED;
@@ -2056,19 +2078,21 @@ SrvMarshallFileNameInfo(
     NTSTATUS ntStatus        = STATUS_SUCCESS;
     PBYTE    pData           = NULL;
     USHORT   usBytesRequired = 0;
-    BOOLEAN  bInLock         = FALSE;
+    BOOLEAN  bShareInLock         = FALSE;
     PWSTR     pwszTreePath   = NULL; // Do not free
     PFILE_NAME_INFORMATION pFileNameInfo =
                                         (PFILE_NAME_INFORMATION)pInfoBuffer;
     PTRANS2_FILE_NAME_INFORMATION pFileNameInfoPacked = NULL;
 
-    LWIO_LOCK_RWMUTEX_SHARED(bInLock, &pTree->mutex);
+    LWIO_LOCK_RWMUTEX_SHARED(bShareInLock, &pTree->pShareInfo->mutex);
 
     ntStatus = SrvGetTreeRelativePath(
                     pTree->pShareInfo->pwszPath,
                     &pwszTreePath);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pTree->pShareInfo->mutex);
+
     if (STATUS_SUCCESS != SrvMatchPathPrefix(
                                 pFileNameInfo->FileName,
                                 pFileNameInfo->FileNameLength/sizeof(wchar16_t),
@@ -2102,7 +2126,7 @@ SrvMarshallFileNameInfo(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bInLock, &pTree->mutex);
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pTree->pShareInfo->mutex);
 
     return ntStatus;
 
@@ -2398,6 +2422,168 @@ error:
     goto cleanup;
 }
 
+static
+NTSTATUS
+SrvQueryNetworkOpenInfo(
+    PSRV_EXEC_CONTEXT pExecContext
+    )
+{
+    NTSTATUS                   ntStatus     = 0;
+    PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol = pExecContext->pProtocolContext;
+    PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
+    PSRV_TRANS2_STATE_SMB_V1   pTrans2State = NULL;
+
+    pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
+
+    ntStatus = pTrans2State->ioStatusBlock.Status;
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    if (!pTrans2State->pData2)
+    {
+        ntStatus = SrvAllocateMemory(
+                        sizeof(FILE_NETWORK_OPEN_INFORMATION),
+                        (PVOID*)&pTrans2State->pData2);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        pTrans2State->usBytesAllocated = sizeof(FILE_NETWORK_OPEN_INFORMATION);
+
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+
+        ntStatus = IoQueryInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        pTrans2State->pData2,
+                        pTrans2State->usBytesAllocated,
+                        FileNetworkOpenInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
+
+error:
+
+    return ntStatus;
+}
+
+static
+NTSTATUS
+SrvBuildQueryNetworkOpenInfoResponse(
+    PSRV_EXEC_CONTEXT pExecContext
+    )
+{
+    NTSTATUS ntStatus = 0;
+    PLWIO_SRV_CONNECTION       pConnection  = pExecContext->pConnection;
+    PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol = pExecContext->pProtocolContext;
+    PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
+    ULONG                      iMsg         = pCtxSmb1->iMsg;
+    PSRV_MESSAGE_SMB_V1        pSmbRequest  = &pCtxSmb1->pRequests[iMsg];
+    PSRV_MESSAGE_SMB_V1        pSmbResponse = &pCtxSmb1->pResponses[iMsg];
+    PBYTE   pOutBuffer        = pSmbResponse->pBuffer;
+    ULONG   ulBytesAvailable  = pSmbResponse->ulBytesAvailable;
+    ULONG   ulOffset          = 0;
+    USHORT  usBytesUsed       = 0;
+    ULONG   ulTotalBytesUsed  = 0;
+    USHORT  usParam           = 0;
+    PUSHORT pSetup            = NULL;
+    BYTE    setupCount        = 0;
+    USHORT  usDataOffset      = 0;
+    USHORT  usParameterOffset = 0;
+    PFILE_NETWORK_OPEN_INFORMATION       pFileNetOpenInfo          = NULL;
+    TRANS2_FILE_NETWORK_OPEN_INFORMATION fileNetOpenInfoPacked     = {0};
+    PSRV_TRANS2_STATE_SMB_V1             pTrans2State              = NULL;
+
+    pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
+
+    pFileNetOpenInfo = (PFILE_NETWORK_OPEN_INFORMATION)pTrans2State->pData2;
+
+    if (!pSmbResponse->ulSerialNum)
+    {
+        ntStatus = SrvMarshalHeader_SMB_V1(
+                        pOutBuffer,
+                        ulOffset,
+                        ulBytesAvailable,
+                        COM_TRANSACTION2,
+                        STATUS_SUCCESS,
+                        TRUE,
+                        pTrans2State->pTree->tid,
+                        SMB_V1_GET_PROCESS_ID(pSmbRequest->pHeader),
+                        pTrans2State->pSession->uid,
+                        pSmbRequest->pHeader->mid,
+                        pConnection->serverProperties.bRequireSecuritySignatures,
+                        &pSmbResponse->pHeader,
+                        &pSmbResponse->pWordCount,
+                        &pSmbResponse->pAndXHeader,
+                        &pSmbResponse->usHeaderSize);
+    }
+    else
+    {
+        ntStatus = SrvMarshalHeaderAndX_SMB_V1(
+                        pOutBuffer,
+                        ulOffset,
+                        ulBytesAvailable,
+                        COM_TRANSACTION2,
+                        &pSmbResponse->pWordCount,
+                        &pSmbResponse->pAndXHeader,
+                        &pSmbResponse->usHeaderSize);
+    }
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    pOutBuffer       += pSmbResponse->usHeaderSize;
+    ulOffset         += pSmbResponse->usHeaderSize;
+    ulBytesAvailable -= pSmbResponse->usHeaderSize;
+    ulTotalBytesUsed += pSmbResponse->usHeaderSize;
+
+    *pSmbResponse->pWordCount = 10 + setupCount;
+
+    fileNetOpenInfoPacked.CreationTime = pFileNetOpenInfo->CreationTime;
+    fileNetOpenInfoPacked.LastAccessTime = pFileNetOpenInfo->LastAccessTime;
+    fileNetOpenInfoPacked.LastWriteTime = pFileNetOpenInfo->LastWriteTime;
+    fileNetOpenInfoPacked.ChangeTime = pFileNetOpenInfo->ChangeTime;
+    fileNetOpenInfoPacked.AllocationSize = pFileNetOpenInfo->AllocationSize;
+    fileNetOpenInfoPacked.EndOfFile = pFileNetOpenInfo->EndOfFile;
+    fileNetOpenInfoPacked.FileAttributes = pFileNetOpenInfo->FileAttributes;
+
+    ntStatus = WireMarshallTransaction2Response(
+                    pOutBuffer,
+                    ulBytesAvailable,
+                    ulOffset,
+                    pSetup,
+                    setupCount,
+                    (PBYTE)&usParam,
+                    sizeof(usParam),
+                    (PBYTE)&fileNetOpenInfoPacked,
+                    sizeof(fileNetOpenInfoPacked),
+                    &usDataOffset,
+                    &usParameterOffset,
+                    &usBytesUsed);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    // pOutBuffer       += usBytesUsed;
+    // ulOffset         += usBytesUsed;
+    // ulBytesAvailable -= usBytesUsed;
+    ulTotalBytesUsed += usBytesUsed;
+
+    pSmbResponse->ulMessageSize = ulTotalBytesUsed;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    if (ulTotalBytesUsed)
+    {
+        pSmbResponse->pHeader = NULL;
+        pSmbResponse->pAndXHeader = NULL;
+        memset(pSmbResponse->pBuffer, 0, ulTotalBytesUsed);
+    }
+
+    pSmbResponse->ulMessageSize = 0;
+
+    goto cleanup;
+}
 
 static
 NTSTATUS
