@@ -109,7 +109,7 @@ SrvSetFileInfo(
         case SMB_SET_FILE_ALLOCATION_INFO :
         case SMB_SET_FILE_ALLOCATION_INFO_ALIAS:
 
-	    ntStatus = SrvSetAllocationInfo(pExecContext);
+            ntStatus = SrvSetAllocationInfo(pExecContext);
 
             break;
 
@@ -184,7 +184,7 @@ SrvSetPathInfo(
         case SMB_SET_FILE_ALLOCATION_INFO :
         case SMB_SET_FILE_ALLOCATION_INFO_ALIAS:
 
-	    ntStatus = SrvSetAllocationInfo(pExecContext);
+            ntStatus = SrvSetAllocationInfo(pExecContext);
 
             break;
 
@@ -372,7 +372,6 @@ SrvSetBasicInfo(
     PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol   = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1       = pCtxProtocol->pSmb1Context;
     PSRV_TRANS2_STATE_SMB_V1   pTrans2State   = NULL;
-    PFILE_BASIC_INFORMATION    pFileBasicInfo = NULL;
 
     pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -382,21 +381,28 @@ SrvSetBasicInfo(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    pFileBasicInfo = (PFILE_BASIC_INFORMATION)pTrans2State->pData;
 
-    SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+    if (!pTrans2State->bSetInfoAttempted)
+    {
+        PFILE_BASIC_INFORMATION pFileBasicInfo =
+                            (PFILE_BASIC_INFORMATION)pTrans2State->pData;
 
-    ntStatus = IoSetInformationFile(
-                    (pTrans2State->pFile ? pTrans2State->pFile->hFile :
-                                           pTrans2State->hFile),
-                    pTrans2State->pAcb,
-                    &pTrans2State->ioStatusBlock,
-                    pFileBasicInfo,
-                    sizeof(FILE_BASIC_INFORMATION),
-                    FileBasicInformation);
-    BAIL_ON_NT_STATUS(ntStatus);
+        pTrans2State->bSetInfoAttempted = TRUE;
 
-    SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+
+        ntStatus = IoSetInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        pFileBasicInfo,
+                        sizeof(FILE_BASIC_INFORMATION),
+                        FileBasicInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
 
 error:
 
@@ -413,7 +419,6 @@ SrvSetDispositionInfo(
     PSRV_PROTOCOL_EXEC_CONTEXT    pCtxProtocol = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1      pCtxSmb1     = pCtxProtocol->pSmb1Context;
     PSRV_TRANS2_STATE_SMB_V1      pTrans2State = NULL;
-    PFILE_DISPOSITION_INFORMATION pFileDispositionInfo = NULL;
 
     pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -423,21 +428,29 @@ SrvSetDispositionInfo(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    pFileDispositionInfo = (PFILE_DISPOSITION_INFORMATION)pTrans2State->pData;
+    if (!pTrans2State->bSetInfoAttempted)
+    {
+        PFILE_DISPOSITION_INFORMATION pFileDispositionInfo =
+                        (PFILE_DISPOSITION_INFORMATION)pTrans2State->pData;
 
-    SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+        SrvUnmarshallBoolean(&pFileDispositionInfo->DeleteFile);
 
-    ntStatus = IoSetInformationFile(
-                    (pTrans2State->pFile ? pTrans2State->pFile->hFile :
-                                           pTrans2State->hFile),
-                    pTrans2State->pAcb,
-                    &pTrans2State->ioStatusBlock,
-                    pFileDispositionInfo,
-                    sizeof(FILE_DISPOSITION_INFORMATION),
-                    FileDispositionInformation);
-    BAIL_ON_NT_STATUS(ntStatus);
+		pTrans2State->bSetInfoAttempted = TRUE;
 
-    SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+
+        ntStatus = IoSetInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        pFileDispositionInfo,
+                        sizeof(FILE_DISPOSITION_INFORMATION),
+                        FileDispositionInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
 
 error:
 
@@ -454,7 +467,6 @@ SrvSetAllocationInfo(
     PSRV_PROTOCOL_EXEC_CONTEXT   pCtxProtocol = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1     pCtxSmb1     = pCtxProtocol->pSmb1Context;
     PSRV_TRANS2_STATE_SMB_V1     pTrans2State = NULL;
-    PFILE_ALLOCATION_INFORMATION pFileAllocationInfo = NULL;
 
     pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -464,21 +476,27 @@ SrvSetAllocationInfo(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    pFileAllocationInfo = (PFILE_ALLOCATION_INFORMATION)pTrans2State->pData;
+    if (!pTrans2State->bSetInfoAttempted)
+    {
+        PFILE_ALLOCATION_INFORMATION pFileAllocationInfo =
+                            (PFILE_ALLOCATION_INFORMATION)pTrans2State->pData;
 
-    SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+		pTrans2State->bSetInfoAttempted = TRUE;
 
-    ntStatus = IoSetInformationFile(
-                    (pTrans2State->pFile ? pTrans2State->pFile->hFile :
-                                           pTrans2State->hFile),
-                    pTrans2State->pAcb,
-                    &pTrans2State->ioStatusBlock,
-                    pFileAllocationInfo,
-                    sizeof(FILE_ALLOCATION_INFORMATION),
-                    FileAllocationInformation);
-    BAIL_ON_NT_STATUS(ntStatus);
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
 
-    SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+        ntStatus = IoSetInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        pFileAllocationInfo,
+                        sizeof(FILE_ALLOCATION_INFORMATION),
+                        FileAllocationInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
 
 error:
 
@@ -495,7 +513,6 @@ SrvSetEndOfFileInfo(
     PSRV_PROTOCOL_EXEC_CONTEXT    pCtxProtocol = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1      pCtxSmb1     = pCtxProtocol->pSmb1Context;
     PSRV_TRANS2_STATE_SMB_V1      pTrans2State = NULL;
-    PFILE_END_OF_FILE_INFORMATION pFileEofInfo = NULL;
 
     pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -505,21 +522,27 @@ SrvSetEndOfFileInfo(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    pFileEofInfo = (PFILE_END_OF_FILE_INFORMATION)pTrans2State->pData;
+    if (!pTrans2State->bSetInfoAttempted)
+    {
+        PFILE_END_OF_FILE_INFORMATION pFileEofInfo =
+                            (PFILE_END_OF_FILE_INFORMATION)pTrans2State->pData;
 
-    SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+		pTrans2State->bSetInfoAttempted = TRUE;
 
-    ntStatus = IoSetInformationFile(
-                    (pTrans2State->pFile ? pTrans2State->pFile->hFile :
-                                           pTrans2State->hFile),
-                    pTrans2State->pAcb,
-                    &pTrans2State->ioStatusBlock,
-                    pFileEofInfo,
-                    sizeof(FILE_END_OF_FILE_INFORMATION),
-                    FileEndOfFileInformation);
-    BAIL_ON_NT_STATUS(ntStatus);
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
 
-    SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+        ntStatus = IoSetInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        pFileEofInfo,
+                        sizeof(FILE_END_OF_FILE_INFORMATION),
+                        FileEndOfFileInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
 
 error:
 
@@ -536,7 +559,6 @@ SrvRenameFile(
     PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
     PSRV_TRANS2_STATE_SMB_V1   pTrans2State = NULL;
-    BOOLEAN                    bTreeInLock  = FALSE;
 
     pTrans2State = (PSRV_TRANS2_STATE_SMB_V1)pCtxSmb1->hState;
 
@@ -580,6 +602,8 @@ SrvRenameFile(
         }
         BAIL_ON_NT_STATUS(ntStatus);
 
+        pTrans2State->dirPath.RootFileHandle = pTrans2State->pTree->hFile;
+
         // Catch failed CreateFile calls when they come back around
 
         ntStatus = pTrans2State->ioStatusBlock.Status;
@@ -604,7 +628,7 @@ SrvRenameFile(
                                 FILE_DIRECTORY_FILE,
                                 NULL, /* EA Buffer */
                                 0,    /* EA Length */
-                                &pTrans2State->pEcpList
+                                pTrans2State->pEcpList
                                 );
         BAIL_ON_NT_STATUS(ntStatus);
 
@@ -633,9 +657,6 @@ SrvRenameFile(
 
 error:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                        &pCtxSmb1->pTree->pShareInfo->mutex);
-
     return ntStatus;
 }
 
@@ -749,19 +770,24 @@ SrvSetEaList(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
+    if (!pTrans2State->bSetInfoAttempted)
+    {
+        SrvPrepareTrans2StateAsync(pTrans2State, pExecContext);
 
-    ntStatus = IoSetInformationFile(
-                    (pTrans2State->pFile ? pTrans2State->pFile->hFile :
-                                           pTrans2State->hFile),
-                    pTrans2State->pAcb,
-                    &pTrans2State->ioStatusBlock,
-                    (PFILE_FULL_EA_INFORMATION)pTrans2State->pData2,
-                    pTrans2State->usBytesAllocated,
-                    FileFullEaInformation);
-    BAIL_ON_NT_STATUS(ntStatus);
+		pTrans2State->bSetInfoAttempted = TRUE;
 
-    SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+        ntStatus = IoSetInformationFile(
+                        (pTrans2State->pFile ? pTrans2State->pFile->hFile :
+                                               pTrans2State->hFile),
+                        pTrans2State->pAcb,
+                        &pTrans2State->ioStatusBlock,
+                        (PFILE_FULL_EA_INFORMATION)pTrans2State->pData2,
+                        pTrans2State->usBytesAllocated,
+                        FileFullEaInformation);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTrans2StateAsync(pTrans2State); // completed synchronously
+    }
 
 error:
 
