@@ -33,6 +33,7 @@
 static
 NTSTATUS
 SrvBuildTreeConnectState(
+    PLWIO_SRV_CONNECTION            pConnection,
     PTREE_CONNECT_REQUEST_HEADER    pRequestHeader,
     PBYTE                           pszPassword,
     PBYTE                           pszService,
@@ -65,6 +66,25 @@ SrvReleaseTreeConnectStateAsync(
     PSRV_TREE_CONNECT_STATE_SMB_V1 pTConState
     );
 
+static
+NTSTATUS
+SrvCreateTreeRootHandle(
+    PSRV_EXEC_CONTEXT pExecContext
+    );
+
+static
+NTSTATUS
+SrvIoPrepareEcpList(
+    PSRV_TREE_CONNECT_STATE_SMB_V1 pTConState,  /* IN     */
+    PIO_ECP_LIST*                  ppEcpList    /* IN OUT */
+    );
+
+static
+VOID
+SrvIoFreeEcpString(
+    IN PVOID pContext
+    );
+
 static
 NTSTATUS
 SrvBuildTreeConnectResponse(
@@ -147,6 +167,9 @@ SrvProcessTreeConnectAndX(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = UnmarshallTreeConnectRequest(
                         pBuffer,
                         ulBytesAvailable,
@@ -174,6 +197,7 @@ SrvProcessTreeConnectAndX(
         }
 
         ntStatus = SrvBuildTreeConnectState(
+                        pConnection,
                         pRequestHeader,
                         pszPassword,
                         pszService,
@@ -227,7 +251,25 @@ SrvProcessTreeConnectAndX(
 
             pTConState->bRemoveTreeFromSession = TRUE;
 
-            pTConState->stage = SRV_TREE_CONNECT_STAGE_SMB_V1_ATTEMPT_QUERY_INFO;
+            pTConState->stage =
+                    SRV_TREE_CONNECT_STAGE_SMB_V1_CREATE_TREE_ROOT_HANDLE;
+
+            // Intentional fall through
+
+        case SRV_TREE_CONNECT_STAGE_SMB_V1_CREATE_TREE_ROOT_HANDLE:
+
+            if (pTConState->pTree->pShareInfo->service ==
+                    SHARE_SERVICE_DISK_SHARE)
+            {
+                ntStatus = pTConState->ioStatusBlock.Status;
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                ntStatus = SrvCreateTreeRootHandle(pExecContext);
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
+            pTConState->stage =
+                    SRV_TREE_CONNECT_STAGE_SMB_V1_ATTEMPT_QUERY_INFO;
 
             // Intentional fall through
 
@@ -241,7 +283,8 @@ SrvProcessTreeConnectAndX(
             ntStatus = SrvQueryTreeConnectInfo(pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            pTConState->stage = SRV_TREE_CONNECT_STAGE_SMB_V1_QUERY_INFO_COMPLETED;
+            pTConState->stage =
+                    SRV_TREE_CONNECT_STAGE_SMB_V1_QUERY_INFO_COMPLETED;
 
             // Intentional fall through
 
@@ -323,6 +366,7 @@ error:
 static
 NTSTATUS
 SrvBuildTreeConnectState(
+    PLWIO_SRV_CONNECTION            pConnection,
     PTREE_CONNECT_REQUEST_HEADER    pRequestHeader,
     PBYTE                           pszPassword,
     PBYTE                           pszService,
@@ -350,6 +394,9 @@ SrvBuildTreeConnectState(
     pTConState->pszService     = pszService;
     pTConState->pwszPath       = pwszPath;
 
+    pTConState->clientAddress         = pConnection->clientAddress;
+    pTConState->ulClientAddressLength = pConnection->clientAddrLen;
+
     *ppTConState = pTConState;
 
 cleanup:
@@ -499,6 +546,156 @@ SrvReleaseTreeConnectStateAsync(
     }
 }
 
+static
+NTSTATUS
+SrvCreateTreeRootHandle(
+    PSRV_EXEC_CONTEXT pExecContext
+    )
+{
+    NTSTATUS                   ntStatus     = STATUS_SUCCESS;
+    PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol = pExecContext->pProtocolContext;
+    PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
+    PSRV_TREE_CONNECT_STATE_SMB_V1 pTConState  = NULL;
+    BOOLEAN                        bShareInLock = FALSE;
+
+    pTConState = (PSRV_TREE_CONNECT_STATE_SMB_V1)pCtxSmb1->hState;
+
+    if (!pTConState->fileName.FileName)
+    {
+        LWIO_LOCK_RWMUTEX_SHARED(bShareInLock, &pTConState->pShareInfo->mutex);
+
+        ntStatus = SrvAllocateStringW(
+                        pTConState->pShareInfo->pwszPath,
+                        &pTConState->fileName.FileName);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        LWIO_UNLOCK_RWMUTEX(bShareInLock, &pTConState->pShareInfo->mutex);
+
+        ntStatus = SrvIoPrepareEcpList(
+                        pTConState,
+                        &pTConState->pEcpList);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    if (!pTConState->pTree->hFile)
+    {
+        SrvPrepareTreeConnectStateAsync(pTConState, pExecContext);
+
+        ntStatus = SrvIoCreateFile(
+                        pTConState->pTree->pShareInfo,
+                        &pTConState->pTree->hFile,
+                        pTConState->pAcb,
+                        &pTConState->ioStatusBlock,
+                        pTConState->pSession->pIoSecurityContext,
+                        &pTConState->fileName,
+                        pTConState->pSecurityDescriptor,
+                        pTConState->pSecurityQOS,
+                        FILE_READ_ATTRIBUTES,
+                        0,
+                        FILE_ATTRIBUTE_NORMAL,
+                        FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
+                        FILE_OPEN,
+                        0,
+                        NULL, /* EA Buffer */
+                        0,    /* EA Length */
+                        pTConState->pEcpList
+                        );
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        SrvReleaseTreeConnectStateAsync(pTConState); // completed synchronously
+    }
+
+cleanup:
+
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pTConState->pShareInfo->mutex);
+
+    return ntStatus;
+
+error:
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SrvIoPrepareEcpList(
+    PSRV_TREE_CONNECT_STATE_SMB_V1 pTConState,  /* IN     */
+    PIO_ECP_LIST*                  ppEcpList    /* IN OUT */
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    PUNICODE_STRING pShareName = NULL;
+
+    if (SrvElementsGetShareNameEcpEnabled())
+    {
+        if (!*ppEcpList)
+        {
+            ntStatus = IoRtlEcpListAllocate(ppEcpList);
+            BAIL_ON_NT_STATUS(ntStatus);
+        }
+
+        ntStatus = RTL_ALLOCATE(&pShareName, UNICODE_STRING, sizeof(*pShareName));
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        ntStatus = RtlUnicodeStringAllocateFromWC16String(
+                        pShareName,
+                        pTConState->pTree->pShareInfo->pwszName);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        ntStatus = IoRtlEcpListInsert(
+                        *ppEcpList,
+                        SRV_ECP_TYPE_SHARE_NAME,
+                        pShareName,
+                        sizeof(*pShareName),
+                        SrvIoFreeEcpString);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        pShareName = NULL;
+    }
+
+    if (SrvElementsGetClientAddressEcpEnabled())
+    {
+        if (!*ppEcpList)
+        {
+            ntStatus = IoRtlEcpListAllocate(ppEcpList);
+            BAIL_ON_NT_STATUS(ntStatus);
+        }
+
+        ntStatus = IoRtlEcpListInsert(
+                        *ppEcpList,
+                        SRV_ECP_TYPE_CLIENT_ADDRESS,
+                        &pTConState->clientAddress,
+                        pTConState->ulClientAddressLength,
+                        NULL);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+cleanup:
+
+    RTL_FREE(&pShareName);
+
+    return ntStatus;
+
+error:
+
+    goto cleanup;
+}
+
+static
+VOID
+SrvIoFreeEcpString(
+    IN PVOID pContext
+    )
+{
+    PUNICODE_STRING pEcpString = (PUNICODE_STRING) pContext;
+
+    if (pEcpString)
+    {
+        RtlUnicodeStringFree(pEcpString);
+        RtlMemoryFree(pEcpString);
+    }
+}
+
 static
 NTSTATUS
 SrvBuildTreeConnectResponse(
@@ -660,51 +857,10 @@ SrvGetNativeFilesystem(
     PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
     PSRV_TREE_CONNECT_STATE_SMB_V1 pTConState  = NULL;
     PFILE_FS_ATTRIBUTE_INFORMATION pFsAttrInfo = NULL;
-    BOOLEAN  bInLock = FALSE;
     BOOLEAN  bContinue = TRUE;
 
     pTConState = (PSRV_TREE_CONNECT_STATE_SMB_V1)pCtxSmb1->hState;
 
-    if (!pTConState->fileName.FileName)
-    {
-        LWIO_LOCK_RWMUTEX_SHARED(bInLock, &pTConState->pShareInfo->mutex);
-
-        ntStatus = SrvAllocateStringW(
-                        pTConState->pShareInfo->pwszPath,
-                        &pTConState->fileName.FileName);
-        BAIL_ON_NT_STATUS(ntStatus);
-
-        LWIO_UNLOCK_RWMUTEX(bInLock, &pTConState->pShareInfo->mutex);
-    }
-
-    if (!pTConState->hFile)
-    {
-        SrvPrepareTreeConnectStateAsync(pTConState, pExecContext);
-
-        ntStatus = SrvIoCreateFile(
-                        pTConState->pTree->pShareInfo,
-                        &pTConState->hFile,
-                        pTConState->pAcb,
-                        &pTConState->ioStatusBlock,
-                        pTConState->pSession->pIoSecurityContext,
-                        &pTConState->fileName,
-                        pTConState->pSecurityDescriptor,
-                        pTConState->pSecurityQOS,
-                        FILE_READ_ATTRIBUTES,
-                        0,
-                        FILE_ATTRIBUTE_NORMAL,
-                        FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
-                        FILE_OPEN,
-                        0,
-                        NULL, /* EA Buffer */
-                        0,    /* EA Length */
-                        &pTConState->pEcpList
-                        );
-        BAIL_ON_NT_STATUS(ntStatus);
-
-        SrvReleaseTreeConnectStateAsync(pTConState); // completed synchronously
-    }
-
     if (!pTConState->pVolumeInfo)
     {
         pTConState->ioStatusBlock.Status = STATUS_BUFFER_TOO_SMALL;
@@ -748,7 +904,7 @@ SrvGetNativeFilesystem(
         SrvPrepareTreeConnectStateAsync(pTConState, pExecContext);
 
         ntStatus = IoQueryVolumeInformationFile(
-                        pTConState->hFile,
+                        pTConState->pTree->hFile,
                         pTConState->pAcb,
                         &pTConState->ioStatusBlock,
                         pTConState->pVolumeInfo,
@@ -791,8 +947,6 @@ SrvGetNativeFilesystem(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bInLock, &pTConState->pShareInfo->mutex);
-
     return ntStatus;
 
 error:
@@ -851,11 +1005,6 @@ SrvFreeTreeConnectState(
         SrvShareReleaseInfo(pTConState->pShareInfo);
     }
 
-    if (pTConState->hFile)
-    {
-        IoCloseFile(pTConState->hFile);
-    }
-
     if (pTConState->pszService2)
     {
         SrvFreeMemory(pTConState->pszService2);
