@@ -32,16 +32,22 @@
 
 static
 NTSTATUS
-SrvBuildWriteXState(
-    PWRITE_ANDX_REQUEST_HEADER pRequestHeader,
-    PBYTE                      pData,
-    PLWIO_SRV_FILE             pFile,
-    PSRV_WRITEX_STATE_SMB_V1*  ppWriteState
+SrvExecuteWriteAndX(
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
+    PSRV_EXEC_CONTEXT pExecContext
     );
 
 static
 NTSTATUS
-SrvExecuteWriteAndX(
+SrvExecuteWriteXZctIo(
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
+    PSRV_EXEC_CONTEXT pExecContext
+    );
+
+static
+NTSTATUS
+SrvExecuteWriteXZctComplete(
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
     PSRV_EXEC_CONTEXT pExecContext
     );
 
@@ -58,6 +64,13 @@ SrvExecuteWriteXAsyncCB(
     PVOID pContext
     );
 
+static
+VOID
+SrvExecuteWriteXReceiveZctCB(
+    IN PVOID pContext,
+    IN NTSTATUS Status
+    );
+
 static
 VOID
 SrvReleaseWriteXStateAsync(
@@ -70,12 +83,6 @@ SrvBuildWriteAndXResponse(
     PSRV_EXEC_CONTEXT pExecContext
     );
 
-static
-VOID
-SrvReleaseWriteXStateHandle(
-    HANDLE hWriteState
-    );
-
 static
 VOID
 SrvReleaseWriteXState(
@@ -116,7 +123,8 @@ SrvProcessWriteAndX(
         PBYTE pBuffer          = pSmbRequest->pBuffer + pSmbRequest->usHeaderSize;
         ULONG ulOffset         = pSmbRequest->usHeaderSize;
         ULONG ulBytesAvailable = pSmbRequest->ulMessageSize - pSmbRequest->usHeaderSize;
-        PWRITE_ANDX_REQUEST_HEADER pRequestHeader = NULL; // Do not free
+        PWRITE_ANDX_REQUEST_HEADER_WC_12 pRequestHeader_WC_12 = NULL; // Do not free
+        PWRITE_ANDX_REQUEST_HEADER_WC_14 pRequestHeader_WC_14 = NULL; // Do not free
         PBYTE                      pData          = NULL; // Do not free
 
         ntStatus = SrvConnectionFindSession_SMB_V1(
@@ -126,6 +134,9 @@ SrvProcessWriteAndX(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSessionInfo(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSessionFindTree_SMB_V1(
                         pCtxSmb1,
                         pSession,
@@ -133,23 +144,58 @@ SrvProcessWriteAndX(
                         &pTree);
         BAIL_ON_NT_STATUS(ntStatus);
 
-        ntStatus = WireUnmarshallWriteAndXRequest(
-                        pBuffer,
-                        ulBytesAvailable,
-                        ulOffset,
-                        &pRequestHeader,
-                        &pData);
-        BAIL_ON_NT_STATUS(ntStatus);
+        switch (*pSmbRequest->pWordCount)
+        {
+            case 12:
 
-        ntStatus = SrvTreeFindFile_SMB_V1(
-                        pCtxSmb1,
-                        pTree,
-                        pRequestHeader->fid,
-                        &pFile);
-        BAIL_ON_NT_STATUS(ntStatus);
+                ntStatus = WireUnmarshallWriteAndXRequest_WC_12(
+                                pBuffer,
+                                ulBytesAvailable,
+                                ulOffset,
+                                &pRequestHeader_WC_12,
+                                &pData);
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                ntStatus = SrvTreeFindFile_SMB_V1(
+                                pCtxSmb1,
+                                pTree,
+                                pRequestHeader_WC_12->fid,
+                                &pFile);
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                break;
+
+            case 14:
+
+                ntStatus = WireUnmarshallWriteAndXRequest_WC_14(
+                                pBuffer,
+                                ulBytesAvailable,
+                                ulOffset,
+                                &pRequestHeader_WC_14,
+                                &pData);
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                ntStatus = SrvTreeFindFile_SMB_V1(
+                                pCtxSmb1,
+                                pTree,
+                                pRequestHeader_WC_14->fid,
+                                &pFile);
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                break;
+
+            default:
+
+                ntStatus = STATUS_INVALID_NETWORK_RESPONSE;
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                break;
+        }
 
         ntStatus = SrvBuildWriteXState(
-                        pRequestHeader,
+                        *pSmbRequest->pWordCount,
+                        pRequestHeader_WC_12,
+                        pRequestHeader_WC_14,
                         pData,
                         pFile,
                         &pWriteState);
@@ -166,20 +212,6 @@ SrvProcessWriteAndX(
     {
         case SRV_WRITEX_STAGE_SMB_V1_INITIAL:
 
-            pWriteState->llDataOffset =
-                (((LONG64)pWriteState->pRequestHeader->offsetHigh) << 32) |
-                ((LONG64)pWriteState->pRequestHeader->offset);
-
-            pWriteState->llDataLength =
-                (((LONG64)pWriteState->pRequestHeader->dataLengthHigh) << 16) |
-                ((LONG64)pWriteState->pRequestHeader->dataLength);
-
-            if (pWriteState->llDataLength > UINT32_MAX)
-            {
-                ntStatus = STATUS_INVALID_PARAMETER;
-                BAIL_ON_NT_STATUS(ntStatus);
-            }
-
             pWriteState->ulKey = pSmbRequest->pHeader->pid;
 
             pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_ATTEMPT_WRITE;
@@ -188,9 +220,39 @@ SrvProcessWriteAndX(
 
         case SRV_WRITEX_STAGE_SMB_V1_ATTEMPT_WRITE:
 
-            ntStatus = SrvExecuteWriteAndX(pExecContext);
+            ntStatus = SrvExecuteWriteAndX(pWriteState, pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
+            pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_ZCT_IO;
+
+            // intentional fall through
+
+        case SRV_WRITEX_STAGE_SMB_V1_ZCT_IO:
+
+            if (pWriteState->zct.pZct)
+            {
+                ntStatus = SrvExecuteWriteXZctIo(pWriteState, pExecContext);
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
+            pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_ZCT_COMPLETE;
+
+            // intentional fall through
+
+        case SRV_WRITEX_STAGE_SMB_V1_ZCT_COMPLETE:
+
+            if (pWriteState->zct.pZct)
+            {
+                LwZctDestroy(&pWriteState->zct.pZct);
+
+                SrvProtocolTransportResumeFromZct(pWriteState->zct.pPausedConnection);
+                SrvConnectionRelease(pWriteState->zct.pPausedConnection);
+                pWriteState->zct.pPausedConnection = NULL;
+
+                ntStatus = SrvExecuteWriteXZctComplete(pWriteState, pExecContext);
+                BAIL_ON_NT_STATUS(ntStatus);
+            }
+
             pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_BUILD_RESPONSE;
 
             // intentional fall through
@@ -258,13 +320,14 @@ error:
     goto cleanup;
 }
 
-static
 NTSTATUS
 SrvBuildWriteXState(
-    PWRITE_ANDX_REQUEST_HEADER pRequestHeader,
-    PBYTE                      pData,
-    PLWIO_SRV_FILE             pFile,
-    PSRV_WRITEX_STATE_SMB_V1*  ppWriteState
+    UCHAR                            ucWordCount,
+    PWRITE_ANDX_REQUEST_HEADER_WC_12 pRequestHeader_WC_12,
+    PWRITE_ANDX_REQUEST_HEADER_WC_14 pRequestHeader_WC_14,
+    PBYTE                            pData,
+    PLWIO_SRV_FILE                   pFile,
+    PSRV_WRITEX_STATE_SMB_V1*        ppWriteState
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
@@ -282,10 +345,43 @@ SrvBuildWriteXState(
 
     pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_INITIAL;
 
-    pWriteState->pRequestHeader = pRequestHeader;
-    pWriteState->pData          = pData;
     pWriteState->pFile          = SrvFileAcquire(pFile);
 
+    pWriteState->ucWordCount = ucWordCount;
+
+    switch (ucWordCount)
+    {
+        case 12:
+
+            pWriteState->pRequestHeader_WC_12 = pRequestHeader_WC_12;
+            pWriteState->llOffset =
+                ((LONG64)pWriteState->pRequestHeader_WC_12->offset);
+            pWriteState->ulLength =
+                (((LONG64)pWriteState->pRequestHeader_WC_12->dataLengthHigh) << 16) |
+                ((LONG64)pWriteState->pRequestHeader_WC_12->dataLength);
+            break;
+
+        case 14:
+
+            pWriteState->pRequestHeader_WC_14 = pRequestHeader_WC_14;
+            pWriteState->llOffset =
+                (((LONG64)pWriteState->pRequestHeader_WC_14->offsetHigh) << 32) |
+                ((LONG64)pWriteState->pRequestHeader_WC_14->offset);
+            pWriteState->ulLength =
+                (((LONG64)pWriteState->pRequestHeader_WC_14->dataLengthHigh) << 16) |
+                ((LONG64)pWriteState->pRequestHeader_WC_14->dataLength);
+            break;
+
+        default:
+
+            ntStatus = STATUS_INVALID_PARAMETER;
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            break;
+    }
+
+    pWriteState->pData          = pData;
+
     *ppWriteState = pWriteState;
 
 cleanup:
@@ -307,63 +403,249 @@ error:
 static
 NTSTATUS
 SrvExecuteWriteAndX(
-    PSRV_EXEC_CONTEXT pExecContext
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
+    PSRV_EXEC_CONTEXT        pExecContext
     )
 {
-    NTSTATUS                   ntStatus     = 0;
-    PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol = pExecContext->pProtocolContext;
-    PSRV_EXEC_CONTEXT_SMB_V1   pCtxSmb1     = pCtxProtocol->pSmb1Context;
-    PSRV_WRITEX_STATE_SMB_V1   pWriteState  = NULL;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
 
-    pWriteState = (PSRV_WRITEX_STATE_SMB_V1)pCtxSmb1->hState;
-
-    ntStatus = pWriteState->ioStatusBlock.Status; // async response status
-    BAIL_ON_NT_STATUS(ntStatus);
-
-    pWriteState->llDataLength -=
-                pWriteState->ioStatusBlock.BytesTransferred;
-    pWriteState->ullBytesWritten +=
-                pWriteState->ioStatusBlock.BytesTransferred;
-
-    while (pWriteState->llDataLength > 0)
+    if (pWriteState->zct.pZct)
     {
-        ULONG ulDataLength = 0;
-
-        if (pWriteState->llDataLength > UINT32_MAX)
+        if (!pWriteState->bStartedIo)
         {
-            ulDataLength = UINT32_MAX;
+            SrvPrepareWriteXStateAsync(pWriteState, pExecContext);
+            pWriteState->bStartedIo = TRUE;
+
+            ntStatus = IoPrepareZctWriteFile(
+                            pWriteState->pFile->hFile,
+                            pWriteState->pAcb,
+                            &pWriteState->ioStatusBlock,
+                            0,
+                            pWriteState->zct.pZct,
+                            pWriteState->ulLength,
+                            &pWriteState->llOffset,
+                            &pWriteState->ulKey,
+                            &pWriteState->zct.pZctCompletion);
+            if (ntStatus == STATUS_NOT_SUPPORTED)
+            {
+                // Retry as non-ZCT
+                LwZctDestroy(&pWriteState->zct.pZct);
+                pWriteState->ioStatusBlock.Status = ntStatus = STATUS_SUCCESS;
+            }
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            // completed synchronously
+            SrvReleaseWriteXStateAsync(pWriteState);
+        }
+
+        pWriteState->bStartedIo = FALSE;
+
+        ntStatus = pWriteState->ioStatusBlock.Status;
+        if (ntStatus == STATUS_NOT_SUPPORTED)
+        {
+            // Retry as non-ZCT
+            LwZctDestroy(&pWriteState->zct.pZct);
+            pWriteState->ioStatusBlock.Status = ntStatus = STATUS_SUCCESS;
+        }
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        if (!pWriteState->zct.pZct)
+        {
+            // Cannot do ZCT
+
+            // Read rest of data into packet.
+            ntStatus = SrvProtocolTransportContinueAsNonZct(
+                            pExecContext->pConnection,
+                            pExecContext);
+
+            // For now, returns STATUS_PENDING.
+            LWIO_ASSERT(STATUS_PENDING == ntStatus);
+
+            SrvConnectionRelease(pWriteState->zct.pPausedConnection);
+            pWriteState->zct.pPausedConnection = NULL;
+
+            BAIL_ON_NT_STATUS(ntStatus);
         }
         else
         {
-            ulDataLength = (ULONG)pWriteState->llDataLength;
+            ULONG zctLength = LwZctGetLength(pWriteState->zct.pZct);
+
+            LWIO_ASSERT(zctLength == pWriteState->ulLength);
+        }
+    }
+
+    // This will also retry as non-ZCT if ZCT failed above.
+    if (!pWriteState->zct.pZct)
+    {
+        if (!pWriteState->bStartedIo)
+        {
+            SrvPrepareWriteXStateAsync(pWriteState, pExecContext);
+
+            ntStatus = IoWriteFile(
+                            pWriteState->pFile->hFile,
+                            pWriteState->pAcb,
+                            &pWriteState->ioStatusBlock,
+                            pWriteState->pData,
+                            pWriteState->ulLength,
+                            &pWriteState->llOffset,
+                            &pWriteState->ulKey);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            // completed synchronously
+            SrvReleaseWriteXStateAsync(pWriteState);
         }
 
-        SrvPrepareWriteXStateAsync(pWriteState, pExecContext);
+        pWriteState->bStartedIo = FALSE;
 
-        ntStatus = IoWriteFile(
-                        pWriteState->pFile->hFile,
-                        pWriteState->pAcb,
-                        &pWriteState->ioStatusBlock,
-                        pWriteState->pData + pWriteState->ullBytesWritten,
-                        ulDataLength,
-                        &pWriteState->llDataOffset,
-                        &pWriteState->ulKey);
+        ntStatus = pWriteState->ioStatusBlock.Status;
         BAIL_ON_NT_STATUS(ntStatus);
-
-        SrvReleaseWriteXStateAsync(pWriteState); // completed synchronously
-
-        pWriteState->llDataLength -=
-                    pWriteState->ioStatusBlock.BytesTransferred;
-        pWriteState->ullBytesWritten +=
-                    pWriteState->ioStatusBlock.BytesTransferred;
     }
 
+    pWriteState->ulBytesWritten =
+            pWriteState->ioStatusBlock.BytesTransferred;
+
 cleanup:
 
     return ntStatus;
 
 error:
 
+    if (ntStatus != STATUS_PENDING)
+    {
+        pWriteState->bStartedIo = FALSE;
+    }
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SrvExecuteWriteXZctIo(
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
+    PSRV_EXEC_CONTEXT pExecContext
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    PSRV_EXEC_CONTEXT pZctContext = NULL;
+
+    if (pWriteState->zct.ulSkipBytes)
+    {
+        // Skip extra padding
+        LW_ZCT_ENTRY entry;
+
+        ntStatus = SrvAllocateMemory(pWriteState->zct.ulSkipBytes, OUT_PPVOID(&pWriteState->zct.pPadding));
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        entry.Type = LW_ZCT_ENTRY_TYPE_MEMORY;
+        entry.Length = pWriteState->zct.ulSkipBytes;
+        entry.Data.Memory.Buffer = pWriteState->zct.pPadding;
+
+        ntStatus = LwZctPrepend(pWriteState->zct.pZct, &entry, 1);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        pWriteState->zct.ulPaddingSize = pWriteState->zct.ulSkipBytes;
+        pWriteState->zct.ulSkipBytes = 0;
+    }
+
+    ntStatus = LwZctPrepareIo(pWriteState->zct.pZct);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    if (pWriteState->zct.ulDataBytesResident)
+    {
+        // Copy any bytes we already have
+        ntStatus = LwZctReadBufferIo(
+                        pWriteState->zct.pZct,
+                        pWriteState->pData,
+                        pWriteState->zct.ulDataBytesResident,
+                        &pWriteState->ulBytesWritten,
+                        NULL);
+        LWIO_ASSERT(STATUS_MORE_PROCESSING_REQUIRED != ntStatus);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        LWIO_ASSERT(pWriteState->ulBytesWritten == pWriteState->zct.ulDataBytesResident);
+
+        pWriteState->zct.ulDataBytesResident = 0;
+    }
+
+    LWIO_ASSERT(pWriteState->zct.ulDataBytesMissing > 0);
+
+    pZctContext = SrvAcquireExecContext(pExecContext);
+
+    // Do ZCT read from the transport for remaining bytes
+    ntStatus = SrvProtocolTransportReceiveZct(
+                    pExecContext->pConnection,
+                    pWriteState->zct.pZct,
+                    SrvExecuteWriteXReceiveZctCB,
+                    pZctContext);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    // completed synchronously
+    SrvReleaseExecContext(pZctContext);
+
+    pWriteState->ulBytesWritten += pWriteState->zct.ulDataBytesMissing - pWriteState->zct.ulPaddingSize;
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    if (pZctContext && (ntStatus != STATUS_PENDING))
+    {
+        SrvReleaseExecContext(pZctContext);
+    }
+
+    goto cleanup;
+}
+
+static
+NTSTATUS
+SrvExecuteWriteXZctComplete(
+    PSRV_WRITEX_STATE_SMB_V1 pWriteState,
+    PSRV_EXEC_CONTEXT pExecContext
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+
+    if (!pWriteState->bStartedIo)
+    {
+        SrvPrepareWriteXStateAsync(pWriteState, pExecContext);
+        pWriteState->bStartedIo = TRUE;
+
+        ntStatus = IoCompleteZctWriteFile(
+                        pWriteState->pFile->hFile,
+                        pWriteState->pAcb,
+                        &pWriteState->ioStatusBlock,
+                        0,
+                        pWriteState->zct.pZctCompletion,
+                        pWriteState->ulBytesWritten);
+        BAIL_ON_NT_STATUS(ntStatus);
+
+        // completed synchronously
+        SrvReleaseWriteXStateAsync(pWriteState);
+        pWriteState->bStartedIo = FALSE;
+    }
+
+    ntStatus = pWriteState->ioStatusBlock.Status;
+    if (ntStatus)
+    {
+        LWIO_LOG_ERROR("Failed to complete ZCT write file (status = 0x%08x)",
+                       ntStatus);
+    }
+    BAIL_ON_NT_STATUS(ntStatus);
+
+cleanup:
+
+    return ntStatus;
+
+error:
+
+    if (ntStatus != STATUS_PENDING)
+    {
+        SrvReleaseWriteXStateAsync(pWriteState);
+        pWriteState->bStartedIo = FALSE;
+    }
+
     goto cleanup;
 }
 
@@ -420,6 +702,42 @@ SrvExecuteWriteXAsyncCB(
     }
 }
 
+static
+VOID
+SrvExecuteWriteXReceiveZctCB(
+    IN PVOID pContext,
+    IN NTSTATUS Status
+    )
+{
+    NTSTATUS                   ntStatus         = STATUS_SUCCESS;
+    PSRV_EXEC_CONTEXT          pExecContext     = (PSRV_EXEC_CONTEXT)pContext;
+    PSRV_PROTOCOL_EXEC_CONTEXT pProtocolContext = pExecContext->pProtocolContext;
+    PSRV_WRITEX_STATE_SMB_V1   pWriteState       = NULL;
+    BOOLEAN                    bInLock          = FALSE;
+
+    pWriteState =
+        (PSRV_WRITEX_STATE_SMB_V1)pProtocolContext->pSmb1Context->hState;
+
+    LWIO_LOCK_MUTEX(bInLock, &pWriteState->mutex);
+    pWriteState->stage = SRV_WRITEX_STAGE_SMB_V1_ZCT_COMPLETE;
+    if (Status == STATUS_SUCCESS)
+    {
+        pWriteState->ulBytesWritten += pWriteState->zct.ulDataBytesMissing - pWriteState->zct.ulPaddingSize;
+    }
+    LWIO_UNLOCK_MUTEX(bInLock, &pWriteState->mutex);
+
+    LWIO_LOG_DEBUG("queue exec context = %p (conn = %p)", pContext, pExecContext->pConnection);
+
+    ntStatus = SrvProdConsEnqueue(gProtocolGlobals_SMB_V1.pWorkQueue, pContext);
+    if (ntStatus != STATUS_SUCCESS)
+    {
+        LWIO_LOG_ERROR("Failed to enqueue execution context (status = 0x%08x)",
+                       ntStatus);
+
+        SrvReleaseExecContext(pExecContext);
+    }
+}
+
 static
 VOID
 SrvReleaseWriteXStateAsync(
@@ -528,9 +846,9 @@ SrvBuildWriteAndXResponse(
     pResponseHeader->remaining = 0;
     pResponseHeader->reserved = 0;
     pResponseHeader->count =
-                    (pWriteState->ullBytesWritten & 0x00000000FFFFFFFFLL);
+                    (USHORT) (pWriteState->ulBytesWritten & 0xFFFF);
     pResponseHeader->countHigh =
-                    (pWriteState->ullBytesWritten & 0xFFFFFFFF00000000LL) >> 32;
+                    (USHORT) (pWriteState->ulBytesWritten >> 16);
 
     pResponseHeader->byteCount = 0;
 
@@ -554,7 +872,6 @@ error:
     goto cleanup;
 }
 
-static
 VOID
 SrvReleaseWriteXStateHandle(
     HANDLE hWriteState
@@ -598,3 +915,14 @@ SrvFreeWriteXState(
 
     SrvFreeMemory(pWriteState);
 }
+
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
