@@ -63,6 +63,18 @@ SrvBuildCreateState_SMB_V2(
     PSRV_CREATE_STATE_SMB_V2*   ppCreateState
     );
 
+static
+VOID
+SrvCancelCreateStateHandle_SMB_V2(
+    HANDLE hCreateState
+    );
+
+static
+VOID
+SrvCancelCreateStateHandle_SMB_V2_inlock(
+    PSRV_CREATE_STATE_SMB_V2 pCreateState
+    );
+
 static
 NTSTATUS
 SrvQueryFileInformation_SMB_V2(
@@ -189,6 +201,9 @@ SrvProcessCreate_SMB_V2(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSession2Info(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSession2FindTree_SMB_V2(
                         pCtxSmb2,
                         pSession,
@@ -214,6 +229,7 @@ SrvProcessCreate_SMB_V2(
         ntStatus = SrvConnection2CreateAsyncState(
                                 pConnection,
                                 COM2_CREATE,
+                                &SrvCancelCreateStateHandle_SMB_V2,
                                 &SrvReleaseCreateStateHandle_SMB_V2,
                                 &pAsyncState);
         BAIL_ON_NT_STATUS(ntStatus);
@@ -266,7 +282,7 @@ SrvProcessCreate_SMB_V2(
                                 pCreateState->pExtAContext->pData : NULL),
                             (pCreateState->pExtAContext ?
                                 pCreateState->pExtAContext->ulDataLength : 0),
-                            &pCreateState->pEcpList);
+                            pCreateState->pEcpList);
             switch (ntStatus)
             {
                 case STATUS_PENDING:
@@ -373,12 +389,6 @@ SrvProcessCreate_SMB_V2(
             ntStatus = SrvBuildCreateResponse_SMB_V2(pExecContext);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            pCreateState->stage = SRV_CREATE_STAGE_SMB_V2_DONE;
-
-            // intentional fall through
-
-        case SRV_CREATE_STAGE_SMB_V2_DONE:
-
             if (pCreateState->ullAsyncId)
             {
                 ntStatus = SrvConnection2RemoveAsyncState(
@@ -387,8 +397,32 @@ SrvProcessCreate_SMB_V2(
                 BAIL_ON_NT_STATUS(ntStatus);
             }
 
+            ntStatus = SrvElementsRegisterResource(
+                            &pCreateState->pFile->resource,
+                            NULL);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            pCreateState->stage = SRV_CREATE_STAGE_SMB_V2_DONE;
+
+            // intentional fall through
+
+        case SRV_CREATE_STAGE_SMB_V2_DONE:
+
             pCreateState->bRemoveFileFromTree = FALSE;
 
+            if (pCreateState->pRequestHeader->ulDesiredAccess & FILE_READ_DATA)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_READ;
+            }
+            if (pCreateState->pRequestHeader->ulDesiredAccess & FILE_WRITE_DATA)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_WRITE;
+            }
+            if (pCreateState->ulCreateAction == FILE_CREATED)
+            {
+                pCreateState->pFile->ulPermissions |= SRV_PERM_FILE_CREATE;
+            }
+
             pCtxSmb2->pFile = SrvFile2Acquire(pCreateState->pFile);
             pCtxSmb2->llNumSuccessfulCreates++;
 
@@ -484,10 +518,7 @@ SrvCancelCreate_SMB_V2(
 
     LWIO_LOCK_MUTEX(bInLock, &pCreateState->mutex);
 
-    if (pCreateState->pAcb && pCreateState->pAcb->AsyncCancelContext)
-    {
-        IoCancelAsyncCancelContext(pCreateState->pAcb->AsyncCancelContext);
-    }
+    SrvCancelCreateStateHandle_SMB_V2_inlock(pCreateState);
 
 cleanup:
 
@@ -525,8 +556,21 @@ SrvBuildCreateState_SMB_V2(
     PSRV_PROTOCOL_EXEC_CONTEXT pCtxProtocol   = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V2   pCtxSmb2       = pCtxProtocol->pSmb2Context;
     PSRV_CREATE_STATE_SMB_V2   pCreateState   = NULL;
-    BOOLEAN                    bTreeInLock    = FALSE;
     ULONG                      iCtx       = 0;
+    wchar16_t wszBackSlash[]              = { '\\', 0};
+    wchar16_t wszForwardSlash[]           = { '/' , 0};
+
+    if (*pwszFilename->Buffer == wszBackSlash[0])
+    {
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    if (*pwszFilename->Buffer == wszForwardSlash[0])
+    {
+        ntStatus = STATUS_OBJECT_NAME_INVALID;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
 
     ntStatus = SrvAllocateMemory(
                     sizeof(SRV_CREATE_STATE_SMB_V2),
@@ -562,16 +606,12 @@ SrvBuildCreateState_SMB_V2(
 
     pCreateState->pTree = SrvTree2Acquire(pCtxSmb2->pTree);
 
-    LWIO_LOCK_RWMUTEX_SHARED(bTreeInLock, &pCtxSmb2->pTree->mutex);
-
-    ntStatus = SrvBuildFilePath(
-                    pCtxSmb2->pTree->pShareInfo->pwszPath,
+    ntStatus = SrvBuildTreeRelativePath_SMB_V2(
+                    pCtxSmb2->pTree,
                     pCreateState->pwszFilename,
-                    &pCreateState->pFilename->FileName);
+                    pCreateState->pFilename);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb2->pTree->mutex);
-
     /* For named pipes, we need to pipe some extra data into the npfs driver:
      *  - Session key
      *  - Client principal name
@@ -593,6 +633,21 @@ SrvBuildCreateState_SMB_V2(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
+    /* Enable ABE (but only we if know this is not a file access) */
+
+    if ((pCreateState->pTree->pShareInfo->ulFlags & SHARE_INFO_FLAG_ABE_ENABLED) &&
+        !(pRequestHeader->ulCreateOptions & FILE_NON_DIRECTORY_FILE))
+    {
+        if (!pCreateState->pEcpList)
+        {
+            ntStatus = IoRtlEcpListAllocate(&pCreateState->pEcpList);
+            BAIL_ON_NT_STATUS(ntStatus);
+        }
+
+        ntStatus = SrvIoPrepareAbeEcpList(pCreateState->pEcpList);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     pCreateState->pCreateContexts = *ppCreateContexts;
     *ppCreateContexts = NULL;
 
@@ -604,12 +659,15 @@ SrvBuildCreateState_SMB_V2(
 
         switch (pContext->contextItemType)
         {
-            case SMB2_CONTEXT_ITEM_TYPE_EXT_ATTRS:
+            case SMB2_CONTEXT_ITEM_TYPE_MAX_ACCESS:
 
                 pCreateState->pExtAContext = pContext;
 
-                ntStatus = IoRtlEcpListAllocate(&pCreateState->pEcpList);
-                BAIL_ON_NT_STATUS(ntStatus);
+                if (!pCreateState->pEcpList)
+                {
+                    ntStatus = IoRtlEcpListAllocate(&pCreateState->pEcpList);
+                    BAIL_ON_NT_STATUS(ntStatus);
+                }
 
                 ntStatus = IoRtlEcpListInsert(
                                 pCreateState->pEcpList,
@@ -623,7 +681,7 @@ SrvBuildCreateState_SMB_V2(
 
             case SMB2_CONTEXT_ITEM_TYPE_SEC_DESC:
                 {
-                    SECURITY_INFORMATION secInfoAll = DACL_SECURITY_INFORMATION;
+                    SECURITY_INFORMATION secInfoAll = 0;
 
                     if (!pContext->ulDataLength ||
                         !RtlValidRelativeSecurityDescriptor(
@@ -653,8 +711,6 @@ SrvBuildCreateState_SMB_V2(
 
 cleanup:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb2->pTree->mutex);
-
     return ntStatus;
 
 error:
@@ -669,6 +725,35 @@ error:
     goto cleanup;
 }
 
+static
+VOID
+SrvCancelCreateStateHandle_SMB_V2(
+    HANDLE hCreateState
+    )
+{
+    BOOLEAN bInLock = FALSE;
+    PSRV_CREATE_STATE_SMB_V2 pCreateState =
+                (PSRV_CREATE_STATE_SMB_V2)hCreateState;
+
+    LWIO_LOCK_MUTEX(bInLock, &pCreateState->mutex);
+
+    SrvCancelCreateStateHandle_SMB_V2_inlock(pCreateState);
+
+    LWIO_UNLOCK_MUTEX(bInLock, &pCreateState->mutex);
+}
+
+static
+VOID
+SrvCancelCreateStateHandle_SMB_V2_inlock(
+    PSRV_CREATE_STATE_SMB_V2 pCreateState
+    )
+{
+    if (pCreateState->pAcb && pCreateState->pAcb->AsyncCancelContext)
+    {
+        IoCancelAsyncCancelContext(pCreateState->pAcb->AsyncCancelContext);
+    }
+}
+
 static
 NTSTATUS
 SrvQueryFileInformation_SMB_V2(
@@ -790,8 +875,7 @@ SrvProcessCreateContexts_SMB_V2(
         {
             case SMB2_CONTEXT_ITEM_TYPE_MAX_ACCESS:
 
-                // TODO: Find the real max access mask on the file
-                pCreateState->ulMaximalAccessMask = 0x1F01FF;
+                // The driver should have filled this in.
 
                 break;
 
@@ -1433,3 +1517,13 @@ error:
 
     goto cleanup;
 }
+
+
+/*
+local variables:
+mode: c
+c-basic-offset: 4
+indent-tabs-mode: nil
+tab-width: 4
+end:
+*/
