@@ -145,6 +145,9 @@ SrvProcessIOCTL_SMB_V2(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSession2Info(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSession2FindTree_SMB_V2(
                         pCtxSmb2,
                         pSession,
@@ -346,6 +349,14 @@ SrvExecuteIOCTL_SMB_V2(
 
     pIOCTLState = (PSRV_IOCTL_STATE_SMB_V2)pCtxSmb2->hState;
 
+    if (pExecContext->pStatInfo)
+    {
+        ntStatus = SrvStatisticsSetIOCTL(
+                        pExecContext->pStatInfo,
+                        pIOCTLState->pRequestHeader->ulFunctionCode);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     ntStatus = pIOCTLState->ioStatusBlock.Status;
     BAIL_ON_NT_STATUS(ntStatus);
 
@@ -360,7 +371,10 @@ SrvExecuteIOCTL_SMB_V2(
                         &pIOCTLState->sResponseBufferLen);
         BAIL_ON_NT_STATUS(ntStatus);
 
-        pIOCTLState->ulResponseBufferLen = pIOCTLState->sResponseBufferLen;
+        pIOCTLState->ulResponseBufferLen =
+            pIOCTLState->pRequestHeader->ulMaxOutLength > 0 ?
+            pIOCTLState->pRequestHeader->ulMaxOutLength:
+            pIOCTLState->sResponseBufferLen;
 
         SrvPrepareIOCTLStateAsync_SMB_V2(pIOCTLState, pExecContext);
 
