@@ -61,14 +61,6 @@ SrvSendInterimResponse_SMB_V2(
     PSRV_EXEC_CONTEXT pExecContext
     );
 
-static
-NTSTATUS
-SrvBuildExecContext_SMB_V2(
-    PLWIO_SRV_CONNECTION      pConnection,
-    PSMB_PACKET               pSmbRequest,
-    PSRV_EXEC_CONTEXT_SMB_V2* ppSmb2Context
-    );
-
 static
 PCSTR
 SrvGetCommandDescription_SMB_V2(
@@ -100,6 +92,8 @@ SrvProtocolExecute_SMB_V2(
     NTSTATUS                 ntStatus     = STATUS_SUCCESS;
     PLWIO_SRV_CONNECTION     pConnection  = pExecContext->pConnection;
     PSRV_EXEC_CONTEXT_SMB_V2 pSmb2Context = NULL;
+    USHORT   usCurrentCommand = 0;
+    BOOLEAN  bPopStatMessage  = FALSE;
 
     if (!pExecContext->pProtocolContext->pSmb2Context)
     {
@@ -134,6 +128,7 @@ SrvProtocolExecute_SMB_V2(
          pSmb2Context->iMsg++)
     {
         ULONG iMsg = pSmb2Context->iMsg;
+        PSRV_MESSAGE_SMB_V2 pRequest = &pSmb2Context->pRequests[iMsg];
         PSRV_MESSAGE_SMB_V2 pResponse = &pSmb2Context->pResponses[iMsg];
         PSRV_MESSAGE_SMB_V2 pPrevResponse = NULL;
 
@@ -169,8 +164,20 @@ SrvProtocolExecute_SMB_V2(
 
         pResponse->ulMessageSize = 0;
         pResponse->ulBytesAvailable =   pExecContext->pSmbResponse->bufferLen -
-                                        pExecContext->pSmbResponse->bufferUsed -
-                                        sizeof(NETBIOS_HEADER);
+                                        pExecContext->pSmbResponse->bufferUsed;
+
+        if (pExecContext->pStatInfo)
+        {
+            usCurrentCommand = pRequest->pHeader->command;
+
+            ntStatus = SrvStatisticsPushMessage(
+                            pExecContext->pStatInfo,
+                            usCurrentCommand,
+                            pRequest->ulMessageSize);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            bPopStatMessage = TRUE;
+        }
 
         ntStatus = SrvProcessRequestSpecific_SMB_V2(pExecContext);
 
@@ -221,9 +228,30 @@ SrvProtocolExecute_SMB_V2(
         }
 
         pExecContext->pSmbResponse->bufferUsed += pResponse->ulMessageSize;
+
+        if (bPopStatMessage)
+        {
+            NTSTATUS ntStatus2 =
+                    SrvStatisticsPopMessage(
+                            pExecContext->pStatInfo,
+                            usCurrentCommand,
+                            (pResponse->ulMessageSize ?
+                                    pResponse->ulMessageSize :
+                                    pResponse->ulZctMessageSize),
+                            ntStatus);
+            if (ntStatus2)
+            {
+                LWIO_LOG_ERROR( "Error: Failed to notify statistics "
+                                "module on end of message processing "
+                                "[error: %u]", ntStatus2);
+            }
+
+            bPopStatMessage = FALSE;
+            usCurrentCommand = 0;
+        }
     }
 
-    ntStatus = SMB2MarshalFooter(pExecContext->pSmbResponse);
+    ntStatus = SMBPacketMarshallFooter(pExecContext->pSmbResponse);
     BAIL_ON_NT_STATUS(ntStatus);
 
 cleanup:
@@ -232,9 +260,64 @@ cleanup:
 
 error:
 
+    switch (ntStatus)
+    {
+        case STATUS_PENDING:
+
+            break;
+
+        default:
+
+            if (bPopStatMessage)
+            {
+                NTSTATUS ntStatus2 =
+                        SrvStatisticsPopMessage(
+                                pExecContext->pStatInfo,
+                                usCurrentCommand,
+                                pExecContext->pSmbResponse->bufferUsed,
+                                ntStatus);
+                if (ntStatus2)
+                {
+                    LWIO_LOG_ERROR( "Error: Failed to notify statistics "
+                                    "module on end of message processing "
+                                    "[error: %u]", ntStatus2);
+                }
+            }
+
+            break;
+    }
+
     goto cleanup;
 }
 
+NTSTATUS
+SrvProtocolCloseFile_SMB_V2(
+    PLWIO_SRV_TREE_2 pTree,
+    PLWIO_SRV_FILE_2 pFile
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+
+    if (!pTree || !pFile)
+    {
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    SrvFile2ResetOplockState(pFile);
+
+    ntStatus = SrvTree2RemoveFile(
+                    pTree,
+                    &pFile->fid);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    SrvFile2Rundown(pFile);
+
+error:
+
+    return ntStatus;
+}
+
 static
 NTSTATUS
 SrvProcessRequestSpecific_SMB_V2(
@@ -494,9 +577,10 @@ SrvSendInterimResponse_SMB_V2(
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
 
-    ntStatus = SrvTransportSendResponse(
+    ntStatus = SrvProtocolTransportSendResponse(
                     pExecContext->pConnection,
-                    pExecContext->pInterimResponse);
+                    pExecContext->pInterimResponse,
+                    NULL /* Statistics info */);
     BAIL_ON_NT_STATUS(ntStatus);
 
 cleanup:
@@ -644,7 +728,7 @@ SrvBuildInterimResponse_SMB_V2(
 
     pInterimResponse->bufferUsed += ulTotalBytesUsed;
 
-    ntStatus = SMB2MarshalFooter(pInterimResponse);
+    ntStatus = SMBPacketMarshallFooter(pInterimResponse);
     BAIL_ON_NT_STATUS(ntStatus);
 
     if (pExecContext->pInterimResponse)
@@ -675,7 +759,6 @@ error:
     goto cleanup;
 }
 
-static
 NTSTATUS
 SrvBuildExecContext_SMB_V2(
     PLWIO_SRV_CONNECTION      pConnection,
@@ -686,7 +769,7 @@ SrvBuildExecContext_SMB_V2(
     NTSTATUS ntStatus = STATUS_SUCCESS;
     ULONG    ulNumRequests    = 0;
     ULONG    iRequest         = 0;
-    ULONG    ulBytesAvailable = pSmbRequest->bufferLen;
+    ULONG    ulBytesAvailable = pSmbRequest->bufferUsed;
     PBYTE    pBuffer          = pSmbRequest->pRawBuffer;
     PSRV_EXEC_CONTEXT_SMB_V2 pSmb2Context = NULL;
 
@@ -752,7 +835,7 @@ SrvBuildExecContext_SMB_V2(
     pSmb2Context->ulNumResponses = ulNumRequests;
 
     pBuffer = pSmbRequest->pRawBuffer + sizeof(NETBIOS_HEADER);
-    ulBytesAvailable = pSmbRequest->bufferLen - sizeof(NETBIOS_HEADER);
+    ulBytesAvailable = pSmbRequest->bufferUsed - sizeof(NETBIOS_HEADER);
 
     for (; iRequest < ulNumRequests; iRequest++)
     {
@@ -801,12 +884,11 @@ error:
     goto cleanup;
 }
 
-NTSTATUS
+VOID
 SrvProtocolShutdown_SMB_V2(
     VOID
     )
 {
-    NTSTATUS status = STATUS_SUCCESS;
     BOOLEAN bInLock = FALSE;
 
     LWIO_LOCK_MUTEX(bInLock, &gProtocolGlobals_SMB_V2.mutex);
@@ -814,8 +896,6 @@ SrvProtocolShutdown_SMB_V2(
     gProtocolGlobals_SMB_V2.pWorkQueue = NULL;
 
     LWIO_UNLOCK_MUTEX(bInLock, &gProtocolGlobals_SMB_V2.mutex);
-
-    return status;
 }
 
 static
