@@ -176,6 +176,9 @@ SrvProcessLock_SMB_V2(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSession2Info(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSession2FindTree_SMB_V2(
                         pCtxSmb2,
                         pSession,
@@ -232,6 +235,7 @@ SrvProcessLock_SMB_V2(
                 ntStatus = SrvConnection2CreateAsyncState(
                                 pConnection,
                                 COM2_LOCK,
+                                &SrvCancelAsyncLockState_SMB_V2,
                                 &SrvReleaseAsyncLockStateHandle_SMB_V2,
                                 &pAsyncState);
                 BAIL_ON_NT_STATUS(ntStatus);
@@ -681,6 +685,8 @@ SrvProcessSyncLockRequest_SMB_V2(
 
         pLockRequestState->iUnlock++;
         pLockRequestState->bUnlockPending = FALSE;
+
+        SrvFile2RegisterUnlock(pLockRequestState->pFile);
     }
 
     if (pLockRequestState->bLockPending)
@@ -690,6 +696,8 @@ SrvProcessSyncLockRequest_SMB_V2(
 
         pLockRequestState->iLock++;
         pLockRequestState->bLockPending = FALSE;
+
+        SrvFile2RegisterLock(pLockRequestState->pFile);
     }
 
     // Unlock requests
@@ -714,6 +722,8 @@ SrvProcessSyncLockRequest_SMB_V2(
         }
         BAIL_ON_NT_STATUS(ntStatus);
 
+        SrvFile2RegisterUnlock(pLockRequestState->pFile);
+
         SrvReleaseSyncLockStateAsync_SMB_V2(pLockRequestState); // completed sync
     }
 
@@ -742,6 +752,8 @@ SrvProcessSyncLockRequest_SMB_V2(
         }
         BAIL_ON_NT_STATUS(ntStatus);
 
+        SrvFile2RegisterLock(pLockRequestState->pFile);
+
         SrvReleaseSyncLockStateAsync_SMB_V2(pLockRequestState); // completed sync
     }
 
@@ -879,6 +891,10 @@ SrvClearLocks_SMB_V2_inlock(
         {
             LWIO_LOG_ERROR("Failed in unlock. error code [%d]", ntStatus1);
         }
+        else
+        {
+            SrvFile2RegisterUnlock(pLockRequestState->pFile);
+        }
     }
 
     return;
