@@ -74,6 +74,12 @@ SrvBuildAsyncLockResponse_SMB_V2(
     PSRV_EXEC_CONTEXT      pExecContext
     );
 
+static
+VOID
+SrvCancelAsyncLockState_SMB_V2_inlock(
+    PSRV_ASYNC_LOCK_REQUEST_STATE_SMB_V2 pAsyncLockState
+    );
+
 static
 VOID
 SrvPrepareAsyncLockStateAsync_SMB_V2(
@@ -262,7 +268,7 @@ SrvBuildExecContextAsyncLock_SMB_V2(
 
     pSmbRequest2->bufferUsed += ulTotalBytesUsed;
 
-    ntStatus = SMB2MarshalFooter(pSmbRequest2);
+    ntStatus = SMBPacketMarshallFooter(pSmbRequest2);
     BAIL_ON_NT_STATUS(ntStatus);
 
     *ppExecContextAsync = pExecContextAsync;
@@ -324,10 +330,7 @@ SrvCancelLock_SMB_V2(
 
     LWIO_LOCK_MUTEX(bInLock, &pLockState->mutex);
 
-    if (pLockState->pAcb && pLockState->pAcb->AsyncCancelContext)
-    {
-        IoCancelAsyncCancelContext(pLockState->pAcb->AsyncCancelContext);
-    }
+    SrvCancelAsyncLockState_SMB_V2_inlock(pLockState);
 
 cleanup:
 
@@ -348,6 +351,22 @@ error:
     goto cleanup;
 }
 
+VOID
+SrvCancelAsyncLockState_SMB_V2(
+    HANDLE hLockState
+    )
+{
+    BOOLEAN bInLock = FALSE;
+    PSRV_ASYNC_LOCK_REQUEST_STATE_SMB_V2 pAsyncLockState =
+            (PSRV_ASYNC_LOCK_REQUEST_STATE_SMB_V2)hLockState;
+
+    LWIO_LOCK_MUTEX(bInLock, &pAsyncLockState->mutex);
+
+    SrvCancelAsyncLockState_SMB_V2_inlock(pAsyncLockState);
+
+    LWIO_UNLOCK_MUTEX(bInLock, &pAsyncLockState->mutex);
+}
+
 NTSTATUS
 SrvProcessAsyncLockRequest_SMB_V2(
     PSRV_EXEC_CONTEXT pExecContext
@@ -751,6 +770,18 @@ error:
     goto cleanup;
 }
 
+static
+VOID
+SrvCancelAsyncLockState_SMB_V2_inlock(
+    PSRV_ASYNC_LOCK_REQUEST_STATE_SMB_V2 pAsyncLockState
+    )
+{
+    if (pAsyncLockState->pAcb && pAsyncLockState->pAcb->AsyncCancelContext)
+    {
+        IoCancelAsyncCancelContext(pAsyncLockState->pAcb->AsyncCancelContext);
+    }
+}
+
 static
 VOID
 SrvPrepareAsyncLockStateAsync_SMB_V2(
