@@ -56,6 +56,12 @@ SrvBuildNotifyRequestState_SMB_V2(
     PSRV_NOTIFY_REQUEST_STATE_SMB_V2*  ppNotifyRequestState
     );
 
+static
+VOID
+SrvCancelNotifyState_SMB_V2_inlock(
+    PSRV_NOTIFY_STATE_SMB_V2 pNotifyState
+    );
+
 static
 NTSTATUS
 SrvExecuteChangeNotify_SMB_V2(
@@ -133,6 +139,9 @@ SrvProcessNotify_SMB_V2(
                         &pSession);
         BAIL_ON_NT_STATUS(ntStatus);
 
+        ntStatus = SrvSetStatSession2Info(pExecContext, pSession);
+        BAIL_ON_NT_STATUS(ntStatus);
+
         ntStatus = SrvSession2FindTree_SMB_V2(
                         pCtxSmb2,
                         pSession,
@@ -187,6 +196,7 @@ SrvProcessNotify_SMB_V2(
              ntStatus = SrvConnection2CreateAsyncState(
                              pConnection,
                              COM2_NOTIFY,
+                             &SrvCancelNotifyState_SMB_V2,
                              &SrvNotifyStateReleaseHandle_SMB_V2,
                              &pAsyncState);
              BAIL_ON_NT_STATUS(ntStatus);
@@ -479,10 +489,7 @@ SrvCancelChangeNotify_SMB_V2(
 
     LWIO_LOCK_MUTEX(bInLock, &pNotifyState->mutex);
 
-    if (pNotifyState->pAcb && pNotifyState->pAcb->AsyncCancelContext)
-    {
-        IoCancelAsyncCancelContext(pNotifyState->pAcb->AsyncCancelContext);
-    }
+    SrvCancelNotifyState_SMB_V2_inlock(pNotifyState);
 
 cleanup:
 
@@ -503,6 +510,22 @@ error:
     goto cleanup;
 }
 
+VOID
+SrvCancelNotifyState_SMB_V2(
+    HANDLE hNotifyState
+    )
+{
+    BOOLEAN bInLock = FALSE;
+    PSRV_NOTIFY_STATE_SMB_V2 pNotifyState =
+                        (PSRV_NOTIFY_STATE_SMB_V2)hNotifyState;
+
+    LWIO_LOCK_MUTEX(bInLock, &pNotifyState->mutex);
+
+    SrvCancelNotifyState_SMB_V2_inlock(pNotifyState);
+
+    LWIO_UNLOCK_MUTEX(bInLock, &pNotifyState->mutex);
+}
+
 static
 NTSTATUS
 SrvBuildNotifyRequestState_SMB_V2(
@@ -545,6 +568,18 @@ error:
     goto cleanup;
 }
 
+static
+VOID
+SrvCancelNotifyState_SMB_V2_inlock(
+    PSRV_NOTIFY_STATE_SMB_V2 pNotifyState
+    )
+{
+    if (pNotifyState->pAcb && pNotifyState->pAcb->AsyncCancelContext)
+    {
+        IoCancelAsyncCancelContext(pNotifyState->pAcb->AsyncCancelContext);
+    }
+}
+
 static
 NTSTATUS
 SrvExecuteChangeNotify_SMB_V2(
