@@ -183,6 +183,13 @@ SrvBuildInterimResponse_SMB_V2(
     ULONG64           ullAsyncId
     );
 
+NTSTATUS
+SrvBuildExecContext_SMB_V2(
+    PLWIO_SRV_CONNECTION      pConnection,
+    PSMB_PACKET               pSmbRequest,
+    PSRV_EXEC_CONTEXT_SMB_V2* ppSmb2Context
+    );
+
 // lock.c
 
 NTSTATUS
@@ -190,6 +197,11 @@ SrvProcessLock_SMB_V2(
     PSRV_EXEC_CONTEXT pContext
     );
 
+VOID
+SrvCancelAsyncLockState_SMB_V2(
+    HANDLE hLockState
+    );
+
 NTSTATUS
 SrvDetermineLocks_SMB_V2(
     PSMB2_LOCK_REQUEST_HEADER pLockRequestHeader,
@@ -278,6 +290,11 @@ SrvCancelChangeNotify_SMB_V2(
     PSRV_EXEC_CONTEXT pExecContext
     );
 
+VOID
+SrvCancelNotifyState_SMB_V2(
+    HANDLE hNotifyState
+    );
+
 // notify_state.c
 
 NTSTATUS
@@ -345,6 +362,7 @@ SrvProcessOplockBreak_SMB_V2(
 NTSTATUS
 SrvAcknowledgeOplockBreak_SMB_V2(
     PSRV_OPLOCK_STATE_SMB_V2 pOplockState,
+    PUCHAR  pucNewOplockLevel,
     BOOLEAN bFileIsClosed
     );
 
@@ -445,6 +463,21 @@ SrvProcessTreeDisconnect_SMB_V2(
     PSRV_EXEC_CONTEXT pExecContext
     );
 
+// utils.c
+
+NTSTATUS
+SrvBuildTreeRelativePath_SMB_V2(
+    PLWIO_SRV_TREE_2 pTree,
+    PWSTR            pwszFilename,
+    PIO_FILE_NAME    pFilename
+    );
+
+NTSTATUS
+SrvSetStatSession2Info(
+    PSRV_EXEC_CONTEXT   pExecContext,
+    PLWIO_SRV_SESSION_2 pSession
+    );
+
 // wire.c
 
 NTSTATUS
@@ -749,9 +782,9 @@ SMB2MarshalError(
     PULONG   pulBytesUsed
     );
 
-NTSTATUS
-SMB2MarshalFooter(
-    PSMB_PACKET pPacket
+VOID
+SMB2UnmarshallBoolean(
+    PBOOLEAN pbValue
     );
 
 // write.c
