@@ -631,6 +631,7 @@ SrvSetFileDispositionInfo_SMB_V2(
     PSRV_PROTOCOL_EXEC_CONTEXT   pCtxProtocol  = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V2     pCtxSmb2      = pCtxProtocol->pSmb2Context;
     PSRV_SET_INFO_STATE_SMB_V2   pSetInfoState = NULL;
+    PFILE_DISPOSITION_INFORMATION pFileDispositionInfo = NULL;
 
     pSetInfoState = (PSRV_SET_INFO_STATE_SMB_V2)pCtxSmb2->hState;
 
@@ -641,13 +642,17 @@ SrvSetFileDispositionInfo_SMB_V2(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
+    pFileDispositionInfo = (PFILE_DISPOSITION_INFORMATION)pSetInfoState->pData;
+
+    SMB2UnmarshallBoolean(&pFileDispositionInfo->DeleteFile);
+
     SrvPrepareSetInfoStateAsync_SMB_V2(pSetInfoState, pExecContext);
 
     ntStatus = IoSetInformationFile(
                     pSetInfoState->pFile->hFile,
                     pSetInfoState->pAcb,
                     &pSetInfoState->ioStatusBlock,
-                    (PFILE_DISPOSITION_INFORMATION)pSetInfoState->pData,
+                    pFileDispositionInfo,
                     sizeof(FILE_DISPOSITION_INFORMATION),
                     FileDispositionInformation);
     BAIL_ON_NT_STATUS(ntStatus);
@@ -669,7 +674,6 @@ SrvSetFileRenameInfo_SMB_V2(
     PSRV_PROTOCOL_EXEC_CONTEXT   pCtxProtocol  = pExecContext->pProtocolContext;
     PSRV_EXEC_CONTEXT_SMB_V2     pCtxSmb2      = pCtxProtocol->pSmb2Context;
     PSRV_SET_INFO_STATE_SMB_V2   pSetInfoState = NULL;
-    BOOLEAN                      bTreeInLock   = FALSE;
 
     pSetInfoState = (PSRV_SET_INFO_STATE_SMB_V2)pCtxSmb2->hState;
 
@@ -686,17 +690,12 @@ SrvSetFileRenameInfo_SMB_V2(
     }
     else if (!pSetInfoState->hDir)
     {
-        LWIO_LOCK_RWMUTEX_SHARED(   bTreeInLock,
-                                    &pCtxSmb2->pTree->pShareInfo->mutex);
-
-        ntStatus = SrvAllocateStringW(
-                        pCtxSmb2->pTree->pShareInfo->pwszPath,
-                        &pSetInfoState->dirPath.FileName);
+        ntStatus = SrvBuildTreeRelativePath_SMB_V2(
+                        pCtxSmb2->pTree,
+                        NULL,
+                        &pSetInfoState->dirPath);
         BAIL_ON_NT_STATUS(ntStatus);
 
-        LWIO_UNLOCK_RWMUTEX(bTreeInLock,
-                            &pCtxSmb2->pTree->pShareInfo->mutex);
-
         // Catch failed CreateFile calls when they come back around
 
         ntStatus = pSetInfoState->ioStatusBlock.Status;
@@ -721,7 +720,7 @@ SrvSetFileRenameInfo_SMB_V2(
                                 FILE_DIRECTORY_FILE,
                                 NULL, /* EA Buffer */
                                 0,    /* EA Length */
-                                &pSetInfoState->pEcpList
+                                pSetInfoState->pEcpList
                                 );
         BAIL_ON_NT_STATUS(ntStatus);
 
@@ -749,8 +748,6 @@ SrvSetFileRenameInfo_SMB_V2(
 
 error:
 
-    LWIO_UNLOCK_RWMUTEX(bTreeInLock, &pCtxSmb2->pTree->pShareInfo->mutex);
-
     return ntStatus;
 }
 
@@ -816,7 +813,7 @@ SrvUnmarshalRenameHeader_SMB_V2(
     BAIL_ON_NT_STATUS(ntStatus);
 
     ((PFILE_RENAME_INFORMATION)pSetInfoState->pData2)->ReplaceIfExists =
-                    pRenameInfoHeader->ucReplaceIfExists;
+                    pRenameInfoHeader->ucReplaceIfExists ? TRUE : FALSE;
     ((PFILE_RENAME_INFORMATION)pSetInfoState->pData2)->FileNameLength =
                     pRenameInfoHeader->ulFileNameLength;
 
