@@ -687,6 +687,18 @@ typedef struct _SMB2_FILE_NAME_INFORMATION
 } __attribute__((__packed__)) SMB2_FILE_NAME_INFORMATION,
                              *PSMB2_FILE_NAME_INFORMATION;
 
+typedef struct _SMB2_FILE_NETWORK_OPEN_INFORMATION {
+    LONG64          llCreationTime;
+    LONG64          llLastAccessTime;
+    LONG64          llLastWriteTime;
+    LONG64          llChangeTime;
+    ULONG64         ullAllocationSize;
+    ULONG64         ullEndOfFile;
+    FILE_ATTRIBUTES ulFileAttributes;
+    ULONG           ulPad;
+} __attribute__((__packed__)) SMB2_FILE_NETWORK_OPEN_INFORMATION,
+                            *PSMB2_FILE_NETWORK_OPEN_INFORMATION;
+
 typedef struct
 {
     USHORT   usLength;
@@ -698,6 +710,48 @@ typedef struct
 } __attribute__((__packed__)) SMB2_NOTIFY_CHANGE_HEADER,
                              *PSMB2_NOTIFY_CHANGE_HEADER;
 
+typedef enum
+{
+    SRV_TREE_CONNECT_STAGE_SMB_V2_INITIAL = 0,
+    SRV_TREE_CONNECT_STAGE_SMB_V2_CREATE_TREE_ROOT_HANDLE,
+    SRV_TREE_CONNECT_STAGE_SMB_V2_BUILD_RESPONSE,
+    SRV_TREE_CONNECT_STAGE_SMB_V2_DONE
+} SRV_TREE_CONNECT_STAGE_SMB_V2;
+
+typedef struct _SRV_TREE_CONNECT_STATE_SMB_V2
+{
+    LONG                          refCount;
+
+    pthread_mutex_t               mutex;
+    pthread_mutex_t*              pMutex;
+
+    SRV_TREE_CONNECT_STAGE_SMB_V2 stage;
+
+    IO_STATUS_BLOCK               ioStatusBlock;
+
+    IO_ASYNC_CONTROL_BLOCK        acb;
+    PIO_ASYNC_CONTROL_BLOCK       pAcb;
+
+    PSMB2_TREE_CONNECT_REQUEST_HEADER pRequestHeader; // Do not free
+
+    PWSTR                         pwszPath;
+
+    PVOID                         pSecurityDescriptor;
+    PVOID                         pSecurityQOS;
+    PIO_ECP_LIST                  pEcpList;
+
+    PLWIO_SRV_SESSION_2           pSession;
+    PLWIO_SRV_TREE_2              pTree;
+
+    IO_FILE_NAME                  fileName;
+
+    struct sockaddr               clientAddress;
+    ULONG                         ulClientAddressLength;
+
+    BOOLEAN                       bRemoveTreeFromSession;
+
+} SRV_TREE_CONNECT_STATE_SMB_V2, *PSRV_TREE_CONNECT_STATE_SMB_V2;
+
 typedef VOID (*PFN_SRV_MESSAGE_STATE_RELEASE_SMB_V2)(HANDLE hState);
 
 typedef struct _SRV_OPLOCK_INFO
@@ -835,6 +889,7 @@ typedef enum
     SRV_READ_STAGE_SMB_V2_ATTEMPT_READ,
     SRV_READ_STAGE_SMB_V2_ATTEMPT_READ_COMPLETED,
     SRV_READ_STAGE_SMB_V2_BUILD_RESPONSE,
+    SRV_READ_STAGE_SMB_V2_ZCT_COMPLETE,
     SRV_READ_STAGE_SMB_V2_DONE
 } SRV_READ_STAGE_SMB_V2;
 
@@ -854,21 +909,41 @@ typedef struct _SRV_READ_STATE_SMB_V2
 
     PLWIO_SRV_FILE_2          pFile;
 
-    LONG64                    llByteOffset;
     ULONG                     ulBytesRead;
     ULONG                     ulRemaining;
 
     PSMB2_READ_REQUEST_HEADER pRequestHeader; // Do not free
 
+    // I/O Parameters
+    LONG64                    llByteOffset;
+    LONG                      ulBytesToRead;
     PBYTE                     pData;
     ULONG                     ulKey;
 
+    // I/O Handling
+    BOOLEAN                   bStartedRead;
+    PLW_ZCT_VECTOR            pZct;
+    PVOID                     pZctCompletion;
+
 } SRV_READ_STATE_SMB_V2, *PSRV_READ_STATE_SMB_V2;
 
+typedef struct _SRV_ZCT_WRITE_STATE_SMB_V2 {
+    ULONG ulDataBytesMissing;
+    ULONG ulDataBytesResident;
+    ULONG ulSkipBytes;
+    PLW_ZCT_VECTOR pZct;
+    PVOID pZctCompletion;
+    PVOID pPadding;
+    ULONG ulPaddingSize;
+    PLWIO_SRV_CONNECTION pPausedConnection;
+} SRV_ZCT_WRITE_STATE_SMB_V2, *PSRV_ZCT_WRITE_STATE_SMB_V2;
+
 typedef enum
 {
     SRV_WRITE_STAGE_SMB_V2_INITIAL = 0,
     SRV_WRITE_STAGE_SMB_V2_ATTEMPT_WRITE,
+    SRV_WRITE_STAGE_SMB_V2_ZCT_IO,
+    SRV_WRITE_STAGE_SMB_V2_ZCT_COMPLETE,
     SRV_WRITE_STAGE_SMB_V2_BUILD_RESPONSE,
     SRV_WRITE_STAGE_SMB_V2_DONE
 } SRV_WRITE_STAGE_SMB_V2;
@@ -891,10 +966,15 @@ typedef struct _SRV_WRITE_STATE_SMB_V2
 
     PSMB2_WRITE_REQUEST_HEADER pRequestHeader; // Do not free
     PBYTE                      pData;          // Do not free
-
-    ULONG                      ulBytesWritten;
-    LONG64                     llDataOffset;
+    LONG64                     llOffset;
+    ULONG                      ulLength;
     ULONG                      ulKey;
+    ULONG                      ulBytesWritten;
+
+    BOOLEAN bStartedIo;
+
+    // ZCT information
+    SRV_ZCT_WRITE_STATE_SMB_V2 Zct;
 
 } SRV_WRITE_STATE_SMB_V2, *PSRV_WRITE_STATE_SMB_V2;
 
@@ -1193,6 +1273,7 @@ typedef struct __SRV_MESSAGE_SMB_V2
     PSMB2_HEADER pHeader;
     ULONG        ulHeaderSize;
     ULONG        ulMessageSize;
+    ULONG        ulZctMessageSize;
 
     ULONG        ulBytesAvailable;
 
