@@ -1442,10 +1442,10 @@ SMB2MarshalReadResponse(
     if (pData)
     {
         memcpy(pDataCursor, pData, ulBytesRead);
-        ulBytesUsed += ulBytesRead;
-        ulBytesAvailable -= ulBytesRead;
-        pDataCursor += ulBytesRead;
     }
+    ulBytesUsed += ulBytesRead;
+    ulBytesAvailable -= ulBytesRead;
+    pDataCursor += ulBytesRead;
 
     *pulDataOffset = ulDataOffset;
     *pulBytesUsed = ulBytesUsed;
@@ -2187,12 +2187,10 @@ error:
     goto cleanup;
 }
 
-NTSTATUS
-SMB2MarshalFooter(
-    PSMB_PACKET pPacket
+VOID
+SMB2UnmarshallBoolean(
+    PBOOLEAN pbValue
     )
 {
-    pPacket->pNetBIOSHeader->len = htonl(pPacket->bufferUsed - sizeof(NETBIOS_HEADER));
-
-    return 0;
+    *pbValue = *pbValue > 0 ? TRUE : FALSE;
 }
