@@ -62,6 +62,19 @@ SrvShareRemoveFromList_inlock(
     IN     PWSTR                      pwszShareName
     );
 
+static
+int
+SrvShareCompareInfo(
+    PVOID pKey1,
+    PVOID pKey2
+    );
+
+static
+VOID
+SrvShareReleaseInfoHandle(
+    PVOID hShareInfo
+    );
+
 static
 VOID
 SrvShareFreeInfo(
@@ -78,7 +91,7 @@ SrvShareInitList(
     HANDLE   hResume = NULL;
     PSRV_SHARE_INFO* ppShareInfoList = NULL;
     PSRV_SHARE_ENTRY pShareEntry = NULL;
-    ULONG            ulLimit  = 256;
+    ULONG            ulBatchLimit  = 256;
     ULONG            ulNumSharesFound = 0;
 
     pthread_rwlock_init(&pShareList->mutex, NULL);
@@ -86,12 +99,19 @@ SrvShareInitList(
 
     pShareList->pShareEntry = NULL;
 
+    ntStatus = LwRtlRBTreeCreate(
+                    &SrvShareCompareInfo,
+                    NULL,
+                    &SrvShareReleaseInfoHandle,
+                    &pShareList->pShareCollection);
+    BAIL_ON_NT_STATUS(ntStatus);
+
     ntStatus = gSrvShareApi.pFnTable->pfnShareRepositoryOpen(&hRepository);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = gSrvShareApi.pFnTable->pfnShareRepositoryBeginEnum(
                                             hRepository,
-                                            ulLimit,
+                                            ulBatchLimit,
                                             &hResume);
     BAIL_ON_NT_STATUS(ntStatus);
 
@@ -122,15 +142,22 @@ SrvShareInitList(
                             (PVOID*)&pShareEntry);
             BAIL_ON_NT_STATUS(ntStatus);
 
-            pShareEntry->pInfo = pShareInfo;
-            InterlockedIncrement(&pShareInfo->refcount);
+            pShareEntry->pInfo = SrvShareAcquireInfo(pShareInfo);
 
             pShareEntry->pNext = pShareList->pShareEntry;
             pShareList->pShareEntry = pShareEntry;
             pShareEntry = NULL;
+
+            ntStatus = LwRtlRBTreeAdd(
+                            pShareList->pShareCollection,
+                            pShareInfo->pwszName,
+                            pShareInfo);
+            BAIL_ON_NT_STATUS(ntStatus);
+
+            InterlockedIncrement(&pShareInfo->refcount);
         }
 
-    } while (ulNumSharesFound == ulLimit);
+    } while (ulNumSharesFound == ulBatchLimit);
 
 cleanup:
 
@@ -191,31 +218,15 @@ SrvShareFindByName_inlock(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    PSRV_SHARE_ENTRY pShareEntry = NULL;
     PSRV_SHARE_INFO pShareInfo = NULL;
 
-    pShareEntry = pShareList->pShareEntry;
+    ntStatus = LwRtlRBTreeFind(
+                    pShareList->pShareCollection,
+                    pwszShareName,
+                    (PVOID*)&pShareInfo);
+    BAIL_ON_NT_STATUS(ntStatus);
 
-    while (pShareEntry)
-    {
-        if (SMBWc16sCaseCmp(pwszShareName, pShareEntry->pInfo->pwszName) == 0)
-        {
-            pShareInfo = pShareEntry->pInfo;
-            break;
-        }
-
-        pShareEntry = pShareEntry->pNext;
-    }
-
-    if (!pShareInfo)
-    {
-        ntStatus = STATUS_NOT_FOUND;
-        BAIL_ON_NT_STATUS(ntStatus);
-    }
-
-    InterlockedIncrement(&pShareInfo->refcount);
-
-    *ppShareInfo = pShareInfo;
+    *ppShareInfo = SrvShareAcquireInfo(pShareInfo);
 
 cleanup:
 
@@ -237,7 +248,8 @@ SrvShareAdd(
     IN     PWSTR                      pwszShareComment,
     IN     PBYTE                      pSecDesc,
     IN     ULONG                      ulSecDescLen,
-    IN     PWSTR                      pwszShareType
+    IN     PWSTR                      pwszShareType,
+    IN     ULONG                      ulShareFlags
     )
 {
     NTSTATUS ntStatus = 0;
@@ -245,6 +257,9 @@ SrvShareAdd(
     PSRV_SHARE_ENTRY pShareEntry = NULL;
     PSRV_SHARE_INFO pShareInfo = NULL;
     wchar16_t wszServiceType[] = LWIO_SRV_SHARE_STRING_ID_DISK_W;
+    wchar16_t wszShareComment[] = {0};
+    PWSTR     pwszShareCommentRef =
+                    (pwszShareComment ? pwszShareComment : &wszShareComment[0]);
     HANDLE hRepository = NULL;
 
     if (IsNullOrEmptyString(pwszShareName))
@@ -294,17 +309,15 @@ SrvShareAdd(
                     &pShareInfo->pwszPath);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    if (pwszShareComment)
-    {
-        ntStatus = SrvAllocateStringW(
-                        pwszShareComment,
+    ntStatus = SrvAllocateStringW(
+                        pwszShareCommentRef,
                         &pShareInfo->pwszComment);
-        BAIL_ON_NT_STATUS(ntStatus);
-    }
-    else
-    {
-        pShareInfo->pwszComment = NULL;
-    }
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = SrvShareMapServiceStringToIdW(
+                    pwszShareType,
+                    &pShareInfo->service);
+    BAIL_ON_NT_STATUS(ntStatus);
 
     if (ulSecDescLen)
     {
@@ -316,14 +329,11 @@ SrvShareAdd(
     }
     else
     {
-        pShareInfo->pSecDesc     = NULL;
-        pShareInfo->ulSecDescLen = 0;
+        ntStatus = SrvShareSetDefaultSecurity(pShareInfo);
+        BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    ntStatus = SrvShareMapServiceStringToIdW(
-                    pwszShareType,
-                    &pShareInfo->service);
-    BAIL_ON_NT_STATUS(ntStatus);
+    pShareInfo->ulFlags = ulShareFlags;
 
     pShareInfo->bMarkedForDeletion = FALSE;
 
@@ -332,8 +342,7 @@ SrvShareAdd(
                     (PVOID*)&pShareEntry);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    pShareEntry->pInfo = pShareInfo;
-    InterlockedIncrement(&pShareInfo->refcount);
+    pShareEntry->pInfo = SrvShareAcquireInfo(pShareInfo);
 
     ntStatus = gSrvShareApi.pFnTable->pfnShareRepositoryOpen(&hRepository);
     BAIL_ON_NT_STATUS(ntStatus);
@@ -346,9 +355,18 @@ SrvShareAdd(
                                             pShareInfo->pwszComment,
                                             (PBYTE)pShareInfo->pSecDesc,
                                             pShareInfo->ulSecDescLen,
-                                            pwszShareType);
+                                            pwszShareType,
+                                            ulShareFlags);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    ntStatus = LwRtlRBTreeAdd(
+                    pShareList->pShareCollection,
+                    pShareInfo->pwszName,
+                    pShareInfo);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    pShareInfo = NULL;
+
     pShareEntry->pNext = pShareList->pShareEntry;
     pShareList->pShareEntry = pShareEntry;
 
@@ -387,6 +405,7 @@ SrvShareUpdate(
     NTSTATUS ntStatus = STATUS_SUCCESS;
     WCHAR wszServiceType[] = LWIO_SRV_SHARE_STRING_ID_DISK_W;
     BOOLEAN  bInLock = FALSE;
+    BOOLEAN  bShareInLock = FALSE;
     HANDLE   hRepository = NULL;
 
     if (!pShareInfo || IsNullOrEmptyString(pShareInfo->pwszName))
@@ -405,6 +424,8 @@ SrvShareUpdate(
                                             pShareInfo->pwszName);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    LWIO_LOCK_RWMUTEX_SHARED(bShareInLock, &pShareInfo->mutex);
+
     ntStatus = gSrvShareApi.pFnTable->pfnShareRepositoryAdd(
                                             hRepository,
                                             pShareInfo->pwszName,
@@ -412,14 +433,19 @@ SrvShareUpdate(
                                             pShareInfo->pwszComment,
                                             (PBYTE)pShareInfo->pSecDesc,
                                             pShareInfo->ulSecDescLen,
-                                            wszServiceType);
+                                            wszServiceType,
+                                            pShareInfo->ulFlags);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pShareInfo->mutex);
+
     gSrvShareApi.pFnTable->pfnShareRepositoryClose(hRepository);
     hRepository = NULL;
 
 cleanup:
 
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pShareInfo->mutex);
+
     LWIO_UNLOCK_RWMUTEX(bInLock, &pShareList->mutex);
 
     return ntStatus;
@@ -471,6 +497,11 @@ SrvShareDelete(
                         pwszShareName);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    ntStatus = LwRtlRBTreeRemove(
+                        pShareList->pShareCollection,
+                        pwszShareName);
+    BAIL_ON_NT_STATUS(ntStatus);
+
 cleanup:
 
     if (hRepository)
@@ -563,9 +594,8 @@ SrvShareEnum(
         pShareEntry = pShareList->pShareEntry;
         for (; i < ulCount; i++)
         {
-            InterlockedIncrement(&pShareEntry->pInfo->refcount);
-
-            ppShares[i] = pShareEntry->pInfo;
+            ntStatus = SrvShareDuplicateInfo(pShareEntry->pInfo, &ppShares[i]);
+            BAIL_ON_NT_STATUS(ntStatus);
 
             pShareEntry = pShareEntry->pNext;
         }
@@ -593,6 +623,107 @@ error:
     goto cleanup;
 }
 
+NTSTATUS
+SrvShareDuplicateInfo(
+    PSRV_SHARE_INFO  pShareInfo,
+    PSRV_SHARE_INFO* ppShareInfo
+    )
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    BOOLEAN  bInLock = FALSE;
+    PSRV_SHARE_INFO pShareInfoCopy = NULL;
+
+    LWIO_LOCK_RWMUTEX_SHARED(bInLock, &pShareInfo->mutex);
+
+    ntStatus = SrvAllocateMemory(
+                    sizeof(SRV_SHARE_INFO),
+                    (PVOID*)&pShareInfoCopy);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    pShareInfoCopy->refcount = 1;
+
+    pthread_rwlock_init(&pShareInfoCopy->mutex, NULL);
+    pShareInfoCopy->pMutex = &pShareInfoCopy->mutex;
+
+    if (pShareInfo->pwszName)
+    {
+        ntStatus = SrvAllocateStringW(
+                        pShareInfo->pwszName,
+                        &pShareInfoCopy->pwszName);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    if (pShareInfo->pwszPath)
+    {
+        ntStatus = SrvAllocateStringW(
+                        pShareInfo->pwszPath,
+                        &pShareInfoCopy->pwszPath);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    if (pShareInfo->pwszComment)
+    {
+        ntStatus = SrvAllocateStringW(
+                        pShareInfo->pwszComment,
+                        &pShareInfoCopy->pwszComment);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    if (pShareInfo->ulSecDescLen)
+    {
+        ntStatus = SrvShareSetSecurity(
+                        pShareInfoCopy,
+                        pShareInfo->pSecDesc,
+                        pShareInfo->ulSecDescLen);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+    pShareInfoCopy->service = pShareInfo->service;
+
+    pShareInfoCopy->bMarkedForDeletion = pShareInfo->bMarkedForDeletion;
+
+    *ppShareInfo = pShareInfoCopy;
+
+cleanup:
+
+    LWIO_UNLOCK_RWMUTEX(bInLock, &pShareInfo->mutex);
+
+    return ntStatus;
+
+error:
+
+    *ppShareInfo = NULL;
+
+    if (pShareInfoCopy)
+    {
+        SrvShareFreeInfo(pShareInfoCopy);
+    }
+
+    goto cleanup;
+}
+
+static
+int
+SrvShareCompareInfo(
+    PVOID pKey1,
+    PVOID pKey2
+    )
+{
+    PWSTR pwszShareName1 = (PWSTR)pKey1;
+    PWSTR pwszShareName2 = (PWSTR)pKey2;
+
+    return SMBWc16sCaseCmp(pwszShareName1, pwszShareName2);
+}
+
+static
+VOID
+SrvShareReleaseInfoHandle(
+    PVOID hShareInfo
+    )
+{
+    SrvShareReleaseInfo((PSRV_SHARE_INFO)hShareInfo);
+}
+
 VOID
 SrvShareFreeListContents(
     IN OUT PLWIO_SRV_SHARE_ENTRY_LIST pShareList
@@ -604,6 +735,12 @@ SrvShareFreeListContents(
         pShareList->pShareEntry = NULL;
     }
 
+    if (pShareList->pShareCollection)
+    {
+        LwRtlRBTreeFree(pShareList->pShareCollection);
+        pShareList->pShareCollection = NULL;
+    }
+
     if (pShareList->pMutex)
     {
         pthread_rwlock_destroy(&pShareList->mutex);
@@ -652,6 +789,16 @@ SrvShareFreeInfoList(
     SrvFreeMemory(ppInfoList);
 }
 
+PSRV_SHARE_INFO
+SrvShareAcquireInfo(
+    IN PSRV_SHARE_INFO pShareInfo
+    )
+{
+    InterlockedIncrement(&pShareInfo->refcount);
+
+    return pShareInfo;
+}
+
 VOID
 SrvShareReleaseInfo(
     IN PSRV_SHARE_INFO pShareInfo
