@@ -62,12 +62,6 @@ SrvShareCreateAbsoluteSecDescFromRel(
     IN   PSECURITY_DESCRIPTOR_RELATIVE pRelSecDesc
     );
 
-static
-NTSTATUS
-SrvShareDefaultSecurity(
-    PSRV_SHARE_INFO pShareInfo
-    );
-
 NTSTATUS
 SrvGetShareName(
     IN  PCSTR  pszHostname,
@@ -300,7 +294,7 @@ SrvShareSetSecurity(
 
     if (pShareInfo->ulSecDescLen == 0)
     {
-        ntStatus = SrvShareDefaultSecurity(pShareInfo);
+        ntStatus = SrvShareSetDefaultSecurity(pShareInfo);
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
@@ -553,7 +547,7 @@ SrvShareAccessCheck(
     )
 {
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    BOOLEAN bLocked = FALSE;
+    BOOLEAN bShareInLock = FALSE;
     BOOLEAN bAccessResult = FALSE;
 
     if (!pGrantedAccess || !pToken)
@@ -562,19 +556,12 @@ SrvShareAccessCheck(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
-    LWIO_LOCK_RWMUTEX_SHARED(bLocked, &pShareInfo->mutex);
-
-    /* This needs to be fixed in the Share API.  A share should
-       never have a NULL SD, but current both C$ and IPC$ do.
-       A NULL SD should be failure to be safe but allow it for
-       now. */
+    LWIO_LOCK_RWMUTEX_SHARED(bShareInLock, &pShareInfo->mutex);
 
     if (!pShareInfo->pAbsSecDesc)
     {
-        // ntStatus = STATUS_ACCESS_DENIED;
-        ntStatus = STATUS_SUCCESS;
-        *pGrantedAccess = FILE_ALL_ACCESS;
-        goto cleanup;
+        ntStatus = STATUS_ACCESS_DENIED;
+        BAIL_ON_NT_STATUS(ntStatus);
     }
 
     bAccessResult = RtlAccessCheck(
@@ -594,7 +581,7 @@ SrvShareAccessCheck(
     }
 
 cleanup:
-    LWIO_UNLOCK_RWMUTEX(bLocked, &pShareInfo->mutex);
+    LWIO_UNLOCK_RWMUTEX(bShareInLock, &pShareInfo->mutex);
 
     return ntStatus;
 
@@ -604,9 +591,8 @@ error:
 }
 
 
-static
 NTSTATUS
-SrvShareDefaultSecurity(
+SrvShareSetDefaultSecurity(
     PSRV_SHARE_INFO pShareInfo
     )
 {
@@ -614,13 +600,30 @@ SrvShareDefaultSecurity(
     PSECURITY_DESCRIPTOR_RELATIVE pRelSecDesc = NULL;
     ULONG ulRelSecDescLen = 0;
     PSECURITY_DESCRIPTOR_ABSOLUTE pAbsSecDesc = NULL;
-    DWORD dwSidCount = 0;
+    DWORD dwAceCount = 0;
     PSID pOwnerSid = NULL;
     PSID pGroupSid = NULL;
-    PSID pAdministratorsSid = NULL;
-    PSID pEveryoneSid = NULL;
     PACL pDacl = NULL;
     DWORD dwSizeDacl = 0;
+    union
+    {
+        SID sid;
+        BYTE buffer[SID_MAX_SIZE];
+    } administratorsSid;
+    union
+    {
+        SID sid;
+        BYTE buffer[SID_MAX_SIZE];
+    } powerUsersSid;
+    union
+    {
+        SID sid;
+        BYTE buffer[SID_MAX_SIZE];
+    } everyoneSid;
+    ULONG ulAdministratorsSidSize = sizeof(administratorsSid.buffer);
+    ULONG ulPowerUsersSidSize = sizeof(powerUsersSid.buffer);
+    ULONG ulEveryoneSidSize = sizeof(everyoneSid.buffer);
+    ACCESS_MASK worldAccessMask = 0;
 
     /* Clear out any existing SecDesc's.  This is not a normal
        use case, but be paranoid */
@@ -643,9 +646,32 @@ SrvShareDefaultSecurity(
                   SECURITY_DESCRIPTOR_REVISION);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    /* Create some SIDs */
+
+    ntStatus = RtlCreateWellKnownSid(
+                   WinBuiltinAdministratorsSid,
+                   NULL,
+                   (PSID)administratorsSid.buffer,
+                   &ulAdministratorsSidSize);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = RtlCreateWellKnownSid(
+                   WinBuiltinPowerUsersSid,
+                   NULL,
+                   (PSID)powerUsersSid.buffer,
+                   &ulPowerUsersSidSize);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = RtlCreateWellKnownSid(
+                   WinWorldSid,
+                   NULL,
+                   (PSID)everyoneSid.buffer,
+                   &ulEveryoneSidSize);
+    BAIL_ON_NT_STATUS(ntStatus);
+
     /* Owner: Administrators */
 
-    ntStatus = RtlAllocateSidFromCString(&pOwnerSid, "S-1-5-32-544");
+    ntStatus = RtlDuplicateSid(&pOwnerSid, &administratorsSid.sid);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = RtlSetOwnerSecurityDescriptor(
@@ -656,7 +682,7 @@ SrvShareDefaultSecurity(
 
     /* Group: Power Users */
 
-    ntStatus = RtlAllocateSidFromCString(&pGroupSid, "S-1-5-32-547");
+    ntStatus = RtlDuplicateSid(&pGroupSid, &powerUsersSid.sid);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = RtlSetGroupSecurityDescriptor(
@@ -665,21 +691,17 @@ SrvShareDefaultSecurity(
                    FALSE);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    /* DACL: Administrators (Full Control), Everyone (Read) */
+    /* DACL:
+       Administrators - (Full Control)
+       Everyone - (Read) for disk shares, (Read/Write) to IPC shares */
 
-    ntStatus = RtlAllocateSidFromCString(&pAdministratorsSid, "S-1-5-32-544");
-    BAIL_ON_NT_STATUS(ntStatus);
-    dwSidCount++;
-
-    ntStatus = RtlAllocateSidFromCString(&pEveryoneSid, "S-1-1-0");
-    BAIL_ON_NT_STATUS(ntStatus);
-    dwSidCount++;
+    dwAceCount = 2;
 
     dwSizeDacl = ACL_HEADER_SIZE +
-        dwSidCount * sizeof(ACCESS_ALLOWED_ACE) +
-        RtlLengthSid(pAdministratorsSid) +
-        RtlLengthSid(pEveryoneSid) -
-        dwSidCount * sizeof(ULONG);
+        dwAceCount * sizeof(ACCESS_ALLOWED_ACE) +
+        RtlLengthSid(&administratorsSid.sid) +
+        RtlLengthSid(&everyoneSid.sid) -
+        dwAceCount * sizeof(ULONG);
 
     ntStatus= RTL_ALLOCATE(&pDacl, VOID, dwSizeDacl);
     BAIL_ON_NT_STATUS(ntStatus);
@@ -692,15 +714,21 @@ SrvShareDefaultSecurity(
                   ACL_REVISION,
                   0,
                   FILE_ALL_ACCESS,
-                  pAdministratorsSid);
+                  &administratorsSid.sid);
     BAIL_ON_NT_STATUS(ntStatus);
 
+    worldAccessMask = FILE_GENERIC_READ | FILE_GENERIC_EXECUTE;
+    if (pShareInfo->service == SHARE_SERVICE_NAMED_PIPE)
+    {
+        worldAccessMask |= FILE_GENERIC_WRITE;
+    }
+
     ntStatus = RtlAddAccessAllowedAceEx(
                   pDacl,
                   ACL_REVISION,
                   0,
-                  FILE_GENERIC_READ,
-                  pEveryoneSid);
+                  worldAccessMask,
+                  &everyoneSid.sid);
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = RtlSetDaclSecurityDescriptor(
@@ -736,8 +764,6 @@ SrvShareDefaultSecurity(
 
 
 cleanup:
-    RTL_FREE(&pAdministratorsSid);
-    RTL_FREE(&pEveryoneSid);
 
     return ntStatus;
 
