@@ -1,6 +1,6 @@
 /* Editor Settings: expandtabs and use 4 spaces for indentation
  * ex: set softtabstop=4 tabstop=8 expandtab shiftwidth=4: *
- * -*- mode: c, c-basic-offset: 4 -*- */
+ */
 
 /*
  * Copyright Likewise Software    2004-2008
@@ -92,20 +92,20 @@ SrvShareRegFindByName(
     PSRV_SHARE_INFO* ppShareInfo
     )
 {
-    NTSTATUS        ntStatus       = STATUS_SUCCESS;
-    HKEY            hRootKey       = NULL;
-    HKEY            hKey           = NULL;
-    HKEY            hSecKey        = NULL;
-    REG_DATA_TYPE   dataType       = REG_UNKNOWN;
-    ULONG           ulValueLen     = MAX_VALUE_LENGTH;
-    REG_DATA_TYPE   dataSecType    = REG_UNKNOWN;
-    ULONG           ulSecValueLen  = MAX_VALUE_LENGTH;
-    PSRV_SHARE_INFO pShareInfo     = NULL;
-    BYTE            pData[MAX_VALUE_LENGTH]    = {0};
-    BYTE            pSecData[MAX_VALUE_LENGTH] = {0};
-    wchar16_t       wszHKTM[]        = HKEY_THIS_MACHINE_W;
-    wchar16_t       wszSharesKey[]   = REG_KEY_PATH_SRV_SHARES_W;
-    wchar16_t       wszShareSecKey[] = REG_KEY_PATH_SRV_SHARES_SECURITY_W;
+    NTSTATUS        ntStatus         = STATUS_SUCCESS;
+    HKEY            hRootKey         = NULL;
+    HKEY            hKey             = NULL;
+    HKEY            hSecKey          = NULL;
+    REG_DATA_TYPE   dataType         = REG_UNKNOWN;
+    ULONG           ulValueLen       = MAX_VALUE_LENGTH;
+    REG_DATA_TYPE   dataSecType      = REG_UNKNOWN;
+    ULONG           ulSecValueLen    = MAX_VALUE_LENGTH;
+    PSRV_SHARE_INFO pShareInfo       = NULL;
+    BYTE            pData[MAX_VALUE_LENGTH]      = {0};
+    BYTE            pSecData[MAX_VALUE_LENGTH]   = {0};
+    wchar16_t       wszHKTM[]          = HKEY_THIS_MACHINE_W;
+    wchar16_t       wszSharesKey[]     = REG_KEY_PATH_SRV_SHARES_W;
+    wchar16_t       wszShareSecKey[]   = REG_KEY_PATH_SRV_SHARES_SECURITY_W;
 
     ntStatus = NtRegOpenKeyExW(
                     hRepository,
@@ -204,7 +204,8 @@ SrvShareRegAdd(
     IN PWSTR  pwszComment,
     IN PBYTE  pSecDesc,
     IN ULONG  ulSecDescLen,
-    IN PWSTR  pwszService
+    IN PWSTR  pwszService,
+    IN ULONG  ulFlags
     )
 {
     NTSTATUS ntStatus    = 0;
@@ -214,13 +215,17 @@ SrvShareRegAdd(
     PWSTR*   ppwszValues = NULL;
     PBYTE    pOutData    = NULL;
     SSIZE_T  cOutDataLen = 0;
+    PWSTR    pwszFlags   = NULL;
     ULONG    ulCount     = 0;
-    wchar16_t wszHKTM[]        = HKEY_THIS_MACHINE_W;
-    wchar16_t wszSharesKey[]   = REG_KEY_PATH_SRV_SHARES_W;
-    wchar16_t wszShareSecKey[] = REG_KEY_PATH_SRV_SHARES_SECURITY_W;
+    wchar16_t wszHKTM[]          = HKEY_THIS_MACHINE_W;
+    wchar16_t wszSharesKey[]     = REG_KEY_PATH_SRV_SHARES_W;
+    wchar16_t wszShareSecKey[]   = REG_KEY_PATH_SRV_SHARES_SECURITY_W;
     wchar16_t wszPathPrefix[]    = REG_KEY_PATH_PREFIX_W;
+    wchar16_t wszFlagsPrefix[]   = REG_KEY_FLAGS_PREFIX_W;
     ULONG     ulPathPrefixLen =
                         (sizeof(wszPathPrefix)/sizeof(wchar16_t)) - 1;
+    ULONG     ulFlagsPrefixLen =
+                        (sizeof(wszFlagsPrefix)/sizeof(wchar16_t)) - 1;
 
     if (IsNullOrEmptyString(pwszShareName))
     {
@@ -251,7 +256,7 @@ SrvShareRegAdd(
                     &hKey);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    ntStatus = SrvAllocateMemory(sizeof(PWSTR) * 4, (PVOID*)&ppwszValues);
+    ntStatus = SrvAllocateMemory(sizeof(PWSTR) * 5, (PVOID*)&ppwszValues);
     BAIL_ON_NT_STATUS(ntStatus);
 
     // Path
@@ -305,6 +310,22 @@ SrvShareRegAdd(
                 wc16slen(pwszService) * sizeof(wchar16_t));
     }
 
+    // Flags
+    ntStatus = LwRtlWC16StringAllocatePrintfW(&pwszFlags, L"0x%08x", ulFlags);
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    ntStatus = SrvAllocateMemory(
+                            sizeof(wszFlagsPrefix) + wc16slen(pwszFlags) * sizeof(wchar16_t),
+                            OUT_PPVOID(&ppwszValues[++ulCount]));
+    BAIL_ON_NT_STATUS(ntStatus);
+
+    memcpy((PBYTE)&ppwszValues[ulCount][0],
+           (PBYTE)&wszFlagsPrefix[0],
+           sizeof(wszFlagsPrefix) - sizeof(wchar16_t));
+    memcpy((PBYTE)&ppwszValues[ulCount][ulFlagsPrefixLen],
+           (PBYTE)pwszFlags,
+           wc16slen(pwszFlags) * sizeof(wchar16_t));
+
     ntStatus = NtRegMultiStrsToByteArrayW(ppwszValues, &pOutData, &cOutDataLen);
     BAIL_ON_NT_STATUS(ntStatus);
 
@@ -366,6 +387,8 @@ cleanup:
         RegFreeMemory(pOutData);
     }
 
+    RTL_FREE(&pwszFlags);
+
     return ntStatus;
 
 error:
@@ -376,18 +399,23 @@ error:
 NTSTATUS
 SrvShareRegBeginEnum(
     HANDLE  hRepository,
-    ULONG   ulLimit,
+    ULONG   ulBatchLimit,
     PHANDLE phResume
     )
 {
     NTSTATUS  ntStatus       = 0;
-    ULONG     ulValueCount   = 0;
     HKEY      hRootKey       = NULL;
     HKEY      hKey           = NULL;
     wchar16_t wszHKTM[]      = HKEY_THIS_MACHINE_W;
     wchar16_t wszSharesKey[] = REG_KEY_PATH_SRV_SHARES_W;
     PSRV_SHARE_REG_ENUM_CONTEXT pEnumContext = NULL;
 
+    if (!ulBatchLimit || (ulBatchLimit == UINT32_MAX))
+    {
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     ntStatus = SrvAllocateMemory(
                     sizeof(SRV_SHARE_REG_ENUM_CONTEXT),
                     (PVOID*)&pEnumContext);
@@ -420,21 +448,14 @@ SrvShareRegBeginEnum(
                     NULL,
                     NULL,
                     NULL,
-                    &ulValueCount,
+                    &pEnumContext->ulValuesAvailable,
                     &pEnumContext->ulMaxValueNameLen,
                     &pEnumContext->ulMaxValueLen,
                     NULL,
                     NULL);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    if (ulLimit > ulValueCount)
-    {
-        pEnumContext->ulMaxIndex = ulValueCount;
-    }
-    else
-    {
-        pEnumContext->ulMaxIndex = ulLimit;
-    }
+    pEnumContext->ulBatchLimit = ulBatchLimit;
 
     *phResume = (HANDLE)pEnumContext;
 
@@ -470,6 +491,7 @@ SrvShareRegEnum(
 {
     NTSTATUS ntStatus          = 0;
     ULONG    ulIndex           = 0;
+    ULONG    ulBatchIndex      = 0;
     HKEY     hRootKey          = NULL;
     HKEY     hKey              = NULL;
     HKEY     hSecKey           = NULL;
@@ -485,12 +507,10 @@ SrvShareRegEnum(
     BYTE             pSecData[MAX_VALUE_LENGTH] = {0};
     PSRV_SHARE_REG_ENUM_CONTEXT pResume = (PSRV_SHARE_REG_ENUM_CONTEXT)hResume;
 
-    if (!pResume->ulMaxIndex)
+    if (!pResume)
     {
-        *pppShareInfoList = NULL;
-        *pulNumSharesFound = 0;
-
-        goto cleanup;
+        ntStatus = STATUS_INVALID_PARAMETER;
+        BAIL_ON_NT_STATUS(ntStatus);
     }
 
     ntStatus = NtRegOpenKeyExW(
@@ -521,11 +541,13 @@ SrvShareRegEnum(
     BAIL_ON_NT_STATUS(ntStatus);
 
     ntStatus = SrvAllocateMemory(
-                  pResume->ulMaxIndex * sizeof(*ppShareInfoList),
-                  (PVOID) &ppShareInfoList);
+                    pResume->ulBatchLimit * sizeof(*ppShareInfoList),
+                    (PVOID) &ppShareInfoList);
     BAIL_ON_NT_STATUS(ntStatus);
 
-    for (ulIndex = 0; ulIndex < pResume->ulMaxIndex; ulIndex++)
+    ulBatchIndex = pResume->ulBatchIndex;
+
+    for (ulIndex = 0; ulIndex < pResume->ulBatchLimit; ulIndex++)
     {
         ULONG ulMaxValueNameLen = 0;
         ULONG ulMaxValueLen     = 0;
@@ -557,13 +579,18 @@ SrvShareRegEnum(
         ntStatus = NtRegEnumValueW(
                       hRepository,
                       hKey,
-                      ulIndex,
+                      ulBatchIndex++,
                       pwszValueName,
                       &ulMaxValueNameLen,
                       NULL,
                       &dataType,
                       pData,
                       &ulMaxValueLen);
+        if (ntStatus == STATUS_NO_MORE_MATCHES)
+        {
+            ntStatus = STATUS_SUCCESS;
+            break;
+        }
         BAIL_ON_NT_STATUS(ntStatus);
 
         if (REG_MULTI_SZ != dataType)
@@ -599,6 +626,8 @@ SrvShareRegEnum(
         BAIL_ON_NT_STATUS(ntStatus);
     }
 
+    pResume->ulBatchIndex = ulBatchIndex;
+
     *pppShareInfoList = ppShareInfoList;
     *pulNumSharesFound = ulNumSharesFound;
 
@@ -803,6 +832,7 @@ SrvShareRegWriteToShareInfo(
     NTSTATUS        ntStatus     = STATUS_SUCCESS;
     PSRV_SHARE_INFO pShareInfo   = NULL;
     PWSTR*          ppwszValues  = NULL;
+    PSTR            pszFlags     = NULL;
 
     ntStatus = SrvAllocateMemory(sizeof(*pShareInfo), (PVOID*)&pShareInfo);
     BAIL_ON_NT_STATUS(ntStatus);
@@ -828,6 +858,10 @@ SrvShareRegWriteToShareInfo(
         ULONG     ulServicePrefixLen =
                             (sizeof(wszServicePrefix)/sizeof(wchar16_t)) - 1;
 
+        wchar16_t wszFlagsPrefix[]   = REG_KEY_FLAGS_PREFIX_W;
+        ULONG     ulFlagsPrefixLen =
+                            (sizeof(wszFlagsPrefix)/sizeof(wchar16_t)) - 1;
+
         ntStatus = NtRegByteArrayToMultiStrs(pData, ulDataLen, &ppwszValues);
         BAIL_ON_NT_STATUS(ntStatus);
 
@@ -861,14 +895,33 @@ SrvShareRegWriteToShareInfo(
                 BAIL_ON_NT_STATUS(ntStatus);
 
             }
+            else if (!wc16sncmp(&wszFlagsPrefix[0], pwszValue, ulFlagsPrefixLen))
+            {
+                ntStatus = RtlCStringAllocateFromWC16String(
+                                &pszFlags,
+                                &pwszValue[ulFlagsPrefixLen]);
+                BAIL_ON_NT_STATUS(ntStatus);
+
+                pShareInfo->ulFlags = strtol(pszFlags, NULL, 16);
+            }
             else
             {
-                    ntStatus = STATUS_INVALID_PARAMETER_3;
-                    BAIL_ON_NT_STATUS(ntStatus);
+                ntStatus = STATUS_INVALID_PARAMETER_3;
+                BAIL_ON_NT_STATUS(ntStatus);
             }
         }
     }
 
+    if (!pShareInfo->pwszComment)
+    {
+        wchar16_t wszComment[] = {0};
+
+        ntStatus = SrvAllocateStringW(
+                        &wszComment[0],
+                        &pShareInfo->pwszComment);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
     if (ulSecDataLen)
     {
         ntStatus = SrvShareSetSecurity(
@@ -877,6 +930,13 @@ SrvShareRegWriteToShareInfo(
                        ulSecDataLen);
         BAIL_ON_NT_STATUS(ntStatus);
     }
+    else
+    {
+        ntStatus = SrvShareSetDefaultSecurity(pShareInfo);
+        BAIL_ON_NT_STATUS(ntStatus);
+    }
+
+
 
     *ppShareInfo = pShareInfo;
 
@@ -887,6 +947,8 @@ cleanup:
         RegFreeMultiStrsW(ppwszValues);
     }
 
+    RTL_FREE(&pszFlags);
+
     return ntStatus;
 
 error:
