@@ -1,11 +1,8 @@
 #ifndef __FSERV_H__
 #define __FSERV_H__
 
-/* Opaque handle to a server */
-typedef struct FServ FServ;
-
 /* Opqaue handle to a file */
-typedef struct FServFile FServFile;
+typedef struct FileHandle FServFile;
 
 typedef enum FServMode
 {
@@ -14,22 +11,9 @@ typedef enum FServMode
     FSERV_MODE_APPEND = 4
 } FServMode;
 
-/* Connect to local fserv */
-int
-fserv_connect(
-    FServ** out_fserv
-    );
-
-/* Disconnect an fserv connection */
-int
-fserv_disconnect(
-    FServ* fserv
-    );
-
-/* Open a file using an fserv connection */
+/* Open a file */
 int
 fserv_open(
-    FServ* fserv,
     const char* path,
     FServMode mode,
     FServFile** out_file
