@@ -116,6 +116,7 @@ typedef struct LWMsgDispatchSpec
 #endif
 const LWMsgDispatchSpec;
 
+#ifndef LWMSG_DISABLE_DEPRECATED
 /**
  * @brief Define message handler in a dispatch table <b>(DEPRECATED)</b>
  *
@@ -128,6 +129,7 @@ const LWMsgDispatchSpec;
  */
 #define LWMSG_DISPATCH(tag, func) \
     {LWMSG_DISPATCH_TYPE_OLD, (tag), (void*) (LWMsgAssocDispatchFunction) (func)}
+#endif
 
 /**
  * @brief Define blocking message handler
@@ -170,6 +172,12 @@ const LWMsgDispatchSpec;
  */
 #define LWMSG_DISPATCH_END {LWMSG_DISPATCH_TYPE_END, -1, NULL}
 
+/**
+ * @brief Peer structure
+ *
+ * An opaque structure from which calls can be answered
+ * or dispatched.
+ */
 typedef struct LWMsgPeer LWMsgPeer;
 
 /**
@@ -210,7 +218,7 @@ typedef
 LWMsgStatus
 (*LWMsgPeerCallFunction) (
     LWMsgCall* call,
-    LWMsgParams* in,
+    const LWMsgParams* in,
     LWMsgParams* out,
     void* data
     );
@@ -221,9 +229,9 @@ LWMsgStatus
  * A callback function which is invoked when an unexpected error occurs
  * in the process of handling incoming calls.
  *
- * @param peer the peer handle
- * @param status the status code of the error
- * @param data a user data pointer
+ * @param[in] peer the peer handle
+ * @param[in] status the status code of the error
+ * @param[in] data a user data pointer
  */
 typedef
 void
@@ -264,7 +272,7 @@ lwmsg_peer_new(
  * a peer which is listening for incoming calls will block
  * until all outstanding incoming calls can be cancelled.
  *
- * @param peer the peer object to delete
+ * @param[in,out] peer the peer object to delete
  */
 void
 lwmsg_peer_delete(
@@ -277,9 +285,9 @@ lwmsg_peer_delete(
  * Sets the specified timeout to the specified value.
  * See #lwmsg_assoc_set_timeout() for more information.
  *
- * @param peer the peer object
- * @param type the type of timeout to set
- * @param value the value, or NULL for no timeout
+ * @param[in,out] peer the peer object
+ * @param[in] type the type of timeout to set
+ * @param[in] value the value, or NULL for no timeout
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{UNSUPPORTED, the specified timeout type is not supported}
@@ -300,8 +308,8 @@ lwmsg_peer_set_timeout(
  * simultaneously.  Associations beyond this will wait until a slot becomes
  * available.
  *
- * @param peer the peer object
- * @param max_clients the maximum number of simultaneous associations to support
+ * @param[in,out] peer the peer object
+ * @param[in] max_clients the maximum number of simultaneous associations to support
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already listening}
@@ -320,8 +328,8 @@ lwmsg_peer_set_max_listen_clients(
  * waiting until a client slot becomes available.  Pending associations beyond
  * this value will be rejected outright.
  *
- * @param peer the peer object
- * @param max_backlog the maximum number of clients to queue
+ * @param[in,out] peer the peer object
+ * @param[in] max_backlog the maximum number of clients to queue
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already active}
@@ -339,8 +347,8 @@ lwmsg_peer_set_max_listen_backlog(
  * Adds a set of message dispatch functions to the specified
  * peer object.
  *
- * @param peer the peer object
- * @param spec the dispatch specification
+ * @param[in,out] peer the peer object
+ * @param[in] spec the dispatch specification
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_memory
@@ -356,14 +364,14 @@ lwmsg_peer_add_dispatch_spec(
  * @brief Add listen socket
  *
  * Adds a socket on which the peer will accept incoming associations.
- * This function must be passed a valid file descriptor which is socket
+ * This function must be passed a valid socket descriptor
  * that matches the specified mode and is already listening (has had
  * listen() called on it).  The peer will assume ownership of this
  * fd.
  *
- * @param peer the peer object
- * @param type the endpoint type
- * @param fd the socket on which to listen
+ * @param[in,out] peer the peer object
+ * @param[in] type the endpoint type
+ * @param[in] fd the socket on which to listen
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already active}
@@ -382,10 +390,10 @@ lwmsg_peer_add_listen_fd(
  *
  * Adds an endpoint on which the peer will listen for connections.
  *
- * @param peer the peer object
- * @param type the type of endpoint
- * @param endpoint the endpoint path on which to listen
- * @param permissions permissions for the endpoint (only applicable to local endpoints)
+ * @param[in,out] peer the peer object
+ * @param[in] type the type of endpoint
+ * @param[in] endpoint the endpoint path on which to listen
+ * @param[in] permissions permissions for the endpoint (only applicable to local endpoints)
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already active}
@@ -401,17 +409,16 @@ lwmsg_peer_add_listen_endpoint(
     );
 
 /**
- * @brief Set session construct and destruct functions for incoming assocations
+ * @brief Set session construct and destruct functions for incoming clients
  *
- * Sets functions which will be called when a new session
- * is established through an incoming association.  The constructor function
- * may set up a session context which the destructor function
- * should clean up when the session is terminated.
+ * Sets functions which will be called when a client is accepted into a session
+ * for the first time.  The constructor function may set up a session context
+ * which the destructor function should clean up when the session is terminated.
  *
- * @param peer the peer handle
- * @param construct a session constructor function
- * @param destruct a session destructor function
- * @param data a user data pointer to be passed to both functions
+ * @param[in,out] peer the peer handle
+ * @param[in] construct a session constructor function
+ * @param[in] destruct a session destructor function
+ * @param[in] data a user data pointer to be passed to both functions
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already active}
@@ -431,8 +438,8 @@ lwmsg_peer_set_listen_session_functions(
  * Sets the user data pointer which is passed to dispatch functions.
  * This function may only be used while the peer is inactive.
  *
- * @param peer the peer object
- * @param data the data pointer
+ * @param[in,out] peer the peer object
+ * @param[in] data the data pointer
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{INVALID_STATE, the peer is already active}
@@ -450,7 +457,7 @@ lwmsg_peer_set_dispatch_data(
  * Gets the user data pointer which is passed to dispatch functions.
  * If no pointer was explicitly set, the value defaults to NULL.
  *
- * @param peer the peer object
+ * @param[in] peer the peer object
  * @return the data pointer
  */
 void*
@@ -466,7 +473,7 @@ lwmsg_peer_get_dispatch_data(
  * function returns once all endpoints are ready to accept incoming
  * associations.
  *
- * @param peer the peer object
+ * @param[in,out] peer the peer object
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_endstatus
@@ -484,8 +491,7 @@ lwmsg_peer_start_listen(
  * incoming calls.  This function returns once all incoming associations have
  * been terminated.
  *
- * @param peer the peer object
- * @param peer the peer object
+ * @param[in,out] peer the peer object
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_endstatus
@@ -505,9 +511,9 @@ lwmsg_peer_stop_listen(
  *
  * @warning Do not call #lwmsg_peer_stop_listen() from an exception handler
  *
- * @param peer the peer handle
- * @param except the handler function
- * @param except_data a user data pointer to pass to the handler function
+ * @param[in,out] peer the peer handle
+ * @param[in] except the handler function
+ * @param[in] except_data a user data pointer to pass to the handler function
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_endstatus
@@ -525,9 +531,9 @@ lwmsg_peer_set_exception_function(
  * Adds an endpoint which will be used when establishing an outgoing association
  * with #lwmsg_peer_connect().
  *
- * @param peer the peer handle
- * @param type the type of endpoint
- * @param endpoint the endpoint path
+ * @param[in,out] peer the peer handle
+ * @param[in] type the type of endpoint
+ * @param[in] endpoint the endpoint path
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_endstatus
@@ -540,16 +546,19 @@ lwmsg_peer_add_connect_endpoint(
     );
 
 /**
- * @brief Establish outgoing association
+ * @brief Establish outgoing session
  *
- * Establishes an outgoing association on one of the endpoints registered
+ * Establishes an outgoing session on one of the endpoints registered
  * with #lwmsg_peer_add_connect_endpoint().  The endpoints will be tried
  * in the order they were added until one succeeds.  If an outgoing
- * association is already established, this function is a no-op.  If an
- * outgoing association is in the process of being established, this
- * function will block until it either succeeds or fails.
+ * session is already established, this function will perform no action
+ * but will still return the established session.  If an outgoing session
+ * is in the process of being established, this function will block until
+ * it either succeeds or fails.  The session returned is guaranteed to
+ * remain valid until #lwmsg_peer_disconnect() is used.
  *
- * @param peer the peer handle
+ * @param[in,out] peer the peer handle
+ * @param[out] session the established session
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_code{CONNECTION_REFUSED, the connection was refused}
@@ -558,18 +567,19 @@ lwmsg_peer_add_connect_endpoint(
  */
 LWMsgStatus
 lwmsg_peer_connect(
-    LWMsgPeer* peer
+    LWMsgPeer* peer,
+    LWMsgSession** session
     );
 
 /**
- * @brief Close outgoing association
+ * @brief Close outgoing session
  *
- * Closes any assocation established by #lwmsg_peer_connect().  All outstanding
- * outgoing calls will be cancelled.  If no association is currently established,
- * thus function is a no-op.  If the association is already being disconnected,
+ * Closes the session established by #lwmsg_peer_connect().  All outstanding
+ * outgoing calls will be cancelled.  If no session is currently established,
+ * this function is a no-op.  If the association is already being disconnected,
  * this function will block until it either succeeds or fails.
  *
- * @param peer the peer handle
+ * @param[in,out] peer the peer handle
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_endstatus
@@ -588,7 +598,7 @@ lwmsg_peer_disconnect(
  * #lwmsg_peer_connect(), this function will do so implicitly.  The acquired
  * call handle should be released with #lwmsg_call_release() when no longer needed.
  *
- * @param[in] peer the peer handle
+ * @param[in,out] peer the peer handle
  * @param[out] call the acquired call handle
  * @lwmsg_status
  * @lwmsg_success
