@@ -98,8 +98,9 @@
  * Type specifications have the following restrictions compared to C type definitions:
  * - The length of arrays and pointer referents must be specified:
  *     - As a static length with #LWMSG_ATTR_LENGTH_STATIC()
- *     - As the value of a <i>previous</i> integer structure member with #LWMSG_ATTR_LENGTH_MEMBER()
- *     - As being implicit through nul-termination with #LWMSG_ATTR_STRING
+ *     - As the value of a <i>previous</i> integer or enum structure member with
+ *       #LWMSG_ATTR_LENGTH_MEMBER()
+ *     - As being implicit through zero-termination with #LWMSG_ATTR_ZERO_TERMINATED
  * - The active arm of unions must be specified through a discriminator:
  *     - Each union arm must be marked with an integer tag with #LWMSG_ATTR_TAG()
  *     - Each instance of a union type must be correlated with a <i>previous</i> integer
@@ -111,8 +112,8 @@
  *   Members will be marshalled in the order of the type specification.
  * - The specified width of an integer type may differ from the C type.  The
  *   integer will be marshalled to the specified size, with widening or truncating
- *   conversions applied automatically.  <b>Caution</b>: truncating conversions which
- *   cause overflow or underflow will generate runtime errors.
+ *   conversions applied automatically.  <b>Caution</b>: Truncating conversions which
+ *   cause overflow will generate runtime errors.
  */
 
 /* @{ */
@@ -123,15 +124,15 @@ struct LWMsgContext;
 
 typedef enum LWMsgKind
 {
-    LWMSG_KIND_NONE,
-    /* Atomic integral type */
-    LWMSG_KIND_INTEGER,
-    LWMSG_KIND_STRUCT,
-    LWMSG_KIND_UNION,
-    LWMSG_KIND_ARRAY,
-    LWMSG_KIND_POINTER,
-    LWMSG_KIND_CUSTOM,
-    LWMSG_KIND_VOID
+    LWMSG_KIND_NONE = 0,
+    LWMSG_KIND_VOID = 1,
+    LWMSG_KIND_INTEGER = 2,
+    LWMSG_KIND_ENUM = 3,
+    LWMSG_KIND_STRUCT = 4,
+    LWMSG_KIND_UNION = 5,
+    LWMSG_KIND_ARRAY = 6,
+    LWMSG_KIND_POINTER = 7,
+    LWMSG_KIND_CUSTOM = 128
 } LWMsgKind;
 
 typedef enum LWMsgArrayTermination
@@ -164,110 +165,135 @@ typedef enum LWMsgSignage
     LWMSG_UNSIGNED
 } LWMsgSignage;
 
-typedef struct LWMsgTypeAttrs
+/**
+ * @brief Type flag bitmask
+ *
+ * Describes set/unset attributes of a type
+ */
+typedef enum LWMsgTypeFlags
 {
-    size_t custom;
-    size_t range_low;
-    size_t range_high;
-    unsigned nonnull:1;
-    unsigned aliasable:1;
-    unsigned sensitive:1;
-    unsigned promoted:1;
-} LWMsgTypeAttrs;
+    /**
+     * Type may not be NULL
+     * @hideinitializer
+     */
+    LWMSG_TYPE_FLAG_NOT_NULL  = 0x01,
+    /**
+     * Type contains sensitive data
+     * @hideinitializer
+     */
+    LWMSG_TYPE_FLAG_SENSITIVE = 0x02,
+    /**
+     * Type was implicitly promoted to a pointer
+     * @hideinitializer
+     */
+    LWMSG_TYPE_FLAG_PROMOTED  = 0x04,
+    /**
+     * Type has a range constraint
+     * @hideinitializer
+     */
+    LWMSG_TYPE_FLAG_RANGE     = 0x08,
+    /**
+     * Pointer type is aliasable
+     * @hideinitializer
+     */
+    LWMSG_TYPE_FLAG_ALIASABLE = 0x08
+} LWMsgTypeFlags;
+
+/**
+ * @brief Type
+ *
+ * An opqaue handle representing a type.
+ */
+typedef struct LWMsgType LWMsgType;
 
 /**
  * @brief Custom marshal function
  *
- * A callback function type which implements marshalling logic for
- * a custom marshaller type.
+ * A callback function which converts a presented type to its transmitted form.
+ * The transmitted_object parameter will point to a block of memory large enough
+ * for the transmitted type.  Any resources allocated to construct the transmitted
+ * object can be freed by the corresponding #LWMsgTypeDestroyTransmittedFunction.
  *
- * @param context the marshalling context
- * @param object_size the in-memory size of the object to marshal, or 0 if unknown
- * @param object the in-memory object to marshal
- * @param attrs attributes of the type to marshal
- * @param buffer the marshalling buffer with the cursor set to where the 
- * serialized representation should be written
- * @param data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
+ * @param[in]  context the data context
+ * @param[in]  type the type to marshal
+ * @param[in]  presented_object the presented object
+ * @param[out] transmitted_object the object which will be transmitted
+ * @param[in]  data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
  * in the type specification
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_etc{implementation-specific error}
  * @lwmsg_endstatus
  */
-typedef LWMsgStatus (*LWMsgCustomMarshalFunction) (
+typedef LWMsgStatus (*LWMsgTypeMarshalFunction) (
     struct LWMsgDataContext* context,
-    size_t object_size,
+    LWMsgType* type,
     void* object,
-    LWMsgTypeAttrs* attrs,
-    LWMsgBuffer* buffer,
+    void* transmit_object,
     void* data
     );
 
 /**
  * @brief Custom unmarshal function
  *
- * A callback function type which implements unmarshalling logic for
- * a custom marshaller type.
+ * A callback function type which converts a transmitted type to its presented form.
+ * The presented_object parameter will point to a block of memory large enough for
+ * the presented type.  Any resources allocated to construct the presented object can
+ * be freed by the corresponding #LWMsgTypeDestroyPresentedFunction
  *
- * @param context the marshalling context
- * @param buffer the marshalling buffer with the cursor set to the start of 
- * the serialized representation
- * @param object_size the in-memory size of the object to unmarshal, or 0 if known
- * @param attrs attributes of the type to unmarshal
- * @param object the location to unmarshal to
- * @param data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
+ * @param[in]  context the data context
+ * @param[in]  type the type to marshal
+ * @param[in]  transmit_object the transmitted object
+ * @param[out] presented_object space for the presented object
+ * @param[in]  data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
  * in the type specification
  * @lwmsg_status
  * @lwmsg_success
  * @lwmsg_etc{implementation-specific error}
  * @lwmsg_endstatus
  */
-typedef LWMsgStatus (*LWMsgCustomUnmarshalFunction) (
+typedef LWMsgStatus (*LWMsgTypeUnmarshalFunction) (
     struct LWMsgDataContext* context,
-    LWMsgBuffer* buffer,
-    size_t object_size,
-    LWMsgTypeAttrs* attrs,
+    LWMsgType* type,
+    void* transmit_object,
     void* object,
     void* data
     );
 
 /**
- * @brief Custom free function
+ * @brief Destroy presented object function
  *
- * A callback function type which frees an instance of a custom type.
+ * A callback function type which releases any resources associated with a
+ * presented object.
  *
- * @param context the marshalling context
- * @param object_size the in-memory size of the object to free, or 0 if unknown
- * @param attrs attributes of the type to free
+ * @param context the data context
+ * @param type the type to free
  * @param object the address of the object to free
  * @param data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
  * in the type specification
  */
-typedef void (*LWMsgCustomFreeFunction) (
-    const struct LWMsgContext* context,
-    size_t object_size,
-    LWMsgTypeAttrs* attr,
+typedef void (*LWMsgTypeDestroyPresentedFunction) (
+    struct LWMsgDataContext* context,
+    LWMsgType* type,
     void* object,
     void* data
     );
 
 /**
- * @brief Print callback function
+ * @brief Destroy transmitted object function
  *
- * A callback function used to print text by #lwmsg_data_print_graph()
+ * A callback function which releases any resources associated with a
+ * transmitted object.
  *
- * @param text the text to print
- * @param length the length of text
- * @param data a custom data pointer
- * @lwmsg_status
- * @lwmsg_success
- * @lwmsg_etc{implementation-specific error}
- * @lwmsg_endstatus
+ * @param[in] context the data context
+ * @param[in,out] object the transmitted object
+ * @param[in] data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
+ * in the type specification
  */
-typedef LWMsgStatus
-(*LWMsgDataPrintFunction) (
-    const char* text,
-    size_t length,
+typedef void (*LWMsgTypeDestroyTransmittedFunction) (
+    struct LWMsgDataContext* context,
+    LWMsgType* type,
+    void* object,
     void* data
     );
 
@@ -277,69 +303,19 @@ typedef LWMsgStatus
  * A callback function type which prints the representation of a custom type,
  * using the provided type printing callback.
  *
- * @param context the marshalling context
- * @param object_size the in-memory size of the object to free, or 0 if unknown
+ * @param context the data context
  * @param object the address of the object to print
- * @param attrs attributes of the type to print
+ * @param type the type to print
  * @param data the user data pointer specified to LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM()
  * in the type specification
- * @param print the type print callback
- * @param print_data the user data pointer to pass to the type print callback
+ * @param buffer the buffer into which to write
  */
-typedef LWMsgStatus (*LWMsgCustomPrintFunction) (
-    const struct LWMsgContext* context,
-    size_t object_size,
-    void* object,
-    LWMsgTypeAttrs* attr,
-    void* data,
-    LWMsgDataPrintFunction print,
-    void* print_data
-    );
-
-/**
- * @brief Custom marshaller type class
- *
- * Describes a custom marshaller type which may be used with
- * LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM() in a type specification
- *
- */
-typedef struct LWMsgCustomTypeClass
-{
-    /** @brief Whether the type should be considered a pointer */
-    LWMsgBool is_pointer;
-    /** @brief Marshal callback function */
-    LWMsgCustomMarshalFunction marshal;
-    /** @brief Unmarshal callback function */
-    LWMsgCustomUnmarshalFunction unmarshal;
-    /** @brief Free callback function */
-    LWMsgCustomFreeFunction free;
-    /** @brief Print callback function */
-    LWMsgCustomPrintFunction print;
-} LWMsgCustomTypeClass;
-
-/**
- * @brief Custom data verification function
- *
- * A callback function which performs verification of
- * in-memory data immediately before marshalling or immediately
- * after unmarshalling.
- *
- * @param context the marshalling context
- * @param unmarshalling true whether the operation being performed is unmarshalling
- * @param object_size the size of the object to verify in bytes
- * @param object the object to verify
- * @param data the user data pointer given to LWMSG_ATTR_VERIFY()
- * @lwmsg_status
- * @lwmsg_success
- * @lwmsg_code{MALFORMED, the object did not pass verification}
- * @lwmsg_endstatus
- */
-typedef LWMsgStatus (*LWMsgVerifyFunction) (
+typedef LWMsgStatus (*LWMsgTypePrintFunction) (
     struct LWMsgDataContext* context,
-    LWMsgBool unmarshalling,
-    size_t object_size,
+    LWMsgType* attr,
     void* object,
-    void *data
+    void* data,
+    LWMsgBuffer* buffer
     );
 
 /**
@@ -351,6 +327,71 @@ typedef LWMsgStatus (*LWMsgVerifyFunction) (
  */
 typedef size_t const LWMsgTypeSpec;
 
+/**
+ * @brief Custom type class
+ *
+ * Describes a custom type which may be used with
+ * LWMSG_CUSTOM() or LWMSG_MEMBER_CUSTOM() in a type specification.
+ *
+ * A custom type actually comprises two types:
+ * - The presented type, which is the type seen by the application.
+ * - The transmitted type, which is the type represented in the
+ *   data stream.
+ *
+ * The presented type's size must be known to the marshaller so that
+ * it can allocate adequate memory for it when reconstructing
+ * the object graph from the data stream.  All other details
+ * of the type are up to the application, which must provide
+ * functions to convert to and from the transmitted type,
+ * functions to destroy the presented and transmitted types,
+ * and a function to print the presented type in human-readable form.
+ */
+typedef struct LWMsgTypeClass
+{
+    /**
+     * @brief Whether the type is a pointer
+     *
+     * If a custom type is a pointer, it will not be subject to automatic
+     * pointer promotion when passed to #lwmsg_data_marshal() et al.
+     */
+    LWMsgBool is_pointer;
+    /** @brief The type of the transmitted form */
+    LWMsgTypeSpec* transmit_type;
+    /** @brief Marshal function */
+    LWMsgTypeMarshalFunction marshal;
+    /** @brief Unmarshal function */
+    LWMsgTypeUnmarshalFunction unmarshal;
+    /** @brief Destroy presented function */
+    LWMsgTypeDestroyPresentedFunction destroy_presented;
+    /** @brief Destroy transmitted function */
+    LWMsgTypeDestroyTransmittedFunction destroy_transmitted;
+    /** @brief Print callback function */
+    LWMsgTypePrintFunction print;
+} LWMsgTypeClass;
+
+/**
+ * @brief Custom data verification function
+ *
+ * A callback function which performs verification of
+ * in-memory data immediately before marshalling or immediately
+ * after unmarshalling.
+ *
+ * @param context the data context
+ * @param unmarshalling true when the operation being performed is unmarshalling
+ * @param object the object to verify
+ * @param data the user data pointer given to LWMSG_ATTR_VERIFY()
+ * @lwmsg_status
+ * @lwmsg_success
+ * @lwmsg_code{MALFORMED, the object did not pass verification}
+ * @lwmsg_endstatus
+ */
+typedef LWMsgStatus (*LWMsgVerifyFunction) (
+    struct LWMsgDataContext* context,
+    LWMsgBool unmarshalling,
+    void* object,
+    void *data
+    );
+
 #ifndef DOXYGEN
 typedef enum LWMsgTypeDirective
 {
@@ -372,6 +413,11 @@ typedef enum LWMsgTypeDirective
         LWMSG_CMD_VOID,
         LWMSG_CMD_ENCODING,
         LWMSG_CMD_SENSITIVE,
+        LWMSG_CMD_ALIASABLE,
+        LWMSG_CMD_ENUM,
+        LWMSG_CMD_ENUM_VALUE,
+        LWMSG_CMD_ENUM_MASK,
+        LWMSG_CMD_MAX_ALLOC,
         LWMSG_FLAG_MEMBER = 0x10000,
         LWMSG_FLAG_META = 0x20000,
         LWMSG_FLAG_DEBUG = 0x40000
@@ -382,14 +428,25 @@ typedef enum LWMsgTypeDirective
 #define _TYPECMD(_c_) ((size_t) ((_c_) | _TYPEFLAG_DEBUG)) _TYPEDEBUG
 #define _TYPECMD_TYPE(_c_, _t_) ((size_t) ((_c_) | _TYPEFLAG_META | _TYPEFLAG_DEBUG)) _TYPEMETA(_t_) _TYPEDEBUG
 #define _TYPECMD_MEMBER(_c_, _t_, _m_) ((size_t) ((_c_) | LWMSG_FLAG_MEMBER | _TYPEFLAG_META | _TYPEFLAG_DEBUG)) _TYPEMETA_MEMBER(_t_, _m_) _TYPEDEBUG
+#define _TYPECMD_NAMED(_c_, _n_) ((size_t) ((_c_) | _TYPEFLAG_META | _TYPEFLAG_DEBUG)) _TYPENAME(_n_) _TYPEDEBUG
+
+#ifndef LWMSG_SPEC_META
+#define LWMSG_SPEC_META
+#endif
+
+#ifdef LWMSG_OMIT_TYPESPEC_METADATA
+#  undef LWMSG_SPEC_META
+#endif
 
 #ifdef LWMSG_SPEC_META
 #define _TYPEFLAG_META (LWMSG_FLAG_META)
 #define _TYPEMETA(type) ,_TYPEARG(#type)
+#define _TYPENAME(name) ,_TYPEARG(name)
 #define _TYPEMETA_MEMBER(type, field) ,_TYPEARG(#field)
 #else
 #define _TYPEFLAG_META (0)
 #define _TYPEMETA(type)
+#define _TYPENAME(name)
 #define _TYPEMETA_MEMBER(type, field)
 #endif
 
@@ -406,12 +463,25 @@ typedef enum LWMsgTypeDirective
 /**
  * @brief Specify an empty type
  *
- * Specifies an empty (zero-length) type.  This is primarily useful
- * for indicating empty arms of a union.
+ * Specifies an empty (zero-length) type.
+ * @hideinitializer
  */
 #define LWMSG_VOID \
     _TYPECMD(LWMSG_CMD_VOID)
 
+/**
+ * @brief Specify an empty member
+ *
+ * Specifies an empty (zero-length) member.  This is primarily useful
+ * for indicating empty arms of a union.  The field name can be arbitrary.
+ *
+ * @param type the name of the containing type
+ * @param field the name of the member
+ * @hideinitializer
+ */
+#define LWMSG_MEMBER_VOID(type, field) \
+    _TYPECMD_MEMBER(LWMSG_CMD_VOID, type, field)
+
 /**
  * @brief Reference another type specification
  *
@@ -637,12 +707,31 @@ typedef enum LWMsgTypeDirective
         _TYPEARG(LWMSG_TERM_ZERO)
 
 /**
- * @brief Indicate string encoding
+ * @brief Indicate data encoding
  *
  * Indicates that the immediately previous array or pointer
- * represents text in the specified encoding.  This is used
- * as a hint to #lwmsg_data_print_graph() to aid in debugging
- * and does not affect the behavior of the marshaller.
+ * represents data in the specified encoding.  This is used
+ * as a hint by functions such as #lwmsg_data_print_graph_alloc()
+ * to show the data in a human-readable form, but has no effect
+ * on how the data is marshalled.  The encoding is specified as
+ * an all-uppercase string which should be a valid encoding name
+ * recognized by the system iconv_open() function.  Alternatively,
+ * it can be one of the following special lowercase strings
+ * recognized by <tt>lwmsg</tt>:
+ *
+ * - <tt>"ucs-2"</tt>: specifies UCS-2 encoding with native endianness.
+ *   The iconv-recognized "UCS-2" encoding name is inconsistent across different
+ *   implementations -- on some platforms it indicates native endianness,
+ *   while on others it is always big or little regardless of CPU architecture.
+ *   Use this alias instead for consistent cross-platform behavior.
+ * - <tt>"hex+ascii"</tt>: indicates that the data should be displayed in
+ *   side-by-side hexadecimal and ASCII form, like the <tt>hexdump -C</tt>
+ *   command available on many UNIX systems. Each 16-byte block is shown on its
+ *   own line with its 32-bit offset in hex, the hex value of each byte, and the
+ *   ASCII interpretation of each byte if it is a valid ASCII codepoint and not
+ *   a control character, or '<tt>.</tt>' otherwise.
+ *
+ * @param enc a string specifying the encoding
  * @hideinitializer
  */
 #define LWMSG_ATTR_ENCODING(enc)                \
@@ -654,12 +743,24 @@ typedef enum LWMsgTypeDirective
  *
  * Indicates that the immediately previous type or member
  * contains sensitive information (e.g. a password) and should
- * not be displayed when printed with lwmsg_data_print_graph().
+ * not be displayed when printed.
  * @hideinitializer
  */
 #define LWMSG_ATTR_SENSITIVE                    \
     _TYPECMD(LWMSG_CMD_SENSITIVE)
 
+/**
+ * @brief Indicate aliasable pointer
+ *
+ * Indicates that the immediately previous type or member,
+ * which must be a pointer or pointer-like type,
+ * is aliasable -- that is, multiple instances of the same
+ * pointer may appear in the data object graph.
+ * @hideinitializer
+ */
+#define LWMSG_ATTR_ALIASABLE                    \
+    _TYPECMD(LWMSG_CMD_ALIASABLE)
+
 /**
  * @brief Indicate C string
  *
@@ -672,6 +773,20 @@ typedef enum LWMsgTypeDirective
     LWMSG_ATTR_ZERO_TERMINATED,                 \
     LWMSG_ATTR_ENCODING("")
 
+/**
+ * @brief Limit memory allocation for pointer referent
+ *
+ * Indicates that the marshaller should refuse to allocate more
+ * than the given number of bytes when unmarshalling the referent
+ * of the immediately previous pointer type.  An attempt to exceed
+ * this value will result in an #LWMSG_STATUS_OVERFLOW error.
+ * @param max the maximum number of bytes to allocate
+ * @hideinitializer
+ */
+#define LWMSG_ATTR_MAX_ALLOC(max) \
+    _TYPECMD(LWMSG_CMD_MAX_ALLOC), \
+        _TYPEARG((max))
+
 /**
  * @brief Indicate union tag
  *
@@ -808,15 +923,17 @@ typedef enum LWMsgTypeDirective
  *
  * Defines a custom type with user-specified marshaller logic.
  *
+ * @param type the C type
  * @param tclass a constant pointer to the LWMsgCustomTypeClass structure
  * containing marshalling methods
  * @param tdata a constant pointer to arbitrary data which will be passed
  * to the marshalling methods
  * @hideinitializer
  */
-#define LWMSG_CUSTOM(tclass, tdata)             \
-    _TYPECMD(LWMSG_CMD_CUSTOM),                 \
-        _TYPEARG(tclass),                       \
+#define LWMSG_CUSTOM(type, tclass, tdata)        \
+    _TYPECMD_TYPE(LWMSG_CMD_CUSTOM, type),       \
+        _TYPEARG(sizeof(type)),                  \
+        _TYPEARG(tclass),                        \
         _TYPEARG(tdata)
 
 /**
@@ -855,6 +972,87 @@ typedef enum LWMsgTypeDirective
     _TYPECMD(LWMSG_CMD_CUSTOM_ATTR),            \
         _TYPEARG(value)
 
+/**
+ * @brief Begin enum definition
+ *
+ * Defines an enumerated integer type with the given
+ * underlying C type, width, and sign.  An enum value
+ * can consist of one of a set of scalar values (specified
+ * with #LWMSG_ENUM_VALUE()) and/or a bitwise-or'd
+ * set of mask values (specified with #LWMSG_ENUM_MASK())
+ *
+ * @param type the name of the C enum type
+ * @param width the width of the enum when marshalled, in bytes
+ * @param sign the sign of the enum when marshalled, an #LWMsgSignage value
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_BEGIN(type, width, sign)    \
+    _TYPECMD_TYPE(LWMSG_CMD_ENUM, type),       \
+        _TYPEARG(sizeof(type)),                \
+        _TYPEARG(width),                       \
+        _TYPEARG(sign)
+
+/**
+ * @brief End enum definition
+ *
+ * Ends an enum definition started by #LWMSG_ENUM_BEGIN()
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_END \
+    _TYPECMD(LWMSG_CMD_END)
+
+/**
+ * @brief Define enum scalar value
+ *
+ * Defines a possible scalar value of an enum type
+ *
+ * @param value the value
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_VALUE(value)                   \
+    _TYPECMD_TYPE(LWMSG_CMD_ENUM_VALUE, value),   \
+        _TYPEARG((value))
+
+/**
+ * @brief Define enum scalar value with explicit name
+ *
+ * Defines a possible scalar value of an enum type with
+ * an explicit name.
+ *
+ * @param name the name of the constant
+ * @param value the value
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_NAMED_VALUE(name, value)       \
+    _TYPECMD_NAMED(LWMSG_CMD_ENUM_VALUE, name),   \
+        _TYPEARG((value))
+
+/**
+ * @brief Define enum mask value
+ *
+ * Defines a potential bitmask value of an enum type
+ *
+ * @param value the value
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_MASK(value)                      \
+    _TYPECMD_TYPE(LWMSG_CMD_ENUM_MASK, value),      \
+        _TYPEARG((value))
+
+/**
+ * @brief Define enum mask value with explicit name
+ *
+ * Defines a potential bitmask value of an enum type with
+ * an explicit name.
+ *
+ * @param name the name of the constant
+ * @param value the value
+ * @hideinitializer
+ */
+#define LWMSG_ENUM_NAMED_MASK(name, value)          \
+    _TYPECMD_NAMED(LWMSG_CMD_ENUM_MASK, name),      \
+        _TYPEARG((value))
+
 /* Handy aliases for more complicated commands */
 
 /**
@@ -1249,6 +1447,55 @@ typedef enum LWMsgTypeDirective
     __VA_ARGS__,                               \
     LWMSG_ARRAY_END
 
+/**
+ * @brief Get type flags
+ *
+ * Gets the flags for the specified type.
+ *
+ * @param[in] type the type
+ * @return the flags for the type
+ */
+LWMsgTypeFlags
+lwmsg_type_get_flags(
+    const LWMsgType* type
+    );
+
+/**
+ * @brief Get custom type flags
+ *
+ * Gets custom flags for the specified type.
+ *
+ * @param[in] type the type
+ * @return custom flags for the type
+ */
+size_t
+lwmsg_type_get_custom_flags(
+    const LWMsgType* type
+    );
+
+/**
+ * @brief Get integer range constraint
+ *
+ * Gets the bounds of an integer type.
+ * The specified type must be an integer type
+ * and must have the #LWMSG_TYPE_FLAG_RANGE flag
+ * set.
+ *
+ * @param[in] type the type
+ * @param[out] low the lowest allowed value
+ * @param[out] high the highest allowed value
+ * @lwmsg_status
+ * @lwmsg_success
+ * @lwmsg_code{INVALID_PARAMETER, the type was not an integer or did not have a range set}
+ * @lwmsg_endstatus
+ */
+LWMsgStatus
+lwmsg_type_get_integer_range(
+    const LWMsgType* type,
+    size_t* low,
+    size_t* high
+    );
+
 /*@}*/
 
 #endif
