@@ -38,7 +38,8 @@
 #include "data-private.h"
 #include "util-private.h"
 #include "type-private.h"
-#include "convert.h"
+#include "buffer-private.h"
+#include "convert-private.h"
 #include "config.h"
 
 #include <string.h>
@@ -104,7 +105,7 @@ lwmsg_data_marshal_indirect_prologue(
     case LWMSG_TERM_ZERO:
         if (!object)
         {
-            if (iter->attrs.nonnull)
+            if (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL)
             {
                 BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
             }
@@ -179,16 +180,17 @@ lwmsg_data_marshal_indirect(
     /* Enforce nullability.  Even if the pointer is marked
        as non-null, allow it to be null if there are no
        elements to marshal */
-    if (iter->attrs.nonnull && !object && count != 0)
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL && !object && count != 0)
     {
         BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
     }
 
-    if (inner_iter.kind == LWMSG_KIND_INTEGER &&
+    if ((inner_iter.kind == LWMSG_KIND_INTEGER ||
+         inner_iter.kind == LWMSG_KIND_ENUM) &&
         inner_iter.info.kind_integer.width == 1 &&
         inner_iter.size == 1)
     {
-        /* As an optimization, if the element type if an 1-byte integer both
+        /* As an optimization, if the element type is a 1-byte integer both
            packed and unpacked, we can write the entire array directly into
            the output.  This is important for character strings */
         BAIL_ON_ERROR(status = lwmsg_buffer_write(
@@ -236,7 +238,69 @@ lwmsg_data_marshal_integer(
     in_size = iter->size;
 
     /* If a valid range is defined, check value against it */
-    if (iter->attrs.range_low < iter->attrs.range_high)
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_RANGE)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_verify_range(
+                          &context->error,
+                          iter,
+                          object,
+                          in_size));
+    }
+
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(in,
+                                                 in_size,
+                                                 LWMSG_NATIVE_ENDIAN,
+                                                 out,
+                                                 out_size,
+                                                 context->byte_order,
+                                                 iter->info.kind_integer.sign));
+
+    BAIL_ON_ERROR(status = lwmsg_buffer_write(buffer, out, out_size));
+
+
+error:
+
+    return status;
+}
+
+static LWMsgStatus
+lwmsg_data_marshal_enum(
+    LWMsgDataContext* context,
+    LWMsgMarshalState* state,
+    LWMsgTypeIter* iter,
+    unsigned char* object,
+    LWMsgBuffer* buffer)
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    unsigned char* in = object;
+    unsigned char out[MAX_INTEGER_SIZE];
+    size_t in_size;
+    size_t out_size;
+    uint64_t value = 0;
+    uint64_t mask = 0;
+    uint64_t res = 0;
+
+    out_size = iter->info.kind_integer.width;
+    in_size = iter->size;
+
+    /* Make sure we can decode the value */
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(
+                      in,
+                      in_size,
+                      LWMSG_NATIVE_ENDIAN,
+                      &value,
+                      sizeof(value),
+                      LWMSG_NATIVE_ENDIAN,
+                      iter->info.kind_integer.sign));
+
+    BAIL_ON_ERROR(status = lwmsg_data_decode_enum_value(
+                      iter,
+                      value,
+                      &mask,
+                      &res));
+
+    /* If a valid range is defined, check value against it */
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_RANGE)
     {
         BAIL_ON_ERROR(status = lwmsg_data_verify_range(
                           &context->error,
@@ -270,33 +334,70 @@ lwmsg_data_marshal_custom(
     LWMsgBuffer* buffer)
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    void* transmit_object = NULL;
+    LWMsgTypeClass* typeclass = iter->info.kind_custom.typeclass;
+    LWMsgTypeIter transmit_iter;
+    LWMsgMarshalState my_state = {NULL, state->map};
 
+    lwmsg_type_iterate(typeclass->transmit_type, &transmit_iter);
+
+    /* Allocate space for the transmitted object */
+    BAIL_ON_ERROR(status = lwmsg_data_alloc_memory(context, transmit_iter.size, &transmit_object));
+
+    /* Convert presented object into transmitted object */
     BAIL_ON_ERROR(status = iter->info.kind_custom.typeclass->marshal(
                       context,
-                      iter->size,
-                      object,
                       &iter->attrs,
-                      buffer,
+                      object,
+                      transmit_object,
                       iter->info.kind_custom.typedata));
 
+    /* Marshal transmitted object */
+    BAIL_ON_ERROR(status = lwmsg_data_marshal_internal(
+                      context,
+                      &my_state,
+                      &transmit_iter,
+                      transmit_object,
+                      buffer));
+
+
 error:
 
+    if (transmit_object)
+    {
+        /* Although we have the type spec for the transmitted object,
+           we do not call lwmsg_data_free_graph_internal() to clean it up
+           because we did not build it ourselves.  Instead, we
+           call the destroy_transmitted function of the type class. */
+        if (iter->info.kind_custom.typeclass->destroy_transmitted)
+        {
+            iter->info.kind_custom.typeclass->destroy_transmitted(
+                context,
+                &transmit_iter.attrs,
+                transmit_object,
+                iter->info.kind_custom.typedata);
+        }
+
+        /* Free object itself */
+        lwmsg_data_free_memory(context, transmit_object);
+    }
+
     return status;
 }
 
 static LWMsgStatus
 lwmsg_data_marshal_struct_member(
     LWMsgDataContext* context,
+    LWMsgMarshalState* state,
     LWMsgTypeIter* struct_iter,
     LWMsgTypeIter* member_iter,
     unsigned char* object,
-    LWMsgBuffer* buffer)
+    LWMsgBuffer* buffer
+    )
 {
-    LWMsgMarshalState my_state;
+    LWMsgMarshalState my_state = {object, state->map};
     unsigned char* member_object = object + member_iter->offset;
 
-    my_state.dominating_object = object;
-
     return lwmsg_data_marshal_internal(
         context,
         &my_state,
@@ -306,7 +407,13 @@ lwmsg_data_marshal_struct_member(
 }
 
 static LWMsgStatus
-lwmsg_data_marshal_struct(LWMsgDataContext* context, LWMsgTypeIter* iter, unsigned char* object, LWMsgBuffer* buffer)
+lwmsg_data_marshal_struct(
+    LWMsgDataContext* context,
+    LWMsgMarshalState* state,
+    LWMsgTypeIter* iter,
+    unsigned char* object,
+    LWMsgBuffer* buffer
+    )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     LWMsgTypeIter member;
@@ -319,6 +426,7 @@ lwmsg_data_marshal_struct(LWMsgDataContext* context, LWMsgTypeIter* iter, unsign
     {
         BAIL_ON_ERROR(status = lwmsg_data_marshal_struct_member(
                           context,
+                          state,
                           iter,
                           &member,
                           object,
@@ -356,12 +464,81 @@ error:
 }
 
 static LWMsgStatus
-lwmsg_data_marshal_pointer(
+lwmsg_data_marshal_aliasable_pointer(
     LWMsgDataContext* context,
     LWMsgMarshalState* state,
     LWMsgTypeIter* iter,
     unsigned char* object,
-    LWMsgBuffer* buffer)
+    LWMsgBuffer* buffer
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgObjectID id = 0;
+    unsigned char id_rep[4];
+    LWMsgBool write_pointee = LWMSG_FALSE;
+
+    if (*(void**) object)
+    {
+        status = lwmsg_data_object_map_find_object(
+            state->map,
+            *(void**) object,
+            &id);
+
+        if (status == LWMSG_STATUS_NOT_FOUND)
+        {
+            /* Not previous marshalled.  Add to object map and write pointee */
+            write_pointee = LWMSG_TRUE;
+
+            BAIL_ON_ERROR(status = lwmsg_data_object_map_insert(
+                              state->map,
+                              *(void**) object,
+                              iter,
+                              &id));
+        }
+        else
+        {
+            BAIL_ON_ERROR(status);
+        }
+    }
+    else if (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
+    }
+
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(
+                      &id,
+                      sizeof(id),
+                      LWMSG_NATIVE_ENDIAN,
+                      id_rep,
+                      sizeof(id_rep),
+                      context->byte_order,
+                      LWMSG_UNSIGNED));
+
+    BAIL_ON_ERROR(status = lwmsg_buffer_write(buffer, id_rep, sizeof(id_rep)));
+
+    if (write_pointee)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_marshal_indirect(
+                          context,
+                          state,
+                          iter,
+                          *(unsigned char**) object,
+                          buffer));
+    }
+
+error:
+
+    return status;
+}
+
+static LWMsgStatus
+lwmsg_data_marshal_unaliasable_pointer(
+    LWMsgDataContext* context,
+    LWMsgMarshalState* state,
+    LWMsgTypeIter* iter,
+    unsigned char* object,
+    LWMsgBuffer* buffer
+    )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     unsigned char ptr_flag;
@@ -370,12 +547,12 @@ lwmsg_data_marshal_pointer(
     ptr_flag = *(void**) object ? 0xFF : 0x00;
 
     /* Only write pointer flag for nullable pointers */
-    if (!iter->attrs.nonnull)
+    if (!(iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL))
     {
         BAIL_ON_ERROR(status = lwmsg_buffer_write(buffer, &ptr_flag, 1));
     }
 
-    if (ptr_flag || iter->attrs.nonnull)
+    if (ptr_flag || (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL))
     {
         /* Pointer is present, so also write pointee */
         BAIL_ON_ERROR(status = lwmsg_data_marshal_indirect(
@@ -391,6 +568,41 @@ error:
     return status;
 }
 
+static LWMsgStatus
+lwmsg_data_marshal_pointer(
+    LWMsgDataContext* context,
+    LWMsgMarshalState* state,
+    LWMsgTypeIter* iter,
+    unsigned char* object,
+    LWMsgBuffer* buffer
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_marshal_aliasable_pointer(
+                          context,
+                          state,
+                          iter,
+                          object,
+                          buffer));
+    }
+    else
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_marshal_unaliasable_pointer(
+                          context,
+                          state,
+                          iter,
+                          object,
+                          buffer));
+    }
+
+error:
+
+    return status;
+}
+
 static LWMsgStatus
 lwmsg_data_marshal_internal(
     LWMsgDataContext* context,
@@ -403,7 +615,7 @@ lwmsg_data_marshal_internal(
 
     if (iter->verify)
     {
-        BAIL_ON_ERROR(status = iter->verify(context, LWMSG_FALSE, iter->size, object, iter->verify_data));
+        BAIL_ON_ERROR(status = iter->verify(context, LWMSG_FALSE, object, iter->verify_data));
     }
 
     switch (iter->kind)
@@ -414,11 +626,14 @@ lwmsg_data_marshal_internal(
     case LWMSG_KIND_INTEGER:
         BAIL_ON_ERROR(status = lwmsg_data_marshal_integer(context, state, iter, object, buffer));
         break;
+    case LWMSG_KIND_ENUM:
+        BAIL_ON_ERROR(status = lwmsg_data_marshal_enum(context, state, iter, object, buffer));
+        break;
     case LWMSG_KIND_CUSTOM:
         BAIL_ON_ERROR(status = lwmsg_data_marshal_custom(context, state, iter, object, buffer));
         break;
     case LWMSG_KIND_STRUCT:
-        BAIL_ON_ERROR(status = lwmsg_data_marshal_struct(context, iter, object, buffer));
+        BAIL_ON_ERROR(status = lwmsg_data_marshal_struct(context, state, iter, object, buffer));
         break;
     case LWMSG_KIND_UNION:
         BAIL_ON_ERROR(status = lwmsg_data_marshal_union(context, state, iter, object, buffer));
@@ -448,9 +663,12 @@ LWMsgStatus
 lwmsg_data_marshal(LWMsgDataContext* context, LWMsgTypeSpec* type, void* object, LWMsgBuffer* buffer)
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    LWMsgMarshalState state = {NULL};
+    LWMsgObjectMap map;
+    LWMsgMarshalState state = {NULL, &map};
     LWMsgTypeIter iter;
 
+    memset(&map, 0, sizeof(map));
+
     lwmsg_type_iterate_promoted(type, &iter);
 
     BAIL_ON_ERROR(status = lwmsg_data_marshal_internal(context, &state, &iter, (unsigned char*) &object, buffer));
@@ -462,6 +680,8 @@ lwmsg_data_marshal(LWMsgDataContext* context, LWMsgTypeSpec* type, void* object,
 
 error:
 
+    lwmsg_data_object_map_destroy(&map);
+
     return status;
 }
 
