@@ -36,12 +36,13 @@
  *
  */
 
+#include "config.h"
 #include "data-private.h"
 #include "util-private.h"
 #include "type-private.h"
-#include "config.h"
-#include "convert.h"
+#include "convert-private.h"
 #include "context-private.h"
+#include "buffer-private.h"
 
 #include <string.h>
 
@@ -115,7 +116,7 @@ lwmsg_object_free(
 }
 
 static LWMsgStatus
-lwmsg_data_unmarshal_integer_immediate(
+lwmsg_data_unmarshal_integer(
     LWMsgDataContext* context,
     LWMsgUnmarshalState* state,
     LWMsgTypeIter* iter,
@@ -143,7 +144,7 @@ lwmsg_data_unmarshal_integer_immediate(
                       iter->info.kind_integer.sign));
 
     /* If a valid range is defined, check value against it */
-    if (iter->attrs.range_low < iter->attrs.range_high)
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_RANGE)
     {
         BAIL_ON_ERROR(status = lwmsg_data_verify_range(
                           &context->error,
@@ -158,7 +159,7 @@ error:
 }
 
 static LWMsgStatus
-lwmsg_data_unmarshal_custom_immediate(
+lwmsg_data_unmarshal_enum(
     LWMsgDataContext* context,
     LWMsgUnmarshalState* state,
     LWMsgTypeIter* iter,
@@ -167,23 +168,114 @@ lwmsg_data_unmarshal_custom_immediate(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    unsigned char temp[MAX_INTEGER_SIZE];
+    size_t in_size;
+    size_t out_size;
+    uint64_t value = 0;
+    uint64_t mask = 0;
+    uint64_t res = 0;
 
+    in_size = iter->info.kind_integer.width;
+    out_size = iter->size;
+
+    BAIL_ON_ERROR(status = lwmsg_buffer_read(buffer, temp, in_size));
+
+    /* Make sure we can decode the value */
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(
+                      temp,
+                      in_size,
+                      context->byte_order,
+                      &value,
+                      sizeof(value),
+                      LWMSG_NATIVE_ENDIAN,
+                      iter->info.kind_integer.sign));
+
+    BAIL_ON_ERROR(status = lwmsg_data_decode_enum_value(
+                      iter,
+                      value,
+                      &mask,
+                      &res));
+
+    /* Now convert it into its destination */
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(
+                      temp,
+                      in_size,
+                      context->byte_order,
+                      object,
+                      out_size,
+                      LWMSG_NATIVE_ENDIAN,
+                      iter->info.kind_integer.sign));
+
+    /* If a valid range is defined, check value against it */
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_RANGE)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_verify_range(
+                          &context->error,
+                          iter,
+                          object,
+                          out_size));
+    }
+
+error:
+
+    return status;
+}
+
+
+static LWMsgStatus
+lwmsg_data_unmarshal_custom(
+    LWMsgDataContext* context,
+    LWMsgUnmarshalState* state,
+    LWMsgTypeIter* iter,
+    LWMsgBuffer* buffer,
+    unsigned char* object
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgTypeClass* typeclass = iter->info.kind_custom.typeclass;
+    LWMsgTypeIter transmit_iter;
+    void* transmit_object = NULL;
+    LWMsgUnmarshalState my_state = {NULL, state->map};
+
+    lwmsg_type_iterate(typeclass->transmit_type, &transmit_iter);
+
+    /* Allocate memory for transmitted object */
+    BAIL_ON_ERROR(status = lwmsg_data_alloc_memory(context, transmit_iter.size, &transmit_object));
+
+    /* Unmarshal transmitted object */
+    BAIL_ON_ERROR(status = lwmsg_data_unmarshal_internal(
+                      context,
+                      &my_state,
+                      &transmit_iter,
+                      buffer,
+                      transmit_object));
+
+    /* Convert transmitted object into presented object */
     BAIL_ON_ERROR(status = iter->info.kind_custom.typeclass->unmarshal(
                       context,
-                      buffer,
-                      iter->size,
                       &iter->attrs,
+                      transmit_object,
                       object,
                       iter->info.kind_custom.typedata));
 
 error:
 
+    if (transmit_object)
+    {
+        /* Since we constructed the transmitted object ourselves, we do not call
+           the destroy_transmitted function of the type class */
+        lwmsg_data_free_graph_internal(context, &transmit_iter, transmit_object);
+        /* Free object itself */
+        lwmsg_data_free_memory(context, transmit_object);
+    }
+
     return status;
 }
 
 static LWMsgStatus
 lwmsg_data_unmarshal_struct_member(
     LWMsgDataContext* context,
+    LWMsgUnmarshalState* state,
     LWMsgTypeIter* struct_iter,
     LWMsgTypeIter* member_iter,
     LWMsgBuffer* buffer,
@@ -191,11 +283,9 @@ lwmsg_data_unmarshal_struct_member(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    LWMsgUnmarshalState my_state;
+    LWMsgUnmarshalState my_state = {struct_object, state->map};
     unsigned char* member_object = struct_object + member_iter->offset;
 
-    my_state.dominating_object = struct_object;
-
     BAIL_ON_ERROR(status = lwmsg_data_unmarshal_internal(
                       context,
                       &my_state,
@@ -269,7 +359,8 @@ lwmsg_data_unmarshal_indirect(
     size_t i;
     unsigned char* element = NULL;
 
-    if (inner->kind == LWMSG_KIND_INTEGER &&
+    if ((inner->kind == LWMSG_KIND_INTEGER ||
+         inner->kind == LWMSG_KIND_ENUM) &&
         inner->info.kind_integer.width == 1 &&
         inner->size == 1)
     {
@@ -314,6 +405,7 @@ lwmsg_data_unmarshal_pointees(
     size_t referent_size = 0;
     unsigned char* object = NULL;
     LWMsgTypeIter inner;
+    LWMsgObjectID id = 0;
 
     lwmsg_type_enter(iter, &inner);
 
@@ -326,7 +418,7 @@ lwmsg_data_unmarshal_pointees(
                       buffer,
                       &count));
 
-    /* Context a single struct pointee as a special case as it could
+    /* Treat a single struct pointee as a special case as it could
        hold a flexible array member */
     if (inner.kind == LWMSG_KIND_STRUCT && count == 1)
     {
@@ -354,9 +446,32 @@ lwmsg_data_unmarshal_pointees(
         /* Calculate the referent size */
         BAIL_ON_ERROR(status = lwmsg_multiply_unsigned(full_count, inner.size, &referent_size));
 
+        if (iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE &&
+            referent_size < sizeof(void*))
+        {
+            /* Make sure aliasable objects are always large enough to hold a pointer */
+            referent_size = sizeof(void*);
+        }
+
+        /* Enforce MAX_ALLOC attribute */
+        if (iter->attrs.max_alloc && referent_size > iter->attrs.max_alloc)
+        {
+            BAIL_ON_ERROR(status = LWMSG_STATUS_OVERFLOW);
+        }
+
         /* Allocate the referent */
         BAIL_ON_ERROR(status = lwmsg_object_alloc(context, referent_size, &object));
 
+        if (iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE)
+        {
+            /* If this is the referent of an aliasable pointer, insert it into the object map now */
+            BAIL_ON_ERROR(status = lwmsg_data_object_map_insert(
+                              state->map,
+                              object,
+                              iter,
+                              &id));
+        }
+
         /* Unmarshal elements */
         BAIL_ON_ERROR(status = lwmsg_data_unmarshal_indirect(
                           context,
@@ -387,17 +502,78 @@ error:
 }
 
 static LWMsgStatus
-lwmsg_data_unmarshal_pointer(
+lwmsg_data_unmarshal_aliasable_pointer(
     LWMsgDataContext* context,
     LWMsgUnmarshalState* state,
     LWMsgTypeIter* iter,
     LWMsgBuffer* buffer,
-    unsigned char** out)
+    unsigned char** out
+    )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    unsigned char ptr_flag;
+    unsigned char id_rep[4];
+    LWMsgObjectID id = 0;
+    void* object = NULL;
 
-    if (iter->attrs.nonnull)
+    /* Read the object id from the stream */
+    BAIL_ON_ERROR(status = lwmsg_buffer_read(buffer, id_rep, sizeof(id_rep)));
+
+    BAIL_ON_ERROR(status = lwmsg_convert_integer(
+                      id_rep,
+                      sizeof(id_rep),
+                      context->byte_order,
+                      &id,
+                      sizeof(id),
+                      LWMSG_NATIVE_ENDIAN,
+                      LWMSG_UNSIGNED));
+
+    if (id != 0)
+    {
+        status = lwmsg_data_object_map_find_id(
+            state->map,
+            id,
+            iter,
+            &object);
+    }
+    else if (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_MALFORMED);
+    }
+
+    if (status == LWMSG_STATUS_NOT_FOUND)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_pointees(
+                          context,
+                          state,
+                          iter,
+                          buffer,
+                          (unsigned char**) (void*) &object));
+    }
+    else
+    {
+        BAIL_ON_ERROR(status);
+    }
+
+    *(void**) out = object;
+
+error:
+
+    return status;
+}
+
+static LWMsgStatus
+lwmsg_data_unmarshal_unaliasable_pointer(
+    LWMsgDataContext* context,
+    LWMsgUnmarshalState* state,
+    LWMsgTypeIter* iter,
+    LWMsgBuffer* buffer,
+    unsigned char** out
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    unsigned char ptr_flag = 0;
+
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_NOT_NULL)
     {
         /* If pointer is never null, there is no flag in the stream */
         ptr_flag = 0xFF;
@@ -424,6 +600,41 @@ error:
     return status;
 }
 
+static LWMsgStatus
+lwmsg_data_unmarshal_pointer(
+    LWMsgDataContext* context,
+    LWMsgUnmarshalState* state,
+    LWMsgTypeIter* iter,
+    LWMsgBuffer* buffer,
+    unsigned char** out
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    if (iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE)
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_aliasable_pointer(
+                          context,
+                          state,
+                          iter,
+                          buffer,
+                          out));
+    }
+    else
+    {
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_unaliasable_pointer(
+                          context,
+                          state,
+                          iter,
+                          buffer,
+                          out));
+    }
+
+error:
+
+    return status;
+}
+
 static LWMsgStatus
 lwmsg_data_unmarshal_array(
     LWMsgDataContext* context,
@@ -465,6 +676,7 @@ error:
 static LWMsgStatus
 lwmsg_data_unmarshal_struct(
     LWMsgDataContext* context,
+    LWMsgUnmarshalState* state,
     LWMsgTypeIter* iter,
     LWMsgBuffer* buffer,
     unsigned char* object,
@@ -497,6 +709,7 @@ lwmsg_data_unmarshal_struct(
 
         BAIL_ON_ERROR(status = lwmsg_data_unmarshal_struct_member(
                           context,
+                          state,
                           iter,
                           &member,
                           buffer,
@@ -555,22 +768,48 @@ lwmsg_data_unmarshal_struct_pointee(
     unsigned char* base_object = NULL;
     unsigned char* full_object = NULL;
     LWMsgTypeSpec* flexible_member = NULL;
+    size_t base_size = 0;
     size_t full_size = 0;
+    LWMsgObjectID id = 0;
+
+    base_size = struct_iter->size;
+
+    if (pointer_iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE &&
+        base_size < sizeof(void*))
+    {
+        base_size = sizeof(void*);
+    }
+
+    /* Enforce MAX_ALLOC attribute */
+    if (pointer_iter->attrs.max_alloc && base_size > pointer_iter->attrs.max_alloc)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_OVERFLOW);
+    }
 
     /* Allocate enough memory to hold the base of the object */
     BAIL_ON_ERROR(status = lwmsg_object_alloc(
                       context,
-                      struct_iter->size,
+                      base_size,
                       &base_object));
 
+    if (pointer_iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE)
+    {
+        /* If this is the referent of an aliasable pointer, insert it into the object map now */
+        BAIL_ON_ERROR(status = lwmsg_data_object_map_insert(
+                          state->map,
+                          base_object,
+                          pointer_iter,
+                          &id));
+    }
+
     /* Unmarshal all base members of the structure and find any flexible member */
     BAIL_ON_ERROR(status = lwmsg_data_unmarshal_struct(
                       context,
+                      state,
                       struct_iter,
                       buffer,
                       base_object,
-                      &flexible_member
-                      ));
+                      &flexible_member));
 
     /* Now that the base of the object is unmarshalled, we can see if we need to
        reallocate space for a flexible member */
@@ -578,7 +817,7 @@ lwmsg_data_unmarshal_struct_pointee(
     {
         LWMsgTypeIter flex_iter;
         LWMsgTypeIter inner_iter;
-        LWMsgUnmarshalState my_state;
+        LWMsgUnmarshalState my_state = {base_object, state->map};
         size_t count = 0;
         size_t full_count = 0;
         size_t flexible_size = 0;
@@ -586,8 +825,6 @@ lwmsg_data_unmarshal_struct_pointee(
         lwmsg_type_iterate(flexible_member, &flex_iter);
         lwmsg_type_enter(&flex_iter, &inner_iter);
 
-        my_state.dominating_object = base_object;
-
         BAIL_ON_ERROR(status = lwmsg_data_unmarshal_indirect_prologue(
                           context,
                           &my_state,
@@ -613,6 +850,18 @@ lwmsg_data_unmarshal_struct_pointee(
         /* Calculate the size of the full structure */
         BAIL_ON_ERROR(status = lwmsg_add_unsigned(struct_iter->size, flexible_size, &full_size));
 
+        if (pointer_iter->attrs.flags & LWMSG_TYPE_FLAG_ALIASABLE &&
+            full_size < sizeof(void*))
+        {
+            full_size = sizeof(void*);
+        }
+
+        /* Enforce MAX_ALLOC attribute */
+        if (pointer_iter->attrs.max_alloc && full_size > pointer_iter->attrs.max_alloc)
+        {
+            BAIL_ON_ERROR(status = LWMSG_STATUS_OVERFLOW);
+        }
+
         /* Allocate the full object */
         BAIL_ON_ERROR(status = lwmsg_object_realloc(
                           context,
@@ -712,7 +961,15 @@ lwmsg_data_unmarshal_internal(
         /* Nothing to unmarshal */
         break;
     case LWMSG_KIND_INTEGER:
-        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_integer_immediate(
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_integer(
+                          context,
+                          state,
+                          iter,
+                          buffer,
+                          object));
+        break;
+    case LWMSG_KIND_ENUM:
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_enum(
                           context,
                           state,
                           iter,
@@ -720,7 +977,7 @@ lwmsg_data_unmarshal_internal(
                           object));
         break;
    case LWMSG_KIND_CUSTOM:
-        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_custom_immediate(
+        BAIL_ON_ERROR(status = lwmsg_data_unmarshal_custom(
                           context,
                           state,
                           iter,
@@ -730,6 +987,7 @@ lwmsg_data_unmarshal_internal(
     case LWMSG_KIND_STRUCT:
         BAIL_ON_ERROR(status = lwmsg_data_unmarshal_struct(
                           context,
+                          state,
                           iter,
                           buffer,
                           object,
@@ -765,7 +1023,7 @@ lwmsg_data_unmarshal_internal(
 
     if (iter->verify)
     {
-        BAIL_ON_ERROR(status = iter->verify(context, LWMSG_TRUE, iter->size, object, iter->verify_data));
+        BAIL_ON_ERROR(status = iter->verify(context, LWMSG_TRUE, object, iter->verify_data));
     }
 
 error:
@@ -778,9 +1036,12 @@ LWMsgStatus
 lwmsg_data_unmarshal(LWMsgDataContext* context, LWMsgTypeSpec* type, LWMsgBuffer* buffer, void** out)
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    LWMsgUnmarshalState my_state = {NULL};
+    LWMsgObjectMap map;
+    LWMsgUnmarshalState my_state = {NULL, &map};
     LWMsgTypeIter iter;
 
+    memset(&map, 0, sizeof(map));
+
     lwmsg_type_iterate_promoted(type, &iter);
 
     BAIL_ON_ERROR(status = lwmsg_data_unmarshal_internal(context, &my_state, &iter, buffer, (unsigned char*) out));
@@ -792,17 +1053,56 @@ lwmsg_data_unmarshal(LWMsgDataContext* context, LWMsgTypeSpec* type, LWMsgBuffer
 
 error:
 
+    lwmsg_data_object_map_destroy(&map);
+
     return status;
 }
 
 LWMsgStatus
-lwmsg_data_unmarshal_flat(LWMsgDataContext* context, LWMsgTypeSpec* type, void* buffer, size_t length, void** out)
+lwmsg_data_unmarshal_into(
+    LWMsgDataContext* context,
+    LWMsgTypeSpec* type,
+    LWMsgBuffer* buffer,
+    void* object,
+    size_t size
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgObjectMap map;
+    LWMsgUnmarshalState my_state = {NULL, &map};
+    LWMsgTypeIter iter;
+
+    memset(&map, 0, sizeof(map));
+
+    lwmsg_type_iterate(type, &iter);
+
+    if (size < iter.size)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_BUFFER_TOO_SMALL);
+    }
+
+    BAIL_ON_ERROR(status = lwmsg_data_unmarshal_internal(context, &my_state, &iter, buffer, object));
+
+    if (buffer->wrap)
+    {
+        BAIL_ON_ERROR(status = buffer->wrap(buffer, 0));
+    }
+
+error:
+
+    lwmsg_data_object_map_destroy(&map);
+
+    return status;
+}
+
+LWMsgStatus
+lwmsg_data_unmarshal_flat(LWMsgDataContext* context, LWMsgTypeSpec* type, const void* buffer, size_t length, void** out)
 {
     LWMsgBuffer mbuf;
 
-    mbuf.base = buffer;
-    mbuf.cursor = buffer;
-    mbuf.end = buffer + length;
+    mbuf.base = (unsigned char*) buffer;
+    mbuf.cursor = mbuf.base;
+    mbuf.end = mbuf.base + length;
     mbuf.wrap = NULL;
 
     return lwmsg_data_unmarshal(context, type, &mbuf, out);
