@@ -319,14 +319,46 @@ lwmsg_peer_task_delete(
     PeerAssocTask* task
     )
 {
+    LWMsgHashIter iter = {0};
+    PeerCall* call = NULL;
     LWMsgRing* ring = NULL;
     LWMsgRing* next = NULL;
-    PeerCall* call = NULL;
 
-    for (ring = task->calls.next; ring != &task->calls; ring = next)
+    lwmsg_hash_iter_begin(&task->incoming_calls, &iter);
+    while ((call = lwmsg_hash_iter_next(&task->incoming_calls, &iter)))
+    {
+        lwmsg_hash_remove_entry(&task->incoming_calls, call);
+        lwmsg_peer_call_delete(call);
+    }
+    lwmsg_hash_iter_end(&task->incoming_calls, &iter);
+
+    lwmsg_hash_destroy(&task->incoming_calls);
+
+    lwmsg_hash_iter_begin(&task->outgoing_calls, &iter);
+    while ((call = lwmsg_hash_iter_next(&task->outgoing_calls, &iter)))
+    {
+        lwmsg_hash_remove_entry(&task->outgoing_calls, call);
+        lwmsg_peer_call_delete(call);
+    }
+    lwmsg_hash_iter_end(&task->outgoing_calls, &iter);
+
+    lwmsg_hash_destroy(&task->outgoing_calls);
+
+    for (ring = task->active_incoming_calls.next;
+         ring != &task->active_incoming_calls;
+         ring = next)
     {
         next = ring->next;
-        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, ring);
+        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, queue_ring);
+        lwmsg_peer_call_delete(call);
+    }
+
+    for (ring = task->active_outgoing_calls.next;
+         ring != &task->active_outgoing_calls;
+         ring = next)
+    {
+        next = ring->next;
+        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, queue_ring);
         lwmsg_peer_call_delete(call);
     }
 
@@ -390,10 +422,37 @@ lwmsg_peer_task_ref(
     lwmsg_peer_unlock(task->peer);
 }
 
-LWMsgStatus
+static
+void*
+lwmsg_peer_call_get_key(
+    const void* entry
+    )
+{
+    return &((PeerCall*) entry)->cookie;
+}
+
+static
+size_t
+lwmsg_peer_call_digest(
+    const void* key
+    )
+{
+    return (size_t) *(LWMsgCookie*) key;
+}
+
+static
+LWMsgBool
+lwmsg_peer_call_equal(
+    const void* key1,
+    const void* key2
+    )
+{
+    return *((const LWMsgCookie*) key1) == *((const LWMsgCookie*) key2);
+}
+
+static LWMsgStatus
 lwmsg_peer_assoc_task_new(
     LWMsgPeer* peer,
-    LWMsgTaskGroup* group,
     PeerAssocTaskType type,
     PeerAssocTask** task
     )
@@ -415,18 +474,29 @@ lwmsg_peer_assoc_task_new(
     BAIL_ON_ERROR(status = lwmsg_error_map_errno(pthread_cond_init(&my_task->call_event, NULL)));
     free_cond = LWMSG_TRUE;
 
-    BAIL_ON_ERROR(status = lwmsg_task_new(
-                      peer->task_manager,
-                      group,
-                      lwmsg_peer_task_run,
-                      my_task,
-                      &my_task->event_task));
-
     my_task->peer = peer;
     my_task->type = type;
     my_task->refcount = 2;
 
-    lwmsg_ring_init(&my_task->calls);
+    lwmsg_ring_init(&my_task->active_incoming_calls);
+    lwmsg_ring_init(&my_task->active_outgoing_calls);
+
+    BAIL_ON_ERROR(status = lwmsg_hash_init(
+                      &my_task->incoming_calls,
+                      31,
+                      lwmsg_peer_call_get_key,
+                      lwmsg_peer_call_digest,
+                      lwmsg_peer_call_equal,
+                      offsetof(PeerCall, hash_ring)));
+
+    BAIL_ON_ERROR(status = lwmsg_hash_init(
+                      &my_task->outgoing_calls,
+                      31,
+                      lwmsg_peer_call_get_key,
+                      lwmsg_peer_call_digest,
+                      lwmsg_peer_call_equal,
+                      offsetof(PeerCall, hash_ring)));
+
     lwmsg_message_init(&my_task->incoming_message);
     lwmsg_message_init(&my_task->outgoing_message);
 
@@ -632,36 +702,75 @@ lwmsg_peer_assoc_task_new_accept(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerAssocTask* my_task = NULL;
 
-    BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, peer->listen_tasks, PEER_TASK_BEGIN_ACCEPT, &my_task));
+    BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, PEER_TASK_BEGIN_ACCEPT, &my_task));
 
     my_task->assoc = assoc;
 
+    status = lwmsg_task_new(
+        peer->task_manager,
+        peer->listen_tasks,
+        lwmsg_peer_task_run,
+        my_task,
+        &my_task->event_task);
+    if (status != LWMSG_STATUS_SUCCESS)
+    {
+        my_task->assoc = NULL;
+    }
+    BAIL_ON_ERROR(status);
+
     *task = my_task;
 
+done:
+
+    return status;
+
 error:
 
-    return status;
+    if (my_task)
+    {
+        lwmsg_peer_task_delete(my_task);
+    }
+
+    goto done;
 }
 
 LWMsgStatus
 lwmsg_peer_assoc_task_new_connect(
     LWMsgPeer* peer,
     LWMsgAssoc* assoc,
+    LWMsgSession* session,
     PeerAssocTask** task
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerAssocTask* my_task = NULL;
 
-    BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, peer->connect_tasks, PEER_TASK_BEGIN_CONNECT, &my_task));
+    BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new(peer, PEER_TASK_BEGIN_CONNECT, &my_task));
 
     my_task->assoc = assoc;
+    my_task->session = session;
 
+
+    BAIL_ON_ERROR(status = lwmsg_task_new(
+                          peer->task_manager,
+                          peer->connect_tasks,
+                          lwmsg_peer_task_run,
+                          my_task,
+                          &my_task->event_task));
     *task = my_task;
 
+done:
+
+    return status;
+
 error:
 
-    return status;
+    if (my_task)
+    {
+        lwmsg_peer_task_delete(my_task);
+    }
+
+    goto done;
 }
 
 static
@@ -722,7 +831,8 @@ lwmsg_peer_task_subject_to_timeout(
         handle_count = lwmsg_session_get_handle_count(task->session);
         num_clients = lwmsg_peer_get_num_clients(peer);
 
-        return handle_count == 0 && num_clients == peer->max_clients && lwmsg_ring_is_empty(&task->calls);
+        return handle_count == 0 && num_clients == peer->max_clients &&
+            lwmsg_hash_get_count(&task->incoming_calls) == 0;
     }
     else
     {
@@ -831,12 +941,11 @@ lwmsg_peer_task_run_listen(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     PeerListenTask* task = data;
-    struct sockaddr addr;
-    socklen_t addrlen = sizeof(addr);
     int client_fd = -1;
     LWMsgAssoc* assoc = NULL;
     PeerAssocTask* client_task = NULL;
     LWMsgBool slot = LWMSG_FALSE;
+    int err = 0;
 
     while (status == LWMSG_STATUS_SUCCESS)
     {
@@ -848,37 +957,44 @@ lwmsg_peer_task_run_listen(
         if (trigger & LWMSG_TASK_TRIGGER_CANCEL)
         {
             lwmsg_peer_listen_task_delete_self(task);
-            BAIL_ON_ERROR(status = LWMSG_STATUS_CANCELLED);
+            *next_trigger = 0;
+            goto done;
         }
 
         if ((slot = lwmsg_peer_acquire_client_slot(task->peer)))
         {
-            client_fd = accept(task->fd, &addr, &addrlen);
+            do
+            {
+                client_fd = accept(task->fd, NULL, NULL);
 
-            if (client_fd < 0)
-            {
-            switch (errno)
-            {
-            case EAGAIN:
-            case EINTR:
-                *next_trigger = LWMSG_TASK_TRIGGER_FD_READABLE;
-                BAIL_ON_ERROR(status = LWMSG_STATUS_PENDING);
-            default:
-                BAIL_ON_ERROR(status = lwmsg_error_map_errno(errno));
-            }
-            }
+                if (client_fd < 0)
+                {
+                    switch (errno)
+                    {
+                    case EAGAIN:
+#if defined(EWOULDBLOCK) && (EWOULDBLOCK != EAGAIN)
+                    case EWOULDBLOCK:
+#endif
+                        /* We are blocked, wake up when fd is readable */
+                        *next_trigger = LWMSG_TASK_TRIGGER_FD_READABLE;
+                        goto done;
+                    case EINTR:
+                    case ECONNABORTED:
+                        /* Trivially retriable errors.  Ignore them and loop again */
+                        continue;
+                    default:
+                        err = errno;
+                        LWMSG_LOG_ERROR(task->peer->context, "System error on accept(): %i", err);
+                        BAIL_ON_ERROR(status = lwmsg_error_map_errno(err));
+                    }
+                }
+            } while (client_fd < 0);
 
             BAIL_ON_ERROR(status = lwmsg_set_close_on_exec(client_fd));
 
             /* Create new connection with client fd, put it into task, schedule task */
             BAIL_ON_ERROR(status = lwmsg_connection_new(task->peer->context, task->peer->protocol, &assoc));
             BAIL_ON_ERROR(status = lwmsg_connection_set_fd(assoc, LWMSG_CONNECTION_MODE_LOCAL, client_fd));
-            BAIL_ON_ERROR(status = lwmsg_assoc_set_session_manager(assoc, task->peer->session_manager));
-            BAIL_ON_ERROR(status = lwmsg_assoc_set_session_functions(
-                              assoc,
-                              task->peer->session_construct,
-                              task->peer->session_destruct,
-                              task->peer->session_construct_data));
             BAIL_ON_ERROR(status = lwmsg_assoc_set_nonblock(assoc, LWMSG_TRUE));
             BAIL_ON_ERROR(status = lwmsg_peer_assoc_task_new_accept(task->peer, assoc, &client_task));
             assoc = NULL;
@@ -891,10 +1007,10 @@ lwmsg_peer_task_run_listen(
         }
         else
         {
-            /* We've run out of client slots, so stop waiting for new ones
+            /* We've run out of client slots, so stop accepting clients
                until we are explicitly woken up */
             *next_trigger = LWMSG_TASK_TRIGGER_EXPLICIT;
-            BAIL_ON_ERROR(status = LWMSG_STATUS_PENDING);
+            goto done;
         }
     }
 
@@ -924,9 +1040,26 @@ error:
         lwmsg_peer_task_cancel_and_unref(client_task);
     }
 
-    if (status == LWMSG_STATUS_CANCELLED)
+    /* If the listen task aborts, the server will be left in a state
+       where it is running but cannot be contacted.  Invoke the
+       exception function set on the server to give the application
+       a chance to bail out if it wishes.  Also, attempt to keep running
+       if the error appears to be recoverable */
+    if (task->peer->except)
     {
+        task->peer->except(task->peer, status, task->peer->except_data);
+    }
+
+    switch (status)
+    {
+    case LWMSG_STATUS_BUSY:
+    case LWMSG_STATUS_MEMORY:
+    case LWMSG_STATUS_RESOURCE_LIMIT:
+        *next_trigger = LWMSG_TASK_TRIGGER_YIELD;
+        break;
+    default:
         *next_trigger = 0;
+        break;
     }
 
     goto done;
@@ -944,12 +1077,11 @@ lwmsg_peer_task_run_accept(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
-    status = lwmsg_assoc_establish(task->assoc);
+    status = lwmsg_assoc_accept(task->assoc, peer->session_manager, &task->session);
 
     switch (status)
     {
     case LWMSG_STATUS_SUCCESS:
-        BAIL_ON_ERROR(status = lwmsg_assoc_get_session(task->assoc, &task->session));
         task->type = PEER_TASK_DISPATCH;
         BAIL_ON_ERROR(status = lwmsg_peer_log_accept(peer, task->assoc));
         break;
@@ -986,13 +1118,12 @@ lwmsg_peer_task_run_connect(
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
 
-    status = lwmsg_assoc_establish(task->assoc);
+    status = lwmsg_assoc_connect(task->assoc, task->session);
 
     switch (status)
     {
     case LWMSG_STATUS_SUCCESS:
         task->type = PEER_TASK_DISPATCH;
-        BAIL_ON_ERROR(status = lwmsg_assoc_get_session(task->assoc, &task->session));
         BAIL_ON_ERROR(status = lwmsg_peer_log_connect(peer, task->assoc));
         lwmsg_peer_task_notify_status(task, LWMSG_STATUS_SUCCESS);
         break;
@@ -1028,48 +1159,49 @@ lwmsg_peer_task_rundown(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    LWMsgRing* ring = NULL;
-    LWMsgRing* next = NULL;
+    LWMsgHashIter iter = {0};
     PeerCall* call = NULL;
     LWMsgMessage cancel = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage message = LWMSG_MESSAGE_INITIALIZER;
 
     pthread_mutex_lock(&task->call_lock);
 
-    for (ring = task->calls.next; ring != &task->calls; ring = next)
+    lwmsg_hash_iter_begin(&task->incoming_calls, &iter);
+    while ((call = lwmsg_hash_iter_next(&task->incoming_calls, &iter)))
     {
-        next = ring->next;
-        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, ring);
-
-        if (!call->is_outgoing && call->state & PEER_CALL_COMPLETED)
+        if (call->state & PEER_CALL_COMPLETED)
         {
             message.tag = call->params.incoming.out.tag;
             message.data = call->params.incoming.out.data;
             lwmsg_assoc_destroy_message(task->assoc, &message);
+            lwmsg_hash_remove_entry(&task->incoming_calls, call);
             lwmsg_peer_call_delete(call);
         }
-        else if (!(call->state & PEER_CALL_COMPLETED))
+        else if (!(call->state & PEER_CALL_CANCELLED))
         {
-            if (call->is_outgoing)
-            {
-                if (task->status)
-                {
-                    cancel.status = task->status;
-                }
-                else
-                {
-                        cancel.status = LWMSG_STATUS_CANCELLED;
-                }
-                lwmsg_peer_call_complete_outgoing(call, &cancel);
-            }
-            else if (!(call->state & PEER_CALL_CANCELLED))
-            {
-                lwmsg_peer_call_cancel_incoming(call);
-            }
+            lwmsg_peer_call_cancel_incoming(call);
         }
     }
+    lwmsg_hash_iter_end(&task->incoming_calls, &iter);
 
-    if (!lwmsg_ring_is_empty(&task->calls))
+    lwmsg_hash_iter_begin(&task->outgoing_calls, &iter);
+    while ((call = lwmsg_hash_iter_next(&task->outgoing_calls, &iter)))
+    {
+        if (task->status)
+        {
+            cancel.status = task->status;
+        }
+        else
+        {
+            cancel.status = LWMSG_STATUS_CANCELLED;
+        }
+        lwmsg_hash_remove_entry(&task->outgoing_calls, call);
+        lwmsg_peer_call_complete_outgoing(call, &cancel);
+    }
+    lwmsg_hash_iter_end(&task->outgoing_calls, &iter);
+
+    if (lwmsg_hash_get_count(&task->incoming_calls) != 0 ||
+        lwmsg_hash_get_count(&task->outgoing_calls) != 0)
     {
         *next_trigger = LWMSG_TASK_TRIGGER_EXPLICIT;
         BAIL_ON_ERROR(status = LWMSG_STATUS_PENDING);
@@ -1143,19 +1275,14 @@ lwmsg_peer_task_cancel_call(
     LWMsgCookie cookie
     )
 {
-    LWMsgRing* ring = NULL;
     PeerCall* call = NULL;
 
-    for (ring = task->calls.next; ring != &task->calls; ring = ring->next)
-    {
-        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, ring);
+    call = lwmsg_hash_find_key(&task->incoming_calls, &cookie);
 
-        if (call->cookie == cookie && !call->is_outgoing)
-        {
-            lwmsg_peer_log_message(task->peer, task->assoc, &task->incoming_message, LWMSG_FALSE);
-            lwmsg_peer_call_cancel_incoming(call);
-            break;
-        }
+    if (call)
+    {
+        lwmsg_peer_log_message(task->peer, task->assoc, &task->incoming_message, LWMSG_FALSE);
+        lwmsg_peer_call_cancel_incoming(call);
     }
 }
 
@@ -1166,20 +1293,15 @@ lwmsg_peer_task_complete_call(
     LWMsgCookie cookie
     )
 {
-    LWMsgRing* ring = NULL;
     PeerCall* call = NULL;
 
-    for (ring = task->calls.next; ring != &task->calls; ring = ring->next)
-    {
-        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, ring);
+    call = lwmsg_hash_find_key(&task->outgoing_calls, &cookie);
 
-        if (call->cookie == cookie && call->is_outgoing)
-        {
-            lwmsg_ring_remove(&call->ring);
-            lwmsg_peer_log_message(task->peer, task->assoc, &task->incoming_message, LWMSG_TRUE);
-            lwmsg_peer_call_complete_outgoing(call, &task->incoming_message);
-            break;
-        }
+    if (call)
+    {
+        lwmsg_hash_remove_entry(&task->outgoing_calls, call);
+        lwmsg_peer_log_message(task->peer, task->assoc, &task->incoming_message, LWMSG_TRUE);
+        lwmsg_peer_call_complete_outgoing(call, &task->incoming_message);
     }
 }
 
@@ -1195,16 +1317,20 @@ lwmsg_peer_task_dispatch_incoming_message(
     LWMsgPeer* peer = task->peer;
     PeerCall* call = NULL;
 
+    /* If the incoming message is a reply to one of our calls,
+       find it and complete it */
     if (task->incoming_message.flags & LWMSG_MESSAGE_FLAG_REPLY)
     {
         lwmsg_peer_task_complete_call(task, task->incoming_message.cookie);
         goto error;
     }
+    /* If the incoming message is a control message... */
     else if (task->incoming_message.flags & LWMSG_MESSAGE_FLAG_CONTROL)
     {
         switch (task->incoming_message.status)
         {
         case LWMSG_STATUS_CANCELLED:
+            /* The caller cancelled a previous call, so find it and cancel it here as well */
             lwmsg_peer_task_cancel_call(task, task->incoming_message.cookie);
             lwmsg_assoc_destroy_message(task->assoc, &task->incoming_message);
             goto error;
@@ -1217,10 +1343,29 @@ lwmsg_peer_task_dispatch_incoming_message(
 
     lwmsg_peer_log_message(peer, task->assoc, &task->incoming_message, LWMSG_FALSE);
 
-    BAIL_ON_ERROR(status = lwmsg_peer_call_new(task, &call));
-    lwmsg_peer_task_ref(task);
-    lwmsg_ring_enqueue(&task->calls, &call->ring);
+    /* Check if this cookie is already in use */
+    if (lwmsg_hash_find_key(&task->incoming_calls, &task->incoming_message.cookie))
+    {
+        /* This association is toast */
+        status = lwmsg_peer_task_handle_assoc_error(
+            peer,
+            task,
+            LWMSG_STATUS_MALFORMED);
+        lwmsg_assoc_destroy_message(task->assoc, &task->incoming_message);
+        goto error;
+    }
 
+    /* Create a call structure to track call */
+    BAIL_ON_ERROR(status = lwmsg_peer_call_new(task, &call));
+    /* The call structure holds a reference to the task */
+    lwmsg_peer_task_ref(task);
+
+    /* Set cookie and insert into incoming call table */
+    call->cookie = task->incoming_message.cookie;
+    lwmsg_hash_insert_entry(&task->incoming_calls, call);
+
+    /* If the incoming message is synthetic for some bizarre reason,
+       just echo back the status code to the caller */
     if (task->incoming_message.flags & LWMSG_MESSAGE_FLAG_SYNTHETIC)
     {
         status = lwmsg_peer_task_handle_call_error(
@@ -1231,6 +1376,7 @@ lwmsg_peer_task_dispatch_incoming_message(
         goto error;
     }
 
+    /* Make sure the tag is within the bounds of the dispatch vector */
     if (tag < 0 || tag >= peer->dispatch.vector_length)
     {
         status = lwmsg_peer_task_handle_call_error(
@@ -1241,6 +1387,7 @@ lwmsg_peer_task_dispatch_incoming_message(
         goto error;
     }
 
+    /* Make sure the call is supported */
     spec = peer->dispatch.vector[tag];
     if (spec == NULL)
     {
@@ -1252,6 +1399,7 @@ lwmsg_peer_task_dispatch_incoming_message(
         goto error;
     }
 
+    /* Dispatch the call */
     status = lwmsg_peer_call_dispatch_incoming(
         call,
         spec,
@@ -1260,10 +1408,11 @@ lwmsg_peer_task_dispatch_incoming_message(
     switch (status)
     {
     case LWMSG_STATUS_SUCCESS:
-        lwmsg_assoc_destroy_message(task->assoc, &task->incoming_message);
+        /* Message data is now owned by the call parameters */
+        lwmsg_message_init(&task->incoming_message);
         break;
     case LWMSG_STATUS_PENDING:
-        /* The message data is now owned by the call */
+        /* Message data is now owned by the call parameters */
         lwmsg_message_init(&task->incoming_message);
         status = LWMSG_STATUS_SUCCESS;
         break;
@@ -1325,12 +1474,10 @@ lwmsg_peer_task_run_finish(
             switch (task->type)
             {
             case PEER_TASK_FINISH_ACCEPT:
-                BAIL_ON_ERROR(status = lwmsg_assoc_get_session(task->assoc, &task->session));
                 BAIL_ON_ERROR(status = lwmsg_peer_log_accept(peer, task->assoc));
                 task->type = PEER_TASK_DISPATCH;
                 break;
             case PEER_TASK_FINISH_CONNECT:
-                BAIL_ON_ERROR(status = lwmsg_assoc_get_session(task->assoc, &task->session));
                 BAIL_ON_ERROR(status = lwmsg_peer_log_connect(peer, task->assoc));
                 lwmsg_peer_task_notify_status(task, LWMSG_STATUS_SUCCESS);
                 task->type = PEER_TASK_DISPATCH;
@@ -1418,18 +1565,23 @@ lwmsg_peer_task_dispatch_calls(
     )
 {
     LWMsgStatus status = LWMSG_STATUS_PENDING;
-    LWMsgRing* ring = NULL;
-    LWMsgRing* next = NULL;
     PeerCall* call = NULL;
     LWMsgCookie cookie = 0;
+    LWMsgMessage message = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgRing* ring = NULL;
+    LWMsgRing* next = NULL;
 
-    for (ring = task->calls.next; !task->outgoing && ring != &task->calls; ring = next)
+    for (ring = task->active_outgoing_calls.next;
+         ring != &task->active_outgoing_calls;
+         ring = next)
     {
         next = ring->next;
-        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, ring);
+        lwmsg_ring_remove(ring);
 
-        /* Undispatched outgoing call -- send request */
-        if (call->is_outgoing && !call->state & PEER_CALL_DISPATCHED)
+        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, queue_ring);
+
+        /* Undispatched outgoing call -- send request*/
+        if (!(call->state & PEER_CALL_DISPATCHED))
         {
             call->state |= PEER_CALL_DISPATCHED;
 
@@ -1447,15 +1599,15 @@ lwmsg_peer_task_dispatch_calls(
                 break;
             case LWMSG_STATUS_PENDING:
                 task->outgoing = LWMSG_TRUE;
+                BAIL_ON_ERROR(status);
                 break;
             default:
                 BAIL_ON_ERROR(status);
             }
         }
         /* Cancelled outgoing call -- send cancel request */
-        else if (call->is_outgoing &&
-                 call->state & PEER_CALL_DISPATCHED &&
-                 call->state & PEER_CALL_CANCELLED &&
+        else if ((call->state & PEER_CALL_DISPATCHED) &&
+                 (call->state & PEER_CALL_CANCELLED) &&
                  call->status != LWMSG_STATUS_CANCELLED)
         {
             call->status = LWMSG_STATUS_CANCELLED;
@@ -1473,14 +1625,26 @@ lwmsg_peer_task_dispatch_calls(
                 break;
             case LWMSG_STATUS_PENDING:
                 task->outgoing = LWMSG_TRUE;
+                BAIL_ON_ERROR(status);
                 break;
             default:
                 BAIL_ON_ERROR(status);
             }
         }
+    }
+
+    for (ring = task->active_incoming_calls.next;
+         ring != &task->active_incoming_calls;
+         ring = next)
+    {
+        next = ring->next;
+        lwmsg_ring_remove(ring);
+
+        call = LWMSG_OBJECT_FROM_MEMBER(ring, PeerCall, queue_ring);
+
         /* Completed incoming call -- send reply */
-        else if (call->state & PEER_CALL_COMPLETED &&
-                 call->state & PEER_CALL_DISPATCHED)
+        if ((call->state & PEER_CALL_COMPLETED) &&
+            (call->state & PEER_CALL_DISPATCHED))
         {
             lwmsg_message_init(&task->outgoing_message);
             task->outgoing_message.flags = LWMSG_MESSAGE_FLAG_REPLY;
@@ -1490,7 +1654,14 @@ lwmsg_peer_task_dispatch_calls(
             task->outgoing_message.status = call->status;
 
             cookie = call->cookie;
-            lwmsg_ring_remove(&call->ring);
+
+            /* It's now safe to destroy the incoming call parameters */
+            message.tag = call->params.incoming.in.tag;
+            message.data = call->params.incoming.in.data;
+            lwmsg_assoc_destroy_message(task->assoc, &message);
+
+            /* Delete the call structure */
+            lwmsg_hash_remove_entry(&task->incoming_calls, call);
             lwmsg_peer_call_delete(call);
 
             do
@@ -1505,13 +1676,15 @@ lwmsg_peer_task_dispatch_calls(
                 case LWMSG_STATUS_PENDING:
                     task->outgoing = LWMSG_TRUE;
                     task->destroy_outgoing = LWMSG_TRUE;
+                    BAIL_ON_ERROR(status);
                     break;
                 case LWMSG_STATUS_MALFORMED:
                 case LWMSG_STATUS_INVALID_HANDLE:
                 case LWMSG_STATUS_OVERFLOW:
                 case LWMSG_STATUS_UNDERFLOW:
-                    /* Our reply itself was unsendable, so send a synthetic reply
-                       instead so the caller at least knows the call is complete */
+                    /* Our reply could not be sent because the dispatch function gave us
+                       bogus response parameters.  Send a synthetic reply instead so the
+                       caller at least knows the call is complete */
                     LWMSG_LOG_WARNING(task->peer->context,
                                       "(assoc:0x%lx >> %u) Response payload could not be sent: %s",
                                       LWMSG_POINTER_AS_ULONG(task->assoc),
