@@ -44,37 +44,42 @@
 #include <unistd.h>
 #include <string.h>
 #include <pthread.h>
+#include <limits.h>
+
+typedef struct SharedHandleKey
+{
+    LWMsgHandleType type;
+    LWMsgHandleID id;
+} SharedHandleKey;
 
 typedef struct SharedSession
 {
     LWMsgSession base;
-    /* Remote session identifier */
-    LWMsgSessionID rsmid;
+    /* Session identifier */
+    LWMsgSessionID id;
     /* Security token of session creator */
     LWMsgSecurityToken* sec_token;
     /* Reference count */
     size_t volatile refs;
-    /* Pointer to linked list of handles */
-    struct SharedHandle* volatile handles;
-    /* Number of handles */
-    size_t num_handles;
-    /* Links to other sessions in the manager */
-    struct SharedSession * volatile next, * volatile prev;
+    /* Hash of handles by ptr (local handles only) */
+    LWMsgHashTable handle_by_ptr;
+    /* Hash of handles by id */
+    LWMsgHashTable handle_by_id;
     /* Lock */
     pthread_mutex_t lock;
     /* Next handle ID */
     unsigned long volatile next_hid;
     /* User data pointer */
     void* data;
-    /* Destruct function */
-    LWMsgSessionDestructFunction destruct;
+    /* Link in hash table by rsmid */
+    LWMsgRing ring;
 } SharedSession;
 
 typedef struct SharedHandle
 {
-    /* Pointer to associated session */
-    SharedSession* session;
-    /* Validity vit */
+    /* Key */
+    SharedHandleKey key;
+    /* Validity bit */
     LWMsgBool volatile valid;
     /* Reference count */
     size_t volatile refs;
@@ -82,26 +87,127 @@ typedef struct SharedHandle
     const char* type;
     /* Handle pointer */
     void* pointer;
-    /* Handle locality */
-    LWMsgHandleType locality;
-    /* Handle id */
-    unsigned long hid;
     /* Handle cleanup function */
     void (*cleanup)(void*);
-    /* Links to other handles in the session */
-    struct SharedHandle* next, *prev;
+    /* Link in hash table by ptr */
+    LWMsgRing ptr_ring;
+    /* Link in hash table by id */
+    LWMsgRing id_ring;
 } SharedHandle;
 
 typedef struct SharedManager
 {
     LWMsgSessionManager base;
-    SharedSession* volatile sessions;
+    LWMsgHashTable sessions;
+    LWMsgSessionConstructFunction construct;
+    LWMsgSessionDestructFunction destruct;
+    void* construct_data;
     pthread_mutex_t lock;
 } SharedManager;
 
 #define SHARED_MANAGER(obj) ((SharedManager*) (obj))
 #define SHARED_SESSION(obj) ((SharedSession*) (obj))
 
+static
+void*
+shared_session_get_key(
+    const void* entry
+    )
+{
+    return &((SharedSession*) entry)->id.connect;
+}
+
+static
+size_t
+shared_session_digest(
+    const void* key
+    )
+{
+    const LWMsgSessionCookie* cookie = key;
+    size_t hash = 0;
+    int i = 0;
+
+    for (i = 0; i < sizeof(cookie->bytes); i++)
+    {
+        hash = hash * 31 + cookie->bytes[i];
+    }
+
+    return hash;
+}
+
+static
+LWMsgBool
+shared_session_equal(
+    const void* key1,
+    const void* key2
+    )
+{
+    return memcmp(key1, key2, sizeof(LWMsgSessionCookie)) == 0;
+}
+
+static
+void*
+shared_handle_get_key_ptr(
+    const void* entry
+    )
+{
+    return ((SharedHandle*) entry)->pointer;
+}
+
+static
+size_t
+shared_handle_digest_ptr(
+    const void* key
+    )
+{
+    return (size_t) key;
+}
+
+static
+LWMsgBool
+shared_handle_equal_ptr(
+    const void* key1,
+    const void* key2
+    )
+{
+    return key1 == key2;
+}
+
+static
+void*
+shared_handle_get_key_id(
+    const void* entry
+    )
+{
+    return &((SharedHandle*) entry)->key;
+}
+
+static
+size_t
+shared_handle_digest_id(
+    const void* key
+    )
+{
+    const SharedHandleKey* hkey = key;
+
+    return hkey->type == LWMSG_HANDLE_LOCAL ? hkey->id : ~hkey->id;
+}
+
+static
+LWMsgBool
+shared_handle_equal_id(
+    const void* key1,
+    const void* key2
+    )
+{
+    const SharedHandleKey* hkey1 = key1;
+    const SharedHandleKey* hkey2 = key2;
+
+    return
+        hkey1->type == hkey2->type &&
+        hkey1->id == hkey2->id;
+}
+
 static inline
 void
 shared_lock(
@@ -140,24 +246,18 @@ session_unlock(
 
 static void
 shared_free_handle(
+    SharedSession* session,
     SharedHandle* entry
     )
 {
+    lwmsg_hash_remove_entry(&session->handle_by_ptr, entry);
+    lwmsg_hash_remove_entry(&session->handle_by_id, entry);
+
     if (entry->cleanup)
     {
         entry->cleanup(entry->pointer);
     }
 
-    if (entry->prev)
-    {
-        entry->prev->next = entry->next;
-    }
-    
-    if (entry->next)
-    {
-        entry->next->prev = entry->prev;
-    }
-
     free(entry);
 }
 
@@ -177,31 +277,25 @@ shared_add_handle(
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-    handle = calloc(1, sizeof(*handle));
-
-    if (!handle)
-    {
-        BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
-    }
+    BAIL_ON_ERROR(status = LWMSG_ALLOC(&handle));
 
     handle->type = type;
     handle->valid = LWMSG_TRUE;
     handle->refs = 1;
     handle->pointer = pointer ? pointer : handle;
     handle->cleanup = cleanup;
-    handle->hid = hid;
-    handle->locality = locality;
+    handle->key.id = hid;
+    handle->key.type = locality;
 
-    handle->next = my_session->handles;
+    lwmsg_ring_init(&handle->ptr_ring);
+    lwmsg_ring_init(&handle->id_ring);
 
-    if (my_session->handles)
+    if (pointer)
     {
-        my_session->handles->prev = handle;
+        lwmsg_hash_insert_entry(&my_session->handle_by_ptr, handle);
     }
 
-    my_session->handles = handle;
-
-    my_session->num_handles++;
+    lwmsg_hash_insert_entry(&my_session->handle_by_id, handle);
 
     *out_handle = handle;
 
@@ -214,39 +308,37 @@ static
 SharedSession*
 shared_find_session(
     SharedManager* priv,
-    const LWMsgSessionID* rsmid
+    const LWMsgSessionCookie* cookie
     )
 {
-    SharedSession* entry = NULL;
-
-    for (entry = priv->sessions; entry; entry = entry->next)
-    {
-        if (!memcmp(rsmid->bytes, entry->rsmid.bytes, sizeof(rsmid->bytes)))
-        {
-            return entry;
-        }
-    }
-
-    return NULL;
+    return lwmsg_hash_find_key(&priv->sessions, cookie);
 }
 
 static
 void
 shared_free_session(
+    SharedManager* manager,
     SharedSession* session
     )
 {
-    SharedHandle* handle, *next;
+    SharedHandle* handle = NULL;
+    LWMsgHashIter iter = {0};
 
-    for (handle = session->handles; handle; handle = next)
+    lwmsg_hash_remove_entry(&manager->sessions, session);
+
+    lwmsg_hash_iter_begin(&session->handle_by_id, &iter);
+    while ((handle = lwmsg_hash_iter_next(&session->handle_by_id, &iter)))
     {
-        next = handle->next;
-        shared_free_handle(handle);
+        shared_free_handle(session, handle);
     }
+    lwmsg_hash_iter_end(&session->handle_by_id, &iter);
 
-    if (session->destruct && session->data)
+    lwmsg_hash_destroy(&session->handle_by_id);
+    lwmsg_hash_destroy(&session->handle_by_ptr);
+
+    if (manager->destruct && session->data)
     {
-        session->destruct(session->sec_token, session->data);
+        manager->destruct(session->sec_token, session->data);
     }
 
     if (session->sec_token)
@@ -258,6 +350,28 @@ shared_free_session(
     free(session);
 }
 
+static
+void
+shared_reset_session(
+    SharedManager* manager,
+    SharedSession* session
+    )
+{
+    SharedHandle* handle = NULL;
+    LWMsgHashIter iter = {0};
+
+    lwmsg_hash_iter_begin(&session->handle_by_id, &iter);
+    while ((handle = lwmsg_hash_iter_next(&session->handle_by_id, &iter)))
+    {
+        if (handle->key.type == LWMSG_HANDLE_REMOTE)
+        {
+            handle->valid = LWMSG_FALSE;
+            lwmsg_hash_remove_entry(&session->handle_by_id, handle);
+        }
+    }
+    lwmsg_hash_iter_end(&session->handle_by_id, &iter);
+}
+
 static
 void
 shared_delete(
@@ -265,14 +379,17 @@ shared_delete(
     )
 {
     SharedManager* priv = SHARED_MANAGER(manager);
-    SharedSession* entry, *next;
+    LWMsgHashIter iter = {0};
+    SharedSession* entry = NULL;
 
-    for (entry = priv->sessions; entry; entry = next)
+    lwmsg_hash_iter_begin(&priv->sessions, &iter);
+    while ((entry = lwmsg_hash_iter_next(&priv->sessions, &iter)))
     {
-        next = entry->next;
-
-        shared_free_session(entry);
+        shared_free_session(priv, entry);
     }
+    lwmsg_hash_iter_end(&priv->sessions, &iter);
+
+    lwmsg_hash_destroy(&priv->sessions);
 
     pthread_mutex_destroy(&priv->lock);
 
@@ -281,33 +398,92 @@ shared_delete(
 
 static
 LWMsgStatus
-shared_enter_session(
+shared_create(
     LWMsgSessionManager* manager,
-    const LWMsgSessionID* rsmid,
-    LWMsgSecurityToken* rtoken,
-    LWMsgSessionConstructFunction construct,
-    LWMsgSessionDestructFunction destruct,
-    void* construct_data,
     LWMsgSession** out_session
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     SharedManager* priv = SHARED_MANAGER(manager);
-    SharedSession* session;
+    SharedSession* session = NULL;
 
     shared_lock(priv);
 
-    session = shared_find_session(priv, rsmid);
-    
+    session = calloc(1, sizeof(*session));
+    if (!session)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_MEMORY);
+    }
+
+    session->base.manager = manager;
+
+    lwmsg_ring_init(&session->ring);
+
+    BAIL_ON_ERROR(status = lwmsg_hash_init(
+                      &session->handle_by_id,
+                      31,
+                      shared_handle_get_key_id,
+                      shared_handle_digest_id,
+                      shared_handle_equal_id,
+                      offsetof(SharedHandle, id_ring)));
+
+    BAIL_ON_ERROR(status = lwmsg_hash_init(
+                      &session->handle_by_ptr,
+                      31,
+                      shared_handle_get_key_ptr,
+                      shared_handle_digest_ptr,
+                      shared_handle_equal_ptr,
+                      offsetof(SharedHandle, ptr_ring)));
+
+    lwmsg_session_generate_cookie(&session->id.connect);
+
+    session->refs = 1;
+
+    *out_session = LWMSG_SESSION(session);
+
+done:
+
+    shared_unlock(priv);
+
+    return status;
+
+error:
+
     if (session)
     {
-        if (!session->sec_token || !lwmsg_security_token_can_access(session->sec_token, rtoken))
+        shared_free_session(priv, session);
+    }
+
+    goto done;
+}
+
+static
+LWMsgStatus
+shared_accept(
+    LWMsgSessionManager* manager,
+    const LWMsgSessionCookie* connect,
+    LWMsgSecurityToken* token,
+    LWMsgSession** out_session
+    )
+{
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    SharedManager* priv = SHARED_MANAGER(manager);
+    SharedSession* session = NULL;
+
+    shared_lock(priv);
+
+    session = shared_find_session(priv, connect);
+
+    if (session)
+    {
+        if (!session->sec_token || !lwmsg_security_token_can_access(session->sec_token, token))
         {
             session = NULL;
             BAIL_ON_ERROR(status = LWMSG_STATUS_SECURITY);
         }
 
         session->refs++;
+        lwmsg_security_token_delete(token);
     }
     else
     {
@@ -319,33 +495,40 @@ shared_enter_session(
 
         session->base.manager = manager;
 
-        BAIL_ON_ERROR(status = lwmsg_error_map_errno(pthread_mutex_init(&session->lock, NULL)));
+        lwmsg_ring_init(&session->ring);
 
-        memcpy(session->rsmid.bytes, rsmid->bytes, sizeof(rsmid->bytes));
+        BAIL_ON_ERROR(status = lwmsg_hash_init(
+                          &session->handle_by_id,
+                          31,
+                          shared_handle_get_key_id,
+                          shared_handle_digest_id,
+                          shared_handle_equal_id,
+                          offsetof(SharedHandle, id_ring)));
 
-        if (rtoken)
-        {
-            BAIL_ON_ERROR(status = lwmsg_security_token_copy(rtoken, &session->sec_token));
-        }
+        BAIL_ON_ERROR(status = lwmsg_hash_init(
+                          &session->handle_by_ptr,
+                          31,
+                          shared_handle_get_key_ptr,
+                          shared_handle_digest_ptr,
+                          shared_handle_equal_ptr,
+                          offsetof(SharedHandle, ptr_ring)));
+
+        memcpy(session->id.connect.bytes, connect->bytes, sizeof(connect->bytes));
+        lwmsg_session_generate_cookie(&session->id.accept);
 
         session->refs = 1;
-        session->destruct = destruct;
 
-        if (construct)
+        if (priv->construct)
         {
-            BAIL_ON_ERROR(status = construct(
-                              session->sec_token,
-                              construct_data,
+            BAIL_ON_ERROR(status = priv->construct(
+                              token,
+                              priv->construct_data,
                               &session->data));
         }
 
-        if (priv->sessions)
-        {
-            priv->sessions->prev = session;
-        }
-        session->next = priv->sessions;
-        priv->sessions = session;
+        session->sec_token = token;
 
+        lwmsg_hash_insert_entry(&priv->sessions, session);
     }
 
     *out_session = LWMSG_SESSION(session);
@@ -360,7 +543,7 @@ error:
 
     if (session)
     {
-        shared_free_session(session);
+        shared_free_session(priv, session);
     }
 
     goto done;
@@ -368,13 +551,64 @@ error:
 
 static
 LWMsgStatus
-shared_leave_session(
-    LWMsgSessionManager* manager,
-    LWMsgSession* session
+shared_connect(
+    LWMsgSession* session,
+    const LWMsgSessionCookie* accept,
+    LWMsgSecurityToken* token
     )
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
-    SharedManager* priv = SHARED_MANAGER(manager);
+    SharedSession* ssession = SHARED_SESSION(session);
+
+    session_lock(ssession);
+
+    if (ssession->sec_token)
+    {
+        /* Session has been connected before */
+        if (!lwmsg_security_token_can_access(ssession->sec_token, token))
+        {
+            BAIL_ON_ERROR(status = LWMSG_STATUS_SECURITY);
+        }
+
+        /* Has session state been lost? */
+        if (memcmp(&ssession->id.accept.bytes, accept->bytes, sizeof(accept->bytes)) &&
+            lwmsg_hash_get_count(&ssession->handle_by_id) > 0)
+        {
+            /* Reset the session */
+            shared_reset_session(SHARED_MANAGER(session->manager), ssession);
+        }
+
+        lwmsg_security_token_delete(token);
+        /* Update accept cookie if it changed */
+        memcpy(ssession->id.accept.bytes, accept->bytes, sizeof(accept->bytes));
+    }
+    else
+    {
+        /* Session has not been connected bfore */
+        ssession->sec_token = token;
+        memcpy(ssession->id.accept.bytes, accept->bytes, sizeof(accept->bytes));
+    }
+
+    ssession->refs++;
+
+done:
+
+    session_unlock(ssession);
+
+    return status;
+
+error:
+
+    goto done;
+}
+
+static
+void
+shared_release(
+    LWMsgSession* session
+    )
+{
+    SharedManager* priv = SHARED_MANAGER(session->manager);
     SharedSession* my_session = SHARED_SESSION(session);
 
     shared_lock(priv);
@@ -383,50 +617,15 @@ shared_leave_session(
 
     if (my_session->refs == 0)
     {
-        if (priv->sessions == my_session)
-        {
-            priv->sessions = priv->sessions->next;
-        }
-        
-        if (my_session->next)
-        {
-            my_session->next->prev = my_session->prev;
-        }
-        
-        if (my_session->prev)
-        {
-            my_session->prev->next = my_session->next;
-        }
-
-        shared_free_session(my_session);
+        shared_free_session(priv, my_session);
     }
 
     shared_unlock(priv);
-
-    return status;
-}
-
-static
-void
-shared_retain_session(
-    LWMsgSessionManager* manager,
-    LWMsgSession* session
-    )
-{
-    SharedManager* priv = SHARED_MANAGER(manager);
-    SharedSession* my_session = SHARED_SESSION(session);
-
-    shared_lock(priv);
-
-    my_session->refs++;
-
-    shared_unlock(priv);
 }
 
 static
 LWMsgSecurityToken*
-shared_get_session_peer_security_token (
-    LWMsgSessionManager* manager,
+shared_get_peer_security_token (
     LWMsgSession* session
     )
 {
@@ -435,11 +634,39 @@ shared_get_session_peer_security_token (
     return my_session->sec_token;
 }
 
+static
+SharedHandle*
+shared_find_handle_by_id(
+    SharedSession* session,
+    LWMsgHandleType type,
+    LWMsgHandleID id
+    )
+{
+    SharedHandleKey key = {0};
+
+    key.type = type;
+    key.id = id;
+
+    return lwmsg_hash_find_key(&session->handle_by_id, &key);
+}
+
+static
+SharedHandle*
+shared_find_handle_by_ptr(
+    SharedSession* session,
+    void* ptr
+    )
+{
+    SharedHandle* handle = lwmsg_hash_find_key(&session->handle_by_ptr, ptr);
+
+    /* If the pointer was not found in the table, it must be a proxy
+       that we can cast directly */
+    return handle ? handle : (SharedHandle*) ptr;
+}
 
 static
 LWMsgStatus
 shared_register_handle_remote(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     const char* type,
     LWMsgHandleID hid,
@@ -476,7 +703,6 @@ error:
 static
 LWMsgStatus
 shared_register_handle_local(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     const char* type,
     void* pointer,
@@ -487,9 +713,26 @@ shared_register_handle_local(
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
+    SharedHandleKey key = {0};
+
+    if (!pointer)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_PARAMETER);
+    }
 
     session_lock(my_session);
 
+    if (lwmsg_hash_get_count(&my_session->handle_by_id) == UINT32_MAX)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_RESOURCE_LIMIT);
+    }
+
+    do
+    {
+        key.type = LWMSG_HANDLE_LOCAL;
+        key.id = my_session->next_hid++;
+    } while (lwmsg_hash_find_key(&my_session->handle_by_id, &key));
+
     BAIL_ON_ERROR(status = shared_add_handle(
                       my_session,
                       type,
@@ -501,7 +744,7 @@ shared_register_handle_local(
 
     if (hid)
     {
-        *hid = handle->hid;
+        *hid = handle->key.id;
     }
 
 error:
@@ -514,7 +757,6 @@ error:
 static
 LWMsgStatus
 shared_retain_handle(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     void* ptr
     )
@@ -523,34 +765,26 @@ shared_retain_handle(
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-    session_lock(my_session);
-
-    for (handle = my_session->handles; handle; handle = handle->next)
+    if (!ptr)
     {
-        if (handle->pointer == ptr)
-        {
-            handle->refs++;
-            goto done;
-        }
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_PARAMETER);
     }
 
-    BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
+    session_lock(my_session);
 
-done:
+    handle = shared_find_handle_by_ptr(my_session, ptr);
+    handle->refs++;
 
     session_unlock(my_session);
 
-    return status;
-
 error:
 
-    goto done;
+    return status;
 }
 
 static
 LWMsgStatus
 shared_release_handle(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     void* ptr
     )
@@ -559,43 +793,29 @@ shared_release_handle(
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-    session_lock(my_session);
-
-    for (handle = my_session->handles; handle; handle = handle->next)
+    if (!ptr)
     {
-        if (handle->pointer == ptr)
-        {
-            if (--handle->refs == 0)
-            {
-                if (handle == my_session->handles)
-                {
-                    my_session->handles = my_session->handles->next;
-                }
-
-                shared_free_handle(handle);
-                my_session->num_handles--;
-            }
-            goto done;
-        }
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_PARAMETER);
     }
 
-    BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
+    session_lock(my_session);
 
-done:
+    handle = shared_find_handle_by_ptr(my_session, ptr);
+    if (--handle->refs == 0)
+    {
+        shared_free_handle(my_session, handle);
+    }
 
     session_unlock(my_session);
 
-    return status;
-
 error:
 
-    goto done;
+    return status;
 }
 
 static
 LWMsgStatus
 shared_unregister_handle(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     void* ptr
     )
@@ -604,34 +824,26 @@ shared_unregister_handle(
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-    session_lock(my_session);
-
-    for (handle = my_session->handles; handle; handle = handle->next)
+    if (!ptr)
     {
-        if (handle->pointer == ptr)
-        {
-            if (!handle->valid)
-            {
-                BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
-            }
-
-            handle->valid = LWMSG_FALSE;
-
-            if (--handle->refs == 0)
-            {
-                if (handle == my_session->handles)
-                {
-                    my_session->handles = my_session->handles->next;
-                }
-
-                shared_free_handle(handle);
-                my_session->num_handles--;
-            }
-            goto done;
-        }
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_PARAMETER);
     }
 
-    BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
+    session_lock(my_session);
+
+    handle = shared_find_handle_by_ptr(my_session, ptr);
+
+    if (!handle->valid)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
+    }
+
+    handle->valid = LWMSG_FALSE;
+
+    if (--handle->refs == 0)
+    {
+        shared_free_handle(my_session, handle);
+    }
 
 done:
 
@@ -647,7 +859,6 @@ error:
 static
 LWMsgStatus
 shared_handle_pointer_to_id(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     void* pointer,
     const char** type,
@@ -659,34 +870,32 @@ shared_handle_pointer_to_id(
     SharedHandle* handle = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-    session_lock(my_session);
-
-    for (handle = my_session->handles; handle; handle = handle->next)
+    if (!pointer)
     {
-        if (handle->pointer == pointer)
-        {
-            if (!handle->valid)
-            {
-                BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
-            }
-
-            if (type)
-            {
-                *type = handle->type;
-            }
-            if (htype)
-            {
-                *htype = handle->locality;
-            }
-            if (hid)
-            {
-                *hid = handle->hid;
-            }
-            goto done;
-        }
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
     }
 
-    BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
+    session_lock(my_session);
+
+    handle = shared_find_handle_by_ptr(my_session, pointer);
+
+    if (!handle->valid)
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
+    }
+
+    if (type)
+    {
+        *type = handle->type;
+    }
+    if (htype)
+    {
+        *htype = handle->key.type;
+    }
+    if (hid)
+    {
+        *hid = handle->key.id;
+    }
 
 done:
 
@@ -702,7 +911,6 @@ error:
 static
 LWMsgStatus
 shared_handle_id_to_pointer(
-    LWMsgSessionManager* manager,
     LWMsgSession* session,
     const char* type,
     LWMsgHandleType htype,
@@ -716,27 +924,20 @@ shared_handle_id_to_pointer(
 
     session_lock(my_session);
 
-    for (handle = my_session->handles; handle; handle = handle->next)
+    handle = shared_find_handle_by_id(my_session, htype, hid);
+
+    if (!handle)
     {
-        if (handle->hid == hid && handle->locality == htype)
-        {
-            if (!handle->valid)
-            {
-                BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
-            }
-
-            if (type && strcmp(type, handle->type))
-            {
-                BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
-            }
-
-            *pointer = handle->pointer;
-            handle->refs++;
-            goto done;
-        }
+        BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
     }
 
-    BAIL_ON_ERROR(status = LWMSG_STATUS_NOT_FOUND);
+    if (!handle->valid || (type && strcmp(type, handle->type)))
+    {
+        BAIL_ON_ERROR(status = LWMSG_STATUS_INVALID_HANDLE);
+    }
+
+    *pointer = handle->pointer;
+    handle->refs++;
 
 done:
 
@@ -751,15 +952,13 @@ error:
 
 static
 void*
-shared_get_session_data (
-    LWMsgSessionManager* manager,
+shared_get_data (
     LWMsgSession* session
     )
 {
     void* data = NULL;
     SharedSession* my_session = SHARED_SESSION(session);
 
-
     session_lock(my_session);
 
     data = my_session->data;
@@ -771,18 +970,16 @@ shared_get_session_data (
 
 static
 const LWMsgSessionID*
-shared_get_session_id(
-    LWMsgSessionManager* manager,
+shared_get_id(
     LWMsgSession* session
     )
 {
-    return &SHARED_SESSION(session)->rsmid;
+    return &SHARED_SESSION(session)->id;
 }
 
 static
 size_t
-shared_get_session_assoc_count(
-    LWMsgSessionManager* manager,
+shared_get_assoc_count(
     LWMsgSession* session
     )
 {
@@ -797,8 +994,7 @@ shared_get_session_assoc_count(
 
 static
 size_t
-shared_get_session_handle_count(
-    LWMsgSessionManager* manager,
+shared_get_handle_count(
     LWMsgSession* session
     )
 {
@@ -806,7 +1002,7 @@ shared_get_session_handle_count(
     SharedSession* my_session = SHARED_SESSION(session);
 
     session_lock(my_session);
-    handles = my_session->num_handles;
+    handles = lwmsg_hash_get_count(&my_session->handle_by_id);
     session_unlock(my_session);
     return handles;
 }
@@ -814,9 +1010,10 @@ shared_get_session_handle_count(
 static LWMsgSessionManagerClass shared_class = 
 {
     .delete = shared_delete,
-    .enter_session = shared_enter_session,
-    .leave_session = shared_leave_session,
-    .retain_session = shared_retain_session,
+    .create = shared_create,
+    .accept = shared_accept,
+    .connect = shared_connect,
+    .release = shared_release,
     .register_handle_local = shared_register_handle_local,
     .register_handle_remote = shared_register_handle_remote,
     .retain_handle = shared_retain_handle,
@@ -824,15 +1021,18 @@ static LWMsgSessionManagerClass shared_class =
     .unregister_handle = shared_unregister_handle,
     .handle_pointer_to_id = shared_handle_pointer_to_id,
     .handle_id_to_pointer = shared_handle_id_to_pointer,
-    .get_session_data = shared_get_session_data,
-    .get_session_id = shared_get_session_id,
-    .get_session_assoc_count = shared_get_session_assoc_count,
-    .get_session_handle_count = shared_get_session_handle_count,
-    .get_session_peer_security_token = shared_get_session_peer_security_token
+    .get_data = shared_get_data,
+    .get_id = shared_get_id,
+    .get_assoc_count = shared_get_assoc_count,
+    .get_handle_count = shared_get_handle_count,
+    .get_peer_security_token = shared_get_peer_security_token
 };
                                          
 LWMsgStatus
 lwmsg_shared_session_manager_new(
+    LWMsgSessionConstructFunction construct,
+    LWMsgSessionDestructFunction destruct,
+    void* construct_data,
     LWMsgSessionManager** manager
     )
 {
@@ -840,6 +1040,19 @@ lwmsg_shared_session_manager_new(
     SharedManager* my_manager = NULL;
 
     BAIL_ON_ERROR(status = LWMSG_ALLOC(&my_manager));
+
+    my_manager->construct = construct;
+    my_manager->destruct = destruct;
+    my_manager->construct_data = construct_data;
+
+    BAIL_ON_ERROR(status = lwmsg_hash_init(
+                      &my_manager->sessions,
+                      31,
+                      shared_session_get_key,
+                      shared_session_digest,
+                      shared_session_equal,
+                      offsetof(SharedSession, ring)));
+
     BAIL_ON_ERROR(status = lwmsg_session_manager_init(&my_manager->base, &shared_class));
 
     BAIL_ON_ERROR(status = lwmsg_error_map_errno(pthread_mutex_init(&my_manager->lock, NULL)));
