@@ -40,6 +40,7 @@
 #include <moonunit/moonunit.h>
 #include <config.h>
 #include <pthread.h>
+#include <sys/un.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -69,7 +70,7 @@ empty_sender(void* _assoc)
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_connect(assoc, NULL));
     
     request_msg.tag = EMPTY_REQUEST;
     request_msg.data = NULL;
@@ -91,7 +92,7 @@ empty_receiver(void* _assoc)
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_accept(assoc, NULL, NULL));
     
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &request_msg));
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, request_msg.tag, EMPTY_REQUEST);
@@ -219,7 +220,7 @@ foo_sender(void* _assoc)
     FooReply* reply;
     size_t i;
     
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_connect(assoc, NULL));
 
     request.size = 1024;
     request_msg.tag = FOO_REQUEST;
@@ -255,7 +256,7 @@ foo_receiver(void* _assoc)
     FooReply* reply;
     size_t i;
     
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_accept(assoc, NULL, NULL));
 
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &request_msg));
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, request_msg.tag, FOO_REQUEST);
@@ -408,7 +409,7 @@ MU_TEST(assoc, foo_send_recv_fragment)
 }
 
 
-MU_TEST(assoc, foo_send_timeout_connect)
+MU_TEST(assoc, foo_send_timeout_connect_pair)
 {
     int sockets[2];
     LWMsgAssoc* send_assoc = NULL;
@@ -435,7 +436,49 @@ MU_TEST(assoc, foo_send_timeout_connect)
     
     lwmsg_assoc_set_timeout(send_assoc, LWMSG_TIMEOUT_ESTABLISH, &time);
 
-    status = lwmsg_assoc_establish(send_assoc);
+    status = lwmsg_assoc_connect(send_assoc, NULL);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_TIMEOUT);
+
+    MU_TRY_ASSOC(send_assoc, lwmsg_assoc_close(send_assoc));
+    lwmsg_assoc_delete(send_assoc);
+}
+
+MU_TEST(assoc, foo_send_timeout_connect_endpoint)
+{
+    int sock;
+    LWMsgAssoc* send_assoc = NULL;
+    static LWMsgProtocol* foo_protocol = NULL;
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgTime time = {0, 200000};
+    struct sockaddr_un sockaddr;
+    const char* endpoint = "/tmp/.lwmsg_test_endpoint";
+
+    MU_ASSERT((sock = socket(AF_UNIX, SOCK_STREAM, 0)) >= 0);
+
+    sockaddr.sun_family = AF_UNIX;
+    strcpy(sockaddr.sun_path, endpoint);
+    unlink(sockaddr.sun_path);
+
+    MU_ASSERT(bind(sock, (struct sockaddr*) &sockaddr, sizeof(sockaddr)) == 0);
+
+    MU_ASSERT(listen(sock, 8) == 0);
+
+    MU_TRY(lwmsg_protocol_new(NULL, &foo_protocol));
+    MU_TRY_PROTOCOL(foo_protocol, lwmsg_protocol_add_protocol_spec(foo_protocol, FooProtocol_spec));
+
+    MU_TRY(lwmsg_connection_new(
+               NULL,
+               foo_protocol,
+               &send_assoc));
+
+    MU_TRY(lwmsg_connection_set_endpoint(
+               send_assoc,
+               LWMSG_CONNECTION_MODE_LOCAL,
+               endpoint));
+
+    lwmsg_assoc_set_timeout(send_assoc, LWMSG_TIMEOUT_ESTABLISH, &time);
+
+    status = lwmsg_assoc_connect(send_assoc, NULL);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_TIMEOUT);
 
     MU_TRY_ASSOC(send_assoc, lwmsg_assoc_close(send_assoc));
@@ -496,7 +539,7 @@ auth_sender(void* _assoc)
     request_msg.tag = AUTH_REQUEST;
     request_msg.data = &request;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_connect(assoc, NULL));
 
     MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message_transact(assoc, &request_msg, &reply_msg));
 
@@ -527,8 +570,9 @@ auth_receiver(void* _assoc)
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgSecurityToken* token;
+    LWMsgSession* session = NULL;
     
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_accept(assoc, NULL, &session));
 
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &request_msg));
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, request_msg.tag, AUTH_REQUEST);
@@ -536,7 +580,8 @@ auth_receiver(void* _assoc)
 
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, request->fortytwo, 42);
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_peer_security_token(assoc, &token));
+    token = lwmsg_session_get_peer_security_token(session);
+
     MU_ASSERT(token != NULL);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, lwmsg_security_token_get_type(token), "local");
     MU_TRY(lwmsg_local_token_get_eid(token, &reply.uid, &reply.gid));
@@ -737,13 +782,15 @@ handle_sender(void* _assoc)
     HandleDestroyReply* destroy_reply = NULL;
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgSession* session = NULL;
     
     create_request.save_string = (char*) "Hello, world!";
     create_request.save_integer = 42;
     request_msg.tag = HANDLE_CREATE_REQUEST;
     request_msg.data = &create_request;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_connect(assoc, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
 
     MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message_transact(assoc, &request_msg, &reply_msg));
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, reply_msg.tag, HANDLE_CREATE_REPLY);
@@ -787,7 +834,7 @@ handle_sender(void* _assoc)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, destroy_reply->status, 0);
     free(destroy_reply);
     
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_unregister_handle(assoc, handle));
+    MU_TRY_ASSOC(assoc, lwmsg_session_unregister_handle(session, handle));
 
     MU_TRY_ASSOC(assoc, lwmsg_assoc_close(assoc));
     lwmsg_assoc_delete(assoc);
@@ -795,7 +842,8 @@ handle_sender(void* _assoc)
     return NULL;
 }
 
-static LWMsgStatus
+static
+LWMsgStatus
 handle_create_srv(
     LWMsgAssoc* assoc,
     const LWMsgMessage* in,
@@ -805,17 +853,19 @@ handle_create_srv(
 {
     HandleCreateRequest* request = in->data;
     Handle* handle = NULL;
+    LWMsgSession* session = NULL;
 
     handle = malloc(sizeof(*handle));
     handle->saved_string = strdup(request->save_string);
     handle->saved_integer = request->save_integer;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "Handle", handle, free));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "Handle", handle, free));
 
     out->tag = HANDLE_CREATE_REPLY;
     out->data = handle;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_retain_handle(assoc, handle));
+    MU_TRY_ASSOC(assoc, lwmsg_session_retain_handle(session, handle));
     
     return LWMSG_STATUS_SUCCESS;
 }
@@ -830,13 +880,15 @@ handle_destroy_srv(
 {
     Handle* handle = in->data;
     HandleDestroyReply* reply = malloc(sizeof(*reply));
+    LWMsgSession* session = NULL;
 
     reply->status = 0;
 
     out->tag = HANDLE_DESTROY_REPLY;
     out->data = reply;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_unregister_handle(assoc, handle));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
+    MU_TRY_ASSOC(assoc, lwmsg_session_unregister_handle(session, handle));
 
     return LWMSG_STATUS_EOF;
 }
@@ -870,12 +922,16 @@ handle_get_srv(
     return LWMSG_STATUS_SUCCESS;
 }
 
-static LWMsgDispatchSpec handle_dispatch[] =
+static struct
 {
-    LWMSG_DISPATCH(HANDLE_CREATE_REQUEST, handle_create_srv),
-    LWMSG_DISPATCH(HANDLE_DESTROY_REQUEST, handle_destroy_srv),
-    LWMSG_DISPATCH(HANDLE_GET_REQUEST, handle_get_srv),
-    LWMSG_DISPATCH_END
+    LWMsgTag tag;
+    void* data;
+} handle_dispatch[] =
+{
+    {HANDLE_CREATE_REQUEST, handle_create_srv},
+    {HANDLE_DESTROY_REQUEST, handle_destroy_srv},
+    {HANDLE_GET_REQUEST, handle_get_srv},
+    {-1, NULL}
 };
 
 static void*
@@ -883,16 +939,16 @@ handle_receiver(void* _assoc)
 {
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
     LWMsgAssoc* assoc = _assoc;
-    LWMsgMessage recv_message;
-    LWMsgMessage send_message;
+    LWMsgMessage recv_message = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage send_message = LWMSG_MESSAGE_INITIALIZER;
     size_t i;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_accept(assoc, NULL, NULL));
 
     do
     {
         MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv_message));
-        for (i = 0; handle_dispatch[i].type != LWMSG_DISPATCH_TYPE_END; i++)
+        for (i = 0; handle_dispatch[i].tag != -1; i++)
         {
             if (recv_message.tag == handle_dispatch[i].tag)
             {
@@ -1040,7 +1096,7 @@ fd_sender(void* _assoc)
         MU_FAILURE("write(): %s", strerror(errno));
     }
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_connect(assoc, NULL));
 
     request_msg.tag = FD_REQUEST;
     request_msg.data = &request;
@@ -1072,7 +1128,7 @@ fd_receiver(void* _assoc)
     FdReply reply;
     char buffer[1024];
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_establish(assoc));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_accept(assoc, NULL, NULL));
     
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &request_msg));
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, request_msg.tag, FD_REQUEST);
@@ -1154,6 +1210,8 @@ MU_TEST(assoc, fd_send_recv)
     }
 }
 
+typedef struct AHandle AHandle;
+
 static LWMsgTypeSpec local_handle_spec[] =
 {
     LWMSG_HANDLE(AHandle),
@@ -1187,12 +1245,14 @@ send_local_recv_back_success(LWMsgAssoc* assoc)
     int dummy;
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgSession* session = NULL;
 
     request_msg.tag = TRIVIAL_REMOTE;
     request_msg.data = &dummy;
 
     /* The handle is being created by us, so it is REMOTE for the receiver */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy, NULL));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &request_msg));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &reply_msg));
 
@@ -1219,15 +1279,20 @@ static void
 send_local_recv_back_failure(LWMsgAssoc* assoc)
 {
     int dummy = 0;
-    int* dummy2 = NULL;
-    LWMsgTag tag = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgSession* session = NULL;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
+
+    send.tag = TRIVIAL_REMOTE;
+    send.data = &dummy;
 
     /* The handle is being created by us, so it is REMOTE for the recvr */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy, NULL));
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_REMOTE, &dummy));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
     /* The peer is going to bomb out on us, so expect a disconnect */
-    status = lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy2);
+    status = lwmsg_assoc_recv_message(assoc, &recv);
 
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_PEER_CLOSE);
 }
@@ -1237,16 +1302,21 @@ send_local_recv_back_failure(LWMsgAssoc* assoc)
 static void
 recv_remote_send_back_failure(LWMsgAssoc* assoc)
 {
-    int* dummy = NULL;
     int dummy2 = 0;
-    LWMsgTag tag = 0;
     LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgSession* session = NULL;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy));
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_REMOTE);
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_REMOTE);
     /* Send back a local handle instead of what it expects */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy2, NULL));
-    status = lwmsg_assoc_send(assoc, TRIVIAL_LOCAL, &dummy2);
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy2, NULL));
+    send.tag = TRIVIAL_LOCAL;
+    send.data = &dummy2;
+    status = lwmsg_assoc_send_message(assoc, &send);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_INVALID_HANDLE);
     MU_VERBOSE("%s", lwmsg_assoc_get_error_message(assoc, status));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_close(assoc));
@@ -1255,59 +1325,80 @@ recv_remote_send_back_failure(LWMsgAssoc* assoc)
 static void
 send_null_recv_null_success(LWMsgAssoc* assoc)
 {
-    int* dummy2 = NULL;
-    LWMsgTag tag = 0;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_REMOTE, NULL));
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy2));
+    send.tag = TRIVIAL_REMOTE;
+    send.data = NULL;
 
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_LOCAL);
-    MU_ASSERT_EQUAL(MU_TYPE_POINTER, dummy2, NULL);
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_LOCAL);
+    MU_ASSERT_EQUAL(MU_TYPE_POINTER, recv.data, NULL);
 }
 
 static void
 recv_null_send_null_success(LWMsgAssoc* assoc)
 {
-    int* dummy = NULL;
-    LWMsgTag tag = 0;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy));
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_REMOTE);
-    MU_ASSERT_EQUAL(MU_TYPE_POINTER, dummy, NULL);
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_LOCAL, dummy));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_REMOTE);
+    MU_ASSERT_EQUAL(MU_TYPE_POINTER, recv.data, NULL);
+
+    send.tag = TRIVIAL_LOCAL;
+    send.data = recv.data;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
 }
 
 static void
 send_local_recv_back_send_back_success(LWMsgAssoc* assoc)
 {
     int dummy = 0;
-    int* dummy2 = NULL;
-    LWMsgTag tag = 0;
+    LWMsgSession* session = NULL;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
+
+    send.tag = TRIVIAL_REMOTE;
+    send.data = &dummy;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
 
     /* The handle is being created by us, so it is REMOTE for the receiver */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy, NULL));
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_REMOTE, &dummy));
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy2));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
 
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_LOCAL);
-    MU_ASSERT_EQUAL(MU_TYPE_POINTER, dummy2, &dummy);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_LOCAL);
+    MU_ASSERT_EQUAL(MU_TYPE_POINTER, recv.data, &dummy);
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_REMOTE, dummy2));
+    send.tag = TRIVIAL_REMOTE;
+    send.data = recv.data;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
 }
 
 static void
 recv_remote_send_back_recv_back_success(LWMsgAssoc* assoc)
 {
     int* dummy = 0;
-    int* dummy2 = NULL;
-    LWMsgTag tag = 0;
+    LWMsgMessage send = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgMessage recv = LWMSG_MESSAGE_INITIALIZER;
 
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy));
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_REMOTE);
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_send(assoc, TRIVIAL_LOCAL, dummy));
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv(assoc, &tag, (void**) (void*) &dummy2));
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, tag, TRIVIAL_REMOTE);
-    MU_ASSERT_EQUAL(MU_TYPE_POINTER, dummy, dummy2);
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_REMOTE);
+
+    dummy = recv.data;
+    send.tag = TRIVIAL_LOCAL;
+    send.data = dummy;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &send));
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &recv));
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, recv.tag, TRIVIAL_REMOTE);
+    MU_ASSERT_EQUAL(MU_TYPE_POINTER, recv.data, dummy);
 }
 
 static void
@@ -1335,15 +1426,18 @@ send_local_recv_back_local_send_failure(LWMsgAssoc* assoc)
     int dummy;
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgSession* session = NULL;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
 
     request_msg.tag = TRIVIAL_REMOTE;
     request_msg.data = &dummy;
 
     /* The handle is being created by us, so it is REMOTE for the receiver */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy, NULL));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &request_msg));
     /* Before receiving it back, we sabotage things by unregistering the handle */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_unregister_handle(assoc, &dummy));
+    MU_TRY_ASSOC(assoc, lwmsg_session_unregister_handle(session, &dummy));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &reply_msg));
 
     MU_ASSERT(reply_msg.flags & LWMSG_MESSAGE_FLAG_SYNTHETIC);
@@ -1382,12 +1476,15 @@ send_local_recv_back_bogus(LWMsgAssoc* assoc)
     int dummy;
     LWMsgMessage request_msg = LWMSG_MESSAGE_INITIALIZER;
     LWMsgMessage reply_msg = LWMSG_MESSAGE_INITIALIZER;
+    LWMsgSession* session = NULL;
+
+    MU_TRY_ASSOC(assoc, lwmsg_assoc_get_session(assoc, &session));
 
     request_msg.tag = TRIVIAL_REMOTE;
     request_msg.data = &dummy;
 
     /* The handle is being created by us, so it is REMOTE for the receiver */
-    MU_TRY_ASSOC(assoc, lwmsg_assoc_register_handle(assoc, "AHandle", &dummy, NULL));
+    MU_TRY_ASSOC(assoc, lwmsg_session_register_handle(session, "AHandle", &dummy, NULL));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_send_message(assoc, &request_msg));
     MU_TRY_ASSOC(assoc, lwmsg_assoc_recv_message(assoc, &reply_msg));
 
