@@ -42,9 +42,102 @@
 #include <string.h>
 
 #include "test-private.h"
+#include "type-private.h"
+#include "context-private.h"
 
 static LWMsgContext* context = NULL;
 static LWMsgDataContext* dcontext = NULL;
+static size_t allocs = 0;
+
+const char pattern[16] = "deadbeefdeadbeef";
+
+static LWMsgStatus
+debug_alloc (
+    size_t size,
+    void** out,
+    void* data)
+{
+    void* object = malloc(size + sizeof(pattern));
+
+    if (!object)
+    {
+        return LWMSG_STATUS_MEMORY;
+    }
+    else
+    {
+        memset(object, 0, size + sizeof(pattern));
+
+        memcpy(object, pattern, sizeof(pattern));
+
+        *out = ((unsigned char*) object) + sizeof(pattern);
+
+        allocs++;
+
+        return LWMSG_STATUS_SUCCESS;
+    }
+}
+
+static
+void
+debug_free (
+    void* object,
+    void* data
+    )
+{
+    if (object)
+    {
+        object = ((unsigned char*) object) - sizeof(pattern);
+
+        MU_ASSERT(!memcmp(object, pattern, sizeof(pattern)));
+        memset(object, 0, sizeof(pattern));
+
+        allocs--;
+
+        free(object);
+    }
+}
+
+static LWMsgStatus
+debug_realloc (
+    void* object,
+    size_t old_size,
+    size_t new_size,
+    void** new_object,
+    void* data)
+{
+    if (object)
+    {
+        object = ((unsigned char*) object) - sizeof(pattern);
+
+        MU_ASSERT(!memcmp(object, pattern, sizeof(pattern)));
+
+        memset(object, 0, sizeof(pattern));
+    }
+
+    void* nobj = realloc(object, new_size + sizeof(pattern));
+
+    if (!nobj)
+    {
+        return LWMSG_STATUS_MEMORY;
+    }
+    else
+    {
+        memcpy(nobj, pattern, sizeof(pattern));
+
+        if (new_size > old_size)
+        {
+            memset(nobj + sizeof(pattern) + old_size, 0, new_size - old_size);
+        }
+        *new_object = ((unsigned char*) nobj) + sizeof(pattern);
+
+        if (!object)
+        {
+            allocs++;
+        }
+
+        return LWMSG_STATUS_SUCCESS;
+    }
+}
 
 static void
 allocate_buffer(LWMsgBuffer* buffer)
@@ -67,9 +160,22 @@ rewind_buffer(LWMsgBuffer* buffer)
 MU_FIXTURE_SETUP(marshal)
 {
     MU_TRY(lwmsg_context_new(NULL, &context));
+
+    lwmsg_context_set_memory_functions(
+               context,
+               debug_alloc,
+               debug_free,
+               debug_realloc,
+               NULL);
+
     MU_TRY(lwmsg_data_context_new(context, &dcontext));
 }
 
+MU_FIXTURE_TEARDOWN(marshal)
+{
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, allocs, 0);
+}
+
 typedef struct basic_struct
 {
     short foo;
@@ -82,15 +188,12 @@ LWMsgStatus
 basic_verify_foo(
     LWMsgDataContext* dcontext,
     LWMsgBool unmarshalling,
-    size_t object_size,
     void* object,
     void* data
     )
 {
     short* fooptr = (short*) object;
 
-    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, object_size, sizeof(short));
-
     if (*fooptr != -42)
     {
         return LWMSG_STATUS_MALFORMED;
@@ -150,6 +253,56 @@ MU_TEST(marshal, basic)
     MU_ASSERT(basic.len == out->len);
     MU_ASSERT(basic.long_ptr[0] == out->long_ptr[0]);
     MU_ASSERT(basic.long_ptr[1] == out->long_ptr[1]);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+    lwmsg_context_free(context, buffer);
+}
+
+MU_TEST(marshal, basic_into)
+{
+    static const unsigned char expected[] =
+    {
+        /* -42 */
+        0xFF, 0xD6,
+        /* 2 */
+        0x00, 0x00, 0x00, 0x02,
+        /* 1234 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0xD2,
+        /* 4321 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0xE1
+    };
+    LWMsgTypeSpec* type = basic_spec;
+    void* buffer;
+    size_t length;
+    basic_struct basic;
+    basic_struct out;
+    long longs[2];
+    LWMsgBuffer mbuf = {0};
+
+    basic.foo = (short) -42;
+    basic.len = 2;
+    basic.long_ptr = longs;
+    longs[0] = 1234;
+    longs[1] = 4321;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &basic, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    mbuf.base = buffer;
+    mbuf.end = buffer + length;
+    mbuf.cursor = mbuf.base;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_into(dcontext, type, &mbuf, &out, sizeof(out)));
+
+    MU_ASSERT(basic.foo == out.foo);
+    MU_ASSERT(basic.len == out.len);
+    MU_ASSERT(basic.long_ptr[0] == out.long_ptr[0]);
+    MU_ASSERT(basic.long_ptr[1] == out.long_ptr[1]);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_destroy_graph(dcontext, type, &out));
+    lwmsg_context_free(context, buffer);
 }
 
 MU_TEST(marshal, basic_verify_marshal_failure)
@@ -254,6 +407,257 @@ MU_TEST(marshal, basic_verify_null_failure)
         LWMSG_STATUS_EOF);
 }
 
+typedef struct alias_struct
+{
+    long *ptr1;
+    long *ptr2;
+} alias_struct;
+
+static LWMsgTypeSpec long_pointer_spec[] =
+{
+    LWMSG_POINTER(LWMSG_INT64(long)),
+    LWMSG_ATTR_LENGTH_STATIC(2),
+    LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec alias_spec[] =
+{
+    LWMSG_STRUCT_BEGIN(alias_struct),
+    LWMSG_MEMBER_TYPESPEC(alias_struct, ptr1, long_pointer_spec),
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_ATTR_ALIASABLE,
+    LWMSG_MEMBER_TYPESPEC(alias_struct, ptr2, long_pointer_spec),
+    LWMSG_ATTR_ALIASABLE,
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+MU_TEST(marshal, alias)
+{
+    static const unsigned char expected[] =
+    {
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01,
+        /* 1234 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0xD2,
+        /* 4321 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0xE1,
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01
+    };
+
+    LWMsgTypeSpec* type = alias_spec;
+    void* buffer;
+    size_t length;
+    alias_struct alias;
+    alias_struct *out;
+    long longs[2];
+    char* text = NULL;
+
+    alias.ptr1 = longs;
+    alias.ptr2 = longs;
+
+    longs[0] = 1234;
+    longs[1] = 4321;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &alias, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT(alias.ptr1[0] == out->ptr1[0]);
+    MU_ASSERT(alias.ptr1[1] == out->ptr1[1]);
+    MU_ASSERT(out->ptr1 == out->ptr2);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_print_graph_alloc(dcontext, type, out, &text));
+
+    MU_VERBOSE("\n%s", text);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+    lwmsg_context_free(context, text);
+    lwmsg_context_free(context, buffer);
+}
+
+MU_TEST(marshal, alias_null)
+{
+    static const unsigned char expected[] =
+    {
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01,
+        /* 1234 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0xD2,
+        /* 4321 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0xE1,
+        /* object id (NULL) */
+        0x00, 0x00, 0x00, 0x00
+    };
+
+    LWMsgTypeSpec* type = alias_spec;
+    void* buffer;
+    size_t length;
+    alias_struct alias;
+    alias_struct *out;
+    long longs[2];
+
+    alias.ptr1 = longs;
+    alias.ptr2 = NULL;
+
+    longs[0] = 1234;
+    longs[1] = 4321;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &alias, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT(alias.ptr1[0] == out->ptr1[0]);
+    MU_ASSERT(alias.ptr1[1] == out->ptr1[1]);
+    MU_ASSERT(out->ptr2 == NULL);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+    lwmsg_context_free(context, buffer);
+}
+
+MU_TEST(marshal, alias_verify_null_marshal_failure)
+{
+    LWMsgTypeSpec* type = alias_spec;
+    void* buffer;
+    size_t length;
+    alias_struct alias;
+    long longs[2];
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    alias.ptr2 = longs;
+    alias.ptr1 = NULL;
+
+    longs[0] = 1234;
+    longs[1] = 4321;
+
+    status = lwmsg_data_marshal_flat_alloc(dcontext, type, &alias, &buffer, &length);
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_MALFORMED);
+}
+
+MU_TEST(marshal, alias_verify_null_unmarshal_failure)
+{
+    static const unsigned char data[] =
+    {
+        /* object id (NULL) */
+        0x00, 0x00, 0x00, 0x00,
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01,
+        /* 1234 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0xD2,
+        /* 4321 */
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0xE1,
+    };
+
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+    LWMsgTypeSpec* type = alias_spec;
+    alias_struct *out;
+
+    status = lwmsg_data_unmarshal_flat(dcontext, type, data, sizeof(data), (void**) (void*) &out);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_MALFORMED);
+}
+
+typedef struct ring_struct
+{
+    int number;
+    struct ring_struct* next;
+} ring_struct;
+
+static LWMsgTypeSpec ring_ptr_spec[];
+
+static LWMsgTypeSpec ring_spec[] =
+{
+    LWMSG_STRUCT_BEGIN(ring_struct),
+    LWMSG_MEMBER_INT32(ring_struct, number),
+    LWMSG_MEMBER_TYPESPEC(ring_struct, next, ring_ptr_spec),
+    LWMSG_ATTR_NOT_NULL,
+    LWMSG_STRUCT_END,
+    LWMSG_TYPE_END
+};
+
+static LWMsgTypeSpec ring_ptr_spec[] =
+{
+    LWMSG_POINTER(LWMSG_TYPESPEC(ring_spec)),
+    LWMSG_ATTR_ALIASABLE,
+    LWMSG_TYPE_END
+};
+
+MU_TEST(marshal, ring)
+{
+    static const unsigned char expected[] =
+    {
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01,
+        /* 10 */
+        0x00, 0x00, 0x00, 0x0A,
+        /* object id 2 */
+        0x00, 0x00, 0x00, 0x02,
+        /* 20 */
+        0x00, 0x00, 0x00, 0x14,
+        /* object id 3 */
+        0x00, 0x00, 0x00, 0x03,
+        /* 30 */
+        0x00, 0x00, 0x00, 0x1e,
+        /* object id 1 */
+        0x00, 0x00, 0x00, 0x01
+    };
+
+    LWMsgTypeSpec* type = ring_ptr_spec;
+    void* buffer;
+    size_t length;
+    ring_struct ring1, ring2, ring3;
+    ring_struct *out;
+    char* text = NULL;
+
+    ring1.number = 10;
+    ring1.next = &ring2;
+    ring2.number = 20;
+    ring2.next = &ring3;
+    ring3.number = 30;
+    ring3.next = &ring1;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &ring1, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out->number, 10);
+    MU_ASSERT(out->next != NULL);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out->next->number, 20);
+    MU_ASSERT(out->next->next != NULL);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, out->next->next->number, 30);
+    MU_ASSERT_EQUAL(MU_TYPE_POINTER, out, out->next->next->next);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_print_graph_alloc(dcontext, type, out, &text));
+
+    MU_VERBOSE("\n%s", text);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+    lwmsg_context_free(context, text);
+    lwmsg_context_free(context, buffer);
+}
+
+MU_TEST(marshal, ring_print_type)
+{
+    LWMsgTypeSpec* type = ring_ptr_spec;
+    char* text = NULL;
+
+    MU_TRY(lwmsg_type_print_spec_alloc(context, type, &text));
+
+    MU_VERBOSE("\n%s", text);
+
+    lwmsg_context_free(context, text);
+}
+
 typedef struct
 {
     const char* foo;
@@ -264,6 +668,7 @@ static LWMsgTypeSpec string_spec[] =
 {
     LWMSG_STRUCT_BEGIN(string_struct),
     LWMSG_MEMBER_PSTR(string_struct, foo),
+    LWMSG_ATTR_MAX_ALLOC(256),
     LWMSG_MEMBER_PSTR(string_struct, bar),
     LWMSG_ATTR_SENSITIVE,
     LWMSG_STRUCT_END,
@@ -313,6 +718,34 @@ MU_TEST(marshal, string)
 
     MU_ASSERT_EQUAL(MU_TYPE_STRING, strings.foo, out->foo);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, strings.bar, out->bar);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+    lwmsg_context_free(context, text);
+}
+
+MU_TEST(marshal, string_max_alloc_fail)
+{
+    LWMsgTypeSpec* type = string_spec;
+    LWMsgBuffer buffer;
+    string_struct strings;
+    string_struct* out = NULL;
+    static char foo[512] = {0};
+
+    memset(foo, 'x', sizeof(foo) - 1);
+
+    strings.foo = foo;
+    strings.bar = "bar";
+
+    allocate_buffer(&buffer);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal(dcontext, type, &strings, &buffer));
+
+    rewind_buffer(&buffer);
+
+    MU_ASSERT_EQUAL(
+        MU_TYPE_INTEGER,
+        lwmsg_data_unmarshal(dcontext, type, &buffer, (void**) (void*) &out),
+        LWMSG_STATUS_OVERFLOW);
 }
 
 typedef struct
@@ -378,6 +811,8 @@ MU_TEST(marshal, struct_array)
     MU_ASSERT(structs.foo[0].b == out->foo[0].b);
     MU_ASSERT(structs.foo[1].a == out->foo[1].a);
     MU_ASSERT(structs.foo[1].b == out->foo[1].b);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 typedef struct
@@ -605,6 +1040,8 @@ MU_TEST(marshal, two_union)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.tag2, out->tag2);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.u1->number, out->u1->number);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, unions.u2->string, out->u2->string);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 typedef struct
@@ -672,6 +1109,8 @@ MU_TEST(marshal, nested_struct)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, nested.inner.foo, out->inner.foo);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, nested.inner.bar, out->inner.bar);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, nested.bar, out->bar);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 typedef struct
@@ -737,6 +1176,8 @@ MU_TEST(marshal, nested_union)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.tag2, out->tag2);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, unions.u1.string, out->u1.string);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.u2.number, out->u2.number);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 MU_TEST(marshal, nested_union_empty)
@@ -773,6 +1214,8 @@ MU_TEST(marshal, nested_union_empty)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.tag1, out->tag1);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.tag2, out->tag2);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, unions.u2.number, out->u2.number);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 typedef struct
@@ -833,6 +1276,8 @@ MU_TEST(marshal, inline_array)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in.array[0], out->array[0]);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in.array[1], out->array[1]);
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in.bar, out->bar);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
 typedef struct
@@ -887,9 +1332,23 @@ MU_TEST(marshal, flexible_string)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in->foo, out->foo);
     MU_ASSERT_EQUAL(MU_TYPE_STRING, in->string, out->string);
 
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+
     free(in);
 }
 
+MU_TEST(marshal, flexible_string_print_type)
+{
+    LWMsgTypeSpec* type = flexible_string_spec;
+    char* text = NULL;
+
+    MU_TRY(lwmsg_type_print_spec_alloc(context, type, &text));
+
+    MU_VERBOSE("\n%s", text);
+
+    lwmsg_context_free(context, text);
+}
+
 typedef struct
 {
     int len;
@@ -945,8 +1404,22 @@ MU_TEST(marshal, flexible_array)
     MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in->array[1], out->array[1]);
 
     free(in);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
 
+MU_TEST(marshal, flexible_array_print_type)
+{
+    LWMsgTypeSpec* type = flexible_array_spec;
+    char* text = NULL;
+
+    MU_TRY(lwmsg_type_print_spec_alloc(context, type, &text));
+    MU_VERBOSE("\n%s", text);
+
+    lwmsg_context_free(context, text);
+}
+
+
 typedef struct info_level_struct
 {
     unsigned int length;
@@ -992,6 +1465,16 @@ static LWMsgTypeSpec info_level_spec[] =
     LWMSG_TYPE_END
 };
 
+MU_TEST(marshal, info_level_print_type)
+{
+    char* text = NULL;
+
+    MU_TRY(lwmsg_type_print_spec_alloc(context, info_level_spec, &text));
+    MU_VERBOSE("\n%s", text);
+
+    lwmsg_context_free(context, text);
+}
+
 MU_TEST(marshal, info_level_1)
 {
     static const unsigned char expected[] =
@@ -1099,4 +1582,211 @@ MU_TEST(marshal, info_level_2)
         MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in.array.level_2[0].number1, out->array.level_2[0].number1);
         MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in.array.level_2[0].number2, out->array.level_2[0].number2);
     }
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+}
+
+MU_TEST(marshal, type_rep_print_type)
+{
+    char* text = NULL;
+
+    MU_TRY(lwmsg_type_print_spec_alloc(context, lwmsg_type_rep_spec, &text));
+
+    MU_VERBOSE("\n%s", text);
+    lwmsg_context_free(context, text);
+}
+
+MU_TEST(marshal, type_spec_ping_pong)
+{
+    LWMsgMemoryList list;
+    LWMsgTypeRep* rep = NULL;
+    LWMsgTypeSpec* spec = NULL;
+    char* text = NULL;
+
+    lwmsg_memlist_init(&list, context);
+
+    MU_TRY(lwmsg_type_rep_from_spec(context, lwmsg_type_rep_spec, &rep));
+    MU_TRY(lwmsg_type_spec_from_rep(lwmsg_memlist_context(&list), rep, &spec));
+    MU_TRY(lwmsg_type_print_spec_alloc(context, spec, &text));
+
+    MU_VERBOSE("\n%s", text);
+    lwmsg_context_free(context, text);
+    lwmsg_type_free_rep(context, rep);
+    lwmsg_memlist_destroy(&list);
+}
+
+MU_TEST(marshal, type_rep_rep_self_assignable)
+{
+    LWMsgTypeRep* rep = NULL;
+
+    MU_TRY(lwmsg_type_rep_from_spec(context, lwmsg_type_rep_spec, &rep));
+
+    MU_TRY(lwmsg_type_rep_is_assignable(rep, rep));
+
+    lwmsg_type_free_rep(context, rep);
+}
+
+typedef enum MixedEnum
+{
+    MIXED_VALUE_1 = 1,
+    MIXED_VALUE_2 = 2,
+    MIXED_MASK_1 = 0x1000,
+    MIXED_MASK_2 = 0x2000,
+} MixedEnum;
+
+static LWMsgTypeSpec MixedEnum_spec[] =
+{
+    LWMSG_ENUM_BEGIN(MixedEnum, 2, LWMSG_UNSIGNED),
+    LWMSG_ENUM_VALUE(MIXED_VALUE_1),
+    LWMSG_ENUM_VALUE(MIXED_VALUE_2),
+    LWMSG_ENUM_MASK(MIXED_MASK_1),
+    LWMSG_ENUM_MASK(MIXED_MASK_2),
+    LWMSG_ENUM_END,
+    LWMSG_TYPE_END,
+};
+
+MU_TEST(marshal, mixed_enum_mixed)
+{
+    static const unsigned char expected[] =
+    {
+        /* value */
+        0x20, 0x01
+    };
+
+    LWMsgTypeSpec* type = MixedEnum_spec;
+    void* buffer = NULL;
+    MixedEnum in = MIXED_VALUE_1 | MIXED_MASK_2;
+    MixedEnum* out = 0;
+    char* text = NULL;
+    size_t length = 0;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &in, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in, *out);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_print_graph_alloc(dcontext, type, &in, &text));
+
+    MU_VERBOSE("%s", text);
+
+    lwmsg_context_free(context, text);
+    lwmsg_context_free(context, buffer);
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+}
+
+MU_TEST(marshal, mixed_enum_value)
+{
+    static const unsigned char expected[] =
+    {
+        /* value */
+        0x00, 0x01
+    };
+
+    LWMsgTypeSpec* type = MixedEnum_spec;
+    void* buffer = NULL;
+    MixedEnum in = MIXED_VALUE_1;
+    MixedEnum* out = 0;
+    char* text = NULL;
+    size_t length = 0;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &in, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in, *out);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_print_graph_alloc(dcontext, type, &in, &text));
+
+    MU_VERBOSE("%s", text);
+
+    lwmsg_context_free(context, text);
+    lwmsg_context_free(context, buffer);
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
+}
+
+MU_TEST(marshal, mixed_enum_unmarshal_zero_fail)
+{
+    static const unsigned char data[] =
+    {
+        /* value */
+        0x10, 0x00
+    };
+
+    LWMsgTypeSpec* type = MixedEnum_spec;
+    MixedEnum* out = 0;
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    status = lwmsg_data_unmarshal_flat(dcontext, type, data, sizeof(data), (void**) (void*) &out);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_MALFORMED);
+}
+
+MU_TEST(marshal, mixed_enum_unmarshal_bogus_fail)
+{
+    static const unsigned char data[] =
+    {
+        /* value */
+        0x00, 0x03
+    };
+
+    LWMsgTypeSpec* type = MixedEnum_spec;
+    MixedEnum* out = 0;
+    LWMsgStatus status = LWMSG_STATUS_SUCCESS;
+
+    status = lwmsg_data_unmarshal_flat(dcontext, type, data, sizeof(data), (void**) (void*) &out);
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, status, LWMSG_STATUS_MALFORMED);
+}
+
+typedef enum MaskEnum
+{
+    MASK_MASK_1 = 0x1000,
+    MASK_MASK_2 = 0x2000,
+} MaskEnum;
+
+static LWMsgTypeSpec MaskEnum_spec[] =
+{
+    LWMSG_ENUM_BEGIN(MaskEnum, 2, LWMSG_UNSIGNED),
+    LWMSG_ENUM_MASK(MASK_MASK_1),
+    LWMSG_ENUM_MASK(MASK_MASK_2),
+    LWMSG_ENUM_END,
+    LWMSG_TYPE_END,
+};
+
+MU_TEST(marshal, mask_enum_zero)
+{
+    static const unsigned char expected[] =
+    {
+        /* value */
+        0x00, 0x00
+    };
+
+    LWMsgTypeSpec* type = MaskEnum_spec;
+    void* buffer = NULL;
+    MaskEnum in = 0;
+    MaskEnum* out = NULL;
+    char* text = NULL;
+    size_t length = 0;
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_marshal_flat_alloc(dcontext, type, &in, &buffer, &length));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, sizeof(expected), length);
+    MU_ASSERT(!memcmp(buffer, expected, sizeof(expected)));
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_unmarshal_flat(dcontext, type, buffer, length, (void**) (void*) &out));
+
+    MU_ASSERT_EQUAL(MU_TYPE_INTEGER, in, *out);
+
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_print_graph_alloc(dcontext, type, &in, &text));
+
+    MU_VERBOSE("%s", text);
+
+    lwmsg_context_free(context, text);
+    lwmsg_context_free(context, buffer);
+    MU_TRY_DCONTEXT(dcontext, lwmsg_data_free_graph(dcontext, type, out));
 }
