@@ -47,17 +47,21 @@
  */
 #include "client.h"
 
+static REG_CLIENT_CONNECTION_CONTEXT gContext = {0};
+static volatile LONG glLibraryRefCount = 0;
+static pthread_mutex_t gLock = PTHREAD_MUTEX_INITIALIZER;
+
 static
 NTSTATUS
-RegIpcUnregisterHandle(
-    LWMsgCall* pCall,
-    PVOID pHandle
+RegIpcReleaseHandle(
+    IN HANDLE hConnection,
+    IN PVOID pHandle
     )
 {
     NTSTATUS status = 0;
-    LWMsgSession* pSession = lwmsg_call_get_session(pCall);
+    PREG_CLIENT_CONNECTION_CONTEXT pContext = hConnection;
 
-    status = MAP_LWMSG_ERROR(lwmsg_session_unregister_handle(pSession, pHandle));
+    status = MAP_LWMSG_ERROR(lwmsg_session_release_handle(pContext->pSession, pHandle));
     BAIL_ON_NT_STATUS(status);
 
 error:
@@ -74,73 +78,77 @@ RegOpenServer(
 		NtRegOpenServer(phConnection));
 }
 
+NTSTATUS
+NtRegOpenServerInitialize(
+    VOID
+    )
+{
+    NTSTATUS status = 0;
+
+    pthread_mutex_lock(&gLock);
+
+    if (!gContext.pProtocol)
+    {
+        status = MAP_LWMSG_ERROR(lwmsg_protocol_new(NULL, &gContext.pProtocol));
+        BAIL_ON_NT_STATUS(status);
+
+        status = MAP_LWMSG_ERROR(lwmsg_protocol_add_protocol_spec(gContext.pProtocol, RegIPCGetProtocolSpec()));
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    if (!gContext.pClient)
+    {
+        status = MAP_LWMSG_ERROR(lwmsg_peer_new(NULL, gContext.pProtocol, &gContext.pClient));
+        BAIL_ON_NT_STATUS(status);
+
+        status = MAP_LWMSG_ERROR(lwmsg_peer_add_connect_endpoint(
+                                     gContext.pClient,
+                                     LWMSG_CONNECTION_MODE_LOCAL,
+                                     CACHEDIR "/" REG_SERVER_FILENAME));
+        BAIL_ON_NT_STATUS(status);
+    }
+
+    if (!gContext.pSession)
+    {
+        status = MAP_LWMSG_ERROR(lwmsg_peer_connect(gContext.pClient, &gContext.pSession));
+        BAIL_ON_NT_STATUS(status);
+    }
+
+cleanup:
+
+    pthread_mutex_unlock(&gLock);
+
+    return status;
+
+error:
+
+    goto cleanup;
+}
+
+
 NTSTATUS
 NtRegOpenServer(
     OUT PHANDLE phConnection
     )
 {
     NTSTATUS status = 0;
-    PREG_CLIENT_CONNECTION_CONTEXT pContext = NULL;
-    static LWMsgTime connectTimeout = {2, 0};
 
     BAIL_ON_NT_INVALID_POINTER(phConnection);
 
-    status = LW_RTL_ALLOCATE((PVOID*)&pContext, REG_CLIENT_CONNECTION_CONTEXT, sizeof(*pContext));
+    status = NtRegOpenServerInitialize();
     BAIL_ON_NT_STATUS(status);
 
-    status = MAP_LWMSG_ERROR(lwmsg_protocol_new(NULL, &pContext->pProtocol));
-    BAIL_ON_NT_STATUS(status);
-
-    status = MAP_LWMSG_ERROR(lwmsg_protocol_add_protocol_spec(pContext->pProtocol, RegIPCGetProtocolSpec()));
-    BAIL_ON_NT_STATUS(status);
-
-    status = MAP_LWMSG_ERROR(lwmsg_connection_new(NULL, pContext->pProtocol, &pContext->pAssoc));
-    BAIL_ON_NT_STATUS(status);
-
-    status = MAP_LWMSG_ERROR(lwmsg_connection_set_endpoint(
-                                  pContext->pAssoc,
-                                  LWMSG_CONNECTION_MODE_LOCAL,
-                                  CACHEDIR "/" REG_SERVER_FILENAME));
-    BAIL_ON_NT_STATUS(status);
-
-    if (getenv("LW_DISABLE_CONNECT_TIMEOUT") == NULL)
-    {
-        /* Give up connecting within 2 seconds in case lsassd
-           is unresponsive (e.g. it's being traced in a debugger) */
-	status = MAP_LWMSG_ERROR(lwmsg_assoc_set_timeout(
-                                      pContext->pAssoc,
-                                      LWMSG_TIMEOUT_ESTABLISH,
-                                      &connectTimeout));
-        BAIL_ON_NT_STATUS(status);
-    }
-
-    status = MAP_LWMSG_ERROR(lwmsg_assoc_establish(pContext->pAssoc));
-    BAIL_ON_NT_STATUS(status);
-
-    *phConnection = (HANDLE)pContext;
+    *phConnection = (HANDLE) &gContext;
 
 cleanup:
+
     return status;
 
 error:
-    if (pContext)
-    {
-        if (pContext->pAssoc)
-        {
-            lwmsg_assoc_delete(pContext->pAssoc);
-        }
-
-        if (pContext->pProtocol)
-        {
-            lwmsg_protocol_delete(pContext->pProtocol);
-        }
-
-        LwRtlMemoryFree(pContext);
-    }
 
     if (phConnection)
     {
-        *phConnection = (HANDLE)NULL;
+        *phConnection = NULL;
     }
 
     goto cleanup;
@@ -157,27 +165,43 @@ RegCloseServer(
 
 VOID
 NtRegCloseServer(
-    IN HANDLE hConnection
+    HANDLE hConnection
     )
 {
-    PREG_CLIENT_CONNECTION_CONTEXT pContext =
-                     (PREG_CLIENT_CONNECTION_CONTEXT)hConnection;
+    return;
+}
 
-    if (!pContext)
-        return;
+static
+__attribute__((constructor))
+VOID
+NtRegOpenServerOnce(
+    VOID
+    )
+{
+    LwInterlockedIncrement(&glLibraryRefCount);
+}
 
-    if (pContext->pAssoc)
+static
+__attribute__((destructor))
+VOID
+NtRegCloseServerOnce(
+    VOID
+    )
+{
+    if (!LwInterlockedDecrement(&glLibraryRefCount))
     {
-        lwmsg_assoc_close(pContext->pAssoc);
-        lwmsg_assoc_delete(pContext->pAssoc);
-    }
+        if (gContext.pClient)
+        {
+            lwmsg_peer_delete(gContext.pClient);
+        }
 
-    if (pContext->pProtocol)
-    {
-        lwmsg_protocol_delete(pContext->pProtocol);
-    }
+        if (gContext.pProtocol)
+        {
+            lwmsg_protocol_delete(gContext.pProtocol);
+        }
 
-    LwRtlMemoryFree(pContext);
+        memset(&gContext, 0, sizeof(gContext));
+    }
 }
 
 NTSTATUS
@@ -189,7 +213,7 @@ RegIpcAcquireCall(
     NTSTATUS status = 0;
     PREG_CLIENT_CONNECTION_CONTEXT pContext = hConnection;
 
-    status = MAP_LWMSG_ERROR(lwmsg_assoc_acquire_call(pContext->pAssoc, ppCall));
+    status = MAP_LWMSG_ERROR(lwmsg_client_acquire_call(pContext->pClient, ppCall));
     BAIL_ON_NT_STATUS(status);
 
 error:
@@ -461,9 +485,6 @@ RegTransactCloseKey(
     switch (out.tag)
     {
         case REG_R_CLOSE_KEY:
-		status = RegIpcUnregisterHandle(pCall, hKey);
-            BAIL_ON_NT_STATUS(status);
-
             break;
 
         case REG_R_ERROR:
@@ -478,6 +499,10 @@ RegTransactCloseKey(
     }
 
 cleanup:
+
+    /* Release handle no matter what */
+    RegIpcReleaseHandle(hConnection, hKey);
+
     if (pCall)
     {
         lwmsg_call_destroy_params(pCall, &out);
