@@ -1215,19 +1215,30 @@ NtRegSetValueExA(
         BAIL_ON_NT_STATUS(status);
     }
 
+    if (dwType == REG_MULTI_SZ)
+    {
+        if (!pData)
+        {
+            pData = (PBYTE) "";
+        }
+        if (cbData == 0)
+        {
+            cbData = 1;
+        }
+    }
     if (pData)
     {
-	if (REG_MULTI_SZ == dwType)
-	{
-		status = NtRegConvertByteStreamA2W((PBYTE)pData,
-				                          cbData,
-				                          &pOutData,
-				                          &cbOutDataLen);
-		BAIL_ON_NT_STATUS(status);
+        if (REG_MULTI_SZ == dwType)
+        {
+                status = NtRegConvertByteStreamA2W((PBYTE)pData,
+                                                          cbData,
+                                                          &pOutData,
+                                                          &cbOutDataLen);
+                BAIL_ON_NT_STATUS(status);
 
-		bIsStrType = TRUE;
-	}
-	else if (REG_SZ == dwType)
+                bIsStrType = TRUE;
+        }
+        else if (REG_SZ == dwType)
 	{
             /* Verify correct null termination of input data */
             if (strlen((char *) pData) != (cbData-1))
@@ -1276,6 +1287,17 @@ NtRegSetValueExW(
     IN DWORD cbData
     )
 {
+    if (dwType == REG_MULTI_SZ)
+    {
+        if (!pData)
+        {
+            pData = (PBYTE) "\0\0";
+        }
+        if (cbData == 0)
+        {
+            cbData = sizeof(WCHAR);
+        }
+    }
     return RegTransactSetValueExW(
         hRegConnection,
         hKey,
