@@ -419,6 +419,11 @@ strtoull(
 
 #endif /* defined(HAVE_STRTOULL) */
 
+DWORD
+RegGetCurrentTimeSeconds(
+    OUT time_t* pTime
+    );
+
 DWORD
 RegRemoveFile(
     PCSTR pszPath
@@ -852,14 +857,6 @@ RegStripWhitespace(
     BOOLEAN bTrailing
     );
 
-NTSTATUS
-RegCopyValueBytes(
-    IN PBYTE pValue,
-    IN DWORD dwValueLen,
-    OUT OPTIONAL PBYTE pData,
-    IN OUT OPTIONAL PDWORD pcbData
-    );
-
 DWORD
 RegAllocateMemory(
     size_t Size,
@@ -889,6 +886,19 @@ RegCStringAllocateFromWC16String(
     IN PCWSTR pszOriginalString
     );
 
+DWORD
+RegWC16StringAllocatePrintfW(
+    LW_OUT LW_PWSTR* ppszString,
+    LW_IN const wchar_t* pszFormat,
+    LW_IN ...
+    );
+
+DWORD
+RegWC16StringDuplicate(
+    PWSTR* ppwszNewString,
+    PCWSTR pwszOriginalString
+    );
+
 DWORD
 RegCStringAllocatePrintf(
     OUT PSTR* ppszString,
@@ -908,4 +918,41 @@ RegStrchr(
     wchar16_t wch
     );
 
+DWORD
+RegBuildLogInfo(
+    RegLogLevel    maxAllowedLogLevel,
+    RegLogTarget   logTarget,
+    PCSTR          pszPath,
+    PREG_LOG_INFO* ppLogInfo
+    );
+
+DWORD
+RegSetLogLevel(
+    HANDLE      hRegConnection,
+    RegLogLevel logLevel
+    );
+
+DWORD
+RegGetLogInfo(
+    HANDLE         hRegConnection,
+    PREG_LOG_INFO* ppLogInfo
+    );
+
+DWORD
+RegSetLogInfo(
+    HANDLE        hRegConnection,
+    PREG_LOG_INFO pLogInfo
+    );
+
+VOID
+RegFreeLogInfo(
+    PREG_LOG_INFO pLogInfo
+    );
+
+LW_DWORD
+RegGetErrorMessageForLoggingEvent(
+    LW_DWORD dwError,
+    LW_PSTR* ppszErrorMsg
+    );
+
 #endif /* __REG_UTILS_H__ */
