@@ -180,6 +180,37 @@ c_delbefore(EditLine *el, int num)
 }
 
 
+/* c_delbefore1():
+ *	Delete the character before the cursor, do not yank
+ */
+#ifdef __LW_MULTIBYTE__
+protected void
+c_delbefore1(EditLine *el)
+{
+	int len = 0;
+	int i = 0;
+	char *cp = NULL;
+
+	/* Backspace character is in length map, but not in the data */
+	if (el->el_multibyte.charlen_map_indx < 1)
+		return;
+
+	/* Character before backspace character */
+	len = el->el_multibyte.charlen_map[el->el_multibyte.charlen_map_indx-1];
+
+	/* Nuke out the previous character byte values */
+	cp = el->el_line.cursor - len;
+        while (cp < el->el_line.lastchar)
+        {
+            *cp = cp[1];
+            cp++;
+        }
+	el->el_line.lastchar -= len;
+	el->el_multibyte.charlen_map_indx--;
+	el->el_state.argument = 0;
+	el->el_line.cursor -= len;
+}
+#else
 /* c_delbefore1():
  *	Delete the character before the cursor, do not yank
  */
@@ -193,6 +224,7 @@ c_delbefore1(EditLine *el)
 
 	el->el_line.lastchar--;
 }
+#endif
 
 
 /* ce__isword():
