@@ -121,16 +121,30 @@ error:
 
 NTSTATUS
 SqliteCreateKeyHandle(
-    IN ACCESS_MASK AccessGranted,
+    IN PACCESS_TOKEN pToken,
+    IN ACCESS_MASK AccessDesired,
     IN PREG_KEY_CONTEXT pKey,
     OUT PREG_KEY_HANDLE* ppKeyHandle
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
     PREG_KEY_HANDLE pKeyHandle = NULL;
+    ACCESS_MASK AccessGranted = 0;
 
     BAIL_ON_INVALID_KEY_CONTEXT(pKey);
 
+    status = RegSrvAccessCheckKey(pToken,
+                                  pKey->pSecurityDescriptor,
+                                  pKey->ulSecDescLength,
+                                  AccessDesired,
+                                  &AccessGranted);
+    if (STATUS_NO_TOKEN == status)
+    {
+        status = 0;
+        AccessGranted = 0;
+    }
+    BAIL_ON_NT_STATUS(status);
+
     status = LW_RTL_ALLOCATE((PVOID*)&pKeyHandle, REG_KEY_HANDLE, sizeof(*pKeyHandle));
     BAIL_ON_NT_STATUS(status);
 
@@ -150,7 +164,6 @@ error:
 
 NTSTATUS
 SqliteCreateKeyContext(
-    IN ACCESS_MASK AccessGranted,
     IN PREG_DB_KEY pRegEntry,
     OUT PREG_KEY_CONTEXT* ppKeyResult
     )
@@ -222,7 +235,6 @@ SqliteCreateKeyInternal(
     PREG_KEY_CONTEXT pKeyCtx = NULL;
     BOOLEAN bInLock = FALSE;
     PREG_SRV_API_STATE pServerState = (PREG_SRV_API_STATE)handle;
-    ACCESS_MASK AccessGranted = 0;
     PSECURITY_DESCRIPTOR_RELATIVE pSecDescRelToSet = NULL;
     ULONG ulSecDescLengthToSet = 0;
     DWORD dwDisposition = 0;
@@ -230,6 +242,16 @@ SqliteCreateKeyInternal(
     // Full key path
     BAIL_ON_NT_INVALID_STRING(pwszFullKeyName);
 
+    // when starting up lwregd pServerState is NULL and
+    // creating root key can skip ACL check
+    if (pServerState && !pServerState->pToken)
+    {
+        status = RegSrvCreateAccessToken(pServerState->peerUID,
+                                         pServerState->peerGID,
+                                         &pServerState->pToken);
+        BAIL_ON_NT_STATUS(status);
+    }
+
     LWREG_LOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
 
 	status = SqliteOpenKeyInternal_inlock(
@@ -284,26 +306,6 @@ SqliteCreateKeyInternal(
 		BAIL_ON_NT_STATUS(status);
 	}
 
-    // when starting up lwregd pServerState is NULL and
-	// creating root key can skip ACL check
-    if (pServerState)
-    {
-        if (!pServerState->pToken)
-        {
-	    status = RegSrvCreateAccessToken(pServerState->peerUID,
-			                         pServerState->peerGID,
-			                         &pServerState->pToken);
-            BAIL_ON_NT_STATUS(status);
-        }
-
-	status = RegSrvAccessCheckKey(pServerState->pToken,
-			                      pSecDescRelToSet,
-			                      ulSecDescLengthToSet,
-	                                  AccessDesired,
-	                                  &AccessGranted);
-	    BAIL_ON_NT_STATUS(status);
-    }
-
 	// Create key with SD
 	status = RegDbCreateKey(ghCacheConnection,
 							pwszFullKeyName,
@@ -317,14 +319,17 @@ SqliteCreateKeyInternal(
 	    SqliteCacheResetParentKeySubKeyInfo_inlock(pParentKeyCtx->pwszKeyName);
 	}
 
-	status = SqliteCreateKeyContext(AccessDesired, pRegEntry, &pKeyCtx);
+	status = SqliteCreateKeyContext(pRegEntry, &pKeyCtx);
 	BAIL_ON_NT_STATUS(status);
 
 	// Cache this new key in gActiveKeyList
 	status = SqliteCacheInsertActiveKey_inlock(pKeyCtx);
 	BAIL_ON_NT_STATUS(status);
 
-	status = SqliteCreateKeyHandle(AccessGranted, pKeyCtx, &pKeyHandle);
+	status = SqliteCreateKeyHandle(pServerState ? pServerState->pToken : NULL,
+	                               AccessDesired,
+	                               pKeyCtx,
+	                               &pKeyHandle);
 	BAIL_ON_NT_STATUS(status);
 	pKeyCtx = NULL;
 
@@ -369,19 +374,31 @@ error:
 
 NTSTATUS
 SqliteOpenKeyInternal(
-	IN OPTIONAL HANDLE handle,
+	IN HANDLE handle,
     IN PCWSTR pwszFullKeyName, // Full Key Path
     IN ACCESS_MASK AccessDesired,
     OUT OPTIONAL PREG_KEY_HANDLE* ppKeyHandle
     )
 {
 	NTSTATUS status = STATUS_SUCCESS;
+	PREG_SRV_API_STATE pServerState = (PREG_SRV_API_STATE)handle;
     BOOLEAN bInLock = FALSE;
 
     BAIL_ON_NT_INVALID_STRING(pwszFullKeyName);
 
+    BAIL_ON_NT_INVALID_POINTER(handle);
+
+    if (!pServerState->pToken)
+    {
+        status = RegSrvCreateAccessToken(pServerState->peerUID,
+                                         pServerState->peerGID,
+                                         &pServerState->pToken);
+        BAIL_ON_NT_STATUS(status);
+    }
+
     LWREG_LOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
 
+    // pServerState->pToken should be created at this point
     status = SqliteOpenKeyInternal_inlock(handle,
 		                              pwszFullKeyName,
 		                              AccessDesired,
@@ -397,34 +414,6 @@ error:
     goto cleanup;
 }
 
-NTSTATUS
-SqliteOpenKeyInternal_inDblock(
-	IN OPTIONAL HANDLE handle,
-    IN PCWSTR pwszFullKeyName, // Full Key Path
-    IN ACCESS_MASK AccessDesired,
-    OUT OPTIONAL PREG_KEY_HANDLE* ppKeyHandle
-    )
-{
-	NTSTATUS status = STATUS_SUCCESS;
-    BOOLEAN bInLock = FALSE;
-
-    LWREG_LOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
-
-    status = SqliteOpenKeyInternal_inlock_inDblock(handle,
-		                                       pwszFullKeyName,
-		                                       AccessDesired,
-		                                       ppKeyHandle);
-    BAIL_ON_NT_STATUS(status);
-
-cleanup:
-    LWREG_UNLOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
-
-    return status;
-
-error:
-    goto cleanup;
-}
-
 /* Open a key, if not found,
  * do not create a new key */
 NTSTATUS
@@ -440,7 +429,6 @@ SqliteOpenKeyInternal_inlock(
     PREG_DB_KEY pRegEntry = NULL;
     PREG_KEY_HANDLE pKeyHandle = NULL;
     PREG_KEY_CONTEXT pKeyCtx = NULL;
-    ACCESS_MASK AccessGranted = AccessDesired;
 
     BAIL_ON_NT_INVALID_STRING(pwszFullKeyName);
 
@@ -450,7 +438,7 @@ SqliteOpenKeyInternal_inlock(
         status = RegDbOpenKey(ghCacheConnection, pwszFullKeyName, &pRegEntry);
         BAIL_ON_NT_STATUS(status);
 
-        status = SqliteCreateKeyContext(AccessGranted, pRegEntry, &pKeyCtx);
+        status = SqliteCreateKeyContext(pRegEntry, &pKeyCtx);
         BAIL_ON_NT_STATUS(status);
 
         // Cache this new key in gActiveKeyList
@@ -458,25 +446,10 @@ SqliteOpenKeyInternal_inlock(
         BAIL_ON_NT_STATUS(status);
     }
 
-    if (pServerState)
-    {
-        if (!pServerState->pToken)
-        {
-	    status = RegSrvCreateAccessToken(pServerState->peerUID,
-			                         pServerState->peerGID,
-			                         &pServerState->pToken);
-            BAIL_ON_NT_STATUS(status);
-        }
-
-	status = RegSrvAccessCheckKey(pServerState->pToken,
-	                                  pKeyCtx->pSecurityDescriptor,
-	                                  pKeyCtx->ulSecDescLength,
-	                                  AccessDesired,
-	                                  &AccessGranted);
-	    BAIL_ON_NT_STATUS(status);
-    }
-
-    status = SqliteCreateKeyHandle(AccessGranted, pKeyCtx, &pKeyHandle);
+    status = SqliteCreateKeyHandle(pServerState ? pServerState->pToken : NULL,
+                                   AccessDesired,
+                                   pKeyCtx,
+                                   &pKeyHandle);
 	BAIL_ON_NT_STATUS(status);
 	pKeyCtx = NULL;
 
@@ -497,74 +470,66 @@ error:
 
 NTSTATUS
 SqliteOpenKeyInternal_inlock_inDblock(
-	IN OPTIONAL HANDLE handle,
-	IN PCWSTR pwszFullKeyName, // Full Key Path
-	IN ACCESS_MASK AccessDesired,
-	OUT OPTIONAL PREG_KEY_HANDLE* ppKeyHandle
-	)
+    IN OPTIONAL HANDLE handle,
+    IN PCWSTR pwszFullKeyName, // Full Key Path
+    IN ACCESS_MASK AccessDesired,
+    OUT OPTIONAL PREG_KEY_HANDLE* ppKeyHandle
+    )
 {
-	NTSTATUS status = STATUS_SUCCESS;
-	PREG_SRV_API_STATE pServerState = (PREG_SRV_API_STATE)handle;
-	PREG_DB_KEY pRegEntry = NULL;
-	PREG_KEY_HANDLE pKeyHandle = NULL;
-	PREG_KEY_CONTEXT pKeyCtx = NULL;
-	ACCESS_MASK AccessGranted = AccessDesired;
+    NTSTATUS status = STATUS_SUCCESS;
+    PREG_SRV_API_STATE pServerState = (PREG_SRV_API_STATE)handle;
+    PREG_DB_KEY pRegEntry = NULL;
+    PREG_KEY_HANDLE pKeyHandle = NULL;
+    PREG_KEY_CONTEXT pKeyCtx = NULL;
 
-	BAIL_ON_NT_INVALID_STRING(pwszFullKeyName);
+    BAIL_ON_NT_INVALID_STRING(pwszFullKeyName);
 
-	pKeyCtx = SqliteCacheLocateActiveKey_inlock(pwszFullKeyName);
-	if (!pKeyCtx)
-	{
-		status = RegDbOpenKey_inlock(ghCacheConnection, pwszFullKeyName, &pRegEntry);
-		BAIL_ON_NT_STATUS(status);
+    pKeyCtx = SqliteCacheLocateActiveKey_inlock(pwszFullKeyName);
+    if (!pKeyCtx)
+    {
+        status = RegDbOpenKey_inlock(ghCacheConnection, pwszFullKeyName, &pRegEntry);
+        BAIL_ON_NT_STATUS(status);
 
-		status = SqliteCreateKeyContext(AccessGranted, pRegEntry, &pKeyCtx);
-		BAIL_ON_NT_STATUS(status);
+        status = SqliteCreateKeyContext(pRegEntry, &pKeyCtx);
+        BAIL_ON_NT_STATUS(status);
 
-		// Cache this new key in gActiveKeyList
-		status = SqliteCacheInsertActiveKey_inlock(pKeyCtx);
-		BAIL_ON_NT_STATUS(status);
-	}
+        // Cache this new key in gActiveKeyList
+        status = SqliteCacheInsertActiveKey_inlock(pKeyCtx);
+        BAIL_ON_NT_STATUS(status);
+    }
 
-	if (pServerState)
-	{
-        if (!pServerState->pToken)
-        {
-	    status = RegSrvCreateAccessToken(pServerState->peerUID,
-			                         pServerState->peerGID,
-			                         &pServerState->pToken);
-            BAIL_ON_NT_STATUS(status);
-        }
+    status = SqliteCreateKeyHandle(pServerState ? pServerState->pToken : NULL,
+                                   AccessDesired,
+                                   pKeyCtx,
+                                   &pKeyHandle);
+    BAIL_ON_NT_STATUS(status);
+    pKeyCtx = NULL;
 
-		status = RegSrvAccessCheckKey(pServerState->pToken,
-								      pKeyCtx->pSecurityDescriptor,
-								      pKeyCtx->ulSecDescLength,
-								      AccessDesired,
-								      &AccessGranted);
-		BAIL_ON_NT_STATUS(status);
-	}
-
-	status = SqliteCreateKeyHandle(AccessGranted, pKeyCtx, &pKeyHandle);
-	BAIL_ON_NT_STATUS(status);
-	pKeyCtx = NULL;
-
-	*ppKeyHandle = pKeyHandle;
+    *ppKeyHandle = pKeyHandle;
 
 cleanup:
-	SqliteReleaseKeyContext_inlock(pKeyCtx);
-	RegDbSafeFreeEntryKey(&pRegEntry);
+    SqliteReleaseKeyContext_inlock(pKeyCtx);
+    RegDbSafeFreeEntryKey(&pRegEntry);
 
-	return status;
+    return status;
 
 error:
-	SqliteSafeFreeKeyHandle_inlock(pKeyHandle);
-	*ppKeyHandle = NULL;
+    SqliteSafeFreeKeyHandle_inlock(pKeyHandle);
+    *ppKeyHandle = NULL;
 
-	goto cleanup;
+    goto cleanup;
+}
+
+VOID
+SqliteCloseKey_inlock(
+    IN HKEY hKey
+    )
+{
+    SqliteSafeFreeKeyHandle_inlock((PREG_KEY_HANDLE)hKey);
 }
 
 NTSTATUS
-SqliteDeleteKeyInternal(
+SqliteDeleteKeyInternal_inlock(
 	IN HANDLE handle,
     IN PCWSTR pwszKeyName
     )
@@ -576,10 +541,13 @@ SqliteDeleteKeyInternal(
     // Do not free
     PREG_KEY_CONTEXT pKeyCtx = NULL;
 
-    status = SqliteOpenKeyInternal(handle,
-		                       pwszKeyName,
-		                       0,
-                                   &pKeyHandle);
+    status = SqliteDeleteActiveKey_inlock((PCWSTR)pwszKeyName);
+    BAIL_ON_NT_STATUS(status);
+
+    status = SqliteOpenKeyInternal_inlock(handle,
+		                              pwszKeyName,
+		                              0,
+                                          &pKeyHandle);
     BAIL_ON_NT_STATUS(status);
 
     BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
@@ -606,7 +574,7 @@ SqliteDeleteKeyInternal(
 
         if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
         {
-		SqliteCacheResetParentKeySubKeyInfo(pwszParentKeyName);
+		SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
         }
     }
     else
@@ -616,7 +584,7 @@ SqliteDeleteKeyInternal(
     }
 
 cleanup:
-    SqliteSafeFreeKeyHandle(pKeyHandle);
+    SqliteSafeFreeKeyHandle_inlock(pKeyHandle);
 
     LWREG_SAFE_FREE_MEMORY(pwszParentKeyName);
 
@@ -626,9 +594,9 @@ error:
     goto cleanup;
 }
 
-// This can be called when a DB lock is already in held
+// This can be called when a DB lock and active key list lock are both held
 NTSTATUS
-SqliteDeleteKeyInternal_inDblock(
+SqliteDeleteKeyInternal_inlock_inDblock(
 	IN HANDLE handle,
     IN PCWSTR pwszKeyName
     )
@@ -640,7 +608,10 @@ SqliteDeleteKeyInternal_inDblock(
     // Do not free
     PREG_KEY_CONTEXT pKeyCtx = NULL;
 
-    status = SqliteOpenKeyInternal_inDblock(handle,
+    status = SqliteDeleteActiveKey_inlock((PCWSTR)pwszKeyName);
+    BAIL_ON_NT_STATUS(status);
+
+    status = SqliteOpenKeyInternal_inlock_inDblock(handle,
 		                                pwszKeyName,
 		                                0,
                                             &pKeyHandle);
@@ -670,7 +641,7 @@ SqliteDeleteKeyInternal_inDblock(
 
         if (!LW_IS_NULL_OR_EMPTY_STR(pwszParentKeyName))
         {
-		SqliteCacheResetParentKeySubKeyInfo(pwszParentKeyName);
+		SqliteCacheResetParentKeySubKeyInfo_inlock(pwszParentKeyName);
         }
     }
     else
@@ -680,7 +651,7 @@ SqliteDeleteKeyInternal_inDblock(
     }
 
 cleanup:
-    SqliteSafeFreeKeyHandle(pKeyHandle);
+    SqliteSafeFreeKeyHandle_inlock(pKeyHandle);
 
     LWREG_SAFE_FREE_MEMORY(pwszParentKeyName);
 
@@ -696,20 +667,14 @@ SqliteDeleteActiveKey(
     )
 {
     NTSTATUS status = STATUS_SUCCESS;
-    PREG_KEY_CONTEXT pFoundKey = NULL;
     BOOLEAN bInLock = FALSE;
 
     LWREG_LOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
 
-    pFoundKey = SqliteCacheLocateActiveKey_inlock(pwszKeyName);
-    if (pFoundKey)
-    {
-	status = STATUS_RESOURCE_IN_USE;
-        BAIL_ON_NT_STATUS(status);
-    }
+    status = SqliteDeleteActiveKey_inlock(pwszKeyName);
+    BAIL_ON_NT_STATUS(status);
 
 cleanup:
-    SqliteReleaseKeyContext_inlock(pFoundKey);
     LWREG_UNLOCK_MUTEX(bInLock, &gActiveKeyList.mutex);
 
     return status;
@@ -718,106 +683,26 @@ error:
     goto cleanup;
 }
 
-/*delete all subkeys and values of hKey*/
 NTSTATUS
-SqliteDeleteTreeInternal_inDblock(
-    IN HANDLE Handle,
-    IN HKEY hKey
+SqliteDeleteActiveKey_inlock(
+    IN PCWSTR pwszKeyName
     )
 {
-	NTSTATUS status = STATUS_SUCCESS;
-    HKEY hCurrKey = NULL;
-    int iCount = 0;
-    DWORD dwSubKeyCount = 0;
-    LW_WCHAR psubKeyName[MAX_KEY_LENGTH];
-    DWORD dwSubKeyLen = 0;
-    PSTR* ppszSubKey = NULL;
-    // Do not free
-    PSTR pszSubKeyName = NULL;
-    PWSTR pwszSubKey = NULL;
-    PREG_KEY_HANDLE pKeyHandle = (PREG_KEY_HANDLE)hKey;
-    PREG_KEY_CONTEXT pKeyCtx = NULL;
+    NTSTATUS status = STATUS_SUCCESS;
+    PREG_KEY_CONTEXT pFoundKey = NULL;
 
-    BAIL_ON_NT_INVALID_POINTER(pKeyHandle);
-    pKeyCtx = pKeyHandle->pKey;
-    BAIL_ON_INVALID_KEY_CONTEXT(pKeyCtx);
-
-    status = RegDbQueryInfoKeyCount_inlock(ghCacheConnection,
-		                               pKeyCtx->qwId,
-                                           QuerySubKeys,
-                                           (size_t*)&dwSubKeyCount);
-    BAIL_ON_NT_STATUS(status);
-
-    if (dwSubKeyCount)
+    pFoundKey = SqliteCacheLocateActiveKey_inlock(pwszKeyName);
+    if (pFoundKey)
     {
-        status = LW_RTL_ALLOCATE((PVOID*)&ppszSubKey, PSTR, sizeof(*ppszSubKey) * dwSubKeyCount);
+        status = STATUS_RESOURCE_IN_USE;
         BAIL_ON_NT_STATUS(status);
     }
 
-    for (iCount = 0; iCount < dwSubKeyCount; iCount++)
-    {
-        dwSubKeyLen = MAX_KEY_LENGTH;
-        memset(psubKeyName, 0, MAX_KEY_LENGTH);
-
-        status = SqliteEnumKeyEx_inDblock(Handle,
-                                  hKey,
-                                  iCount,
-                                  psubKeyName,
-                                  &dwSubKeyLen,
-                                  NULL,
-                                  NULL,
-                                  NULL,
-                                  NULL);
-        BAIL_ON_NT_STATUS(status);
-
-
-	status = LwRtlCStringAllocateFromWC16String(&ppszSubKey[iCount], psubKeyName);
-        BAIL_ON_NT_STATUS(status);
-    }
-
-    for (iCount = 0; iCount < dwSubKeyCount; iCount++)
-    {
-        pszSubKeyName = strrchr(ppszSubKey[iCount], '\\');
-
-	status = LwRtlWC16StringAllocateFromCString(&pwszSubKey,
-			                 pszSubKeyName ? pszSubKeyName+1 : ppszSubKey[iCount]);
-	BAIL_ON_NT_STATUS(status);
-
-        status = SqliteOpenKeyEx_inDblock(Handle,
-                                  hKey,
-                                  pwszSubKey,
-                                  0,
-                                  KEY_ALL_ACCESS,
-                                  &hCurrKey);
-        BAIL_ON_NT_STATUS(status);
-
-        status = SqliteDeleteTreeInternal_inDblock(Handle,
-                                           hCurrKey);
-        BAIL_ON_NT_STATUS(status);
-
-        if (hCurrKey)
-        {
-            SqliteCloseKey(hCurrKey);
-            hCurrKey = NULL;
-        }
-
-        status = SqliteDeleteKey_inDblock(Handle, hKey, pwszSubKey);
-        BAIL_ON_NT_STATUS(status);
-
-        LWREG_SAFE_FREE_MEMORY(pwszSubKey);
-    }
-
 cleanup:
-    if (hCurrKey)
-    {
-        SqliteCloseKey(hCurrKey);
-    }
-    RegFreeStringArray(ppszSubKey, dwSubKeyCount);
-    LWREG_SAFE_FREE_MEMORY(pwszSubKey);
+    SqliteReleaseKeyContext_inlock(pFoundKey);
 
     return status;
 
-
 error:
     goto cleanup;
 }
