@@ -45,6 +45,9 @@
 #include <pwd.h>
 #include "histedit.h"
 
+#if 0
+#define _LW_DEBUG 1
+#endif
 #define REGSHELL_ESC_CHAR '|'
 
 static int gCaughtSignal;
@@ -803,7 +806,7 @@ RegShellProcessCmd(
                 BAIL_ON_REG_ERROR(dwError);
                 pszNewKeyName [strlen(pszNewKeyName) - 1] = '\0';
                 pszKeyName = pszNewKeyName;
-                pszToken = strtok_r(pszKeyName, "\\/", &pszStrtokState);
+                pszToken = strtok_r(pszKeyName, "\\", &pszStrtokState);
                 if (!pszToken)
                 {
                     /*
@@ -872,16 +875,7 @@ RegShellProcessCmd(
                             }
                             else
                             {
-                                pszPwd = strrchr(pszNewDefaultKey,
-                                                 '/');
-                                if (pszPwd)
-                                {
-                                    pszPwd[0] = '\0';
-                                }
-                                else
-                                {
-                                    LWREG_SAFE_FREE_MEMORY(pszNewDefaultKey);
-                                }
+                                LWREG_SAFE_FREE_MEMORY(pszNewDefaultKey);
                             }
                         }
                     }
@@ -959,7 +953,7 @@ RegShellProcessCmd(
                             BAIL_ON_REG_ERROR(dwError);
                         }
                     }
-                    pszToken = strtok_r(pszKeyName, "\\/", &pszStrtokState);
+                    pszToken = strtok_r(pszKeyName, "\\", &pszStrtokState);
                 }
                 if (bChdirOk)
                 {
@@ -2120,7 +2114,6 @@ int main(int argc, char *argv[])
     dwError = RegShellInitParseState(&parseState);
     BAIL_ON_REG_ERROR(dwError);
 
-
     memset(&action, 0, sizeof(action));
     action.sa_handler = pfnRegShellSignal;
     action.sa_flags = SA_RESTART;
