@@ -41,10 +41,7 @@
  *          Adam Bernstein (abernstein@likewise.com)
  *
  */
-#include "rsutils.h"
-#include <../parse/includes.h>
-#include <regclient.h>
-#include <reg/lwreg.h>
+#include "includes.h"
 
 static
 DWORD
@@ -52,6 +49,7 @@ ProcessSubKeys(
     HANDLE hReg,
     FILE* fp,
     HKEY hKey,
+    PCSTR pszFullKeyName,
     DWORD dwNumSubKeys,
     DWORD dwMaxSubKeyLen
     );
@@ -120,9 +118,9 @@ error:
 
 DWORD
 RegExportEntry(
-    PSTR keyName,
+    PCSTR keyName,
     REG_DATA_TYPE valueType,
-    PSTR valueName,
+    PCSTR valueName,
     REG_DATA_TYPE type,
     LW_PVOID value,
     DWORD valueLen,
@@ -182,7 +180,7 @@ RegExportEntry(
 DWORD
 RegExportDword(
     REG_DATA_TYPE valueType,
-    PSTR valueName,
+    PCSTR valueName,
     DWORD value,
     PSTR *dumpString,
     PDWORD dumpStringLen)
@@ -228,7 +226,7 @@ error:
 
 DWORD
 RegExportRegKey(
-    PSTR keyName,
+    PCSTR keyName,
     PSTR *dumpString,
     PDWORD dumpStringLen)
 {
@@ -262,8 +260,8 @@ error:
 
 DWORD RegExportString(
     REG_DATA_TYPE valueType,
-    PSTR valueName,
-    PSTR value,
+    PCSTR valueName,
+    PCSTR value,
     PSTR *dumpString,
     PDWORD retDumpStringLen)
 {
@@ -278,8 +276,6 @@ DWORD RegExportString(
     BAIL_ON_INVALID_POINTER(dumpString);
     BAIL_ON_INVALID_POINTER(retDumpStringLen);
 
-    /*
-     */
     dwError = RegShellUtilEscapeString(
                   value,
                   &valueEscName,
@@ -346,7 +342,7 @@ error:
 DWORD
 RegExportBinaryData(
     REG_DATA_TYPE valueType,
-    PSTR valueName,
+    PCSTR valueName,
     REG_DATA_TYPE type,
     UCHAR *value,
     DWORD valueLen,
@@ -428,9 +424,9 @@ static
 DWORD
 PrintToRegFile(
     IN FILE* fp,
-    IN PSTR pszKeyName,
+    IN PCSTR pszKeyName,
     IN REG_DATA_TYPE dataType,
-    IN PSTR pszValueName,
+    IN PCSTR pszValueName,
     IN REG_DATA_TYPE type,
     IN PVOID value,
     IN DWORD dwValueLen,
@@ -488,22 +484,25 @@ ProcessExportedKeyInfo(
     IN HANDLE hReg,
     IN FILE* fp,
     IN HKEY hKey,
-    IN PSTR pszKeyName,
+    IN PCSTR pszFullKeyName,
     IN OUT PREG_DATA_TYPE pPrevType
     )
 {
     DWORD dwError = 0;
     DWORD dwValueNameLen = MAX_KEY_LENGTH;
-    CHAR valueName[MAX_KEY_LENGTH];   // buffer for subkey name
+    WCHAR pwszValueName[MAX_KEY_LENGTH];   // buffer for subkey name WCHAR format
+    PSTR  pszValueName = NULL; // buffer for subkey name
+    PSTR pszValue = NULL;
     REG_DATA_TYPE dataType = REG_UNKNOWN;
-    BYTE value[MAX_VALUE_LENGTH] = {0};
+    BYTE value[MAX_VALUE_LENGTH * 2] = {0};
     DWORD dwValueLen = 0;
     int iCount = 0;
     DWORD dwValuesCount = 0;
+    PVOID pValue = NULL;
 
     dwError = PrintToRegFile(
                           fp,
-                          pszKeyName,
+                          pszFullKeyName,
                           REG_KEY,
                           NULL,
                           REG_KEY,
@@ -512,7 +511,7 @@ ProcessExportedKeyInfo(
                           pPrevType);
     BAIL_ON_REG_ERROR(dwError);
 
-    dwError = RegQueryInfoKeyA(
+    dwError = RegQueryInfoKeyW(
         hReg,
         hKey,
         NULL,
@@ -535,36 +534,49 @@ ProcessExportedKeyInfo(
 
     for (iCount = 0; iCount < dwValuesCount; iCount++)
    {
-       memset(valueName, 0, MAX_KEY_LENGTH);
+       memset(pwszValueName, 0, MAX_KEY_LENGTH);
        dwValueNameLen = MAX_KEY_LENGTH;
        memset(value, 0, MAX_VALUE_LENGTH);
        dwValueLen = MAX_VALUE_LENGTH;
 
-       dwError = RegEnumValueA((HANDLE)hReg,
+       dwError = RegEnumValueW((HANDLE)hReg,
                                hKey,
                                iCount,
-                               valueName,
+                               pwszValueName,
                                &dwValueNameLen,
                                NULL,
                                &dataType,
                                value,
                                &dwValueLen);
        BAIL_ON_REG_ERROR(dwError);
+       dwError = RegCStringAllocateFromWC16String(&pszValueName, pwszValueName);
+       BAIL_ON_REG_ERROR(dwError);
+
+       if (dataType == REG_SZ)
+       {
+           dwError = RegCStringAllocateFromWC16String(&pszValue, (PWSTR) value);
+           pValue = pszValue;
+       }
+       else
+       {
+           pValue = value;
+       }
 
        dwError = PrintToRegFile(
                       fp,
-                      pszKeyName,
+                      pszFullKeyName,
                       dataType,
-                      valueName,
+                      pszValueName,
                       dataType,
-                      value,
+                      pValue,
                       dwValueLen,
                       pPrevType);
        BAIL_ON_REG_ERROR(dwError);
    }
 
 cleanup:
-    memset(valueName, 0 , MAX_KEY_LENGTH);
+    LWREG_SAFE_FREE_STRING(pszValueName);
+    LWREG_SAFE_FREE_MEMORY(pszValue);
     memset(value, 0 , MAX_KEY_LENGTH);
 
     return dwError;
@@ -580,6 +592,7 @@ ProcessSubKeys(
     HANDLE hReg,
     FILE* fp,
     HKEY hKey,
+    PCSTR pszFullKeyName,
     DWORD dwNumSubKeys,
     DWORD dwMaxSubKeyLen
     )
@@ -587,28 +600,25 @@ ProcessSubKeys(
     DWORD dwError = 0;
     int iCount = 0;
     DWORD dwSubKeyLen = dwMaxSubKeyLen+1;
-    PWSTR subKey = NULL;   // buffer for subkey name
+    PWSTR pwszSubKey = NULL;   // buffer for subkey name
     PSTR pszSubKey = NULL;
+    PSTR pszFullSubKeyName = NULL;
     HKEY hSubKey = NULL;
     DWORD dwNumSubSubKeys = 0;
     DWORD dwMaxSubSubKeyLen = 0;
-    CHAR c = '\\';
-    //Do not free
-    PSTR pszSubKeyName = NULL;
-    PWSTR pSubKey = NULL;
 
     // Get the subkeys and values under this key from registry
     for (iCount = 0; iCount < dwNumSubKeys; iCount++)
     {
 	dwSubKeyLen = dwMaxSubKeyLen+1;
 
-        dwError = RegAllocateMemory(sizeof(*subKey) * dwSubKeyLen, (PVOID*)&subKey);
+        dwError = RegAllocateMemory(sizeof(*pwszSubKey) * dwSubKeyLen, (PVOID*)&pwszSubKey);
         BAIL_ON_REG_ERROR(dwError);
 
         dwError = RegEnumKeyExW((HANDLE)hReg,
                                 hKey,
                                 iCount,
-                                subKey,
+                                pwszSubKey,
                                 &dwSubKeyLen,
                                 NULL,
                                 NULL,
@@ -616,28 +626,10 @@ ProcessSubKeys(
                                 NULL);
         BAIL_ON_REG_ERROR(dwError);
 
-	dwError = RegCStringAllocateFromWC16String(&pszSubKey, subKey);
-        BAIL_ON_REG_ERROR(dwError);
-
-        LWREG_SAFE_FREE_MEMORY(subKey);
-        subKey = NULL;
-
-        pszSubKeyName = strrchr(pszSubKey, c);
-
-        if (!pszSubKeyName || LW_IS_NULL_OR_EMPTY_STR(pszSubKeyName+1))
-        {
-            continue;
-        }
-
-        //Open the subkey
-	dwError = LwRtlWC16StringAllocateFromCString(&pSubKey,
-			                                     pszSubKeyName+1);
-	BAIL_ON_REG_ERROR(dwError);
-
         dwError = RegOpenKeyExW(
             hReg,
             hKey,
-            pSubKey,
+            pwszSubKey,
             0,
             KEY_READ,
             &hSubKey);
@@ -659,11 +651,22 @@ ProcessSubKeys(
             NULL);
         BAIL_ON_REG_ERROR(dwError);
 
+        // Get the pszFullSubKeyName
+	dwError = RegCStringAllocateFromWC16String(&pszSubKey, pwszSubKey);
+        BAIL_ON_REG_ERROR(dwError);
+
+        dwError = RegCStringAllocatePrintf(
+                  &pszFullSubKeyName,
+                  "%s\\%s",
+                  pszFullKeyName,
+                  pszSubKey);
+        BAIL_ON_REG_ERROR(dwError);
+
         dwError = RegShellUtilExport(
                         hReg,
                         fp,
                         hSubKey,
-                        pszSubKey,
+                        pszFullSubKeyName,
                         dwNumSubSubKeys,
                         dwMaxSubSubKeyLen);
         BAIL_ON_REG_ERROR(dwError);
@@ -676,10 +679,14 @@ ProcessSubKeys(
             hSubKey = NULL;
         }
 
+        LWREG_SAFE_FREE_STRING(pszFullSubKeyName);
         LWREG_SAFE_FREE_STRING(pszSubKey);
+        LWREG_SAFE_FREE_MEMORY(pwszSubKey);
         dwNumSubSubKeys = 0;
         dwMaxSubSubKeyLen = 0;
-        LWREG_SAFE_FREE_MEMORY(pSubKey);
+        pszFullSubKeyName = NULL;
+        pszSubKey = NULL;
+        pwszSubKey = NULL;
     }
 
 cleanup:
@@ -689,10 +696,10 @@ cleanup:
         hSubKey = NULL;
     }
 
+    LWREG_SAFE_FREE_STRING(pszFullSubKeyName);
     LWREG_SAFE_FREE_STRING(pszSubKey);
-    LWREG_SAFE_FREE_MEMORY(subKey);
+    LWREG_SAFE_FREE_MEMORY(pwszSubKey);
     dwNumSubKeys = 0;
-    LWREG_SAFE_FREE_MEMORY(pSubKey);
 
     return dwError;
 
@@ -791,7 +798,7 @@ RegShellUtilExport(
     HANDLE hReg,
     FILE* fp,
     HKEY hKey,
-    PSTR pszKeyName,
+    PCSTR pszKeyName,
     DWORD dwNumSubKeys,
     DWORD dwMaxSubKeyLen
     )
@@ -813,6 +820,7 @@ RegShellUtilExport(
         dwError = ProcessSubKeys(hReg,
                                  fp,
                                  hKey,
+                                 pszKeyName,
                                  dwNumSubKeys,
                                  dwMaxSubKeyLen);
         BAIL_ON_REG_ERROR(dwError);
