@@ -41,12 +41,12 @@
  *          Adam Bernstein (abernstein@likewise.com)
  *
  */
-#include "rsutils.h"
+#include "includes.h"
 
 DWORD
 RegShellCanonicalizePath(
-    PSTR pszInDefaultKey,
-    PSTR pszInKeyName,
+    PCSTR pszInDefaultKey,
+    PCSTR pszInKeyName,
     PSTR *ppszFullPath,
     PSTR *ppszParentPath,
     PSTR *ppszSubKey)
@@ -203,8 +203,8 @@ error:
 DWORD
 RegShellIsValidKey(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszKey)
+    PCSTR pszRootKeyName,
+    PCSTR pszKey)
 {
     DWORD dwError = 0;
     HKEY pFullKey = NULL;
@@ -254,9 +254,9 @@ error:
 DWORD
 RegShellUtilAddKeySecDesc(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR pszKeyName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR pszKeyName,
     BOOLEAN bDoBail,
     IN ACCESS_MASK AccessDesired,
     IN OPTIONAL PSECURITY_DESCRIPTOR_ABSOLUTE pSecurityDescriptor
@@ -360,9 +360,9 @@ error:
 DWORD
 RegShellUtilAddKey(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR pszKeyName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR pszKeyName,
     BOOLEAN bDoBail
     )
 {
@@ -380,9 +380,9 @@ RegShellUtilAddKey(
 DWORD
 RegShellUtilDeleteKey(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName)
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName)
 {
     HANDLE hRegLocal = NULL;
     PWSTR pwszSubKey = NULL;
@@ -473,9 +473,9 @@ error:
 DWORD
 RegShellUtilDeleteTree(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName)
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName)
 {
     HANDLE hRegLocal = NULL;
     PWSTR pwszSubKey = NULL;
@@ -561,9 +561,9 @@ error:
 DWORD
 RegShellUtilGetKeys(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
     LW_WCHAR ***pppRetSubKeys,
     PDWORD pdwRetSubKeyCount)
 {
@@ -580,6 +580,7 @@ RegShellUtilGetKeys(
     PSTR pszParentPath = NULL;
     PWSTR pwszSubKey = NULL;
     PSTR pszKeyName = NULL;
+    PWSTR pwszParentKeyName = NULL;
 
 
     if (!hReg)
@@ -620,12 +621,23 @@ RegShellUtilGetKeys(
                       KEY_READ,
                       &pFullKey);
         BAIL_ON_REG_ERROR(dwError);
+
+        dwError = RegWC16StringAllocatePrintfW(
+                        &pwszParentKeyName,
+                        L"%s\\%ws",
+                        pszRootKeyName,
+                        pwszSubKey);
+        BAIL_ON_REG_ERROR(dwError);
+
         LWREG_SAFE_FREE_MEMORY(pwszSubKey);
     }
     else
     {
         pFullKey = pRootKey;
         pRootKey = NULL;
+
+        dwError = RegWC16StringAllocateFromCString(&pwszParentKeyName, pszRootKeyName);
+        BAIL_ON_REG_ERROR(dwError);
     }
 
     dwError = RegQueryInfoKeyA(
@@ -674,8 +686,12 @@ RegShellUtilGetKeys(
                                 NULL);
         BAIL_ON_REG_ERROR(dwError);
 
-	dwError = RegWC16StringAllocateFromCString(&subKeys[i], pszKeyName);
-	BAIL_ON_REG_ERROR(dwError);
+        dwError = RegWC16StringAllocatePrintfW(
+                        &subKeys[i],
+                        L"%ws\\%s",
+                        pwszParentKeyName,
+                        pszKeyName);
+        BAIL_ON_REG_ERROR(dwError);
 
         LWREG_SAFE_FREE_STRING(pszKeyName);
         pszKeyName = NULL;
@@ -698,6 +714,7 @@ cleanup:
     }
     LWREG_SAFE_FREE_STRING(pszParentPath);
     LWREG_SAFE_FREE_STRING(pszKeyName);
+    LWREG_SAFE_FREE_MEMORY(pwszParentKeyName);
 
     return dwError;
 
@@ -713,10 +730,10 @@ error:
 DWORD
 RegShellUtilSetValue(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName,
-    PSTR valueName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
+    PCSTR valueName,
     REG_DATA_TYPE type,
     LW_PVOID data,
     DWORD dataLen
@@ -730,6 +747,7 @@ RegShellUtilSetValue(
     HKEY pRootKey = NULL;
     PSTR pszParentPath = NULL;
     DWORD dwOffset = 0;
+    PBYTE pMultiStr = NULL;
 
     if (!hReg)
     {
@@ -786,23 +804,30 @@ RegShellUtilSetValue(
     switch (type)
     {
         case REG_MULTI_SZ:
+            if (!data)
+            {
+                data = "";
+            }
             dwError = RegMultiStrsToByteArrayA(
-                                        data,
-                                        &pData,
-                                        &dwDataLen);
+                          data,
+                          &pMultiStr,
+                          &dwDataLen);
             BAIL_ON_REG_ERROR(dwError);
+
+            pData = (PBYTE) pMultiStr;
+            dwDataLen = dwDataLen * sizeof(CHAR);
             break;
 
         case REG_SZ:
-
             dwError = RegCStringDuplicate((LW_PVOID) &pData, data?data:"");
             BAIL_ON_REG_ERROR(dwError);
             dwDataLen = strlen((PSTR) pData)+1;
-
             break;
 
         case REG_DWORD:
-            dwError = RegAllocateMemory(sizeof(*pData) * sizeof(DWORD), (PVOID*)&pData);
+            dwError = RegAllocateMemory(
+                          sizeof(*pData) * sizeof(DWORD),
+                          (PVOID*)&pData);
             BAIL_ON_REG_ERROR(dwError);
 
             dwDataLen = (SSIZE_T) dataLen;
@@ -828,10 +853,6 @@ RegShellUtilSetValue(
         pData,
         dwDataLen);
     BAIL_ON_REG_ERROR(dwError);
-    if (type != REG_BINARY)
-    {
-        LWREG_SAFE_FREE_MEMORY(pData);
-    }
 
 cleanup:
     RegCloseServer(hRegLocal);
@@ -843,6 +864,10 @@ cleanup:
     {
         RegCloseKey(hReg, pRootKey);
     }
+    if (type != REG_BINARY)
+    {
+        LWREG_SAFE_FREE_MEMORY(pData);
+    }
     LWREG_SAFE_FREE_STRING(pszParentPath);
     return dwError;
 
@@ -879,9 +904,9 @@ error:
 DWORD
 RegShellUtilGetValues(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
     PREGSHELL_UTIL_VALUE *valueArray,
     PDWORD pdwValueArrayLen
     )
@@ -1054,10 +1079,10 @@ error:
 DWORD
 RegShellUtilDeleteValue(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR keyName,
-    PSTR valueName)
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR keyName,
+    PCSTR valueName)
 {
     HANDLE hRegLocal = NULL;
     HKEY hRootKey = NULL;
@@ -1145,7 +1170,7 @@ RegShellUtilAllocateMemory(
     HANDLE hReg,
     HKEY hKey,
     REG_DATA_TYPE regType,
-    PSTR pszValueName,
+    PCSTR pszValueName,
     PVOID *ppRetBuf,
     PDWORD pdwRetBufLen)
 {
@@ -1194,10 +1219,10 @@ error:
 DWORD
 RegShellUtilGetValue(
     HANDLE hReg,
-    PSTR pszRootKeyName,
-    PSTR pszDefaultKey,
-    PSTR pszKeyName,
-    PSTR pszValueName,
+    PCSTR pszRootKeyName,
+    PCSTR pszDefaultKey,
+    PCSTR pszKeyName,
+    PCSTR pszValueName,
     PREG_DATA_TYPE pRegType,
     PVOID *ppValue,
     PDWORD pdwValueLen)
@@ -1420,7 +1445,7 @@ error:
 
 DWORD
 RegShellUtilEscapeString(
-    PSTR pszValue,
+    PCSTR pszValue,
     PSTR *ppszRetValue,
     PDWORD pdwEscapeValueLen)
 {
