@@ -48,6 +48,7 @@ static struct
     LW_SM_LOG_LEVEL logLevel;
     PCSTR pszLogFilePath;
     BOOLEAN bSyslog;
+    BOOLEAN bDisableAutostart;
 } gState = 
 {
     .pIpcContext = NULL,
@@ -57,7 +58,8 @@ static struct
     .notifyPipe = {-1, -1},
     .logLevel = LW_SM_LOG_LEVEL_WARNING,
     .pszLogFilePath = NULL,
-    .bSyslog = FALSE
+    .bSyslog = FALSE,
+    .bDisableAutostart = FALSE
 };
 
 static
@@ -168,6 +170,13 @@ main(
     dwError = LwSmPopulateTable();
     BAIL_ON_ERROR(dwError);
 
+    /* Start the 'Autostart' services */
+    if (!gState.bDisableAutostart)
+    {
+        dwError = LwSmAutostartServices();
+        BAIL_ON_ERROR(dwError);
+    }
+
     /* Start IPC server */
     dwError = LwSmStartIpcServer();
     BAIL_ON_ERROR(dwError);
@@ -258,6 +267,10 @@ LwSmParseArguments(
 
             gState.pszLogFilePath = ppszArgv[i];
         }
+        else if (!strcmp(ppszArgv[i], "--disable-autostart"))
+        {
+            gState.bDisableAutostart = TRUE;
+        }
     }
 
 error:
@@ -624,7 +637,7 @@ LwSmLogIpc (
         break;
     }
 
-    if (LwSmGetMaxLogLevel() >= level)
+    if (LwSmGetMaxLogLevel() >= smLevel)
     {
         if (pszMessage)
         {
