@@ -175,8 +175,8 @@ RegComp(
     
     if (dwError)
     {
-        dwError = EINVAL;
-        BAIL_ON_UP_ERROR(EINVAL);
+        dwError = LwMapErrnoToLwError(EINVAL);
+        BAIL_ON_UP_ERROR(dwError);
     }
 
     *pbCompiled = TRUE;
