@@ -50,11 +50,12 @@
 #include "config.h"
 #include "lwnet-system.h"
 #include "lwnet-def.h"
+#include <lwstr.h>
+#include <lw/atomic.h>
 
 #include "lwnet.h"
 
 #include "lwnet-utils.h"
 #include "lwnet-ipc.h"
-#include "lwnet-client.h"
 
 #include "ipc_client_p.h"
