@@ -95,6 +95,8 @@
   #include <arpa/nameser.h>
   #include <netinet/in.h>
   #include <resolv.h>
+  #include <locale.h>
+  #include <poll.h>
 
 #ifdef HAVE_SOCKET_H
   #include <socket.h>
@@ -110,11 +112,8 @@
 #include <wc16str.h>
 #endif
 
-#if defined(__hpux__) && defined(_XOPEN_SOURCE_EXTENDED)
-#    include "xpg_socket.h"
-#endif
-
 #include <uuid/uuid.h>
+#include <lw/winerror.h>
 
 /*
 local variables:
