@@ -156,7 +156,7 @@ LWNetCacheDbUpdate(
     IN LWNET_CACHE_DB_HANDLE DbHandle,
     IN PCSTR pszDnsDomainName,
     IN OPTIONAL PCSTR pszSiteName,
-    IN DWORD dwDsFlags,
+    IN LWNET_CACHE_DB_QUERY_TYPE QueryType,
     IN LWNET_UNIX_TIME_T LastDiscovered,
     IN LWNET_UNIX_TIME_T LastPinged,
     IN BOOLEAN IsBackoffToWritableDc,
@@ -1167,7 +1167,7 @@ LWNetCacheDbUpdate(
     IN LWNET_CACHE_DB_HANDLE DbHandle,
     IN PCSTR pszDnsDomainName,
     IN OPTIONAL PCSTR pszSiteName,
-    IN DWORD dwDsFlags,
+    IN LWNET_CACHE_DB_QUERY_TYPE QueryType,
     IN LWNET_UNIX_TIME_T LastDiscovered,
     IN LWNET_UNIX_TIME_T LastPinged,
     IN BOOLEAN IsBackoffToWritableDc,
@@ -1180,7 +1180,6 @@ LWNetCacheDbUpdate(
     PLWNET_CACHE_DB_ENTRY pOldEntry = NULL;
     PDLINKEDLIST pOldListEntry = NULL;
     BOOLEAN isAcquired = FALSE;
-    LWNET_CACHE_DB_QUERY_TYPE QueryType = LWNetCacheDbQueryToQueryType(dwDsFlags);
 
     dwError = LWNetAllocateMemory(sizeof(*pNewEntry), (PVOID *)&pNewEntry);
     BAIL_ON_LWNET_ERROR(dwError);
@@ -1552,12 +1551,13 @@ LWNetCacheUpdate(
     )
 {
     DWORD dwError = 0;
+    LWNET_CACHE_DB_QUERY_TYPE QueryType = LWNetCacheDbQueryToQueryType(dwDsFlags);
 
     dwError = LWNetCacheDbUpdate(
                     gDbHandle,
                     pszDnsDomainName,
                     pszSiteName,
-                    dwDsFlags,
+                    QueryType,
                     LastDiscovered,
                     LastPinged,
                     IsBackoffToWritableDc,
@@ -1572,7 +1572,7 @@ LWNetCacheUpdate(
                         gDbHandle,
                         pszDnsDomainName,
                         pDcInfo->pszDCSiteName,
-                        dwDsFlags,
+                        QueryType,
                         LastDiscovered,
                         LastPinged,
                         IsBackoffToWritableDc,
