@@ -49,55 +49,49 @@
 #ifndef __LWNET_SERVER_CFG_H__
 #define __LWNET_SERVER_CFG_H__
 
-DWORD
-LWNetSrvParseConfigFile(
-    PCSTR pszFilePath
-    );
 
 DWORD
-LWNetSrvCfgStartSection(
-    PCSTR pszSectionName,
-    PVOID pData,
-    PBOOLEAN pbSkipSection,
-    PBOOLEAN pbContinue
-    );
+LWNetSrvReadRegistry(
+   );
 
-DWORD
-LWNetSrvCfgNameValuePair(
-    PCSTR pszName,
-    PCSTR pszValue,
-    PVOID pData,
-    PBOOLEAN pbContinue
-    );
-
-DWORD
-LWNetSetCacheReaperTimeoutSecs(
-    DWORD dwCacheReaperTimeoutSecs
-    );
-
-DWORD 
-LWNetGetCacheReaperTimeoutSecs(
+PCSTR
+LWNetConfigGetPluginPath(
     VOID
     );
 
 DWORD
-LWNetSetCacheEntryExpirySeconds(
-    DWORD dwExpirySecs
-    ); 
-
-DWORD
-LWNetGetCacheEntryExpirySeconds(
+LWNetConfigGetPingAgainTimeoutSeconds(
     VOID
     );
 
 DWORD
-LWNetSetCurrentSiteName(
-    PCSTR pszCurrentSiteName
+LWNetConfigGetNegativeCacheTimeoutSeconds(
+    VOID
     );
 
 DWORD
-LWNetGetCurrentSiteName(
-        PSTR* ppszCurrentSiteName
-        );
+LWNetConfigGetWritableRediscoveryTimeoutSeconds(
+    VOID
+    );
+
+DWORD
+LWNetConfigGetWritableTimestampMinimumChangeSeconds(
+    VOID
+    );
+
+DWORD
+LWNetConfigGetCLdapMaximumConnections(
+    VOID
+    );
+
+DWORD
+LWNetConfigGetCLdapSearchTimeoutSeconds(
+    VOID
+    );
+
+DWORD
+LWNetConfigGetCLdapSingleConnectionTimeoutSeconds(
+    VOID
+    );
 
 #endif /* __LWNET_SERVER_CFG_H__ */
