@@ -55,9 +55,10 @@ main(
     )
 {
     DWORD dwError = 0;
-    pthread_t listenerThreadId;
-    pthread_t* pListenerThreadId = NULL;
-    void* threadResult = NULL;
+    PCSTR pszSmNotify = NULL;
+    int notifyFd = -1;
+    char notifyCode = 0;
+    int ret = 0;
 
     dwError = LWNetSrvSetDefaults();
     BAIL_ON_LWNET_ERROR(dwError);
@@ -70,34 +71,65 @@ main(
 
     LWNET_LOG_VERBOSE("Logging started");
 
-    if (atexit(LWNetSrvExitHandler) < 0)
-    {
-       dwError = errno;
-       BAIL_ON_LWNET_ERROR(dwError);
-    }
-
     if (LWNetSrvShouldStartAsDaemon())
     {
        dwError = LWNetSrvStartAsDaemon();
        BAIL_ON_LWNET_ERROR(dwError);
     }
 
+    if (atexit(LWNetSrvExitHandler) < 0)
+    {
+       dwError = LwMapErrnoToLwError(errno);
+       BAIL_ON_LWNET_ERROR(dwError);
+    }
+
     // Test system to see if dependent configuration tasks are completed prior to starting our process.
     dwError = LWNetStartupPreCheck();
     BAIL_ON_LWNET_ERROR(dwError);
 
+#ifdef ENABLE_PIDFILE
     // ISSUE-2008/07/03-dalmeida -- Should return/check for errors
     LWNetSrvCreatePIDFile();
+#endif
 
     dwError = LWNetBlockSelectedSignals();
     BAIL_ON_LWNET_ERROR(dwError);
 
+    dwError = LwDsCacheAddPidException(getpid());
+    if (dwError == LW_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK)
+    {
+        LWNET_LOG_ERROR("Could not register process pid (%d) with Mac DirectoryService Cache plugin", (int) getpid());
+        BAIL_ON_LWNET_ERROR(dwError);
+    }
+
     dwError = LWNetSrvInitialize();
     BAIL_ON_LWNET_ERROR(dwError);
 
-    dwError = LWNetSrvStartListenThread(&listenerThreadId, &pListenerThreadId);
+    dwError = LWNetSrvStartListenThread();
     BAIL_ON_LWNET_ERROR(dwError);
 
+    if ((pszSmNotify = getenv("LIKEWISE_SM_NOTIFY")) != NULL)
+    {
+        notifyFd = atoi(pszSmNotify);
+
+        do
+        {
+            ret = write(notifyFd, &notifyCode, sizeof(notifyCode));
+        } while(ret != sizeof(notifyCode) && errno == EINTR);
+
+        if (ret < 0)
+        {
+            LWNET_LOG_ERROR("Could not notify service manager: %s (%i)", strerror(errno), errno);
+            dwError = LwMapErrnoToLwError(errno);
+            BAIL_ON_LWNET_ERROR(dwError);
+        }
+
+        close(notifyFd);
+    }
+
+    // Post service started event to eventlog
+    LWNetSrvLogProcessStartedEvent();
+
     // Handle signals, blocking until we are supposed to exit.
     dwError = LWNetSrvHandleSignals();
     BAIL_ON_LWNET_ERROR(dwError);
@@ -106,12 +138,12 @@ main(
 
     LWNET_LOG_VERBOSE("LWNet main cleaning up");
 
-    LWNetSrvStopProcess();
+    // Post service stopped event to eventlog
+    LWNetSrvLogProcessStoppedEvent(dwError);
 
-    if (pListenerThreadId)
-    {
-        pthread_join(listenerThreadId, &threadResult);
-    }
+    LWNetSrvStopListenThread();
+
+    LwDsCacheRemovePidException(getpid());
 
     LWNetSrvApiShutdown();
 
@@ -127,6 +159,9 @@ main(
 
     LWNET_LOG_ERROR("LWNET Process exiting due to error [Code:%d]", dwError);
 
+    // Post service failed event to eventlog
+    LWNetSrvLogProcessFailureEvent(dwError);
+
     goto cleanup;
 }
 
@@ -161,15 +196,11 @@ LWNetStartupPreCheck(
 
     if (iter >= STARTUP_PRE_CHECK_WAIT)
     {
-        dwError = LWNET_ERROR_FAILED_STARTUP_PREREQUISITE_CHECK;
+        dwError = ERROR_SERVICE_DEPENDENCY_FAIL;
         LWNET_LOG_ERROR("LWNet start up pre-check failed to get updated hostname after 2 minutes of waiting [Code:%d]", dwError);
         BAIL_ON_LWNET_ERROR(dwError);
     }
 
-    // Now that we are running, we need to flush the DirectoryService process of any negative cache entries
-    dwError = FlushDirectoryServiceCache();
-    BAIL_ON_LWNET_ERROR(dwError);
-
 cleanup:
 
     LWNET_SAFE_FREE_STRING(pszHostname);
@@ -186,60 +217,6 @@ error:
 #endif
 }
 
-#if defined (__LWI_DARWIN__)
-DWORD
-FlushDirectoryServiceCache(
-    VOID
-    )
-{
-    DWORD dwError = 0;
-    int i;
-    const char* cacheUtils[] = {
-        "/usr/sbin/lookupd", /* Before Mac OS X 10.5 */
-        "/usr/bin/dscacheutil" /* On Mac OS X 10.5 */
-    };
-    const char* cacheUtilCmd[] = {
-        "/usr/sbin/lookupd -flushcache", /* Before Mac OS X 10.5 */
-        "/usr/bin/dscacheutil -flushcache" /* On Mac OS X 10.5 */
-    };
-
-    LWNET_LOG_VERBOSE("Going to flush the Mac DirectoryService cache ...");
-
-    for (i = 0; i < (sizeof(cacheUtils) / sizeof(cacheUtils[0])); i++)
-    {
-        const char* util = cacheUtils[i];
-        const char* command = cacheUtilCmd[i];
-        BOOLEAN exists;
-
-        /* Sanity check */
-        if (!util)
-        {
-            continue;
-        }
-
-        dwError = LWNetCheckFileExists(util, &exists);
-        BAIL_ON_LWNET_ERROR(dwError);
-
-        if (!exists)
-        {
-            continue;
-        }
-
-        system(command);
-
-        /* Bail regardless */
-        goto error;
-    }
-
-    LWNET_LOG_ERROR("Could not locate cache flush utility");
-    dwError = LWNET_ERROR_MAC_FLUSH_DS_CACHE_FAILED;
-
-error:
-
-    return dwError;
-}
-#endif
-
 DWORD
 LWNetSrvSetDefaults(
     VOID
@@ -247,13 +224,15 @@ LWNetSrvSetDefaults(
 {
     DWORD dwError = 0;
 
-    gpServerInfo->dwLogLevel = LOG_LEVEL_ERROR;
+    gpServerInfo->dwLogLevel = LWNET_LOG_LEVEL_ERROR;
 
     *(gpServerInfo->szLogFilePath) = '\0';
 
     strcpy(gpServerInfo->szCachePath, LWNET_CACHE_DIR);
     strcpy(gpServerInfo->szPrefixPath, PREFIXDIR);
 
+    setlocale(LC_ALL, "");
+
     return (dwError);
 }
 
@@ -292,7 +271,11 @@ LWNetSrvParseArgs(
             }
             // ISSUE-2008/07/03-dalmeida -- not safe
             pArg = argv[++iArg];
-            strcpy(pLWNetServerInfo->szLogFilePath, pArg + 1);
+            strcpy(pLWNetServerInfo->szLogFilePath, pArg);
+        }
+        else if (strcmp(pArg, "--syslog") == 0)
+        {
+            pLWNetServerInfo->bLogToSyslog = TRUE;
         }
         else if (strcmp(pArg, "--loglevel") == 0)
         {
@@ -305,24 +288,29 @@ LWNetSrvParseArgs(
             pArg = argv[++iArg];
             if (!strcasecmp(pArg, "error"))
             {
-                pLWNetServerInfo->dwLogLevel = LOG_LEVEL_ERROR;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_ERROR;
             }
             else if (!strcasecmp(pArg, "warning"))
             {
-                pLWNetServerInfo->dwLogLevel = LOG_LEVEL_WARNING;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_WARNING;
             }
             else if (!strcasecmp(pArg, "info"))
             {
-                pLWNetServerInfo->dwLogLevel = LOG_LEVEL_INFO;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_INFO;
             }
             else if (!strcasecmp(pArg, "verbose"))
             {
-                pLWNetServerInfo->dwLogLevel = LOG_LEVEL_VERBOSE;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_VERBOSE;
             }
             else if (!strcasecmp(pArg, "debug"))
             {
                 pLWNetServerInfo->bEnableDebugLogs = TRUE;
-                pLWNetServerInfo->dwLogLevel = LOG_LEVEL_DEBUG;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_DEBUG;
+            }
+            else if (!strcasecmp(pArg, "trace"))
+            {
+                pLWNetServerInfo->bEnableDebugLogs = TRUE;
+                pLWNetServerInfo->dwLogLevel = LWNET_LOG_LEVEL_TRACE;
             }
             else
             {
@@ -384,6 +372,7 @@ ShowUsage(
            "  Options:\n"
            "\n"
            "    --start-as-daemon         start in daemon mode\n"
+           "    --syslog                  log to syslog\n"
            "    --logfile <logFilePath>   log to specified file\n"
            "    --loglevel <logLevel>     log at the specified detail level, which\n"
            "                              can be one of:\n"
@@ -408,11 +397,14 @@ LWNetSrvExitHandler(
 
     sprintf(szErrCodeFilePath, "%s/LWNetsd.err", pszCachePath);
 
-    dwError = LWNetCheckFileExists(szErrCodeFilePath, &bFileExists);
+    dwError = LwCheckFileTypeExists(
+                    szErrCodeFilePath,
+                    LWFILE_REGULAR,
+                    &bFileExists);
     BAIL_ON_LWNET_ERROR(dwError);
 
     if (bFileExists) {
-        dwError = LWNetRemoveFile(szErrCodeFilePath);
+        dwError = LwRemoveFile(szErrCodeFilePath);
         BAIL_ON_LWNET_ERROR(dwError);
     }
 
@@ -422,8 +414,8 @@ LWNetSrvExitHandler(
     if (dwExitCode) {
        fp = fopen(szErrCodeFilePath, "w");
        if (fp == NULL) {
-          dwError = errno;
-          BAIL_ON_LWNET_ERROR(dwError);
+            dwError = LwMapErrnoToLwError(errno);
+            BAIL_ON_LWNET_ERROR(dwError);
        }
        fprintf(fp, "%d\n", dwExitCode);
     }
@@ -443,9 +435,8 @@ LWNetSrvInitialize(
     )
 {
     DWORD dwError = 0;
-    PCSTR pszConfigFilePath = LWNET_CONFIG_DIR "/netlogon.conf";
     
-    dwError = LWNetSrvApiInit(pszConfigFilePath);
+    dwError = LWNetSrvApiInit();
     BAIL_ON_LWNET_ERROR(dwError);
     
 cleanup:
@@ -498,7 +489,7 @@ LWNetSrvStartAsDaemon(
     // this signal, we are ensuring that our second child will
     // ignore this signal and will continue execution.
     if (signal(SIGHUP, SIG_IGN) < 0) {
-        dwError = errno;
+        dwError = LwMapErrnoToLwError(errno);
         BAIL_ON_LWNET_ERROR(dwError);
     }
 
@@ -584,7 +575,7 @@ LWNetSrvGetCachePath(
     LWNET_LOCK_SERVERINFO(bInLock);
     
     if (IsNullOrEmptyString(gpServerInfo->szCachePath)) {
-      dwError = LWNET_ERROR_INVALID_CACHE_PATH;
+      dwError = ERROR_PATH_NOT_FOUND;
       BAIL_ON_LWNET_ERROR(dwError);
     }
     
@@ -620,7 +611,7 @@ LWNetSrvGetPrefixPath(
     LWNET_LOCK_SERVERINFO(bInLock);
     
     if (IsNullOrEmptyString(gpServerInfo->szPrefixPath)) {
-      dwError = LWNET_ERROR_INVALID_PREFIX_PATH;
+      dwError = ERROR_PATH_NOT_FOUND;
       BAIL_ON_LWNET_ERROR(dwError);
     }
     
@@ -745,7 +736,10 @@ LWNetSrvInitLogging(
 
     LWNET_LOCK_SERVERINFO(bInLock);
 
-    if (gpServerInfo->dwStartAsDaemon) {
+    if ((gpServerInfo->dwStartAsDaemon &&
+            gpServerInfo->szLogFilePath[0] == '\0') ||
+            gpServerInfo->bLogToSyslog)
+    {
       
       dwError = lwnet_init_logging_to_syslog(gpServerInfo->dwLogLevel,
                                            gpServerInfo->bEnableDebugLogs,
@@ -841,3 +835,170 @@ LWNetSrvSetProcessToExit(
 
     LWNET_UNLOCK_SERVERINFO(bInLock);
 }
+
+VOID
+LWNetSrvLogProcessStartedEvent(
+    VOID
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszDescription = NULL;
+
+    dwError = LwAllocateStringPrintf(
+                 &pszDescription,
+                 "The Likewise site manager service was started.");
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    LWNetSrvLogInformationEvent(
+            LWNET_EVENT_INFO_SERVICE_STARTED,
+            SERVICE_EVENT_CATEGORY,
+            pszDescription,
+            NULL);
+
+cleanup:
+
+    LWNET_SAFE_FREE_STRING(pszDescription);
+
+    return;
+
+error:
+
+    goto cleanup;
+}
+
+VOID
+LWNetSrvLogProcessStoppedEvent(
+    DWORD dwExitCode
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszDescription = NULL;
+    PSTR pszData = NULL;
+
+    dwError = LwAllocateStringPrintf(
+                 &pszDescription,
+                 "The Likewise site manager service was stopped");
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    dwError = LWNetGetErrorMessageForLoggingEvent(
+                         dwExitCode,
+                         &pszData);
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    if (dwExitCode)
+    {
+        LWNetSrvLogErrorEvent(
+                LWNET_EVENT_ERROR_SERVICE_STOPPED,
+                SERVICE_EVENT_CATEGORY,
+                pszDescription,
+                pszData);
+    }
+    else
+    {
+        LWNetSrvLogInformationEvent(
+                LWNET_EVENT_INFO_SERVICE_STOPPED,
+                SERVICE_EVENT_CATEGORY,
+                pszDescription,
+                pszData);
+    }
+
+cleanup:
+
+    LWNET_SAFE_FREE_STRING(pszDescription);
+    LWNET_SAFE_FREE_STRING(pszData);
+
+    return;
+
+error:
+
+    goto cleanup;
+}
+
+VOID
+LWNetSrvLogProcessFailureEvent(
+    DWORD dwErrCode
+    )
+{
+    DWORD dwError = 0;
+    PSTR pszDescription = NULL;
+    PSTR pszData = NULL;
+
+    dwError = LwAllocateStringPrintf(
+                 &pszDescription,
+                 "The Likewise site manager service stopped running due to an error");
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    dwError = LWNetGetErrorMessageForLoggingEvent(
+                         dwErrCode,
+                         &pszData);
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    LWNetSrvLogErrorEvent(
+            LWNET_EVENT_ERROR_SERVICE_START_FAILURE,
+            SERVICE_EVENT_CATEGORY,
+            pszDescription,
+            pszData);
+
+cleanup:
+
+    LWNET_SAFE_FREE_STRING(pszDescription);
+    LWNET_SAFE_FREE_STRING(pszData);
+
+    return;
+
+error:
+
+    goto cleanup;
+}
+
+DWORD
+LWNetGetErrorMessageForLoggingEvent(
+    DWORD dwErrCode,
+    PSTR* ppszErrorMsg
+    )
+{
+    DWORD dwErrorBufferSize = 0;
+    DWORD dwError = 0;
+    DWORD dwLen = 0;
+    PSTR  pszErrorMsg = NULL;
+    PSTR  pszErrorBuffer = NULL;
+
+    dwErrorBufferSize = LwGetErrorString(dwErrCode, NULL, 0);
+
+    if (!dwErrorBufferSize)
+        goto cleanup;
+
+    dwError = LWNetAllocateMemory(
+                dwErrorBufferSize,
+                (PVOID*)&pszErrorBuffer);
+    BAIL_ON_LWNET_ERROR(dwError);
+
+    dwLen = LwGetErrorString(dwErrCode, pszErrorBuffer, dwErrorBufferSize);
+
+    if ((dwLen == dwErrorBufferSize) && !IsNullOrEmptyString(pszErrorBuffer))
+    {
+        dwError = LwAllocateStringPrintf(
+                     &pszErrorMsg,
+                     "Error: %s [error code: %d]",
+                     pszErrorBuffer,
+                     dwErrCode);
+        BAIL_ON_LWNET_ERROR(dwError);
+    }
+
+    *ppszErrorMsg = pszErrorMsg;
+
+cleanup:
+
+    LWNET_SAFE_FREE_STRING(pszErrorBuffer);
+
+    return dwError;
+
+error:
+
+    LWNET_SAFE_FREE_STRING(pszErrorMsg);
+
+    *ppszErrorMsg = NULL;
+
+    goto cleanup;
+}
+
