@@ -48,7 +48,10 @@
 #include "lwnet-system.h"
 #include "lwnet-def.h"
 #include "lwnet.h"
-#include "lwnet-client.h"
+#include "lwnet-utils.h"
+#include "lwerror.h"
+
+#define LW_PRINTF_STRING(x) ((x) ? (x) : "<null>")
 
 static
 void
@@ -58,7 +61,7 @@ ShowUsage()
     printf("        [--force] [--ds-required] [--gc-required]\n");
     printf("        [--pdc-required] [--background-only] [--kdc-required]\n");
     printf("        [--timeserv-required] [--writeable-required] [--good-timeserv-required]\n");
-    printf("        [--avoid-self]\n\n"); 
+    printf("        [--avoid-self] [--preferred-domain <domain name>]\n\n");
 }
 
 DWORD
@@ -73,12 +76,12 @@ AddFlag(
     if(dwFlags & dwFlag)
     {
         printf("Duplicate flag entered: [0x%.8X]\n", dwFlag);
-        dwError = LWNET_ERROR_INVALID_PARAMETER;
+        dwError = ERROR_INVALID_PARAMETER;
     }
     else if(!(dwFlag & LWNET_SUPPORTED_DS_INPUT_FLAGS))
     {
         printf("Invalid flag entered: [0x%.8X]\n", dwFlag);
-        dwError = LWNET_ERROR_INVALID_PARAMETER;
+        dwError = ERROR_INVALID_PARAMETER;
     }
     BAIL_ON_LWNET_ERROR(dwError);
     
@@ -98,12 +101,14 @@ ParseArgs(
     char*  argv[],
     PSTR*  ppszTargetFQDN,
     PSTR*  ppszSiteName,
+    PSTR*  ppszPrimaryDomain,
     PDWORD pdwFlags
     )
 {
     typedef enum {
             PARSE_MODE_OPEN = 0,
             PARSE_MODE_SITENAME,
+            PARSE_MODE_PREFERRED_DOMAIN,
             PARSE_MODE_OPTIONS
         } ParseMode;
 
@@ -113,6 +118,7 @@ ParseArgs(
     ParseMode parseMode = PARSE_MODE_OPEN;
     PSTR pszTargetFQDN = NULL;
     PSTR pszSiteName = NULL;
+    PSTR pszPrimaryDomain = NULL;
     DWORD dwFlags = 0;
 
     do {
@@ -146,6 +152,10 @@ ParseArgs(
                 {
                     parseMode = PARSE_MODE_SITENAME;
                 }
+                else if(strcmp(pszArg, "--preferred-domain") == 0)
+                {
+                    parseMode = PARSE_MODE_PREFERRED_DOMAIN;
+                }
                 else if(strcmp(pszArg, "--force") == 0)
                 {
                     dwError = AddFlag(DS_FORCE_REDISCOVERY, &dwFlags);
@@ -199,7 +209,7 @@ ParseArgs(
                 else 
                 {
                     LWNET_LOG_ERROR("Invalid argument: %s", pszArg);
-                    dwError = LWNET_ERROR_INVALID_PARAMETER;
+                    dwError = ERROR_INVALID_PARAMETER;
                     BAIL_ON_LWNET_ERROR(dwError);
                 }
                 break;
@@ -208,13 +218,27 @@ ParseArgs(
                 if(!IsNullOrEmptyString(pszSiteName))
                 {
                     LWNET_LOG_ERROR("Invalid argument: %s", pszArg);
-                    dwError = LWNET_ERROR_INVALID_PARAMETER;
+                    dwError = ERROR_INVALID_PARAMETER;
                     BAIL_ON_LWNET_ERROR(dwError);
                 }
                 
                 dwError = LWNetAllocateString(pszArg, &pszSiteName);
                 BAIL_ON_LWNET_ERROR(dwError);
                 
+                parseMode = PARSE_MODE_OPTIONS;
+                break;
+            case PARSE_MODE_PREFERRED_DOMAIN:
+
+                if(!IsNullOrEmptyString(pszPrimaryDomain))
+                {
+                    LWNET_LOG_ERROR("Invalid argument: %s", pszArg);
+                    dwError = ERROR_INVALID_PARAMETER;
+                    BAIL_ON_LWNET_ERROR(dwError);
+                }
+
+                dwError = LWNetAllocateString(pszArg, &pszPrimaryDomain);
+                BAIL_ON_LWNET_ERROR(dwError);
+
                 parseMode = PARSE_MODE_OPTIONS;
                 break;
         }
@@ -233,11 +257,13 @@ error:
     {
         LWNET_SAFE_FREE_STRING(pszTargetFQDN);
         LWNET_SAFE_FREE_STRING(pszSiteName);
+        LWNET_SAFE_FREE_STRING(pszPrimaryDomain);
         dwFlags = 0;
     }
 
     *ppszTargetFQDN = pszTargetFQDN;
     *ppszSiteName = pszSiteName;
+    *ppszPrimaryDomain = pszPrimaryDomain;
     *pdwFlags = dwFlags;
 
     return dwError;
@@ -277,6 +303,7 @@ main(
 
     PSTR pszTargetFQDN = NULL;
     PSTR pszSiteName = NULL;
+    PSTR pszPrimaryDomain = NULL;
     PLWNET_DC_INFO pDCInfo = NULL;
     DWORD dwFlags = 0;
     CHAR szErrorBuf[1024];
@@ -288,17 +315,21 @@ main(
                 argv,
                 &pszTargetFQDN,
                 &pszSiteName,
+                &pszPrimaryDomain,
                 &dwFlags
                 );
     BAIL_ON_LWNET_ERROR(dwError);
 
-    lwnet_init_logging_to_file(LOG_LEVEL_VERBOSE, TRUE, "");
+    lwnet_init_logging_to_file(LWNET_LOG_LEVEL_VERBOSE, TRUE, "");
 
-    dwError = LWNetGetDCName(
+    dwError = LWNetGetDCNameExt(
                 NULL,
                 pszTargetFQDN,
                 pszSiteName,
+                pszPrimaryDomain,
                 dwFlags,
+                0,
+                NULL,
                 &pDCInfo
                 );
     BAIL_ON_LWNET_ERROR(dwError); 
@@ -340,9 +371,27 @@ main(
 error:
     if (dwError)
     {
-        LWNetGetErrorString(dwError, szErrorBuf, 1024);
-        LWNET_LOG_ERROR("Error: [%s] [code=%u]", szErrorBuf, dwError);
+        DWORD dwLen = LwGetErrorString(dwError, szErrorBuf, 1024);
+
+        if (dwLen)
+        {
+            fprintf(
+                 stderr,
+                 "Failed communication with the LWNET Agent.  Error code %u (%s).\n%s\n",
+                 dwError,
+                 LW_PRINTF_STRING(LwWin32ExtErrorToName(dwError)),
+                 szErrorBuf);
+        }
+        else
+        {
+            fprintf(
+                 stderr,
+                 "Failed communication with the LWNET Agent.  Error code %u (%s).\n",
+                 dwError,
+                 LW_PRINTF_STRING(LwWin32ExtErrorToName(dwError)));
+        }
     }
+
     LWNET_SAFE_FREE_DC_INFO(pDCInfo);
     LWNET_SAFE_FREE_STRING(pszTargetFQDN);
     LWNET_SAFE_FREE_STRING(pszSiteName);
