@@ -52,11 +52,24 @@ static const PSTR WARN_TAG    = "WARNING";
 static const PSTR INFO_TAG    = "INFO";
 static const PSTR VERBOSE_TAG = "VERBOSE";
 static const PSTR DEBUG_TAG   = "DEBUG";
+static const PSTR TRACE_TAG   = "TRACE";
 
 static const PSTR LOG_TIME_FORMAT = "%Y%m%d%H%M%S";
 
-#define LWNET_LOCK_LOGGER   pthread_mutex_lock(&gLwnetLogInfo.lock)
-#define LWNET_UNLOCK_LOGGER pthread_mutex_unlock(&gLwnetLogInfo.lock)
+#define LWNET_LOCK_LOGGER(bInLock) \
+    do { \
+        pthread_mutex_lock(&gLwnetLogInfo.lock); \
+        bInLock = TRUE; \
+    } while (0)
+
+#define LWNET_UNLOCK_LOGGER(bInLock) \
+    do { \
+        if (bInLock) \
+        { \
+            pthread_mutex_unlock(&gLwnetLogInfo.lock); \
+            bInLock = FALSE; \
+        } \
+    } while (0)
 
 DWORD
 lwnet_validate_log_level(
@@ -65,11 +78,22 @@ lwnet_validate_log_level(
 {
     DWORD dwError = 0;
 
-    if (dwLogLevel < 1) {
-        dwError = EINVAL;
+    switch (dwLogLevel)
+    {
+        case LWNET_LOG_LEVEL_ERROR:
+        case LWNET_LOG_LEVEL_WARNING:
+        case LWNET_LOG_LEVEL_INFO:
+        case LWNET_LOG_LEVEL_VERBOSE:
+        case LWNET_LOG_LEVEL_DEBUG:
+        case LWNET_LOG_LEVEL_TRACE:
+            dwError = 0;
+            break;
+        default:
+            dwError = ERROR_INVALID_PARAMETER;
+            break;
     }
 
-    return (dwError);
+    return dwError;
 }
 
 void
@@ -77,40 +101,9 @@ lwnet_set_syslogmask(
     DWORD dwLogLevel
     )
 {
-    DWORD dwSysLogLevel;
+    DWORD dwSysLogLevel = 0;
 
-    switch (dwLogLevel)
-    {
-        case LOG_LEVEL_ALWAYS:
-        {
-            dwSysLogLevel = LOG_UPTO(LOG_INFO);
-            break;
-        }
-        case LOG_LEVEL_ERROR:
-        {
-            dwSysLogLevel = LOG_UPTO(LOG_ERR);
-            break;
-        }
-
-        case LOG_LEVEL_WARNING:
-        {
-            dwSysLogLevel = LOG_UPTO(LOG_WARNING);
-            break;
-        }
-
-        case LOG_LEVEL_INFO:
-        case LOG_LEVEL_VERBOSE:
-        {
-            dwSysLogLevel = LOG_UPTO(LOG_INFO);
-            break;
-        }
-
-        default:
-        {
-            dwSysLogLevel = LOG_UPTO(LOG_DEBUG);
-            break;
-        }
-    }
+    dwSysLogLevel = LOG_UPTO(LOG_DEBUG);
 
     setlogmask(dwSysLogLevel);
 }
@@ -125,13 +118,19 @@ lwnet_init_logging_to_syslog(
     )
 {
     DWORD dwError = 0;
-    
-    LWNET_LOCK_LOGGER;
+    BOOLEAN bInLock = FALSE;
 
     dwError = lwnet_validate_log_level(dwLogLevel);
+    if (dwError)
+    {
+        // Since logging is not yet initialized.
+        fprintf(stderr, "An invalid log level [%d] was specified.", dwLogLevel);
+    }
     BAIL_ON_LWNET_ERROR(dwError);
 
-    gLwnetLogInfo.logTarget = LOG_TO_SYSLOG;
+    LWNET_LOCK_LOGGER(bInLock);
+
+    gLwnetLogInfo.logTarget = LWNET_LOG_TARGET_SYSLOG;
     
     gLwnetLogInfo.bDebug = bEnableDebug;
 
@@ -148,15 +147,10 @@ lwnet_init_logging_to_syslog(
 
     gLwnetLogInfo.bLoggingInitiated = 1;
 
-    LWNET_UNLOCK_LOGGER;
+error:
+    LWNET_UNLOCK_LOGGER(bInLock);
 
-    return (dwError);
-
-  error:
-
-    LWNET_UNLOCK_LOGGER;
-    
-    return (dwError);
+    return dwError;
 }
 
 DWORD
@@ -167,58 +161,63 @@ lwnet_init_logging_to_file(
     )
 {
     DWORD dwError = 0;
+    BOOLEAN bInLock = FALSE;
 
-    LWNET_LOCK_LOGGER;
-
-    gLwnetLogInfo.logTarget = LOG_TO_FILE;
-    gLwnetLogInfo.bDebug = bEnableDebug;
-    
-    if (IsNullOrEmptyString(pszLogFilePath)) {
-
-       gLwnetLogInfo.bLogToConsole = TRUE;
-       gLwnetLogInfo.logfile.logHandle = stdout;
-
-    } else {
-
-       strncpy(gLwnetLogInfo.logfile.szLogPath, pszLogFilePath, PATH_MAX);
-       *(gLwnetLogInfo.logfile.szLogPath+PATH_MAX) = '\0';
-    
-       gLwnetLogInfo.logfile.logHandle = NULL;
-       if (gLwnetLogInfo.logfile.szLogPath[0] != '\0') {
-          gLwnetLogInfo.logfile.logHandle = freopen(gLwnetLogInfo.logfile.szLogPath, "w", stdout);
-          if (gLwnetLogInfo.logfile.logHandle == NULL) {
-             dwError = errno;
-             fprintf(stderr, "Failed to redirect logging. %s", strerror(errno));
-             goto error;
-          }
-       }
-    }
-  
     dwError = lwnet_validate_log_level(dwLogLevel);
-    if (dwError) {
-       fprintf(stderr, "An invalid log level [%d] was specified.", dwLogLevel);
-       goto error;
+    if (dwError)
+    {
+        // Since logging is not yet initialized.
+        fprintf(stderr, "An invalid log level [%d] was specified.", dwLogLevel);
+        goto error;
     }
 
+    LWNET_LOCK_LOGGER(bInLock);
+
+    gLwnetLogInfo.logTarget = LWNET_LOG_TARGET_FILE;
+    gLwnetLogInfo.bDebug = bEnableDebug;
+
+    if (IsNullOrEmptyString(pszLogFilePath))
+    {
+        gLwnetLogInfo.bLogToConsole = TRUE;
+        gLwnetLogInfo.logfile.logHandle = stdout;
+    }
+    else
+    {
+        strncpy(gLwnetLogInfo.logfile.szLogPath, pszLogFilePath, PATH_MAX);
+        *(gLwnetLogInfo.logfile.szLogPath+PATH_MAX) = '\0';
+
+        gLwnetLogInfo.logfile.logHandle = NULL;
+        if (gLwnetLogInfo.logfile.szLogPath[0] != '\0')
+        {
+            gLwnetLogInfo.logfile.logHandle = fopen(gLwnetLogInfo.logfile.szLogPath, "w");
+            if (gLwnetLogInfo.logfile.logHandle == NULL)
+            {
+                dwError = LwMapErrnoToLwError(errno);
+                fprintf(stderr, "Failed to redirect logging. %s", strerror(errno));
+                goto error;
+            }
+        }
+    }
+
+
     gLwnetLogInfo.dwLogLevel = dwLogLevel;
 
     gLwnetLogInfo.bLoggingInitiated = 1;
 
-    LWNET_UNLOCK_LOGGER;
+cleanup:
+    LWNET_UNLOCK_LOGGER(bInLock);
 
-    return (dwError);
-
-  error:
+    return dwError;
 
+error:
     if (!gLwnetLogInfo.bLogToConsole &&
-        (gLwnetLogInfo.logfile.logHandle != NULL)) {
+        (gLwnetLogInfo.logfile.logHandle != NULL))
+    {
         fclose(gLwnetLogInfo.logfile.logHandle);
         gLwnetLogInfo.logfile.logHandle = NULL;
     }
 
-    LWNET_UNLOCK_LOGGER;
-
-    return (dwError);
+    goto cleanup;
 }
 
 void
@@ -226,25 +225,28 @@ lwnet_close_log(
     void
     )
 {
-    LWNET_LOCK_LOGGER;
+    BOOLEAN bInLock = FALSE;
 
-    if (!gLwnetLogInfo.bLoggingInitiated) {
-        LWNET_UNLOCK_LOGGER;
-        return;
+    LWNET_LOCK_LOGGER(bInLock);
+
+    if (!gLwnetLogInfo.bLoggingInitiated)
+    {
+        goto cleanup;
     }
 
     switch (gLwnetLogInfo.logTarget)
     {
-        case LOG_TO_SYSLOG:
+        case LWNET_LOG_TARGET_SYSLOG:
             {
                 /* close connection to syslog */
                 closelog();
             }
             break;
-        case LOG_TO_FILE:
+        case LWNET_LOG_TARGET_FILE:
             {
                 if (!gLwnetLogInfo.bLogToConsole &&
-                    (gLwnetLogInfo.logfile.logHandle != NULL)) {
+                    (gLwnetLogInfo.logfile.logHandle != NULL))
+                {
                     fclose(gLwnetLogInfo.logfile.logHandle);
                     gLwnetLogInfo.logfile.logHandle = NULL;
                 }
@@ -252,11 +254,38 @@ lwnet_close_log(
             break;
     }
 
-    LWNET_UNLOCK_LOGGER;
+cleanup:
+    LWNET_UNLOCK_LOGGER(bInLock);
 }
 
-void
+static PSTR logLevel2EntryType(
+    DWORD dwLogLevel
+    )
+{
+    switch (dwLogLevel)
+    {
+        case LWNET_LOG_LEVEL_ALWAYS:
+            return INFO_TAG;
+        case LWNET_LOG_LEVEL_ERROR:
+            return(ERROR_TAG);
+        case LWNET_LOG_LEVEL_WARNING:
+            return(WARN_TAG);
+        case LWNET_LOG_LEVEL_INFO:
+            return(INFO_TAG);
+        case LWNET_LOG_LEVEL_VERBOSE:
+            return(VERBOSE_TAG);
+        case LWNET_LOG_LEVEL_DEBUG:
+            return(DEBUG_TAG);
+        case LWNET_LOG_LEVEL_TRACE:
+            return(TRACE_TAG);
+    }
+
+    return "UNKNOWN";
+}
+
+static void
 lwnet_log_to_file_mt_unsafe(
+    LOGINFO *pgLwnetLogInfo,
     DWORD dwLogLevel,
     PSTR pszFormat,
     va_list msgList
@@ -266,58 +295,26 @@ lwnet_log_to_file_mt_unsafe(
     time_t currentTime;
     struct tm tmp;
     char timeBuf[1024];
-    FILE* pTarget = NULL;
+    FILE* pTarget = pgLwnetLogInfo->logfile.logHandle;
 
     switch (dwLogLevel)
     {
-        case LOG_LEVEL_ALWAYS:
-        {
-            pszEntryType = INFO_TAG;
-            pTarget = stdout;
+        case LWNET_LOG_LEVEL_ERROR:
+        case LWNET_LOG_LEVEL_WARNING:
+            pTarget = pTarget ? pTarget : stderr;
             break;
-        }
-        case LOG_LEVEL_ERROR:
-        {
-            pszEntryType = ERROR_TAG;
-            pTarget = stderr;
-            break;
-        }
-
-        case LOG_LEVEL_WARNING:
-        {
-            pszEntryType = WARN_TAG;
-            pTarget = stderr;
-            break;
-        }
-
-        case LOG_LEVEL_INFO:
-        {
-            pszEntryType = INFO_TAG;
-            pTarget = stdout;
-            break;
-        }
-
-        case LOG_LEVEL_VERBOSE:
-        {
-            pszEntryType = VERBOSE_TAG;
-            pTarget = stdout;
-            break;
-        }
-
         default:
-        {
-            pszEntryType = DEBUG_TAG;
-            pTarget = stdout;
+            pTarget = pTarget ? pTarget : stdout;
             break;
-        }
     }
 
+    pszEntryType = logLevel2EntryType(dwLogLevel);
     currentTime = time(NULL);
     localtime_r(&currentTime, &tmp);
 
     strftime(timeBuf, sizeof(timeBuf), LOG_TIME_FORMAT, &tmp);
 
-    fprintf(pTarget, "%s:0x%x:%s:", timeBuf, (unsigned int)pthread_self(), pszEntryType);
+    fprintf(pTarget, "%s:0x%lx:%s:", timeBuf, (unsigned long) pthread_self(), pszEntryType);
     vfprintf(pTarget, pszFormat, msgList);
     fprintf(pTarget, "\n");
     fflush(pTarget);
@@ -332,25 +329,25 @@ lwnet_log_to_syslog_mt_unsafe(
 {
     switch (dwLogLevel)
     {
-        case LOG_LEVEL_ALWAYS:
+        case LWNET_LOG_LEVEL_ALWAYS:
         {
             lwnet_vsyslog(LOG_INFO, pszFormat, msgList);
             break;
         }
-        case LOG_LEVEL_ERROR:
+        case LWNET_LOG_LEVEL_ERROR:
         {
             lwnet_vsyslog(LOG_ERR, pszFormat, msgList);
             break;
         }
 
-        case LOG_LEVEL_WARNING:
+        case LWNET_LOG_LEVEL_WARNING:
         {
             lwnet_vsyslog(LOG_WARNING, pszFormat, msgList);
             break;
         }
 
-        case LOG_LEVEL_INFO:
-        case LOG_LEVEL_VERBOSE:
+        case LWNET_LOG_LEVEL_INFO:
+        case LWNET_LOG_LEVEL_VERBOSE:
         {
             lwnet_vsyslog(LOG_INFO, pszFormat, msgList);
             break;
@@ -370,40 +367,42 @@ lwnet_log_message(
     PSTR pszFormat,...
     )
 {
+    BOOLEAN bInLock = FALSE;
     va_list argp;
   
-    LWNET_LOCK_LOGGER;
+    LWNET_LOCK_LOGGER(bInLock);
 
     if ( !gLwnetLogInfo.bLoggingInitiated ||
-         gLwnetLogInfo.logTarget == LOG_DISABLED ) {
-        LWNET_UNLOCK_LOGGER;
-        return;
+         gLwnetLogInfo.logTarget == LWNET_LOG_TARGET_DISABLED )
+    {
+        goto cleanup;
     }
 
-    if (gLwnetLogInfo.dwLogLevel < dwLogLevel) {
-        LWNET_UNLOCK_LOGGER;
-        return;
+    if (gLwnetLogInfo.dwLogLevel < dwLogLevel)
+    {
+        goto cleanup;
     }
 
     va_start(argp, pszFormat);
 
     switch (gLwnetLogInfo.logTarget)
     {
-        case LOG_TO_SYSLOG:
+        case LWNET_LOG_TARGET_SYSLOG:
         {
             lwnet_log_to_syslog_mt_unsafe(dwLogLevel, pszFormat, argp);
             break;
         }
-        case LOG_TO_FILE:
+        case LWNET_LOG_TARGET_FILE:
         {
-            lwnet_log_to_file_mt_unsafe(dwLogLevel, pszFormat, argp);
+            lwnet_log_to_file_mt_unsafe(&gLwnetLogInfo, dwLogLevel, pszFormat, argp);
             break;
         }
     }
 
     va_end(argp);
 
-    LWNET_UNLOCK_LOGGER;
+cleanup:
+    LWNET_UNLOCK_LOGGER(bInLock);
 }
 
 DWORD
@@ -412,22 +411,71 @@ lwnet_set_log_level(
     )
 {
     DWORD dwError = 0;
-
-    LWNET_LOCK_LOGGER;
+    BOOLEAN bInLock = FALSE;
 
     dwError = lwnet_validate_log_level(dwLogLevel);
     BAIL_ON_LWNET_ERROR(dwError);
 
+    LWNET_LOCK_LOGGER(bInLock);
     gLwnetLogInfo.dwLogLevel = dwLogLevel;
 
-    LWNET_UNLOCK_LOGGER;
-
-    return (dwError);
-    
 error:
+    LWNET_UNLOCK_LOGGER(bInLock);
 
-    LWNET_UNLOCK_LOGGER;
+    return dwError;
+}
 
-    return (dwError);
+DWORD
+lwnet_get_log_info(
+    OUT PDWORD pdwLogLevel,
+    OUT PDWORD pdwLogTarget,
+    OUT PSTR* ppszLogPath
+    )
+{
+    DWORD dwError = 0;
+    BOOLEAN bInLock = FALSE;
+    DWORD dwLogLevel = 0;
+    DWORD dwLogTarget = 0;
+    PSTR pszLogPath = NULL;
+
+    LWNET_LOCK_LOGGER(bInLock);
+
+    dwLogLevel = gLwnetLogInfo.dwLogLevel;
+    dwLogTarget = gLwnetLogInfo.logTarget;
+
+    if (LWNET_LOG_TARGET_FILE == dwLogTarget)
+    {
+        if (gLwnetLogInfo.bLogToConsole)
+        {
+            dwLogTarget = LWNET_LOG_TARGET_CONSOLE;
+        }
+        else
+        {
+            dwError = LWNetAllocateString(
+                            gLwnetLogInfo.logfile.szLogPath,
+                            &pszLogPath);
+            // cannot use bail macro because it logs and we are in the lock.
+            if (dwError)
+            {
+                goto error;
+            }
+        }
+    }
+
+error:
+    LWNET_UNLOCK_LOGGER(bInLock);
+
+    if (dwError)
+    {
+        dwLogLevel = 0;
+        dwLogTarget = 0;
+        LWNET_SAFE_FREE_STRING(pszLogPath);
+    }
+
+    *pdwLogLevel = dwLogLevel;
+    *pdwLogTarget = dwLogTarget;
+    *ppszLogPath = pszLogPath;
+
+    return dwError;
 }
 
