@@ -48,202 +48,11 @@
 
 #ifndef _WIN32
 
-#if HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
-
-#ifndef true
-#define true 1
-#endif
-
-#ifndef false
-#define false 0
-#endif
-
-#ifndef DWORD_DEFINED
-#define DWORD_DEFINED 1
-
-typedef uint32_t        DWORD, *PDWORD;
+#include <lw/types.h>
+#include <lw/attrs.h>
 
 #endif
 
-#ifndef INT_DEFINED
-#define INT_DEFINED 1
-
-typedef int             INT, *PINT;
-
-#endif
-
-#ifndef UINT64_DEFINED
-#define UINT64_DEFINED 1
-
-typedef uint64_t        UINT64, *PUINT64;
-
-#endif
-
-#ifndef UINT32_DEFINED
-#define UINT32_DEFINED 1
-
-typedef uint32_t        UINT32, *PUINT32;
-
-#endif
-
-#ifndef UINT16_DEFINED
-#define UINT16_DEFINED 1
-
-typedef uint16_t        UINT16, *PUINT16;
-
-#endif
-
-#ifndef WORD_DEFINED
-#define WORD_DEFINED 1
-
-typedef uint16_t WORD, *PWORD;
-
-#endif
-
-#ifndef USHORT_DEFINED
-#define USHORT_DEFINED 1
-
-typedef unsigned short  USHORT, *PUSHORT;
-
-#endif
-
-#ifndef ULONG_DEFINED
-#define ULONG_DEFINED 1
-
-typedef unsigned long   ULONG, *PULONG;
-
-#endif
-
-#ifndef ULONGLONG_DEFINED
-#define ULONGLONG_DEFINED 1
-
-typedef unsigned long long ULONGLONG, *PULONGLONG;
-
-#endif
-
-#ifndef UINT8_DEFINED
-#define UINT8_DEFINED 1
-
-typedef uint8_t         UINT8, *PUINT8;
-
-#endif
-
-#ifndef BYTE_DEFINED
-#define BYTE_DEFINED
-
-typedef uint8_t BYTE, *PBYTE;
-
-#endif
-
-#ifndef UCHAR_DEFINED
-#define UCHAR_DEFINED 1
-
-typedef uint8_t UCHAR, *PUCHAR;
-
-#endif
-
-#ifndef HANDLE_DEFINED
-#define HANDLE_DEFINED 1
-
-typedef unsigned long   HANDLE, *PHANDLE;
-
-#endif
-
-#ifndef CHAR_DEFINED
-#define CHAR_DEFINED 1
-
-typedef char            CHAR;
-
-#endif
-
-#ifndef PSTR_DEFINED
-#define PSTR_DEFINED 1
-
-typedef char *          PSTR;
-
-#endif
-
-#ifndef PCSTR_DEFINED
-#define PCSTR_DEFINED 1
-
-typedef const char *    PCSTR;
-
-#endif
-
-#ifndef VOID_DEFINED
-#define VOID_DEFINED 1
-
-typedef void            VOID, *PVOID;
-
-#endif
-
-#ifndef PCVOID_DEFINED
-#define PCVOID_DEFINED 1
-
-typedef const void      *PCVOID;
-
-#endif
-
-#ifndef BOOLEAN_DEFINED
-#define BOOLEAN_DEFINED 1
-
-typedef int             BOOLEAN, *PBOOLEAN;
-
-#endif
-
-#ifndef TRUE
-#define TRUE  1
-#endif
-
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#if HAVE_WCHAR16_T
-
-#ifndef PWSTR_DEFINED
-#define PWSTR_DEFINED 1
-
-typedef wchar16_t * PWSTR;
-
-#endif
-
-#ifndef PCWSTR_DEFINED
-typedef const wchar16_t * PCWSTR;
-#define PCWSTR_DEFINED 1
-#endif
-
-#endif // HAVE_WCHAR16_T
-
-#ifndef OPTIONAL
-#define OPTIONAL
-#endif
-
-#ifndef IN
-#define IN
-#endif
-
-#ifndef OUT
-#define OUT
-#endif
-
-#ifndef DWORD_MAX
-#ifdef UINT_MAX
-#define DWORD_MAX UINT_MAX
-#else
-#define DWORD_MAX 4294967295U
-#endif /* UINT_MAX */
-#endif /* DWORD_MAX */
-
-#ifndef UINT32_MAX
-#define UINT32_MAX UINT_MAX
-#endif
-
-#endif
-
-
 /* ERRORS */
 #define LWPS_ERROR_SUCCESS                   0x0000
 #define LWPS_ERROR_INVALID_CACHE_PATH        0x4000 // 16384
@@ -271,7 +80,9 @@ typedef const wchar16_t * PCWSTR;
 #define LWPS_ERROR_INVALID_ACCOUNT           0x4016 // 16406
 #define LWPS_ERROR_INVALID_HANDLE            0x4017 // 16407
 #define LWPS_ERROR_DB_RECORD_NOT_FOUND       0x4018 // 16408
-#define LWPS_ERROR_SENTINEL                  0x4019 // 16409
+#define LWPS_ERROR_INVALID_MESSAGE           0x4019 // 16409
+#define LWPS_ERROR_ACCESS_DENIED             0x401A // 16410
+#define LWPS_ERROR_SENTINEL                  0x401B // 16411
 
 #define LWPS_ERROR_MASK(_e_)             (_e_ & 0x4000)
 
@@ -281,6 +92,7 @@ typedef struct __LWPS_PASSWORD_INFO
     wchar16_t* pwszDnsDomainName;
     wchar16_t* pwszSID;
     wchar16_t* pwszHostname;
+    wchar16_t* pwszHostDnsDomain;
     wchar16_t* pwszMachineAccount;
     wchar16_t* pwszMachinePassword;
     time_t     last_change_time;
@@ -293,7 +105,9 @@ typedef enum
     LWPS_PASSWORD_STORE_UNKNOWN = 0,
     LWPS_PASSWORD_STORE_DEFAULT,
     LWPS_PASSWORD_STORE_SQLDB,
-    LWPS_PASSWORD_STORE_TDB
+    LWPS_PASSWORD_STORE_TDB,
+    LWPS_PASSWORD_STORE_FILEDB,
+    LWPS_PASSWORD_STORE_REGDB
 } LwpsPasswordStoreType;
 
 DWORD
@@ -309,6 +123,12 @@ LwpsGetPasswordByHostName(
     PLWPS_PASSWORD_INFO* ppInfo
     );
 
+DWORD
+LwpsGetPasswordByCurrentHostName(
+    HANDLE hStore,
+    PLWPS_PASSWORD_INFO* ppInfo
+    );
+
 DWORD
 LwpsGetPasswordByDomainName(
     HANDLE hStore,
@@ -316,13 +136,28 @@ LwpsGetPasswordByDomainName(
     PLWPS_PASSWORD_INFO* ppInfo
     );
 
+DWORD
+LwpsGetHostListByDomainName(
+    HANDLE hStore,
+    PCSTR  pszDomainName,
+    PSTR **pppszHostnames,
+    DWORD *pdwNumEntries
+    );
+
 DWORD
 LwpsWritePasswordToAllStores(
     PLWPS_PASSWORD_INFO pInfo
     );
 
 DWORD
-LwpsDeleteEntriesInAllStores();
+LwpsDeleteEntriesInAllStores(
+    VOID
+    );
+
+DWORD
+LwpsDeleteHostInAllStores(
+    PCSTR pszDomainName
+    );
 
 VOID
 LwpsFreePasswordInfo(
@@ -330,6 +165,12 @@ LwpsFreePasswordInfo(
     PLWPS_PASSWORD_INFO pInfo
     );
 
+VOID
+LwpsFreeHostList(
+    PSTR *ppszHostnames,
+    DWORD dwNumEntries
+    );
+
 DWORD
 LwpsClosePasswordStore(
     HANDLE hStore
@@ -347,5 +188,30 @@ LwpsGetErrorString(
     size_t stBufSize
     );
 
-#endif /* __LWPS_H__ */
+typedef enum
+{
+    LWPS_LOG_LEVEL_ALWAYS = 0,
+    LWPS_LOG_LEVEL_ERROR,
+    LWPS_LOG_LEVEL_WARNING,
+    LWPS_LOG_LEVEL_INFO,
+    LWPS_LOG_LEVEL_VERBOSE,
+    LWPS_LOG_LEVEL_DEBUG
+} LwpsLogLevel;
 
+typedef VOID (*PLWPS_LOG_CALLBACK)(LwpsLogLevel level, PVOID pUserData, PCSTR pszMessage);
+
+DWORD
+LwpsSetLogFunction(
+    IN LwpsLogLevel maxLevel,
+    IN PLWPS_LOG_CALLBACK pCallback,
+    IN PVOID pUserData
+    );
+
+DWORD
+LwpsLogMessage(
+    IN LwpsLogLevel level,
+    IN PCSTR pszFormat,
+    ...
+    );
+
+#endif /* __LWPS_H__ */
