@@ -65,10 +65,6 @@ LWPS_INITIALIZE_PROVIDER(regdb)(
     dwError = LwNtStatusToWin32Error(LwMapSecurityInitialize());
     BAIL_ON_LWPS_ERROR(dwError);
 
-    dwError = LwNtStatusToWin32Error(
-                  LwMapSecurityCreateContext(&gpRegLwMapSecurityCtx));
-    BAIL_ON_LWPS_ERROR(dwError);
-
 cleanup:
     return dwError;
 
@@ -102,8 +98,12 @@ RegDB_OpenProvider(
 
     pContext->hReg = hReg;
 
+    dwError = LwNtStatusToWin32Error(
+                  LwMapSecurityCreateContext(&pContext->pRegLwMapSecurityCtx));
+    BAIL_ON_LWPS_ERROR(dwError);
+
     dwError = LwpsCreateRWLock(
-                  LWPS_CONFIG_DIR "/" LWPS_LOCK_FILE,
+                  LWPS_LOCK_DIR "/" LWPS_LOCK_FILE,
                   &pContext->hRWLock);
     BAIL_ON_LWPS_ERROR(dwError);
 
@@ -213,7 +213,10 @@ RegDB_ReadPassword(
                   NULL,
                   (PVOID) &pszHostDnsDomain,
                   NULL);
-    BAIL_ON_LWPS_ERROR(dwError);
+    if (dwError != LWREG_ERROR_NO_SUCH_KEY_OR_VALUE)
+    {
+        BAIL_ON_LWPS_ERROR(dwError);
+    }
 
     dwError = RegUtilGetValue(
                   pContext->hReg,
@@ -296,8 +299,16 @@ RegDB_ReadPassword(
     dwError = LwpsMbsToWc16s(pszDomainSID, &pInfo->pwszSID);
     BAIL_ON_LWPS_ERROR(dwError);
 
-    dwError = LwpsMbsToWc16s(pszHostDnsDomain, &pInfo->pwszHostDnsDomain);
-    BAIL_ON_LWPS_ERROR(dwError);
+    if (IsNullOrEmptyString(pszHostDnsDomain))
+    {
+        dwError = LwpsMbsToWc16s(pszDomainDnsName, &pInfo->pwszHostDnsDomain);
+        BAIL_ON_LWPS_ERROR(dwError);
+    }
+    else
+    {
+        dwError = LwpsMbsToWc16s(pszHostDnsDomain, &pInfo->pwszHostDnsDomain);
+        BAIL_ON_LWPS_ERROR(dwError);
+    }
 
     dwError = LwpsMbsToWc16s(pszHostNameValue, &pInfo->pwszHostname);
     BAIL_ON_LWPS_ERROR(dwError);
@@ -601,7 +612,9 @@ RegDB_WritePassword(
      * stored under this key.
      */
     dwError = LwNtStatusToWin32Error(
-                  RegDB_CreateRestrictedSecDescAbs(&pSecDescAbs));
+                  RegDB_CreateRestrictedSecDescAbs(
+                      pContext->pRegLwMapSecurityCtx,
+                      &pSecDescAbs));
     BAIL_ON_LWPS_ERROR(status);
 
     dwError = RegUtilAddKeySecDesc(
@@ -844,7 +857,6 @@ LWPS_SHUTDOWN_PROVIDER(regdb)(
 
     BAIL_IF_NOT_SUPERUSER(geteuid());
 
-    LwMapSecurityFreeContext(&gpRegLwMapSecurityCtx);
     LwMapSecurityCleanup();
 
 cleanup:
@@ -865,5 +877,6 @@ LwpsFreeProviderContext(
         LwpsFreeRWLock(pContext->hRWLock);
     }
     RegCloseServer(pContext->hReg);
+    LwMapSecurityFreeContext(&pContext->pRegLwMapSecurityCtx);
     LwpsFreeMemory(pContext);
 }
