@@ -37,12 +37,14 @@
 #include "config.h"
 #include "srvsvcsys.h"
 
+#include <lw/base.h>
 #include <lw/security-api.h>
-#include <srvsvc/srvsvc.h>
+#include <lw/srvsvc.h>
 #include <lsa/lsa.h>
 #include <lwerror.h>
 #include <lwmem.h>
 #include <lwstr.h>
+#include <lwio/io-types.h>
 #include <stdio.h>
 #include <sys/param.h>
 #include <srvsvcdefs.h>
@@ -141,6 +143,7 @@ Usage(
 {
     printf(
         "Usage: lw-share [ <options> ... ] enum\n"
+        "       lw-share [ <options> ... ] enum-info\n"
         "       lw-share [ <options> ... ] add [ <add options> ... ] <share>\n"
         "       lw-share [ <options> ... ] del <share>\n"
         "       lw-share [ <options> ... ] get-info <share>\n"
@@ -846,6 +849,95 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+DumpShareInfo(
+    PSHARE_INFO_502 pShareInfo
+    )
+{
+    DWORD dwError = 0;
+    DWORD dwAllowUserCount = 0;
+    PWSTR* ppwszAllowUsers = NULL;
+    DWORD dwDenyUserCount = 0;
+    PWSTR* ppwszDenyUsers = NULL;
+    BOOLEAN bReadOnly = FALSE;
+    PSTR pszUserName = NULL;
+    DWORD dwIndex = 0;
+
+    dwError = PrintStringAttribute("name", pShareInfo->shi502_netname);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    dwError = PrintStringAttribute("path", pShareInfo->shi502_path);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    dwError = PrintStringAttribute("comment", pShareInfo->shi502_remark);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (pShareInfo->shi502_security_descriptor)
+    {
+        dwError = DeconstructSecurityDescriptor(
+            pShareInfo->shi502_reserved,
+            (PSECURITY_DESCRIPTOR_RELATIVE) pShareInfo->shi502_security_descriptor,
+            &dwAllowUserCount,
+            &ppwszAllowUsers,
+            &dwDenyUserCount,
+            &ppwszDenyUsers,
+            &bReadOnly);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+
+        if (dwAllowUserCount)
+        {
+            printf("allow=");
+
+            for (dwIndex = 0; dwIndex < dwAllowUserCount; dwIndex++)
+            {
+                dwError = LwWc16sToMbs(ppwszAllowUsers[dwIndex], &pszUserName);
+                BAIL_ON_SRVSVC_ERROR(dwError);
+
+                printf("%s", pszUserName);
+                if (dwIndex < dwAllowUserCount - 1)
+                {
+                    printf(",");
+                }
+            }
+
+            printf("\n");
+        }
+
+        if (dwDenyUserCount)
+        {
+            printf("deny=");
+
+            for (dwIndex = 0; dwIndex < dwDenyUserCount; dwIndex++)
+            {
+                dwError = LwWc16sToMbs(ppwszDenyUsers[dwIndex], &pszUserName);
+                BAIL_ON_SRVSVC_ERROR(dwError);
+
+                printf("%s", pszUserName);
+                if (dwIndex < dwDenyUserCount - 1)
+                {
+                    printf(",");
+                }
+            }
+
+            printf("\n");
+        }
+
+        printf("read_only=%s\n", bReadOnly ? "true" : "false");
+    }
+
+cleanup:
+
+    FreeStringArray(dwAllowUserCount, ppwszAllowUsers);
+    FreeStringArray(dwDenyUserCount, ppwszDenyUsers);
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
 static
 DWORD
 Enum(
@@ -913,6 +1005,78 @@ error:
     goto cleanup;
 }
 
+static
+DWORD
+EnumInfo(
+    int argc,
+    char** ppszArgv
+    )
+{
+    static const DWORD dwLevel = 502;
+    static const DWORD dwMaxLen = 128;
+
+    DWORD dwError = 0;
+    PSHARE_INFO_502 pShareInfo = NULL;
+    DWORD dwNumEntries = 0;
+    DWORD dwTotalEntries = 0;
+    DWORD dwSeenEntries = 0;
+    DWORD dwResume = 0;
+    DWORD dwIndex = 0;
+    PSTR pszShareName = NULL;
+
+    do
+    {
+        dwError = NetShareEnum(
+            gState.pwszServerName,
+            dwLevel,
+            OUT_PPVOID(&pShareInfo),
+            dwMaxLen,
+            &dwNumEntries,
+            &dwTotalEntries,
+            &dwResume);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+
+        dwSeenEntries += dwNumEntries;
+
+        for (dwIndex = 0; dwIndex < dwNumEntries; dwIndex++)
+        {
+            dwError = LwWc16sToMbs(pShareInfo[dwIndex].shi502_netname, &pszShareName);
+            BAIL_ON_SRVSVC_ERROR(dwError);
+
+            printf("[%s]\n", pszShareName);
+
+            dwError = DumpShareInfo(&pShareInfo[dwIndex]);
+            BAIL_ON_SRVSVC_ERROR(dwError);
+
+            printf("\n");
+
+            LW_SAFE_FREE_STRING(pszShareName);
+        }
+
+        if (pShareInfo)
+        {
+            SrvSvcFreeMemory(pShareInfo);
+            pShareInfo = NULL;
+        }
+    } while (dwSeenEntries < dwTotalEntries);
+
+cleanup:
+
+    LW_SAFE_FREE_STRING(pszShareName);
+
+    if (pShareInfo)
+    {
+        SrvSvcFreeMemory(pShareInfo);
+        pShareInfo = NULL;
+    }
+
+    return dwError;
+
+error:
+
+    goto cleanup;
+}
+
 static
 DWORD
 GetInfo(
@@ -924,13 +1088,6 @@ GetInfo(
     DWORD dwError = 0;
     PSHARE_INFO_502 pShareInfo = NULL;
     PWSTR pwszShareName = NULL;
-    DWORD dwAllowUserCount = 0;
-    PWSTR* ppwszAllowUsers = NULL;
-    DWORD dwDenyUserCount = 0;
-    PWSTR* ppwszDenyUsers = NULL;
-    DWORD dwIndex = 0;
-    PSTR pszUserName = NULL;
-    BOOLEAN bReadOnly = FALSE;
 
     dwError = LwMbsToWc16s(ppszArgv[1], &pwszShareName);
     BAIL_ON_SRVSVC_ERROR(dwError);
@@ -942,68 +1099,9 @@ GetInfo(
         OUT_PPVOID(&pShareInfo));
     BAIL_ON_SRVSVC_ERROR(dwError);
 
-    dwError = PrintStringAttribute("name", pShareInfo->shi502_netname);
+    dwError = DumpShareInfo(pShareInfo);
     BAIL_ON_SRVSVC_ERROR(dwError);
 
-    dwError = PrintStringAttribute("path", pShareInfo->shi502_path);
-    BAIL_ON_SRVSVC_ERROR(dwError);
-
-    dwError = PrintStringAttribute("comment", pShareInfo->shi502_remark);
-    BAIL_ON_SRVSVC_ERROR(dwError);
-
-    if (pShareInfo->shi502_security_descriptor)
-    {
-        dwError = DeconstructSecurityDescriptor(
-            pShareInfo->shi502_reserved,
-            (PSECURITY_DESCRIPTOR_RELATIVE) pShareInfo->shi502_security_descriptor,
-            &dwAllowUserCount,
-            &ppwszAllowUsers,
-            &dwDenyUserCount,
-            &ppwszDenyUsers,
-            &bReadOnly);
-        BAIL_ON_SRVSVC_ERROR(dwError);
-
-        if (dwAllowUserCount)
-        {
-            printf("allow=");
-
-            for (dwIndex = 0; dwIndex < dwAllowUserCount; dwIndex++)
-            {
-                dwError = LwWc16sToMbs(ppwszAllowUsers[dwIndex], &pszUserName);
-                BAIL_ON_SRVSVC_ERROR(dwError);
-
-                printf("%s", pszUserName);
-                if (dwIndex < dwAllowUserCount - 1)
-                {
-                    printf(",");
-                }
-            }
-
-            printf("\n");
-        }
-
-        if (dwDenyUserCount)
-        {
-            printf("deny=");
-
-            for (dwIndex = 0; dwIndex < dwDenyUserCount; dwIndex++)
-            {
-                dwError = LwWc16sToMbs(ppwszDenyUsers[dwIndex], &pszUserName);
-                BAIL_ON_SRVSVC_ERROR(dwError);
-
-                printf("%s", pszUserName);
-                if (dwIndex < dwDenyUserCount - 1)
-                {
-                    printf(",");
-                }
-            }
-
-            printf("\n");
-        }
-
-        printf("read_only=%s\n", bReadOnly ? "true" : "false");
-    }
-
 cleanup:
 
     if (pShareInfo)
@@ -1013,9 +1111,6 @@ cleanup:
 
     LW_SAFE_FREE_MEMORY(pwszShareName);
 
-    FreeStringArray(dwAllowUserCount, ppwszAllowUsers);
-    FreeStringArray(dwDenyUserCount, ppwszDenyUsers);
-
     return dwError;
 
 error:
@@ -1093,6 +1188,7 @@ SetInfo(
         &dwSecDescSize);
     BAIL_ON_SRVSVC_ERROR(dwError);
 
+    newShareInfo.shi502_type = pShareInfo->shi502_type;
     newShareInfo.shi502_reserved = dwSecDescSize;
     newShareInfo.shi502_security_descriptor = (PBYTE) pSecDesc;
 
@@ -1250,6 +1346,12 @@ main(
             BAIL_ON_SRVSVC_ERROR(dwError);
             break;
         }
+        else if (!strcasecmp(ppszArgv[dwIndex], "enum-info"))
+        {
+            dwError = EnumInfo(argc - dwIndex, ppszArgv + dwIndex);
+            BAIL_ON_SRVSVC_ERROR(dwError);
+            break;
+        }
         else if (!strcasecmp(ppszArgv[dwIndex], "get-info"))
         {
             dwError = GetInfo(argc - dwIndex, ppszArgv + dwIndex);
