@@ -181,6 +181,9 @@ main(
 
     SrvSvcBlockSelectedSignals();
 
+    dwError = SrvSvcInitSecurity();
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
     dwError = SrvSvcRpcInitialize();
     BAIL_ON_SRVSVC_ERROR(dwError);
 
@@ -190,9 +193,6 @@ main(
     dwError = SrvSvcSMNotify();
     BAIL_ON_SRVSVC_ERROR(dwError);
 
-    dwError = SrvSvcInitSecurity();
-    BAIL_ON_SRVSVC_ERROR(dwError);
-
     dwError = SrvSvcHandleSignals();
     BAIL_ON_SRVSVC_ERROR(dwError);
 
@@ -732,29 +732,7 @@ SrvSvcRpcInitialize(
     /* Bail if we still haven't succeeded after several attempts */
     BAIL_ON_SRVSVC_ERROR(dwError);
 
-    /* Now register the wkssvc pipe */
-
-    for (dwBindAttempts = 0; dwBindAttempts < dwMaxBindAttempts; dwBindAttempts++)
-    {
-        dwError = WksSvcRegisterForRPC(
-                        "Likewise Workstation Service",
-                        &gServerInfo.pWkstaBinding);
-        if (dwError)
-        {
-            SRVSVC_LOG_INFO("Failed to bind wkssvc endpoint; retrying in "
-                            "%i seconds...",
-                            (int) dwBindSleepSeconds);
-            sleep(dwBindSleepSeconds);
-        }
-        else
-        {
-            break;
-        }
-    }
-    /* Bail if we still haven't succeeded after several attempts */
-    BAIL_ON_SRVSVC_ERROR(dwError);
-
-    /* Now register the wkssvc pipe */
+    /* Now register the winreg pipe */
 
     for (dwBindAttempts = 0; dwBindAttempts < dwMaxBindAttempts; dwBindAttempts++)
     {
@@ -817,14 +795,6 @@ SrvSvcRpcShutdown(
         gServerInfo.pServerBinding = NULL;
     }
 
-    if (gServerInfo.pWkstaBinding)
-    {
-        dwError = WksSvcUnregisterForRPC(gServerInfo.pWkstaBinding);
-        BAIL_ON_SRVSVC_ERROR(dwError);
-
-        gServerInfo.pWkstaBinding = NULL;
-    }
-
     if (gServerInfo.pRegistryBinding)
     {
         dwError = WinRegUnregisterForRPC(gServerInfo.pRegistryBinding);
@@ -849,6 +819,12 @@ SrvSvcRpcShutdown(
         BAIL_ON_SRVSVC_ERROR(dwError);
     }
 
+    if (gServerInfo.pSessionSecDesc)
+    {
+        SrvSvcSrvDestroyServerSecurityDescriptor(gServerInfo.pSessionSecDesc);
+        gServerInfo.pSessionSecDesc = NULL;
+    }
+
 cleanup:
 
     SRVSVC_UNLOCK_MUTEX(bInLock, &gServerInfo.mutex);
