@@ -39,19 +39,6 @@
 
 #include "includes.h"
 
-static PSECURITY_DESCRIPTOR_ABSOLUTE gpServerSecDesc = NULL;
-static GENERIC_MAPPING gGenericMapping =
-{
-    .GenericRead = FILE_GENERIC_READ,
-    .GenericWrite = FILE_GENERIC_WRITE,
-    .GenericExecute = FILE_GENERIC_EXECUTE,
-    .GenericAll = FILE_ALL_ACCESS
-};
-
-#define SECURITY_UNMAPPED_UNIX_AUTHORITY    { 0, 0, 0, 0, 0, 22 }
-#define SECURITY_UNMAPPED_UNIX_UID_RID      1
-#define SECURITY_UNMAPPED_UNIX_RID_COUNT    2
-
 static
 NET_API_STATUS
 InitializeUnixRootSid(
@@ -83,17 +70,21 @@ SrvSvcInitSecurity(
 {
     DWORD dwError = 0;
     PSECURITY_DESCRIPTOR_ABSOLUTE pAbsolute = NULL;
+    PSECURITY_DESCRIPTOR_ABSOLUTE pSessionSecDesc = NULL;
     union
     {
         SID sid;
         BYTE buffer[SID_MAX_SIZE];
     } unixRootSid;
     PSID pBuiltinAdminsSid = NULL;
-    PSID pAuthedUsersSid = NULL;
+    PSID pWorldSid = NULL;
     DWORD dwSidSize = 0;
     DWORD dwDaclSize = 0;
     PACL pDacl = NULL;
 
+    dwError = SrvSvcSrvInitServerSecurityDescriptor(&pSessionSecDesc);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
     dwError = LwCreateWellKnownSid(
             WinBuiltinAdministratorsSid,
             NULL,
@@ -102,9 +93,9 @@ SrvSvcInitSecurity(
     BAIL_ON_SRVSVC_ERROR(dwError);
 
     dwError = LwCreateWellKnownSid(
-            WinAuthenticatedUserSid,
+            WinWorldSid,
             NULL,
-            &pAuthedUsersSid,
+            &pWorldSid,
             &dwSidSize);
     BAIL_ON_SRVSVC_ERROR(dwError);
 
@@ -113,7 +104,7 @@ SrvSvcInitSecurity(
 
     dwDaclSize = ACL_HEADER_SIZE +
         sizeof(ACCESS_ALLOWED_ACE) + RtlLengthSid(pBuiltinAdminsSid) - sizeof(ULONG) +
-        sizeof(ACCESS_ALLOWED_ACE) + RtlLengthSid(pAuthedUsersSid) - sizeof(ULONG) +
+        sizeof(ACCESS_ALLOWED_ACE) + RtlLengthSid(pWorldSid) - sizeof(ULONG) +
         sizeof(ACCESS_ALLOWED_ACE) + RtlLengthSid(&unixRootSid.sid) - sizeof(ULONG) +
         RtlLengthSid(pBuiltinAdminsSid);
 
@@ -150,7 +141,7 @@ SrvSvcInitSecurity(
             ACL_REVISION,
             0,
             FILE_GENERIC_READ,
-            pAuthedUsersSid));
+            pWorldSid));
     BAIL_ON_SRVSVC_ERROR(dwError);
 
     dwError = LwAllocateMemory(
@@ -186,10 +177,12 @@ SrvSvcInitSecurity(
             FALSE));
     BAIL_ON_SRVSVC_ERROR(dwError);
 
-    gpServerSecDesc = pAbsolute;
+    gServerInfo.pServerSecDesc = pAbsolute;
+    gServerInfo.pSessionSecDesc = pSessionSecDesc;
 
 cleanup:
-    LW_SAFE_FREE_MEMORY(pAuthedUsersSid);
+
+    LW_SAFE_FREE_MEMORY(pWorldSid);
 
     return dwError;
 
@@ -199,6 +192,11 @@ error:
     LW_SAFE_FREE_MEMORY(pDacl);
     LW_SAFE_FREE_MEMORY(pBuiltinAdminsSid);
 
+    if (pSessionSecDesc)
+    {
+        SrvSvcSrvDestroyServerSecurityDescriptor(pSessionSecDesc);
+    }
+
     goto cleanup;
 }
 
@@ -221,11 +219,11 @@ SrvSvcAccessCheck(
     BAIL_ON_SRVSVC_ERROR(dwError);
 
     if (!RtlAccessCheck(
-            gpServerSecDesc,
+            gServerInfo.pServerSecDesc,
             pToken,
             access,
             0,
-            &gGenericMapping,
+            &gServerInfo.genericMapping,
             &granted,
             &status))
     {
@@ -314,69 +312,335 @@ NET_API_STATUS _NetrConnectionEnum(
     return dwError;
 }
 
-NET_API_STATUS _NetrFileEnum(
-    /* [in] */ handle_t IDL_handle,
-    /* [in] */ wchar16_t *server_name,
-    /* [in] */ wchar16_t *basepath,
-    /* [in] */ wchar16_t *username,
-    /* [in, out] */ UINT32 *level,
-    /* [in, out] */ srvsvc_NetFileCtr *ctr,
-    /* [in] */ UINT32 prefered_maximum_length,
-    /* [out] */ UINT32 *total_entries,
-    /* [in, out] */ UINT32 *resume_handle
+NET_API_STATUS
+_NetrFileEnum(
+    handle_t           IDL_handle,           /* [in]      */
+    PWSTR              pwszServername,       /* [in]      */
+    PWSTR              pwszBasepath,         /* [in]      */
+    PWSTR              pwszUsername,         /* [in]      */
+    PDWORD             pdwInfoLevel,         /* [in, out] */
+    srvsvc_NetFileCtr* pInfo,                /* [in, out] */
+    DWORD              dwPreferredMaxLength, /* [in]      */
+    PDWORD             pdwTotalEntries,      /* [out]     */
+    PDWORD             pdwResumeHandle       /* [in, out] */
     )
 {
-    DWORD dwError = ERROR_NOT_SUPPORTED;
+    DWORD dwError        = 0;
+    DWORD dwInfoLevel    = 0;
+    DWORD dwEntriesRead  = 0;
+    DWORD dwTotalEntries = 0;
+    DWORD dwResumeHandle = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    GENERIC_MAPPING genericMapping = {0};
+    DWORD dwRequiredAccessRights = SRVSVC_ACCESS_GET_INFO_ADMINS;
+    DWORD dwAccessGranted     = 0;
+    SRVSVC_SRV_CONTEXT srvCtx = {0};
+
+    BAIL_ON_INVALID_PTR(pdwInfoLevel,    dwError);
+    BAIL_ON_INVALID_PTR(pInfo,           dwError);
+    BAIL_ON_INVALID_PTR(pdwTotalEntries, dwError);
+
+    dwInfoLevel    = *pdwInfoLevel;
+    if (pdwResumeHandle)
+    {
+        dwResumeHandle = *pdwResumeHandle;
+    }
+
+    dwError = SrvSvcSrvInitAuthInfo(IDL_handle, &srvCtx);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (!RtlAccessCheck(gServerInfo.pSessionSecDesc,
+                        srvCtx.pUserToken,
+                        dwRequiredAccessRights,
+                        0,
+                        &genericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+    }
+
+    dwError = SrvSvcNetFileEnum(
+                    IDL_handle,
+                    pwszServername,
+                    pwszBasepath,
+                    pwszUsername,
+                    dwInfoLevel,
+                    pInfo,
+                    dwPreferredMaxLength,
+                    &dwEntriesRead,
+                    &dwTotalEntries,
+                    pdwResumeHandle ? &dwResumeHandle : NULL);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    *pdwTotalEntries = dwTotalEntries;
+    if (pdwResumeHandle)
+    {
+        *pdwResumeHandle = dwResumeHandle;
+    }
+
+cleanup:
+
+    SrvSvcSrvFreeAuthInfoContents(&srvCtx);
 
     return dwError;
+
+error:
+
+    if (pdwTotalEntries)
+    {
+        *pdwTotalEntries = dwTotalEntries;
+    }
+
+    goto cleanup;
 }
 
-
-NET_API_STATUS _NetrFileGetInfo(
-    /* [in] */ handle_t IDL_handle,
-    /* [in] */ wchar16_t *server_name,
-    /* [in] */ UINT32 fileid,
-    /* [in] */ UINT32 level,
-    /* [out] */ srvsvc_NetFileInfo *info
+NET_API_STATUS
+_NetrFileGetInfo(
+    handle_t            IDL_handle,     /* [in] */
+    PWSTR               pwszServername, /* [in] */
+    DWORD               dwFileId,       /* [in] */
+    DWORD               dwInfoLevel,    /* [in] */
+    srvsvc_NetFileInfo* pInfo           /* [out] */
     )
 {
-    DWORD dwError = ERROR_NOT_SUPPORTED;
+    DWORD dwError        = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    GENERIC_MAPPING genericMapping = {0};
+    DWORD dwRequiredAccessRights = SRVSVC_ACCESS_GET_INFO_ADMINS;
+    DWORD dwAccessGranted     = 0;
+    SRVSVC_SRV_CONTEXT srvCtx = {0};
+
+    BAIL_ON_INVALID_PTR(pInfo, dwError);
+
+    dwError = SrvSvcSrvInitAuthInfo(IDL_handle, &srvCtx);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (!RtlAccessCheck(gServerInfo.pSessionSecDesc,
+                        srvCtx.pUserToken,
+                        dwRequiredAccessRights,
+                        0,
+                        &genericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+    }
+
+    dwError = SrvSvcNetFileGetInfo(
+                    IDL_handle,
+                    pwszServername,
+                    dwFileId,
+                    dwInfoLevel,
+                    pInfo);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+cleanup:
+
+    SrvSvcSrvFreeAuthInfoContents(&srvCtx);
 
     return dwError;
+
+error:
+
+    goto cleanup;
 }
 
-NET_API_STATUS _NetrFileClose(
-    /* [in] */ handle_t IDL_handle,
-    /* [in] */ wchar16_t *server_name,
-    /* [in] */ UINT32 fileid
+NET_API_STATUS
+_NetrFileClose(
+    handle_t IDL_handle,     /* [in] */
+    PWSTR    pwszServername, /* [in] */
+    DWORD    dwFileId        /* [in] */
     )
 {
-    DWORD dwError = ERROR_NOT_SUPPORTED;
+    DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    GENERIC_MAPPING genericMapping = {0};
+    DWORD dwRequiredAccessRights = SRVSVC_ACCESS_DELETE;
+    DWORD dwAccessGranted     = 0;
+    SRVSVC_SRV_CONTEXT srvCtx = {0};
+
+    dwError = SrvSvcSrvInitAuthInfo(IDL_handle, &srvCtx);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (!RtlAccessCheck(gServerInfo.pSessionSecDesc,
+                        srvCtx.pUserToken,
+                        dwRequiredAccessRights,
+                        0,
+                        &genericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+    }
+
+    dwError = SrvSvcNetFileClose(
+                    IDL_handle,
+                    pwszServername,
+                    dwFileId);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+cleanup:
+
+    SrvSvcSrvFreeAuthInfoContents(&srvCtx);
 
     return dwError;
+
+error:
+
+    goto cleanup;
 }
 
-NET_API_STATUS _NetrSessionEnum(
-    /* [in] */ handle_t IDL_handle,
-    /* [in] */ wchar16_t *server_name,
-    /* [in] */ wchar16_t *unc_client_name,
-    /* [in] */ wchar16_t *username,
-    /* [in, out] */ UINT32 *level,
-    /* [in, out] */ srvsvc_NetSessCtr *ctr,
-    /* [in] */ UINT32 prefered_maximum_length,
-    /* [out] */ UINT32 *total_entries,
-    /* [in, out] */ UINT32 *resume_handle
+NET_API_STATUS
+_NetrSessionEnum(
+    handle_t           IDL_handle,           /* [in]      */
+    PWSTR              pwszServername,       /* [in]      */
+    PWSTR              pwszUncClientname,    /* [in]      */
+    PWSTR              pwszUsername,         /* [in]      */
+    PDWORD             pdwInfoLevel,         /* [in, out] */
+    srvsvc_NetSessCtr* pInfo,                /* [in, out] */
+    DWORD              dwPreferredMaxLength, /* [in]      */
+    PDWORD             pdwTotalEntries,      /* [out]     */
+    PDWORD             pdwResumeHandle       /* [in, out] */
     )
 {
-    DWORD dwError = ERROR_NOT_SUPPORTED;
+    DWORD dwError        = 0;
+    NTSTATUS ntStatus    = STATUS_SUCCESS;
+    DWORD dwInfoLevel    = 0;
+    DWORD dwEntriesRead  = 0;
+    DWORD dwTotalEntries = 0;
+    DWORD dwResumeHandle = 0;
+    GENERIC_MAPPING genericMapping = {0};
+    DWORD dwRequiredAccessRights = 0;
+    DWORD dwAccessGranted     = 0;
+    SRVSVC_SRV_CONTEXT srvCtx = {0};
+
+    BAIL_ON_INVALID_PTR(pdwInfoLevel,      dwError);
+    BAIL_ON_INVALID_PTR(pInfo,             dwError);
+    BAIL_ON_INVALID_PTR(pdwTotalEntries,   dwError);
+
+    dwInfoLevel    = *pdwInfoLevel;
+    dwResumeHandle = pdwResumeHandle ? *pdwResumeHandle : 0;
+
+    switch (dwInfoLevel)
+    {
+        case 0:
+        case 10:
+
+            dwRequiredAccessRights = SRVSVC_ACCESS_GET_INFO_AUTHENTICATED_USERS;
+            break;
+
+        case 1:
+        case 2:
+        case 502:
+
+            dwRequiredAccessRights = SRVSVC_ACCESS_GET_INFO_ADMINS;
+            break;
+
+        default:
+
+            dwError = LwNtStatusToWin32Error(STATUS_INVALID_INFO_CLASS);
+            BAIL_ON_SRVSVC_ERROR(dwError);
+
+            break;
+    }
+
+    dwError = SrvSvcSrvInitAuthInfo(IDL_handle, &srvCtx);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (!RtlAccessCheck(gServerInfo.pSessionSecDesc,
+                        srvCtx.pUserToken,
+                        dwRequiredAccessRights,
+                        0,
+                        &genericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+    }
+
+    dwError = SrvSvcNetSessionEnum(
+                    IDL_handle,
+                    pwszServername,
+                    pwszUncClientname,
+                    pwszUsername,
+                    dwInfoLevel,
+                    pInfo,
+                    dwPreferredMaxLength,
+                    &dwEntriesRead,
+                    &dwTotalEntries,
+                    pdwResumeHandle ? &dwResumeHandle : NULL);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    *pdwTotalEntries = dwTotalEntries;
+    *pdwResumeHandle = dwResumeHandle;
+
+cleanup:
+
+    SrvSvcSrvFreeAuthInfoContents(&srvCtx);
 
     return dwError;
+
+error:
+
+    if (pdwTotalEntries)
+    {
+        *pdwTotalEntries = dwTotalEntries;
+    }
+
+    goto cleanup;
 }
 
-void _srvsvc_FunctionD(
-    /* [in] */ handle_t IDL_handle
+NET_API_STATUS
+_NetrSessionDel(
+    handle_t IDL_handle,        /* [in] */
+    PWSTR    pwszServername,    /* [in] */
+    PWSTR    pwszUncClientname, /* [in] */
+    PWSTR    pwszUsername       /* [in] */
     )
 {
+    DWORD dwError = 0;
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    GENERIC_MAPPING genericMapping = {0};
+    DWORD dwRequiredAccessRights = SRVSVC_ACCESS_DELETE;
+    DWORD dwAccessGranted     = 0;
+    SRVSVC_SRV_CONTEXT srvCtx = {0};
+
+    dwError = SrvSvcSrvInitAuthInfo(IDL_handle, &srvCtx);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+    if (!RtlAccessCheck(gServerInfo.pSessionSecDesc,
+                        srvCtx.pUserToken,
+                        dwRequiredAccessRights,
+                        0,
+                        &genericMapping,
+                        &dwAccessGranted,
+                        &ntStatus))
+    {
+        dwError = LwNtStatusToWin32Error(ntStatus);
+        BAIL_ON_SRVSVC_ERROR(dwError);
+    }
+
+    dwError = SrvSvcNetSessionDel(
+                    IDL_handle,
+                    pwszServername,
+                    pwszUncClientname,
+                    pwszUsername);
+    BAIL_ON_SRVSVC_ERROR(dwError);
+
+cleanup:
+
+    SrvSvcSrvFreeAuthInfoContents(&srvCtx);
+
+    return dwError;
+
+error:
+
+    goto cleanup;
 }
 
 NET_API_STATUS _NetrShareAdd(
@@ -404,7 +668,10 @@ cleanup:
     return dwError;
 
 error:
-    *parm_error = 0;
+    if (parm_error)
+    {
+        *parm_error = 0;
+    }
 
     goto cleanup;
 }
@@ -441,8 +708,15 @@ cleanup:
 error:
     memset(ctr, 0, sizeof(*ctr));
 
-    *total_entries = 0;
-    *resume_handle = 0;
+    if (total_entries)
+    {
+        *total_entries = 0;
+    }
+
+    if (resume_handle)
+    {
+        *resume_handle = 0;
+    }
 
     goto cleanup;
 }
@@ -503,7 +777,10 @@ cleanup:
     return dwError;
 
 error:
-    *parm_error = 0;
+    if (parm_error)
+    {
+        *parm_error = 0;
+    }
 
     goto cleanup;
 }
