@@ -8,13 +8,12 @@
 #include <lwstr.h>
 #include <wc16str.h>
 #include <lwio/io-types.h>
-#include <lwrpc/unicodestring.h>
 
-#include <srvsvc/srvsvc.h>
-#include <srvsvc/srvsvcdefs.h>
+#include <lw/srvsvc.h>
 
 #include <srvsvcdefs.h>
 #include <srvsvcutils.h>
+#include <dce/rpc.h>
 
 #include "defs.h"
 #include "structs.h"
